﻿//*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* DTAPI.h *#*#*#*#*#*#*#*#*#* (C) 2000-2004 DEKTEC

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- License -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// This program is free software; You can freely use/modify this source code in any way
// you desire, including for commercial applications.
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	TD	2007.01.31	Added support for GetIoConfig
//	TD  2007.01.30  Added support for the DTA-160 (ASI only)
//	TD	2007.01.23	Added support for the DTA-145 and DTA-2145
//  TD  2007.01.23  DTAPI_TXMODE_SDI_FULL definition added
//  MG  2006.12.21  Support for DTA-110T (OFDM modulation)
//  MG  2006.08.14	Added support for DTU-245 + some updates for DTA-105
//  MG  2005.08.25  Added support for DTA-545, DTA-105 and DTA-110
//	MG	2004.12.01	Added support for Dtu2xx driver
//	MG	2004.10.20	Adapted for new DtDevice class
//	MG	2003.06.30	Added support for store MP2 RXMODE_STMP2 and timestamping
//					(RX_TIMESTAMP)
//	MG	2003.05.09	Added implementation for PciCard::GetFirmwareVersion
//	MG	2003.04.23	Added implementation for VPD reading and writing
//	MG	19-11-2002	Created	based on DTAPI for windows

#ifndef __DTAPI_H
#define __DTAPI_H

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTAPI Version -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define DTAPI_VERSION_MAJOR		9
#define DTAPI_VERSION_MINOR		8
#define DTAPI_VERSION_BUGFIX	2
#define DTAPI_VERSION_BUILD		38

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ DTAPI Support Types +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Elementary Types -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
typedef unsigned long DTAPI_RESULT;
typedef long long __int64;

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DtDeviceDesc -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Structure describing a device.
//
struct DtDeviceDesc {
	int	 m_Category;			// Device category
	__int64	 m_Serial;			// Unique serial number of the device
	int	 m_PciBusNumber;		// PCI-bus number
	int	 m_SlotNumber;			// PCI-slot number
	int  m_UsbAddress;			// USB address
	int  m_TypeNumber;			// Device type number
	int	 m_DeviceId;			// Device ID
	int	 m_VendorId;			// Vendor ID
	int	 m_SubsystemId;			// Subsystem
	int	 m_SubVendorId;			// Subsystem Vendor ID
	int	 m_NumHwFuncs;			// #Hardware funtions hosted by device
	int	 m_NumDtInpChan;		// #Input channels
	int	 m_NumDtOutpChan;		// #Output channels
	int  m_FirmwareVersion;		// Firmware version
};

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Device Categories -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTAPI_CAT_PCI	0
#define  DTAPI_CAT_USB	1

//-.-.-.-.-.-.-.-.-.-.-.-.-.- Hardware-Function Channel Types -.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTAPI_CHAN_OUTPUT		1
#define  DTAPI_TS_OUTPUT		1			// Obsolete, for backward compatibility
#define  DTAPI_CHAN_INPUT		2
#define  DTAPI_TS_INPUT			2			// Obsolete, for backward compatibility
#define  DTAPI_CHAN_DBLBUF		4

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Hardware-Function Stream Types -.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTAPI_ASI_SDI			1
#define  DTAPI_TS_MOD			2
#define  DTAPI_TS_OVER_IP		3
#define  DTAPI_TS_SPI			4

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Hardware-Function Capabilities -.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTAPI_CAP_ASI			0x00000001
#define  DTAPI_CAP_BIDIR		0x00000002
#define  DTAPI_CAP_DBLBUF		0x00000004
#define  DTAPI_CAP_SDI			0x00000008
#define  DTAPI_CAP_FAILSAFE		0x00000010

// Modulation-related
#define  DTAPI_CAP_OFDM			0x00000010
#define  DTAPI_CAP_ISDBT		0x00000020
#define  DTAPI_CAP_QAM_A		0x00000100
#define  DTAPI_CAP_QAM_B		0x00000200
#define  DTAPI_CAP_QAM_C		0x00000400
#define  DTAPI_CAP_QPSK			0x00001000		// DVB-S legacy definition
#define  DTAPI_CAP_DVBS			0x00001000
#define  DTAPI_CAP_DVBS2		0x00002000
#define  DTAPI_CAP_VSB			0x00010000
#define  DTAPI_CAP_8VSB			0x00010000		// LEGACY
#define  DTAPI_CAP_VHF			0x00100000		// VHF-band 47-470MHz
#define  DTAPI_CAP_UHF			0x00200000		// UHF-band 400-862MHz
#define  DTAPI_CAP_LBAND		0x00400000		// L-band 950-2150MHz

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DtHwFuncDesc -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Structure describing a hardware function.
//
struct DtHwFuncDesc {
	DtDeviceDesc  m_DvcDesc;	// Device descriptor
	int  m_ChanType;			// Compatible channel types
	int	 m_Flags;				// Hardware-function flags
	int	 m_IndexOnDvc;			// Index of hardware function
        int  m_Port;                    // Physical port number
};


//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ DtDevice +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=
//
// Class to represent a DEKTEC Device.
//
class DtDevice {

	// Public attributes
public:
	DtDeviceDesc  m_DvcDesc;	// Device Descriptor

	// Public access functions
public:
	int  Category(void)			{ return m_DvcDesc.m_Category; }
	int  FirmwareVersion(void)	{ return m_DvcDesc.m_FirmwareVersion; }
	int  TypeNumber(void)		{ return m_DvcDesc.m_TypeNumber; }
	bool IsAttached(void)		{ return m_Attached; }

	// Public member functions
public:
	DTAPI_RESULT  AttachToSerial(__int64 SerialNumber);
	DTAPI_RESULT  AttachToSlot(int PciBusNumber, int SlotNumber);
	DTAPI_RESULT  AttachToType(int TypeNumber, int DeviceNo=0);
	DTAPI_RESULT  Detach(void);
	DTAPI_RESULT  GetDescriptor(DtDeviceDesc& DvcDesc);
	DTAPI_RESULT  GetDeviceDriverVersion(int& DriverVersionMajor,
										 int& DriverVersionMinor);
	DTAPI_RESULT  GetDeviceDriverVersion(
								int& DriverVersionMajor, int& DriverVersionMinor,
								int& DriverVersionBugFix, int& DriverVersionBuild);
	DTAPI_RESULT  GetFirmwareVersion(int& FirmwareVersion);
	DTAPI_RESULT  GetIoConfig(int Port, int& IoConfig);
	DTAPI_RESULT  GetRefClkCnt(int& RefClkCnt);
	DTAPI_RESULT  LedControl(int LedControl);
	DTAPI_RESULT  SetIoConfig(int Port, int IoConfig, int ParXtra=-1);
	DTAPI_RESULT  VpdDelete(char* pTag);
	DTAPI_RESULT  VpdDelete(wchar_t* pTag);
	DTAPI_RESULT  VpdRead(char* pTag, char* pVpdItem);
	DTAPI_RESULT  VpdRead(wchar_t* pTag, wchar_t* pVpdItem);
	DTAPI_RESULT  VpdWrite(char* pTag, char* pVpdItem);
	DTAPI_RESULT  VpdWrite(wchar_t* pTag, wchar_t* pVpdItem);

	// Encapsulated data
public:
	bool  m_Attached;			// Attached-to-hardware flag
	int   m_fdDriver;			// Handle to device driver

	// Constructor, destructor
public:
	DtDevice();
	virtual ~DtDevice();

	// Friends
	friend class  TsInpChannel;
	friend class  TsOutpChannel;
};
//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DtDevice Constants -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.



//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- I/O Configuration -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTAPI_IOCONFIG_NOTSUP		0
#define  DTAPI_IOCONFIG_INPUT		1
#define  DTAPI_IOCONFIG_OUTPUT		2
#define  DTAPI_IOCONFIG_DBLBUF		3
#define  DTAPI_IOCONFIG_IP			4
#define  DTAPI_IOCONFIG_FAILSAFE	5



//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Support Flags -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

// Used in DtInpChannel and DtOutpChannel
#define  DTAPI_SUPPORTS_TS			1
#define  DTAPI_SUPPORTS_SDI			2



//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ TsInpChannel +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
//
// Class to represent a transport-stream input channel.
//
class TsInpChannel {

	// Public attributes
public:
	DtHwFuncDesc  m_HwFuncDesc;	// Hardware function descriptor

	// Public access functions
public:
	int  Category(void)			{ return m_HwFuncDesc.m_DvcDesc.m_Category; }
	int  FirmwareVersion(void)	{ return m_HwFuncDesc.m_DvcDesc.m_FirmwareVersion; }
	int  TypeNumber(void)		{ return m_HwFuncDesc.m_DvcDesc.m_TypeNumber; }
	int  GetIndexOnDevice(void)	{ return m_HwFuncDesc.m_IndexOnDvc; }
	int  GetChannelType(void)	{ return m_HwFuncDesc.m_ChanType; }
	bool IsAttached(void)		{ return m_Attached; }

public:
	DTAPI_RESULT  Attach(DtDevice* pDtDvc, int InpIndex=0);
	DTAPI_RESULT  AttachToPort(DtDevice* pDtDvc, int Port=1);
	DTAPI_RESULT  ClearFifo(void);
	DTAPI_RESULT  ClearFlags(int Latched);
	DTAPI_RESULT  Detach(int DetachMode);
	DTAPI_RESULT  Equalise(int EqualiserSetting);
	DTAPI_RESULT  GetDescriptor(DtHwFuncDesc& HwFunDesc);
	DTAPI_RESULT  GetFifoLoad(int& FifoLoad);
	DTAPI_RESULT  GetFlags(int& Status, int& Latched);
	DTAPI_RESULT  GetRxControl(int& RxControl);
	DTAPI_RESULT  GetStatistics(int& ViolCount);
	DTAPI_RESULT  GetStatus(int& PacketSize, int& NumInv, int& ClkDet,
							int& AsiLock, int& RateOk, int& AsiInv);
	DTAPI_RESULT  GetTargetId(int& Present, int& TargetId);
	DTAPI_RESULT  GetTsRateBps(int& TsRate);
	DTAPI_RESULT  LedControl(int LedControl);
	DTAPI_RESULT  PolarityControl(int Polarity);
	DTAPI_RESULT  Read(char* pBuffer, int NumBytesToRead);
	DTAPI_RESULT  ReadDirect(char* pBuffer,	int NumBytesToRead, int& NumBytesRead);
	DTAPI_RESULT  ReadUsingDma(char* pBuffer, int NumBytesToRead);
	DTAPI_RESULT  Reset(int ResetMode);
	DTAPI_RESULT  SetLoopBackMode(int Mode);
	DTAPI_RESULT  SetPower(int Power);
	DTAPI_RESULT  SetRxControl(int RxControl);
	DTAPI_RESULT  SetRxMode(int RxMode);
	DTAPI_RESULT  WriteLoopBackData(char* pBuffer, int NumBytesToWrite);

	// Encapsulated data
protected:
	bool  m_Attached;			// Attached to hardware flag
	int	  m_fdDevice;			// Handle to device driver

	// Constructor, destructor
public:
	TsInpChannel();
	virtual ~TsInpChannel();
};
typedef TsInpChannel DtInpChannel; // TEMP: for compatibility with windows version

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsInpChannel Constants -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

// Feature not supported
#define  DTAPI_NOT_SUPPORTED	-1

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- ASI Polarity-Control Status -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTAPI_ASIINV_NORMAL	0
#define  DTAPI_ASIINV_INVERT	1

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- ASI Input-Clock Lock -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTAPI_ASI_NOLOCK		0
#define  DTAPI_ASI_INLOCK		1

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Clock Detector -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTAPI_CLKDET_FAIL		0
#define  DTAPI_CLKDET_OK		1

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Input Rate Ok -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTAPI_INPRATE_LOW		0
#define  DTAPI_INPRATE_OK		1

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- #Invalid Bytes per Packet -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTAPI_NUMINV_NONE		0
#define  DTAPI_NUMINV_16		1
#define  DTAPI_NUMINV_OTHER		2

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Packet Size -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTAPI_PCKSIZE_INV		0
#define  DTAPI_PCKSIZE_188		2
#define  DTAPI_PCKSIZE_204		3

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Receive Control -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTAPI_RXCTRL_IDLE		0
#define  DTAPI_RXCTRL_RCV		1

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Receive Mode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTAPI_RXMODE_ST188		0
#define  DTAPI_RXMODE_ST204		1
#define  DTAPI_RXMODE_STMP2		2
#define  DTAPI_RXMODE_STRAW		3
#define  DTAPI_RXMODE_STTRP		4
#define  DTAPI_RX_TIMESTAMP		8


//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Status Flags -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTAPI_RX_DMA_PENDING	1
#define  DTAPI_RX_FIFO_OVF		2
#define  DTAPI_RX_SYNC_ERR		4
#define  DTAPI_RX_RATE_OVF		8

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ TsOutpChannel +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
//
// Class to represent a transport-stream output channel.
//
class TsOutpChannel {

	// Public attributes
public:
	DtHwFuncDesc  m_HwFuncDesc;	// Hardware function descriptor

	// Public access functions
public:
        int  Category(void)			{ return m_HwFuncDesc.m_DvcDesc.m_Category; }
        int  FirmwareVersion(void)	{ return m_HwFuncDesc.m_DvcDesc.m_FirmwareVersion; }
        int  TypeNumber(void)		{ return m_HwFuncDesc.m_DvcDesc.m_TypeNumber; }
        int  GetIndexOnDevice(void)	{ return m_HwFuncDesc.m_IndexOnDvc; }
        int  GetChannelType(void)	{ return m_HwFuncDesc.m_ChanType; }
        bool IsAttached(void)		{ return m_Attached; }


public:
	DTAPI_RESULT  Attach(DtDevice* pDtDvc, int OutpIndex=0);
        DTAPI_RESULT  AttachToPort(DtDevice* pDtDvc, int Port);
	DTAPI_RESULT  ClearFifo(void);
	DTAPI_RESULT  ClearFlags(int Latched);
	DTAPI_RESULT  Detach(int DetachMode);
	DTAPI_RESULT  GetDescriptor(DtHwFuncDesc& HwFunDesc);
	DTAPI_RESULT  GetExtClkFreq(int& ExtClkFreq);
	DTAPI_RESULT  GetFifoLoad(int& FifoLoad);
	DTAPI_RESULT  GetFlags(int& Status, int& Latched);
	DTAPI_RESULT  GetTargetId(int& Present, int& TargetId);
	DTAPI_RESULT  GetMaxFifoSize(int& MaxFifoSize);
	DTAPI_RESULT  GetModControl(int& ModType, int& CodeRate, int& ParXtra1, int& ParXtra2);
	DTAPI_RESULT  GetRfControl(__int64& RfRate, int& LockStatus);
	DTAPI_RESULT  GetTsRateBps(int& ClockGenMode, int& TsRate);
        DTAPI_RESULT  GetTsRateBps(int& TsRate);
	DTAPI_RESULT  GetTxControl(int& TxControl);
	DTAPI_RESULT  GetTxMode(int& TxPacketMode, int& TxStuffMode);
	DTAPI_RESULT  ReadLoopBackData(char* pBuffer, int BytesToRead);
	DTAPI_RESULT  Reset(int ResetMode);
	DTAPI_RESULT  SetFifoSize(int FifoSize);
	DTAPI_RESULT  SetFifoSizeMax(void);
	DTAPI_RESULT  SetLoopBackMode(int Mode);
	DTAPI_RESULT  SetModControl(int ModType, int CodeRate, int ParXtra1, int ParXtra2);
	DTAPI_RESULT  SetPower(int Power);
	DTAPI_RESULT  SetRfControl(__int64 RfRate);
	DTAPI_RESULT  SetTsRateBps(int ClockGenMode, int TsRate);
        DTAPI_RESULT  SetTsRateBps(int TsRate);
	DTAPI_RESULT  SetTxControl(int TxControl);
	DTAPI_RESULT  SetTxMode(int TxPacketMode, int TxStuffMode);
	DTAPI_RESULT  SetTxPolarity(int TxPolarity);
	DTAPI_RESULT  Write(char* pBuffer, int NumBytesToWrite);
	DTAPI_RESULT  WriteDirect(char* pBuffer, int NumBytesToWrite);
	DTAPI_RESULT  WriteUsingDma(char* pBuffer, int NumBytesToWrite);

	// Encapsulated data
protected:
	bool  m_Attached;			// Attached to hardware flag
	int	  m_fdDevice;			// Handle to device driver

	// Cached settings
	int   m_TxPckMode;			// Cached packet transmit-mode setting
	int   m_TsRate;				// Cached TsRate setting
	int   m_ClockGenMode;		        // Cached Clock Gen setting
	int   m_CodeRate;			// Chached code rate setting
        int   m_ModType;                        // Cached modultation type

	// Constructor, destructor
public:
	TsOutpChannel();
	virtual ~TsOutpChannel();
};
typedef TsOutpChannel DtOutpChannel; // TEMP: for compatibility with windows version

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel Constants -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Clock-Generator Modes -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTAPI_TXCLOCK_INTERNAL	0
#define  DTAPI_TXCLOCK_EXTERNAL	1

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Detach Mode flags -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTAPI_INSTANT_DETACH	1
#define  DTAPI_ZERO_OUTPUT		2
#define  DTAPI_TRISTATE_OUTPUT	4
#define  DTAPI_WAIT_UNTIL_SENT	8

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Equaliser Settings -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTAPI_EQUALISER_OFF	0
#define  DTAPI_EQUALISER_ON		1

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Hardware-Function Types -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTAPI_CHAN_OUTPUT		1
#define  DTAPI_TS_OUTPUT		1			// Obsolete, for backward compatibility
#define  DTAPI_CHAN_INPUT		2
#define  DTAPI_TS_INPUT			2			// Obsolete, for backward compatibility

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- LED Control -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTAPI_LED_OFF			0
#define  DTAPI_LED_GREEN		1
#define  DTAPI_LED_RED			2
#define  DTAPI_LED_YELLOW		3
#define  DTAPI_LED_HARDWARE		4

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Loop-Back Mode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTAPI_NO_LOOPBACK		0
#define  DTAPI_LOOPBACK_MODE	1

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Modulation Parameters -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTAPI_MOD_DVBS_QPSK		0		// Native DVB-S on DTA-107
#define  DTAPI_MOD_QPSK			0		// Legacy
#define  DTAPI_MOD_BPSK			1
#define  DTAPI_MOD_QAM4			3
#define  DTAPI_MOD_QAM16		4
#define  DTAPI_MOD_QAM32		5
#define  DTAPI_MOD_QAM64		6
#define  DTAPI_MOD_QAM128		7
#define  DTAPI_MOD_QAM256		8
#define  DTAPI_MOD_OFDM			9		// Legacy
#define  DTAPI_MOD_DVBT			9
#define  DTAPI_MOD_VSB8			10
#define  DTAPI_MOD_VSB16		11
#define  DTAPI_MOD_ISDBT		12
#define  DTAPI_MOD_SYM4B		14		// 4-bit symbols for DTA-107S2
#define  DTAPI_MOD_IQDIRECT		15
#define  DTAPI_MOD_DVBS_QPSK_SOFT	16		// Soft DVB-S for DTA-107S2
#define  DTAPI_MOD_DVBS2_QPSK		32
#define  DTAPI_MOD_DVBS2_8PSK		33

#define  DTAPI_MOD_1_2			0	// Code rate 1/2
#define  DTAPI_MOD_2_3			1	// Code rate 2/3
#define  DTAPI_MOD_3_4			2	// Code rate 3/4
#define  DTAPI_MOD_4_5			3	// Code rate 4/5
#define  DTAPI_MOD_5_6			4	// Code rate 5/6
#define  DTAPI_MOD_6_7			5	// Code rate 6/7
#define  DTAPI_MOD_7_8			6	// Code rate 7/8
#define  DTAPI_MOD_1_4			7		// Code rate 1/4
#define  DTAPI_MOD_1_3			8		// Code rate 1/3
#define  DTAPI_MOD_2_5			9		// Code rate 2/5
#define  DTAPI_MOD_3_5			10		// Code rate 3/5
#define  DTAPI_MOD_8_9			11		// Code rate 8/9
#define  DTAPI_MOD_9_10			12		// Code rate 9/10

#define  DTAPI_MOD_J83_A		2		// J.83 annex A (DVB-C)
#define  DTAPI_MOD_ROLLOFF_15		2		// Roll-off factor 15% (LEGACY)
#define  DTAPI_MOD_J83_B		3		// J.83 annex B (�American QAM�)
#define  DTAPI_MOD_J83_C		1		// J.83 annex C (�Japanese QAM�)
#define  DTAPI_MOD_ROLLOFF_13		1		// Roll-off factor 13% (LEGACY)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Modulation Parameters (DVB-S2) -.-.-.-.-.-.-.-.-.-.-.-.-.-.

// ParXtra1
#define  DTAPI_MOD_S2_NOPILOTS		0		// Pilots disabled
#define  DTAPI_MOD_S2_PILOTS		1		// Pilots enabled
#define  DTAPI_MOD_S2_PILOTS_MSK	1		// Mask for pilots field

#define  DTAPI_MOD_S2_SHORTFRM		2		// Short FECFRAME
#define  DTAPI_MOD_S2_LONGFRM		0		// Long FECFRAME
#define  DTAPI_MOD_S2_FRM_MSK		2		// Mask for FECFRAME field

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Modulation Parameters (ISDBT) -.-.-.-.-.-.-.-.-.-.-.-.-.-.-

#define  DTAPI_ISDBT_INP_TS188		0
#define  DTAPI_ISDBT_INP_TS204		1		// Decode parameters from TMCC info
#define  DTAPI_ISDBT_INP_MSK		1		// Mask for input-mode field

#define  DTAPI_ISDBT_BTYPE_TV		0		// 13-segment TV broadcast
#define  DTAPI_ISDBT_BTYPE_RAD1		1		// 1-segment radio broadcast
#define  DTAPI_ISDBT_BTYPE_RAD3		2		// 3-segment radio broadcast

#define  DTAPI_ISDBT_GUARD_1_32		0
#define  DTAPI_ISDBT_GUARD_1_16		1
#define  DTAPI_ISDBT_GUARD_1_8		2
#define  DTAPI_ISDBT_GUARD_1_4		3

#define  DTAPI_ISDBT_MOD_DQPSK		0
#define  DTAPI_ISDBT_MOD_QPSK		1
#define  DTAPI_ISDBT_MOD_QAM16		2
#define  DTAPI_ISDBT_MOD_QAM64		3

#define  DTAPI_ISDBT_RATE_1_2		0
#define  DTAPI_ISDBT_RATE_2_3		1
#define  DTAPI_ISDBT_RATE_3_4		2
#define  DTAPI_ISDBT_RATE_5_6		3
#define  DTAPI_ISDBT_RATE_7_8		4

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Modulation Parameters (OFDM) -.-.-.-.-.-.-.-.-.-.-.-.-.-.-

#define  DTAPI_MOD_OFDM_5MHZ		0x00000001
#define  DTAPI_MOD_OFDM_6MHZ		0x00000002
#define  DTAPI_MOD_OFDM_7MHZ		0x00000003
#define  DTAPI_MOD_OFDM_8MHZ		0x00000004
#define  DTAPI_MOD_OFDM_BW_MSK		0x0000000F

#define  DTAPI_MOD_OFDM_QPSK		0x00000010
#define  DTAPI_MOD_OFDM_QAM16		0x00000020
#define  DTAPI_MOD_OFDM_QAM64		0x00000030
#define  DTAPI_MOD_OFDM_CO_MSK		0x000000F0

#define  DTAPI_MOD_OFDM_G_1_32		0x00000100
#define  DTAPI_MOD_OFDM_G_1_16		0x00000200
#define  DTAPI_MOD_OFDM_G_1_8		0x00000300
#define  DTAPI_MOD_OFDM_G_1_4		0x00000400
#define  DTAPI_MOD_OFDM_GU_MSK		0x00000F00

#define  DTAPI_MOD_OFDM_INDEPTH		0x00001000
#define  DTAPI_MOD_OFDM_NATIVE		0x00002000
#define  DTAPI_MOD_OFDM_IL_MSK		0x0000F000

#define  DTAPI_MOD_OFDM_2K			0x00010000
#define  DTAPI_MOD_OFDM_4K			0x00020000
#define  DTAPI_MOD_OFDM_8K			0x00030000
#define  DTAPI_MOD_OFDM_MD_MSK		0x000F0000

#define  DTAPI_MOD_OFDM_S48_OFF		0x00000000
#define  DTAPI_MOD_OFDM_S48			0x00100000
#define  DTAPI_MOD_OFDM_S48_MSK		0x00100000

#define  DTAPI_MOD_OFDM_S49_OFF		0x00000000
#define  DTAPI_MOD_OFDM_S49			0x00200000
#define  DTAPI_MOD_OFDM_S49_MSK		0x00200000

#define  DTAPI_MOD_OFDM_ENA4849		0x00000000
#define  DTAPI_MOD_OFDM_DIS4849		0x00400000
#define  DTAPI_MOD_OFDM_4849_MSK	0x00400000

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Packet Transmit Mode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTAPI_TXMODE_188		0
#define  DTAPI_TXMODE_204		1
#define  DTAPI_TXMODE_ADD16		2
#define  DTAPI_TXMODE_RAW		3
#define  DTAPI_TXMODE_192		4		// Supported on DTA-102 Firmware Rev >= 2
#define  DTAPI_TXMODE_130		5		// Supported on DTA-102 Firmware Rev >= 7
#define  DTAPI_TXMODE_MIN16		6		// Supported on DTA-100 Firmware Rev >= 5
										// and          DTA-102 Firmware Rev >= 8
#define  DTAPI_TXMODE_IPRAW		7
#define  DTAPI_TXMODE_MASK		0x00F	// Mask for TxMode without burst flag
#define  DTAPI_TXMODE_BURST		0x010	// Can be OR-ed with one of 188/192/204/ADD16/RAW
#define	 DTAPI_TXMODE_SDI		0x040
#define	 DTAPI_TXMODE_SDI_10B	0x080	// Can be OR-ed with DTAPI_TXMODE_SDI
#define  DTAPI_TXMODE_SDI_525	0x000	// Can be OR-ed with DTAPI_TXMODE_SDI/10BIT
#define  DTAPI_TXMODE_SDI_625	0x100	// Can be OR-ed with DTAPI_TXMODE_SDI/10BIT
#define  DTAPI_TXMODE_SDI_RAW	(DTAPI_TXMODE_SDI | DTAPI_TXMODE_RAW)
#define  DTAPI_TXMODE_SDI_FULL	DTAPI_TXMODE_SDI_RAW	// Temporary definition

#define  DTAPI_TXSTUFF_MODE_OFF	0		// Stuff-mode is disabled
#define  DTAPI_TXSTUFF_MODE_ON	1		// Stuff-mode is enabled

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Transmit Polarity -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTAPI_TXPOL_NORMAL		0
#define  DTAPI_TXPOL_INVERTED	1

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Polarity Control -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTAPI_POLARITY_AUTO	0
#define  DTAPI_POLARITY_NORMAL	2
#define  DTAPI_POLARITY_INVERT	3

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Power Mode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTAPI_POWER_OFF		0
#define  DTAPI_POWER_ON			1

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Reset Mode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTAPI_FIFO_RESET		0
#define  DTAPI_FULL_RESET		1

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Status Flags -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTAPI_RX_DMA_PENDING	1
#define  DTAPI_RX_FIFO_OVF		2
#define  DTAPI_RX_SYNC_ERR		4
#define  DTAPI_RX_RATE_OVF		8
#define  DTAPI_RX_TARGET_ERR	16

#define  DTAPI_TX_DMA_PENDING	1
#define  DTAPI_TX_FIFO_UFL		2
#define  DTAPI_TX_SYNC_ERR		4
#define  DTAPI_TX_READBACK_ERR	8
#define  DTAPI_TX_TARGET_ERR	16

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Target Adapter Present -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTAPI_NO_CONNECTION	0
#define  DTAPI_DVB_SPI_SINK		1		// For output channels
#define  DTAPI_DVB_SPI_SOURCE	1		// For input channels
#define  DTAPI_TARGET_PRESENT	2
#define  DTAPI_TARGET_UNKNOWN	3

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Transmit Control -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTAPI_TXCTRL_IDLE		0
#define  DTAPI_TXCTRL_HOLD		1
#define  DTAPI_TXCTRL_SEND		3

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Global DtAPI Functions +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

DTAPI_RESULT  DtapiCheckDeviceDriverVersion(void);
DTAPI_RESULT  DtapiDeviceScan(int NumEntries, int& NumEntriesResult,
							  DtDeviceDesc* DvcDescArr);
DTAPI_RESULT  DtapiGetDeviceDriverVersion(int&, int&, int&, int&);
DTAPI_RESULT  DtapiGetDeviceDriverVersion(int, int&, int&, int&, int&);
DTAPI_RESULT  DtapiGetVersion(int& LibVersion, int& LibVersionMinor);
DTAPI_RESULT  DtapiGetVersion(int& LibVersion, int& LibVersionMinor,
							  int& LibVersionBugFix, int& LibVersionBuild);


//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Return Codes +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+

#define  DTAPI_OK				0
#define  DTAPI_OK_FAILSAFE			1
#define  DTAPI_E				0x1000
#define  DTAPI_E_ATTACHED		(DTAPI_E + 0)
#define  DTAPI_E_BUF_TOO_SMALL	(DTAPI_E + 1)
#define  DTAPI_E_DEV_DRIVER		(DTAPI_E + 2)
#define  DTAPI_E_EEPROM_FULL	(DTAPI_E + 3)
#define  DTAPI_E_EEPROM_READ	(DTAPI_E + 4)
#define  DTAPI_E_EEPROM_WRITE	(DTAPI_E + 5)
#define  DTAPI_E_EEPROM_FORMAT	(DTAPI_E + 6)
#define  DTAPI_E_FIFO_FULL		(DTAPI_E + 7)
#define  DTAPI_E_IN_USE			(DTAPI_E + 8)
#define  DTAPI_E_INVALID_BUF	(DTAPI_E + 9)
#define  DTAPI_E_INVALID_CGMODE	(DTAPI_E + 10)
#define  DTAPI_E_INVALID_FLAGS	(DTAPI_E + 11)
#define  DTAPI_E_INVALID_MODE	(DTAPI_E + 12)
#define  DTAPI_E_INVALID_RATE	(DTAPI_E + 13)
#define  DTAPI_E_INVALID_SIZE	(DTAPI_E + 14)
#define  DTAPI_E_KEYWORD		(DTAPI_E + 15)
#define  DTAPI_E_NO_DEVICE		(DTAPI_E + 16)
#define  DTAPI_E_NO_DTA_CARD	(DTAPI_E + 16)	// BACKWARD COMPATIBILITY
#define  DTAPI_E_NO_LOOPBACK	(DTAPI_E + 17)
#define  DTAPI_E_NO_SUCH_DEVICE	(DTAPI_E + 18)
#define  DTAPI_E_NO_SUCH_OUTPUT	(DTAPI_E + 19)
#define  DTAPI_E_NO_DT_OUTPUT		(DTAPI_E + 20)
#define  DTAPI_E_NO_TS_OUTPUT	(DTAPI_E + 20)
#define  DTAPI_E_NOT_ATTACHED	(DTAPI_E + 21)
#define  DTAPI_E_NOT_FOUND		(DTAPI_E + 22)
#define  DTAPI_E_NOT_SUPPORTED	(DTAPI_E + 23)
#define  DTAPI_E_PCICARD		(DTAPI_E + 24)	// BACKWARD COMPATIBILITY
#define  DTAPI_E_DEVICE			(DTAPI_E + 24)
#define  DTAPI_E_TOO_LONG		(DTAPI_E + 25)
#define  DTAPI_E_UNDERFLOW		(DTAPI_E + 26)
#define  DTAPI_E_NO_SUCH_INPUT	(DTAPI_E + 27)
#define  DTAPI_E_NO_DT_INPUT		(DTAPI_E + 28)
#define  DTAPI_E_NO_TS_INPUT	(DTAPI_E + 28)
#define  DTAPI_E_DRIVER_INCOMP	(DTAPI_E + 29)
#define  DTAPI_E_INTERNAL		(DTAPI_E + 30)
#define  DTAPI_E_OUT_OF_MEM			(DTAPI_E + 31)
#define  DTAPI_E_INVALID_J83ANNEX	(DTAPI_E + 32)
#define  DTAPI_E_INVALID_ROLLOFF	(DTAPI_E + 32)
#define  DTAPI_E_IDLE				(DTAPI_E + 33)
#define  DTAPI_E_INSUF_LOAD			(DTAPI_E + 34)
#define  DTAPI_E_INVALID_BANDWIDTH	(DTAPI_E + 35)
#define  DTAPI_E_INVALID_CONSTEL	(DTAPI_E + 36)
#define  DTAPI_E_INVALID_GUARD		(DTAPI_E + 37)
#define  DTAPI_E_INVALID_INTERLVNG	(DTAPI_E + 38)
#define  DTAPI_E_INVALID_TRANSMODE	(DTAPI_E + 39)
#define  DTAPI_E_INVALID_TSTYPE		(DTAPI_E + 40)
#define  DTAPI_E_NO_IPPARS			(DTAPI_E + 41)
#define  DTAPI_E_NO_TSRATE			(DTAPI_E + 42)
#define  DTAPI_E_NOT_IDLE			(DTAPI_E + 43)
#define  DTAPI_E_INVALID_ARG		(DTAPI_E + 44)
#define  DTAPI_E_NDIS_DRIVER		(DTAPI_E + 45)
#define  DTAPI_E_DST_MAC_ADDR		(DTAPI_E + 46)
#define  DTAPI_E_NO_SUCH_PORT		(DTAPI_E + 47)
#define  DTAPI_E_WINSOCK			(DTAPI_E + 48)
#define  DTAPI_E_MULTICASTJOIN		(DTAPI_E + 49)
#define  DTAPI_E_EMBEDDED			(DTAPI_E + 50)
#define  DTAPI_E_LOCKED				(DTAPI_E + 51)
#define  DTAPI_E_NO_VALID_CALDATA	(DTAPI_E + 52)
#define  DTAPI_E_NO_LINK			(DTAPI_E + 53)
#define  DTAPI_E_INVALID_HEADER		(DTAPI_E + 54)


//=+=+=+=+=+=+=+=+=+=+=+=+=+=+ BACKWARD COMPATIBILITY SECTION +=+=+=+=+=+=+=+=+=+=+=+=+=+=

// Map old PciCard class on DtDevice
typedef class DtDevice PciCard;

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DtapiHwFunc -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Structure that describes a hardware function on a PCI card.
//
struct DtapiHwFunc {
	int  m_nPciBusNumber;		// PCI-bus number
	int  m_nSlotNumber;			// PCI-slot number
	int  m_nTypeNumber;			// PCI-card type number
	int  m_nDeviceId;			// Device ID of PCI card
	int  m_nVendorId;			// Vendor ID of PCI card
	int  m_nSubsystemId;		// Subsystem ID of PCI card
	int  m_nSubVendorId;		// Subsystem Vendor ID of PCI card
	int  m_nHwFuncType;			// Hardware-function type
	int  m_nHwFuncFlags;		// Hardware-function flags
	int  m_nIndexOnCard;		// Index of hardware function
};

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Global functions -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
DTAPI_RESULT  DtapiHwFuncScan(int NumEntries, int& NumEntriesResult,
							  DtHwFuncDesc* pHwFuncs);
DTAPI_RESULT  DtapiPciScan(int NumFuncEntries, int& NumFuncEntriesResult,
						   DtapiHwFunc* pHwFuncs);

const char*  DtapiResult2Str(DTAPI_RESULT DtapiResult);

//=+=+=+=+=+=+=+=+=+=+=+=+ END OF BACKWARD COMPATIBILITY SECTION +=+=+=+=+=+=+=+=+=+=+=+=+

#endif //#ifndef __DTAPI_H
