//#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* DeviceVpd.cpp *#*#*#*#*#*#*#*# (C) 2000-2004 DEKTEC
//

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- License -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// This program is free software; You can freely use/modify this source code in any way
// you desire, including for commercial applications.
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	2004.10.21	MG	Created	based on DTAPI for windows

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#include "StdAfx.h"


//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DtDevice::VpdDelete -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Delete a Vital-Product Data (VPD) item from the VPD read/write section in the serial
// EEPROM on the PCI card.
//
DTAPI_RESULT  DtDevice::VpdDelete(
	char*  pKeyword)		// Keyword identifying VPD item, e.g. "Y0"
{
	DtDeviceInt*  pDvcInt = (DtDeviceInt*) this;	// For calling internal functions
	DTAPI_RESULT  dr;

	// Must be attached to PCI card
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	// Read raw data in VPD read/write resources area
	unsigned char  VpdRwSection[256];
	CHECKDR(pDvcInt->VpdReadRaw(VpdRwSection, 0x100, 256))

	// Delete resource from VpdRwSection array
	dr = pDvcInt->VpdDeleteResource(pKeyword, VpdRwSection, 255);
	if (dr == DTAPI_E_NOT_FOUND)
		return dr;

	// Write resources back to EEPROM
	dr = pDvcInt->VpdWriteRaw(VpdRwSection, 0x100, 0x100);
	if (dr != DTAPI_OK)
		return dr;

	return DTAPI_OK;
}
/*
// UNICODE variation
DTAPI_RESULT  DtDevice::VpdDelete(wchar_t* pKeyword)
{
	// Convert UNICODE keyword to char string
	int  Len = wcslen(pKeyword);
	if (Len > 5)
		return DTAPI_E_KEYWORD;
	char  Keyword[8];		// Keyword as char string
	for (int i=0; i<Len; i++)
		Keyword[i] = char(pKeyword[i]);
	Keyword[i] = '\0';

	return VpdDelete(Keyword);
}
*/


//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DtDevice::VpdRead -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Read a Vital-Product Data (VPD) item from the EEPROM on the device.
//
DTAPI_RESULT  DtDevice::VpdRead(
	char*  pKeyword,
	char*  pVpdItem)
{
	DtDeviceInt*  pDvcInt = (DtDeviceInt*) this;	// For calling internal functions
	DTAPI_RESULT  dr;
	char*  pTo = pVpdItem;							// For copying VPD item

	// Must be attached to Device
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	// Be careful
	if (pKeyword == NULL)
		return DTAPI_E_KEYWORD;

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- VPDID -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	// Check for special keyword "VPDID"
	if (strcmp(pKeyword, "VPDID") == 0) {
		unsigned char  VpdIdSection[EEPROM_VPD_ID_RO_LENGTH];
		// Read raw data in VPD ID String area
		CHECKDR(pDvcInt->VpdReadRaw(VpdIdSection, EEPROM_VPD_ID_START,
									EEPROM_VPD_ID_RO_LENGTH))
		int StartIdx;
		if (VpdIdSection[0] == 0x82)
			StartIdx = 0;
		else if (VpdIdSection[11] == 0x82)
			StartIdx = 11;
		else
			return DTAPI_E_EEPROM_FORMAT;

		unsigned int  Length = VpdIdSection[StartIdx+1] + (VpdIdSection[StartIdx+2]<<8);
		if (Length > VPD_ITEM_MAX_LENGTH)
			Length = VPD_ITEM_MAX_LENGTH;
	 	char*  pFrom = (char*) VpdIdSection + StartIdx + 3;
		for (unsigned int i=0; i<Length; i++)
			*pTo++ = *pFrom++;
		*pTo = '\0';
		
		return DTAPI_OK;
	}

	// Keyword must be two alpha-numeric characters
	if (strlen(pKeyword) != 2)
		return DTAPI_E_KEYWORD;
	if (!VALID_VPD_KEYW_CHAR(pKeyword[0]) || !VALID_VPD_KEYW_CHAR(pKeyword[1]))
		return DTAPI_E_KEYWORD;

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Check Read-Only Resources -.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	// Read VPD ID string- and read-only- section.
	unsigned char  VpdIdRo[EEPROM_VPD_ID_RO_LENGTH];
	CHECKDR(pDvcInt->VpdReadRaw(VpdIdRo, EEPROM_VPD_ID_START, EEPROM_VPD_ID_RO_LENGTH))


		int StartIdx;
		if (VpdIdRo[0] == 0x82)
			StartIdx = 0;
		else if (VpdIdRo[11] == 0x82)
			StartIdx = 11;
		else
		return DTAPI_E_EEPROM_FORMAT;

	unsigned int  VpdIdLength = 3 + VpdIdRo[StartIdx+1] + (VpdIdRo[StartIdx+2]<<8);

	// Start of VPD read-only section.
	unsigned char*  pVpdRo = VpdIdRo + StartIdx + VpdIdLength;
	if (pVpdRo >= VpdIdRo+EEPROM_VPD_ID_RO_LENGTH)
		return DTAPI_E_EEPROM_FORMAT;
	if (pVpdRo[0] != 0x90)
		return DTAPI_E_EEPROM_FORMAT;
	unsigned int  RoLength = &VpdIdRo[EEPROM_VPD_ID_RO_LENGTH] - pVpdRo;

	// If resource is found => copy to VpdItem.
	unsigned char*  pResource;		// Resource to be read
	if (pDvcInt->VpdFindResource(pKeyword, pVpdRo, RoLength, pResource)) {
 		char*  pFrom = (char*) pResource + 3;
		unsigned int  Length = pResource[2];
		if (Length > VPD_ITEM_MAX_LENGTH)
			Length = VPD_ITEM_MAX_LENGTH;	// Protect against invalid content of EEPROM
		for (unsigned int i=0; i<Length; i++)
			*pTo++ = *pFrom++;
		*pTo = '\0';

		return DTAPI_OK;
	}

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Check Read-Write Resources -.-.-.-.-.-.-.-.-.-.-.-.-.-.

	// Read VPD read/write section
	unsigned char  VpdRwSection[EEPROM_VPD_RW_LENGTH];
	CHECKDR(pDvcInt->VpdReadRaw(VpdRwSection, EEPROM_VPD_RW_START, EEPROM_VPD_RW_LENGTH))

	// If resource is found => copy to VpdItem.
	if (pDvcInt->VpdFindResource(pKeyword, VpdRwSection, EEPROM_VPD_RW_LENGTH,
																			 pResource)) {
 		char*  pFrom = (char*)pResource + 3;
		unsigned int  Length = pResource[2];
		if (Length > VPD_ITEM_MAX_LENGTH)
			Length = VPD_ITEM_MAX_LENGTH;	// Protect against invalid content of EEPROM
		for (unsigned int i=0; i<Length; i++)
			*pTo++ = *pFrom++;
		*pTo = '\0';
		return DTAPI_OK;
	}

	return DTAPI_E_NOT_FOUND;
}
/*
// UNICODE variation
DTAPI_RESULT  DtDevice::VpdRead(wchar_t* pKeyword, wchar_t* pVpdItem)
{
	// Convert UNICODE keyword to char string
	int  KeywordLen = wcslen(pKeyword);
	if (KeywordLen > 5)
		return DTAPI_E_KEYWORD;
	char  Keyword[8];		// Keyword as char string
	for (int i=0; i<KeywordLen; i++)
		Keyword[i] = char(pKeyword[i]);
	Keyword[i] = '\0';

	// Let char version of VpdRead do the work
	char  VpdItem[VPD_ITEM_MAX_LENGTH+1];		// VPD item as char string
	DTAPI_RESULT  dr = VpdRead(Keyword, VpdItem);
	if (dr != DTAPI_OK)
		return dr;

	// Convert VPD item string to UNICODE
	int  VpdItemLen = strlen(VpdItem);
	if (VpdItemLen > VPD_ITEM_MAX_LENGTH)
		return DTAPI_E_EEPROM_FORMAT;			// Error in EEPROM format
	for (int i=0; i<VpdItemLen; i++)
		pVpdItem[i] = wchar_t(VpdItem[i]);
	pVpdItem[i] = L'\0';

	return dr;
}
*/

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DtDevice::VpdWrite -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Write a Vital-Product Data (VPD) item to the VPD read/write section in the serial
// EEPROM on the device.
//
DTAPI_RESULT  DtDevice::VpdWrite(
	char*  pKeyword,		// Keyword identifying VPD item, e.g. "Y1"
	char*  pVpdItem)		// String to be written to the EEPROM
{
	DtDeviceInt*  pDvcInt = (DtDeviceInt*) this;	// For calling internal functions
	DTAPI_RESULT  dr;

	// Must be attached to PCI card
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	// Check string length of Keyword
	if (strlen(pKeyword) > 2)
		return DTAPI_E_KEYWORD;

	// Read raw data in VPD read/write resources area
	unsigned char  VpdRwSection[256];
	dr = pDvcInt->VpdReadRaw(VpdRwSection, 0x100, 256);
	if (dr != DTAPI_OK)
		return dr;

	// Check structure in serial EEPROM
	if (VpdRwSection[0]!=0x91 || VpdRwSection[1]!=0xFC || VpdRwSection[2]!=0x00
													   || VpdRwSection[0xFF]!=0x78) {

		// Basic VPD Read/Write Resources data structure not present. Create it.
		for (unsigned int i=0; i<256; i++)
			VpdRwSection[i] = 0x00;
		VpdRwSection[0] = 0x91;
		VpdRwSection[1] = 0xFC;
		VpdRwSection[0xFF] = 0x78;
		dr = pDvcInt->VpdWriteRaw(VpdRwSection, 0x100, 0x100);
		if (dr != DTAPI_OK)
			return dr;
	}

	// If resource present => delete resource first
	pDvcInt->VpdDeleteResource(pKeyword, VpdRwSection, 255);

	// Get #bytes occupied by VPD fields
	unsigned int  VpdSectLength = pDvcInt->VpdSectionLength(VpdRwSection, 255);
	unsigned char*  pNewRes = VpdRwSection + VpdSectLength;

	// Check whether new resource fits
	unsigned int  StrLength = strlen(pVpdItem);		// Length of VPD item without header
	unsigned int  ResLength = 3 + StrLength;		// Includes 3-byte header
	if (3 + VpdSectLength + ResLength > 255)
		return DTAPI_E_EEPROM_FULL;

	// Append new resource
	unsigned char* pRes = pNewRes;
	*pRes++ = pKeyword[0];
	*pRes++ = pKeyword[1];
	*pRes++ = StrLength;
	strncpy((char*)pRes, pVpdItem, StrLength);

	// Write resources back to EEPROM
	dr = pDvcInt->VpdWriteRaw(VpdRwSection, 0x100, 0x100);
	if (dr != DTAPI_OK)
		return dr;

	return DTAPI_OK;
}
/*
// UNICODE variation
DTAPI_RESULT  DtDevice::VpdWrite(wchar_t* pKeyword, wchar_t* pVpdItem)
{
	// Convert UNICODE keyword to char string
	int  KeywordLen = wcslen(pKeyword);
	if (KeywordLen > 2)
		return DTAPI_E_KEYWORD;
	char  Keyword[3];		// Keyword as char string
	for (int i=0; i<KeywordLen; i++)
		Keyword[i] = char(pKeyword[i]);
	Keyword[i] = '\0';

	// Convert UNICODE VPD item to char string
	int  VpdItemLen = wcslen(pVpdItem);
	if (VpdItemLen > VPD_ITEM_MAX_LENGTH)
		return DTAPI_E_KEYWORD;
	char  VpdItem[VPD_ITEM_MAX_LENGTH+1];		// VPD item as char string
	for (int i=0; i<VpdItemLen; i++)
		VpdItem[i] = char(pVpdItem[i]);
	VpdItem[i] = '\0';

	// Let char version of VpdWrite do the work
	return VpdWrite(Keyword, VpdItem);
}
*/

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Internal Helper Functions +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+


//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DtDeviceInt::GetSerialFromVpd -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Get device's serial number from VPD resource "SN".
// This function is used to initialise the serial number in a device descriptor.
// Returns 0 in case of errors.
//
__int64  DtDeviceInt::GetSerialFromVpd(void)
{
	// Read VPD resource
	char  VpdStr[64];
	if (VpdRead("SN", VpdStr) != DTAPI_OK)
		return 0;

	// Convert to __int64
	return (__int64)atoll(VpdStr);
}


//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DtDeviceInt::VpdDeleteResource -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Keyword-based VPD-resource search in VPD section
//
DTAPI_RESULT  DtDeviceInt::VpdDeleteResource(
	char*   pKeyword,		// Keyword of resource to be deleted
	unsigned char*  pSection,		// VPD section (starts with VPD tag)
	unsigned int    SectLength)		// Real section length (should correspond to pSection[1..2]
{
	unsigned char*  pResource;		// Resource to be deleted

	// If resource cannot be found => return appropriate error code
	if (!VpdFindResource(pKeyword, pSection, SectLength, pResource))
		return DTAPI_E_NOT_FOUND;

	// Delete resource in a loop, so that multiple instances are deleted too.
	do {
		unsigned char*  pTo = pResource;
		unsigned char*  pNext = pResource + 3 + pResource[2];		// Next resource
		while (VpdIsValidResource(pNext, pSection, SectLength)) {
			unsigned int  NextResourceLength = 3 + pNext[2];
			for (unsigned int i=0; i<NextResourceLength; i++)
				*pTo++ = *pNext++;
		}
		// Fill rest of section with zero's
		while (pTo < pSection + SectLength)
			*pTo++ = 0;
	} while (VpdFindResource(pKeyword, pSection, SectLength, pResource));

	return DTAPI_OK;
}


//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DtDeviceInt::VpdFindResource -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Keyword-based VPD-resource search in VPD section
//
bool  DtDeviceInt::VpdFindResource(
	char*   pKeyword,		// Keyword to be found
	unsigned char*  pSection,		// VPD section (starts with VPD tag)
	unsigned int    SectLength,		// Real section length (should correspond to pSection[1..2]
	unsigned char*& pResource)		// Points to VPD resource if keyword is found
{
	unsigned char*  pRes = pSection + 3;

	while (VpdIsValidResource(pRes, pSection, SectLength)) {
		if (strncmp((char*) pRes, (char*) pKeyword, 2) == 0) {
			pResource = pRes;
			return true;
		}
		unsigned int  ResLength = pRes[2] + 3;
		pRes += ResLength;
	}
	// No more valid keywords => can't find keyword
	return false;
}


//-.-.-.-.-.-.-.-.-.-.-.-.-.- DtDeviceInt::VpdIsValidResource -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
bool  DtDeviceInt::VpdIsValidResource(
	unsigned char*  pResource,		// Resource to be checked for validity
	unsigned char*  pSection,		// Section in which keyword is located
	unsigned int    SectLength)		// Real section length (should correspond to pSection[1..2]
{
	if (pResource > pSection + SectLength - 3)
		return false;
	if (!VALID_VPD_KEYW_CHAR(pResource[0]) || !VALID_VPD_KEYW_CHAR(pResource[1]))
		return false;
	return (pResource + 3 + (unsigned int)(pResource[2]) <= pSection + SectLength);
}


//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DtDeviceInt::VpdSectionLength -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Determine the length of a VPD section by walking through the VPD resources.
// Length EXCLUDES stuffing bytes.
//
unsigned int  DtDeviceInt::VpdSectionLength(
	unsigned char*  pSection,		// Points to VPD-tag that starts the section
	unsigned int    Length)			// Total length of section, including stuffing
{
	unsigned int  LengthSoFar = 3;
	unsigned char*  pResource = pSection + 3;				// Skip VPD-tag and 2-byte length

	while (VpdIsValidResource(pResource, pSection, Length)) {
		unsigned int  ResLength = 3 + pResource[2];			// 3 for keyword + 1-byte length
		pResource += ResLength;
		Length -= ResLength;
		LengthSoFar += ResLength;
	}
	return LengthSoFar;
}


//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Low-Level Read / Write +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=


//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DtDeviceInt::VpdReadRaw -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Read raw VPD bytes.
//
DTAPI_RESULT  DtDeviceInt::VpdReadRaw(
	unsigned char*  pBuffer,// Buffer to receive data
	unsigned int    Start,			// Start address in serial EEPROM
	unsigned int    Length)			// Number of bytes to read
{
	DTAPI_RESULT dr;

	// Processing depends on device category
	if (Category() == DTAPI_CAT_PCI) {
		
		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));

		iod.m_nChIdx				= 0;
		iod.m_Data.m_Vpd.m_nStart	= Start;
		iod.m_Data.m_Vpd.m_nLength	= Length;
		iod.m_Data.m_Vpd.m_pVpdBuf	= pBuffer;

		dr = ioctl(m_fdDriver, DTA1XX_IOCTL_READ_VPD, &iod);
		if (dr != DTAPI_OK)
			return DTAPI_E_EEPROM_READ;

		// Copy data from VPD-transfer buffer to caller-supplied buffer
//		for (unsigned int i=0; i<Length; i++)
//			pBuffer[i] = Dta1xxVpdTransfer.m_VpdBuf[i];
	}
	else if (Category() == DTAPI_CAT_USB) {
		
		DTU2XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTU2XX_IOCTL_DATA));

		iod.m_nChIdx				= 0;
		iod.m_Data.m_Vpd.m_nStart	= Start;
		iod.m_Data.m_Vpd.m_nLength	= Length;
		iod.m_Data.m_Vpd.m_pVpdBuf	= pBuffer;

		dr = ioctl(m_fdDriver, DTU2XX_IOCTL_READ_VPD, &iod);
		if (dr != DTAPI_OK)
			return DTAPI_E_EEPROM_READ;
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}


//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DtDevice::VpdWriteRaw -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Write raw VPD bytes.
//
DTAPI_RESULT  DtDeviceInt::VpdWriteRaw(
	unsigned char*  pBuffer,		// Buffer with data to be written
	unsigned int    Start,			// Start address in serial EEPROM
	unsigned int    Length)			// Number of bytes to write
{
	DTAPI_RESULT dr;

	// Processing depends on device category
	if (Category() == DTAPI_CAT_PCI) {

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx				= 0;
		iod.m_Data.m_Vpd.m_nStart	= Start;
		iod.m_Data.m_Vpd.m_nLength	= Length;
		iod.m_Data.m_Vpd.m_pVpdBuf	= pBuffer;

		// Copy data from caller-supplied buffer to VPD-transfer buffer
//		for (unsigned int i=0; i<Length; i++)
//			Dta1xxVpdTransfer.m_VpdBuf[i] = pBuffer[i];

		dr = ioctl(m_fdDriver, DTA1XX_IOCTL_WRITE_VPD, &iod);

		if (dr != DTAPI_OK)
			return DTAPI_E_EEPROM_WRITE;
	}
	else if (Category() == DTAPI_CAT_USB) {

		DTU2XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTU2XX_IOCTL_DATA));

		iod.m_nChIdx				= 0;
		iod.m_Data.m_Vpd.m_nStart	= Start;
		iod.m_Data.m_Vpd.m_nLength	= Length;
		iod.m_Data.m_Vpd.m_pVpdBuf	= pBuffer;

		dr = ioctl(m_fdDriver, DTU2XX_IOCTL_WRITE_VPD, &iod);
		if (dr != DTAPI_OK)
			return DTAPI_E_EEPROM_WRITE;
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}
