//*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* DtDevice.cpp *#*#*#*#*#*#*#*# (C) 2000-2004 DEKTEC

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- License -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// This program is free software; You can freely use/modify this source code in any way
// you desire, including for commercial applications.
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	TD	2007.01.31	Dta1xx IO configuration is read from the driver
//	TD	2007.01.29	Added support for DTA-160 ASI-only
//	TD	2007.01.23	Added support for the DTA-145 and DTA-2145
//	2004.10.21	MG	Created	based on DTAPI for windows

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#include "StdAfx.h"

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ DtDevice implementation +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DtDevice::DtDevice -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
DtDevice::DtDevice() :
	m_Attached(false)
{
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DtDevice::~DtDevice -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
DtDevice::~DtDevice()
{
	if (m_Attached)
		close(m_fdDriver);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DtDevice::AttachToSerial -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Attach device object to the device hardware, based on serial number.
//
DTAPI_RESULT  DtDevice::AttachToSerial(
	__int64  SerialNumber)				// Serial Number
{
	// Cannot attach twice
	if (m_Attached)
		return DTAPI_E_ATTACHED;

	// Let ::DtapiDeviceScan do the work
	int  Dummy;
	return ::DtapiDeviceScan(ATTACH_TO_SERIAL, 0, Dummy, NULL, this,
							 0, 0, 0, 0, SerialNumber);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DtDevice::AttachToSlot -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Attach device object to the hardware, based on PCI-bus number and slot number.
//
DTAPI_RESULT  DtDevice::AttachToSlot(
	int  PciBusNumber,
	int  SlotNumber)
{
	// Cannot attach twice
	if (m_Attached)
		return DTAPI_E_ATTACHED;

	// Let ::DtapiDeviceScan do the work
	int  Dummy;
	return ::DtapiDeviceScan(ATTACH_TO_SLOT, 0, Dummy, NULL, this,
							 PciBusNumber, SlotNumber, 0, 0, 0);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DtDevice::AttachToType -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Attach device object to the device hardware, based on type number.
//
DTAPI_RESULT  DtDevice::AttachToType(
	int  TypeNumber,			// Which device to look for
	int  DeviceNo)				// Relative device number
{
	// Cannot attach twice
	if (m_Attached)
		return DTAPI_E_ATTACHED;

	// Let ::DtapiDeviceScan do the work
	int  Dummy;
	return ::DtapiDeviceScan(ATTACH_TO_TYPE, 0, Dummy, NULL, this,
							 0, 0, TypeNumber, DeviceNo, 0);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DtDevice::Detach -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
DTAPI_RESULT  DtDevice::Detach()
{
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	// On PCI cards with a LED: release direct control of general-status LED
	// On USB devices with a LED: release direct control of general and Rx status LED
	if (Category() == DTAPI_CAT_PCI) {
		if (   TypeNumber()==120 || TypeNumber()==122 || TypeNumber()==124
			|| TypeNumber()==140 || TypeNumber()==145 || TypeNumber()==2145
			|| TypeNumber()==160)
		{
			LedControl(DTAPI_LED_HARDWARE);
		}
	}
	else if (Category() == DTAPI_CAT_USB) {
		if ( TypeNumber()==205 || TypeNumber()==225 || TypeNumber()==245 )
		{
			LedControl(DTAPI_LED_HARDWARE);
		}
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	// Close driver and mark device structure as "not attached"
	close(m_fdDriver);
	m_Attached = false;

	return DTAPI_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DtDevice::GetDescriptor -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Get device descriptor.
//
DTAPI_RESULT  DtDevice::GetDescriptor(
	DtDeviceDesc&  DvcDesc)
{
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DvcDesc = m_DvcDesc;
	return DTAPI_OK;
}


//-.-.-.-.-.-.-.-.-.-.-.-.-.- DtDevice::GetDeviceDriverVersion -.-.-.-.-.-.-.-.-.-.-.-.-.-
//
DTAPI_RESULT  DtDevice::GetDeviceDriverVersion(
	int&  DriverVersionMajor,
	int&  DriverVersionMinor)
{
	int n;
    return GetDeviceDriverVersion(DriverVersionMajor, DriverVersionMinor, n, n);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.- DtDevice::GetDeviceDriverVersion -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Second overloaded version of GetDeviceDriverVersion which also returns bugfix- and
// build number.
//
DTAPI_RESULT  DtDevice::GetDeviceDriverVersion(
	int&  DriverVersionMajor,
	int&  DriverVersionMinor,
	int&  DriverVersionBugFix,
	int&  DriverVersionBuild)
{
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	// Device-category dependent processing for reading device driver version
	if (Category() == DTAPI_CAT_PCI) {
		
		DTA1XX_IOCTL_DATA iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		DTAPI_RESULT dr = ioctl(m_fdDriver, DTA1XX_IOCTL_GET_DRIVER_VERSION, &iod);
		if (dr!=DTAPI_OK ) {
			return DTAPI_E_DEV_DRIVER;
		}
		
		// Store the driver version
		DriverVersionMajor  = iod.m_Data.m_DriverVersion.m_nVersionMajor;
		DriverVersionMinor  = iod.m_Data.m_DriverVersion.m_nVersionMinor;
		DriverVersionBugFix = iod.m_Data.m_DriverVersion.m_nVersionMicro;
		DriverVersionBuild  = iod.m_Data.m_DriverVersion.m_nVersionBuild;
	}
	else if (Category() == DTAPI_CAT_USB) {

		DTU2XX_IOCTL_DATA iod;
		memset(&iod, 0, sizeof(DTU2XX_IOCTL_DATA));
		DTAPI_RESULT dr = ioctl(m_fdDriver, DTU2XX_IOCTL_GET_DRIVER_VERSION, &iod);
		if (dr!=DTAPI_OK ) {
			return DTAPI_E_DEV_DRIVER;
		}
		
		// Store the driver version
		DriverVersionMajor  = iod.m_Data.m_DriverVersion.m_nVersionMajor;
		DriverVersionMinor  = iod.m_Data.m_DriverVersion.m_nVersionMinor;
		DriverVersionBugFix = iod.m_Data.m_DriverVersion.m_nVersionMicro;
		DriverVersionBuild  = iod.m_Data.m_DriverVersion.m_nVersionBuild;
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DtDevice::GetFirmwareVersion -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Get version of the firmware loaded on the device.
//
DTAPI_RESULT  DtDevice::GetFirmwareVersion(
	int&  FirmwareVersion)
{
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	// Get firmware version from device descriptor
	FirmwareVersion = m_DvcDesc.m_FirmwareVersion;
 
	return DTAPI_OK;
}
//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DtDevice::GetIoConfig -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Reconfigures the number of IO ports on a device. Only supported by some devices 
//
// Port: 1.. max number of ports
//
DTAPI_RESULT  DtDevice::GetIoConfig(
	Int  Port,				// Port number
	Int&  IoConfig)			// New I/O Configuration
{
	// Must be attached
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	// Check port number
	if (Port<1 || Port>m_DvcDesc.m_NumHwFuncs)
		return DTAPI_E_NO_SUCH_PORT;

	return (__DTAPI::GetIoConfigViaDta1xx(m_fdDriver, Port-1, IoConfig)
			? DTAPI_OK : DTAPI_E_DEV_DRIVER);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DtDevice::GetRefClkCnt -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Get a sample of the reference-clock counter.
//
DTAPI_RESULT  DtDevice::GetRefClkCnt(
	int&  RefClkCnt)				// Sample of reference-clock counter
{
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	// Does board support reference-clock-count register?
	if ( !(   (TypeNumber()==105) || (TypeNumber()==107) || (TypeNumber()==110) 
		   || (TypeNumber()==120 && FirmwareVersion()>=4)
		   || (TypeNumber()==122 && FirmwareVersion()>=4)
		   || (TypeNumber()==124) || (TypeNumber()==140) || (TypeNumber()==145)
		   || (TypeNumber()==160) || (TypeNumber()==2145) ) )
		return DTAPI_E_NOT_SUPPORTED;

	// Get the sample
	if (Category() == DTAPI_CAT_PCI) {
		DTA1XX_IOCTL_DATA iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		DTAPI_RESULT dr = ioctl(m_fdDriver, DTA1XX_IOCTL_GET_REF_CLOCK_COUNT, &iod);
		if (dr!=DTAPI_OK ) {
			return DTAPI_E_DEV_DRIVER;
		}
		RefClkCnt = iod.m_Data.m_uiValue;
	}
	else if (Category() == DTAPI_CAT_USB)
	{
		// NOT SUPPORTED YET
		return DTAPI_E_INTERNAL;		 // Error in Device Category
	} else
		return DTAPI_E_INTERNAL;		 // Error in Device Category
	


	return DTAPI_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DtDevice::LedControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Take direct control of general-status LED, or let hardware drive the LED.
//
DTAPI_RESULT  DtDevice::LedControl(
	int  LedControl)		// DTAPI_LED_XXX
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	// Check mode
	if (LedControl!=DTAPI_LED_HARDWARE && LedControl!=DTAPI_LED_OFF &&
		LedControl!=DTAPI_LED_GREEN && LedControl!=DTAPI_LED_RED &&
		LedControl!=DTAPI_LED_YELLOW)
		return DTAPI_E_INVALID_MODE;

	// Check whether the card has a LED
	if (   TypeNumber()!=120 && TypeNumber()!=122 && TypeNumber()!=124 && TypeNumber()!=140
		&& TypeNumber()!=145 && TypeNumber()!=2145 && TypeNumber()!=160
		&& TypeNumber()!=205 && TypeNumber()!=225 && TypeNumber()!=245 )
		return DTAPI_E_NOT_SUPPORTED;

	// Set LED
	if (Category() == DTAPI_CAT_PCI) {
		// ADD IMPLEMENTATION
	}
	else if (Category() == DTAPI_CAT_USB)
	{
		DTU2XX_IOCTL_DATA iod;
		memset(&iod, 0, sizeof(DTU2XX_IOCTL_DATA));
		iod.m_Data.m_uiValue = LedControl;

		DTAPI_RESULT dr = ioctl(m_fdDriver, DTU2XX_IOCTL_LED_CONTROL, &iod);
		if (dr!=DTAPI_OK ) {
			return DTAPI_E_DEV_DRIVER;
		}
	} else
		return DTAPI_E_INTERNAL;		 // Error in Device Category

	return DTAPI_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DtDevice::SetIoConfig -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
// 
// Reconfigures the number of IO ports on a device. Only supported by some devices and
// particular ChannelTypes.
//
// Port: 1.. max number of ports
//
DTAPI_RESULT  DtDevice::SetIoConfig(
	Int  Port,				// Port number
	Int  IoConfig,			// New I/O Configuration
	Int  ParXtra)			// Extra config option (not used for the moment)
{
	__int64 SerialNo;

	// Must be attached
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	// Check port number
	if (Port<1 || Port>m_DvcDesc.m_NumHwFuncs)
		return DTAPI_E_NO_SUCH_PORT;

	if (!__DTAPI::SetIoConfigViaDta1xx(m_fdDriver, Port-1, IoConfig))
		return DTAPI_E_DEV_DRIVER;

	// For now, do a complete detach and attach again
	SerialNo = m_DvcDesc.m_Serial;

	// Do a Detach and AttachToSerial to rebuild the internal device structure
	Detach();
	
	return AttachToSerial(SerialNo);
}
