//*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* TsInpChannel.cpp *#*#*#*#*#*#*# (C) 2000-2004 DEKTEC
//
// Class TsInpChannel: implementation of general methods.

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- License -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// This program is free software; You can freely use/modify this source code in any way
// you desire, including for commercial applications.
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	TD	2007.01.23	Added support for the DTA-145 and DTA-2145
//  MG  2004.10.21  Adapted for new DtDevice class
//	MG	19-11-2002	Created	based on DTAPI for windows

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#include "StdAfx.h"

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Constants -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ TsInpChannel implementation +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsInpChannel::TsInpChannel -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
TsInpChannel::TsInpChannel() :
	m_Attached(false)
{
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsInpChannel::~TsInpChannel -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
TsInpChannel::~TsInpChannel()
{
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsInpChannel::Attach -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Attach the Transport-Stream input-channel object to a hardware function hosted
// by a device.
//
DTAPI_RESULT  TsInpChannel::Attach(
	DtDevice*  pDtDvc,		// Object representing DEKTEC Device
	int  InpIndex)			// Input index on DEKTEC Device
{
	// Cannot attach twice
	if (m_Attached)
		return DTAPI_E_ATTACHED;

	// pDtDvc must be valid and attached to the hardware
	if (pDtDvc==NULL || !pDtDvc->m_Attached)
		return DTAPI_E_DEVICE;

	// Do we have a valid input channel?
	if (  InpIndex<0 || InpIndex>=pDtDvc->m_DvcDesc.m_NumDtInpChan )
		return DTAPI_E_NO_TS_INPUT;

	// Duplicate handle from DtDevice object to TsInpChannel object.
	int fd = dup(pDtDvc->m_fdDriver);
	if ( fd == -1 )
		return DTAPI_E_DEV_DRIVER;
	m_fdDevice = fd;


	// Create hardware-function descriptor
	// Must be here, because other functions use m_HwFuncDesc.m_DvcDesc.m_Category
	m_HwFuncDesc.m_DvcDesc	  = pDtDvc->m_DvcDesc;	// Copy device-descriptor part
	m_HwFuncDesc.m_ChanType	  = DTAPI_TS_INPUT;		// Should be an input!
	m_HwFuncDesc.m_Flags	  = 0;					// Flags not used yet
	m_HwFuncDesc.m_IndexOnDvc = InpIndex;			// Use the selected input

	// Assume success for other initialisation
	DTAPI_RESULT  Status = DTAPI_OK;

	// Device-category dependent processing
	if (pDtDvc->Category() == DTAPI_CAT_PCI) {

		// Request exclusive access

		// Attached!
		m_Attached = true;

		// Initialise hardware
		Reset(DTAPI_FIFO_RESET);
		SetLoopBackMode(DTAPI_NO_LOOPBACK);		// No Loop-Back Mode
		ClearFlags(-1);							// Clear all latched flags

		// If an error occurred => release exclusive-access lock and duplicated handle.
		if (Status != DTAPI_OK) {
			close(m_fdDevice);
			m_Attached = false;
		}
	}
	else if (pDtDvc->Category() == DTAPI_CAT_USB) {

		// Request exclusive access

		// Attached!
		m_Attached = true;

		// Initialise hardware
		Reset(DTAPI_FIFO_RESET);
		ClearFlags(-1);							// Clear all latched flags

		// If an error occurred => release exclusive-access lock and duplicated handle.
		if (Status != DTAPI_OK) {
			close(m_fdDevice);
			m_Attached = false;
		}
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return Status;
}
// FOR NOW: simple call attach (regard less of port)
DTAPI_RESULT  TsInpChannel::AttachToPort(
	DtDevice*  pDtDvc,		// Object representing DEKTEC Device
	int  Port)				// Input index on DEKTEC Device
{
	// NOTE: we assume here that the lowest port numbers are the inputs
	if ( Port <=0 || Port>pDtDvc->m_DvcDesc.m_NumDtInpChan )
		return DTAPI_E_NO_SUCH_PORT;

	return Attach(pDtDvc, Port-1);
}


//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsInpChannel::ClearFifo -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Clear contents of the Receive FIFO and set receive-control state to DTAPI_RXCTRL_IDLE.
// Clear status flag DTAPI_RX_FIFO_OVF.
//
DTAPI_RESULT  TsInpChannel::ClearFifo(void)
{
	return Reset(DTAPI_FIFO_RESET);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsInpChannel::ClearFlags -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
DTAPI_RESULT  TsInpChannel::ClearFlags(
	int  Latched)			// Latched status flags to be cleared
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		// Clear flags through device driver
		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();
		iod.m_Data.m_uiValue = Latched;

		dr = ioctl(m_fdDevice, DTA1XX_RX_IOCTL_CLEAR_FLAGS, &iod);

		if ( dr != DTAPI_OK )
			return dr;
	}
	else if (Category() == DTAPI_CAT_USB) {

		// Clear flags through device driver
		DTU2XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTU2XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();
		iod.m_Data.m_uiValue = Latched;

		dr = ioctl(m_fdDevice, DTU2XX_RX_IOCTL_CLEAR_FLAGS, &iod);

		if ( dr != DTAPI_OK )
			return dr;
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsInpChannel::Detach -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
DTAPI_RESULT  TsInpChannel::Detach(
	int  DetachMode)
{
	// Must be attached
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	// Implement detach flags: reset FIFO for instant detach
	if (DetachMode & DTAPI_INSTANT_DETACH)
		Reset(DTAPI_FIFO_RESET);

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {
		// DO NOT NEED TO DO ANYTHING HERE
	}
	else if (Category() == DTAPI_CAT_USB) {
		// DO NOT NEED TO DO ANYTHING HERE
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	// Close driver-handle and mark channel as detached
	if ( 0!=close(m_fdDevice) )
		return DTAPI_E_INTERNAL;
	m_Attached = false;
	return DTAPI_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsInpChannel::Equalise -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Turn DVB/ASI cable equalisation on or off (DTA-120 only).
//
DTAPI_RESULT  TsInpChannel::Equalise(
	int  EqualiserSetting)		// Equaliser on/off
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	// Check parameters
	if (EqualiserSetting!=DTAPI_EQUALISER_ON && EqualiserSetting!=DTAPI_EQUALISER_OFF)
		return DTAPI_E_INVALID_MODE;

	DTAPI_RESULT dr;

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {
		//-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx			= GetIndexOnDevice();
		iod.m_Data.m_uiValue	= EqualiserSetting;

		DTAPI_RESULT dr = ioctl(m_fdDevice, DTA1XX_RX_IOCTL_EQUALISE, &iod);
	}
	else
		return DTAPI_E_NOT_SUPPORTED;	// NO equaliser setting on USB devices

	return dr;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsInpChannel::GetDescriptor -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Get hardware-function descriptor
//
DTAPI_RESULT  TsInpChannel::GetDescriptor(
	DtHwFuncDesc&  HwFuncDesc)
{
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	HwFuncDesc = m_HwFuncDesc;
	return DTAPI_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsInpChannel::GetFifoLoad -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Get the current load of the channel's Output FIFO.
//
DTAPI_RESULT  TsInpChannel::GetFifoLoad(
	int&  FifoLoad)
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();

		dr = ioctl(m_fdDevice, DTA1XX_RX_IOCTL_GET_FIFO_LOAD, &iod);

		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
		// Copy retrieved fifo load
		FifoLoad = iod.m_Data.m_uiValue;
	}
	else if (Category() == DTAPI_CAT_USB) {

		DTU2XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTU2XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();

		dr = ioctl(m_fdDevice, DTU2XX_RX_IOCTL_GET_FIFO_LOAD, &iod);

		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
		// Copy retrieved fifo load
		FifoLoad = iod.m_Data.m_uiValue;
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsInpChannel::GetFlags -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
DTAPI_RESULT  TsInpChannel::GetFlags(
	int&  Status,			// Status flags
	int&  Latched)			// Latched status flags
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		//-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();
		dr = ioctl(m_fdDevice, DTA1XX_RX_IOCTL_GET_FLAGS, &iod);

		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}

		// Copy retrieved flags
		Status		= iod.m_Data.m_Flags.m_nStatus;
		Latched		= iod.m_Data.m_Flags.m_nLatched;
	}
	else if (Category() == DTAPI_CAT_USB) {

		//-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-

		DTU2XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTU2XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();
		dr = ioctl(m_fdDevice, DTU2XX_RX_IOCTL_GET_FLAGS, &iod);

		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}

		// Copy retrieved flags
		Status		= iod.m_Data.m_Flags.m_nStatus;
		Latched		= iod.m_Data.m_Flags.m_nLatched;
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsInpChannel::GetRxControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Get the channel's receive-control state.
//
DTAPI_RESULT  TsInpChannel::GetRxControl(
	int&  RxControl)			// Receive-control state
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		//-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();
		dr = ioctl(m_fdDevice, DTA1XX_RX_IOCTL_GET_RX_CONTROL, &iod);

		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
		// Copy retrieved Rx-control value
		RxControl		= iod.m_Data.m_uiValue;
	}
	else if (Category() == DTAPI_CAT_USB)
	{
		//-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-

		DTU2XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTU2XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();
		dr = ioctl(m_fdDevice, DTU2XX_RX_IOCTL_GET_RX_CONTROL, &iod);

		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
		// Copy retrieved Rx-control value
		RxControl		= iod.m_Data.m_uiValue;
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsInpChannel::GetStatistics -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Get miscellaneous statistics information from the input channel.
//
DTAPI_RESULT  TsInpChannel::GetStatistics(
	int&  ViolCount)			// Code-violation counter
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		//-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();
		dr = ioctl(m_fdDevice, DTA1XX_RX_IOCTL_GET_STATISTICS, &iod);

		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
		// Copy retrieved statistics
		ViolCount		= iod.m_Data.m_uiValue;
	}
	else if (Category() == DTAPI_CAT_USB)
	{
		//-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-

		DTU2XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTU2XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();
		dr = ioctl(m_fdDevice, DTU2XX_RX_IOCTL_GET_VIOL_COUNT, &iod);

		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
		// Copy retrieved statistics
		ViolCount		= iod.m_Data.m_uiValue;
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsInpChannel::GetStatus -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Get status information from the input channel. If a PCI card does not support a
// certain feature, the corresponding status variable is set to DTAPI_NOT_SUPPORTED.
//
// The status flags are read directly from the hardware (not through device driver).
//
DTAPI_RESULT  TsInpChannel::GetStatus(
	int&  PacketSize,		// Size of incoming transport packets: DTAPI_PCKSIZE_XXX
	int&  NumInv,			// Number of invalid bytes per packet: DTAPI_NUMINV_XXX
	int&  ClkDet,			// Receive-clock signal detected: DTAPI_CLKDET_XXX
	int&  AsiLock,			// DVB/ASI PLL locked: DTAPI_ASILOCK_XXX
	int&  RateOk,			// Input rate above minimum
	int&  AsiInv)			// DVB/ASI-Input status
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		//-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();

		dr = ioctl(m_fdDevice, DTA1XX_RX_IOCTL_GET_STATUS, &iod);

		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
		// Copy retrieved status info
		PacketSize	= iod.m_Data.m_RxStatus.m_nPacketSize;
		NumInv		= iod.m_Data.m_RxStatus.m_nNumInv;
		ClkDet		= iod.m_Data.m_RxStatus.m_nClkDet;
		AsiLock		= iod.m_Data.m_RxStatus.m_nAsiLock;
		RateOk		= iod.m_Data.m_RxStatus.m_nRateOk;
		AsiInv		= iod.m_Data.m_RxStatus.m_nAsiInv;
	}
	else if (Category() == DTAPI_CAT_USB) {

		//-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-

		DTU2XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTU2XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();

		dr = ioctl(m_fdDevice, DTU2XX_RX_IOCTL_GET_STATUS, &iod);

		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
		// Copy retrieved status info
		PacketSize	= iod.m_Data.m_RxStatus.m_nPacketSize;
		NumInv		= iod.m_Data.m_RxStatus.m_nNumInv;
		ClkDet		= iod.m_Data.m_RxStatus.m_nClkDet;
		AsiLock		= iod.m_Data.m_RxStatus.m_nAsiLock;
		RateOk		= iod.m_Data.m_RxStatus.m_nRateOk;
		AsiInv		= iod.m_Data.m_RxStatus.m_nAsiInv;
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsInpChannel::GetTargetId -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
DTAPI_RESULT  TsInpChannel::GetTargetId(
	int&  Present,			// Target adapter present?
	int&  TargetId)			// Target-adapter identifier
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	// For DTA 122 only (=> No USB)
	if (TypeNumber() != 122)
		return DTAPI_E_NOT_SUPPORTED;

	//-.-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-.-

	DTA1XX_IOCTL_DATA  iod;
	memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
	iod.m_nChIdx = GetIndexOnDevice();

	DTAPI_RESULT dr = ioctl(m_fdDevice, DTA1XX_RX_IOCTL_GET_TARGETID, &iod);

	// IO control failed??
	if (dr!=DTAPI_OK) {
		return dr;
	}

	// Copy retrieved target info
	Present		= iod.m_Data.m_TargetInfo.m_nPresent;
	TargetId	= iod.m_Data.m_TargetInfo.m_nTargetId;

	return DTAPI_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsInpChannel::GetTsRateBps -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
DTAPI_RESULT  TsInpChannel::GetTsRateBps (
	int&  TsRate)			// Transport-stream rate in bps
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		//-.-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-.-

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();

		DTAPI_RESULT dr = ioctl(m_fdDevice, DTA1XX_RX_IOCTL_GET_TS_RATE_BPS, &iod);

		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}

		// Copy retrieved bit-rate
		TsRate = iod.m_Data.m_uiValue;
	}
    else if (Category() == DTAPI_CAT_USB) {

		//-.-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-.-

		DTU2XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTU2XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();

		DTAPI_RESULT dr = ioctl(m_fdDevice, DTU2XX_RX_IOCTL_GET_RX_CLOCK, &iod);

		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
		// Copy retrieved bit-rate
		TsRate = iod.m_Data.m_uiValue;
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsInpChannel::LedControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Take direct control of input-status LED, or let hardware drive the LED.
//
DTAPI_RESULT  TsInpChannel::LedControl(
	int  LedControl)		// DTAPI_LED_XXX
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		//-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx			= GetIndexOnDevice();
		iod.m_Data.m_uiValue	= LedControl;

		dr = ioctl(m_fdDevice, DTA1XX_IOCTL_LED_CONTROL, &iod);
		if ( dr != DTAPI_OK )
			return dr;
	}
	else if (Category() == DTAPI_CAT_USB)
	{
		//-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-

		DTU2XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTU2XX_IOCTL_DATA));
		iod.m_nChIdx			= GetIndexOnDevice();
		iod.m_Data.m_uiValue	= LedControl;

		dr = ioctl(m_fdDevice, DTU2XX_RX_IOCTL_LED_CONTROL, &iod);
		if ( dr != DTAPI_OK )
			return dr;
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}


//.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsInpChannel::PolarityControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Control the automatic polarity-detection circuitry of a DVB/ASI Transport-Stream
// input channel (DTA-120/140).
//
DTAPI_RESULT  TsInpChannel::PolarityControl(
	int  Polarity)		// Polarity control setting
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		//-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx			= GetIndexOnDevice();
		iod.m_Data.m_uiValue	= Polarity;

		dr = ioctl(m_fdDevice, DTA1XX_RX_IOCTL_POLARITY_CONTROL, &iod);

		if ( dr != DTAPI_OK )
			return dr;

	}
	else if (Category() == DTAPI_CAT_USB) {
		return DTAPI_E_NOT_SUPPORTED;
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsInpChannel::Reset -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Reset transport-stream input channel.
//
DTAPI_RESULT  TsInpChannel::Reset(
	int  ResetMode)		// How to reset the channel
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		//-.-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-.-

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx			= GetIndexOnDevice();
		iod.m_Data.m_uiValue	= ResetMode;

		dr = ioctl(m_fdDevice, DTA1XX_RX_IOCTL_RESET, &iod);
		if ( dr != DTAPI_OK )
			return dr;
	}
	else if (Category() == DTAPI_CAT_USB) {

		//-.-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-.-

		DTU2XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTU2XX_IOCTL_DATA));
		iod.m_nChIdx			= GetIndexOnDevice();
		iod.m_Data.m_uiValue	= ResetMode;

		dr = ioctl(m_fdDevice, DTU2XX_RX_IOCTL_RESET, &iod);
		if ( dr != DTAPI_OK )
			return dr;
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsInpChannel::SetLoopBackMode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Set the channel's loop-back mode.
//
DTAPI_RESULT  TsInpChannel::SetLoopBackMode(
	int  Mode)				// New loop-back mode
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		//-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx			= GetIndexOnDevice();
		iod.m_Data.m_uiValue	= Mode;

		dr = ioctl(m_fdDevice, DTA1XX_RX_IOCTL_SET_LOOPBACK_MODE, &iod);
		if ( dr != DTAPI_OK )
			return dr;
	}
	else if (Category() == DTAPI_CAT_USB)
	{
		// NOT SUPPORTED YET
		return DTAPI_E_INTERNAL;			// Unknown device category
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsInpChannel::SetPower -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Set the channel's power mode.
//
DTAPI_RESULT  TsInpChannel::SetPower(
	int  Power)				// New power mode
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		//-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx			= GetIndexOnDevice();
		iod.m_Data.m_uiValue	= Power;

		dr = ioctl(m_fdDevice, DTA1XX_RX_IOCTL_SET_POWER, &iod);
		if ( dr != DTAPI_OK )
			return dr;
	}
	else if (Category() == DTAPI_CAT_USB)
	{
		// NOT SUPPORTED YET
		return DTAPI_E_INTERNAL;			// Unknown device category
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsInpChannel::SetRxControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Set the channel's receive-control state.
//
DTAPI_RESULT  TsInpChannel::SetRxControl(
	int  RxControl)			// Receive control
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		//-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx			= GetIndexOnDevice();
		iod.m_Data.m_uiValue	= RxControl;

		dr = ioctl(m_fdDevice, DTA1XX_RX_IOCTL_SET_RX_CONTROL, &iod);
		if ( dr != DTAPI_OK )
			return dr;
	}
	else if (Category() == DTAPI_CAT_USB)
	{
		//-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-

		DTU2XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTU2XX_IOCTL_DATA));
		iod.m_nChIdx			= GetIndexOnDevice();
		iod.m_Data.m_uiValue	= RxControl;

		dr = ioctl(m_fdDevice, DTU2XX_RX_IOCTL_SET_RX_CONTROL, &iod);
		if ( dr != DTAPI_OK )
			return dr;
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsInpChannel::SetRxMode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Set the channel's receive mode.
//
DTAPI_RESULT  TsInpChannel::SetRxMode(
	int  RxMode)			// Receive mode
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		//-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx			= GetIndexOnDevice();
		iod.m_Data.m_uiValue	= RxMode;

		dr = ioctl(m_fdDevice, DTA1XX_RX_IOCTL_SET_RX_MODE, &iod);
		if ( dr != DTAPI_OK )
			return dr;
	}
	else if (Category() == DTAPI_CAT_USB)
	{
		//-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-

		DTU2XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTU2XX_IOCTL_DATA));
		iod.m_nChIdx			= GetIndexOnDevice();
		iod.m_Data.m_uiValue	= RxMode;

		dr = ioctl(m_fdDevice, DTU2XX_RX_IOCTL_SET_RX_MODE, &iod);
		if ( dr != DTAPI_OK )
			return dr;
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}
