//#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* TsInpRead.cpp *#*#*#*#*#*#*#*# (C) 2000-2004 DEKTEC
//
// Class TsInpChannel: implementation of input-channel read methods.

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- License -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// This program is free software; You can freely use/modify this source code in any way
// you desire, including for commercial applications.
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	MG	2004.11.23	Added support for Dtu2xx driver
//  MG  2004.10.21  Adapted for new DtDevice class
//	MG	19-11-2002	Created	based on DTAPI for windows

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#include "StdAfx.h"

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ TsInpChannel implementation +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsInpChannel::Read -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Read data bytes from the input channel using DMA transfers.
//
DTAPI_RESULT TsInpChannel::Read(
	char*  pBuffer,			// [in] Buffer to store data
	int  NumBytesToRead)	// [in] Number of bytes to be read from Device
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	// Validity check on parameters
	if (NumBytesToRead<0 || NumBytesToRead%4!=0)
		return DTAPI_E_INVALID_SIZE;
	if ((unsigned int)(pBuffer)%4 != 0)
		return DTAPI_E_INVALID_BUF;

	// Avoid strange effects when NumBytesToRead is zero
	if (NumBytesToRead == 0)
		return DTAPI_OK;

	DTAPI_RESULT dr;

	// Processing is dependent on device category
	if (m_HwFuncDesc.m_DvcDesc.m_Category == DTAPI_CAT_PCI) {

		//-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx								= GetIndexOnDevice();
		iod.m_Data.m_DataTransDesc.m_pBuffer		= pBuffer;
		iod.m_Data.m_DataTransDesc.m_nNumBytesToTr	= NumBytesToRead;

		dr = ioctl(m_fdDevice, DTA1XX_IOCTL_DMA_USERMEM_READ, &iod);
		if ( dr != DTAPI_OK )
			return dr;
	}
	else if (Category() == DTAPI_CAT_USB) {

		// Read using file IO
		int ret = read(m_fdDevice, pBuffer, NumBytesToRead);
		if ( ret < 0 )
			return DTAPI_E_DEV_DRIVER;
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsInpChannel::ReadDirect -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
DTAPI_RESULT TsInpChannel::ReadDirect(
	char*  pBuffer,			//  [in] Buffer to store data
	int  NumBytesToRead,	//  [in] Number of bytes to be read
	int&  NumBytesRead)		// [out] Number of bytes read
{
	return DTAPI_E_NOT_SUPPORTED;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsInpChannel::ReadUsingDma -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Old name of Read. Still here for backward compatibility.
//
DTAPI_RESULT TsInpChannel::ReadUsingDma(
	char*  pBuffer,			// [in] Buffer to store data
	int  NumBytesToRead)		// [in] Number of bytes to be written to PCI card
{
	return Read(pBuffer, NumBytesToRead);
}
