//#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* TsOutpChannel.cpp *#*#*#*#*#*#*# (C) 2000-2004 DEKTEC
//
// Class TsOutpChannel: implementation of general methods.

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- License -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// This program is free software; You can freely use/modify this source code in any way
// you desire, including for commercial applications.
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	TD	2007.02.08	Fixed ChannelIdx error in SetTxControl and Reset
//	TD	2007.01.29	Added support for DTA-160 ASI-only
//	TD	2007.01.23	Added support for the DTA-145 and DTA-2145
//      MG      2005.09.27      Fixed error in GetRfControl
//	JH	2005.07.13	Added support for DTA-105/110/545
//	MG	2004.11.23	Added support for Dtu2xx driver
//  MG  2004.10.21  - Adapted for new DtDevice class
//					- Fixed bug in TsOutpChannel::GetTxControl (wrong ioctl command was
//					  used)
//	MG	19-11-2002	Created	based on DTAPI for windows

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#include "StdAfx.h"

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Constants -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ TsOutpChannel implementation +=+=+=+=+=+=+=+=+=+=+=+=+=+=+

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::TsOutpChannel -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
TsOutpChannel::TsOutpChannel() :
	m_Attached(false),
	m_TsRate(0),
	m_TxPckMode(DTAPI_TXMODE_188),
	m_ClockGenMode(DTAPI_TXCLOCK_INTERNAL),
	m_CodeRate(DTAPI_MOD_1_2),
	m_ModType(DTAPI_MOD_QPSK)
{
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::~TsOutpChannel -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
TsOutpChannel::~TsOutpChannel()
{
    if ( IsAttached() )
      Detach(0);

}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::Attach -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Attach the Transport-Stream output-channel object to a hardware function hosted
// by a device.
//
DTAPI_RESULT  TsOutpChannel::Attach(
	DtDevice*  pDtDvc,		// Object representing DEKTEC Device
	int  OutpIndex)			// Output index on DEKTEC Device
{
	// Cannot attach twice
	if (m_Attached)
		return DTAPI_E_ATTACHED;

	// pDtDvc must be valid and attached to the hardware
	if (pDtDvc==NULL || !pDtDvc->m_Attached)
		return DTAPI_E_DEVICE;

	// Do we have a valid output channel?
	if (  OutpIndex<0 || OutpIndex>=pDtDvc->m_DvcDesc.m_NumDtOutpChan )
		return DTAPI_E_NO_TS_OUTPUT;


	// Duplicate handle from DtDevice object to TsInpChannel object.
	int fd = dup(pDtDvc->m_fdDriver);
	if ( fd == -1 )
		return DTAPI_E_DEV_DRIVER;
	m_fdDevice = fd;

	// Create hardware-function descriptor// Create hardware-function descriptor
	// Must be here, because other functions use m_HwFuncDesc.m_DvcDesc.m_Category
	m_HwFuncDesc.m_DvcDesc	  = pDtDvc->m_DvcDesc;	// Copy device-descriptor part
	m_HwFuncDesc.m_ChanType	  = DTAPI_TS_OUTPUT;	// Should be an output!
	m_HwFuncDesc.m_Flags	  = 0;			// Flags not used yet
	m_HwFuncDesc.m_IndexOnDvc = OutpIndex;	        // Select correct output

	// Assume success for other initialisation
	DTAPI_RESULT  Status = DTAPI_OK;

	// Device-category dependent processing
	if (pDtDvc->Category() == DTAPI_CAT_PCI) {

		// Request exclusive access

		// Attached!
		m_Attached = true;

		// Initialise hardware
		Status = ClearFifo();				// Clear Output FIFO
		SetFifoSizeMax();				// Maximum FIFO size
		SetLoopBackMode(DTAPI_NO_LOOPBACK);	// No Loop-Back Mode
		SetTxControl(DTAPI_TXCTRL_IDLE);	// Set transmit control to IDLE (redundant:
											// already done by ClearFifo call)
		ClearFlags(-1);						// Clear all latched transmit flags

		// If an error occurred => release exclusive-access lock and duplicated handle.
		if (Status != DTAPI_OK) {
			close(m_fdDevice);
			m_Attached = false;
		}
	}
	else if (pDtDvc->Category() == DTAPI_CAT_USB) {

		// Request exclusive access

		// Attached!
		m_Attached = true;

		// Initialise hardware
		Status = ClearFifo();				// Clear Output FIFO
		SetFifoSizeMax();				// Maximum FIFO size
		SetLoopBackMode(DTAPI_NO_LOOPBACK);	// No Loop-Back Mode
		SetTxControl(DTAPI_TXCTRL_IDLE);	// Set transmit control to IDLE (redundant:
											// already done by ClearFifo call)
		ClearFlags(-1);						// Clear all latched transmit flags

		// If an error occurred => release exclusive-access lock and duplicated handle.
		if (Status != DTAPI_OK) {
			close(m_fdDevice);
			m_Attached = false;
		}
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	// Cache initial settings

	int Dummy;
	GetTxMode(m_TxPckMode, Dummy);
	GetTsRateBps(m_ClockGenMode, m_TsRate);
	GetModControl(Dummy, m_CodeRate, Dummy, Dummy);

	return Status;
}
// FOR NOW: simple call attach (regard less of port)
DTAPI_RESULT TsOutpChannel::AttachToPort(DtDevice* pDtDvc, int Port)
{
	int TxChan = Port - pDtDvc->m_DvcDesc.m_NumDtInpChan - 1;
  return Attach(pDtDvc, TxChan);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::ClearFifo -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Clear contents of the Output FIFO and set transmit-control state to DTAPI_TXCTRL_IDLE.
// Furthermore, clear status flags DTAPI_TX_FIFO_UFL and DTAPI_TX_SYNC_ERR.
//
DTAPI_RESULT  TsOutpChannel::ClearFifo(void)
{
	return Reset(DTAPI_FIFO_RESET);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::ClearFlags -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
DTAPI_RESULT  TsOutpChannel::ClearFlags(
	int  Latched)				// Latched status flags to be cleared
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		// Clear flags through device driver
		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();
		iod.m_Data.m_uiValue = Latched;

		dr = ioctl(m_fdDevice, DTA1XX_TX_IOCTL_CLEAR_FLAGS, &iod);
		if ( dr != DTAPI_OK )
			return dr;
	}
	else if (Category() == DTAPI_CAT_USB) {

		// Clear flags through device driver
		DTU2XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTU2XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();
		iod.m_Data.m_uiValue = Latched;

		dr = ioctl(m_fdDevice, DTU2XX_TX_IOCTL_CLEAR_FLAGS, &iod);
		if ( dr != DTAPI_OK )
			return dr;
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::Detach -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
DTAPI_RESULT  TsOutpChannel::Detach(
	int  DetachMode)
{
	// Must be attached
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	// Implement detach flags: reset FIFO for instant detach
	if (DetachMode & DTAPI_INSTANT_DETACH) {
		ClearFifo();
	} else if (DetachMode & DTAPI_WAIT_UNTIL_SENT) {
		int FifoLoad(0);
		GetFifoLoad(FifoLoad);
		while ( FifoLoad!=0) {
			sleep(1);
			GetFifoLoad(FifoLoad);
		}
	}

        // Set transmmision-control to IDLE
        // all resources before tx-control is IDLE
        SetTxControl(DTAPI_TXCTRL_IDLE);

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI)
        {


	// Then: zeroing and/or tri-stating output
	if (DetachMode & DTAPI_ZERO_OUTPUT) {
		SetTsRateBps(DTAPI_TXCLOCK_INTERNAL, 0);
	}

	}
	else if (Category() == DTAPI_CAT_USB) {
		// NO SPECIAL ACTIONS NEEDED FOR NOW
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	// Close driver-handle and mark channel as detached
	if ( 0!=close(m_fdDevice) )
		return DTAPI_E_INTERNAL;
	m_Attached = false;
	return DTAPI_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsInpChannel::GetDescriptor -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Get hardware-function descriptor
//
DTAPI_RESULT  TsOutpChannel::GetDescriptor(
	DtHwFuncDesc&  HwFuncDesc)
{
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	HwFuncDesc = m_HwFuncDesc;
	return DTAPI_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::GetExtClkFreq -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
DTAPI_RESULT  TsOutpChannel::GetExtClkFreq(
	int&  ExtClkFreq)			// Transport-stream rate of external clock in bps
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	// Not supported by USB devices
	if (Category() == DTAPI_CAT_USB)
		return DTAPI_E_NOT_SUPPORTED;

	//-.-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-.-

	DTA1XX_IOCTL_DATA  iod;
	memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
	iod.m_nChIdx = GetIndexOnDevice();

	DTAPI_RESULT dr = ioctl(m_fdDevice, DTA1XX_TX_IOCTL_GET_EXT_CLK_FREQ, &iod);

	// IO control failed??
	if (dr!=DTAPI_OK) {
		return dr;
	}

	// Copy external clock rate from rate-info structure
	ExtClkFreq = iod.m_Data.m_RateInfo.m_nTsRate;

	return DTAPI_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::GetFifoLoad -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Get the current load of the channel's Output FIFO.
//
DTAPI_RESULT  TsOutpChannel::GetFifoLoad(
	int&  FifoLoad)
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();

		dr = ioctl(m_fdDevice, DTA1XX_TX_IOCTL_GET_FIFO_LOAD, &iod);
		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
		// Copy retrieved fifo load
		FifoLoad = iod.m_Data.m_uiValue;
	}
	else if (Category() == DTAPI_CAT_USB) {

		DTU2XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTU2XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();

		dr = ioctl(m_fdDevice, DTU2XX_TX_IOCTL_GET_FIFO_LOAD, &iod);
		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
		// Copy retrieved fifo load
		FifoLoad = iod.m_Data.m_uiValue;
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::GetFlags -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
DTAPI_RESULT  TsOutpChannel::GetFlags(
	int&  Status,				// Status flags
	int&  Latched)				// Latched status flags
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();

		dr = ioctl(m_fdDevice, DTA1XX_TX_IOCTL_GET_FLAGS, &iod);
		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
		// Copy output parameter from Flags
		Status	= iod.m_Data.m_Flags.m_nStatus;
		Latched	= iod.m_Data.m_Flags.m_nLatched;
	}
	else if (Category() == DTAPI_CAT_USB) {

		DTU2XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTU2XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();

		dr = ioctl(m_fdDevice, DTU2XX_TX_IOCTL_GET_FLAGS, &iod);
		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
		// Copy output parameter from Flags
		Status	= iod.m_Data.m_Flags.m_nStatus;
		Latched	= iod.m_Data.m_Flags.m_nLatched;
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::GetMaxFifoSize -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
DTAPI_RESULT  TsOutpChannel::GetMaxFifoSize(
	int&  MaxFifoSize)
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();

		dr = ioctl(m_fdDevice, DTA1XX_TX_IOCTL_GET_MAX_FIFO_SIZE, &iod);
		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
		// Copy retrieved maximum
		MaxFifoSize = iod.m_Data.m_uiValue;
	}
	else if (Category() == DTAPI_CAT_USB) {

		DTU2XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTU2XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();

		dr = ioctl(m_fdDevice, DTU2XX_TX_IOCTL_GET_MAX_FIFO_SIZE, &iod);
		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
		// Copy retrieved maximum
		MaxFifoSize = iod.m_Data.m_uiValue;
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::GetModControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Get modulation control settings
//
DTAPI_RESULT  TsOutpChannel::GetModControl(
	int&  ModType,			// Modulation type
	int&  CodeRate,			// Convolutional rate
	int&  ParXtra1,			// Extra parameter #1
	int&  ParXtra2)			// Extra parameter #2
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	// Supported on modulator cards only
	if (TypeNumber()!=107 && TypeNumber()!=110)
		return DTAPI_E_NOT_SUPPORTED;

	// Let the driver do the work
	DTA1XX_IOCTL_DATA  iod;
	memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
	iod.m_nChIdx = GetIndexOnDevice();

	DTAPI_RESULT dr = ioctl(m_fdDevice, DTA1XX_TX_IOCTL_GET_MOD_CONTROL, &iod);
	// IO control failed??
	if (dr!=DTAPI_OK) {
		return dr;
	}

	// Store settings
	ModType		= iod.m_Data.m_ModCtrl.m_nModType;
	CodeRate	= iod.m_Data.m_ModCtrl.m_nCodeRate;
	ParXtra1	= iod.m_Data.m_ModCtrl.m_nParXtra1;
	ParXtra2	= iod.m_Data.m_ModCtrl.m_nParXtra2;

	return DTAPI_OK;;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::GetRfControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Get up-converter parameters for devices with on-board RF up-converter.
//
DTAPI_RESULT  TsOutpChannel::GetRfControl(
	__int64&  RfRate,			// RF frequency in Hz
	int&  LockStatus)			// Lock status of RF PLL
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	// Supported on modulator cards only
	if (TypeNumber()!=107 && TypeNumber()!=110)
		return DTAPI_E_NOT_SUPPORTED;

	// Let the driver do the work
	DTA1XX_IOCTL_DATA  iod;
	memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
	iod.m_nChIdx = GetIndexOnDevice();

	DTAPI_RESULT dr = ioctl(m_fdDevice, DTA1XX_TX_IOCTL_GET_RF_CONTROL, &iod);
	// IO control failed??
	if (dr!=DTAPI_OK) {
		return dr;
	}

	unsigned int A, B;

	// Store settings
	A		= iod.m_Data.m_RfCtrl.m_nPllA;
	B		= iod.m_Data.m_RfCtrl.m_nPllB;
	LockStatus	= iod.m_Data.m_RfCtrl.m_nLockStatus;

	// Compute frequency from A and B
        unsigned int  Q = (TypeNumber()==107) ? 32 : 8;
	double  RfFreq = 0.2 * (A + Q*B);

	// Round to exact multiple of 200kHz and convert to __int64
	RfRate = __int64(RfFreq * 5 + 0.5) * __int64(200000);
	return DTAPI_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::GetTargetId -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
DTAPI_RESULT  TsOutpChannel::GetTargetId(
	int&  Present,				// Target adapter present?
	int&  TargetId)				// Target-adapter identifier
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	// For DTA 102 only
	if (TypeNumber() != 102)
		return DTAPI_E_NOT_SUPPORTED;

	//-.-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-.-

	DTA1XX_IOCTL_DATA  iod;
	memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
	iod.m_nChIdx = GetIndexOnDevice();

	DTAPI_RESULT dr = ioctl(m_fdDevice, DTA1XX_TX_IOCTL_GET_TARGET_ID, &iod);

	// IO control failed??
	if (dr!=DTAPI_OK) {
		return dr;
	}

	// Copy output parameter from TargetInfo
	Present = iod.m_Data.m_TargetInfo.m_nPresent;
	if (Present==DTAPI_TARGET_PRESENT) {
		TargetId = iod.m_Data.m_TargetInfo.m_nTargetId;
	}
	return DTAPI_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::GetTsRateBps -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
DTAPI_RESULT  TsOutpChannel::GetTsRateBps(
	int&  ClockGenMode,			// Internal or external clock
	int&  TsRate)				// Transport-stream rate in bps
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;

	__int64 PhaseIncr(0);	// Phase increment; 64-bit for extended precision computations
	int SymbRateM(0);		// Symbol rate multiplier

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();

		dr = ioctl(m_fdDevice, DTA1XX_TX_IOCTL_GET_TX_CLOCK, &iod);
		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
		// Copy output parameters from rate-info structure
		PhaseIncr		= (__int64)iod.m_Data.m_TxClockInfo.m_nPhaseIncr;
		SymbRateM		= iod.m_Data.m_TxClockInfo.m_nSymbRateMult;
		ClockGenMode	= iod.m_Data.m_TxClockInfo.m_nClockGenMode;
	}
	else if (Category() == DTAPI_CAT_USB) {

		DTU2XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTU2XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();

		dr = ioctl(m_fdDevice, DTU2XX_TX_IOCTL_GET_TX_CLOCK, &iod);
		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
		// Copy output parameters from rate-info structure
		PhaseIncr		= (__int64)iod.m_Data.m_RateInfo.m_nTxClock;
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Compute TsRate -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	// Special processing for modulator cards
	if ( TypeNumber() == 107) {

		// *** Modulator card

		int  CodeRateP, CodeRateQ;
		__int64  SymRate;		// Symbol rate

		// Convert cached code rate to quotient P/Q
		switch (m_CodeRate) {
		case 0: CodeRateP = 1; CodeRateQ = 2; break;
		case 1: CodeRateP = 2; CodeRateQ = 3; break;
		case 2: CodeRateP = 3; CodeRateQ = 4; break;
		case 3: CodeRateP = 4; CodeRateQ = 5; break;
		case 4: CodeRateP = 5; CodeRateQ = 6; break;
		case 5: CodeRateP = 6; CodeRateQ = 7; break;
		case 6: CodeRateP = 7; CodeRateQ = 8; break;
		default: return DTAPI_E_INTERNAL;
		}

		// Compute symbol rate, multiplied by 16 (for rounding)
		// Be careful with 1<<(25+M): cast 1 to 64-bit first!
		SymRate = (PhaseIncr * (__int64)25000000) / ((__int64)1 << (25+SymbRateM));

		// Compute TS rate
		TsRate = (int) ( (SymRate * __int64(2) /* QPSK */ * __int64(188) * __int64(CodeRateP)) /
							(__int64)(__int64(204) * __int64(CodeRateQ)) );
		TsRate = (TsRate + 8) >> 4;			// Rounding

	} else {
		// *** Not a modulator card

		// Determine current packet size
		int PckSize = TxMode2PacketSize(m_TxPckMode);

		// Compute transport-stream rate, depends on phase increment and packet size
		if (PckSize == 188)
			TsRate = (int) ((PhaseIncr * 421875*2 / 8388608 + 1) >> 1);
		else if (PckSize == 192)
			TsRate = (int) ((PhaseIncr * 19828125*2 / 402653184 + 1) >> 1);
		else	// nPckSize == 204
			TsRate = (int) ((PhaseIncr * 6609375*2 / 142606336 + 1) >> 1);
	}
	return DTAPI_OK;
}
// OVERLOAD: without internal or external clock parameter
DTAPI_RESULT  TsOutpChannel::GetTsRateBps(
        int&  TsRate)				// Transport-stream rate in bps
{
        int Dummy;
        return GetTsRateBps(Dummy, TsRate);
}


//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::GetTxControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Get the channel's transmit-control state.
//
DTAPI_RESULT  TsOutpChannel::GetTxControl(
	int&  TxControl)			// Transmit-control state
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();

		dr = ioctl(m_fdDevice, DTA1XX_TX_IOCTL_GET_TX_CONTROL, &iod);
		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
		// Copy output parameter from iod
		TxControl = iod.m_Data.m_uiValue;
	}
	else if (Category() == DTAPI_CAT_USB) {

		DTU2XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTU2XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();

		dr = ioctl(m_fdDevice, DTU2XX_TX_IOCTL_GET_TX_CONTROL, &iod);
		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
		// Copy output parameter from iod
		TxControl = iod.m_Data.m_uiValue;
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::GetTxMode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Get the channel's transmit mode.
//
DTAPI_RESULT  TsOutpChannel::GetTxMode(
	int&  TxPacketMode,			// Packet transmit mode
	int&  TxStuffMode)			// Null-packet stuffing on/off
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();

		dr = ioctl(m_fdDevice, DTA1XX_TX_IOCTL_GET_TX_MODE, &iod);
		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
		// Copy output parameters from TxMode structure
		TxPacketMode	= iod.m_Data.m_TxMode.m_nTxPacketMode;
		TxStuffMode		= iod.m_Data.m_TxMode.m_nTxStuffMode;
	}
	else if (Category() == DTAPI_CAT_USB) {

		DTU2XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTU2XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();

		dr = ioctl(m_fdDevice, DTU2XX_TX_IOCTL_GET_TX_MODE, &iod);
		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
		// Copy output parameters from TxMode structure
		TxPacketMode	= iod.m_Data.m_TxMode.m_nTxPacketMode;
		TxStuffMode		= iod.m_Data.m_TxMode.m_nTxStuffMode;
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::Reset -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Reset transport-stream output channel.
//
DTAPI_RESULT  TsOutpChannel::Reset(
	int  ResetMode)				// How to reset the channel
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		int TxChan = GetIndexOnDevice();
		__DTAPI::ResetOutputChannel(m_fdDevice, TxChan, ResetMode);
	}
	else if (Category() == DTAPI_CAT_USB) {

		DTU2XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTU2XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();
		iod.m_Data.m_uiValue = ResetMode;

		dr = ioctl(m_fdDevice, DTU2XX_TX_IOCTL_RESET, &iod);
		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	// Reset transport-stream rate
	if (ResetMode==DTAPI_FULL_RESET) {
		SetTsRateBps(DTAPI_TXCLOCK_INTERNAL, 0);
	}

	return DTAPI_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::SetFifoSize -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
DTAPI_RESULT  TsOutpChannel::SetFifoSize(
	int  FifoSize)				// Size of FIFO in bytes
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();
		iod.m_Data.m_uiValue = FifoSize;

		dr = ioctl(m_fdDevice, DTA1XX_TX_IOCTL_SET_FIFO_SIZE, &iod);
		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
	}
	else if (Category() == DTAPI_CAT_USB) {

		DTU2XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTU2XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();
		iod.m_Data.m_uiValue = FifoSize;

		dr = ioctl(m_fdDevice, DTU2XX_TX_IOCTL_SET_FIFO_SIZE, &iod);
		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::SetFifoSizeMax -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Set FIFO size to the maximum valid value.
//
DTAPI_RESULT  TsOutpChannel::SetFifoSizeMax(void)
{
	return SetFifoSize(8*1024*1024 - 16);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::SetLoopBackMode -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Set the channel's loop-back mode.
//
DTAPI_RESULT  TsOutpChannel::SetLoopBackMode(
	int  Mode)					// New loop-back mode
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();
		iod.m_Data.m_uiValue = Mode;

		dr = ioctl(m_fdDevice, DTA1XX_TX_IOCTL_SET_LOOPBACK_MODE, &iod);
		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
	}
	else if (Category() == DTAPI_CAT_USB) {
		// NOT SUPPORTED YET
		return DTAPI_E_INTERNAL;			// Unknown device category
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::SetModControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Set modulation-control parameters for output channels with a built-in modulator.
//
DTAPI_RESULT  TsOutpChannel::SetModControl(
	int  ModType,			// Modulation type
	int  CodeRate,			// Convolutional rate
	int  ParXtra1,			// Extra parameter #1
	int  ParXtra2)			// Extra parameter #2
{
  DTAPI_RESULT dr;
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	// Supported on modulator cards only
	if (TypeNumber()!=107 && TypeNumber()!=110)
		return DTAPI_E_NOT_SUPPORTED;

	// Let the driver perform the work
	DTA1XX_IOCTL_DATA  iod;
	memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
	iod.m_nChIdx = GetIndexOnDevice();
	iod.m_Data.m_ModCtrl.m_nModType		= ModType;
	iod.m_Data.m_ModCtrl.m_nCodeRate	= CodeRate;
	iod.m_Data.m_ModCtrl.m_nParXtra1	= ParXtra1;
	iod.m_Data.m_ModCtrl.m_nParXtra2	= ParXtra2;

	dr = ioctl(m_fdDevice, DTA1XX_TX_IOCTL_SET_MOD_CONTROL, &iod);
	// IO control failed??
	if (dr!=DTAPI_OK) {
		return dr;
	}

	// Remeber old code rate and modulation type
	int OldCodeRate = m_CodeRate;
        int OldModType = m_ModType;
	// Cache code rate and modulation type
	m_CodeRate = CodeRate;
        m_ModType = ModType;
        // Restore the bit-rate (after changing the code rate or modulation type)
	if ( CodeRate != OldCodeRate || m_ModType!=OldModType )
		SetTsRateBps(m_ClockGenMode, m_TsRate);

	return DTAPI_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::SetPower -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Set the channel's power mode.
//
DTAPI_RESULT  TsOutpChannel::SetPower(
	int  Power)					// New power mode
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	// Supported on DTA-102 only
	if (TypeNumber() != 102)
		return DTAPI_E_NOT_SUPPORTED;

	//-.-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-.-

	DTA1XX_IOCTL_DATA  iod;
	memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
	iod.m_nChIdx			= GetIndexOnDevice();
	iod.m_Data.m_uiValue	= Power;

	DTAPI_RESULT dr = ioctl(m_fdDevice, DTA1XX_TX_IOCTL_SET_POWER, &iod);
	return dr;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::SetRfControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Set up-converter parameters for devices with on-board RF up-converter.
//
DTAPI_RESULT TsOutpChannel::SetRfControl(
	__int64  RfRate)			// RF frequency in Hz
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	// Supported on modulator cards only
	if (TypeNumber()!=107 && TypeNumber()!=110)
		return DTAPI_E_NOT_SUPPORTED;

	// Start from RF frequency in MHz
	double	RfFreq = double(RfRate) / 1e6;

	// Check RF frequency against valid range
	if ( TypeNumber()==107 && (RfFreq<950.0 || RfFreq>2150.0) )
		return DTAPI_E_INVALID_RATE;
	if ( TypeNumber()==110 && (RfFreq<400.0 || RfFreq>862.0) )
        	return DTAPI_E_INVALID_RATE;

	// Compute A and B
        unsigned int  Q = (TypeNumber()==107) ? 32 : 8;
	unsigned int  APlus32B = (unsigned int)((RfFreq * 5.0) + 0.5);
	unsigned int  A = APlus32B % Q;
	unsigned int  B = APlus32B / Q;

	// Send settings to driver
	DTA1XX_IOCTL_DATA  iod;
	memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
	iod.m_nChIdx = GetIndexOnDevice();
	iod.m_Data.m_RfCtrl.m_nPllA = A;
	iod.m_Data.m_RfCtrl.m_nPllB = B;

	DTAPI_RESULT dr = ioctl(m_fdDevice, DTA1XX_TX_IOCTL_SET_RF_CONTROL, &iod);
	// IO control failed??
	if (dr!=DTAPI_OK) {
		return dr;
	}
	return DTAPI_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::SetTsRateBps -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
DTAPI_RESULT  TsOutpChannel::SetTsRateBps(
	int  ClockGenMode,			// Internal or external clock
	int  TsRate)				// Transport-stream rate in bps
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;
	int PhaseIncr(0);		// TxClock pahse increment
	int Multiplier(0);		// Symbol-rate multiplier

	// Compute phase increment
	if ( TypeNumber() == 107 ) {

                //-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA-107 -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

                int  CodeRate, CodeRateP, CodeRateQ;
		__int64  TsRate64=TsRate; // TS rate; 64-bit for extended precision computations
		__int64  SymRate;		// Symbol rate
		__int64  RateBase;		// Intermediate variable in bit-rate computation


		// Get code rate and convert to quotient P/Q
		switch (m_CodeRate) {
		case 0: CodeRateP = 1; CodeRateQ = 2; break;
		case 1: CodeRateP = 2; CodeRateQ = 3; break;
		case 2: CodeRateP = 3; CodeRateQ = 4; break;
		case 3: CodeRateP = 4; CodeRateQ = 5; break;
		case 4: CodeRateP = 5; CodeRateQ = 6; break;
		case 5: CodeRateP = 6; CodeRateQ = 7; break;
		case 6: CodeRateP = 7; CodeRateQ = 8; break;
		default: return DTAPI_E_INTERNAL;
		}

		// Compute symbol rate
		// NOTE1: Packet size always 204 bytes on modulator cards
		// NOTE2: *16 for rounding
		SymRate = __int64((TsRate64 * __int64(204) * __int64(CodeRateQ)) /
							__int64(__int64(2) /* QPSK */ * __int64(188) * __int64(CodeRateP)) );

		// Compute multiplier M
		if (SymRate <= (__int64)175783)		Multiplier = 8;
		else if (SymRate <= (__int64)351566)	Multiplier = 7;
		else if (SymRate <= (__int64)703133)	Multiplier = 6;
		else if (SymRate <= (__int64)1406250)	Multiplier = 5;
		else if (SymRate <= (__int64)2812500)	Multiplier = 4;
		else if (SymRate <= (__int64)5625000)	Multiplier = 3;
		else if (SymRate <= (__int64)11250000)	Multiplier = 2;
		else if (SymRate <= (__int64)22500000)	Multiplier = 1;
		else					Multiplier = 0;

		// Compute symbol-rate base, with rounding
		RateBase = (TsRate64 << __int64(Multiplier)) * __int64(CodeRateQ) / __int64(CodeRateP);
		PhaseIncr = (int)(RateBase * __int64(11651241) / __int64(1000000));

	}
        else if ( TypeNumber() == 110 ) {

                //-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA-110 -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

                __int64  TsRate64=TsRate;       // TS rate; 64-bit for extended precision computations
                __int64  SymRate;		// Symbol rate
                int BitsPerSymbol(0);

               //-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA-110 -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

               // Get modulation type and convert to #bits per symbol
               switch (m_ModType) {
               case DTAPI_MOD_QAM4:   BitsPerSymbol = 2; break;
               case DTAPI_MOD_QAM16:  BitsPerSymbol = 4; break;
               case DTAPI_MOD_QAM32:  BitsPerSymbol = 5; break;
               case DTAPI_MOD_QAM64:  BitsPerSymbol = 6; break;
               case DTAPI_MOD_QAM128: BitsPerSymbol = 7; break;
               case DTAPI_MOD_QAM256: BitsPerSymbol = 8; break;
               default: return DTAPI_E_INVALID_MODE;
               }

               // Compute symbol rate
               // NOTE1: Packet size always 204 bytes on modulator cards
               SymRate = (TsRate64 * 204) / __int64(BitsPerSymbol * 188);

               // Compute dPhi
               PhaseIncr = (int) (((__int64)SymRate << 31) / 25000000);
        }
	else {
		// **** NOT a Modulator boards

		int PckSize = TxMode2PacketSize(m_TxPckMode);

                __int64  TsRate64=TsRate; // TS rate; 64-bit for extended precision computations

		// Compute phase-increment, depends on transport-stream rate and packet size
		if (PckSize == 188)
			// p / q = 2^32 / 216.000.000
			// p and q both divided by 512
			PhaseIncr = (int) ((TsRate64 * 8388608*2 / 421875 + 1) >> 1);
		else if (PckSize == 192)
			PhaseIncr = (int) ((TsRate64 * 402653184*2 / 19828125 + 1) >> 1);
		else	// PckSize == 204
			PhaseIncr = (int) ((TsRate64 * 142606336*2 / 6609375 + 1) >> 1);
	}

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();
		iod.m_Data.m_TxClockInfo.m_nClockGenMode	= ClockGenMode;
		iod.m_Data.m_TxClockInfo.m_nPhaseIncr 		= PhaseIncr;
		// Symbol rate multiplier is only used for modulator boards
		iod.m_Data.m_TxClockInfo.m_nSymbRateMult 	= Multiplier;

		dr = ioctl(m_fdDevice, DTA1XX_TX_IOCTL_SET_TX_CLOCK, &iod);
		// IO control failed??
		if (dr!=DTAPI_OK)
			return dr;
	}
	else if (Category() == DTAPI_CAT_USB) {

		DTU2XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTU2XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();
		iod.m_Data.m_RateInfo.m_nTxClock = PhaseIncr;

		dr = ioctl(m_fdDevice, DTU2XX_TX_IOCTL_SET_TX_CLOCK, &iod);
		// IO control failed??
		if (dr!=DTAPI_OK)
			return dr;
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	// Cache settings
	m_ClockGenMode = ClockGenMode;
	m_TsRate = TsRate;

	return DTAPI_OK;
}
// OVERLOAD: without internal or external clock parameter (default to internal clock)
DTAPI_RESULT  TsOutpChannel::SetTsRateBps(
        int  TsOrSymRate)				// Transport-stream rate in bps
{
        return SetTsRateBps(DTAPI_TXCLOCK_INTERNAL, TsOrSymRate);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::SetTxControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Set the channel's transmit-control state.
//
DTAPI_RESULT  TsOutpChannel::SetTxControl(
	int  TxControl)				// Transmit-control state
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();
		iod.m_Data.m_uiValue = TxControl;

		dr = ioctl(m_fdDevice, DTA1XX_TX_IOCTL_SET_TX_CONTROL, &iod);
		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
	}
	else if (Category() == DTAPI_CAT_USB) {

		DTU2XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTU2XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();
		iod.m_Data.m_uiValue = TxControl;

		dr = ioctl(m_fdDevice, DTU2XX_TX_IOCTL_SET_TX_CONTROL, &iod);
		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::SetTxMode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Set the channel's transmit mode.
//
DTAPI_RESULT  TsOutpChannel::SetTxMode(
	int  TxPacketMode,			// Packet transmit mode
	int  TxStuffMode)			// Null-packet stuffing on/off
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();
		iod.m_Data.m_TxMode.m_nTxPacketMode	= TxPacketMode;
		iod.m_Data.m_TxMode.m_nTxStuffMode 	= TxStuffMode;

		DTAPI_RESULT dr = ioctl(m_fdDevice, DTA1XX_TX_IOCTL_SET_TX_MODE, &iod);
		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
	}
	else if (Category() == DTAPI_CAT_USB) {

		DTU2XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTU2XX_IOCTL_DATA));
		iod.m_nChIdx = GetIndexOnDevice();
		iod.m_Data.m_TxMode.m_nTxPacketMode	= TxPacketMode;
		iod.m_Data.m_TxMode.m_nTxStuffMode 	= TxStuffMode;

		DTAPI_RESULT dr = ioctl(m_fdDevice, DTU2XX_TX_IOCTL_SET_TX_MODE, &iod);
		// IO control failed??
		if (dr!=DTAPI_OK) {
			return dr;
		}
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	// Determine new and old packet size
	int OldPckSize, NewPckSize;
	OldPckSize = TxMode2PacketSize(m_TxPckMode);
	NewPckSize = TxMode2PacketSize(TxPacketMode);

	// Cache packet TxMode
	m_TxPckMode = TxPacketMode;

	// TxMode changed => restore bit-rate
	if ( OldPckSize != NewPckSize )
		SetTsRateBps(m_ClockGenMode, m_TsRate);

	return DTAPI_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::SetTxPolarity -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Set polarity of DVB/ASI output signal.
//
DTAPI_RESULT  TsOutpChannel::SetTxPolarity(
	int  TxPolarity)			// DTAPI_POLARITY_NORMAL / DTAPI_POLARITY_INVERTED
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	// Check parameters
	if (TxPolarity!=DTAPI_TXPOL_NORMAL && TxPolarity!=DTAPI_TXPOL_INVERTED)
		return DTAPI_E_INVALID_MODE;

	// Not supported by DTU-205
	if (TypeNumber() == 205)
		return DTAPI_E_NOT_SUPPORTED;

	//-.-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-.-

	DTA1XX_IOCTL_DATA  iod;
	memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
	iod.m_nChIdx			= GetIndexOnDevice();
 	iod.m_Data.m_uiValue	=  TxPolarity;

	DTAPI_RESULT dr = ioctl(m_fdDevice, DTA1XX_TX_IOCTL_SET_TX_POLARITY, &iod);
	return dr;
}

/*
//-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::ReadLoopBackData -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
DTAPI_RESULT TsOutpChannel::ReadLoopBackData(
	char* pBuffer,			// Pointer to data read back from Output FIFO
	int  nBytesToRead)		// Number of bytes to be read
{
	//-.-.-.-.-.-.-.-.-.-.- Call the appropiate device IoCtl handler -.-.-.-.-.-.-.-.-.-.-

	DTA1XX_IOCTL_DATA  iod;
	memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
	iod.m_nChIdx								= 0;
	iod.m_Data.m_DataTransDesc.m_pBuffer		= pBuffer;
	iod.m_Data.m_DataTransDesc.m_nNumBytesToTr	= nBytesToRead;

	DTAPI_RESULT dr = ioctl(m_fdDevice, DTA1XX_TX_IOCTL_RD_LOOPBACK_DATA, &iod);
	return dr;
}
*/
