//*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* TsOutpWrite.cpp *#*#*#*#*#*#*#* (C) 2000-2004 DEKTEC
//
// Class TsOutpChannel: implementation of write methods.

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- License -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// This program is free software; You can freely use/modify this source code in any way
// you desire, including for commercial applications.
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	MG	2004.11.23	Added support for Dtu2xx driver
//  MG  2004.10.21  Adapted for new DtDevice class
//	MG	19-11-2002	Created	based on DTAPI for windows

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#include "StdAfx.h"

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ TsOutpChannel implementation +=+=+=+=+=+=+=+=+=+=+=+=+=+=+

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::Write -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Write data bytes to the output channel using DMA transfers.
//
DTAPI_RESULT TsOutpChannel::Write(
	char*  pBuffer,			// [in] Pointer to data to be written to PCI card
	int  NumBytesToWrite)	// [in] Number of bytes to be written to PCI card
{
	// Must be attached.
	if (!m_Attached)
		return DTAPI_E_NOT_ATTACHED;

	DTAPI_RESULT dr;

	// Device-category dependent processing
	if (Category() == DTAPI_CAT_PCI) {

		DTA1XX_IOCTL_DATA  iod;
		memset(&iod, 0, sizeof(DTA1XX_IOCTL_DATA));
		iod.m_nChIdx								= GetIndexOnDevice();
		iod.m_Data.m_DataTransDesc.m_pBuffer		= pBuffer;
		iod.m_Data.m_DataTransDesc.m_nNumBytesToTr	= NumBytesToWrite;

		dr = ioctl(m_fdDevice, DTA1XX_IOCTL_DMA_USERMEM_WRITE, &iod);
		if ( dr != DTAPI_OK )
			return dr;
	}
	else if (Category() == DTAPI_CAT_USB) {
		
		// Write using file IO
		int ret = write(m_fdDevice, pBuffer, NumBytesToWrite);
		if ( ret < 0 )
			return DTAPI_E_DEV_DRIVER;
	}
	else
		return DTAPI_E_INTERNAL;			// Unknown device category

	return DTAPI_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::WriteDirect -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
DTAPI_RESULT TsOutpChannel::WriteDirect(
	char*  pBuffer,			// [in] Pointer to data to be written to PCI card
	int  NumBytesToWrite)	// [in] Number of bytes to be written to PCI card
{
	return DTAPI_E_NOT_SUPPORTED;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TsOutpChannel::WriteUsingDma -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Old name of Write. Still here for backward compatibility.
//
DTAPI_RESULT TsOutpChannel::WriteUsingDma(
	char*  pBuffer,			// [in] Pointer to data to be written to PCI card
	int  NumBytesToWrite)	// [in] Number of bytes to be written to PCI card
{
	return Write(pBuffer, NumBytesToWrite);
}
