//#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* DtPlay.cpp *#*#*#*#*#*#*#*#* (C) 2000-2007 DekTec
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	2006.10.08	MG	Created

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#include <stdio.h>
#include "DTAPI.h"
#include "DtPlay.h"

#ifdef WIN32
	#include <conio.h>
#else
	#include <stdlib.h>
	#include <string.h>
	#include <unistd.h>
#endif


//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DtPlay Version -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define DTPLAY_VERSION_MAJOR		3
#define DTPLAY_VERSION_MINOR		1
#define DTPLAY_VERSION_BUGFIX		1

// Command line option flags
const char c_LoopCountCmdLineFlag[]			= "l";

const char c_DvcTypeCmdLineFlag[]			= "t";
const char c_DvcNumCmdLineFlag[]			= "n";

const char c_TxModeCmdLineFlag[]			= "m";
const char c_TxRateCmdLineFlag[]			= "r";

const char c_ModTypeCmdLineFlag[]			= "mt";
const char c_CarrierFreqCmdLineFlag[]		= "mf";
const char c_CodeRateCmdLineFlag[]			= "mc";
const char c_QamJ83AnnexFlag[]				= "ma";

const char c_OfdmConstellCmdLineFlag[]		= "mC";
const char c_OfdmBandwidthCmdLineFlag[]		= "mB";
const char c_OfdmTxModeCmdLineFlag[]		= "mT";
const char c_OfdmGaurdItvCmdLineFlag[]		= "mG";

const char c_DvbS2EnablePilotCmdLineFlag[]	= "mP";
const char c_DvbS2FecFrameLengthCmdLineFlag[] = "mF";

const char c_SilentModeCmdLineFlag[]		= "s";
const char c_HelpCmdLineFlag[]				= "?";

const int c_BufSize = 2*1024*1024;		// Data transfer buffer size
const int c_MinFifoLoad	= 4*1024*1024;	// Minimum fifo load before starting transmission

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Error messages -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

// Command line errors
const char c_CleUnknownOption[]			= "Unknown command-line option (-%s) encountered";
const char c_CleMissingArgument[]		= "Missing argument for command-line option: -%s";
const char c_CleNoArgumentExpected[]	= "-%s command-line option does not have a argument";
const char c_CleInvalidArgument[]		= "Invalid argument for command line option: -%s";
const char c_CleNoPlayFile[]			= "No play file specified";
const char c_CleTxRateNotDefined[]		= "Transport-Stream rate not specified";

// General errors
const char c_ErrDtapiHwScanFailed[]		= "DtapiHwFuncScan failed (ERROR: %s)";
const char c_ErrNoOutputFound[]			= "No output device in the system";
const char c_ErrNoDta1xxFound[]			= "No DTA-%d found";
const char c_ErrNoDtu2xxFound[]			= "No DTU-%d found";
const char c_ErrCouldNotFindOutputN[]	= "Could not find output board (#%d) in the system";
const char c_ErrCouldNotFindDta1xxN[]	= "Could not find DTA-%d (#%d) in the system";
const char c_ErrCouldNotFindDtu2xxN[]	= "Could not find DTU-%d (#%d) in the system";
const char c_ErrFailToAttachToDta1xx[]	= "Failed to attach to the DTA-%d on Bus: %d and Slot: %d";
const char c_ErrFailToAttachToDtu2xx[]	= "Failed to attach to the DTU-%d";
const char c_ErrFailToAttachToChan[]	= "Can't attach to the channel (ERROR: %s)";
const char c_ErrFailToOpenFile[]		= "Can't open '%s' for reading";
const char c_ErrReadFile[]				= "File read error";
const char c_ErrFailSetTxControl[]		= "SetTxControl failed (ERROR: %s)";
const char c_ErrFailSetTxMode[]			= "SetTxMode failed (ERROR: %s)";
const char c_ErrFailSetModControl[]		= "SetModControl failed (ERROR: %s)";
const char c_ErrFailSetRfControl[]		= "SetRfControl failed (ERROR: %s)";
const char c_ErrFailSetTsRate[]			= "SetTsrateBps failed (ERROR: %s)";
const char c_ErrFailWrite[]				= "Write failed (ERROR: %s)";


const char c_ErrDriverInCompPci[] =
	"The current Dta1xx driver (V%d.%d.%d %d) is not compatible with this\n" \
	"version of DtPlay.\n" \
	"Please install the latest version of the Dta1xx driver.";

const char c_ErrDriverInCompUsb[] =
	"The current Dtu2xx driver (V%d.%d.%d %d) is not compatible with this\n" \
	"version of DtPlay.\n" \
	"Please install the latest version of the Dtu2xx driver.";



//+=+=+=+=+=+=+=+=+=+=+=+=+=+ CommandLineParams implementation +=+=+=+=+=+=+=+=+=+=+=+=+=+

//-.-.-.-.-.-.-.-.-.-.-.-.- CommandLineParams::CommandLineParams -.-.-.-.-.-.-.-.-.-.-.-.-
//
CommandLineParams::CommandLineParams() :
	m_pLastFlag(NULL),
	m_LastFlagReqsArg(false),
	m_Error(false)
{
}

//.-.-.-.-.-.-.-.-.-.-.-.- CommandLineParams::~CommandLineParams -.-.-.-.-.-.-.-.-.-.-.-.-
//
CommandLineParams::~CommandLineParams()
{
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- CommandLineParams::Init -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
void CommandLineParams::Init()
{
	m_FileName[0] = '\0';
	m_LoopCnt = 1;

	m_DvcType = -1;
	m_DvcNum = 1;

	m_TxMode = -1;
	m_TxRate = -1;

	m_ModType = -1;
	m_QamJ83Annex = DTAPI_MOD_J83_A;
	m_CarrierFreq = -1.0;
	m_CodeRate = DTAPI_MOD_3_4;

	m_OfdmConstellation = DTAPI_MOD_OFDM_QAM64;
	m_OfdmBandwidth = DTAPI_MOD_OFDM_8MHZ;
	m_OfdmTxMode = DTAPI_MOD_OFDM_8K;
	m_OfdmGaurdItv = DTAPI_MOD_OFDM_G_1_4;

	m_DvbS2Pilots = DTAPI_MOD_S2_NOPILOTS;
	m_DvbS2FecFrameLength = DTAPI_MOD_S2_LONGFRM;

	m_SilentMode = false;
	m_ShowHelp = false;
}

//-.-.-.-.-.-.-.-.-.-.-.-.- CommandLineParams::ParseCommandLine -.-.-.-.-.-.-.-.-.-.-.-.-.
//
void CommandLineParams::ParseCommandLine(int argc, char* argv[])
{
	bool First = true;
	for ( int i=1; i<argc; i++ )
	{
		char* pParam = argv[i];
		bool Flag = false;
		bool Last = ( (i+1)==argc );
#ifdef WIN32
		if ( pParam[0]=='-' || pParam[0] == '/' )
#else
                if ( pParam[0]=='-' ) // For Linux only '-' can be used for options
#endif
		{
			// remove flag specifier
			Flag = true;
			++pParam;
		}
		ParseParam(pParam, Flag, First, Last);
		First = false;
	}

	// Check for required parameters
	if ( strlen(m_FileName) == 0 && !m_ShowHelp )
		throw Exc(c_CleNoPlayFile);

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Set dynamic option default -.-.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	// Set default settings for parameters which are not defined on the command line

	// TxMode defined?
	if ( m_TxMode == -1 && m_ModType==DTAPI_MOD_ISDBT )
		m_TxMode = DTAPI_TXMODE_204;
	else if ( m_TxMode == -1 )
		m_TxMode = DTAPI_TXMODE_188;

	// Carrier defined?
	if ( m_CarrierFreq == -1.0 )
	{
		if ( m_ModType==DTAPI_MOD_QPSK || m_ModType==DTAPI_MOD_BPSK )
			m_CarrierFreq = 1915.0;		// L-Band
		else /* ISDBT, OFDM, QAM */
			m_CarrierFreq = 578.0;		// UHF-band
	}
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- CommandLineParams::ParseParam -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
void CommandLineParams::ParseParam(
	char* pParam,
	bool Flag,
	bool First,
	bool Last)
{

	if ( Flag )
		ParseParamFlag(pParam, First, Last);
	else
		ParseParamNotFlag(pParam, First, Last);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.- CommandLineParams::ParseParamFlag -.-.-.-.-.-.-.-.-.-.-.-.-.-
//
void  CommandLineParams::ParseParamFlag(char* pParam, bool First, bool Last)
{
	// Did we expect an option??
	if ( m_LastFlagReqsArg && m_pLastFlag != NULL )
		throw Exc(c_CleMissingArgument, m_pLastFlag);


	if ( 0==strcmp(pParam, c_LoopCountCmdLineFlag) )
        m_LastFlagReqsArg = true;
	else if ( 0==strcmp(pParam, c_DvcTypeCmdLineFlag) )
		m_LastFlagReqsArg = true;
	else if ( 0==strcmp(pParam, c_DvcNumCmdLineFlag) )
		m_LastFlagReqsArg = true;
	else if ( 0==strcmp(pParam, c_TxModeCmdLineFlag) )
		m_LastFlagReqsArg = true;
	else if ( 0==strcmp(pParam, c_TxRateCmdLineFlag) )
		m_LastFlagReqsArg = true;
	else if ( 0==strcmp(pParam, c_ModTypeCmdLineFlag) )
		m_LastFlagReqsArg = true;
	else if ( 0==strcmp(pParam, c_QamJ83AnnexFlag) )
		m_LastFlagReqsArg = true;
	else if ( 0==strcmp(pParam, c_CarrierFreqCmdLineFlag) )
		m_LastFlagReqsArg = true;
	else if ( 0==strcmp(pParam, c_CodeRateCmdLineFlag) )
		m_LastFlagReqsArg = true;
	else if ( 0==strcmp(pParam, c_OfdmConstellCmdLineFlag) )
		m_LastFlagReqsArg = true;
	else if ( 0==strcmp(pParam, c_OfdmBandwidthCmdLineFlag) )
		m_LastFlagReqsArg = true;
	else if ( 0==strcmp(pParam, c_OfdmTxModeCmdLineFlag) )
		m_LastFlagReqsArg = true;
	else if ( 0==strcmp(pParam, c_OfdmGaurdItvCmdLineFlag) )
		m_LastFlagReqsArg = true;
	else if ( 0==strcmp(pParam, c_SilentModeCmdLineFlag))
	{
		m_SilentMode = true;
		m_LastFlagReqsArg = false;
	}
#ifdef WIN32
	else if ( 0==strcmp(pParam, c_DvbS2EnablePilotCmdLineFlag) )
		m_LastFlagReqsArg = true;
	else if ( 0==strcmp(pParam, c_DvbS2FecFrameLengthCmdLineFlag) )
		m_LastFlagReqsArg = true;
#endif
	else if ( 0==strcmp(pParam, c_HelpCmdLineFlag))
	{
		m_ShowHelp = true;
		m_LastFlagReqsArg = false;
	}
	else
		throw Exc(c_CleUnknownOption, pParam);

	m_pLastFlag = pParam;
}

//-.-.-.-.-.-.-.-.-.-.-.-.- CommandLineParams::ParseParamNotFlag -.-.-.-.-.-.-.-.-.-.-.-.-
//
void  CommandLineParams::ParseParamNotFlag(char* pParam, bool First, bool Last)
{
	// First option should be our file name
	if ( First )
	{
		strcpy(m_FileName, pParam);
		m_pLastFlag = NULL;
		m_LastFlagReqsArg = false;
		return;
	}

	// Did we expect and option??
	if ( m_pLastFlag==NULL )
		return;
	if ( !m_LastFlagReqsArg )
		throw Exc(c_CleNoArgumentExpected, m_pLastFlag);


	if ( 0==strcmp(m_pLastFlag, c_LoopCountCmdLineFlag) )
	{
		m_LoopCnt = atoi(pParam);
		if ( m_LoopCnt<0  )
			throw Exc(c_CleInvalidArgument, m_pLastFlag);
	}
    else if ( 0==strcmp(m_pLastFlag, c_DvcTypeCmdLineFlag) )
	{
		m_DvcType = atoi(pParam);
		if ( m_DvcType < 100 )
			throw Exc(c_CleInvalidArgument, m_pLastFlag);
	}
	else if ( 0==strcmp(m_pLastFlag, c_DvcNumCmdLineFlag) )
	{
		m_DvcNum = atoi(pParam);
		if ( m_DvcNum < 1 )
			throw Exc(c_CleInvalidArgument, m_pLastFlag);
	}
	else if ( 0==strcmp(m_pLastFlag, c_TxModeCmdLineFlag) )
	{
		// Which tx-mode
		if ( 0 == strcmp(pParam, "188")  )
			m_TxMode = DTAPI_TXMODE_188;
		else if ( 0 == strcmp(pParam, "ADD16")  )
			m_TxMode = DTAPI_TXMODE_ADD16;
		else if ( 0 == strcmp(pParam, "130")  )
			m_TxMode = DTAPI_TXMODE_130;
		else if ( 0 == strcmp(pParam, "192")  )
			m_TxMode = DTAPI_TXMODE_192;
		else if ( 0 == strcmp(pParam, "204")  )
			m_TxMode = DTAPI_TXMODE_204;
		else if ( 0 == strcmp(pParam, "RAW")  )
			m_TxMode = DTAPI_TXMODE_RAW;
		else if ( 0 == strcmp(pParam, "MIN16")  )
			m_TxMode = DTAPI_TXMODE_MIN16;
		else if ( 0 == strcmp(pParam, "SDI8B")  )
			m_TxMode = DTAPI_TXMODE_SDI_FULL;
		else if ( 0 == strcmp(pParam, "SDI10B")  )
			m_TxMode = DTAPI_TXMODE_SDI_FULL | DTAPI_TXMODE_SDI_10B;
		else
			throw Exc(c_CleInvalidArgument, m_pLastFlag);
	}
	else if ( 0==strcmp(m_pLastFlag, c_TxRateCmdLineFlag) )
	{
		m_TxRate = atoi(pParam);
		if ( m_TxRate <= 0 )
			throw Exc(c_CleInvalidArgument, m_pLastFlag);
	}
	else if ( 0==strcmp(m_pLastFlag, c_ModTypeCmdLineFlag) )
	{
		// Which modulation mode
		if ( 0 == strcmp(pParam, "QPSK")  )
			m_ModType = DTAPI_MOD_QPSK;
		else if ( 0 == strcmp(pParam, "BPSK")  )
			m_ModType = DTAPI_MOD_BPSK;
		else if ( 0 == strcmp(pParam, "QAM4")  )
			m_ModType = DTAPI_MOD_QAM4;
		else if ( 0 == strcmp(pParam, "QAM16") )
			m_ModType = DTAPI_MOD_QAM16;
		else if ( 0 == strcmp(pParam, "QAM32") )
			m_ModType = DTAPI_MOD_QAM32;
		else if ( 0 == strcmp(pParam, "QAM64") )
			m_ModType = DTAPI_MOD_QAM64;
		else if ( 0 == strcmp(pParam, "QAM128") )
			m_ModType = DTAPI_MOD_QAM128;
		else if ( 0 == strcmp(pParam, "QAM256") )
			m_ModType = DTAPI_MOD_QAM256;
		else if ( 0 == strcmp(pParam, "OFDM") )
			m_ModType = DTAPI_MOD_OFDM;
#ifdef WIN32
		else if ( 0 == strcmp(pParam, "ISDBT") )
			m_ModType = DTAPI_MOD_ISDBT;
		else if ( 0 == strcmp(pParam, "DVBS2_QPSK") )
			m_ModType = DTAPI_MOD_DVBS2_QPSK;
		else if ( 0 == strcmp(pParam, "DVBS2_8PSK") )
			m_ModType = DTAPI_MOD_DVBS2_8PSK;
#endif
		else
			throw Exc(c_CleInvalidArgument, m_pLastFlag);
	}
	else if ( 0==strcmp(m_pLastFlag, c_CarrierFreqCmdLineFlag) )
	{
		m_CarrierFreq = atof(pParam);
		if ( m_CarrierFreq < 0.0 )
			throw Exc(c_CleInvalidArgument, m_pLastFlag);
	}
	else if ( 0==strcmp(m_pLastFlag, c_CodeRateCmdLineFlag) )
	{
		// Which mode
		if ( 0 == strcmp(pParam, "1/2")  )
			m_CodeRate = DTAPI_MOD_1_2;
		else if ( 0 == strcmp(pParam, "2/3")  )
			m_CodeRate = DTAPI_MOD_2_3;
		else if ( 0 == strcmp(pParam, "3/4")  )
			m_CodeRate = DTAPI_MOD_3_4;
		else if ( 0 == strcmp(pParam, "4/5")  )
			m_CodeRate = DTAPI_MOD_4_5;
		else if ( 0 == strcmp(pParam, "5/6")  )
			m_CodeRate = DTAPI_MOD_5_6;
		else if ( 0 == strcmp(pParam, "6/7")  )
			m_CodeRate = DTAPI_MOD_6_7;
		else if ( 0 == strcmp(pParam, "7/8")  )
			m_CodeRate = DTAPI_MOD_7_8;
#ifdef WIN32
		else if ( 0 == strcmp(pParam, "1/3")  )
			m_CodeRate = DTAPI_MOD_1_3;
		else if ( 0 == strcmp(pParam, "2/5")  )
			m_CodeRate = DTAPI_MOD_2_5;
		else if ( 0 == strcmp(pParam, "3/5")  )
			m_CodeRate = DTAPI_MOD_3_5;
		else if ( 0 == strcmp(pParam, "8/9")  )
			m_CodeRate = DTAPI_MOD_8_9;
		else if ( 0 == strcmp(pParam, "9/10")  )
			m_CodeRate = DTAPI_MOD_9_10;
#endif
		else
			throw Exc(c_CleInvalidArgument, m_pLastFlag);
	}
	else if ( 0==strcmp(m_pLastFlag, c_QamJ83AnnexFlag) )
	{
		if ( 0 == strcmp(pParam, "A")  )
			m_QamJ83Annex = DTAPI_MOD_J83_A;
		else if ( 0 == strcmp(pParam, "B") )
			m_QamJ83Annex = DTAPI_MOD_J83_B;
		else if ( 0 == strcmp(pParam, "C") )
			m_QamJ83Annex = DTAPI_MOD_J83_C;
		else
			throw Exc(c_CleInvalidArgument, m_pLastFlag);
	}
	else if ( 0==strcmp(m_pLastFlag, c_OfdmConstellCmdLineFlag) )
	{
		if ( 0 == strcmp(pParam, "QPSK")  )
			m_OfdmConstellation = DTAPI_MOD_OFDM_QPSK;
		else if ( 0 == strcmp(pParam, "QAM16") )
			m_OfdmConstellation = DTAPI_MOD_OFDM_QAM16;
		else if ( 0 == strcmp(pParam, "QAM64") )
			m_OfdmConstellation = DTAPI_MOD_OFDM_QAM64;
		else
			throw Exc(c_CleInvalidArgument, m_pLastFlag);
	}
	else if ( 0==strcmp(m_pLastFlag, c_OfdmBandwidthCmdLineFlag) )
	{
		if ( 0 == strcmp(pParam, "5") )
			m_OfdmBandwidth = DTAPI_MOD_OFDM_5MHZ;
		else if ( 0 == strcmp(pParam, "6") )
			m_OfdmBandwidth = DTAPI_MOD_OFDM_6MHZ;
		else if ( 0 == strcmp(pParam, "7") )
			m_OfdmBandwidth = DTAPI_MOD_OFDM_7MHZ;
		else if ( 0 == strcmp(pParam, "8") )
			m_OfdmBandwidth = DTAPI_MOD_OFDM_8MHZ;
		else
			throw Exc(c_CleInvalidArgument, m_pLastFlag);

	}
	else if ( 0==strcmp(m_pLastFlag, c_OfdmTxModeCmdLineFlag) )
	{
		if ( 0 == strcmp(pParam, "2k") )
			m_OfdmTxMode = DTAPI_MOD_OFDM_2K;
		else if ( 0 == strcmp(pParam, "4k") )
			m_OfdmTxMode = DTAPI_MOD_OFDM_4K;
		else if ( 0 == strcmp(pParam, "8k") )
			m_OfdmTxMode = DTAPI_MOD_OFDM_8K;
		else
			throw Exc(c_CleInvalidArgument, m_pLastFlag);
	}
	else if ( 0==strcmp(m_pLastFlag, c_OfdmGaurdItvCmdLineFlag) )
	{
		if ( 0 == strcmp(pParam, "1/4") )
			m_OfdmGaurdItv = DTAPI_MOD_OFDM_G_1_4;
		else if ( 0 == strcmp(pParam, "1/8") )
			m_OfdmGaurdItv = DTAPI_MOD_OFDM_G_1_8;
		else if ( 0 == strcmp(pParam, "1/16") )
			m_OfdmGaurdItv = DTAPI_MOD_OFDM_G_1_16;
		else if ( 0 == strcmp(pParam, "1/32") )
			m_OfdmGaurdItv = DTAPI_MOD_OFDM_G_1_32;
		else
			throw Exc(c_CleInvalidArgument, m_pLastFlag);
	}
#ifdef WIN32
	else if ( 0==strcmp(m_pLastFlag, c_DvbS2EnablePilotCmdLineFlag) )
	{
		if ( 0 == strcmp(pParam, "OFF") )
			m_DvbS2Pilots = DTAPI_MOD_S2_NOPILOTS;
		else if ( 0 == strcmp(pParam, "ON") )
			m_DvbS2Pilots = DTAPI_MOD_S2_PILOTS;
		else
			throw Exc(c_CleInvalidArgument, m_pLastFlag);
	}
	else if ( 0==strcmp(m_pLastFlag, c_DvbS2FecFrameLengthCmdLineFlag) )
	{
		if ( 0 == strcmp(pParam, "SHORT") )
			m_DvbS2FecFrameLength = DTAPI_MOD_S2_SHORTFRM;
		else if ( 0 == strcmp(pParam, "LONG") )
			m_DvbS2FecFrameLength = DTAPI_MOD_S2_LONGFRM;
		else
			throw Exc(c_CleInvalidArgument, m_pLastFlag);
	}
#endif
	else
		throw Exc(c_CleUnknownOption, m_pLastFlag);

	// Clear required argument flag
	m_LastFlagReqsArg = false;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- CommandLineParams::TxMode2Str -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
char* CommandLineParams::TxMode2Str() const
{

	switch( m_TxMode )
	{
	case DTAPI_TXMODE_188:		return "188";
	case DTAPI_TXMODE_ADD16:	return "ADD16";
	case DTAPI_TXMODE_130:		return "130";
	case DTAPI_TXMODE_192:		return "192";
	case DTAPI_TXMODE_204:		return "204";
	case DTAPI_TXMODE_RAW:		return "RAW";
	case DTAPI_TXMODE_MIN16:	return "MIN16";
	case DTAPI_TXMODE_SDI_FULL:	return "SDI8B";
	default:					return "?";
	}
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- CommandLineParams::ModType2Str -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
char* CommandLineParams::ModType2Str() const
{
	switch( m_ModType )
	{
	case DTAPI_MOD_QPSK:		return "QPSK";
	case DTAPI_MOD_BPSK:		return "BPSK";
	case DTAPI_MOD_QAM4:		return "QAM4";
	case DTAPI_MOD_QAM16:		return "QAM16";
	case DTAPI_MOD_QAM32:		return "QAM32";
	case DTAPI_MOD_QAM64:		return "QAM64";
	case DTAPI_MOD_QAM128:		return "QAM128";
	case DTAPI_MOD_QAM256:		return "QAM256";
	case DTAPI_MOD_OFDM:		return "OFDM";
#ifdef WIN32
	case DTAPI_MOD_ISDBT:		return "ISDBT";
	case DTAPI_MOD_DVBS2_QPSK:	return "QPSK (DVBS2)";
	case DTAPI_MOD_DVBS2_8PSK:	return "8PSK (DVBS2)";
#endif
	default:					return "?";
	}
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.- CommandLineParams::CodeRate2Str -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
char* CommandLineParams::CodeRate2Str() const
{
	switch( m_CodeRate )
	{
	case DTAPI_MOD_1_2:		return "1/2";
	case DTAPI_MOD_2_3:		return "2/3";
	case DTAPI_MOD_3_4:		return "3/4";
	case DTAPI_MOD_4_5:		return "4/5";
	case DTAPI_MOD_5_6:		return "5/6";
	case DTAPI_MOD_6_7:		return "6/7";
	case DTAPI_MOD_7_8:		return "7/8";
#ifdef WIN32
	case DTAPI_MOD_1_3:		return "1/3";
	case DTAPI_MOD_2_5:		return "2/5";
	case DTAPI_MOD_3_5:		return "3/5";
	case DTAPI_MOD_8_9:		return "8/9";
	case DTAPI_MOD_9_10:	return "9/10";
#endif
	default:				return "?";
	}
}

//.-.-.-.-.-.-.-.-.-.-.-.-.- CommandLineParams::QamJ83Annex2Str -.-.-.-.-.-.-.-.-.-.-.-.-.
//
char* CommandLineParams::QamJ83Annex2Str() const
{
	switch( m_QamJ83Annex )
	{
	case DTAPI_MOD_J83_A:		return "J.83 Annex A";
	case DTAPI_MOD_J83_B:		return "J.83 Annex B";
	case DTAPI_MOD_J83_C:		return "J.83 Annex C";
	default:					return "?";
	}
}

//-.-.-.-.-.-.-.-.-.-.-.-.- CommandLineParams::OfdmBandwidth2Str -.-.-.-.-.-.-.-.-.-.-.-.-
//
char* CommandLineParams::OfdmBandwidth2Str() const
{
	switch( m_OfdmBandwidth )
	{
	case DTAPI_MOD_OFDM_5MHZ:	return "5MHz";
	case DTAPI_MOD_OFDM_6MHZ:	return "6MHz";
	case DTAPI_MOD_OFDM_7MHZ:	return "7MHz";
	case DTAPI_MOD_OFDM_8MHZ:	return "8MHz";
	default:					return "?";
	}
}

//-.-.-.-.-.-.-.-.-.-.-.- CommandLineParams::OfdmConstellation2Str -.-.-.-.-.-.-.-.-.-.-.-
//
char* CommandLineParams::OfdmConstellation2Str() const
{
	switch( m_OfdmConstellation )
	{
	case DTAPI_MOD_OFDM_QPSK:	return "QPSK";
	case DTAPI_MOD_OFDM_QAM16:	return "QAM16";
	case DTAPI_MOD_OFDM_QAM64:	return "QAM64";
	default:					return "?";
	}
}

//-.-.-.-.-.-.-.-.-.-.-.-.- CommandLineParams::OfdmGaurdItv2Str -.-.-.-.-.-.-.-.-.-.-.-.-.
//
char* CommandLineParams::OfdmGaurdItv2Str() const
{
	switch( m_OfdmGaurdItv )
	{
	case DTAPI_MOD_OFDM_G_1_4:	return "1/4";
	case DTAPI_MOD_OFDM_G_1_8:	return "1/8";
	case DTAPI_MOD_OFDM_G_1_16:	return "1/16";
	case DTAPI_MOD_OFDM_G_1_32:	return "1/32";
	default:					return "?";
	}
}

//.-.-.-.-.-.-.-.-.-.-.-.-.- CommandLineParams::OfdmTxMode2Str -.-.-.-.-.-.-.-.-.-.-.-.-.-
//
char* CommandLineParams::OfdmTxMode2Str() const
{
	switch( m_OfdmTxMode )
	{
	case DTAPI_MOD_OFDM_2K:		return "2k";
	case DTAPI_MOD_OFDM_4K:		return "4k";
	case DTAPI_MOD_OFDM_8K:		return "8k";
	default:					return "?";
	}
}

#ifdef WIN32
//.-.-.-.-.-.-.-.-.-.-.-.-.- CommandLineParams::DvbS2Pilots2Str -.-.-.-.-.-.-.-.-.-.-.-.-.
//
char* CommandLineParams::DvbS2Pilots2Str() const
{
	switch ( m_DvbS2Pilots )
	{
	case DTAPI_MOD_S2_NOPILOTS:	return "OFF";
	case DTAPI_MOD_S2_PILOTS:	return "ON";
	default:					return "?";
	}
}

//.-.-.-.-.-.-.-.-.-.-.- CommandLineParams::DvbS2FecFrameLength2Str -.-.-.-.-.-.-.-.-.-.-.
//
char* CommandLineParams::DvbS2FecFrameLength2Str() const
{
	switch ( m_DvbS2FecFrameLength )
	{
	case DTAPI_MOD_S2_SHORTFRM:	return "SHORT";
	case DTAPI_MOD_S2_LONGFRM:	return "LONG";
	default:					return "?";
	}
}
#endif

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Player implementation +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Player::Player -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
Player::Player() : m_pFile(NULL), m_pBuf(NULL)
{
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Player::~Player -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
Player::~Player()
{
	// Detach from channel and device
	m_DtOutp.SetTxControl(DTAPI_TXCTRL_IDLE);
	m_DtOutp.Detach(DTAPI_INSTANT_DETACH);
	m_DtDvc.Detach();

	// Free our buffer
	if ( m_pBuf != NULL )
		delete [] m_pBuf;

	// donot forget to close our file
	if ( m_pFile != NULL )
		::fclose(m_pFile);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Player::AttachToOutput -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
void Player::AttachToOutput()
{
	DTAPI_RESULT dr;
	int n, NumHwFuncsFound(0);
	DtHwFuncDesc HwFuncs[20];

	// Scan for available DekTec devices
	dr = ::DtapiHwFuncScan(sizeof(HwFuncs)/sizeof(HwFuncs[0]), NumHwFuncsFound, HwFuncs);
	if ( dr != DTAPI_OK )
		throw Exc(c_ErrDtapiHwScanFailed, ::DtapiResult2Str(dr) );

	// Loop through found hardware functions
	bool Found(false);
	int DeviceNum(0);
	DtHwFuncDesc* pHwf = &HwFuncs[0];
	for ( n=0; n<NumHwFuncsFound; n++, pHwf++ )
	{
		// Skip non-outputs
		if ( 0==(pHwf->m_ChanType & DTAPI_CHAN_OUTPUT) )
			continue;

		// Looking for a specific type??
		if ( pHwf->m_DvcDesc.m_TypeNumber == m_CmdLineParams.m_DvcType )
			DeviceNum++;	// only count devices matching our search type
		else if ( m_CmdLineParams.m_DvcType == -1 )
			DeviceNum++;	// count every device

		if ( DeviceNum == m_CmdLineParams.m_DvcNum ) {
			Found = true;
			break;
		}
	}
	// Did we find our device?
	if ( !Found && DeviceNum==0 )
	{
		// Could not find any matching output at all
		if (   (m_CmdLineParams.m_DvcType>=100 && m_CmdLineParams.m_DvcType<200)
			|| (m_CmdLineParams.m_DvcType>=2100 && m_CmdLineParams.m_DvcType<2200) )
			throw Exc(c_ErrNoDta1xxFound, m_CmdLineParams.m_DvcType);
		else if (   (m_CmdLineParams.m_DvcType>=200 && m_CmdLineParams.m_DvcType<300) )
			throw Exc(c_ErrNoDtu2xxFound, m_CmdLineParams.m_DvcType);
		else
			throw Exc(c_ErrNoOutputFound);
	}
	else if ( !Found && m_CmdLineParams.m_DvcNum > DeviceNum )
	{
		// Could no find the Nth matching output
		if (   (m_CmdLineParams.m_DvcType>=100 && m_CmdLineParams.m_DvcType<200)
			|| (m_CmdLineParams.m_DvcType>=2100 && m_CmdLineParams.m_DvcType<2200) )
			throw Exc(c_ErrCouldNotFindDta1xxN, m_CmdLineParams.m_DvcType, m_CmdLineParams.m_DvcNum);
		else if (   (m_CmdLineParams.m_DvcType>=200 && m_CmdLineParams.m_DvcType<300) )
			throw Exc(c_ErrCouldNotFindDtu2xxN, m_CmdLineParams.m_DvcType, m_CmdLineParams.m_DvcNum);
		else
			throw Exc(c_ErrCouldNotFindOutputN, m_CmdLineParams.m_DvcNum);
	}

	// Attach to the device
	dr = m_DtDvc.AttachToSerial(pHwf->m_DvcDesc.m_Serial);
	if ( dr == DTAPI_E_DRIVER_INCOMP )
	{
		// Special case: driver version is not compatible with this version of DtPlay,
		// get the driver version and throw an exception
		int DriverVersionMajor(-1), DriverVersionMinor(-1), DriverVersionBugFix(-1),
			DriverVersionBuild(-1);

		// Get device driver version
		::DtapiGetDeviceDriverVersion(pHwf->m_DvcDesc.m_Category,
									  DriverVersionMajor, DriverVersionMinor,
									  DriverVersionBugFix, DriverVersionBuild);

		// PCI or USB device??
		if ( pHwf->m_DvcDesc.m_Category == DTAPI_CAT_PCI )
		{
			throw Exc( c_ErrDriverInCompPci, DriverVersionMajor, DriverVersionMinor,
					   DriverVersionBugFix, DriverVersionBuild );
		}
		else if ( pHwf->m_DvcDesc.m_Category == DTAPI_CAT_USB )
		{
			throw Exc( c_ErrDriverInCompUsb, DriverVersionMajor, DriverVersionMinor,
					   DriverVersionBugFix, DriverVersionBuild );
		}
	}
	else if ( dr != DTAPI_OK )
	{
		// PCI or USB device??
		if ( pHwf->m_DvcDesc.m_Category == DTAPI_CAT_PCI )
			throw Exc(c_ErrFailToAttachToDta1xx, pHwf->m_DvcDesc.m_TypeNumber,
					  pHwf->m_DvcDesc.m_PciBusNumber, pHwf->m_DvcDesc.m_SlotNumber);
		else
			throw Exc(c_ErrFailToAttachToDtu2xx, pHwf->m_DvcDesc.m_TypeNumber);
	}
	
	// Attach to the output channel
	dr = m_DtOutp.AttachToPort(&m_DtDvc, pHwf->m_Port);
	if ( dr != DTAPI_OK )
		throw Exc(c_ErrFailToAttachToChan, ::DtapiResult2Str(dr));
		


	// Check for modulator card i.e DTA-107, DTA-110(T), DTA-112 or DTA-115
	if (   m_DtOutp.TypeNumber()==107 || m_DtOutp.TypeNumber()==110
		|| m_DtOutp.TypeNumber()==112 || m_DtOutp.TypeNumber()==115 )
		m_Modulator = true;
	else
		m_Modulator = false;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Player::DisplayPlayInfo -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
void Player::DisplayPlayInfo()
{
	char sz[256];

	LogF("");
	LogF("Start playing");
	LogF("- Play file name        : %s", m_CmdLineParams.m_FileName);
	LogF("- Loop file             : %dx", m_CmdLineParams.m_LoopCnt);

	// Skip bit-rate for OFDM/ISDBT modulator and in case of SDI
	if ( !(   ( m_Modulator
				&& (   m_CmdLineParams.m_ModType==DTAPI_MOD_OFDM
					|| m_CmdLineParams.m_ModType==DTAPI_MOD_ISDBT ) )
           || ( !m_Modulator && 0!=(m_CmdLineParams.m_TxMode&DTAPI_TXMODE_SDI)) ) )
	{
		LogF("- Transport-Stream rate : %d bps", m_CmdLineParams.m_TxRate);
	}
	LogF("- Transmit Mode         : %s", m_CmdLineParams.TxMode2Str() );
	if ( m_DtDvc.Category() == DTAPI_CAT_USB )
		sprintf(sz, "DTU-%d", m_DtDvc.TypeNumber() );
	else /*if ( m_DtDvc.Category() == DTAPI_CAT_PCI )*/
		sprintf(sz, "DTA-%d", m_DtDvc.TypeNumber() );
	LogF("- Output device         : %s (#%d)", sz, m_CmdLineParams.m_DvcNum );

	// Do we have a modulator
	if ( m_Modulator )
	{
		LogF("- Modulation Type       : %s", m_CmdLineParams.ModType2Str() );
		LogF("- Carrier Frequency     : %.2f MHz", m_CmdLineParams.m_CarrierFreq );

		// Code rate: only for OFDM, QPSK and BPSK
		if (   m_CmdLineParams.m_ModType==DTAPI_MOD_OFDM
			|| m_CmdLineParams.m_ModType==DTAPI_MOD_QPSK
			|| m_CmdLineParams.m_ModType==DTAPI_MOD_BPSK
#ifdef WIN32
			|| m_CmdLineParams.m_ModType==DTAPI_MOD_DVBS2_QPSK
			|| m_CmdLineParams.m_ModType==DTAPI_MOD_DVBS2_8PSK
#endif
			)
			LogF("- Convolutional Rate    : %s", m_CmdLineParams.CodeRate2Str() );

		// OFDM modulation parameters
		if ( m_CmdLineParams.m_ModType==DTAPI_MOD_OFDM )
		{
			LogF("- Bandwitdh             : %s", m_CmdLineParams.OfdmBandwidth2Str() );
			LogF("- Transmission-mode     : %s", m_CmdLineParams.OfdmTxMode2Str() );
			LogF("- Gaurd Interval        : %s", m_CmdLineParams.OfdmGaurdItv2Str() );
			LogF("- Constellation         : %s", m_CmdLineParams.OfdmConstellation2Str() );
		}
#ifdef WIN32
		// DVB-S2 modulation parameters
		else if (    m_CmdLineParams.m_ModType==DTAPI_MOD_DVBS2_QPSK
			      || m_CmdLineParams.m_ModType==DTAPI_MOD_DVBS2_8PSK )
		{
			LogF("- Pilots                : %s", m_CmdLineParams.DvbS2Pilots2Str() );
			LogF("- FEC Frame Length      : %s", m_CmdLineParams.DvbS2FecFrameLength2Str() );
		}
#endif
		// QAM modulation parameters
		else if (    m_CmdLineParams.m_ModType!=DTAPI_MOD_OFDM
				  && m_CmdLineParams.m_ModType!=DTAPI_MOD_ISDBT
				  && m_CmdLineParams.m_ModType!=DTAPI_MOD_QPSK
			      && m_CmdLineParams.m_ModType!=DTAPI_MOD_BPSK )
		{
			LogF("- J.83                  : %s", m_CmdLineParams.QamJ83Annex2Str() );
		}
	}

#ifdef WIN32
	Log("");
	Log("Press any key to stop playing");
	Log("");
#endif
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Player::InitOutput -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
void Player::InitOutput()
{
	DTAPI_RESULT dr;

	// Init channel to initial 'safe' state
	dr = m_DtOutp.SetTxControl(DTAPI_TXCTRL_IDLE);	// Start in IDLE mode
	if ( dr != DTAPI_OK )
		throw Exc( c_ErrFailSetTxControl, ::DtapiResult2Str(dr) );
	dr = m_DtOutp.SetFifoSizeMax();		// Set max FIFO size
	dr = m_DtOutp.ClearFifo();			// Clear FIFO (i.e. start with zero load)

	// Set the transmission mode. NOTE: ISDB-T only supports 204 byte mode
	dr = m_DtOutp.SetTxMode( m_CmdLineParams.m_TxMode, DTAPI_TXSTUFF_MODE_OFF );
	if ( dr != DTAPI_OK )
		throw Exc( c_ErrFailSetTxMode, ::DtapiResult2Str(dr) );

	// Apply modulation settings (if we have a modulator)
	if ( m_Modulator )
	{
		__int64 Freq = (__int64)(m_CmdLineParams.m_CarrierFreq*1E6);
		dr = m_DtOutp.SetRfControl( Freq );
		if ( dr != DTAPI_OK )
			throw Exc( c_ErrFailSetRfControl, ::DtapiResult2Str(dr) );

		// Set modulation control
		if ( m_CmdLineParams.m_ModType==DTAPI_MOD_OFDM )
		{
			int XtraPar1 =    m_CmdLineParams.m_OfdmBandwidth
							| m_CmdLineParams.m_OfdmConstellation
							| m_CmdLineParams.m_OfdmTxMode
							| m_CmdLineParams.m_OfdmGaurdItv
							| DTAPI_MOD_OFDM_NATIVE;

			dr = m_DtOutp.SetModControl( m_CmdLineParams.m_ModType,
										 m_CmdLineParams.m_CodeRate,
										 XtraPar1, 0);
		}
#ifdef WIN32
		else if ( m_CmdLineParams.m_ModType==DTAPI_MOD_ISDBT )
		{
			DtIsdbtPars IsdbtPars;
			// No muxing wanted, we have a 204 byte stream (with TMCC data) 
			IsdbtPars.m_DoMux = false;

			dr = m_DtOutp.SetModControl( IsdbtPars );
		}
#endif
		else if (   m_CmdLineParams.m_ModType==DTAPI_MOD_QPSK
			     || m_CmdLineParams.m_ModType==DTAPI_MOD_BPSK )
		{
			dr = m_DtOutp.SetModControl( m_CmdLineParams.m_ModType,
										 m_CmdLineParams.m_CodeRate,
										 0, 0 );
		}
#ifdef WIN32
		else if (    m_CmdLineParams.m_ModType==DTAPI_MOD_DVBS2_QPSK
                  || m_CmdLineParams.m_ModType==DTAPI_MOD_DVBS2_8PSK )
		{
			int XtraPar1 =   m_CmdLineParams.m_DvbS2Pilots
						   | m_CmdLineParams.m_DvbS2FecFrameLength;

			dr = m_DtOutp.SetModControl( m_CmdLineParams.m_ModType,
										 m_CmdLineParams.m_CodeRate,
										 XtraPar1, 0);
		}
#endif
		else // For now, assume it must be QAM then
		{
			dr = m_DtOutp.SetModControl( m_CmdLineParams.m_ModType,
										 m_CmdLineParams.m_QamJ83Annex,
										 0, 0);
		}
		if ( dr != DTAPI_OK )
			throw Exc( c_ErrFailSetModControl, ::DtapiResult2Str(dr) );
	}

	// Set bit-rate for non-SDI and modulators other than ISDBT/OFDM
    if (    (  !m_Modulator && 0==(m_CmdLineParams.m_TxMode&DTAPI_TXMODE_SDI) )
         || (   m_Modulator && m_CmdLineParams.m_ModType!=DTAPI_MOD_OFDM
             && m_CmdLineParams.m_ModType!=DTAPI_MOD_ISDBT ) )

	{
		dr = m_DtOutp.SetTsRateBps( m_CmdLineParams.m_TxRate );
		if ( dr != DTAPI_OK )
			throw Exc( c_ErrFailSetTsRate, ::DtapiResult2Str(dr) );
	}

	// Set exit load. 4k for all card except DTA-110T in OFDM/ISDBT mode and 
	// DTA-107S2
	m_ExitLoad = (   m_Modulator
				  && (   m_CmdLineParams.m_ModType==DTAPI_MOD_OFDM
					  || m_CmdLineParams.m_ModType==DTAPI_MOD_ISDBT
#ifdef WIN32
					  || m_CmdLineParams.m_ModType==DTAPI_MOD_DVBS2_QPSK
					  || m_CmdLineParams.m_ModType==DTAPI_MOD_DVBS2_8PSK
#endif
				  ) )
						? 1*1024*1024 : 4*1024;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Player::Log -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
void Player::Log(const char* pMessage,  bool IgnoreSilence)
{
    // Check for silent mode
	if ( !IgnoreSilence && m_CmdLineParams.m_SilentMode )
		return;
	// Print message and add new line
	printf(pMessage); printf("\n");
}
// Overload with optional parameters
void Player::LogF(const char* pMessage, ... )
{
	char sz[512];

	va_list  ArgList;
	va_start(ArgList, pMessage);
	_vsnprintf(sz, sizeof(sz)-1, pMessage, ArgList);
	va_end(ArgList);

	Log(sz, false);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Player::Play -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
void Player::LoopFile()
{
	bool EoF(false), TxStarted(false);
	DTAPI_RESULT dr;
	int NumBytesRead, FifoLoad;

	// Init channel to hold mode
	dr = m_DtOutp.SetTxControl(DTAPI_TXCTRL_HOLD);
	if ( dr != DTAPI_OK )
		throw Exc(c_ErrFailSetTxControl, ::DtapiResult2Str(dr));

	// Load number of times to play file
	int LoopCnt = m_CmdLineParams.m_LoopCnt;

#ifdef WIN32
	while ( !_kbhit() ) {
#else
	while ( 1 ) {
#endif
		// Read as much bytes as possible into our buffer
		NumBytesRead = (int)fread( m_pBuf, sizeof(char), c_BufSize, m_pFile );
		if ( NumBytesRead < c_BufSize )
		{
			// Read error or simple end-of-file
			if ( 0 != ferror(m_pFile) )
                throw Exc(c_ErrReadFile);
			else
				EoF = true;
		}

		if (NumBytesRead > 0) {
			// There is data write it to our output
			//
			// NOTE: Write will blocks until all data has been transferred to the output
			dr = m_DtOutp.Write( m_pBuf, NumBytesRead );
			if ( dr != DTAPI_OK )
				throw Exc(c_ErrFailWrite);
		} else {
			// There is no data to transfer, sleep a while before continuing
			#ifdef WIN32
				Sleep(10);
			#else
				usleep(10000);
			#endif 
		}	

		// We wait with starting actual transmission until we have build up a minimum FIFO
		// load. If we reach the end-of-file and only one loop left to go start immediately.
		dr = m_DtOutp.GetFifoLoad(FifoLoad);
		if ( !TxStarted && ( (EoF && LoopCnt==1) || FifoLoad>=c_MinFifoLoad ) )
		{
			// Start transmission
			dr = m_DtOutp.SetTxControl(DTAPI_TXCTRL_SEND);
			if ( dr != DTAPI_OK )
				throw Exc(c_ErrFailSetTxControl, ::DtapiResult2Str(dr));

			TxStarted = true;
		}

        // If we have reached the end-of-file check is the last data has been transmitted
		if ( EoF && TxStarted && FifoLoad<m_ExitLoad && LoopCnt==1 )
			break;	// Reached end-of-file, last loop and transmitted all data
		else if ( EoF && (LoopCnt>1 || LoopCnt==0) )
		{
			// We reached the end-of-file, but this is not the last loop

			// Loop infinitely?
			if ( LoopCnt!=0 )
				LoopCnt--;

			// Reset file position and end-of-file flag
			::fseek(m_pFile, 0, SEEK_SET);
			EoF = false;
		}
    }
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Player::Play -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
int Player::Play(int argc, char* argv[])
{
	int RetValue=0;
	try
	{
		try
		{
			// Parse command line options
			m_CmdLineParams.Init();
			m_CmdLineParams.ParseCommandLine(argc, argv);
		}
		catch ( Exc e ) {
			Log(e, true);
			// Force show help on exception
			m_CmdLineParams.m_ShowHelp = true;

			RetValue = -1;
		}
		catch ( ... ) {
			// Force show help on exception
			m_CmdLineParams.m_ShowHelp = true;

			RetValue = -1;
		}
		if ( m_CmdLineParams.m_ShowHelp )
		{
			// Disable silent mode
			m_CmdLineParams.m_SilentMode = false;
			ShowHelp();
			return RetValue;
		}

		//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Print start message -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

		LogF("DtPlay player V%d.%d.%d (c) 2000-2007 DekTec Digital Video B.V.\n",
			  DTPLAY_VERSION_MAJOR, DTPLAY_VERSION_MINOR, DTPLAY_VERSION_BUGFIX);


		//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Open the play file -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

		// Try to open a file for reading
		m_pFile = fopen( m_CmdLineParams.m_FileName, "rb");
		if ( m_pFile == NULL )
			throw Exc(c_ErrFailToOpenFile, m_CmdLineParams.m_FileName);

		//-.-.-.-.-.-.-.-.-.-.-.-.- Attach to the output channel -.-.-.-.-.-.-.-.-.-.-.-.-

		AttachToOutput();

		//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Final initialisation -.-.-.-.-.-.-.-.-.-.-.-.-.-.-

		// Init output
		InitOutput();

		// Create our transfer buffer
		m_pBuf = new char [c_BufSize];

		//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Start playing -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

		// Print start info
		DisplayPlayInfo();
		// Loop file
		LoopFile();
	}
	catch( Exc e )
	{
        Log(e, true);
		RetValue = -1;
	}

	// Detach from channel and device
	m_DtOutp.SetTxControl(DTAPI_TXCTRL_IDLE);
	m_DtOutp.Detach(DTAPI_INSTANT_DETACH);
	m_DtDvc.Detach();

	// Free our buffer
	if ( m_pBuf != NULL ) {
		delete [] m_pBuf;
		m_pBuf = NULL;
	}

	// donot forget to close our file
	if ( m_pFile != NULL ) {
		::fclose(m_pFile);
		m_pFile = NULL;
	}

	return RetValue;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Player::ShowHelp -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
void Player::ShowHelp()
{
	Log( "" );
	Log( "Usage:");
	Log( "   DtPlay playfile [-l count] [-r rate] [-t type] [-n number] [-m mode]\n" \
		 "          [-mt type] [-mf freq] [-mc coderate] [-ma annex] [-mC constellation]\n" \
		 "          [-mB bandwidth] [-mT txmode] [-mG gaurditv] [-s] [-?]");
	Log( "");
	Log("Where:");
	Log("   playfile is the name of the file to output");
	Log( "");
	Log("Options:");
	Log("   -l  Number of times to loop the file (0=loop infinitely)");
	Log("   -r  Transport-Stream Rate in bps");
	Log("   -t  Device type to use (default: any output device)");
	Log("         Use: 100  (for DTA-100)");
	Log("              102  (for DTA-102)");
	Log("              105  (for DTA-105)");
#ifdef WIN32
	Log("              107  (for DTA-107 or DTA-107S2)");
#else
	Log("              107  (for DTA-107)");
#endif
	Log("              110  (for DTA-110 or DTA-110T)");
	Log("              112  (for DTA-112)");
	Log("              115  (for DTA-115)");
	Log("              140  (for DTA-140)");
	Log("              145  (for DTA-145)");
	Log("              2145 (for DTA-2145)");
	Log("              160  (for DTA-160)");
	Log("              205  (for DTU-205)");
	Log("              245  (for DTU-245)");
	Log("              545  (for DTA-545)");
	Log("   -n  Device number to use (default: 1)");
	Log("   -m  Transmit Mode (default: 188)");
	Log("         Use: 188    188-byte packets in file, 188-byte packets transmitted");
	Log("              ADD16  188-byte packets in file, 204-byte packets transmitted");
	Log("                     (16 dummy bytes are automatically added to the packets)");
	Log("              130    130-byte packets in file, 130-byte packets transmitted");
	Log("              192    192-byte packets in file, 192-byte packets transmitted");
	Log("                     Note: this mode is only supported by the DTA-102");
	Log("              204    204-byte packets in file, 204-byte packets transmitted");
	Log("              MIN16  204-byte packets in file, 188-byte packets transmitted");
	Log("                     (last 16 bytes are invalidated)");
	Log("              RAW    No notion of packets, data is transmitted as is");
	Log("              SDI8B  8-Bit SDI in file, 8-bit SDI transmitted");
	Log("   -mt  Modulation type");
	Log("         Use:  QPSK   (for QPSK modulation)");
	Log("               BPSK   (for BPSK modulation)");
	Log("               QAM4   (for QAM-4 modulation)");
	Log("               QAM16  (for QAM-16 modulation)");
	Log("               QAM32  (for QAM-32 modulation)");
	Log("               QAM64  (for QAM-64 modulation)");
	Log("               QAM128 (for QAM-128 modulation)");
	Log("               QAM256 (for QAM-256 modulation)");
	Log("               OFDM   (for COFDM modulation)");
#ifdef WIN32
	Log("               ISDBT  (for ISDB-T modulation)");
	Log("               DVBS2_QPSK  (for DVBS2 QPSK modulation)");
	Log("               DVBS2_8PSK  (for DVBS2 8PSK modulation)");
#endif
	Log("   -mf  Modulation carrier frequency in MHz (default: 1915.0MHz)");
	Log("   -mc  Convolutional rate (default: 1/2)");
	Log("         Use:  1/2 (convolutional rate = 1/2)");
	Log("               2/3 (convolutional rate = 2/3)");
	Log("               3/4 (convolutional rate = 3/4)");
	Log("               4/5 (convolutional rate = 4/5)");
	Log("               5/6 (convolutional rate = 5/6)");
	Log("               6/7 (convolutional rate = 6/7)");
	Log("               7/8 (convolutional rate = 7/8)");
#ifdef WIN32
	Log("               1/3 (convolutional rate = 1/3)");
	Log("               2/5 (convolutional rate = 2/5)");
	Log("               3/5 (convolutional rate = 3/5)");
	Log("               8/9 (convolutional rate = 8/9)");
	Log("               9/10 (convolutional rate = 9/10)");
#endif
	Log("   -ma  J.83 Annex (default: Annex A");
	Log("         Use:  A   (for J.83 annex A (DVB-C))");
	Log("               B   (for J.83 annex B ('American QAM'))");
	Log("               C   (for J.83 annex C ('Japanese QAM'))");

	Log("   -mC  OFDM constellation (default: QAM64)");
	Log("         Use:  QPSK   (for QPSK modulation)");
	Log("               QAM16  (for QAM-16 modulation)");
	Log("               QAM64  (for QAM-64 modulation)");
	Log("   -mB  OFDM bandwidth (default: 8MHz)");
	Log("         Use:  5   (for 5MHz)");
	Log("               6   (for 6MHz)");
	Log("               7   (for 7MHz)");
	Log("               8   (for 8MHz)");
	Log("   -mT  OFDM transmission mode (default: 8k)");
	Log("         Use:  2k  (for 2k)");
	Log("               4k  (for 4k)");
	Log("               8k  (for 8k)");
	Log("   -mG  OFDM guard interval (default: 1/4)");
	Log("         Use:  1/4  (G=1/4)");
	Log("               1/8  (G=1/8)");
	Log("               1/16 (G=1/16)");
	Log("               1/32 (G=1/32)");
#ifdef WIN32
	Log("   -mP  Enable Pilots in DVB-S2 (default: OFF)");
	Log("         Use:  OFF  (Pilots disabled)");
	Log("               ON   (Pilots enabled)");
	Log("   -mF  Use long/short FEC frame in DVB-S2 (default: LONG)");
	Log("         Use:  SHORT  (Short FECFRAME)");
	Log("               LONG   (Long FECFRAME)");
#endif
	Log("   -s  Silent mode. No messages printed");
	Log("   -?  Display this help");
	Log("");
	Log("NOTE: The first option can only be the playfile or the help option");
	Log("");
	Log("Examples:");
	Log("   DtPlay myfile.ts -r 38000000");
	Log("   DtPlay myfile.ts -r 38000000 -t 100 -n 2 -m RAW ");
	Log("   DtPlay myfile.ts -r 38000000 -t 107 -mt QPSK -mf 1915.0 -mc 7/8");
	Log("");
	LogF("DtPlay version: %d.%d.%d\n", DTPLAY_VERSION_MAJOR, DTPLAY_VERSION_MINOR,
		 DTPLAY_VERSION_BUGFIX);
}

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Application entry point +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- main -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
int main(int argc, char* argv[])
{
	int RetValue(0);
	Player ThePlayer;
	try
	{
		RetValue = ThePlayer.Play(argc, argv);
	}
	catch(...)
	{
		RetValue = -1;
	}
	return RetValue;
}
