//*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* DtRecord.cpp *#*#*#*#*#*#*#*# (C) 2000-2007 DekTec
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	2007.02.10	MG	Created

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#include <stdio.h>
#include "DTAPI.h"
#include "DtRecord.h"

#ifdef WIN32
	#include <conio.h>
#else
	#include <stdlib.h>
	#include <string.h>
	#include <unistd.h>
#endif

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DtRecord Version -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define DTRECORD_VERSION_MAJOR		3
#define DTRECORD_VERSION_MINOR		0
#define DTRECORD_VERSION_BUGFIX		1

// Command line option flags
const char c_DvcTypeCmdLineFlag[]			= "t";
const char c_DvcNumCmdLineFlag[]			= "n";
const char c_PortCmdLineFlag[]				= "i";

const char c_RxModeCmdLineFlag[]			= "m";
const char c_MaxSizeCmdLineFlag[]			= "x";

const char c_SilentModeCmdLineFlag[]		= "s";
const char c_HelpCmdLineFlag[]				= "?";

const int c_BufSize = 2*1024*1024;		// Data transfer buffer size
const int c_MinFifoLoad	= 4*1024*1024;	// Minimum fifo load before starting transmission

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Error messages -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

// Command line errors
const char c_CleUnknownOption[]			= "Unknown command-line option (-%s) encountered";
const char c_CleMissingArgument[]		= "Missing argument for command-line option: -%s";
const char c_CleNoArgumentExpected[]	= "-%s command-line option does not have a argument";
const char c_CleInvalidArgument[]		= "Invalid argument for command line option: -%s";
const char c_CleNoRecFile[]			= "No record file specified";
const char c_CleTxRateNotDefined[]		= "Transport-Stream rate not specified";

// General errors
const char c_ErrDtapiHwScanFailed[]		= "DtapiHwFuncScan failed (ERROR: %s)";
const char c_ErrNoInputFound[]			= "No input device in the system";
const char c_ErrNoDta1xxFound[]			= "No DTA-%d with input port %d found";
const char c_ErrNoDtu2xxFound[]			= "No DTU-%d with input port %d found";
const char c_ErrCouldNotFindInputN[]	= "Could not find input board (#%d, port=%d) in the system";
const char c_ErrCouldNotFindDta1xxN[]	= "Could not find DTA-%d (#%d, port=%d) in the system";
const char c_ErrCouldNotFindDtu2xxN[]	= "Could not find DTU-%d (#%d, port=%d) in the system";
const char c_ErrFailToAttachToDta1xx[]	= "Failed to attach to the DTA-%d on Bus: %d and Slot: %d";
const char c_ErrFailToAttachToDtu2xx[]	= "Failed to attach to the DTU-%d";
const char c_ErrFailToAttachToChan[]	= "Can't attach to the channel (ERROR: %s)";
const char c_ErrFailToOpenFile[]		= "Can't open '%s' for writing";
const char c_ErrWriteFile[]				= "File write error";
const char c_ErrFailSetRxControl[]		= "SetRxControl failed (ERROR: %s)";
const char c_ErrFailSetRxMode[]			= "SetRxMode failed (ERROR: %s)";
const char c_ErrFailRead[]				= "Read failed (ERROR: %s)";


const char c_ErrDriverInCompPci[] =
	"The current Dta1xx driver (V%d.%d.%d %d) is not compatible with this\n" \
	"version of DtRecord.\n" \
	"Please install the latest version of the Dta1xx driver.";

const char c_ErrDriverInCompUsb[] =
	"The current Dtu2xx driver (V%d.%d.%d %d) is not compatible with this\n" \
	"version of DtRecord.\n" \
	"Please install the latest version of the Dtu2xx driver.";


//+=+=+=+=+=+=+=+=+=+=+=+=+=+ CommandLineParams implementation +=+=+=+=+=+=+=+=+=+=+=+=+=+

//-.-.-.-.-.-.-.-.-.-.-.-.- CommandLineParams::CommandLineParams -.-.-.-.-.-.-.-.-.-.-.-.-
//
CommandLineParams::CommandLineParams() :
	m_pLastFlag(NULL),
	m_LastFlagReqsArg(false),
	m_Error(false)
{
}

//.-.-.-.-.-.-.-.-.-.-.-.- CommandLineParams::~CommandLineParams -.-.-.-.-.-.-.-.-.-.-.-.-
//
CommandLineParams::~CommandLineParams()
{
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- CommandLineParams::Init -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
void CommandLineParams::Init()
{
	m_FileName[0] = '\0';

	m_DvcType = -1;
	m_DvcNum = 1;
	m_Port = 1;

	m_RxMode = -1;
	m_MaxSize = -1;

	m_SilentMode = false;
	m_ShowHelp = false;
}

//-.-.-.-.-.-.-.-.-.-.-.-.- CommandLineParams::ParseCommandLine -.-.-.-.-.-.-.-.-.-.-.-.-.
//
void CommandLineParams::ParseCommandLine(int argc, char* argv[])
{
	bool First = true;
	for ( int i=1; i<argc; i++ )
	{
		char* pParam = argv[i];
		bool Flag = false;
		bool Last = ( (i+1)==argc );
#ifdef WIN32
		if ( pParam[0]=='-' || pParam[0] == '/' )
#else
        if ( pParam[0]=='-' ) // For Linux only '-' can be used for options
#endif
		{
			// remove flag specifier
			Flag = true;
			++pParam;
		}
		ParseParam(pParam, Flag, First, Last);
		First = false;
	}

	// Check for required parameters
	if ( strlen(m_FileName) == 0 && !m_ShowHelp )
		throw Exc(c_CleNoRecFile);

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Set dynamic option default -.-.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	// Set default settings for parameters which are not defined on the command line

	// RxMode defined?
	if ( m_RxMode == -1  )
		m_RxMode = DTAPI_RXMODE_ST188;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- CommandLineParams::ParseParam -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
void CommandLineParams::ParseParam(
	char* pParam,
	bool Flag,
	bool First,
	bool Last)
{

	if ( Flag )
		ParseParamFlag(pParam, First, Last);
	else
		ParseParamNotFlag(pParam, First, Last);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.- CommandLineParams::ParseParamFlag -.-.-.-.-.-.-.-.-.-.-.-.-.-
//
void  CommandLineParams::ParseParamFlag(char* pParam, bool First, bool Last)
{
	// Did we expect an option??
	if ( m_LastFlagReqsArg && m_pLastFlag != NULL )
		throw Exc(c_CleMissingArgument, m_pLastFlag);

	if ( 0==strcmp(pParam, c_DvcTypeCmdLineFlag) )
		m_LastFlagReqsArg = true;
	else if ( 0==strcmp(pParam, c_DvcNumCmdLineFlag) )
		m_LastFlagReqsArg = true;
	else if ( 0==strcmp(pParam, c_PortCmdLineFlag) )
		m_LastFlagReqsArg = true;
	else if ( 0==strcmp(pParam, c_RxModeCmdLineFlag) )
		m_LastFlagReqsArg = true;
	else if ( 0==strcmp(pParam, c_MaxSizeCmdLineFlag) )
		m_LastFlagReqsArg = true;
	else if ( 0==strcmp(pParam, c_SilentModeCmdLineFlag))
	{
		m_SilentMode = true;
		m_LastFlagReqsArg = false;
	}
	else if ( 0==strcmp(pParam, c_HelpCmdLineFlag))
	{
		m_ShowHelp = true;
		m_LastFlagReqsArg = false;
	}
	else
		throw Exc(c_CleUnknownOption, pParam);

	m_pLastFlag = pParam;
}

//-.-.-.-.-.-.-.-.-.-.-.-.- CommandLineParams::ParseParamNotFlag -.-.-.-.-.-.-.-.-.-.-.-.-
//
void  CommandLineParams::ParseParamNotFlag(char* pParam, bool First, bool Last)
{
	// First option should be our file name
	if ( First )
	{
		strcpy(m_FileName, pParam);
		m_pLastFlag = NULL;
		m_LastFlagReqsArg = false;
		return;
	}

	// Did we expect and option??
	if ( m_pLastFlag==NULL )
		return;
	if ( !m_LastFlagReqsArg )
		throw Exc(c_CleNoArgumentExpected, m_pLastFlag);


	if ( 0==strcmp(m_pLastFlag, c_DvcTypeCmdLineFlag) )
	{
		m_DvcType = atoi(pParam);
		if ( m_DvcType < 100 )
			throw Exc(c_CleInvalidArgument, m_pLastFlag);
	}
	else if ( 0==strcmp(m_pLastFlag, c_DvcNumCmdLineFlag) )
	{
		m_DvcNum = atoi(pParam);
		if ( m_DvcNum < 1 )
			throw Exc(c_CleInvalidArgument, m_pLastFlag);
	}
	else if ( 0==strcmp(m_pLastFlag, c_PortCmdLineFlag) )
	{
		m_Port = atoi(pParam);
		if ( m_Port < 1 )
			throw Exc(c_CleInvalidArgument, m_pLastFlag);
	}
	else if ( 0==strcmp(m_pLastFlag, c_RxModeCmdLineFlag) )
	{
		// Which rx-mode
		if ( 0 == strcmp(pParam, "ST188")  )
			m_RxMode = DTAPI_RXMODE_ST188;
		else if ( 0 == strcmp(pParam, "ST204")  )
			m_RxMode = DTAPI_RXMODE_ST204;
		else if ( 0 == strcmp(pParam, "STRAW")  )
			m_RxMode = DTAPI_RXMODE_STRAW;
		else
			throw Exc(c_CleInvalidArgument, m_pLastFlag);
	}
	else if ( 0==strcmp(m_pLastFlag, c_MaxSizeCmdLineFlag) )
	{
		m_MaxSize = atoi(pParam);
		if ( m_MaxSize < 0 )
			throw Exc(c_CleInvalidArgument, m_pLastFlag);
	}
	else
		throw Exc(c_CleUnknownOption, m_pLastFlag);

	// Clear required argument flag
	m_LastFlagReqsArg = false;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- CommandLineParams::RxMode2Str -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
char* CommandLineParams::RxMode2Str() const
{

	switch( m_RxMode )
	{
	case DTAPI_RXMODE_ST188:	return "ST188";
	case DTAPI_RXMODE_ST204:	return "ST204";
	case DTAPI_RXMODE_STRAW:	return "STRAW";
	default:					return "?";
	}
}

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Recorder implementation +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Recorder::Recorder -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
Recorder::Recorder() : m_pFile(NULL), m_pBuf(NULL)
{
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Recorder::~Recorder -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
Recorder::~Recorder()
{
	// Detach from channel and device
	m_DtInp.SetRxControl(DTAPI_RXCTRL_IDLE);
	m_DtInp.Detach(DTAPI_INSTANT_DETACH);
	m_DtDvc.Detach();

	// Free our buffer
	if ( m_pBuf != NULL )
		delete [] m_pBuf;

	// donot forget to close our file
	if ( m_pFile != NULL )
		::fclose(m_pFile);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Recorder::AttachToInput -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
void Recorder::AttachToInput()
{
	DTAPI_RESULT dr;
	int n, NumHwFuncsFound(0);
	DtHwFuncDesc HwFuncs[20];

	// Scan for available DekTec devices
	dr = ::DtapiHwFuncScan(sizeof(HwFuncs)/sizeof(HwFuncs[0]), NumHwFuncsFound, HwFuncs);
	if ( dr != DTAPI_OK )
		throw Exc(c_ErrDtapiHwScanFailed, ::DtapiResult2Str(dr) );

	// Loop through found hardware functions
	bool Found(false);
	int DeviceNum(0);
	DtHwFuncDesc* pHwf = &HwFuncs[0];
	for ( n=0; n<NumHwFuncsFound; n++, pHwf++ )
	{
		// Skip non-inputs
		if ( 0==(pHwf->m_ChanType & DTAPI_CHAN_INPUT) )
			continue;

		// Looking for a specific type??
		if (    pHwf->m_DvcDesc.m_TypeNumber == m_CmdLineParams.m_DvcType
		     && pHwf->m_Port==m_CmdLineParams.m_Port )
			DeviceNum++;	// only count devices matching our search type
		else if ( m_CmdLineParams.m_DvcType == -1 )
			DeviceNum++;	// count every device

		if ( DeviceNum==m_CmdLineParams.m_DvcNum )
		{
			Found = true;
			break;
		}
	}
	// Did we find our device?
	if ( !Found && DeviceNum==0 )
	{
		// Could not find any matching input at all
		if (   (m_CmdLineParams.m_DvcType>=100 && m_CmdLineParams.m_DvcType<200)
			|| (m_CmdLineParams.m_DvcType>=2100 && m_CmdLineParams.m_DvcType<2200) )
			throw Exc(c_ErrNoDta1xxFound, m_CmdLineParams.m_DvcType,
				  m_CmdLineParams.m_Port );
		else if (   (m_CmdLineParams.m_DvcType>=200 && m_CmdLineParams.m_DvcType<300) )
			throw Exc(c_ErrNoDtu2xxFound, m_CmdLineParams.m_DvcType,
				  m_CmdLineParams.m_Port );
		else
			throw Exc(c_ErrNoInputFound);
	}
	else if ( !Found && m_CmdLineParams.m_DvcNum > DeviceNum )
	{
		// Could no find the Nth matching input
		if (   (m_CmdLineParams.m_DvcType>=100 && m_CmdLineParams.m_DvcType<200)
			|| (m_CmdLineParams.m_DvcType>=2100 && m_CmdLineParams.m_DvcType<2200) )
			throw Exc(c_ErrCouldNotFindDta1xxN, m_CmdLineParams.m_DvcType,
					  m_CmdLineParams.m_DvcNum, m_CmdLineParams.m_Port);
		else if (   (m_CmdLineParams.m_DvcType>=200 && m_CmdLineParams.m_DvcType<300) )
			throw Exc(c_ErrCouldNotFindDtu2xxN, m_CmdLineParams.m_DvcType,
					  m_CmdLineParams.m_DvcNum, m_CmdLineParams.m_Port);
		else
			throw Exc(c_ErrCouldNotFindInputN, m_CmdLineParams.m_DvcNum,
					  m_CmdLineParams.m_Port);
	}

	// Attach to the device
	dr = m_DtDvc.AttachToSerial(pHwf->m_DvcDesc.m_Serial);
	if ( dr == DTAPI_E_DRIVER_INCOMP )
	{
		// Special case: driver version is not compatible with this version of DtRecord,
		// get the driver version and throw an exception
		int DriverVersionMajor(-1), DriverVersionMinor(-1), DriverVersionBugFix(-1),
			DriverVersionBuild(-1);

		// Get device driver version
		::DtapiGetDeviceDriverVersion(pHwf->m_DvcDesc.m_Category,
									  DriverVersionMajor, DriverVersionMinor,
									  DriverVersionBugFix, DriverVersionBuild);

		// PCI or USB device??
		if ( pHwf->m_DvcDesc.m_Category == DTAPI_CAT_PCI )
		{
			throw Exc( c_ErrDriverInCompPci, DriverVersionMajor, DriverVersionMinor,
					   DriverVersionBugFix, DriverVersionBuild );
		}
		else if ( pHwf->m_DvcDesc.m_Category == DTAPI_CAT_USB )
		{
			throw Exc( c_ErrDriverInCompUsb, DriverVersionMajor, DriverVersionMinor,
					   DriverVersionBugFix, DriverVersionBuild );
		}
	}
	else if ( dr != DTAPI_OK )
	{
		// PCI or USB device??
		if ( pHwf->m_DvcDesc.m_Category == DTAPI_CAT_PCI )
			throw Exc(c_ErrFailToAttachToDta1xx, pHwf->m_DvcDesc.m_TypeNumber,
					  pHwf->m_DvcDesc.m_PciBusNumber, pHwf->m_DvcDesc.m_SlotNumber);
		else
			throw Exc(c_ErrFailToAttachToDtu2xx, pHwf->m_DvcDesc.m_TypeNumber);
	}

	// Attach to the output channel
	dr = m_DtInp.AttachToPort(&m_DtDvc, pHwf->m_Port);
	if ( dr != DTAPI_OK )
		throw Exc(c_ErrFailToAttachToChan, ::DtapiResult2Str(dr));
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Recorder::DisplayPlayInfo -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
void Recorder::DisplayPlayInfo()
{
	char sz[256];

	LogF("");
	LogF("Start recording");
	LogF("- Record file name      : %s", m_CmdLineParams.m_FileName);
	
	if ( m_CmdLineParams.m_MaxSize <= 0 )
		LogF("- Maximum file size     : No maximum set");
	else if ( m_CmdLineParams.m_MaxSize < 1024 )
		LogF("- Maximum file size     : %.1f MB", double(m_CmdLineParams.m_MaxSize));
	else
		LogF("- Maximum file size     : %.1f GB", double(m_CmdLineParams.m_MaxSize/1024.0) );
	
	LogF("- Receive Mode          : %s", m_CmdLineParams.RxMode2Str() );
	
	if ( m_DtDvc.Category() == DTAPI_CAT_USB )
		sprintf(sz, "DTU-%d", m_DtDvc.TypeNumber() );
	else /*if ( m_DtDvc.Category() == DTAPI_CAT_PCI )*/
		sprintf(sz, "DTA-%d", m_DtDvc.TypeNumber() );
	LogF("- Input device          : %s port %d (#%d)", sz, m_CmdLineParams.m_Port, 
		 m_CmdLineParams.m_DvcNum);

#ifdef WIN32
	Log("");
	Log("Press any key to stop recording");
	Log("");
#endif
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Recorder::InitInput -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
void Recorder::InitInput()
{
	DTAPI_RESULT dr;

	// Init channel to initial 'safe' state
	dr = m_DtInp.SetRxControl(DTAPI_RXCTRL_IDLE);	// Start in IDLE mode
	if ( dr != DTAPI_OK )
		throw Exc( c_ErrFailSetRxControl, ::DtapiResult2Str(dr) );
	dr = m_DtInp.ClearFifo();			// Clear FIFO (i.e. start with zero load)

	// Set the receive mode. 
	dr = m_DtInp.SetRxMode( m_CmdLineParams.m_RxMode );
	if ( dr != DTAPI_OK )
		throw Exc( c_ErrFailSetRxMode, ::DtapiResult2Str(dr) );
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Recorder::Log -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
void Recorder::Log(const char* pMessage,  bool IgnoreSilence)
{
    // Check for silent mode
	if ( !IgnoreSilence && m_CmdLineParams.m_SilentMode )
		return;
	// Print message and add new line
	printf(pMessage); printf("\n");
}
// Overload with optional parameters
void Recorder::LogF(const char* pMessage, ... )
{
	char sz[512];

	va_list  ArgList;
	va_start(ArgList, pMessage);
	_vsnprintf(sz, sizeof(sz)-1, pMessage, ArgList);
	va_end(ArgList);

	Log(sz, false);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Recorder::RecordFile -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
void Recorder::RecordFile()
{
	DTAPI_RESULT dr;
	int NumBytesRead, FifoLoad, NumToRead;
	__int64 NumBytesStored = 0;		// Number bytes stored in file
	
	// Start reception
	dr = m_DtInp.SetRxControl(DTAPI_RXCTRL_RCV);
	if ( dr != DTAPI_OK )
		throw Exc(c_ErrFailSetRxControl, ::DtapiResult2Str(dr));

	// Init allignment
	if ( m_CmdLineParams.m_RxMode==DTAPI_RXMODE_ST204 )
		m_Allign = 204;		// Allign on multiples of 204 bytes
	else if ( m_CmdLineParams.m_RxMode==DTAPI_RXMODE_STRAW )
		m_Allign = 4;		// Allign on multiples of 4 bytes
	else
		m_Allign = 188;		// Allign on multiples of 188 bytes


#ifdef WIN32
	while ( !_kbhit() ) {
#else
	while ( 1 ) {
#endif

		// Get FIFO load. Continue if too few bytes available in receive FIFO
		dr = m_DtInp.GetFifoLoad(FifoLoad);
		if ( FifoLoad < c_BufSize+512 )
		{
			// Sleep 1ms, before checking load again
		#ifdef WIN32
			Sleep(1);
		#else
			usleep(1000);
		#endif 
			continue;
		}

		// Read alligned number of data from input channel
		NumToRead = (c_BufSize / m_Allign) * m_Allign;
		dr = m_DtInp.Read( m_pBuf, NumToRead );
		if ( dr != DTAPI_OK )
			throw Exc(c_ErrFailRead, ::DtapiResult2Str(dr) );

		// Compute number of bytes left to store
		__int64 NumLeftToStore;
		if ( m_CmdLineParams.m_MaxSize <=0 )
			NumLeftToStore = NumToRead+1;	// No max => we can store all
		else
			NumLeftToStore = (m_CmdLineParams.m_MaxSize * 1024 * 1024) - NumBytesStored;
		// Account our allignment requirements
		NumLeftToStore = ( NumLeftToStore/m_Allign ) * (__int64)(m_Allign+1);

		// Compute number of bytes to write to file
		int NumToWrite = ( NumLeftToStore >= NumToRead ) ? NumToRead : (int)NumLeftToStore;
				
		// Write data to our file
		if ( NumToWrite != ::fwrite(m_pBuf, sizeof(char), NumToWrite, m_pFile) )
			throw Exc(c_ErrWriteFile);

		// Update number stored
		NumBytesStored += NumToWrite;

		// Should we stop?
		NumLeftToStore -= NumToWrite;
		if ( NumLeftToStore <= 0 )
			break;	// Reached max size => exit record loop
    }
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Recorder::Record -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
int Recorder::Record(int argc, char* argv[])
{
	int RetValue=0;
	try
	{
		try
		{
			// Parse command line options
			m_CmdLineParams.Init();
			m_CmdLineParams.ParseCommandLine(argc, argv);
		}
		catch ( Exc e ) {
			Log(e, true);
			// Force show help on exception
			m_CmdLineParams.m_ShowHelp = true;

			RetValue = -1;
		}
		catch ( ... ) {
			// Force show help on exception
			m_CmdLineParams.m_ShowHelp = true;

			RetValue = -1;
		}
		if ( m_CmdLineParams.m_ShowHelp )
		{
			// Disable silent mode
			m_CmdLineParams.m_SilentMode = false;
			ShowHelp();
			return RetValue;
		}

		//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Print start message -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

		LogF("DtRecord - Recording Utility V%d.%d.%d (c) 2000-2007 DekTec Digital Video B.V.\n",
			 DTRECORD_VERSION_MAJOR, DTRECORD_VERSION_MINOR, DTRECORD_VERSION_BUGFIX );


		//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Open the record file -.-.-.-.-.-.-.-.-.-.-.-.-.-.-

		// Try to open the file for writing
		m_pFile = fopen( m_CmdLineParams.m_FileName, "wb");
		if ( m_pFile == NULL )
			throw Exc(c_ErrFailToOpenFile, m_CmdLineParams.m_FileName);

		//-.-.-.-.-.-.-.-.-.-.-.-.- Attach to the output channel -.-.-.-.-.-.-.-.-.-.-.-.-

		AttachToInput();

		//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Final initialisation -.-.-.-.-.-.-.-.-.-.-.-.-.-.-

		// Init output
		InitInput();

		// Create our transfer buffer
		m_pBuf = new char [c_BufSize];

		//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Start playing -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

		// Print start info
		DisplayPlayInfo();
		// Loop file
		RecordFile();
	}
	catch( Exc e )
	{
        Log(e, true);
		RetValue = -1;
	}

	// Detach from channel and device
	m_DtInp.SetRxControl(DTAPI_RXCTRL_IDLE);
	m_DtInp.Detach(DTAPI_INSTANT_DETACH);
	m_DtDvc.Detach();

	// Free our buffer
	if ( m_pBuf != NULL ) {
		delete [] m_pBuf;
		m_pBuf = NULL;
	}

	// donot forget to close our file
	if ( m_pFile != NULL ) {
		::fclose(m_pFile);
		m_pFile = NULL;
	}

	return RetValue;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Recorder::ShowHelp -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
void Recorder::ShowHelp()
{
	Log( "" );
	Log( "Usage:");
	Log( "   DtRecord recfile [-t type] [-n number] [-i port] [-x maxsize] [-s] [-?]");
	Log( "");
	Log("Where:");
	Log("   recfile is the name of the record file");
	Log( "");
	Log("Options:");
	Log("   -t  Device type to use (default: any input device)");
	Log("         Use: 120  (for DTA-120)");
	Log("              122  (for DTA-122)");
	Log("              124  (for DTA-124)");
	Log("              140  (for DTA-140)");
	Log("              145  (for DTA-145)");
	Log("              160  (for DTA-160)");
	Log("              2145 (for DTA-2145)");
	Log("              225  (for DTU-225)");
	Log("              245  (for DTU-245)");
	Log("              545  (for DTA-545)");
	Log("   -n  Device number to use (default: 1)");
	Log("   -i  Port number of the input channel to use (default: 1) ");
	Log("   -m  Receive Mode (default: ST188)");
	Log("         Use: ST188  Store packets as 188-byte packets");
	Log("              ST204  Store packets as 204-byte packets");
	Log("              STRAW  No notion of packets. Store all valid bytes");
	Log("   -s  Silent mode. No messages printed");
	Log("   -?  Display this help");
	Log("");
	Log("NOTE: The first option can only be the recfile or the help option");
	Log("");
	Log("Examples:");
	Log("   DtRecord myfile.ts -x 10");
	Log("   DtRecord myfile.ts -t 124 -i 2 -m ST204 ");
	Log("");
	LogF("DtRecord version: %d.%d.%d\n", DTRECORD_VERSION_MAJOR, DTRECORD_VERSION_MINOR,
		 DTRECORD_VERSION_BUGFIX);
}

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Application entry point +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- main -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
int main(int argc, char* argv[])
{
	int RetValue(0);
	Recorder TheRecorder;
	try
	{
		RetValue = TheRecorder.Record(argc, argv);
	}
	catch(...)
	{
		RetValue = -1;
	}
	return RetValue;
}
