//*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* Dta1xx.h *#*#*#*#*#*#*#*#*# (C) 2000-2005 DEKTEC

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- License -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// This program is free software; You can freely use/modify this source code in any way
// you desire, including for commercial applications.
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	TD	2007.01.29	Addes support for DTA-160 ASI-only
//	TD	2007.01.24	Addes support for DTA-145/2145
//	TD	2007.01.23	Device reference count added
//	KvR	14-08-2006	DTA-105 registers changed, Vpd-data handling changed
//	JH	22-07-2005	Added support for DTA-110 and DTA-105/545 (non-PLX)
//	MG	26-10-2004	Added support for DTA-107 and initial support for DTA-124
//	MG	30-08-2004	Dta1xxIsr now returns an unsigned long
//	JH	16-08-2004	Added #include <linux/interrupt.h>
//	MG	23-04-2003	Added VPD support
//	MG	11-03-2003	Added DMA done event to DMA_CHANNEL
//	MG	21-10-2002	Created

#ifndef __DTA1XX_H
#define __DTA1XX_H

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#ifdef __KERNEL__

#include <linux/slab.h>			// kmalloc ()
#include <linux/fs.h>			// inode, file
#include <linux/errno.h>		// error codes
#include <linux/pci.h>			// pci_dev
#include <linux/wait.h>			// wait queue
#include <linux/sched.h>		// wait queue
#include <linux/ioctl.h>
#include <linux/interrupt.h>	// request_irq(), free_irq()
#include <asm/semaphore.h>		// semaphore
#include <linux/pagemap.h>		// page_cache_release()

#endif

#include "DriverTypes.h"
#include "Plx905x.h"
#include "Dta1xxCodes.h"
#include "Dta1xxIoCtl.h"
#include "DriverVersion.h"

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- LOG LEVEL -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

// LOG LEVEL with KdPrint
//  0	Off
//	1	Minimal (setting-up / breaking-down of channels)
//	2	Average (up to 10 per second)
//	3	All

// DMA related
#define LOG_LEVEL_DMA				0
#define LOG_LEVEL_CDMA				0

// Other logs (0=off not supported)
#define LOG_LEVEL				0

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Type definitions and constants -.-.-.-.-.-.-.-.-.-.-.-.-.-.

// Module ID information
#define DTA1XX_MAJOR 		0				// Default major number
#define DRIVER_NAME			"Dta1xx"		// Name of the driver

#define DTA1XX_MAX_NMB_CARDS		10		// Maximum number of DTA-1xx cards
											// supported
// Maximum number of Tx+Rx channels used by any card
#define MAX_NUM_CHANNELS			4

// DTA-1XX (sub-)vendor and (sub-)device ID's
#define DTA1XX_VENDORID			0x14B4		// Old DEKTEC vendor ID
#define DTA1XX_VENDORID_DEKTEC	0x1A0E		// DEKTEC vendor ID

#define DTA100_DEVICEID		0xD100			// DTA-100 device ID
#define DTA102_DEVICEID		0xD102			// DTA-102 device ID
#define DTA105_DEVICEID		0x0069			// DTA-105 device ID
#define DTA107_DEVICEID		0xD107			// DTA-107 device ID
#define DTA110_DEVICEID		0xD10A			// DTA-110 device ID
#define DTA120_DEVICEID		0xD114			// DTA-120 device ID
#define DTA122_DEVICEID		0xD116			// DTA-122 device ID
#define DTA124_DEVICEID		0xD118			// DTA-124 device ID
#define DTA140_DEVICEID		0xD128			// DTA-140 device ID
#define DTA145_DEVICEID		0x0091			// DTA-145 device ID
#define DTA2145_DEVICEID	0x0861			// DTA-2145 device ID
#define DTA160_DEVICEID		0x00A0			// DTA-160 device ID
#define DTA545_DEVICEID		0xD2BD			// DTA-545 device ID

// Driver log message helpers
#define DTA1XX_LOG(l,m)					printk(l DRIVER_NAME ": " m "\n");
#define DTA1XX_LOG1(l,m,p1)				printk(l DRIVER_NAME ": " m "\n", p1);
#define DTA1XX_LOG2(l,m,p1,p2)			printk(l DRIVER_NAME ": " m "\n", p1, p2);
#define DTA1XX_LOG3(l,m,p1, p2,p3)		printk(l DRIVER_NAME ": " m "\n", p1, p2, p3);
#define DTA1XX_LOG4(l,m,p1,p2,p3,p4)	printk(l DRIVER_NAME ": " m "\n", p1, p2, p3, p4);
#define DTA1XX_LOG5(l,m,p1,p2,p3,p4,p5)	printk(l DRIVER_NAME ": " m "\n", p1, p2, p3, p4, p5);

// Number of periodic-interrupt periods used to measure input bit rate
#define NUM_RX_MEAS_PERIODS			250

// DMA direction
#define  DTA1XX_DMA_READ			0
#define  DTA1XX_DMA_WRITE			1

// LED flashing
#define  DTA1XX_LEDFLASH_INIT		0
#define  DTA1XX_LEDFLASH_ON			1
#define  DTA1XX_LEDFLASH_OFF		2
#define  DTA1XX_LEDFLASH_EXIT		3

// Target-detection states
#define  DTA1XX_TD_INITIAL			0
#define  DTA1XX_TD_FIND_TARGETID	1
#define  DTA1XX_TD_OPEN_CIRCUIT		2
#define  DTA1XX_TD_SPI_SINK			3
#define  DTA1XX_TD_TARGETID_STABLE	4
#define  DTA1XX_TD_TARGET_ERROR		5

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- VPD Constants -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

// Maximum length of VPD-resource string
#define  VPD_ITEM_MAX_LENGTH		63			// 64 if trailing '\0' is factored in

// Valid VPD characters
#define  VALID_VPD_KEYW_CHAR(c) 	(((c)>='0' && (c)<='9') || ((c)>='A' && (c)<='Z'))

// *** PLX-attached EEPROM

// EEPROM attached to PLX 9054/9056
#define  PLX_VPD_SIZE				512

// Start and end of VPD sections
#define  PLX_VPD_ID_START			0x58	// Read-only section directly follows VPD-ID
#define  PLX_VPD_RW_START			0x100
#define  PLX_VPD_END				(PLX_VPD_SIZE - 1)

// Section lengths
#define  PLX_VPD_ID_RO_LENGTH		(PLX_VPD_RW_START - PLX_VPD_ID_START)
#define  PLX_VPD_RW_LENGTH			(PLX_VPD_END - PLX_VPD_RW_START)

// Total length. The final +1 is for VPD End tag
#define  PLX_VPD_TOTAL_LENGTH		(PLX_VPD_ID_RO_LENGTH + PLX_VPD_RW_LENGTH + 1)

// DMA buffer size
#define DTA1XX_DMA_BUF_SIZE			(64*1024)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Data declaration -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Helper Structs +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- struct BIT_RATE_MEAS -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Structure with variables to measure a bit rate.
//
typedef struct _BIT_RATE_MEAS
{
	// Bit-Rate Measurement
	int  m_ValidDiff[NUM_RX_MEAS_PERIODS];
										// Number of Valid bytes received per period
										// (=difference between successive Valid counts)
	int  m_nLastIndex;					// Index in ValidCnt of last sample
	int  m_nLastValidCnt;				// Last absolute value of Valid counter
	int  m_nNumBytesTotal;				// Total number of valid bytes in measurement period
	int  m_nNumValidSamps;				// Number of "valid" samples in m_ValidCnt[]
	int  m_nValidCount256;				// #Valid bytes per interval, times 256
	int  m_nValidCountSample;			// Valid count sample; read in interrupt routine
} BIT_RATE_MEAS;

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- CdmaSegmDesc -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// CDMA segment descriptor.
// A CDMA buffer is divided into 4 segments 0..3, this structure describes one segment
//
typedef struct _CdmaSegmDesc
{
	UInt32  m_SglAddr;					// Scatter-Gather-List descriptor address
	Int  m_CdmaRelAddr;					// Relative segment address in CDMA buffer
										// SGLs may be multiple pages long, so segment
										// length may differ from exactly 1/4
} CdmaSegmDesc;

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Shared Buffer Descriptor -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Stores state for a buffer that is shared between user application and driver
//
typedef struct _ShBufDesc
{
	Int  m_Allocated;				// Shared buffer has been allocated

	UInt8*  m_pUserBuf;				// Virtual user address

	struct page** m_PageList;		// Buffer with page-list describing locked user buffer
	Int  m_NumPages;				// Number of pages

	Int  m_SglAllocated;			// Scatter-Gather List has been allocated
	UInt8* m_pSglBuf;				// Buffer with scatter-gather descriptors
	Int m_FirstDescOffset;
	Int m_SglBufSize;
	dma_addr_t m_PhysAddrSglBuf;
} ShBufDesc;

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- struct DMA_BUFFER -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// DMA buffer descriptor
//
typedef struct _DMA_BUFFER
{
	unsigned char*	m_pVirtStartAddr;	// Virtual kernel start address of the buffer
	dma_addr_t		m_PhysStartAddr;	// Physical start address of the buffer
} DMA_BUFFER;

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- struct DMA_CHANNEL -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Variables to describe and control a single DMA channel.
//
typedef struct _DMA_CHANNEL
{
	// Exclusive access
	struct semaphore m_Lock;			// Protects DMA channel

	// Register pointers
	unsigned char*	m_pRegBase;			// Base address of DMA registers
	unsigned char*	m_pRegDmaDesc;		// DMA Descriptor Pointer
	unsigned char*	m_pRegCmdStat;		// DMA Command/Status

	int				m_PlxChannel;		// PLX DMA channel number

	// Dma done event
	wait_queue_head_t
 					m_DmaDoneQueue;		// DMA done wait queue
	volatile int	m_nDmaDone;			// DMA done flag
	int				m_nDmaDir;			// Direction: DTA1XX_DMA_READ/DTA1XX_DMA_WRITE
	DMA_BUFFER		m_DmaBuffer;		// The DMA buffer

	// Scatter/gather list descriptor used for DMA transfer to the DTA-124
	SglDesc*		m_pSglDesc;			// Scatter/Gather descriptor
	dma_addr_t		m_PhysAddrSglDesc;	// Physical address of the scatter/gather list
										// descriptor
} DMA_CHANNEL;

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- struct MEM_DESC -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Memory-space descriptor, used for PCI-9054 and Dta1xx operational registers.
//
typedef struct _MEM_DESC {
    unsigned int	m_uiPciAddr;	// PCI-bus relative start address
	unsigned int	m_uiLength;		// Length of memory block
	volatile char*	m_pKernel;		// Address when mapped in kernel space.
} MEM_DESC;

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- struct TARGET_ID_DETECTION -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Target-ID Detection (for DTA-102/DTA-122)
//
typedef struct _TARGET_ID_DETECTION {
	int  m_nTargetId;				// Current best guess for target-adapter ID
	int  m_nPresent;				// Current best guess for target-adapter presence
	int  m_nTdState;				// Target-detection state
	int  m_nLastTargetId;			// Last measurement for target-adapter ID
	int  m_nNumTimesInFind;			// #times in state DTA1XX_TD_FIND_TARGETID
	int  m_nStableCount;			// Counter to assess whether measurement is stable
	int  m_nTargetError;			// Target Error, set in target-detection state
									// machine, cleared in Dta1xxTxProcessStatusFlags
} TARGET_ID_DETECTION;

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ CHANNEL +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=
//
// Describes a single channel, which is either:
//	- Transport-Stream Receive Channel, or
//	- Transport-Stream Transmit Channel
//
typedef struct _CHANNEL
{
	// Identity information: Reference back to device object and index
	struct _DTA1XX_FDO*	m_pFdo;

	int				m_nChanType;		// DTA1XX_CHANNEL_UNUSED/DTA1XX_TS_TX_CHANNEL/...
	int				m_nChanIdx;			// Channel index in Device Extension array
	int				m_nFirmwareVersion;	// Copy of Firmware Version
	int				m_nTypeNumber;		// Copy of type number, for card-dependent actions

	// General
	volatile char*  m_pRegBase;			// Base address of channel registers

	// DMA
	DMA_CHANNEL		m_Dma;				// DMA channel for reading data from channel

	//.-.-.-.-.-.-.-.-.-.-.-.- Members for Continuous DMA (CDMA) -.-.-.-.-.-.-.-.-.-.-.-.-
	// CDMA buffer is divided into 4 segments 0..3
	int  m_CdmaBufSize;					// CMDA buffer size in #bytes
	int  m_CdmaDmaRunning;				// CDMA running
	UInt32  m_CdmaRefClkSamp;			// Reference-clock sample @ last segment interrupt
	volatile int  m_CdmaSegment;		// Segment# in CDMA buffer currently being DMA-ed
	CdmaSegmDesc  m_CdmaSegmDesc[4];	// Segment descriptors
	ShBufDesc  m_CdmaShBuf;				// Shared CDMA-buffer descriptor
	int m_CdmaDone;
	wait_queue_head_t
 					m_CdmaEvent;		// CDMA event queue

	// Status Flags: Updated every periodic interrupt
	int  m_nStatus;						// Actual value of status flags
	int  m_nLatched;					// Latched value of status flags

	// Bit rate measurement
	BIT_RATE_MEAS	m_BrMeas;			// Bit-rate measurement
	TARGET_ID_DETECTION m_TrgIdDet;		// For DTA-102/122 target-ID detection

	//-.-.-.-.-.-.-.-.-.-.- Members Specific for TS Receive Channel -.-.-.-.-.-.-.-.-.-.-.

	// Statistics
	int  m_nViolCountSamp;				// Sample of code-violation counter (DTA-120)

	//-.-.-.-.-.-.-.-.-.-.- Members Specific for TS Transmit Channel -.-.-.-.-.-.-.-.-.-.-

	// Cached settings
	//int	m_TsRate;						// The current TsRate
    //int m_ModSampRate;					// Current modulation sample rate

} CHANNEL;

/*

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ TS_RX_CHANNEL +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=
//
// Describes a single Transport-Stream Receive Channel
//
typedef struct _TS_RX_CHANNEL {

	// General
	volatile char*  m_pRxRegs;			// Pointer to Receive register block
	int  m_nTypeNumber;					// Copy of type number, for card-dependent actions

	// Status Flags: Updated every periodic interrupt
	int  m_nStatus;						// Actual value of status flags
	int  m_nLatched;					// Latched value of status flags

	// Statistics
	int  m_nViolCountSamp;				// Sample of code-violation counter (DTA-120)

	// Sub-structures
	BIT_RATE_MEAS	m_BrMeas;			// Bit-rate measurement (TS input rate)
	DMA_CHANNEL		m_Dma;				// DMA channel for reading data from channel
	TARGET_ID_DETECTION	m_TrgIdDet;		// For DTA-122 target-ID detection

} TS_RX_CHANNEL;


//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ TS_TX_CHANNEL +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=
//
// Describes a single Transport-Stream Transmit Channel
//
typedef struct _TS_TX_CHANNEL {

	// General
	volatile char*  m_pTxRegs;			// Pointer to Transmit register block
	int  m_nFirmwareVersion;			// Copy of Firmware Version
	int  m_nTypeNumber;					// Copy of type number, for card-dependent actions

	// Cached settings
	int	m_TsRate;						// The current TsRate

	// Status Flags: Updated every periodic interrupt
	int  m_nStatus;						// Actual value of status flags
	int  m_nLatched;					// Latched value of status flags

	// Sub-structures
	BIT_RATE_MEAS	m_BrMeas;			// Bit-rate measurement (DTA-102: external clock)
	DMA_CHANNEL		m_Dma;				// DMA channel for writing data to channel
	TARGET_ID_DETECTION m_TrgIdDet;		// For DTA-102 target-ID detection

} TS_TX_CHANNEL;

*/

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ struct DTA1XX_FDO +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
//
typedef struct _DTA1XX_FDO {

	struct pci_dev*	m_pPdo;			// The physical device object (PCI card info)

	int			m_nTypeNumber;		// DTA-1xx type number (e.g. 100, 102, 120 or 122)
	int			m_nFirmwareVersion;	// Firmware revision number

	// Hardware characteristics
	int			m_nUsesGenReg;		// Uses General-Registers block yes/no
	int			m_nUsesDmaInFpga;	// Uses DMA Controller in Altera, not in PLX
	int			m_nUsesPlxChip;		// Uses PLX bridge chip yes/no
	int			m_nUses9054;		// Uses PCI9054 yes/no
	int			m_nUses9056;		// Uses PCI9056 yes/no

	// Channel-specific data
    int			m_nNumIpRxChannels;	// #IP receive channels (for error checking)
	int			m_nNumRxChannels;	// #TS receive channels (for error checking)
	int			m_nNumTxChannels;	// #TS-Transmit channels (for error checking)
	int			m_nNumChannels;		// Total #channels
	CHANNEL  m_Channel[MAX_NUM_CHANNELS];	// Channel data
	CHANNEL* m_IpRx[MAX_NUM_CHANNELS];		// Pointer to IP-Receive channels
	CHANNEL* m_TsRx[MAX_NUM_CHANNELS];		// Pointer to TS-Receive channels
	CHANNEL* m_TsTx[MAX_NUM_CHANNELS];		// Pointer to TS-Transmit channels


	// Pointers to DMA channels
	// DMA-channel data is stored in m_TsRx[] and m_TsTx[]. However, DMA channels are
	// also sequentially numbered. The following array provides a mapping from the
	// sequential DMA-channel number to the DMA descriptor structure.
//	int  m_nNumDmaChannels;				// #DMA channels
//	DMA_CHANNEL*  m_pDma[MAX_NUM_CHANNELS];

 	// Interrupt
	int  m_nIntAttached;				// Interrupt handler attached (yes/no)

	// Device reference count
	int  m_nDvcRefCount;			// Number of times the device is opened

	// LED Flashing after driver start-up
	// Dynamic
	int  m_nLedFlashFlashCounter;		// Counts number of LED flashes
	int  m_nLedFlashStartUp;			// Indicates LED flashing is active
	int  m_nLedFlashStateCounter;		// Counts number of interrupts
	int  m_nLedFlashState;				// State: DTA1XX_LEDFLASH_INIT/...
	// Static
	int  m_nLedFlashDurationExit;		// Duration of EXIT period (LED off)
	int  m_nLedFlashDurationInit;		// Duration of INIT period (LED off)
	int  m_nLedFlashDurationOff;		// Duration of LED-off period
	int  m_nLedFlashDurationOn;			// Duration of LED-on period
	int  m_nLedFlashNumFlashes;			// Number of times LED has to flashes

	// Memory Management
	MEM_DESC  m_Pci905XConfRegs;		// PCI 9054/9056 Local Configuration registers
	MEM_DESC  m_Dta1xxRegsBase;			// Base address of registers outside PCI 9054/9056
	volatile char*  m_pGenRegs;			// Pointer to General register block

    // EEPROM Caching for VPD managment
    // 1. PCI Cards that use the Altera flash for storing VPF
    //    (will probably become obsolete)
    int m_EepromAddrLastSector;     // Address of last sector in EEPROM
    int m_EepromRelAddrVpd;         // Address of VPD data in m_pEepromCache
    int m_EepromSectorSize;         // Size in bytes of a single EEPROM sector
    int m_EepromSize;               // Size in bytes of full EEPROM
	int	m_UseFpgaForVpd;			// If !=0 and m_UsesPlxChip == 0, FPGA is used to
									// store the VPD data.

	// 2. PCI cards with a small, VPD-specific EEPROM connected to FPGA
	Int  m_VpdEepromPageSize;		// Size of 1 page

	// 3. PCI cards with a small, VPD-specific EEPROM connected to PLX
	struct semaphore m_VpdEepromMutex;	// Mutex to protect access to VPD EEPROM

	// Common VPD data
    volatile char* m_pEepromCache;  // Cache for last EEPROM page; this is the page
                                    // containing the VPD
    volatile char* m_pVpdEeprom;    // Cache for VPD EEPROM
	Int  m_VpdIdOffset;				// Offset of start VPD ID
	Int  m_VpdEepromSize;			// Size of VPD area
} DTA1XX_FDO;

extern DTA1XX_FDO	g_Dta1xxCards[DTA1XX_MAX_NMB_CARDS];	// DTA-1xx card data
extern int			g_nNumOfCards;		// Number of DTA-1xx cards found in the system

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xx.c -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
int  Dta1xxInitModule(void);
void Dta1xxCleanUpModule(void);
int  Dta1xxStartCard(struct pci_dev* pPdo, DTA1XX_FDO* pFdo);
void Dta1xxStopCard(DTA1XX_FDO* pFdo);
int  Dta1xxOpen(struct inode *inode, struct file *filp);
int  Dta1xxRelease(struct inode *inode, struct file *filp);

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Init.c -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
int  Dta1xxInitDevice(DTA1XX_FDO* pFdo);
int  Dta1xxInitDeviceHardware(DTA1XX_FDO* pFdo);
int  Dta1xxInitPlx(DTA1XX_FDO* pFdo);
int  Dta1xxInitOpRegs(DTA1XX_FDO* pFdo);
int  Dta1xxCheckFirmwareVersion(DTA1XX_FDO* pFdo);

int  Dta1xxGetIoConfig(DTA1XX_FDO* pFdo, int nChIdx, int* pIoConfig);
int  Dta1xxSetIoConfig(DTA1XX_FDO* pFdo, int nChIdx, int nIoConfig);
//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- IoCtl.c -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
int  Dta1xxIoCtl(struct inode *inode, struct file *filp,unsigned int cmd,
				 unsigned long arg);
DTA1XX_RESULT  Dta1xxGetRefenceClockCount(DTA1XX_FDO* pFdo, unsigned int* pRefClkCnt);
DTA1XX_RESULT  Dta1xxIoCtlResetSoft(DTA1XX_FDO* pFdo);
DTA1XX_RESULT  Dta1xxIoCtlLedControl(DTA1XX_FDO* pFdo, int nLedControl);
DTA1XX_RESULT  Dta1xxIoCtlReadVpd(DTA1XX_FDO* pFdo, int nStart, int nCount,
								  unsigned char* pData);
DTA1XX_RESULT  Dta1xxIoCtlWriteVpd(DTA1XX_FDO* pFdo, int nStart, int nCount,
								  unsigned char* pData);

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- RxIoCtl.c -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
DTA1XX_RESULT  Dta1xxRxIoCtlReset(DTA1XX_FDO* pFdo, int nChIdx, int nResetMode);
DTA1XX_RESULT  Dta1xxRxIoCtlGetStatus(DTA1XX_FDO* pFdo, int nChIdx, int* pPacketSize,
									  int* pNumInv, int* pClkDet, int* pAsiLock,
									  int* pRateOk, int* pAsiInv);
DTA1XX_RESULT  Dta1xxRxIoCtlGetFlags(DTA1XX_FDO* pFdo, int nChIdx, int* pStatus,
									 int* pLatched);
DTA1XX_RESULT  Dta1xxRxIoCtlClearFlags(DTA1XX_FDO* pFdo, int nChIdx, int nLatched);
DTA1XX_RESULT  Dta1xxRxIoCtlGetTsRateBps(DTA1XX_FDO* pFdo, int nChIdx, int* pTsRate);
DTA1XX_RESULT  Dta1xxRxIoCtlGetFifoLoad(DTA1XX_FDO* pFdo, int nChIdx, int* pFifoLoad);
DTA1XX_RESULT  Dta1xxRxIoCtlSetPower(DTA1XX_FDO* pFdo, int nChIdx, int nPower);
DTA1XX_RESULT  Dta1xxRxIoCtlGetTargetId(DTA1XX_FDO* pFdo, int nChIdx, int* pPresent,
							            int* pTargetId);
DTA1XX_RESULT  Dta1xxRxIoCtlSetRxControl(DTA1XX_FDO* pFdo, int nChIdx, int nRxControl);
DTA1XX_RESULT  Dta1xxRxIoCtlGetRxControl(DTA1XX_FDO* pFdo, int nChIdx, int* pRxControl);
DTA1XX_RESULT  Dta1xxRxIoCtlSetRxMode(DTA1XX_FDO* pFdo, int nChIdx, int nRxMode);
DTA1XX_RESULT  Dta1xxRxIoCtlEqualise(DTA1XX_FDO* pFdo, int nChIdx, int nEqualiserSetting);
DTA1XX_RESULT  Dta1xxRxIoCtlPolarityControl(DTA1XX_FDO* pFdo, int nChIdx, int nPolarity);
DTA1XX_RESULT  Dta1xxRxIoCtlSetLoopBackMode(DTA1XX_FDO* pFdo, int nChIdx, int nMode);
DTA1XX_RESULT  Dta1xxRxIoCtlGetStatistics(DTA1XX_FDO* pFdo, int nChIdx, int* pViolCount);
DTA1XX_RESULT  Dta1xxRxIoCtlDma(DTA1XX_FDO* pFdo, int nChIdx, char* pUserBuf,
							    int nBytesToTransfer);

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TxIoCtl.c -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
DTA1XX_RESULT  Dta1xxTxIoCtlReset(DTA1XX_FDO* pFdo, int nChIdx, int nResetMode);
DTA1XX_RESULT  Dta1xxTxIoCtlGetExtClkFreq(DTA1XX_FDO* pFdo, int nChIdx, int* pExtClkFreq);
DTA1XX_RESULT  Dta1xxTxIoCtlGetFlags(DTA1XX_FDO* pFdo, int nChIdx, int* pStatus,
									 int* pLatched);
DTA1XX_RESULT  Dta1xxTxIoCtlClearFlags(DTA1XX_FDO* pFdo, int nChIdx, int nLatched);
DTA1XX_RESULT  Dta1xxTxIoCtlSetTxClock(DTA1XX_FDO* pFdo, int nChIdx, int nClockGenMode,
									   int nPhaseIncr, int nSymbRateMult);
DTA1XX_RESULT  Dta1xxTxIoCtlSetTxControl(DTA1XX_FDO* pFdo, int nChIdx, int nTxControl);
DTA1XX_RESULT  Dta1xxTxIoCtlSetTxMode(DTA1XX_FDO* pFdo, int nChIdx, int nTxPacketMode,
									  int nTxStuffMode);
DTA1XX_RESULT  Dta1xxTxIoCtlSetPower(DTA1XX_FDO* pFdo, int nChIdx, int nPower);
DTA1XX_RESULT  Dta1xxTxIoCtlGetTargetId(DTA1XX_FDO* pFdo, int nChIdx, int* pPresent,
									    int* pTargetId);
DTA1XX_RESULT  Dta1xxTxIoCtlSetFifoSize(DTA1XX_FDO* pFdo, int nChIdx, int nFifoSize);
DTA1XX_RESULT  Dta1xxTxIoCtlSetFifoSizeMax(DTA1XX_FDO* pFdo, int nChIdx);
DTA1XX_RESULT  Dta1xxTxIoCtlGetFifoLoad(DTA1XX_FDO* pFdo, int nChIdx, int* pFifoLoad);
DTA1XX_RESULT  Dta1xxTxIoCtlGetMaxFifoSize(DTA1XX_FDO* pFdo, int nChIdx,
										   int* pMaxFifoSize);
DTA1XX_RESULT  Dta1xxTxIoCtlGetTxClock(DTA1XX_FDO* pFdo, int nChIdx, int* pClockGenMode,
									   int* pPhaseIncr, int* pSymbRateMult);
DTA1XX_RESULT  Dta1xxTxIoCtlGetTxControl(DTA1XX_FDO* pFdo, int nChIdx, int* pTxControl);
DTA1XX_RESULT  Dta1xxTxIoCtlGetTxMode(DTA1XX_FDO* pFdo, int nChIdx, int* pTxPacketMode,
									  int* pTxStuffMode);
DTA1XX_RESULT  Dta1xxTxIoCtlGetModControl(DTA1XX_FDO* pFdo, int nChIdx, int* pModType,
										  int* pCodeRate, int* pPar1, int* pPar2);
DTA1XX_RESULT  Dta1xxTxIoCtlGetRfControl(DTA1XX_FDO* pFdo, int nChIdx, int* pPllA, int* pPllB,
										 int* pLockStatus);
DTA1XX_RESULT  Dta1xxTxIoCtlReadLoopBackData(DTA1XX_FDO* pFdo, int nChIdx, char* pUserBuf,
								             int nBytesToRead);
DTA1XX_RESULT  Dta1xxTxIoCtlSetLoopBackMode(DTA1XX_FDO* pFdo, int nChIdx, int nMode);
DTA1XX_RESULT  Dta1xxTxIoCtlSetTxPolarity(DTA1XX_FDO* pFdo, int nChIdx, int nPolarity);
DTA1XX_RESULT  Dta1xxTxIoCtlSetModControl(DTA1XX_FDO* pFdo, int nChIdx, int nModType,
										  int nCodeRate, int nPar1, int nPar2);
DTA1XX_RESULT  Dta1xxTxIoCtlSetRfControl(DTA1XX_FDO* pFdo, int nChIdx, int nPllA, int nPllB);
DTA1XX_RESULT  Dta1xxTxIoCtlDma(DTA1XX_FDO* pFdo, int nChIdx, char* pUserBuf,
					            int nBytesToTransfer);

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Interrupt.c -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
unsigned long  Dta1xxIsr(int nIrq, void* pContext, struct pt_regs *pRegs);
void  Dta1xxDisableInterrupts(DTA1XX_FDO* pFdo);
void  Dta1xxEnableInterrupts(DTA1XX_FDO* pFdo);
void  Dta1xxPeriodicIntHandler(DTA1XX_FDO* pFdo);

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Rate.c -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
int  Dta1xxGetFreqEst(DTA1XX_FDO* pFdo, int	nChannelType, int nChIdx, int*  pFreqEst);

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dma.c -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
int  Dta1xxCdmaCtrl(DTA1XX_FDO* pFdo, DTA1XX_CDMA_CONTROL* pCdmaCtrl);
int  Dta1xxCdmaStart(CHANNEL* pCh);
int  Dta1xxCdmaStop(CHANNEL* pCh);
int  Dta1xxCdmaWaitSegment(CHANNEL* pCh, DTA1XX_CDMA_CONTROL* pCdmaCtrl);
int  Dta1xxCdmaGetBufPos(CHANNEL* pCh, DTA1XX_CDMA_CONTROL* pCdmaCtrl);
int  Dta1xxDmaTransfer(DTA1XX_FDO* pFdo, int nChIdx, int ChType, int nLocalDmaAddr,
					   int nDmaSize, int nDmaDir);
int  Dta1xxCdmaSglCreate(CHANNEL* pCh);
int  Dta1xxCdmaSglCleanUp(CHANNEL* pCh);

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- LowLevel.c -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
int  Dta1xxFlashLed(DTA1XX_FDO* pFdo, int  LedState);
int  Dta1xxTxReset(CHANNEL*  pCh, int  ResetMode);
int  Dta1xxRxReset(CHANNEL*  pCh, int  ResetMode);
int  Dta1xxReadRegister(DTA1XX_FDO* pFdo, Int Address, Int* pValue);
int  Dta1xxWriteRegister(DTA1XX_FDO* pFdo, Int Address, Int Value);

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- MapMemory.c -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
int  Dta1xxSharedBufAlloc(DTA1XX_FDO* pFdo, DTA1XX_SHAREDBUF_CTRL* pShBufCtrl);
int  Dta1xxSharedBufFree(DTA1XX_FDO* pFdo, DTA1XX_SHAREDBUF_CTRL* pShBufCtrl);
int  Dta1xxSharedBufRelease(CHANNEL* pCh, Int PortIndex, ShBufDesc* pShBufDesc,
							Int Purpose);

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Utility.c -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
int  Dta1xxDeviceId2TypeNumber(int nDeviceId);
int  Dta1xxTxMode2PacketSize(int nTxPacketMode);

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Target.c -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
void Dta1xxTargetDetectStateMachine(DTA1XX_FDO* pFdo, int nChIdx);

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Flags.c -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
void Dta1xxRxProcessStatusFlags(CHANNEL* pTsRx);
void Dta1xxTxProcessStatusFlags(CHANNEL* pTsTx);

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- EepromControl.c -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
int  Dta1xxEepromReadLastPage(DTA1XX_FDO*  pFdo);
int  Dta1xxEepromReadWord16(DTA1XX_FDO* pFdoExt, unsigned int nWordAddr,
								unsigned int* pData);
int  Dta1xxEepromSpiRead(DTA1XX_FDO* pFdo);
int  Dta1xxEepromSpiWrite(DTA1XX_FDO* pFdo);
int  Dta1xxEepromWriteLastPage(DTA1XX_FDO* pFdoExt);
int  Dta1xxSetProtectRegister(DTA1XX_FDO* pFdo);
int  Dta1xxSetProtectRegisterClear(DTA1XX_FDO* pFdo);
int  Dta1xxEepromWriteWord16(DTA1XX_FDO* pFdoExt, unsigned int nWordAddr,
								unsigned int nData);

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Vpd.c -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
int  Dta1xxVpdRead(DTA1XX_FDO* pFdo, unsigned char*, unsigned int, unsigned int);
int  Dta1xxVpdWrite(DTA1XX_FDO* pFdo, unsigned char*, unsigned int, unsigned int);
int  Dta1xxVpdReadFromCache(DTA1XX_FDO* pFdoExt, UInt8*, UInt, UInt);
int  Dta1xxVpdWrite(DTA1XX_FDO* pFdoExt, UInt8*, UInt, UInt);


#endif //__DTA1XX_H
