//#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* Dta1xxCodes.h *#*#*#*#*#*#*#*# (C) 2000-2005 DEKTEC

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- License -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// This program is free software; You can freely use/modify this source code in any way
// you desire, including for commercial applications.
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	TD	2007.01.31	Added definitions for the IoConfig 
//	MG	2003.06.30	Added definitions for reception mode store MP2 and for timestamping
//	MG	21-10-2002	Created

#ifndef __DTA1XXCODES_H
#define __DTA1XXCODES_H

typedef unsigned long DTA1XX_RESULT;

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Common control/status codes +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Channel Definitions -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTA1XX_GEN_REGS			-1		// Special value for GENERIC registers
#define  DTA1XX_TS_TX_CHANNEL		0		// Transport-Stream Transmit Channel
#define  DTA1XX_TS_RX_CHANNEL		1		// Transport-Stream Receive Channel
#define  DTA1XX_CHANNEL_UNUSED		99		// Channel not used

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Hardware-Function Types -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTA1XX_TS_OUTPUT		1
#define  DTA1XX_TS_INPUT		2

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- CDMA Control -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTA1XX_CDMA_SETBUF			1
#define  DTA1XX_CDMA_START			2
#define  DTA1XX_CDMA_STOP			3
#define  DTA1XX_CDMA_WAITSEGM		4
#define  DTA1XX_CDMA_GETBUFPOS		5

//.-.-.-.-.-.-.-.-.-.-.-.-.- Shared Buffer Allocation / Freeing -.-.-.-.-.-.-.-.-.-.-.-.-.
// Purpose
#define  DTA1XX_SHBUF_DIRIQ_CDMA	0		// DIRECT-IQ: CDMA buffer
#define  DTA1XX_SHBUF_IP_RWBUF		1		// IP: Read/Write buffer

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- LED Control -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTA1XX_LED_OFF			1
#define  DTA1XX_LED_GREEN		3
#define  DTA1XX_LED_RED			5
#define  DTA1XX_LED_YELLOW		7
#define  DTA1XX_LED_HARDWARE	0

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Loop-Back Mode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTA1XX_NO_LOOPBACK		0
#define  DTA1XX_LOOPBACK_MODE	1

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- I/O Configuration -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTA1XX_IOCONFIG_NOTSUP		0
#define  DTA1XX_IOCONFIG_INPUT		1
#define  DTA1XX_IOCONFIG_OUTPUT		2
#define  DTA1XX_IOCONFIG_DBLBUF		3
#define  DTA1XX_IOCONFIG_IP		4
#define  DTA1XX_IOCONFIG_FAILSAFE	5

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Modulation Parameters -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
// NOTE:
// Constants below are NOT necessarily written 1-to-1 into the modulation-type register
#define  DTA1XX_MOD_DVBS_QPSK		0		// Native DVB-S on DTA-107
#define  DTA1XX_MOD_BPSK			1
#define  DTA1XX_MOD_QAM4			3
#define  DTA1XX_MOD_QAM16			4
#define  DTA1XX_MOD_QAM32			5
#define  DTA1XX_MOD_QAM64			6
#define  DTA1XX_MOD_QAM128			7
#define  DTA1XX_MOD_QAM256			8
#define  DTA1XX_MOD_OFDM			9
#define  DTA1XX_MOD_VSB8			10
#define  DTA1XX_MOD_VSB16			11
#define  DTA1XX_MOD_ISDBT			12
#define  DTA1XX_MOD_SYM4B			14		// 4-bit symbols for DTA-107S2
#define  DTA1XX_MOD_IQDIRECT		15
#define  DTA1XX_MOD_DVBS_QPSK_SOFT	16		// Soft DVB-S for DTA-107S2
#define  DTA1XX_MOD_DVBS2_QPSK		32
#define  DTA1XX_MOD_DVBS2_8PSK		33

#define  DTA1XX_MOD_1_2				0		// Code rate 1/2
#define  DTA1XX_MOD_2_3				1		// Code rate 2/3
#define  DTA1XX_MOD_3_4				2		// Code rate 3/4
#define  DTA1XX_MOD_4_5				3		// Code rate 4/5
#define  DTA1XX_MOD_5_6				4		// Code rate 5/6
#define  DTA1XX_MOD_6_7				5		// Code rate 6/7
#define  DTA1XX_MOD_7_8				6		// Code rate 7/8
#define  DTA1XX_MOD_ROLLOFF_13		1		// Roll-off factor 13% LEGACY
#define  DTA1XX_MOD_J83_A			2		// J.83 annex A (DVB-C)
#define  DTA1XX_MOD_ROLLOFF_15		2		// Roll-off factor 15% LEGACY
#define  DTA1XX_MOD_J83_B			3		// J.83 annex B (American QAM)
#define  DTA1XX_MOD_J83_C			1		// J.83 annex C (Japanese QAM)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Reset Mode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTA1XX_RST_CLEARFIFO	0
#define  DTA1XX_RST_CHANNEL		1

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Power Mode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTA1XX_POWER_OFF		0
#define  DTA1XX_POWER_ON		1

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Target Adapter Present -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTA1XX_NO_CONNECTION	0
#define  DTA1XX_DVB_SPI_SINK	1		// For output channels
#define  DTA1XX_DVB_SPI_SOURCE	1		// For input channels
#define  DTA1XX_TARGET_PRESENT	2
#define  DTA1XX_TARGET_UNKNOWN	3


//=+=+=+=+=+=+=+=+=+=+=+=+ Transmit channel control/status codes +=+=+=+=+=+=+=+=+=+=+=+=+

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Packet Transmit Mode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTA1XX_TXMODE_188		0
#define  DTA1XX_TXMODE_204		1
#define  DTA1XX_TXMODE_ADD16	2
#define  DTA1XX_TXMODE_RAW		3
#define  DTA1XX_TXMODE_192		4		// Supported on DTA-102 Fw Rev>= 2
#define  DTA1XX_TXMODE_130		5		// Supported on DTA-102 Fw Rev>= 7
#define  DTA1XX_TXMODE_MIN16	6		// Supported on DTA-102 Fw Rev>= 8
										//				DTA-100 Fw Rev >=5
										//				DTA-140 Fw Rev >=0
#define	 DTA1XX_TXMODE_MASK		0x0F	// Mask for TxMode without burst flag
#define	 DTA1XX_TXMODE_BURST	0x10	// Can be OR-ed with one of 188/192/204/ADD16/RAW

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Burst mode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define DTA1XX_BURSTMODE_OFF	0
#define DTA1XX_BURSTMODE_ON		1

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Packet stuff mode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define DTA1XX_STUFFMODE_OFF	0
#define DTA1XX_STUFFMODE_ON		1

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Transmit Polarity -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTA1XX_TXPOL_NORMAL	0
#define  DTA1XX_TXPOL_INVERTED	1

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Status Flags -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTA1XX_TX_DMA_PENDING	1
#define  DTA1XX_TX_FIFO_UFL		2
#define  DTA1XX_TX_SYNC_ERR		4
#define  DTA1XX_TX_READBACK_ERR	8
#define  DTA1XX_TX_TARGET_ERR	16

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Transmit Control -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTA1XX_TXCTRL_IDLE		0
#define  DTA1XX_TXCTRL_HOLD		1
#define  DTA1XX_TXCTRL_SEND		3

//+=+=+=+=+=+=+=+=+=+=+=+=+ Receive channel control/status codes +=+=+=+=+=+=+=+=+=+=+=+=+

//-.-.-.-.-.-.-.-.-.-.-.-.-.- DVB/ASI Polarity-Control Status -.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTA1XX_ASIINV_NORMAL	0
#define  DTA1XX_ASIINV_INVERT	1

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DVB/ASI Input-Clock Lock -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTA1XX_ASI_NOLOCK		0
#define  DTA1XX_ASI_INLOCK		1

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Clock Detector -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTA1XX_CLKDET_FAIL		0
#define  DTA1XX_CLKDET_OK		1

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Input Rate Ok -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTA1XX_INPRATE_LOW		0
#define  DTA1XX_INPRATE_OK		1

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- #Invalid Bytes per Packet -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTA1XX_NUMINV_NONE		0
#define  DTA1XX_NUMINV_16		1
#define  DTA1XX_NUMINV_OTHER	2

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Packet Size -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTA1XX_PCKSIZE_INV		0
#define  DTA1XX_PCKSIZE_188		2
#define  DTA1XX_PCKSIZE_204		3

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Receive Control -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTA1XX_RXCTRL_IDLE		0
#define  DTA1XX_RXCTRL_RCV		1

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Receive Mode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTA1XX_RXMODE_ST188	0
#define  DTA1XX_RXMODE_ST204	1
#define  DTA1XX_RXMODE_STMP2	2
#define  DTA1XX_RXMODE_STRAW	3
#define  DTA1XX_RX_TIMESTAMP	8

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Status Flags -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTA1XX_RX_DMA_PENDING	1
#define  DTA1XX_RX_FIFO_OVF		2
#define  DTA1XX_RX_SYNC_ERR		4
#define  DTA1XX_RX_RATE_OVF		8

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Clock-Generator Modes -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTA1XX_TXCLOCK_INTERNAL	0
#define  DTA1XX_TXCLOCK_EXTERNAL	1

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Equaliser Settings -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTA1XX_EQUALISER_OFF	0
#define  DTA1XX_EQUALISER_ON	1

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Polarity Control -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define  DTA1XX_POLARITY_AUTO		0
#define  DTA1XX_POLARITY_NORMAL		2
#define  DTA1XX_POLARITY_INVERTED	3

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Result codes +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+

// Feature not supported
#define  DTA1XX_NOT_SUPPORTED		-1

#define  DTA1XX_OK			0
#define  DTA1XX_E			0x1000
#define  DTA1XX_E_ATTACHED		(DTA1XX_E + 0)
#define  DTA1XX_E_BUF_TOO_SMALL		(DTA1XX_E + 1)
#define  DTA1XX_E_DEV_DRIVER		(DTA1XX_E + 2)
#define  DTA1XX_E_EEPROM_FULL		(DTA1XX_E + 3)
#define  DTA1XX_E_EEPROM_READ		(DTA1XX_E + 4)
#define  DTA1XX_E_EEPROM_WRITE		(DTA1XX_E + 5)
#define  DTA1XX_E_EEPROM_FORMAT		(DTA1XX_E + 6)
#define  DTA1XX_E_FIFO_FULL		(DTA1XX_E + 7)
#define  DTA1XX_E_IN_USE		(DTA1XX_E + 8)
#define  DTA1XX_E_INVALID_BUF		(DTA1XX_E + 9)
#define  DTA1XX_E_INVALID_CGMODE	(DTA1XX_E + 10)
#define  DTA1XX_E_INVALID_FLAGS		(DTA1XX_E + 11)
#define  DTA1XX_E_INVALID_MODE		(DTA1XX_E + 12)
#define  DTA1XX_E_INVALID_RATE		(DTA1XX_E + 13)
#define  DTA1XX_E_INVALID_SIZE		(DTA1XX_E + 14)
#define  DTA1XX_E_KEYWORD		(DTA1XX_E + 15)
#define  DTA1XX_E_NO_DTA_CARD		(DTA1XX_E + 16)	// BACKWARD COMPATIBILITY
#define  DTA1XX_E_NO_DEVICE		(DTA1XX_E + 16)
#define  DTA1XX_E_NO_LOOPBACK		(DTA1XX_E + 17)
#define  DTA1XX_E_NO_SUCH_DEVICE	(DTA1XX_E + 18)
#define  DTA1XX_E_NO_SUCH_OUTPUT	(DTA1XX_E + 19)
#define  DTA1XX_E_NO_TS_OUTPUT		(DTA1XX_E + 20)
#define  DTA1XX_E_NOT_ATTACHED		(DTA1XX_E + 21)
#define  DTA1XX_E_NOT_FOUND		(DTA1XX_E + 22)
#define  DTA1XX_E_NOT_SUPPORTED		(DTA1XX_E + 23)
#define  DTA1XX_E_PCICARD		(DTA1XX_E + 24)	// BACKWARD COMPATIBILITY
#define  DTA1XX_E_DEVICE		(DTA1XX_E + 24)
#define  DTA1XX_E_TOO_LONG		(DTA1XX_E + 25)
#define  DTA1XX_E_UNDERFLOW		(DTA1XX_E + 26)
#define  DTA1XX_E_NO_SUCH_INPUT		(DTA1XX_E + 27)
#define  DTA1XX_E_NO_TS_INPUT		(DTA1XX_E + 28)
#define  DTA1XX_E_DRIVER_INCOMP		(DTA1XX_E + 29)
#define  DTA1XX_E_INTERNAL		(DTA1XX_E + 30)
#define  DTA1XX_E_OUT_OF_MEM		(DTA1XX_E + 31)
#define  DTA1XX_E_INVALID_ROLLOFF	(DTA1XX_E + 32)

#endif //__DTA1XXCODES_H
