//#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* Dta1xxIoCtl.h *#*#*#*#*#*#*#*# (C) 2000-2005 DEKTEC

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- License -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// This program is free software; You can freely use/modify this source code in any way
// you desire, including for commercial applications.
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	TD	2007.01.31	Added SetIoConfig and GetIoConfig
//	MG	2004.11.05	Added modulation and RF control commands
//	MG	2003.05.09	Added command for getting the firmware version
//	MG	2003.04.25	Moved DTA1XX_FUNCTION_READ_VPD and DTA1XX_FUNCTION_READ_VPD to end for
//					backward-compatibility reasons
//	MG	2003.04.23	Added VPD read and write commands
//	MG	21-10-2002	Created

#ifndef __DTA1XXIOCTL_H
#define __DTA1XXIOCTL_H

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Constants -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define	DTA1XX_IOCTL_MAGIC		0xAA	// Magic number
#define	DTA1XX_IOCTL_BASE		0x40	// Offset for Function
#define	DTA1XX_VPD_MAX_TRANSFER	512		// Maximum supported VPD transfer length

#define	DTA1XX_DEVICE_TYPE		45100	// DeviceType for DeviceIoControl operations
#define DTA1XX_LOC_BUF_SIZE		128		// Size of location-info buffer in #WCHARs
#define	DTA1XX_VPD_BUF_SIZE		512		// Size of VPD buffer

//+=+=+=+=+=+=+=+=+=+=+=+=+ Ioctl Input / Output Data Structures +=+=+=+=+=+=+=+=+=+=+=+=+

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Device-Driver Versioning Info. -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
typedef struct _DTA1XX_DRIVER_VERSION {
	int		m_nVersionMajor;		// Major version number
	int		m_nVersionMinor;		// Minor version number
	int		m_nVersionMicro;		// Micro version number (bug fix)
	int		m_nVersionBuild;		// Build number
} DTA1XX_DRIVER_VERSION;

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Location Info -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
typedef struct _DTA1XX_GET_LOC_INFO {
	int		m_nDeviceId;			// Device ID
	int		m_nVendorId;			// Vendor ID
	int		m_nSubSystemId;			// Subsystem ID
	int		m_nSubVendorId;			// Subsystem Vendor ID
	int		m_nBusNumber;			// PCI-bus number
	int		m_nSlotNumber;			// PCI-slot number
} DTA1XX_LOC_INFO;

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Flags -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
typedef struct _DTA1XX_FLAGS {
	int		m_nStatus;				// Flags
	int		m_nLatched;				// Latched flags (to be cleared)
} DTA1XX_FLAGS;

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Target Info -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
typedef struct _DTA1XX_TARGET_INFO {
	int		m_nPresent;				// 0=Nothing connected; 1=DVB-SPI sink;
									// 2=Target adapter connected
	int		m_nTargetId;			// Target-adapter identifier
} DTA1XX_TARGET_INFO;

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Rx status information -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
typedef struct _DTA1XX_RX_STATUS {
	int			m_nPacketSize;		// Packet size
	int			m_nNumInv;			// Number invalid bytes in packet
	int			m_nClkDet;			// Clock detected (YES/NO)
	int			m_nAsiLock;			// Asi lock detected (YES/NO)
	int			m_nRateOk;			// Rate is OK (higher than minimum)
	int			m_nAsiInv;			// ASI signal inverted (YES/NO)
} DTA1XX_RX_STATUS;

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Transport-Stream Rate Info -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// NOTE: only used with Transport-Stream transmit channel
//
typedef struct _DTA1XX_RATE_INFO {
	int		m_nClockGenMode;		// Internal / external mode
	int		m_nTsRate;				// Transport-stream rate
} DTA1XX_RATE_INFO;

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Transport-Stream Rate Info -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// NOTE: only used with Transport-Stream transmit channel
//
typedef struct _DTA1XX_TXCLOCK_INFO {
	int		m_nClockGenMode;		// Internal / external mode
	int		m_nPhaseIncr;			// TxClock register phase increment
	int		m_nSymbRateMult;		// Symbol rate multiplier (Only for modulator boards)
} DTA1XX_TXCLOCK_INFO;

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Transmit Mode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// NOTE: only used with Transport-Stream transmit channel
//
typedef struct _DTA1XX_TX_MODE {
	int		m_nTxPacketMode;		// Packet transmit mode
	int		m_nTxStuffMode;			// Null-packet stuffing on/off
} DTA1XX_TX_MODE;

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Data transfer decsriptor -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
typedef struct _DTA1XX_DATA_TRANSFER_DESC
{
	char*	m_pBuffer;				// Pointer to user buffer (receiving/providing data to
									// transfer)
	int		m_nNumBytesToTr;		// Number of bytes to transfer (buffer must be at
									// least this size)
} DTA1XX_DATA_TRANSFER_DESC;

//-.-.-.-.-.-.-.-.-.-.-.- Vital Product Data for VPD read or write -.-.-.-.-.-.-.-.-.-.-.-
//
typedef struct _DTA1XX_VPD_TRANSFER {
	int				m_nStart;		// Start BYTE address
	int				m_nLength;		// Number of BYTES to transfer (buffer must be at
									// least this size)
	unsigned char*	m_pVpdBuf;		// Pointer to user buffer (receiving/providing VPD data
									// to transfer)
} DTA1XX_VPD_TRANSFER;

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Modulation Control -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// NOTE: only used with Transport-Stream transmit channel
//
typedef struct _DTA1XX_MOD_CONTROL {
	int		m_nModType;				// Modulation type
	int		m_nCodeRate;			// Convolution rate
	int		m_nParXtra1;			// Extra parameter #1
	int		m_nParXtra2;			// Extra parameter #2
} DTA1XX_MOD_CONTROL;

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- RF control -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
typedef struct _DTA1XX_RF_CONTROL {
	int			m_nPllA;			// PLL A setting
	int			m_nPllB;			// PLL B setting
	int			m_nLockStatus;		// Lock status of RF PLL
} DTA1XX_RF_CONTROL;

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Continuous DMA (CDMA) -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
typedef struct _DTA1XX_CDMA_CONTROL {
	Int  m_ChannelType;				// Channel type: DTA1XX_TS_TX_CHANNEL/...
	Int  m_PortIndex;				// Zero-based port index (0=first port, ...)
	Int  m_Control;					// Control code
	Int  m_Segment;					// CDMA buffer segment
	UInt32  m_RefClkSamp;			// Reference-clock sample at last segment boundary
	Int  m_BufPos;					// Estimate of current CDMA buffer position

        // Modulation sample rate (needed for computing buffer position)
	Int m_ModSampRate;

} DTA1XX_CDMA_CONTROL;

//.-.-.-.-.-.-.-.-.-.-.-.-.- Shared Buffer Allocation / Freeing -.-.-.-.-.-.-.-.-.-.-.-.-.
typedef struct _DTA1XX_SHAREDBUF_CTRL {
	Int  m_Purpose;					// Purpose: DTA1XX_SHBUF_CDMA, ...
	Int  m_ChannelType;				// Channel type: DTA1XX_TS_TX_CHANNEL/...
	Int  m_PortIndex;				// Zero-based port index (0=first port, ...)
	Int  m_ChannelIndex;			// Channel index for IP channels

    // User buffer decsription -> for allocation
    UInt8* m_pSharedBuf;
    Int m_SharedBufSize;

} DTA1XX_SHAREDBUF_CTRL;

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Register access control -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
typedef struct _DTA1XX_REGISTER_CTRL
{
  Int m_Address;           // Address, within the cards address map, of the register
  Int m_RegValue;          // The register value (read or to be written)
} DTA1XX_REGISTER_CTRL;

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Union of all data structures -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
typedef struct _DTA1XX_IOCTL_DATA {
	union {
		DTA1XX_DRIVER_VERSION	m_DriverVersion;	// Versioning info
		DTA1XX_LOC_INFO			m_LocInfo;			// Location info
		DTA1XX_FLAGS			m_Flags;			// Flags
		DTA1XX_RX_STATUS		m_RxStatus;			// Rx Status
		DTA1XX_TARGET_INFO		m_TargetInfo;		// Target-adapter info
		DTA1XX_RATE_INFO		m_RateInfo;			// Transport-stream rate info
		DTA1XX_TXCLOCK_INFO		m_TxClockInfo;		// TRansmit-clock info
		DTA1XX_TX_MODE			m_TxMode;			// Transmit mode
		DTA1XX_DATA_TRANSFER_DESC
								m_DataTransDesc;	// Data transfer descriptor
		DTA1XX_VPD_TRANSFER		m_Vpd;				// VPD transfer data
		DTA1XX_MOD_CONTROL		m_ModCtrl;			// Modulation control
		DTA1XX_RF_CONTROL		m_RfCtrl;			// RF control
		DTA1XX_CDMA_CONTROL		m_CdmaCtrl;			// CDMA control command
		DTA1XX_SHAREDBUF_CTRL	m_ShBufCtrl;		// Shared buffer control message
        DTA1XX_REGISTER_CTRL    m_RegCtrl;          // Register access control
		unsigned int			m_uiValue;			// Simple value
	}			m_Data;
	int			m_nChIdx;		// Channel index (0=first channel, ...)
} DTA1XX_IOCTL_DATA;

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Function Codes +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=
enum {
	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Commom functions -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	DTA1XX_FUNCTION_GET_DRIVER_VERSION  = DTA1XX_IOCTL_BASE,
	DTA1XX_FUNCTION_GET_LOCATION_INFO,
	DTA1XX_FUNCTION_DMA_USERMEM,
	DTA1XX_FUNCTION_LED_CONTROL,
	DTA1XX_FUNCTION_RESET_SOFT,

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Rx channel functions -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	DTA1XX_RX_FUNCTION_EQUALISE,
    DTA1XX_RX_FUNCTION_GET_FIFO_LOAD,
	DTA1XX_RX_FUNCTION_GET_FLAGS,
	DTA1XX_RX_FUNCTION_CLEAR_FLAGS,
	DTA1XX_RX_FUNCTION_GET_STATUS,
	DTA1XX_RX_FUNCTION_GET_TARGETID,
	DTA1XX_RX_FUNCTION_GET_TS_RATE_BPS,
	DTA1XX_RX_FUNCTION_POLARITY_CONTROL,
	DTA1XX_RX_FUNCTION_RESET,
	DTA1XX_RX_FUNCTION_SET_LOOPBACK_MODE,
	DTA1XX_RX_FUNCTION_SET_POWER,
	DTA1XX_RX_FUNCTION_SET_RX_CONTROL,
	DTA1XX_RX_FUNCTION_SET_RX_MODE,

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Tx channel functions -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	DTA1XX_TX_FUNCTION_CLEAR_FIFO,
	DTA1XX_TX_FUNCTION_CLEAR_FLAGS,
	DTA1XX_TX_FUNCTION_GET_EXT_CLK_FREQ,
	DTA1XX_TX_FUNCTION_GET_FIFO_LOAD,
	DTA1XX_TX_FUNCTION_GET_FLAGS,
	DTA1XX_TX_FUNCTION_GET_MAX_FIFO_SIZE,
	DTA1XX_TX_FUNCTION_GET_TARGET_ID,
	DTA1XX_TX_FUNCTION_GET_TS_RATE_BPS,
	DTA1XX_TX_FUNCTION_GET_TX_CONTROL,
	DTA1XX_TX_FUNCTION_GET_TX_MODE,
    DTA1XX_TX_FUNCTION_RD_LOOPBACK_DATA,
	DTA1XX_TX_FUNCTION_RESET,
	DTA1XX_TX_FUNCTION_SET_TX_POLARITY,
	DTA1XX_TX_FUNCTION_SET_FIFO_SIZE,
	DTA1XX_TX_FUNCTION_SET_LOOPBACK_MODE,
	DTA1XX_TX_FUNCTION_SET_POWER,
	DTA1XX_TX_FUNCTION_SET_TS_RATE_BPS,
	DTA1XX_TX_FUNCTION_SET_TX_CONTROL,
	DTA1XX_TX_FUNCTION_SET_TX_MODE,

	//.-.-.-.-.-.- New functions add to end for backward compatibilty reasons -.-.-.-.-.-.
	DTA1XX_FUNCTION_READ_VPD,
	DTA1XX_FUNCTION_WRITE_VPD,
	DTA1XX_FUNCTION_GET_FW_VERSION,
	DTA1XX_TX_FUNCTION_GET_MOD_CONTROL,
	DTA1XX_TX_FUNCTION_SET_MOD_CONTROL,
	DTA1XX_TX_FUNCTION_GET_RF_CONTROL,
	DTA1XX_TX_FUNCTION_SET_RF_CONTROL,
	DTA1XX_FUNCTION_REF_CLOCK_COUNT,
	DTA1XX_TX_FUNCTION_GET_TX_CLOCK,
	DTA1XX_TX_FUNCTION_SET_TX_CLOCK,
	DTA1XX_RX_FUNCTION_GET_RX_CONTROL,
	DTA1XX_RX_FUNCTION_GET_STATISTICS,
	DTA1XX_FUNCTION_CDMA_CTRL,
	DTA1XX_FUNCTION_SHAREDBUF_ALLOC,
	DTA1XX_FUNCTION_SHAREDBUF_FREE,
    DTA1XX_FUNCTION_WRITE_REG,
    DTA1XX_FUNCTION_READ_REG,
	DTA1XX_FUNCTION_GET_IOCONFIG,
	DTA1XX_FUNCTION_SET_IOCONFIG
};

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_CDMA_CTRL -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// InBuffer : DTA1XX_CDMA_CONTROL
// OutBuffer: DTA1XX_CDMA_CONTROL
// Result	: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Continuous DMA (CDMA) - Control.
//
#define DTA1XX_IOCTL_CDMA_CTRL				_IOWR(DTA1XX_IOCTL_MAGIC, DTA1XX_FUNCTION_CDMA_CTRL, DTA1XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_SHAREDBUF_ALLOC -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// InBuffer : DTA1XX_SHAREDBUF_CTRL
// OutBuffer: -
// Result   : DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Supply a data buffer that can be shared between driver and user space application.
//
#define DTA1XX_IOCTL_SHAREDBUF_ALLOC	_IOWR(DTA1XX_IOCTL_MAGIC, DTA1XX_FUNCTION_SHAREDBUF_ALLOC, DTA1XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_SHAREDBUF_FREE -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// InBuffer : DTA1XX_SHAREDBUF_CTRL
// OutBuffer: DTA1XX_SHAREDBUF_RTRN
// Result   : DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Free a shared data buffer.
//
#define DTA1XX_IOCTL_SHAREDBUF_FREE		_IOWR(DTA1XX_IOCTL_MAGIC, DTA1XX_FUNCTION_SHAREDBUF_FREE, DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_WRITE_REG -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// InBuffer : DTA1XX_REGISTER_CTRL
// OutBuffer: DTA1XX_REGISTER_CTRL
// Result   : DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Write to a specific register
//
#define DTA1XX_IOCTL_WRITE_REG			_IOWR(DTA1XX_IOCTL_MAGIC, DTA1XX_FUNCTION_WRITE_REG, DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_READ_REG -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// InBuffer : DTA1XX_REGISTER_CTRL
// OutBuffer: DTA1XX_REGISTER_CTRL
// Result   : DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Read from a specific register
//
#define DTA1XX_IOCTL_READ_REG			_IOWR(DTA1XX_IOCTL_MAGIC, DTA1XX_FUNCTION_READ_REG, DTA1XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_GET_REF_CLOCK_COUNT -.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (clock count)
// Channel index	: not used
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Get the reference clock count value
//
#define DTA1XX_IOCTL_GET_REF_CLOCK_COUNT	_IOR(DTA1XX_IOCTL_MAGIC, DTA1XX_FUNCTION_REF_CLOCK_COUNT, DTA1XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_GET_DRIVER_VERSION -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_DriverVersion
// Channel index	: not used
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Get device-driver version.
//
#define DTA1XX_IOCTL_GET_DRIVER_VERSION	_IOR(DTA1XX_IOCTL_MAGIC, DTA1XX_FUNCTION_GET_DRIVER_VERSION, DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_GET_LOCATION_INFO -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_LocInfo
// Channel index	: not used
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Returns the cards location information
//
#define DTA1XX_IOCTL_GET_LOCATION_INFO	_IOR(DTA1XX_IOCTL_MAGIC, DTA1XX_FUNCTION_GET_LOCATION_INFO, DTA1XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_GET_FW_VERSION -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_uiValue (firmware version)
// Channel index	: not used
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Returns the cards firmware version
//
#define DTA1XX_IOCTL_GET_FW_VERSION	_IOR(DTA1XX_IOCTL_MAGIC, DTA1XX_FUNCTION_GET_FW_VERSION, DTA1XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_LED_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (e.g. DTA1XX_LED_OFF, DTA1XX_LED_GREEN, etc)
// Channel index	: not used
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Front panel LED control
//
#define DTA1XX_IOCTL_LED_CONTROL		_IOW(DTA1XX_IOCTL_MAGIC, DTA1XX_FUNCTION_LED_CONTROL, DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_DMA_USERMEM_READ -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_DataTransDesc
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Read data (using DMA) in an application-supplied buffer in User-Space memory.
//
#define DTA1XX_IOCTL_DMA_USERMEM_READ	_IOR(DTA1XX_IOCTL_MAGIC, DTA1XX_FUNCTION_DMA_USERMEM, DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_DMA_USERMEM_WRITE -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_DataTransDesc
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Write data (using DMA) from an application-supplied buffer in User-Space memory.
//
#define DTA1XX_IOCTL_DMA_USERMEM_WRITE	_IOW(DTA1XX_IOCTL_MAGIC, DTA1XX_FUNCTION_DMA_USERMEM, DTA1XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_RESET_SOFT -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: none
// Channel index	: not used
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Reset the DTA-1xx card.
//
#define DTA1XX_IOCTL_RESET_SOFT			_IOW(DTA1XX_IOCTL_MAGIC, DTA1XX_FUNCTION_RESET_SOFT, DTA1XX_IOCTL_DATA)


//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_RESET_SOFT -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_Vpd
// Channel index	: not used
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Read specified range of VPD data BYTES.
// DTA1XX_VPD_TRANSFER.m_pVpdBuf[0] will contain first the byte read from EEPROM address m_nStart.
//
// NOTE: m_nLength should not be larger than DTA1XX_VPD_MAX_TRANSFER
//
#define DTA1XX_IOCTL_READ_VPD		_IOR(DTA1XX_IOCTL_MAGIC, DTA1XX_FUNCTION_READ_VPD, DTA1XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_RESET_SOFT -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_Vpd
// Channel index	: not used
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Write specified range of VPD data BYTES
// DTA1XX_VPD_TRANSFER.m_pVpdBuf[0] shall contain the first byte to be written to EEPROM address
// m_nStart.
//
// NOTE: m_nLength should not be larger than DTA1XX_VPD_MAX_TRANSFER
//
#define DTA1XX_IOCTL_WRITE_VPD		_IOW(DTA1XX_IOCTL_MAGIC, DTA1XX_FUNCTION_WRITE_VPD, DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_SET_IOCONFIG -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_uiValue (DTA1XX_RXCTRL_IDLE or DTA1XX_RXCTRL_RCV)
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Set IO configuration
//
#define DTA1XX_IOCTL_SET_IOCONFIG	_IOW(DTA1XX_IOCTL_MAGIC, DTA1XX_FUNCTION_SET_IOCONFIG, DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_GET_IOCONFIG -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_uiValue (DTA1XX_RXCTRL_IDLE or DTA1XX_RXCTRL_RCV)
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Get IO configuration
//
#define DTA1XX_IOCTL_GET_IOCONFIG	_IOR(DTA1XX_IOCTL_MAGIC, DTA1XX_FUNCTION_GET_IOCONFIG, DTA1XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_EQUALISE -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (DTA1XX_EQUALISER_ON or DTA1XX_EQUALISER_OFF)
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Enable/Disable the equaliser
//
#define DTA1XX_RX_IOCTL_EQUALISE		_IOW(DTA1XX_IOCTL_MAGIC, DTA1XX_RX_FUNCTION_EQUALISE, DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_GET_FIFO_LOAD -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (returns the fifo load)
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Get the channels hardware fifo load
//
#define DTA1XX_RX_IOCTL_GET_FIFO_LOAD	_IOR(DTA1XX_IOCTL_MAGIC, DTA1XX_RX_FUNCTION_GET_FIFO_LOAD, DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_GET_FLAGS -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_Flags
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Get current value of status flags and latched status flags.
//
#define DTA1XX_RX_IOCTL_GET_FLAGS		_IOR(DTA1XX_IOCTL_MAGIC, DTA1XX_RX_FUNCTION_GET_FLAGS, DTA1XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_CLEAR_FLAGS -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_uiValue (Latched flags to clear)
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Clear selected flags
//
#define DTA1XX_RX_IOCTL_CLEAR_FLAGS		_IOW(DTA1XX_IOCTL_MAGIC, DTA1XX_RX_FUNCTION_CLEAR_FLAGS, DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_GET_STATUS -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_RxStatus
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Get the receive status (Packet size, ASI lock, clock detect ect)
//
#define DTA1XX_RX_IOCTL_GET_STATUS		_IOR(DTA1XX_IOCTL_MAGIC, DTA1XX_RX_FUNCTION_GET_STATUS, DTA1XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_GET_TARGETID -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_TargetInfo
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Get the target adapter ID
//
#define DTA1XX_RX_IOCTL_GET_TARGETID	_IOR(DTA1XX_IOCTL_MAGIC, DTA1XX_RX_FUNCTION_GET_TARGETID, DTA1XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_GET_TS_RATE_BPS -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_uiValue (measured bit-rate)
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Get the measured transport-stream bit-rate
//
#define DTA1XX_RX_IOCTL_GET_TS_RATE_BPS	_IOR(DTA1XX_IOCTL_MAGIC, DTA1XX_RX_FUNCTION_GET_TS_RATE_BPS, DTA1XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_POLARITY_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (DTA1XX_POLARITY_AUTO,  DTA1XX_POLARITY_NORMAL or
//								 DTA1XX_POLARITY_INVERTED)
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Set the ASI polarity control
//
#define DTA1XX_RX_IOCTL_POLARITY_CONTROL	_IOW(DTA1XX_IOCTL_MAGIC, DTA1XX_RX_FUNCTION_POLARITY_CONTROL, DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_RESET -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (DTA1XX_RST_CLEARFIFO or DTA1XX_RST_CHANNEL)
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Reset the Rx Channel
//
#define DTA1XX_RX_IOCTL_RESET			_IOW(DTA1XX_IOCTL_MAGIC, DTA1XX_RX_FUNCTION_RESET, DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_SET_LOOPBACK_MODE -.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (DTA1XX_NO_LOOPBACK or DTA1XX_LOOPBACK_MODE)
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Set the loopback mode
//
#define DTA1XX_RX_IOCTL_SET_LOOPBACK_MODE	_IOW(DTA1XX_IOCTL_MAGIC, DTA1XX_RX_FUNCTION_SET_LOOPBACK_MODE, DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_SET_POWER -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (DTA1XX_POWER_OFF or DTA1XX_POWER_ON)
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Power enable control
//
#define DTA1XX_RX_IOCTL_SET_POWER		_IOW(DTA1XX_IOCTL_MAGIC, DTA1XX_RX_FUNCTION_SET_POWER, DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_SET_RX_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_uiValue (DTA1XX_RXCTRL_IDLE or DTA1XX_RXCTRL_RCV)
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Set receive control
//
#define DTA1XX_RX_IOCTL_SET_RX_CONTROL	_IOW(DTA1XX_IOCTL_MAGIC, DTA1XX_RX_FUNCTION_SET_RX_CONTROL, DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_GET_RX_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_uiValue (DTA1XX_RXCTRL_IDLE or DTA1XX_RXCTRL_RCV)
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Get receive control
//
#define DTA1XX_RX_IOCTL_GET_RX_CONTROL	_IOR(DTA1XX_IOCTL_MAGIC, DTA1XX_RX_FUNCTION_GET_RX_CONTROL, DTA1XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_SET_RX_MODE -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_uiValue (e.g. DTA1XX_RXMODE_ST188, DTA1XX_RXMODE_ST204 etc)
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Set the rx mode
//
#define DTA1XX_RX_IOCTL_SET_RX_MODE		_IOW(DTA1XX_IOCTL_MAGIC, DTA1XX_RX_FUNCTION_SET_RX_MODE, DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_GET_STATISTICS -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_uiValue (Violation count value)
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Get input statistics
//
#define DTA1XX_RX_IOCTL_GET_STATISTICS	_IOR(DTA1XX_IOCTL_MAGIC, DTA1XX_RX_FUNCTION_GET_STATISTICS, DTA1XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_CLEAR_FLAGS -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_uiValue (Latched flags to clear)
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Clear the specified latched status flags
//
#define DTA1XX_TX_IOCTL_CLEAR_FLAGS		_IOW(DTA1XX_IOCTL_MAGIC, DTA1XX_TX_FUNCTION_CLEAR_FLAGS, DTA1XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_GET_EXT_CLK_FREQ -.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (freq. of the external clock)
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Get the measured frequency of the external clock
//
#define DTA1XX_TX_IOCTL_GET_EXT_CLK_FREQ	_IOR(DTA1XX_IOCTL_MAGIC, DTA1XX_TX_FUNCTION_GET_EXT_CLK_FREQ, DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_GET_FIFO_LOAD -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (the fifo load)
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Get the HW fifo load
//
#define DTA1XX_TX_IOCTL_GET_FIFO_LOAD	_IOR(DTA1XX_IOCTL_MAGIC, DTA1XX_TX_FUNCTION_GET_FIFO_LOAD, DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_GET_FLAGS -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_Flags
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Get current value of status flags and latched status flags.
//
#define DTA1XX_TX_IOCTL_GET_FLAGS		_IOR(DTA1XX_IOCTL_MAGIC, DTA1XX_TX_FUNCTION_GET_FLAGS, DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_GET_MAX_FIFO_SIZE -.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (the maximum allowed fifo size)
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Returns the maximum allowed fifo size
//
#define DTA1XX_TX_IOCTL_GET_MAX_FIFO_SIZE	_IOR(DTA1XX_IOCTL_MAGIC, DTA1XX_TX_FUNCTION_GET_MAX_FIFO_SIZE, DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_GET_TARGET_ID -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_TargetInfo
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Get the target adapter ID
//
#define DTA1XX_TX_IOCTL_GET_TARGET_ID	_IOR(DTA1XX_IOCTL_MAGIC, DTA1XX_TX_FUNCTION_GET_TARGET_ID, DTA1XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_GET_TS_RATE_BPS -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_uiValue (the set output bit-rate)
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Gets the set transport-stream bit-rate
//
#define DTA1XX_TX_IOCTL_GET_TS_RATE_BPS	_IOR(DTA1XX_IOCTL_MAGIC, DTA1XX_TX_FUNCTION_GET_TS_RATE_BPS, DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_GET_TX_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_uiValue (DTA1XX_TXCTRL_IDLE, DTA1XX_TXCTRL_HOLD or
//								 DTA1XX_TXCTRL_SEND)
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Get the channel's transmit-control state.
//
#define DTA1XX_TX_IOCTL_GET_TX_CONTROL	_IOR(DTA1XX_IOCTL_MAGIC, DTA1XX_TX_FUNCTION_GET_TX_CONTROL, DTA1XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_GET_TX_MODE -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_TxMode
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Get the channel's transmit-mode state.
//
#define DTA1XX_TX_IOCTL_GET_TX_MODE		_IOR(DTA1XX_IOCTL_MAGIC, DTA1XX_TX_FUNCTION_GET_TX_MODE, DTA1XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_RD_LOOPBACK_DATA -.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_DataTransDesc
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Get the channel's transmit-mode state.
//
#define DTA1XX_TX_IOCTL_RD_LOOPBACK_DATA	_IOR(DTA1XX_IOCTL_MAGIC, DTA1XX_TX_FUNCTION_RD_LOOPBACK_DATA, DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_RESET -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (DTA1XX_RST_CLEARFIFO or DTA1XX_RST_CHANNEL)
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Resets the Tx channel
//
#define DTA1XX_TX_IOCTL_RESET			_IOW(DTA1XX_IOCTL_MAGIC, DTA1XX_TX_FUNCTION_RESET, DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_SET_FIFO_SIZE -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (the fifo size)
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Sets the fifo size
//
#define DTA1XX_TX_IOCTL_SET_FIFO_SIZE	_IOW(DTA1XX_IOCTL_MAGIC, DTA1XX_TX_FUNCTION_SET_FIFO_SIZE,  DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_SET_LOOPBACK_MODE -.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (DTA1XX_NO_LOOPBACK or DTA1XX_LOOPBACK_MODE)
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Set the loopback mode
//
#define DTA1XX_TX_IOCTL_SET_LOOPBACK_MODE	_IOW(DTA1XX_IOCTL_MAGIC, DTA1XX_TX_FUNCTION_SET_LOOPBACK_MODE, DTA1XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_SET_TX_POLARITY -.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (DTAPI_TXPOL_NORMAL or DTAPI_TXPOL_INVERTED)
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Set the ASI tx polarity
//
#define DTA1XX_TX_IOCTL_SET_TX_POLARITY _IOW(DTA1XX_IOCTL_MAGIC, DTA1XX_TX_FUNCTION_SET_TX_POLARITY, DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_SET_POWER -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (DTA1XX_POWER_OFF or DTA1XX_POWER_ON)
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Set the channels power mode
//
#define DTA1XX_TX_IOCTL_SET_POWER		_IOW(DTA1XX_IOCTL_MAGIC, DTA1XX_TX_FUNCTION_SET_POWER, DTA1XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_SET_TS_RATE_BPS -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_RateInfo (DTA1XX_POWER_OFF or DTA1XX_POWER_ON)
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Sets the channels transport-stream bit-rate
//
#define DTA1XX_TX_IOCTL_SET_TS_RATE_BPS	_IOW(DTA1XX_IOCTL_MAGIC, DTA1XX_TX_FUNCTION_SET_TS_RATE_BPS, DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_SET_TX_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_uiValue (DTA1XX_TXCTRL_IDLE, DTA1XX_TXCTRL_HOLD or
//								 DTA1XX_TXCTRL_SEND)
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Sets the channels transmit-control state
//
#define DTA1XX_TX_IOCTL_SET_TX_CONTROL	_IOW(DTA1XX_IOCTL_MAGIC, DTA1XX_TX_FUNCTION_SET_TX_CONTROL, DTA1XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_SET_TX_MODE -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_TxMode
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Sets the channels transmit-mode state
//
#define DTA1XX_TX_IOCTL_SET_TX_MODE		_IOW(DTA1XX_IOCTL_MAGIC, DTA1XX_TX_FUNCTION_SET_TX_MODE, DTA1XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_GET_MOD_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_ModCtrl
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Get modulation parameters.
//
#define DTA1XX_TX_IOCTL_GET_MOD_CONTROL	_IOR(DTA1XX_IOCTL_MAGIC, DTA1XX_TX_FUNCTION_GET_MOD_CONTROL, DTA1XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-  DTA1XX_TX_IOCTL_SET_MOD_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_ModCtrl
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Set modulation-control parameters.
//
#define DTA1XX_TX_IOCTL_SET_MOD_CONTROL	_IOW(DTA1XX_IOCTL_MAGIC, DTA1XX_TX_FUNCTION_SET_MOD_CONTROL, DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_GET_RF_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_RfCtrl
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Get RF-control settings
//
#define DTA1XX_TX_IOCTL_GET_RF_CONTROL	_IOR(DTA1XX_IOCTL_MAGIC, DTA1XX_TX_FUNCTION_GET_RF_CONTROL, DTA1XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_SET_RF_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_RfCtrl
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Set RF-control settings
//
#define DTA1XX_TX_IOCTL_SET_RF_CONTROL	_IOW(DTA1XX_IOCTL_MAGIC, DTA1XX_TX_FUNCTION_SET_RF_CONTROL, DTA1XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_GET_TX_CLOCK -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_TxClockInfo
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Get the transmit clock value
//
#define DTA1XX_TX_IOCTL_GET_TX_CLOCK	_IOR(DTA1XX_IOCTL_MAGIC, DTA1XX_TX_FUNCTION_GET_TX_CLOCK, DTA1XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_SET_TX_CLOCK -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_TxClockInfo
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTA1XX_OK if OK, DTA1XX_E_??? error code if failed
//
// Set the transmit clock value
//
#define DTA1XX_TX_IOCTL_SET_TX_CLOCK	_IOW(DTA1XX_IOCTL_MAGIC, DTA1XX_TX_FUNCTION_SET_TX_CLOCK, DTA1XX_IOCTL_DATA)


#endif // __DTA1XXIOCTL_H
