//#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*# Dta1xxRegs.h #*#*#*#*#*#*#*#* (C) 2000-2005 DEKTEC
//

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- License -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// This program is free software; You can freely use/modify this source code in any way
// you desire, including for commercial applications.
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	MG	21-10-2002	Created	

#ifndef __DTA1XX_REGS_H
#define __DTA1XX_REGS_H


//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#include "Helpers.h"
#include "GenRegs.h"
#include "TxRegs.h"
#include "RxRegs.h"

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Type definitions and constants -.-.-.-.-.-.-.-.-.-.-.-.-.-.

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Dta1xxGen +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
//
// Register block for GENERIC fields.
//
// Supports: DTA-140, DTA-160
//
typedef struct _Dta1xxGen {

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- General-Control register -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	struct {								
	volatile UInt  m_PE				: 1;	// Serial EEPROM Program Enable
	volatile UInt  m_PRE			: 1;	// Serial EEPROM Protect Register Enable
	volatile UInt  m_Reset			: 1;	// Board-wide software reset
			 UInt  m_Reserved1		: 1;	//
	volatile UInt  m_PerIntEn		: 1;	// Periodic-Interrupt Enable
	volatile UInt  m_PerIntInterval	: 3;	// Periodic-Interrupt Interval
	volatile UInt  m_FirmwRev		: 8;	// Firmware Revision
	volatile UInt  m_TypeNum		: 8;	// Board Type Number
	volatile UInt  m_LedControl		: 1;	// Take over LED Control
	volatile UInt  m_LedGreen		: 1;	// State of Green LED (if LedControl is set)
	volatile UInt  m_LedRed			: 1;	// State of Red LED (if LedControl is set)
			 UInt  m_Reserved2		: 5;	//
	}  m_GenControl;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- General-Status register -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
	union {
	 struct {
			  UInt32  m_Reserved1	: 4;	//
	 volatile UInt32  m_PerInt		: 1;	// Periodic Interrupt
			  UInt32  m_Reserved2	: 3;	//
	 volatile UInt32  m_FirmwVar	: 5;	// Variant of product firmware (DTA-160)
	          UInt32  m_Reserved3   : 3;	//
	 volatile UInt32  m_TypeNumExt  : 3;	// Type number extention
	          UInt32  m_Reserved4   : 13;	// 
	 }		 Fields;						// For access to individual field
	 UInt32  All;							// For access to GenStatus in one go
	}  m_GenStatus;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Programming register -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	struct {								
	volatile UInt  m_TRST			: 1;	// Control of TRST# pin
	volatile UInt  m_TDO			: 1;	// Status of TDO pin
	volatile UInt  m_TMS			: 1;	// Control of TMS pin
	volatile UInt  m_TCK			: 1;	// Control of TCK pin
	volatile UInt  m_TDI			: 1;	// Control of TDI pin
	volatile UInt  m_PrgEpc			: 1;	// Enable access to programming i/f of EPC2
	}  m_Programming;

	UInt32  m_RefClkCnt;					// Reference clock counter
	UInt32  m_EpcsCtrl;						// EPCS Control register
	UInt32  m_EpcsData;						// EPCS Data register
	UInt32  m_RefClkCntH	: 22;			// Reference clock counter highest 22 bits
	UInt32  m_Reserved1		: 10;
	UInt32  m_Reserved[25];					// Unused memory addresses
}  Dta1xxGen;

// Generic Registers: BYTE offset
#define DTA1XX_GEN_REG_GENCTRL		0x0000
#define DTA1XX_GEN_REG_GENSTATUS	0x0004
#define DTA1XX_GEN_REG_PROGRAMMING	0x0008
#define DTA1XX_GEN_REG_REFCLKCNT	0x000C
#define DTA1XX_GEN_REG_EPCSCTRL		0x0010
#define DTA1XX_GEN_REG_EPCSDATA		0x0014
#define DTA1XX_GEN_REG_REFCLKCNTH	0x0018
#define DTA1XX_GEN_REG_SPICTRL		0x0024
#define DTA1XX_GEN_REG_SPIDATA		0x0028
#define DTA1XX_GEN_REG_CONTROL0		0x0030
#define DTA1XX_GEN_REG_CONTROL1		0x0034
#define DTA1XX_GEN_REG_STATUS0		0x0038
#define DTA1XX_GEN_REG_STATUS1		0x003C
#define DTA1XX_GEN_REG_LICSERIAL	0x0070
#define DTA1XX_GEN_REG_LICCODE		0x0074
#define DTA1XX_GEN_REG_LICSTATUS1	0x0078
#define DTA1XX_GEN_REG_LICSTATUS2	0x007C

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ DMA Registers +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+

#define DTA1XX_DMA_DESCPTR_REG			0x004
#define DTA1XX_DMA_CMDSTAT_REG			0x008
#define DTA1XX_DMA_PCIUPADDR_REG		0x00C
#define DTA1XX_DMA_TIMEOUT_REG			0x014

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Timeout configuration register -.-.-.-.-.-.-.-.-.-.-.-.-.-.
static inline void  Dta1xxDmaTimeOutSet(unsigned char* pBase, unsigned int Val)
{	// Val = in 2^14 clock cycle increments
	int Enable;
	Enable = (Val != 0);
	Val = Val & 0xFF;
	if (Enable) Val = Val | (1 << 8);

	WRITE_LONG(Val, (int*)pBase, DTA1XX_DMA_TIMEOUT_REG);
}

//-.-.-.-.-.-.-.-.-.-.-.- EPCS Control Register: Access Functions -.-.-.-.-.-.-.-.-.-.-.-.

// EPCS Control: Bit fields
#define DTA1XX_EPCSCTRL_ERASE			0x00000001
#define DTA1XX_EPCSCTRL_PROG			0x00000002
#define DTA1XX_EPCSCTRL_READ			0x00000004
#define DTA1XX_EPCSCTRL_BUSY			0x00000080
#define DTA1XX_EPCSCTRL_ADDR_LSB_MASK	0x0FFFFF00
#define DTA1XX_EPCSCTRL_ADDR_LSB_NBITS	20
#define DTA1XX_EPCSCTRL_ADDR_LSB_SHIFT	8
#define DTA1XX_EPCSCTRL_ADDR_MSB_MASK	0x00000078
#define DTA1XX_EPCSCTRL_ADDR_MSB_SHIFT	3
#define DTA1XX_EPCSCTRL_UNLCK_SHIFT		28

// EPCS Control: Register access
static inline UInt  Dta1xxGenGetEpcsCtrl(Int* pBase) {
	unsigned long ret;
	READ_LONG(ret, pBase,  DTA1XX_GEN_REG_EPCSCTRL);
	return ret;
}
static inline void Dta1xxGenSetEpcsCtrl(Int* pBase, UInt Val) {
	WRITE_LONG(Val, pBase, DTA1XX_GEN_REG_EPCSCTRL);
}

// EPCS Control: Erase
static inline void  Dta1xxEpcsCtrlRegErase(Int* pBase, UInt Addr) {
	UInt  Val = Dta1xxGenGetEpcsCtrl(pBase);
	Val &= ~(DTA1XX_EPCSCTRL_ADDR_LSB_MASK | DTA1XX_EPCSCTRL_ADDR_MSB_MASK |
			 DTA1XX_EPCSCTRL_ERASE | DTA1XX_EPCSCTRL_PROG | DTA1XX_EPCSCTRL_READ);
	Val |= (Addr << DTA1XX_EPCSCTRL_ADDR_LSB_SHIFT) & DTA1XX_EPCSCTRL_ADDR_LSB_MASK;
	Val |= ((Addr>>DTA1XX_EPCSCTRL_ADDR_LSB_NBITS)<<DTA1XX_EPCSCTRL_ADDR_MSB_SHIFT)
				& DTA1XX_EPCSCTRL_ADDR_MSB_MASK;
	Val |= DTA1XX_EPCSCTRL_ERASE;
	Dta1xxGenSetEpcsCtrl(pBase, Val);
}

// EPCS Control: Page Program
static inline void  Dta1xxEpcsCtrlRegProg(Int* pBase, UInt Addr) {
	UInt  Val = Dta1xxGenGetEpcsCtrl(pBase);
	Val &= ~(DTA1XX_EPCSCTRL_ADDR_LSB_MASK | DTA1XX_EPCSCTRL_ADDR_MSB_MASK |
			 DTA1XX_EPCSCTRL_ERASE | DTA1XX_EPCSCTRL_PROG | DTA1XX_EPCSCTRL_READ);
	Val |= (Addr << DTA1XX_EPCSCTRL_ADDR_LSB_SHIFT) & DTA1XX_EPCSCTRL_ADDR_LSB_MASK;
	Val |= ((Addr>>DTA1XX_EPCSCTRL_ADDR_LSB_NBITS)<<DTA1XX_EPCSCTRL_ADDR_MSB_SHIFT)
				& DTA1XX_EPCSCTRL_ADDR_MSB_MASK;
	Val |= DTA1XX_EPCSCTRL_PROG;
	Dta1xxGenSetEpcsCtrl(pBase, Val);
}

// EPCS Control: Read command
// After this call, data can be read from EPCS Data register
static inline void  Dta1xxEpcsCtrlRegRead(Int* pBase, UInt Addr) {
	UInt  Val = Dta1xxGenGetEpcsCtrl(pBase);
	Val &= ~(DTA1XX_EPCSCTRL_ADDR_LSB_MASK | DTA1XX_EPCSCTRL_ADDR_MSB_MASK |
			 DTA1XX_EPCSCTRL_ERASE | DTA1XX_EPCSCTRL_PROG | DTA1XX_EPCSCTRL_READ);
	Val |= (Addr << DTA1XX_EPCSCTRL_ADDR_LSB_SHIFT) & DTA1XX_EPCSCTRL_ADDR_LSB_MASK;
	Val |= ((Addr>>DTA1XX_EPCSCTRL_ADDR_LSB_NBITS)<<DTA1XX_EPCSCTRL_ADDR_MSB_SHIFT)
				& DTA1XX_EPCSCTRL_ADDR_MSB_MASK;
	Val |= DTA1XX_EPCSCTRL_READ;
	Dta1xxGenSetEpcsCtrl(pBase, Val);
}

// EPCS Control: Busy
static inline UInt  Dta1xxEpcsCtrlRegIsBusy(Int* pBase) {
	return ((Dta1xxGenGetEpcsCtrl(pBase)&DTA1XX_EPCSCTRL_BUSY)!=0) ? 1 : 0;
}
// EPCS Control: Unlock
static inline void  Dta1xxEpcsCtrlRegSetUnlock(Int* pBase, UInt UnlockValue) {
	UInt  Val = Dta1xxGenGetEpcsCtrl(pBase);
	Val |= (UnlockValue << DTA1XX_EPCSCTRL_UNLCK_SHIFT);
	Dta1xxGenSetEpcsCtrl(pBase, Val);
}


//-.-.-.-.-.-.-.-.-.-.-.-.- EPCS Data Register: Access Functions -.-.-.-.-.-.-.-.-.-.-.-.-

// EPCS Data: Register access
static inline UInt  Dta1xxGenGetEpcsData(Int* pBase) {
	unsigned long ret;
	READ_LONG( ret, pBase, DTA1XX_GEN_REG_EPCSDATA);
	return ret;
}
static inline void Dta1xxGenSetEpcsData(Int* pBase, UInt Val) {
	WRITE_LONG(Val, pBase, DTA1XX_GEN_REG_EPCSDATA);
}


//.-.-.-.-.-.-.-.-.-.-.-.- SPI Control Register: Access Functions -.-.-.-.-.-.-.-.-.-.-.-.

// SPI Control: Bit fields
#define DTA1XX_SPICTRL_PROG				0x00000002
#define DTA1XX_SPICTRL_READ				0x00000004
#define DTA1XX_SPICTRL_BUSY				0x00000080
#define DTA1XX_SPICTRL_ADDR_MASK		0x0001FF00
#define DTA1XX_SPICTRL_ADDR_SHIFT		8
#define DTA1XX_SPICTRL_UNLCK_SHIFT		28

// SPI Control: Register access
static inline UInt  Dta1xxGenGetSpiCtrl(Int* pBase) {
	UInt ret;
	READ_LONG(ret, pBase, DTA1XX_GEN_REG_SPICTRL);
	return ret;
}
static inline void Dta1xxGenSetSpiCtrl(Int* pBase, UInt Val) {
	WRITE_LONG(Val, pBase, DTA1XX_GEN_REG_SPICTRL);
}

// SPI Control: Page Program
static inline void  Dta1xxSpiCtrlRegProg(Int* pBase, UInt Addr) {
	UInt  Val = Dta1xxGenGetSpiCtrl(pBase);
	Val &= ~(DTA1XX_SPICTRL_ADDR_MASK | DTA1XX_SPICTRL_PROG | DTA1XX_SPICTRL_READ);
	Val |= (Addr << DTA1XX_SPICTRL_ADDR_SHIFT) & DTA1XX_SPICTRL_ADDR_MASK;
	Val |= DTA1XX_SPICTRL_PROG;
	Dta1xxGenSetSpiCtrl(pBase, Val);
}

// SPI Control: Read command
// After this call, data can be read from SPI Data register
static inline void  Dta1xxSpiCtrlRegRead(Int* pBase, UInt Addr) {
	UInt  Val = Dta1xxGenGetSpiCtrl(pBase);
	Val &= ~(DTA1XX_SPICTRL_ADDR_MASK | DTA1XX_SPICTRL_PROG | DTA1XX_SPICTRL_READ);
	Val |= (Addr << DTA1XX_SPICTRL_ADDR_SHIFT) & DTA1XX_SPICTRL_ADDR_MASK;
	Val |= DTA1XX_SPICTRL_READ;
	Dta1xxGenSetSpiCtrl(pBase, Val);
}

// SPI Control: Busy
static inline UInt  Dta1xxSpiCtrlRegIsBusy(Int* pBase) {
	return ((Dta1xxGenGetSpiCtrl(pBase)&DTA1XX_SPICTRL_BUSY)!=0) ? 1 : 0;
}
// SPI Control: Unlock
static inline void  Dta1xxSpiCtrlRegSetUnlock(Int* pBase, UInt UnlockValue) {
	UInt  Val = Dta1xxGenGetSpiCtrl(pBase);
	Val |= (UnlockValue << DTA1XX_SPICTRL_UNLCK_SHIFT);
	Dta1xxGenSetSpiCtrl(pBase, Val);
}


//-.-.-.-.-.-.-.-.-.-.-.-.- SPI Data Register: Access Functions -.-.-.-.-.-.-.-.-.-.-.-.-.

// SPI Data: Register access
static inline UInt  Dta1xxGenGetSpiData(Int* pBase) {
	UInt ret;
	READ_LONG(ret, pBase, DTA1XX_GEN_REG_SPIDATA);
	return ret;
}
static inline void Dta1xxGenSetSpiData(Int* pBase, UInt Val) {
	WRITE_LONG(Val, pBase, DTA1XX_GEN_REG_SPIDATA);
}

//-.-.-.-.-.-.-.-.-.-.-.-.- Control0 Registers: Access Functions -.-.-.-.-.-.-.-.-.-.-.-.-
#define DTA1XX_GEN_CONTROL0_DBLBUF		0x00000001
#define DTA1XX_GEN_CONTROL0_VCOEN		0x00000002

static __inline UInt  Dta1xxGenGetControl0Reg(Int* pBase) {
	UInt ret;
	READ_LONG(ret, pBase, DTA1XX_GEN_REG_CONTROL0);
	return ret;
}
static __inline void Dta1xxGenSetControl0Reg(Int* pBase, UInt Val) {
	WRITE_LONG(Val, pBase, DTA1XX_GEN_REG_CONTROL0);
}

// Get VCO Enable
static __inline UInt Dta1xxGenVcoEnableGet(Int* pBase) {
	return (Dta1xxGenGetControl0Reg(pBase) & DTA1XX_GEN_CONTROL0_VCOEN ? 1 : 0);
}

// Set VCO Enable
static __inline void Dta1xxGenVcoEnableSet(Int* pBase, UInt Enable) {
	UInt Val;
	Val = Dta1xxGenGetControl0Reg(pBase);
	if (Enable) Val = Val | DTA1XX_GEN_CONTROL0_VCOEN;
	else		Val = Val & ~DTA1XX_GEN_CONTROL0_VCOEN;
	Dta1xxGenSetControl0Reg(pBase, Val);
}

// Get DblBuf Enable
static __inline UInt Dta1xxGenDblBufEnableGet(Int* pBase) {
	return (Dta1xxGenGetControl0Reg(pBase) & DTA1XX_GEN_CONTROL0_DBLBUF ? 1 : 0);
}

// Set DblBuf Enable
static __inline void Dta1xxGenDblBufEnableSet(Int* pBase, UInt Enable) {
	UInt Val;
	Val = Dta1xxGenGetControl0Reg(pBase);
	if (Enable) Val = Val | DTA1XX_GEN_CONTROL0_DBLBUF;
	else		Val = Val & ~DTA1XX_GEN_CONTROL0_DBLBUF;
	Dta1xxGenSetControl0Reg(pBase, Val);
}


//-.-.-.-.-.-.-.-.-.-.-.-.- Control1 Registers: Access Functions -.-.-.-.-.-.-.-.-.-.-.-.-
#define DTA1XX_GEN_CONTROL1_WATCHDOG		0x00000001

static __inline UInt  Dta1xxGenGetControl1Reg(Int* pBase) {
	UInt ret;
	READ_LONG(ret, pBase, DTA1XX_GEN_REG_CONTROL1);
	return ret;
}
static __inline void Dta1xxGenSetControl1Reg(Int* pBase, UInt Val) {
	WRITE_LONG(Val, pBase, DTA1XX_GEN_REG_CONTROL1);
}


// Toggle watchdog
static __inline void Dta1xxGenToggleWatchdog(Int* pBase) {
	UInt Val;
	Val = Dta1xxGenGetControl1Reg(pBase);
	if (Val & DTA1XX_GEN_CONTROL1_WATCHDOG) Val = Val & ~DTA1XX_GEN_CONTROL1_WATCHDOG;
	else Val = Val | DTA1XX_GEN_CONTROL1_WATCHDOG;
	Dta1xxGenSetControl1Reg(pBase, Val);
}

// Pulse watchdog 1->0->1 or 0->1->0
static __inline void Dta1xxGenPulseWatchdog(Int* pBase) {
	UInt Val;
	Val = Dta1xxGenGetControl1Reg(pBase);

	// Toggle watchdog bit
	if (Val & DTA1XX_GEN_CONTROL1_WATCHDOG) {
		Dta1xxGenSetControl1Reg(pBase, Val & ~DTA1XX_GEN_CONTROL1_WATCHDOG);
	} else {
		Dta1xxGenSetControl1Reg(pBase, Val | DTA1XX_GEN_CONTROL1_WATCHDOG);
	}
	// Reset watchdog bit
	Dta1xxGenSetControl1Reg(pBase, Val);
}
#endif // __DTA1XX_REGS_H
