//#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* GenRegs.h *#*#*#*#*#*#*#*#*# (C) 2000-2005 DEKTEC

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- License -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// This program is free software; You can freely use/modify this source code in any way
// you desire, including for commercial applications.
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	MG	2003.04.23	Added functions for writing to PE and PRE bit-field
//	MG	16-01-2003	Correct definitions of some DTA1XX_GENCTRL_ constants
//	MG	25-11-2002	Created	

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ General register offset +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=
#define DTA1XX_GEN_REG_GENCTRL			0x0000
#define DTA1XX_GEN_REG_GENSTATUS		0x0004
#define DTA1XX_GEN_REG_PROGRAMMING		0x0008
#define DTA1XX_GEN_REG_REFCLKCNT		0x000C

//-.-.-.-.-.-.-.-.-.-.-.-.-.- General register bit/field masks -.-.-.-.-.-.-.-.-.-.-.-.-.-

// General control register
#define DTA1XX_GENCTRL_PE				0x00000001
#define DTA1XX_GENCTRL_PRE				0x00000002
#define DTA1XX_GENCTRL_RESET			0x00000004
#define DTA1XX_GENCTRL_PERINT_EN		0x00000010
#define DTA1XX_GENCTRL_PERINT_VAL		0x000000E0
#define DTA1XX_GENCTRL_ALTERAREV		0x0000FF00
#define DTA1XX_GENCTRL_TYPENUM			0x00FF0000
#define DTA1XX_GENCTRL_LEDCTRL			0x01000000
#define DTA1XX_GENCTRL_GREENLED			0x02000000
#define DTA1XX_GENCTRL_REDLED			0x04000000

// General status register
#define DTA1XX_GENSTAT_PERINT			0x00000010

// Programming register
#define DTA1XX_PROG_TRST				0x00000001
#define DTA1XX_PROG_TDO					0x00000002
#define DTA1XX_PROG_TMS					0x00000004
#define DTA1XX_PROG_TCK					0x00000008
#define DTA1XX_PROG_TDI					0x00000010
#define DTA1XX_PROG_PRGEPC				0x00000020


//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ General register operations +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxGenGetGenCtrlReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxGenGetGenCtrlReg(char* pBase)
{
	int32_t Val;
	READ_LONG(Val, pBase, DTA1XX_GEN_REG_GENCTRL);
	return Val;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxGenSetGenCtrlReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxGenSetGenCtrlReg(char* pBase, int32_t Val)
{
	WRITE_LONG(Val, pBase, DTA1XX_GEN_REG_GENCTRL);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxGenCtrlRegSetPE -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxGenCtrlRegSetPE(char* pBase, int32_t nEna)
{
	int32_t Val = Dta1xxGenGetGenCtrlReg(pBase);
	if (nEna != 0) {
        Val |= DTA1XX_GENCTRL_PE;
	} else {
		Val &= ~DTA1XX_GENCTRL_PE;
	}
	Dta1xxGenSetGenCtrlReg(pBase, Val);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxGenCtrlRegGetPE -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int Dta1xxGenCtrlRegGetPE(char* pBase)
{
	int32_t Val = Dta1xxGenGetGenCtrlReg(pBase);
	if ((Val & DTA1XX_GENCTRL_PE) != 0) {
		return 1;
	} else {
		return 0;
	}
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxGenCtrlRegSetPRE -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxGenCtrlRegSetPRE(char* pBase, int32_t nEna)
{
	int32_t Val = Dta1xxGenGetGenCtrlReg(pBase);
	if (nEna != 0) {
        Val |= DTA1XX_GENCTRL_PRE;
	} else {
		Val &= ~DTA1XX_GENCTRL_PRE;
	}
	Dta1xxGenSetGenCtrlReg(pBase, Val);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxGenCtrlRegGetPRE -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxGenCtrlRegGetPRE(char* pBase)
{
	int32_t Val = Dta1xxGenGetGenCtrlReg(pBase);
	if ((Val & DTA1XX_GENCTRL_PRE) != 0) {
		return 1;
	} else {
		return 0;
	}
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxGenCtrlRegReset -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxGenCtrlRegReset(char* pBase)
{
	int32_t Val = Dta1xxGenGetGenCtrlReg(pBase);
	Val |= DTA1XX_GENCTRL_RESET;
	Dta1xxGenSetGenCtrlReg(pBase, Val);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxGenCtrlRegGetFirmwareRev -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxGenCtrlRegGetFirmwareRev(char* pBase)
{
	int32_t Val = Dta1xxGenGetGenCtrlReg(pBase);
	Val &= DTA1XX_GENCTRL_ALTERAREV;
	return ( (Val>>8) & 0xFF );
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxGenCtrlRegGetPerIntEn -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxGenCtrlRegGetPerIntEn(char* pBase)
{
	int32_t Val = Dta1xxGenGetGenCtrlReg(pBase);
	if ((Val & DTA1XX_GENCTRL_PERINT_EN) != 0) {
		return 1;
	} else {
		return 0;
	}
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxGenCtrlRegSetPerIntEn -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxGenCtrlRegSetPerIntEn(char* pBase, int32_t nEna)
{
	int32_t Val = Dta1xxGenGetGenCtrlReg(pBase);
	if (nEna != 0) {
        Val |= DTA1XX_GENCTRL_PERINT_EN;
	} else {
		Val &= ~DTA1XX_GENCTRL_PERINT_EN;
	}
	Dta1xxGenSetGenCtrlReg(pBase, Val);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-. Dta1xxGenCtrlRegGetPerIntVal -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int Dta1xxGenCtrlRegGetPerIntVal(char* pBase)
{
	int32_t Val = Dta1xxGenGetGenCtrlReg(pBase);
	return (Val & DTA1XX_GENCTRL_PERINT_VAL)>>5;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxGenCtrlRegSetPerIntVal -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxGenCtrlRegSetPerIntVal(char* pBase, int32_t nIntVal)
{
	int32_t Val = Dta1xxGenGetGenCtrlReg(pBase);
	Val &= ~DTA1XX_GENCTRL_PERINT_VAL;
	Val |= (nIntVal & 0x7) << 5;
	Dta1xxGenSetGenCtrlReg(pBase, Val);
}


//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxGenLedControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxGenLedControl(char* pBase, int32_t nLedCtrl)
{
	int32_t Val = Dta1xxGenGetGenCtrlReg(pBase);
	Val &= ~(DTA1XX_GENCTRL_LEDCTRL | DTA1XX_GENCTRL_GREENLED | DTA1XX_GENCTRL_REDLED);
	Val |= (nLedCtrl & 0x7) << 24;
	Dta1xxGenSetGenCtrlReg(pBase, Val);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxGenGetGenStatusReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int Dta1xxGenGetGenStatusReg(char* pBase)
{
	int32_t Val;
	READ_LONG(Val, pBase, DTA1XX_GEN_REG_GENSTATUS);
	return Val;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxGenSetGenStatusReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static void Dta1xxGenSetGenStatusReg(char* pBase, int32_t Val)
{
	WRITE_LONG(Val, pBase, DTA1XX_GEN_REG_GENSTATUS);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxGenIsPerIntSet -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int Dta1xxGenStatusRegGetPerInt(char* pBase)
{
	int32_t Val = Dta1xxGenGetGenStatusReg(pBase);
	
	if ((Val & DTA1XX_GENSTAT_PERINT) != 0) {
		return 1;
	} else {
        return 0;
	}
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxGenStatusRegClrPerInt -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxGenStatusRegClrPerInt(char* pBase)
{
	Dta1xxGenSetGenStatusReg(pBase, DTA1XX_GENSTAT_PERINT);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxGenSetRefClkCntReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxGenSetRefClkCntReg(char* pBase, int32_t Val)
{
	WRITE_LONG(Val, pBase, DTA1XX_GEN_REG_REFCLKCNT);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxGenGetRefClkCntReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int Dta1xxGenGetRefClkCntReg(char* pBase)
{
	int32_t Val;
	READ_LONG(Val, pBase, DTA1XX_GEN_REG_REFCLKCNT);
	return Val;
}
