//#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*# RxRegs.h #*#*#*#*#*#*#*#*#* (C) 2000-2005 DEKTEC

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- License -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// This program is free software; You can freely use/modify this source code in any way
// you desire, including for commercial applications.
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	MG	15-11-2002	Created

#ifndef __RX_REGS_H
#define __RX_REGS_H

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Dta1xxRx +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
//
// Register block for a RECEIVE channel.
//
// Supports: DTA-120, DTA-122, DTA-140
//
typedef struct _Dta1xxRx {

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- General-Control register -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	//
	// On DTA-140, these fields are moved to the generic register block.
	//
	struct {
	volatile UInt  m_PE				: 1;	// Serial EEPROM Program Enable
	volatile UInt  m_PRE			: 1;	// Serial EEPROM Protect Register Enable
	volatile UInt  m_Reset			: 1;	// Soft reset of DTA 100 circuitry
			 UInt  m_Reserved1		: 5;	//
			 UInt  m_FirmwRev		: 8;	// Firmware Revision
			 UInt  m_TypeNum		: 8;	// Board Type Number
	}  m_GenControl;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Receive-Control register -.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	struct {								
	volatile UInt  m_RxMode			: 2;	//  Receive Mode: St188/St204/StMp2/StRaw
	volatile UInt  m_AsiInv			: 2;	//  Invert ASI-input control (ASI cards)
	volatile UInt  m_Equalise		: 1;	//  Equaliser control (ASI cards)
	volatile UInt  m_RxCtrl			: 1;	//  Receive Control: Idle/Rcv
			 UInt  m_Reserved2		: 1;	//
	volatile UInt  m_RxTimeStamp	: 1;	//  Insert Time Stamps
	volatile UInt  m_PerIntEn		: 1;	//  Periodic-Interrupt Enable
	volatile UInt  m_OvfIntEn		: 1;	//  Input-FIFO Overflow Interrupt Enable
	volatile UInt  m_SyncIntEn		: 1;	//  Synchronisation-Error Interrupt Enable
			 UInt  m_Reserved3		: 1;	//
	volatile UInt  m_ThrIntEn		: 1;	//  Threshold-Crossed Interrupt Enable
	volatile UInt  m_RateOvfIntEn	: 1;	//  Input-Rate Overflow Interrupt Enable
			 UInt  m_Reserved4		: 3;	//
	volatile UInt  m_EnaPwr			: 1;	//  Enable Power to target adapter
			 UInt  m_Reserved5		: 1;	//
	volatile UInt  m_LedControl		: 1;	//  Take over LED Control
	volatile UInt  m_LedGreen		: 1;	//  State of Green LED (if LedControl is set)
	volatile UInt  m_LedRed			: 1;	//  State of Red LED (if LedControl is set)
	volatile UInt  m_RxClrFifo		: 1;	//  Clear Receive FIFO (write only)
	volatile UInt  m_RxRst			: 1;	//  Reset Receive Channel (write only)
	volatile UInt  m_SdiMode		: 1;	//  SDI mode enable/disbale
	volatile UInt  m_Sdi10Bit		: 1;	//  SDI 10-bit mode
	}  m_RxControl;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Receive-Status register -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	// Status register is a union.
	// WATCH OUT: DO NOT clear interrupts by something like p->Fields.m_PerInt = 1;
	// THIS WOULD CLEAR ALL STATUS FLAGS!!
	union {
	struct {
	volatile UInt  m_PckSize		: 2;	//  Packet size: Rx188/Rx204/RxInv
	volatile UInt  m_NumInv			: 2;	//  Number of invalid bytes per packet
	volatile UInt  m_SdramSize		: 4;	//  SDRAM Size
	volatile UInt  m_PerInt			: 1;	//  Periodic Interrupt
	volatile UInt  m_OvfInt			: 1;	//  Input-FIFO Overflow Interrupt
	volatile UInt  m_SyncInt		: 1;	//  Synchronisation-Error Interrupt
			 UInt  m_Reserved1		: 1;	//
	volatile UInt  m_ThrInt			: 1;	//  Threshold-Crossed Interrupt
	volatile UInt  m_RateOvfInt		: 1;	//  Input-Rate Overflow Interrupt
	volatile UInt  m_ClkDet			: 1;	//  Receive-Clock Detected (DTA-120: Asi CD)
	volatile UInt  m_AsiLock		: 1;	//  DVB/ASI PLL locked to input signal
	volatile UInt  m_RateOk			: 1;	//  Input rate above minimum input rate
	volatile UInt  m_AsiInv			: 1;	//  DVB/ASI-Input Invert - Status
	}				 Fields;				// For access to individual fields in RxStatus

			 UInt32  All;					// For access to RxStatus in one go
	} m_RxStatus;

	volatile UInt32  m_Reserved[3];			// Not used

	volatile UInt32  m_FifoLoad;			// FIFO-Load register

	struct {								// Diagnostics register
			 UInt  m_Reserved1		: 8;	//
	volatile UInt  m_LoopBack		: 1;	//	Loop-back mode
	volatile UInt  m_CdcRateOvf		: 1;	//	Clock-Domain-Converter Rate Overflow
	volatile UInt  m_RfRateOvf		: 1;	//	Receive-FIFO Rate Overflow
	volatile UInt  m_SdFull			: 1;	//	SDRAM-Full Flag
			 UInt  m_Reserved2		: 7;	//

	// ONLY FOR: DTA-120, Firmware Revision < 4 (Later revs: Programming register is used)
	volatile UInt  m_TDO			: 1;	// Status of TDO pin
	volatile UInt  m_TMS			: 1;	// Control of TMS pin
	volatile UInt  m_TCK			: 1;	// Control of TCK pin
	volatile UInt  m_TDI			: 1;	// Control of TDI pin
	volatile UInt  m_PrgEpc			: 1;	// Enable access to programming i/f of EPC2

	}  m_Diag;

	volatile UInt32  m_LbData;				// Data written to Input FIFO (8 sign. bits)

	struct {								// Threshold-Control register
	volatile UInt  m_UpdBusy		: 1;	//  Threshold-Update process busy
	volatile UInt  m_CmpB			: 1;	//  Output of comparator B
	volatile UInt  m_CmpA			: 1;	//  Output of comparator A
	volatile UInt  m_AorB			: 1;	//  Update threshold A or threshold B
			 UInt  m_Reserved1		: 4;	//
	volatile UInt  m_ThreshA		: 8;	//  Threshold A
	volatile UInt  m_ThreshB		: 8;	//  Threshold B
	}  m_ThreshControl;

	volatile UInt32  m_PckCnt;				// Packet Count (DTA-122)
	volatile UInt32  m_ClkCnt;				// Clock Count (DTA-122)
	volatile UInt32  m_ValidCnt;			// Valid Count (DTA-120, DTA-122)
	volatile UInt32  m_CodeViolCnt;			// Code-Violation Count (DTA-120)

	volatile UInt32  m_Reserved2[1];		// Not used

	volatile UInt32  m_LockFilter;			// Control value for Lock Filter (DTA-120)

	volatile UInt32  m_FifoData[16];		// Transport-stream data: 4 bytes at a time
} Dta1xxRx;

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ RX register offset +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

#define DTA1XX_RX_REG_GENCTRL			0x0000
#define DTA1XX_RX_REG_RXCTRL			0x0004
#define DTA1XX_RX_REG_RXSTATUS			0x0008
#define DTA1XX_RX_REG_FIFOLOAD			0x0018
#define DTA1XX_RX_REG_DIAG				0x001C
#define DTA1XX_RX_REG_LOOPBKDATA		0x0020
#define DTA1XX_RX_REG_THRESHCTRL		0x0024
#define DTA1XX_RX_REG_PCKCOUNT			0x0028
#define DTA1XX_RX_REG_CLKCOUNT			0x002C
#define DTA1XX_RX_REG_ValIDCOUNT		0x0030
#define DTA1XX_RX_REG_VIOLCOUNT			0x0034

#define DTA1XX_RX_REG_DMA_DESC_PTR		0x0044
#define DTA1XX_RX_REG_DMA_COMMAND_STAT	0x0048
#define DTA1XX_RX_REG_DMA_DAC			0x004C
#define DTA1XX_RX_REG_DMA_PCI_ADDR		0x0050
#define DTA1XX_RX_REG_DMA_LOCAL_ADDR	0x0054
#define DTA1XX_RX_REG_DMA_COUNT			0x0058
#define DTA1XX_RX_REG_DMA_NXTDESC_PTR	0x005C
#define DTA1XX_RX_REG_DMA_PCI_ADDRH		0x0060

#define DTA1XX_RX_REG_FIFO_FIRST		0x0040
#define DTA1XX_RX_REG_FIFO_LAST			0x007C

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Rx register bit/field masks -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

// Rx general control
#define DTA1XX_RX_GENCTRL_PE			0x00000001
#define DTA1XX_RX_GENCTRL_PRE			0x00000002
#define DTA1XX_RX_GENCTRL_RESET			0x00000004
#define DTA1XX_RX_GENCTRL_ALTERAREV		0x0000FF00
#define DTA1XX_RX_GENCTRL_TYPENUM		0x00FF0000

// Rx control register
#define DTA1XX_RXCTRL_RXMODE			0x00000003
#define DTA1XX_RXCTRL_ASIINV			0x0000000C
#define DTA1XX_RXCTRL_EQUALISE			0x00000010
#define DTA1XX_RXCTRL_RXCTRL			0x00000020
#define DTA1XX_RXCTRL_TIMESTAMP         0x00000080
#define DTA1XX_RXCTRL_PERINT_EN			0x00000100
#define DTA1XX_RXCTRL_OVFINT_EN			0x00000200
#define DTA1XX_RXCTRL_SYNCINT_EN		0x00000400
#define DTA1XX_RXCTRL_THRINT_EN			0x00001000
#define DTA1XX_RXCTRL_RATEOVFINT_EN		0x00002000
#define DTA1XX_RXCTRL_ENAPWR			0x00020000
#define DTA1XX_RXCTRL_LEDCTRL			0x00080000
#define DTA1XX_RXCTRL_GREENLED			0x00100000
#define DTA1XX_RXCTRL_REDLED			0x00200000
#define DTA1XX_RXCTRL_CLEARFIFO			0x00400000
#define DTA1XX_RXCTRL_RXRESET			0x00800000
#define DTA1XX_RXCTRL_SDI_MODE			0x01000000

// Rx status register
#define DTA1XX_RXSTAT_PCKSIZE			0x00000003
#define DTA1XX_RXSTAT_NUMINV			0x0000000C
#define DTA1XX_RXSTAT_SDRAMSIZE			0x000000F0
#define DTA1XX_RXSTAT_PERINT			0x00000100
#define DTA1XX_RXSTAT_OVFINT			0x00000200
#define DTA1XX_RXSTAT_SYNCINT			0x00000400
#define DTA1XX_RXSTAT_THRINT			0x00001000
#define DTA1XX_RXSTAT_RATEOVFINT		0x00002000
#define DTA1XX_RXSTAT_ASICD				0x00004000
#define DTA1XX_RXSTAT_ASILOCK			0x00008000
#define DTA1XX_RXSTAT_RATEOK			0x00010000
#define DTA1XX_RXSTAT_ASI_INV			0x00020000

// Rx diagnostic register
#define DTA1XX_RXDIAG_LOOPBACK			0x00000100
#define DTA1XX_RXDIAG_RFRATEOVF			0x00000400
#define DTA1XX_RXDIAG_SDFULL			0x00000800

// Rx threshold control register
#define DTA1XX_RXTHRCTRL_UPD_DONE		0x00000001
#define DTA1XX_RXTHRCTRL_CMPB			0x00000002
#define DTA1XX_RXTHRCTRL_CMPA			0x00000004
#define DTA1XX_RXTHRCTRL_A_N_B			0x00000008
#define DTA1XX_RXTHRCTRL_THRESHA		0x0000FF00
#define DTA1XX_RXTHRCTRL_THRESHB		0x00FF0000

// DMA Command/Status register
// NOTE: other fields in this register follow PCI905X definitions:
//			PCI905X_DMACSR_ENABLE, PCI905X_DMACSR_START, PCI905X_DMACSR_ABORT,
//			PCI905X_DMACSR_CLEARINT, PCI905X_DMACSR_DONE
#define DTA1XX_DMACSR_INTACT		0x00000008		// DMA Channel Interrupt Active

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Rx register operations +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxGetGenCtrlReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int Dta1xxRxGetGenCtrlReg(char* pBase)
{
	int32_t Val;
	READ_LONG(Val, pBase, DTA1XX_RX_REG_GENCTRL);
	return Val;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxSetGenCtrlReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxRxSetGenCtrlReg(char* pBase, int32_t Val)
{
	WRITE_LONG(Val, pBase, DTA1XX_RX_REG_GENCTRL);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxGenCtrlRegSetPE -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxRxGenCtrlRegSetPE(char* pBase, int32_t nEna)
{
	int32_t Val = Dta1xxRxGetGenCtrlReg(pBase);
	if ( nEna==1 ) {
		Val |= DTA1XX_RX_GENCTRL_PE;
	} else {
		Val &= ~DTA1XX_RX_GENCTRL_PE;
	}
	Dta1xxRxSetGenCtrlReg(pBase, Val);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxGenCtrlRegSetPRE -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxRxGenCtrlRegSetPRE(char* pBase, int32_t nEna)
{
	int32_t Val = Dta1xxRxGetGenCtrlReg(pBase);
	if ( nEna==1 ) {
		Val |= DTA1XX_RX_GENCTRL_PRE;
	} else {
		Val &= ~DTA1XX_RX_GENCTRL_PRE;
	}
	Dta1xxRxSetGenCtrlReg(pBase, Val);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxGenCtrlRegReset -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxRxGenCtrlRegReset(char* pBase)
{
	int32_t Val = Dta1xxRxGetGenCtrlReg(pBase);
	Val |= DTA1XX_RX_GENCTRL_RESET;
	Dta1xxRxSetGenCtrlReg(pBase, Val);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxGenCtrlRegGetFirmwareRev -.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int Dta1xxRxGenCtrlRegGetFirmwareRev(char* pBase)
{
	int32_t Val = Dta1xxRxGetGenCtrlReg(pBase);
	Val &= DTA1XX_RX_GENCTRL_ALTERAREV;
	return (Val>>8);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxGenCtrlRegGetTypeNumber -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxRxGenCtrlRegGetTypeNumber(char* pBase)
{
	int32_t Val = Dta1xxRxGetGenCtrlReg(pBase);
	Val &= DTA1XX_RX_GENCTRL_TYPENUM;
	return (Val>>16);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxGetRxCtrlReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int Dta1xxRxGetRxCtrlReg(char* pBase)
{
	int32_t Val;
	READ_LONG(Val, pBase, DTA1XX_RX_REG_RXCTRL);
	return Val;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxSetRxCtrlReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxRxSetRxCtrlReg(char* pBase, int32_t Val)
{
	WRITE_LONG(Val, pBase, DTA1XX_RX_REG_RXCTRL);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxCtrlRegGetRxMode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int Dta1xxRxCtrlRegGetRxMode(char* pBase)
{
	int32_t Val = Dta1xxRxGetRxCtrlReg(pBase);
	Val &= DTA1XX_RXCTRL_RXMODE;
	return Val;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxCtrlRegSetRxMode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxRxCtrlRegSetRxMode(char* pBase, int32_t nMode)
{
	int32_t Val = Dta1xxRxGetRxCtrlReg(pBase);
	Val &= ~DTA1XX_RXCTRL_RXMODE;
	Val |= (nMode&0x3);
	Dta1xxRxSetRxCtrlReg(pBase, Val);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxCtrlRegGetTimeStamping -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxRxCtrlRegGetTimeStamping(char* pBase, int32_t nEna)
{
        int32_t Val = Dta1xxRxGetRxCtrlReg(pBase);
        if ( 0!=(Val&DTA1XX_RXCTRL_TIMESTAMP) ) {
                return 1;
        } else {
                return 0;
        }
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxCtrlRegSetTimeStamping -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxRxCtrlRegSetTimeStamping(char* pBase, int32_t nEna)
{
        int32_t Val = Dta1xxRxGetRxCtrlReg(pBase);
        if ( 0!=nEna ) {
                Val |= DTA1XX_RXCTRL_TIMESTAMP;
        } else {
                Val &= ~DTA1XX_RXCTRL_TIMESTAMP;
        }
        Dta1xxRxSetRxCtrlReg(pBase, Val);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxCtrlRegGetAsiInv -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int Dta1xxRxCtrlRegGetAsiInv(char* pBase)
{
	int32_t Val = Dta1xxRxGetRxCtrlReg(pBase);
	Val &= DTA1XX_RXCTRL_ASIINV;
	return (Val>>2);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxCtrlRegSetAsiInv -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxRxCtrlRegSetAsiInv(char* pBase, int32_t nInvCtrl)
{
	int32_t Val = Dta1xxRxGetRxCtrlReg(pBase);
	Val &= ~DTA1XX_RXCTRL_ASIINV;
	Val |= ((nInvCtrl&0x3)<<2);
	Dta1xxRxSetRxCtrlReg(pBase, Val);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxCtrlRegGetEqualise -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxRxCtrlRegGetEqualise(char* pBase)
{
	int32_t Val = Dta1xxRxGetRxCtrlReg(pBase);
	if ( 0!=(Val & DTA1XX_RXCTRL_EQUALISE) ) {
		return 1;
	} else {
		return 0;
	}
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxCtrlRegSetEqualise -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxRxCtrlRegSetEqualise(char* pBase, int32_t nEna)
{
	int32_t Val = Dta1xxRxGetRxCtrlReg(pBase);
	if ( 0!=nEna ) {
		Val |= DTA1XX_RXCTRL_EQUALISE;
	} else {
		Val &= ~DTA1XX_RXCTRL_EQUALISE;
	}
	Dta1xxRxSetRxCtrlReg(pBase, Val);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxCtrlRegGetRxCtrl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int Dta1xxRxCtrlRegGetRxCtrl(char* pBase)
{
	int32_t Val = Dta1xxRxGetRxCtrlReg(pBase);
	if ( 0!=(Val & DTA1XX_RXCTRL_RXCTRL) ) {
		return 1;
	} else {
		return 0;
	}
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxCtrlRegSetRxCtrl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxRxCtrlRegSetRxCtrl(char* pBase, int32_t nEna)
{
	int32_t Val = Dta1xxRxGetRxCtrlReg(pBase);
	if ( 0!=nEna ) {
		Val |= DTA1XX_RXCTRL_RXCTRL;
	} else {
		Val &= ~DTA1XX_RXCTRL_RXCTRL;
	}
	Dta1xxRxSetRxCtrlReg(pBase, Val);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxCtrlRegGetPerIntEn -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxRxCtrlRegGetPerIntEn(char* pBase)
{
	int32_t Val = Dta1xxRxGetRxCtrlReg(pBase);
	if ( 0!=(Val & DTA1XX_RXCTRL_PERINT_EN) ) {
		return 1;
	} else {
		return 0;
	}
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxCtrlRegSetPerIntEn -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxRxCtrlRegSetPerIntEn(char* pBase, int32_t nEna)
{
	int32_t Val = Dta1xxRxGetRxCtrlReg(pBase);
	if ( 0!=nEna ) {
		Val |= DTA1XX_RXCTRL_PERINT_EN;
	} else {
		Val &= ~DTA1XX_RXCTRL_PERINT_EN;
	}
	Dta1xxRxSetRxCtrlReg(pBase, Val);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxCtrlRegGetOvfIntEn -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxRxCtrlRegGetOvfIntEn(char* pBase)
{
	int32_t Val = Dta1xxRxGetRxCtrlReg(pBase);
	if ( 0!=(Val & DTA1XX_RXCTRL_OVFINT_EN) ) {
		return 1;
	} else {
		return 0;
	}
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxCtrlRegSetOvfIntEn -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxRxCtrlRegSetOvfIntEn(char* pBase, int32_t nEna)
{
	int32_t Val = Dta1xxRxGetRxCtrlReg(pBase);
	if ( 0!=nEna ) {
		Val |= DTA1XX_RXCTRL_OVFINT_EN;
	} else {
		Val &= ~DTA1XX_RXCTRL_OVFINT_EN;
	}
	Dta1xxRxSetRxCtrlReg(pBase, Val);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxCtrlRegGetSyncIntEn -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxRxCtrlRegGetSyncIntEn(char* pBase)
{
	int32_t Val = Dta1xxRxGetRxCtrlReg(pBase);
	if ( 0!=(Val & DTA1XX_RXCTRL_SYNCINT_EN) ) {
		return 1;
	} else {
		return 0;
	}
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxCtrlRegSetSyncIntEn -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxRxCtrlRegSetSyncIntEn(char* pBase, int32_t nEna)
{
	int32_t Val = Dta1xxRxGetRxCtrlReg(pBase);
	if ( 0!=nEna ) {
		Val |= DTA1XX_RXCTRL_SYNCINT_EN;
	} else {
		Val &= ~DTA1XX_RXCTRL_SYNCINT_EN;
	}
	Dta1xxRxSetRxCtrlReg(pBase, Val);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxCtrlRegGetThrIntEn -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxRxCtrlRegGetThrIntEn(char* pBase)
{
	int32_t Val = Dta1xxRxGetRxCtrlReg(pBase);
	if ( 0!=(Val & DTA1XX_RXCTRL_THRINT_EN) ) {
		return 1;
	} else {
		return 0;
	}
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxCtrlRegSetThrIntEn -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxRxCtrlRegSetThrIntEn(char* pBase, int32_t nEna)
{
	int32_t Val = Dta1xxRxGetRxCtrlReg(pBase);
	if ( 0!=nEna ) {
		Val |= DTA1XX_RXCTRL_THRINT_EN;
	} else {
		Val &= ~DTA1XX_RXCTRL_THRINT_EN;
	}
	Dta1xxRxSetRxCtrlReg(pBase, Val);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxCtrlRegGetRateOvfIntEn -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxRxCtrlRegGetRateOvfIntEn(char* pBase)
{
	int32_t Val = Dta1xxRxGetRxCtrlReg(pBase);
	if ( 0!=(Val & DTA1XX_RXCTRL_RATEOVFINT_EN) ) {
		return 1;
	} else {
		return 0;
	}
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxCtrlRegSetRateOvfIntEn -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxRxCtrlRegSetRateOvfIntEn(char* pBase, int32_t nEna)
{
	int32_t Val = Dta1xxRxGetRxCtrlReg(pBase);
	if ( 0!=nEna ) {
		Val |= DTA1XX_RXCTRL_RATEOVFINT_EN;
	} else {
		Val &= ~DTA1XX_RXCTRL_RATEOVFINT_EN;
	}
	Dta1xxRxSetRxCtrlReg(pBase, Val);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxCtrlRegGetEnaPwr -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int Dta1xxRxCtrlRegGetEnaPwr(char* pBase)
{
	int32_t Val = Dta1xxRxGetRxCtrlReg(pBase);
	if ( 0!=(Val & DTA1XX_RXCTRL_ENAPWR) ) {
		return 1;
	} else {
		return 0;
	}
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxCtrlRegSetEnaPwr -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxRxCtrlRegSetEnaPwr(char* pBase, int32_t nEna)
{
	int32_t Val = Dta1xxRxGetRxCtrlReg(pBase);
	if ( 0!=nEna ) {
		Val |= DTA1XX_RXCTRL_ENAPWR;
	} else {
		Val &= ~DTA1XX_RXCTRL_ENAPWR;
	}
	Dta1xxRxSetRxCtrlReg(pBase, Val);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxLedControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxRxLedControl(char* pBase, int32_t nLedCtrl)
{
	int32_t Val = Dta1xxRxGetRxCtrlReg(pBase);
	Val &= ~(DTA1XX_RXCTRL_LEDCTRL | DTA1XX_RXCTRL_GREENLED | DTA1XX_RXCTRL_REDLED);
	Val |= (nLedCtrl & 0x7) << 19;
	Dta1xxRxSetRxCtrlReg(pBase, Val);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxClrFifo -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxRxClrFifo(char* pBase)
{
	int32_t Val = Dta1xxRxGetRxCtrlReg(pBase);
	Val |= DTA1XX_RXCTRL_CLEARFIFO;
	Dta1xxRxSetRxCtrlReg(pBase, Val);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxChannelReset -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxRxChannelReset(char* pBase)
{
	int32_t Val = Dta1xxRxGetRxCtrlReg(pBase);
	Val |= DTA1XX_RXCTRL_RXRESET;
	Dta1xxRxSetRxCtrlReg(pBase, Val);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxGetRxStatusReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxRxGetRxStatusReg(char* pBase)
{
	int32_t Val;
	READ_LONG(Val, pBase, DTA1XX_RX_REG_RXSTATUS);
	return Val;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxSetRxStatusReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxRxSetRxStatusReg(char* pBase, int32_t Val)
{
	WRITE_LONG(Val, pBase, DTA1XX_RX_REG_RXSTATUS);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxStatusRegGetPckSize -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxRxStatusRegGetPckSize(char* pBase)
{
	int32_t Val = Dta1xxRxGetRxStatusReg(pBase);
	Val &= DTA1XX_RXSTAT_PCKSIZE;
	return Val;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxStatusRegGetNumInv -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxRxStatusRegGetNumInv(char* pBase)
{
	int32_t Val = Dta1xxRxGetRxStatusReg(pBase);
	Val &= DTA1XX_RXSTAT_NUMINV;
	return ( Val>>2 );
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxStatusRegGetSdramSize -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int Dta1xxRxStatusRegGetSdramSize(char* pBase)
{
	int32_t Val = Dta1xxRxGetRxStatusReg(pBase);
	Val &= DTA1XX_RXSTAT_SDRAMSIZE;
	return ( Val>>4 );
}


//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxStatusRegGetPerInt -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxRxStatusRegGetPerInt(char* pBase)
{
	int32_t Val = Dta1xxRxGetRxStatusReg(pBase);
	if ( 0!=(Val & DTA1XX_RXSTAT_PERINT) ) {
		return 1;
	} else {
		return 0;
	}
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxStatusRegClrPerInt -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxRxStatusRegClrPerInt(char* pBase)
{
	Dta1xxRxSetRxStatusReg(pBase, DTA1XX_RXSTAT_PERINT);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxStatusRegGetOvfInt -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxRxStatusRegGetOvfInt(char* pBase)
{
	int32_t Val = Dta1xxRxGetRxStatusReg(pBase);
	if ( 0!=(Val & DTA1XX_RXSTAT_OVFINT) ) {
		return 1;
	} else {
		return 0;
	}
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxStatusRegClrOvfInt -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxRxStatusRegClrOvfInt(char* pBase)
{
	Dta1xxRxSetRxStatusReg(pBase, DTA1XX_RXSTAT_OVFINT);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxStatusRegGetSyncInt -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxRxStatusRegGetSyncInt(char* pBase)
{
	int32_t Val = Dta1xxRxGetRxStatusReg(pBase);
	if ( 0!=(Val & DTA1XX_RXSTAT_SYNCINT) ) {
		return 1;
	} else {
		return 0;
	}
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxStatusRegClrSyncInt -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxRxStatusRegClrSyncInt(char* pBase)
{
	Dta1xxRxSetRxStatusReg(pBase, DTA1XX_RXSTAT_SYNCINT);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxStatusRegGetThrInt -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxRxStatusRegGetThrInt(char* pBase)
{
	int32_t Val = Dta1xxRxGetRxStatusReg(pBase);
	if ( 0!=(Val & DTA1XX_RXSTAT_THRINT) ) {
		return 1;
	} else {
		return 0;
	}
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxStatusRegClrThrInt -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxRxStatusRegClrThrInt(char* pBase)
{
	Dta1xxRxSetRxStatusReg(pBase, DTA1XX_RXSTAT_THRINT);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxStatusRegGetRateOvfInt -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxRxStatusRegGetRateOvfInt(char* pBase)
{
	int32_t Val = Dta1xxRxGetRxStatusReg(pBase);
	if ( 0!=(Val & DTA1XX_RXSTAT_RATEOVFINT) ) {
		return 1;
	} else {
		return 0;
	}
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxStatusRegClrRateOvfInt -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxRxStatusRegClrRateOvfInt(char* pBase)
{
	Dta1xxRxSetRxStatusReg(pBase, DTA1XX_RXSTAT_RATEOVFINT);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxStatusRegGetAsiCD -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int Dta1xxRxStatusRegGetAsiCD(char* pBase)
{
	int32_t Val = Dta1xxRxGetRxStatusReg(pBase);
	if ( 0!=(Val & DTA1XX_RXSTAT_ASICD) ) {
		return 1;
	} else {
		return 0;
	}
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxStatusRegGetAsiLock -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxRxStatusRegGetAsiLock(char* pBase)
{
	int32_t Val = Dta1xxRxGetRxStatusReg(pBase);
	if ( 0!=(Val & DTA1XX_RXSTAT_ASILOCK) ) {
		return 1;
	} else {
		return 0;
	}
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxStatusRegGetRateOk -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxRxStatusRegGetRateOk(char* pBase)
{
	int32_t Val = Dta1xxRxGetRxStatusReg(pBase);
	if ( 0!=(Val & DTA1XX_RXSTAT_RATEOK) ) {
		return 1;
	} else {
		return 0;
	}
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxStatusRegGetAsiInv -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxRxStatusRegGetAsiInv(char* pBase)
{
	int32_t Val = Dta1xxRxGetRxStatusReg(pBase);
	if ( 0!=(Val & DTA1XX_RXSTAT_ASI_INV) ) {
		return 1;
	} else {
		return 0;
	}
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxGetFifoLoad -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxRxGetFifoLoad(char* pBase)
{
	int32_t Val;
	READ_LONG(Val, pBase, DTA1XX_RX_REG_FIFOLOAD);
	return Val;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxGetDiagReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxRxGetDiagReg(char* pBase)
{
	int32_t Val;
	READ_LONG(Val, pBase, DTA1XX_RX_REG_DIAG);
	return Val;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxSetDiagReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxRxSetDiagReg(char* pBase, int32_t Val)
{
	WRITE_LONG(Val, pBase, DTA1XX_RX_REG_DIAG);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxDiagRegGetLoopBack -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxRxDiagRegGetLoopBack(char* pBase)
{
	int32_t Val = Dta1xxRxGetDiagReg(pBase);
	Val &= DTA1XX_RXDIAG_LOOPBACK;
	return ( Val >> 8);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxDiagRegSetLoopBack -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxRxDiagRegSetLoopBack(char* pBase, int32_t nEna)
{
	int32_t Val = Dta1xxRxGetDiagReg(pBase);
	if ( 0!=nEna ) {
        Val |= DTA1XX_RXDIAG_LOOPBACK;
	} else {
		Val &= ~DTA1XX_RXDIAG_LOOPBACK;
	}
	Dta1xxRxSetDiagReg(pBase, Val);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxDiagRegGetRfRateOvf -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxRxDiagRegGetRfRateOvf(char* pBase)
{
	int32_t Val = Dta1xxRxGetDiagReg(pBase);
	if ( 0!=(Val & DTA1XX_RXDIAG_RFRATEOVF) ) {
        return 1;
	} else {
		return 0;
	}
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxDiagRegGetSdFull -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int Dta1xxRxDiagRegGetSdFull(char* pBase)
{
	int32_t Val = Dta1xxRxGetDiagReg(pBase);
	if ( 0!=(Val & DTA1XX_RXDIAG_SDFULL) ) {
        return 1;
	} else {
		return 0;
	}
}


//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxSetLoopBackData -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxRxSetLoopBackData(char* pBase, int32_t Val)
{
	WRITE_LONG((Val&0xFF), pBase, DTA1XX_RX_REG_LOOPBKDATA);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxGetThreshCtrlReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int Dta1xxRxGetThreshCtrlReg(char* pBase)
{
	int32_t Val;
	READ_LONG(Val, pBase, DTA1XX_RX_REG_THRESHCTRL);
	return Val;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxSetThreshCtrlReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxRxSetThreshCtrlReg(char* pBase, int32_t Val)
{
	WRITE_LONG(Val, pBase, DTA1XX_RX_REG_THRESHCTRL);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxGetPckCountReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxRxGetPckCountReg(char* pBase)
{
	int32_t Val;
	READ_LONG(Val, pBase, DTA1XX_RX_REG_PCKCOUNT);
	return Val;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxGetClkCountReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxRxGetClkCountReg(char* pBase)
{
	int32_t Val;
	READ_LONG(Val, pBase, DTA1XX_RX_REG_CLKCOUNT);
	return Val;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxGetValidCountReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int Dta1xxRxGetValidCountReg(char* pBase)
{
	int32_t Val;
	READ_LONG(Val, pBase, DTA1XX_RX_REG_ValIDCOUNT);
	return Val;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxGetViolCountReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxRxGetViolCountReg(char* pBase)
{
	int32_t Val;
	READ_LONG(Val, pBase, DTA1XX_RX_REG_VIOLCOUNT);
	return Val;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxGetFifoDataReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int Dta1xxRxGetFifoDataReg(char* pBase)
{
	int32_t Val;
	READ_LONG(Val, pBase, DTA1XX_RX_REG_FIFO_FIRST);
	return Val;
}

// Rx Control: SDI mode
static inline void  Dta1xxRxCtrlRegSetSdiMode(char* pBase, unsigned SdiMode) {
    int32_t  Val = Dta1xxRxGetRxCtrlReg(pBase);
    if (SdiMode != 0)       Val |=  DTA1XX_RXCTRL_SDI_MODE;
    else                    Val &= ~DTA1XX_RXCTRL_SDI_MODE;
    Dta1xxRxSetRxCtrlReg(pBase, Val);
}   

#endif // __RX_REGS_H
