//#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*# TxRegs.h #*#*#*#*#*#*#*#*#* (C) 2000-2005 DEKTEC

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- License -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// This program is free software; You can freely use/modify this source code in any way
// you desire, including for commercial applications.
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	MG	15-11-2002	Created

#ifndef __TX_REGS_H
#define __TX_REGS_H

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Dta1xxTx +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
//
// Register block for a TRANSMISSION channel.
//
// Supports: DTA-100, DTA-102, DTA-140
//
typedef struct _Dta1xxTx {

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- General-Control register -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	//
	// On DTA-140, these fields are moved to the generic register block.
	//
	struct {								// General-Control register
	volatile UInt  m_PE				: 1;	//  Serial EEPROM Program Enable
	volatile UInt  m_PRE			: 1;	//  Serial EEPROM Protect Register Enable
	volatile UInt  m_Reset			: 1;	//  Soft reset of DTA 100 circuitry
			 UInt  m_Reserved1		: 5;	//
			 UInt  m_FirmwRev		: 8;	//  Firmware Revision
			 UInt  m_TypeNum		: 8;	//  Board Type Number
	}  m_GenControl;

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Transmit-Control register -.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	struct {								
	volatile UInt  m_TxMode			: 2;	//  Transmit Mode: 188/204/Add16/Raw
	volatile UInt  m_BurstMode		: 1;	//  Burst Mode: 1 = send packets in bursts
	volatile UInt  m_TxModeExt		: 1;	//	Transmit Mode extension: 192
	volatile UInt  m_PckStuff		: 1;	//  Null-Packet Stuffing On/Off
	volatile UInt  m_TxCtrl			: 2;	//  Transmit Control: Idle/Hold/Send
	volatile UInt  m_TxDis			: 1;	//  Disable LVDS outputs
	volatile UInt  m_PerIntEn		: 1;	//  Periodic-Interrupt Enable
	volatile UInt  m_UflIntEn		: 1;	//  Output-FIFO Underflow Interrupt Enable
	volatile UInt  m_SyncIntEn		: 1;	//  Synchronisation-Error Interrupt Enable
	volatile UInt  m_ShortIntEn		: 1;	//  Short-Circuit-Detected Interrupt Enable
	volatile UInt  m_ThrIntEn		: 1;	//  Threshold-Crossed Interrupt Enable
			 UInt  m_Reserved2		: 3;	//
	volatile UInt  m_UseExtClk		: 1;	//  Use External Clock input
	volatile UInt  m_EnaPwr			: 1;	//  Enable Power to target adapter
	volatile UInt  m_ShortErr		: 1;	//  Short-Circuit-Detected Error
	volatile UInt  m_LedControl		: 1;	//  Take over LED Control
	volatile UInt  m_LedGreen		: 1;	//  State of Green LED (if LedControl is set)
	volatile UInt  m_LedRed			: 1;	//  State of Red LED (if LedControl is set)
	volatile UInt  m_ClrFifo		: 1;	//  Clear FIFO (write only)
	volatile UInt  m_TxRst			: 1;	//  Reset Transmit Channel (write only)
	volatile UInt  m_TxInvAsi		: 1;	//  Invert DVB/ASI output signal
	volatile UInt  m_SdiMode		: 1;	//  SDI mode enable/disbale
	volatile UInt  m_Sdi10Bit		: 1;	//  SDI 10-bit mode
	volatile UInt  m_TxOutEn		: 1;	//  Transmit channel output enable
	}  m_TxControl;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Transmit-Status register -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	//
	// Status register is a union.
	// WATCH OUT: DO NOT clear interrupts by something like p->Fields.m_PerInt = 1;
	// THIS WOULD CLEAR ALL STATUS FLAGS!!
	union {
	struct {
	volatile UInt  m_FifoFilled		: 1;	//  Output-FIFO Filled flag
			 UInt  m_Reserved1		: 3;	//
	volatile UInt  m_SdramSize		: 4;	//  SDRAM Size
	volatile UInt  m_PerInt			: 1;	//  Periodic Interrupt
	volatile UInt  m_UflInt			: 1;	//  Output-FIFO Underflow Interrupt
	volatile UInt  m_SyncInt		: 1;	//  Synchronisation-Error Interrupt
	volatile UInt  m_ShortInt		: 1;	//  Short-Circuit-Detected Interrupt
	volatile UInt  m_ThrInt			: 1;	//  Threshold-Crossed Interrupt
	}				 Fields;				// For access to individual field
			 UInt32  All;					// For access to TxStatus in one go
	} m_TxStatus;

	volatile UInt32  m_TxClock;				// Transmit-Clock register
	volatile UInt32  m_ClkCnt;				// Clock-Count register
	volatile UInt32  m_FifoSize;			// FIFO-Size register
	volatile UInt32  m_FifoLoad;			// FIFO-Load register

	struct {								// Diagnostics register
	volatile UInt  m_SfData			: 8;	//  Data at output of Smoothing FIFO
	volatile UInt  m_LoopBack		: 1;	//	Loop-back output of Smoothing FIFO
	volatile UInt  m_SfDtValid		: 1;	//  Smoothing FIFO Data Vaild
	volatile UInt  m_BfFull			: 1;	//	Burst-FIFO-Full flag
	volatile UInt  m_DmaReq			: 1;	//  DMA Request from Burst FIFO flag
	volatile UInt  m_BfCanBurst		: 1;	//  Burst-FIFO Can-Burst flag
	volatile UInt  m_SfCanBurst		: 1;	//  Smoothing-FIFO Can-Burst flag
	volatile UInt  m_SfLoad			: 10;	//  Smoothing-FIFO Load in #bytes
	volatile UInt  m_BfLoad			: 8;	//  Burst-FIFO Load in #32-bit words
	}  m_Diag;

	volatile UInt32  m_LoopBackData;		// Data at output of Smoothing FIFO

	struct {								// Threshold-Control register
	volatile UInt  m_UpdBusy		: 1;	//  Threshold-Update process busy
	volatile UInt  m_CmpB			: 1;	//  Output of comparator B
	volatile UInt  m_CmpA			: 1;	//  Output of comparator A
	volatile UInt  m_AorB			: 1;	//  Update threshold A or threshold B
			 UInt  m_Reserved1		: 4;	//
	volatile UInt  m_ThreshA		: 8;	//  Threshold A
	volatile UInt  m_ThreshB		: 8;	//  Threshold B
	}  m_ThreshControl;

	volatile UInt32  m_Reserved[6];			// Not used
	volatile UInt32  m_FifoData[16];		// Transport-stream data: 4 bytes at a time
} Dta1xxTx;

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ TX register offset +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=
#define DTA1XX_TX_REG_GENCTRL			0x0000
#define DTA1XX_TX_REG_TXCTRL			0x0004
#define DTA1XX_TX_REG_TXSTATUS			0x0008
#define DTA1XX_TX_REG_TXCLOCK			0x000C
#define DTA1XX_TX_REG_CLOCKCNT			0x0010
#define DTA1XX_TX_REG_FIFOSIZE			0x0014
#define DTA1XX_TX_REG_FIFOLOAD			0x0018
#define DTA1XX_TX_REG_DIAG				0x001C
#define DTA1XX_TX_REG_LOOPBKDATA		0x0020
#define DTA1XX_TX_REG_THRESHCTRL		0x0024
#define DTA1XX_TX_REG_MODCTRL			0x0028
#define DTA1XX_TX_REG_IDACCTRL			0x0030
#define DTA1XX_TX_REG_QDACCTRL			0x0034
#define DTA1XX_TX_REG_RFCTRL			0x0038
#define DTA1XX_TX_REG_FIFO_FIRST		0x0040
#define DTA1XX_TX_REG_FIFO_LAST			0x007C

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TX register bit/field masks -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

// Tx general control
#define DTA1XX_TX_GENCTRL_PE			0x00000001
#define DTA1XX_TX_GENCTRL_PRE			0x00000002
#define DTA1XX_TX_GENCTRL_RESET			0x00000004
#define DTA1XX_TX_GENCTRL_ALTERAREV		0x0000FF00
#define DTA1XX_TX_GENCTRL_TYPENUM		0x00FF0000

// Tx control register
#define DTA1XX_TXCTRL_TXMODE			0x00000003
#define DTA1XX_TXCTRL_BURSTMODE			0x00000004
#define DTA1XX_TXCTRL_TXMODE_EXT		0x00000008
#define DTA1XX_TXCTRL_PCKSTUFF			0x00000010
#define DTA1XX_TXCTRL_TXCTRL			0x00000060
#define DTA1XX_TXCTRL_TXDIS				0x00000080
#define DTA1XX_TXCTRL_PERINT_EN			0x00000100
#define DTA1XX_TXCTRL_UFLINT_EN			0x00000200
#define DTA1XX_TXCTRL_SYNCINT_EN		0x00000400
#define DTA1XX_TXCTRL_SHORTINT_EN		0x00000800
#define DTA1XX_TXCTRL_THRINT_EN			0x00001000
#define DTA1XX_TXCTRL_USE_EXTCLOCK		0x00010000
#define DTA1XX_TXCTRL_ENAPWR			0x00020000
#define DTA1XX_TXCTRL_SHORTERROR		0x00040000
#define DTA1XX_TXCTRL_LEDCTRL			0x00080000
#define DTA1XX_TXCTRL_GREENLED			0x00100000
#define DTA1XX_TXCTRL_REDLED			0x00200000
#define DTA1XX_TXCTRL_CLEARFIFO			0x00400000
#define DTA1XX_TXCTRL_TXRESET			0x00800000
#define DTA1XX_TXCTRL_TXINVASI			0x01000000
#define DTA1XX_TXCTRL_SDI_MODE			0x02000000
#define DTA1XX_TXCTRL_OUT_EN			0x08000000

// Tx status register
#define DTA1XX_TXSTAT_FIFOFILLED		0x00000001
#define DTA1XX_TXSTAT_SDRAMSIZE			0x000000F0
#define DTA1XX_TXSTAT_PERINT			0x00000100
#define DTA1XX_TXSTAT_UFLINT			0x00000200
#define DTA1XX_TXSTAT_SYNCINT			0x00000400
#define DTA1XX_TXSTAT_SHORTINT			0x00000800
#define DTA1XX_TXSTAT_THRINT			0x00001000

// Tx diagnostic register
#define DTA1XX_TXDIAG_SFDATA			0x000000FF
#define DTA1XX_TXDIAG_LOOPBACK			0x00000100
#define DTA1XX_TXDIAG_SFDTValID			0x00000200
#define DTA1XX_TXDIAG_BF_FULL			0x00000400
#define DTA1XX_TXDIAG_DMAREQ			0x00000800
#define DTA1XX_TXDIAG_BFCANBURST		0x00001000
#define DTA1XX_TXDIAG_SFCANBURST		0x00002000
#define DTA1XX_TXDIAG_SFLOAD			0x00FC0000
#define DTA1XX_TXDIAG_BFLOAD			0xFF000000

// Tx threshold control register
#define DTA1XX_TXTHRCTRL_UPD_DONE		0x00000001
#define DTA1XX_TXTHRCTRL_CMPB			0x00000002
#define DTA1XX_TXTHRCTRL_CMPA			0x00000004
#define DTA1XX_TXTHRCTRL_A_N_B			0x00000008
#define DTA1XX_TXTHRCTRL_THRESHA		0x0000FF00
#define DTA1XX_TXTHRCTRL_THRESHB		0x00FF0000

// Modulation-Control register
#define DTA1XX_TXMODC_M_MSK				0x0000000F
#define DTA1XX_TXMODC_M_SH				0x00000000
#define DTA1XX_TXMODC_MODTYPE_MSK		0x000000F0
#define DTA1XX_TXMODC_MODTYPE_SH		0x00000004
#define DTA1XX_TXMODC_CODERATE_MSK		0x00000F00
#define DTA1XX_TXMODC_CODERATE_SH		0x00000008
#define DTA1XX_TXMODC_ROLLOFF_MSK		0x0000F000
#define DTA1XX_TXMODC_ROLLOFF_SH		0x0000000C

// RF-Control register
#define DTA1XX_TXRFCTRL_PLLA_MSK			0x0000001F
#define DTA1XX_TXRFCTRL_PLLA_SH				0
#define DTA1XX_TXRFCTRL_PLLB_MSK			0x00003FE0
#define DTA1XX_TXRFCTRL_PLLB_SH				5
#define DTA1XX_TXRFCTRL_PLL_LOCK			0x00020000
#define DTA1XX_TXRFCTRL_PLLB9_MSK			0x00040000
#define DTA1XX_TXRFCTRL_PLLB9_SH			18
#define DTA1XX_TXRFCTRL_PLLR_MSK			0xFFF80000
#define DTA1XX_TXRFCTRL_PLLR_SH				19

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Tx register operations +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxGetGenCtrlReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int32_t Dta1xxTxGetGenCtrlReg(char* pBase)
{
	int32_t Val;
	READ_LONG(Val, pBase, DTA1XX_TX_REG_GENCTRL);
	return Val;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxSetGenCtrlReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxTxSetGenCtrlReg(char* pBase, int32_t Val)
{
	WRITE_LONG(Val, pBase, DTA1XX_TX_REG_GENCTRL);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxGenCtrlRegSetPE -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxTxGenCtrlRegSetPE(char* pBase, int32_t nEna)
{
	int32_t Val = Dta1xxTxGetGenCtrlReg(pBase);
	if ( nEna==1 ) {
		Val |= DTA1XX_TX_GENCTRL_PE;
	} else {
		Val &= ~DTA1XX_TX_GENCTRL_PE;
	}
	Dta1xxTxSetGenCtrlReg(pBase, Val);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxGenCtrlRegSetPRE -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxTxGenCtrlRegSetPRE(char* pBase, int32_t nEna)
{
	int32_t Val = Dta1xxTxGetGenCtrlReg(pBase);
	if ( nEna==1 ) {
		Val |= DTA1XX_TX_GENCTRL_PRE;
	} else {
		Val &= ~DTA1XX_TX_GENCTRL_PRE;
	}
	Dta1xxTxSetGenCtrlReg(pBase, Val);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxGenCtrlRegReset -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxTxGenCtrlRegReset(char* pBase)
{
	int32_t Val = Dta1xxTxGetGenCtrlReg(pBase);
	Val |= DTA1XX_TX_GENCTRL_RESET;
	Dta1xxTxSetGenCtrlReg(pBase, Val);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxGenCtrlRegGetFirmwareRev -.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int32_t Dta1xxTxGenCtrlRegGetFirmwareRev(char* pBase)
{
	int32_t Val = Dta1xxTxGetGenCtrlReg(pBase);
	Val &= DTA1XX_TX_GENCTRL_ALTERAREV;
	return (Val>>8);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxGenCtrlRegGetTypeNumber -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int32_t Dta1xxTxGenCtrlRegGetTypeNumber(char* pBase)
{
	int32_t Val = Dta1xxTxGetGenCtrlReg(pBase);
	Val &= DTA1XX_TX_GENCTRL_TYPENUM;
	return (Val>>16);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxGetTxCtrlReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int32_t Dta1xxTxGetTxCtrlReg(char* pBase)
{
	int32_t Val;
	READ_LONG(Val, pBase, DTA1XX_TX_REG_TXCTRL);
	return Val;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxSetTxCtrlReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxTxSetTxCtrlReg(char* pBase, int32_t Val)
{
	WRITE_LONG(Val, pBase, DTA1XX_TX_REG_TXCTRL);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxCtrlRegGetTxMode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int32_t Dta1xxTxCtrlRegGetTxMode(char* pBase)
{
	int32_t nTxMode;
	int32_t Val = Dta1xxTxGetTxCtrlReg(pBase);
	nTxMode = (Val & DTA1XX_TXCTRL_TXMODE);
	nTxMode |= ((Val & DTA1XX_TXCTRL_TXMODE_EXT)>>1);
	return nTxMode;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxCtrlRegSetTxMode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxTxCtrlRegSetTxMode(char* pBase, int32_t nTxMode)
{
	int32_t Val = Dta1xxTxGetTxCtrlReg(pBase);
	Val &= ~(DTA1XX_TXCTRL_TXMODE_EXT | DTA1XX_TXCTRL_TXMODE);
	Val |= nTxMode&0x3;
	Val |= (nTxMode&0x4)<<1;
	Dta1xxTxSetTxCtrlReg(pBase, Val);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxCtrlRegGetBurstMode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int32_t Dta1xxTxCtrlRegGetBurstMode(char* pBase)
{
	int32_t Val = Dta1xxTxGetTxCtrlReg(pBase);
	Val &= DTA1XX_TXCTRL_BURSTMODE;
	return (Val >> 2);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxCtrlRegSetBurstMode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxTxCtrlRegSetBurstMode(char* pBase, int32_t nEna)
{
	int32_t Val = Dta1xxTxGetTxCtrlReg(pBase);
	if ( nEna==1 ) {
		Val |= DTA1XX_TXCTRL_BURSTMODE;
	} else {
		Val &= ~DTA1XX_TXCTRL_BURSTMODE;
	}
	Dta1xxTxSetTxCtrlReg(pBase, Val);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxCtrlRegGetPckStuff -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int32_t Dta1xxTxCtrlRegGetPckStuff(char* pBase)
{
	int32_t Val = Dta1xxTxGetTxCtrlReg(pBase);
	Val &= DTA1XX_TXCTRL_PCKSTUFF;
	return (Val >> 4);
}


//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxCtrlRegSetPckStuff -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxTxCtrlRegSetPckStuff(char* pBase, int32_t nEna)
{
	int32_t Val = Dta1xxTxGetTxCtrlReg(pBase);
	if ( nEna==1 ) {
		Val |= DTA1XX_TXCTRL_PCKSTUFF;
	} else {
		Val &= ~DTA1XX_TXCTRL_PCKSTUFF;
	}
	Dta1xxTxSetTxCtrlReg(pBase, Val);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxCtrlRegGetTxCtrl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int32_t Dta1xxTxCtrlRegGetTxCtrl(char* pBase)
{
	int32_t Val = Dta1xxTxGetTxCtrlReg(pBase);
	Val &= DTA1XX_TXCTRL_TXCTRL;
	return (Val>>5);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxCtrlRegSetTxCtrl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxTxCtrlRegSetTxCtrl(char* pBase, int32_t nTxCtrl)
{
	int32_t Val = Dta1xxTxGetTxCtrlReg(pBase);
	Val &= ~DTA1XX_TXCTRL_TXCTRL;
	Val |= (nTxCtrl&0x3) << 5 ;
	Dta1xxTxSetTxCtrlReg(pBase, Val);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxCtrlRegGetTxDis -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int32_t Dta1xxTxCtrlRegGetTxDis(char* pBase)
{
	int32_t Val = Dta1xxTxGetTxCtrlReg(pBase);
	Val &= DTA1XX_TXCTRL_TXDIS;
	return (Val>>7);
}


//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxCtrlRegSetTxDis -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxTxCtrlRegSetTxDis(char* pBase, int32_t nDis)
{
	int32_t Val = Dta1xxTxGetTxCtrlReg(pBase);
	if ( nDis==1 ) {
		Val |= DTA1XX_TXCTRL_TXDIS;
	} else {
		Val &= ~DTA1XX_TXCTRL_TXDIS;
	}
	Dta1xxTxSetTxCtrlReg(pBase, Val);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxCtrlRegSetPerIntEn -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxTxCtrlRegSetPerIntEn(char* pBase, int32_t nEna)
{
	int32_t Val = Dta1xxTxGetTxCtrlReg(pBase);
	if ( nEna==1 ) {
		Val |= DTA1XX_TXCTRL_PERINT_EN;
	} else {
		Val &= ~DTA1XX_TXCTRL_PERINT_EN;
	}
	Dta1xxTxSetTxCtrlReg(pBase, Val);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxCtrlRegSetUflIntEn -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxTxCtrlRegSetUflIntEn(char* pBase, int32_t nEna)
{
	int32_t Val = Dta1xxTxGetTxCtrlReg(pBase);
	if ( nEna==1 ) {
		Val |= DTA1XX_TXCTRL_UFLINT_EN;
	} else {
		Val &= ~DTA1XX_TXCTRL_UFLINT_EN;
	}
	Dta1xxTxSetTxCtrlReg(pBase, Val);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxCtrlRegSetSyncIntEn -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxTxCtrlRegSetSyncIntEn(char* pBase, int32_t nEna)
{
	int32_t Val = Dta1xxTxGetTxCtrlReg(pBase);
	if ( nEna==1 ) {
		Val |= DTA1XX_TXCTRL_SYNCINT_EN;
	} else {
		Val &= ~DTA1XX_TXCTRL_SYNCINT_EN;
	}
	Dta1xxTxSetTxCtrlReg(pBase, Val);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxCtrlRegSetShortIntEn -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxTxCtrlRegSetShortIntEn(char* pBase, int32_t nEna)
{
	int32_t Val = Dta1xxTxGetTxCtrlReg(pBase);
	if ( nEna==1 ) {
		Val |= DTA1XX_TXCTRL_SHORTINT_EN;
	} else {
		Val &= ~DTA1XX_TXCTRL_SHORTINT_EN;
	}
	Dta1xxTxSetTxCtrlReg(pBase, Val);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxCtrlRegSetThrIntEn -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxTxCtrlRegSetThrIntEn(char* pBase, int32_t nEna)
{
	int32_t Val = Dta1xxTxGetTxCtrlReg(pBase);
	if ( nEna==1 ) {
		Val |= DTA1XX_TXCTRL_THRINT_EN;
	} else {
		Val &= ~DTA1XX_TXCTRL_THRINT_EN;
	}
	Dta1xxTxSetTxCtrlReg(pBase, Val);
}


//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxCtrlRegGetUseExtClk -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int32_t Dta1xxTxCtrlRegGetUseExtClk(char* pBase)
{
	int32_t Val = Dta1xxTxGetTxCtrlReg(pBase);
	Val &= DTA1XX_TXCTRL_USE_EXTCLOCK;
	return (Val >> 16);
}


//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxCtrlRegSetUseExtClk -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxTxCtrlRegSetUseExtClk(char* pBase, int32_t nUse)
{
	int32_t Val = Dta1xxTxGetTxCtrlReg(pBase);
	if ( 1==nUse ) {
		Val |= DTA1XX_TXCTRL_USE_EXTCLOCK;
	} else {
		Val &= ~DTA1XX_TXCTRL_USE_EXTCLOCK;
	}
	Dta1xxTxSetTxCtrlReg(pBase, Val);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxCtrlRegSetEnaPwr -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxTxCtrlRegSetEnaPwr(char* pBase, int32_t nEna)
{
	int32_t Val = Dta1xxTxGetTxCtrlReg(pBase);
	if ( nEna==1 ) {
		Val |= DTA1XX_TXCTRL_ENAPWR;
	} else {
		Val &= ~DTA1XX_TXCTRL_ENAPWR;
	}
	Dta1xxTxSetTxCtrlReg(pBase, Val);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxLedControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxTxLedControl(char* pBase, int32_t nLedCtrl)
{
	int32_t Val = Dta1xxTxGetTxCtrlReg(pBase);
	Val &= ~(DTA1XX_TXCTRL_LEDCTRL | DTA1XX_TXCTRL_GREENLED | DTA1XX_TXCTRL_REDLED);
	Val |= (nLedCtrl & 0x7) << 19;
	Dta1xxTxSetTxCtrlReg(pBase, Val);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxClrFifo -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxTxClrFifo(char* pBase)
{
	int32_t Val = Dta1xxTxGetTxCtrlReg(pBase);
	Val |= DTA1XX_TXCTRL_CLEARFIFO;
	Dta1xxTxSetTxCtrlReg(pBase, Val);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxChannelReset -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxTxChannelReset(char* pBase)
{
	int32_t Val = Dta1xxTxGetTxCtrlReg(pBase);
	Val |= DTA1XX_TXCTRL_TXRESET;
	Dta1xxTxSetTxCtrlReg(pBase, Val);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxCtrlSetTxAsiInv -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxTxCtrlSetTxAsiInv(char* pBase, int32_t nEna)
{
	int32_t Val = Dta1xxTxGetTxCtrlReg(pBase);
	if ( nEna==1 ) {
		Val |= DTA1XX_TXCTRL_TXINVASI;
	} else {
		Val &= ~DTA1XX_TXCTRL_TXINVASI;
	}
	Dta1xxTxSetTxCtrlReg(pBase, Val);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxGetTxStatusReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int32_t Dta1xxTxGetTxStatusReg(char* pBase)
{
	int32_t Val;
	READ_LONG(Val, pBase, DTA1XX_TX_REG_TXSTATUS);
	return Val;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxSetTxStatusReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxTxSetTxStatusReg(char* pBase, int32_t Val)
{
	WRITE_LONG(Val, pBase, DTA1XX_TX_REG_TXSTATUS);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxStatusRegGetFifoFilled -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int32_t Dta1xxTxStatusRegGetFifoFilled(char* pBase)
{
	int32_t Val = Dta1xxTxGetTxStatusReg(pBase);
	if ( 0!=(Val & DTA1XX_TXSTAT_FIFOFILLED) ) {
		return 1;
	} else {
		return 0;
	}
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxStatusRegGetSdramSize -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int32_t Dta1xxTxStatusRegGetSdramSize(char* pBase)
{
	int32_t Val = Dta1xxTxGetTxStatusReg(pBase);
	Val &= DTA1XX_TXSTAT_SDRAMSIZE;
	return (Val << 4);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxStatusRegGetPerInt -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int32_t Dta1xxTxStatusRegGetPerInt(char* pBase)
{
	int32_t Val = Dta1xxTxGetTxStatusReg(pBase);
	if ( 0!=(Val & DTA1XX_TXSTAT_PERINT) ) {
		return 1;
	} else {
		return 0;
	}
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxStatusRegClrPerInt -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxTxStatusRegClrPerInt(char* pBase)
{
	Dta1xxTxSetTxStatusReg(pBase, DTA1XX_TXSTAT_PERINT);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxStatusRegGetUflInt -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int32_t Dta1xxTxStatusRegGetUflInt(char* pBase)
{
	int32_t Val = Dta1xxTxGetTxStatusReg(pBase);
	if ( 0!=(Val & DTA1XX_TXSTAT_UFLINT) ) {
		return 1;
	} else {
		return 0;
	}
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxStatusRegClrUflInt -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxTxStatusRegClrUflInt(char* pBase)
{
	Dta1xxTxSetTxStatusReg(pBase, DTA1XX_TXSTAT_UFLINT);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxStatusRegGetSyncInt -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int32_t Dta1xxTxStatusRegGetSyncInt(char* pBase)
{
	int32_t Val = Dta1xxTxGetTxStatusReg(pBase);
	if ( 0!=(Val & DTA1XX_TXSTAT_SYNCINT) ) {
		return 1;
	} else {
		return 0;
	}
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxStatusRegClrSyncInt -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxTxStatusRegClrSyncInt(char* pBase)
{
	Dta1xxTxSetTxStatusReg(pBase, DTA1XX_TXSTAT_SYNCINT);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxStatusRegGetShortInt -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int32_t Dta1xxTxStatusRegGetShortInt(char* pBase)
{
	int32_t Val = Dta1xxTxGetTxStatusReg(pBase);
	if ( 0!=(Val & DTA1XX_TXSTAT_SHORTINT) ) {
		return 1;
	} else {
		return 0;
	}
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxStatusRegClrShortInt -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxTxStatusRegClrShortInt(char* pBase)
{
	Dta1xxTxSetTxStatusReg(pBase, DTA1XX_TXSTAT_SHORTINT);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxStatusRegGetThrInt -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int32_t Dta1xxTxStatusRegGetThrInt(char* pBase)
{
	int32_t Val = Dta1xxTxGetTxStatusReg(pBase);
	if ( 0!=(Val & DTA1XX_TXSTAT_THRINT) ) {
		return 1;
	} else {
		return 0;
	}
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxStatusRegClrThrInt -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxTxStatusRegClrThrInt(char* pBase)
{
	Dta1xxTxSetTxStatusReg(pBase, DTA1XX_TXSTAT_THRINT);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxGetTxClockReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int32_t Dta1xxTxGetTxClockReg(char* pBase)
{
	int32_t Val;
	READ_LONG(Val, pBase, DTA1XX_TX_REG_TXCLOCK);
	return Val;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxSetTxClockReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxTxSetTxClockReg(char* pBase, int32_t Val)
{
	WRITE_LONG(Val, pBase, DTA1XX_TX_REG_TXCLOCK);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxGetClkCountReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int32_t Dta1xxTxGetClkCountReg(char* pBase)
{
	int32_t Val;
	READ_LONG(Val, pBase, DTA1XX_TX_REG_CLOCKCNT);
	return Val;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxGetFifoSizeReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int32_t Dta1xxTxGetFifoSizeReg(char* pBase)
{
	int32_t Val;
	READ_LONG(Val, pBase, DTA1XX_TX_REG_FIFOSIZE);
	return Val;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxSetFifoSizeReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxTxSetFifoSizeReg(char* pBase, int32_t Val)
{
	WRITE_LONG(Val, pBase, DTA1XX_TX_REG_FIFOSIZE);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxGetFifoLoadReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int32_t Dta1xxTxGetFifoLoadReg(char* pBase)
{
	int32_t Val;
	READ_LONG(Val, pBase, DTA1XX_TX_REG_FIFOLOAD);
	return Val;
}


//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxGetDiagReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int32_t Dta1xxTxGetDiagReg(char* pBase)
{
	int32_t Val;
	READ_LONG(Val, pBase, DTA1XX_TX_REG_DIAG);
	return Val;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxSetDiagReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxTxSetDiagReg(char* pBase, int32_t Val)
{
	WRITE_LONG(Val, pBase, DTA1XX_TX_REG_DIAG);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxDiagRegGetSfData -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int32_t Dta1xxTxDiagRegGetSfData(char* pBase)
{
	int32_t Val = Dta1xxTxGetDiagReg(pBase);
	Val &= DTA1XX_TXDIAG_SFDATA;
	return Val;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxDiagRegGetLoopBack -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int32_t Dta1xxTxDiagRegGetLoopBack(char* pBase)
{
	int32_t Val = Dta1xxTxGetDiagReg(pBase);
	Val &= DTA1XX_TXDIAG_LOOPBACK;
	return ( Val >> 8);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxDiagRegSetLoopBack -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxTxDiagRegSetLoopBack(char* pBase, int32_t nEna)
{
	int32_t Val = Dta1xxTxGetDiagReg(pBase);
	if ( 0!=nEna ) {
        Val |= DTA1XX_TXDIAG_LOOPBACK;
	} else {
		Val &= ~DTA1XX_TXDIAG_LOOPBACK;
	}
	Dta1xxTxSetDiagReg(pBase, Val);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxDiagRegGetSfDtValid -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int32_t Dta1xxTxDiagRegGetSfDtValid(char* pBase)
{
	int32_t Val = Dta1xxTxGetDiagReg(pBase);
	if ( 0!=(Val & DTA1XX_TXDIAG_SFDTValID) ) {
        return 1;
	} else {
		return 0;
	}
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxDiagRegGetDmaReq -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int32_t Dta1xxTxDiagRegGetDmaReq(char* pBase)
{
	int32_t Val = Dta1xxTxGetDiagReg(pBase);
	if ( 0!=(Val & DTA1XX_TXDIAG_DMAREQ) ) {
        return 1;
	} else {
		return 0;
	}
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxDiagRegGetBfCanBurst -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int32_t Dta1xxTxDiagRegGetBfCanBurst(char* pBase)
{
	int32_t Val = Dta1xxTxGetDiagReg(pBase);
	if ( 0!=(Val & DTA1XX_TXDIAG_BFCANBURST) ) {
        return 1;
	} else {
		return 0;
	}
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxDiagRegGetSfCanBurst -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int32_t Dta1xxTxDiagRegGetSfCanBurst(char* pBase)
{
	int32_t Val = Dta1xxTxGetDiagReg(pBase);
	if ( 0!=(Val & DTA1XX_TXDIAG_SFCANBURST) ) {
        return 1;
	} else {
		return 0;
	}
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxDiagRegGetSfLoad -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int32_t Dta1xxTxDiagRegGetSfLoad(char* pBase)
{
	int32_t Val = Dta1xxTxGetDiagReg(pBase);
	Val |= DTA1XX_TXDIAG_SFLOAD;
	return (Val >> 14);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxDiagRegGetBfLoad -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int32_t Dta1xxTxDiagRegGetBfLoad(char* pBase)
{
	int32_t Val = Dta1xxTxGetDiagReg(pBase);
	Val |= DTA1XX_TXDIAG_BFLOAD;
	return (Val >> 24);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxGetLoopBackDataReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int32_t Dta1xxTxGetLoopBackDataReg(char* pBase)
{
	int32_t Val;
	READ_LONG(Val, pBase, DTA1XX_TX_REG_LOOPBKDATA);
	return Val;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxGetThreshCtrlReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int32_t Dta1xxTxGetThreshCtrlReg(char* pBase)
{
	int32_t Val;
	READ_LONG(Val, pBase, DTA1XX_TX_REG_THRESHCTRL);
	return Val;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxSetThreshCtrlReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxTxSetThreshCtrlReg(char* pBase, int32_t Val)
{
	WRITE_LONG(Val, pBase, DTA1XX_TX_REG_THRESHCTRL);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxSetModCtrlReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxTxSetModCtrlReg(char* pBase, int32_t Val)
{
	WRITE_LONG(Val, pBase, DTA1XX_TX_REG_MODCTRL);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxGetModCtrlReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int32_t Dta1xxTxGetModCtrlReg(char* pBase)
{
	int32_t Val;
	READ_LONG(Val, pBase, DTA1XX_TX_REG_MODCTRL);
	return Val;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxModcRegGetM -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int32_t Dta1xxTxModcRegGetM(char* pBase)
{
	int32_t Val = Dta1xxTxGetModCtrlReg(pBase) & DTA1XX_TXMODC_M_MSK;
	return (Val >> DTA1XX_TXMODC_M_SH);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxModcRegSetM -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxTxModcRegSetM(char* pBase, int32_t M)
{
	int32_t Val = Dta1xxTxGetModCtrlReg(pBase) & ~DTA1XX_TXMODC_M_MSK;
	Dta1xxTxSetModCtrlReg(pBase, Val | (M << DTA1XX_TXMODC_M_SH));
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxModcRegGetModType -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int32_t Dta1xxTxModcRegGetModType(char* pBase)
{
	int32_t Val = Dta1xxTxGetModCtrlReg(pBase) & DTA1XX_TXMODC_MODTYPE_MSK;
	return (Val >> DTA1XX_TXMODC_MODTYPE_SH);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxModcRegSetModType -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxTxModcRegSetModType(char* pBase, int32_t ModType)
{
	int32_t Val = Dta1xxTxGetModCtrlReg(pBase) & ~DTA1XX_TXMODC_MODTYPE_MSK;
	Dta1xxTxSetModCtrlReg(pBase, Val | (ModType << DTA1XX_TXMODC_MODTYPE_SH));
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxModcRegGetCodeRate -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int32_t Dta1xxTxModcRegGetCodeRate(char* pBase)
{
	int32_t Val = Dta1xxTxGetModCtrlReg(pBase) & DTA1XX_TXMODC_CODERATE_MSK;
	return (Val >> DTA1XX_TXMODC_CODERATE_SH);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxModcRegSetCodeRate -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxTxModcRegSetCodeRate(char* pBase, int32_t CodeRate)
{
	int32_t Val = Dta1xxTxGetModCtrlReg(pBase) & ~DTA1XX_TXMODC_CODERATE_MSK;
	Dta1xxTxSetModCtrlReg(pBase, Val | (CodeRate << DTA1XX_TXMODC_CODERATE_SH));
}

//.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxModcRegSetRollOffFactor -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int32_t Dta1xxTxModcRegGetRollOffFactor(char* pBase)
{
        int32_t Val = Dta1xxTxGetModCtrlReg(pBase) & DTA1XX_TXMODC_ROLLOFF_MSK;
        return (Val >> DTA1XX_TXMODC_ROLLOFF_SH);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-. Dta1xxTxModcRegSetRollOffFactor -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxTxModcRegSetRollOffFactor(char* pBase, int32_t RollOff)
{
        int32_t Val = Dta1xxTxGetModCtrlReg(pBase) & ~DTA1XX_TXMODC_ROLLOFF_MSK;
        Dta1xxTxSetModCtrlReg(pBase, Val | (RollOff << DTA1XX_TXMODC_ROLLOFF_SH));
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxSetRfCtrlReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxTxSetRfCtrlReg(char* pBase, int32_t Val)
{
	WRITE_LONG(Val, pBase, DTA1XX_TX_REG_RFCTRL);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxGetRfCtrlReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int32_t Dta1xxTxGetRfCtrlReg(char* pBase)
{
	int32_t Val;
	READ_LONG(Val, pBase, DTA1XX_TX_REG_RFCTRL);
	return Val;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxRfCtrlRegGetPllA -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int32_t Dta1xxTxRfCtrlRegGetPllA(char* pBase)
{
	int32_t Val = Dta1xxTxGetRfCtrlReg(pBase) & DTA1XX_TXRFCTRL_PLLA_MSK;
	return (Val >> DTA1XX_TXRFCTRL_PLLA_SH);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxRfCtrlRegSetPllA -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void Dta1xxTxRfCtrlRegSetPllA(char* pBase, int32_t PllA)
{
	int32_t Val = Dta1xxTxGetRfCtrlReg(pBase) & ~DTA1XX_TXRFCTRL_PLLA_MSK;
	Dta1xxTxSetRfCtrlReg(pBase, Val | (PllA << DTA1XX_TXRFCTRL_PLLA_SH));
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxRfCtrlRegGetPllB -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline int32_t Dta1xxTxRfCtrlRegGetPllB(char* pBase)
{
	int32_t Val = Dta1xxTxGetRfCtrlReg(pBase);
	int32_t PllBbit8to0 = (Val & DTA1XX_TXRFCTRL_PLLB_MSK)  >> DTA1XX_TXRFCTRL_PLLB_SH;
	int32_t PllBbit9    = (Val & DTA1XX_TXRFCTRL_PLLB9_MSK) >> DTA1XX_TXRFCTRL_PLLB9_SH;
	return ((PllBbit9 << 9) + PllBbit8to0);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxRfCtrlRegSetPllB -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
static inline void  Dta1xxTxRfCtrlRegSetPllB(char* pBase, int32_t PllB)
{
	int32_t Val = Dta1xxTxGetRfCtrlReg(pBase) & ~DTA1XX_TXRFCTRL_PLLB_MSK
											  & ~DTA1XX_TXRFCTRL_PLLB9_MSK;
	int32_t PllBbit8to0 =  PllB & 0x3FF;
	int32_t PllBbit9    = (PllB >> 9) & 1;
	Dta1xxTxSetRfCtrlReg(pBase, Val | (PllBbit8to0 << DTA1XX_TXRFCTRL_PLLB_SH)
									| (PllBbit9    << DTA1XX_TXRFCTRL_PLLB9_SH));
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxRfCtrlRegGetPllLock -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline int32_t Dta1xxTxRfCtrlRegGetPllLock(char* pBase)
{
	return ((Dta1xxTxGetRfCtrlReg(pBase)&DTA1XX_TXRFCTRL_PLL_LOCK)!=0) ? 1 : 0;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxSetFifoDataReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
static inline void Dta1xxTxSetFifoDataReg(char* pBase, int32_t Val)
{
	WRITE_LONG(Val, pBase, DTA1XX_TX_REG_FIFO_FIRST);
}

// Tx Control: SDI mode
static inline void  Dta1xxTxCtrlRegSetSdiMode(char* pBase, unsigned int SdiMode) {
	int32_t Val = Dta1xxTxGetTxCtrlReg(pBase);
	if (SdiMode != 0)		Val |=  DTA1XX_TXCTRL_SDI_MODE;
	else					Val &= ~DTA1XX_TXCTRL_SDI_MODE;
	Dta1xxTxSetTxCtrlReg(pBase, Val);
}

// Tx Control: Output enable
static __inline int32_t  Dta1xxTxCtrlRegGetOutputEn(char* pBase) {
	return ((Dta1xxTxGetTxCtrlReg(pBase)&DTA1XX_TXCTRL_OUT_EN)!=0) ? 1 : 0;
}
static __inline void  Dta1xxTxCtrlRegSetOutputEn(char* pBase, int32_t OutputEnable) {
	int32_t  Val = Dta1xxTxGetTxCtrlReg(pBase);
	if (OutputEnable != 0)	Val |=  DTA1XX_TXCTRL_OUT_EN;
	else					Val &= ~DTA1XX_TXCTRL_OUT_EN;
	Dta1xxTxSetTxCtrlReg(pBase, Val);
}

#endif // __TX_REGS_H
