//#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* Dma.c *#*#*#*#*#*#*#*#*#*# (C) 2000-2005 DEKTEC

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- License -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// This program is free software; You can freely use/modify this source code in any way
// you desire, including for commercial applications.
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	KvR	14-08-2006	DTA-105 registers changed, Vpd-data handling changed
//	JH	16-08-2004	Added some castings to avoid compiler warnings
//	MG	11-03-2003	Replaced DMA done wait loop with wait for event
//	MG	28-10-2002	Created

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#include "../Include/Dta1xx.h"
#include "../Include/Dta1xxRegs.h"
#include <asm/bitops.h> // set_bit

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Type definitions -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Internal function declarations -.-.-.-.-.-.-.-.-.-.-.-.-.-.
Int64 bindiv (Int64 num, Int64 denom, Int64* pRest);

//-.-.-.-.-.-.-.-.-.-.-.-.-.- Data declaration and definition -.-.-.-.-.-.-.-.-.-.-.-.-.-.

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Implementation +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxDmaTransfer -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Executes a DMA transfer.
//
int Dta1xxDmaTransfer(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int ChType,			// Channel type
	int nLocalDmaAddr,	// Local address of the DMA transfer (local to DTA-1xx)
	int nDmaSize,		// Size of the DMA transfer
	int nDmaDir)		// DMA direction (DTA1XX_DMA_READ/DTA1XX_DMA_WRITE)
{
	int		ret=0;
	int		nPlxDmaChan;
	unsigned int DmaDesc;
	CHANNEL* pCh=NULL;
	SglDesc* pSglDesc=NULL;

	if ( ChType == DTA1XX_TS_RX_CHANNEL )
	{
		// Check for valid channel index
		if ( nChIdx<0 || nChIdx>=pFdo->m_nNumRxChannels )
			return -1;

		pCh = pFdo->m_TsRx[nChIdx];
	}
	else
	{
		// Check for valid channel index
		if ( nChIdx<0 || nChIdx>=pFdo->m_nNumTxChannels )
			return -1;

		pCh = pFdo->m_TsTx[nChIdx];
	}
	
	// Check whether no CDMA is running
	if (pCh->m_CdmaDmaRunning != FALSE)
		return -1;

	// DTA-140 uses DMA channel 1 for reads. In all other cases DMA channel 0 is used
	if ( ( pFdo->m_nTypeNumber==140) && ( nDmaDir==DTA1XX_DMA_READ) ) {
		nPlxDmaChan = 1;
	} else {
		nPlxDmaChan = 0;
	}

	// Aquire DMA lock
	down_interruptible( &pCh->m_Dma.m_Lock );

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Setup/Start the DMA -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	// Reset DMA done flag
	clear_bit( 0, (void*)&pCh->m_Dma.m_nDmaDone );

	if ( pFdo->m_nUsesDmaInFpga )
	{
		unsigned long CmdStat;

		// Prepare scatter/gather descriptor
		pSglDesc = pCh->m_Dma.m_pSglDesc;

		pSglDesc->m_nPciAddress		= pCh->m_Dma.m_DmaBuffer.m_PhysStartAddr;
		pSglDesc->m_nTransferSize	= nDmaSize;
		pSglDesc->m_nLocalAddress	= nLocalDmaAddr;
		// This is only descriptor so point to itself
		pSglDesc->m_nNextSglDesc	= pCh->m_Dma.m_PhysAddrSglDesc & ~0x3;
                // Set direction
		if ( nDmaDir == DTA1XX_DMA_READ )
			pSglDesc->m_nNextSglDesc |= PCI905X_DMADPR_DIROFTFR;
		// Mark as last descriptor
		pSglDesc->m_nNextSglDesc |= PCI905X_DMADPR_ENDOFCHAIN;
		// Use "Interrupt after Terminal Count" as DMA-done indicator
		pSglDesc->m_nNextSglDesc |= PCI905X_DMADPR_INTAFTERTC;

		// Write pointer to first SGL in DMA-Channel-0-Descriptor-Pointer register
		DmaDesc = pCh->m_Dma.m_PhysAddrSglDesc | PCI905X_DMADPR_DESCLOC_PCI;
		if (pCh->m_Dma.m_nDmaDir == DTA1XX_DMA_READ) {
			DmaDesc |= PCI905X_DMADPR_DIROFTFR;
		}
		WRITE_LONG( DmaDesc, pCh->m_Dma.m_pRegDmaDesc, 0 );

		// Start transfer!
		// NOTE: be careful with using WRITE_BYTE to access local registers
		//       in the ALTERA. No byte enables!!!
		CmdStat = PCI905X_DMACSR_ENABLE | PCI905X_DMACSR_START;
		WRITE_BYTE( CmdStat, pCh->m_Dma.m_pRegCmdStat, 0 );
	}
	else if ( nPlxDmaChan==0 )
	{
		// Enable Block Mode in PCI 9054
		WRITE_LONG( PCI905X_DMAMODE_BUSWIDTH_32 | PCI905X_DMAMODE_READY_INPEN	|
					PCI905X_DMAMODE_LOCALBURST	 /*| PCI905X_DMAMODE_SCATGATH*/ |
					PCI905X_DMAMODE_DONE_INTEN	 | PCI905X_DMAMODE_LOC_CONST	|

					PCI905X_DMAMODE_DEMAND		 | PCI905X_DMAMODE_INT2PCI,
					pFdo->m_Pci905XConfRegs.m_pKernel, PCI905X_DMA0_MODE);

		// Write target address
		WRITE_LONG( pCh->m_Dma.m_DmaBuffer.m_PhysStartAddr,
					pFdo->m_Pci905XConfRegs.m_pKernel, PCI905X_DMA0_PCI_ADDR);

		// Write local address
		WRITE_LONG( nLocalDmaAddr,
					pFdo->m_Pci905XConfRegs.m_pKernel, PCI905X_DMA0_LOCAL_ADDR);

		// Write transfer size
		WRITE_LONG( nDmaSize, pFdo->m_Pci905XConfRegs.m_pKernel, PCI905X_DMA0_COUNT);

		// Set transfer direction
		if ( nDmaDir==DTA1XX_DMA_READ ) {
			WRITE_LONG(8, pFdo->m_Pci905XConfRegs.m_pKernel, PCI905X_DMA0_DESC_PTR);
		} else {
			WRITE_LONG(0, pFdo->m_Pci905XConfRegs.m_pKernel, PCI905X_DMA0_DESC_PTR);
		}

		// Start transfer!
		WRITE_BYTE(PCI905X_DMACSR_ENABLE | PCI905X_DMACSR_START,
				pFdo->m_Pci905XConfRegs.m_pKernel, PCI905X_DMA0_COMMAND_STAT);
	}
	else
	{
		// Enable Block Mode in PCI 9054
		WRITE_LONG( PCI905X_DMAMODE_BUSWIDTH_32 | PCI905X_DMAMODE_READY_INPEN	|
					PCI905X_DMAMODE_LOCALBURST	 /*| PCI905X_DMAMODE_SCATGATH*/ |
					PCI905X_DMAMODE_DONE_INTEN	 | PCI905X_DMAMODE_LOC_CONST	|
					PCI905X_DMAMODE_DEMAND		 | PCI905X_DMAMODE_INT2PCI,
					pFdo->m_Pci905XConfRegs.m_pKernel, PCI905X_DMA1_MODE);

		// Write target address
		WRITE_LONG( pCh->m_Dma.m_DmaBuffer.m_PhysStartAddr,
					pFdo->m_Pci905XConfRegs.m_pKernel, PCI905X_DMA1_PCI_ADDR);

		// Write local address
		WRITE_LONG( nLocalDmaAddr,
					pFdo->m_Pci905XConfRegs.m_pKernel, PCI905X_DMA1_LOCAL_ADDR);

		// Write transfer size
		WRITE_LONG( nDmaSize, pFdo->m_Pci905XConfRegs.m_pKernel, PCI905X_DMA1_COUNT);

		// Set transfer direction
		if ( nDmaDir==DTA1XX_DMA_READ ) {
			WRITE_LONG(8, pFdo->m_Pci905XConfRegs.m_pKernel, PCI905X_DMA1_DESC_PTR);
		} else {
			WRITE_LONG(0, pFdo->m_Pci905XConfRegs.m_pKernel, PCI905X_DMA1_DESC_PTR);
		}

		// Start transfer!
		WRITE_BYTE(PCI905X_DMACSR_ENABLE | PCI905X_DMACSR_START,
				pFdo->m_Pci905XConfRegs.m_pKernel, PCI905X_DMA1_COMMAND_STAT);
    }

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Wait for DMA done flag -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	ret = wait_event_interruptible( pCh->m_Dma.m_DmaDoneQueue,
     	     		         ( test_bit(0, (void*)&pCh->m_Dma.m_nDmaDone) ) );

	// Were we interrupted by a signal?
	if ( ret != 0 /*-ERESTARTSYS*/) {
				// Have to cancel the current DMA
		if ( pFdo->m_nUsesDmaInFpga )
		{
			WRITE_BYTE( PCI905X_DMACSR_ABORT, pCh->m_Dma.m_pRegCmdStat, 0 );
		}
		else if ( nPlxDmaChan==0 )
		{
			// Abort transfer!
			WRITE_BYTE( PCI905X_DMACSR_ABORT,
				pFdo->m_Pci905XConfRegs.m_pKernel, PCI905X_DMA0_COMMAND_STAT);

		}
		else
		{
			// Abort transfer!
			WRITE_BYTE( PCI905X_DMACSR_ABORT,
					pFdo->m_Pci905XConfRegs.m_pKernel, PCI905X_DMA1_COMMAND_STAT);
		}
	}

	// Release lock
	up(&pCh->m_Dma.m_Lock);

	// Did we cancel the DMA (we where interrupted by a signal)
	if ( ret != 0 ) {
		return -1;
	} else {
		return 0;
	}
}

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ CONTINUOUS DMA (CDMA) +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxCdmaCtrl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
int  Dta1xxCdmaCtrl(DTA1XX_FDO* pFdo, DTA1XX_CDMA_CONTROL* pCdmaCtrl)
{
	int Status=0;
	CHANNEL* pCh=NULL;

	// For now only support for outputs
	if ( pCdmaCtrl->m_ChannelType != DTA1XX_TS_TX_CHANNEL ) {
#if LOG_LEVEL_CDMA > 0
		DTA1XX_LOG( KERN_INFO, "Dta1xxCdmaCtrl: CDMA is only supported for outputs" );
        return -1;
#endif
}
	// For now only index 0 is supported
	if ( pCdmaCtrl->m_PortIndex != 0 ) {

#if LOG_LEVEL_CDMA > 0
		DTA1XX_LOG1( KERN_INFO, "[%d] Dta1xxCdmaCtrl: CDMA is only not supported for "
			                    "this port", pCdmaCtrl->m_PortIndex );
#endif
        return -1;
	}

	pCh = pFdo->m_TsTx[0];

	// Execute control code
	switch ( pCdmaCtrl->m_Control )
	{
		case DTA1XX_CDMA_START:
		Status = Dta1xxCdmaStart(pCh);
		break;

	case DTA1XX_CDMA_STOP:
		Status = Dta1xxCdmaStop(pCh);
		break;

	case DTA1XX_CDMA_WAITSEGM:
		Status = Dta1xxCdmaWaitSegment(pCh, pCdmaCtrl);
		break;

	case DTA1XX_CDMA_GETBUFPOS:
		Status = Dta1xxCdmaGetBufPos(pCh, pCdmaCtrl);
		break;

	default:
#if LOG_LEVEL_CDMA > 0
		DTA1XX_LOG2( KERN_INFO, "[%d] Dta1xxCdmaCtrl: Control=%d UNKNOWN",
					 pCdmaCtrl->m_PortIndex, pCdmaCtrl->m_Control );
#endif
		return -1;
	}
	return Status;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxCdmaStart -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Get estimate of current read position in the CDMA buffer.
// Method: read reference-clock counter register, and extrapolate using the reference-
// clock count samples at the last segment interrupt
//
int  Dta1xxCdmaStart(CHANNEL* pCh)
{
	int Status=0, Pci905xDmaMode, Pci905xDmaDescPtr, Pci905xDmaCommandStat;
	UInt32 DescAddr;

	// CDMA starts with reading segment 0
	// In device extension, intialise CDMA segment# currently being DMA-ed to 0, and
	// get associated reference-clock sample
	UInt32*  pRefClk = (UInt32*) pCh->m_pFdo->m_pGenRegs;
	pCh->m_CdmaSegment = 0;
	pCh->m_CdmaRefClkSamp = Dta1xxGenGetRefClkCntReg((char*)pRefClk);

	// CDMA may not be running already
	if (pCh->m_CdmaDmaRunning)
	{
#if LOG_LEVEL_CDMA > 0
		DTA1XX_LOG1( KERN_INFO, "[%d] Dta1xxCdmaStart: FAIL: CDMA ALREADY RUNNING",
				     pCh->m_nChanIdx );
#endif
		return -1;
	}
#if LOG_LEVEL_CDMA > 0
	DTA1XX_LOG3( KERN_INFO, "[%d] Dta1xxCdmaStart: RefClk=%u, PLX Channel=%d",
			     pCh->m_nChanIdx, (UInt)(pCh->m_CdmaRefClkSamp), pCh->m_Dma.m_PlxChannel);
#endif

	// Check which PLX DMA channel we should use and init address helpers
	if ( pCh->m_Dma.m_PlxChannel == 0 )
	{
		Pci905xDmaMode = PCI905X_DMA0_MODE;
		Pci905xDmaDescPtr = PCI905X_DMA0_DESC_PTR;
		Pci905xDmaCommandStat = PCI905X_DMA0_COMMAND_STAT;
	}
	else /*if ( pCh->m_Dma.m_PlxChannel == 1 )*/
	{
		Pci905xDmaMode = PCI905X_DMA1_MODE;
		Pci905xDmaDescPtr = PCI905X_DMA1_DESC_PTR;
		Pci905xDmaCommandStat = PCI905X_DMA1_COMMAND_STAT;
	}

	// Enable Scatter/Gather Mode in PCI 9054/9056
	WRITE_LONG(PCI905X_DMAMODE_BUSWIDTH_32  | PCI905X_DMAMODE_READY_INPEN |
			   PCI905X_DMAMODE_LOCALBURST	| PCI905X_DMAMODE_SCATGATH    |
			   PCI905X_DMAMODE_LOC_CONST	|
			   PCI905X_DMAMODE_DEMAND		| PCI905X_DMAMODE_INT2PCI,
			   pCh->m_pFdo->m_Pci905XConfRegs.m_pKernel,
			   Pci905xDmaMode);

	// Write pointer to first SGL in DMA-Channel-x-Descriptor-Pointer register
	DescAddr = pCh->m_CdmaShBuf.m_PhysAddrSglBuf + pCh->m_CdmaShBuf.m_FirstDescOffset;
	WRITE_LONG(DescAddr | PCI905X_DMADPR_DESCLOC_PCI,
			   pCh->m_pFdo->m_Pci905XConfRegs.m_pKernel,
			   Pci905xDmaDescPtr);

	// Start transfer!
	WRITE_BYTE(PCI905X_DMACSR_ENABLE | PCI905X_DMACSR_START,
			   pCh->m_pFdo->m_Pci905XConfRegs.m_pKernel,
			   Pci905xDmaCommandStat);

	// Mark CDMA as running in device extension
	pCh->m_CdmaDmaRunning = TRUE;

	return Status;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxCdmaStop -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
int  Dta1xxCdmaStop(CHANNEL* pCh)
{
	int Status=0, Pci905xDmaCommandStat;
#if LOG_LEVEL_CDMA > 2
	DTA1XX_LOG1( KERN_INFO, "[%d] Dta1xxCdmaStop: Abort CDMA transfer",
				 pCh->m_nChanIdx);
#endif

	// CDMA must be running
	if (!pCh->m_CdmaDmaRunning) {
#if LOG_LEVEL_CDMA > 0
		DTA1XX_LOG1( KERN_INFO, "[%d] Dta1xxCdmaStop: FAIL: CDMA NOT RUNNING",
				     pCh->m_nChanIdx);
#endif
		return -1;
	}

        // Check which PLX DMA channel we should use and init address helpers
        if ( pCh->m_Dma.m_PlxChannel == 0 )
                Pci905xDmaCommandStat = PCI905X_DMA0_COMMAND_STAT;
        else /*if ( pCh->m_Dma.m_PlxChannel == 1 )*/
                Pci905xDmaCommandStat = PCI905X_DMA1_COMMAND_STAT;

        // Disable and abort transfer
        WRITE_BYTE( PCI905X_DMACSR_ABORT,
                            pCh->m_pFdo->m_Pci905XConfRegs.m_pKernel,
                            Pci905xDmaCommandStat );

        // Mark CDMA as "not-running" in device extension
        pCh->m_CdmaDmaRunning = FALSE;

	return Status;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxCdmaWaitSegment -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Wait for the segment interrupt, with time-out.
// pCh->m_CdmaSegment is set to the segment that is currently being read (=last completed
// segment + 1)
//
int  Dta1xxCdmaWaitSegment(
	CHANNEL* pCh,
	DTA1XX_CDMA_CONTROL* pCdmaCtrl)
{
	int Status=0, ret, Pci905xDmaCommandStat;

#if LOG_LEVEL_CDMA > 0
	DTA1XX_LOG1(KERN_INFO, "[%d] Dta1xxCdmaWaitSegment: Wait for segment interrupt\n",
			    pCh->m_nChanIdx);
#endif

	// Wait for segment interrupt
	ret = wait_event_interruptible(  pCh->m_CdmaEvent,
									(test_bit(0, (void*)&pCh->m_CdmaDone)) );
	// Reset CDMA done flag
	clear_bit( 0, (void*)&pCh->m_CdmaDone);

	// Were we interrupted by a signal?
	if ( ret!=0 /*-ERESTARTSYS*/ )
	{
		// Have to cancel the current DMA

                // Check which PLX DMA channel we should use and init address helpers
                if ( pCh->m_Dma.m_PlxChannel == 0 )
                        Pci905xDmaCommandStat = PCI905X_DMA0_COMMAND_STAT;
                else /*if ( pCh->m_Dma.m_PlxChannel == 1 )*/
                        Pci905xDmaCommandStat = PCI905X_DMA1_COMMAND_STAT;

                // Disable and abort transfer
                WRITE_BYTE( PCI905X_DMACSR_ABORT,
                            pCh->m_pFdo->m_Pci905XConfRegs.m_pKernel,
                            Pci905xDmaCommandStat );

		Status = -1;
		pCdmaCtrl->m_Segment = -1;		// For safety
	}
	else
	{
		// Get current segment# and associated reference-clock sample
		pCdmaCtrl->m_Segment = pCh->m_CdmaSegment;
		pCdmaCtrl->m_RefClkSamp = pCh->m_CdmaRefClkSamp;
	}
	return Status;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxCdmaGetBufPos -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
int  Dta1xxCdmaGetBufPos(CHANNEL* pCh, DTA1XX_CDMA_CONTROL* pCdmaCtrl)
{
	int Status=0, RefClkDiff, SegmA, SegmB;
	UInt32 RefClkSamp;
        Int64 Offset;
	UInt32* pRefClk = (UInt32*)pCh->m_pFdo->m_pGenRegs;

	// Get sample of the reference clock register
	RefClkSamp = Dta1xxGenGetRefClkCntReg((char*)(pRefClk));

	// Get CDMA-buffer segment from which we currently read, as well as the value of
	// the reference clock at the last segment interrupt
	// This code may be interrupted by the segment interrupt, so read segment twice
	// and loop until both values are equal
	do {
		SegmA = pCh->m_CdmaSegment;
		RefClkDiff = RefClkSamp - pCh->m_CdmaRefClkSamp;
		SegmB = pCh->m_CdmaSegment;		// Read again
	} while (SegmA != SegmB);

	// Compute position in CDMA buffer

        // 64-bit division are not allowed in teh kernel => use bindiv function
        Offset = (Int64)RefClkDiff * (Int64)(pCdmaCtrl->m_ModSampRate);
	// do_div( Offset, (Int64)6250000 );
	Offset = bindiv(Offset,	(Int64)6250000, NULL );

	pCdmaCtrl->m_BufPos =  pCh->m_CdmaSegmDesc[SegmA].m_CdmaRelAddr + (Int)(Offset);

	// Check whether we have wrapped past the end of the CDMA buffer
	if (pCdmaCtrl->m_BufPos >= pCh->m_CdmaBufSize)
		pCdmaCtrl->m_BufPos -= pCh->m_CdmaBufSize;

	// CDMA not running => set BufPos to 0
	if (!pCh->m_CdmaDmaRunning)
		pCdmaCtrl->m_BufPos = 0;

#if LOG_LEVEL_CDMA > 2
	DTA1XX_LOG4(KERN_INFO,
				"[%d] Dta1xxCdmaGetBufPos RefClk=%u Segment=%d RefClkDiff=%u",
				pCh->m_nChanIdx, (UInt)(RefClkSamp), SegmA, RefClkDiff);
#endif
#if LOG_LEVEL_CDMA > 1
	DTA1XX_LOG3(KERN_INFO, "[%d] Dta1xxCdmaGetBufPos: BufPos=%d, ModSampRate=%d",
                                pCh->m_nChanIdx, pCdmaCtrl->m_BufPos,
                                pCdmaCtrl->m_ModSampRate);
#endif
	return Status;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxCdmaSglCreate -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
int  Dta1xxCdmaSglCreate(CHANNEL* pCh)
{
    Int i, Segment, AccuLength, Temp, BlockSize, SglSize=0;
	UInt32* pDesc=NULL;
	UInt32 PhysAddrSglFirst, PhysAddrSgl, PciAddr;
	ShBufDesc* pShBufDesc = &(pCh->m_CdmaShBuf);

#if LOG_LEVEL_CDMA > 0
    DTA1XX_LOG1( KERN_INFO, "[%d] Dta1xxCdmaSglCreate: Creating CDMA scatter-gather-list",
                 pCh->m_nChanIdx );
#endif


	// Scatter-gather list must be alligned to 16-bytes => allocate 16 extra bytes
	SglSize = (pShBufDesc->m_NumPages * (4*sizeof(UInt32))) + 16;

	// If we already have a scatter-gather buffer release it
	if ( pShBufDesc->m_SglAllocated != FALSE && pShBufDesc->m_pSglBuf != NULL )
        {
#if LOG_LEVEL_CDMA > 2
            DTA1XX_LOG1( KERN_INFO, "[%d] Dta1xxCdmaSglCreate: Freeing previous SGL-buffer",
                         pCh->m_nChanIdx );
 #endif

		pci_free_consistent(
                        pCh->m_pFdo->m_pPdo,
                        pShBufDesc->m_SglBufSize,
                        pShBufDesc->m_pSglBuf,
                        pShBufDesc->m_PhysAddrSglBuf);
        }

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Allocate buffer -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	pShBufDesc->m_pSglBuf = NULL;
	pShBufDesc->m_SglBufSize = 0;
	pShBufDesc->m_FirstDescOffset = 0;
	pShBufDesc->m_SglAllocated = FALSE;

	pShBufDesc->m_pSglBuf = (UInt8*)pci_alloc_consistent(pCh->m_pFdo->m_pPdo, SglSize,
														 &pShBufDesc->m_PhysAddrSglBuf);
	if ( pShBufDesc->m_pSglBuf == NULL ) {
#if LOG_LEVEL_CDMA > 0
		DTA1XX_LOG( KERN_INFO, "Dta1xxCdmaSglCreate: Failed to allocate SGL buffer");
#endif
          return -1;
	}
	pShBufDesc->m_SglBufSize = SglSize;

	// Compute offset to for first descriptor (16-byte alligned)
	pShBufDesc->m_FirstDescOffset = (((Int)(pShBufDesc->m_pSglBuf) + 0xF) & ~0xF)
												- (Int)(pShBufDesc->m_pSglBuf);

	//-.-.-.-.-.-.-.-.-.-.-.-.- Create descriptor for all pages -.-.-.-.-.-.-.-.-.-.-.-.-.

	Temp = pCh->m_CdmaBufSize;
	// Get first descriptor
	pDesc = (UInt32*)(pShBufDesc->m_pSglBuf + pShBufDesc->m_FirstDescOffset);
	PhysAddrSglFirst = pShBufDesc->m_PhysAddrSglBuf + pShBufDesc->m_FirstDescOffset;
	PhysAddrSgl = PhysAddrSglFirst;

	for ( i=0; i<pShBufDesc->m_NumPages; i++ )
	{
		if ( Temp >= PAGE_SIZE )
		{
			BlockSize = PAGE_SIZE;
		}
		else
            BlockSize = Temp;
		Temp -= BlockSize;

        PciAddr = pci_map_page(
				pCh->m_pFdo->m_pPdo,                    // Physical device object
				pShBufDesc->m_PageList[i],		// Page
				0,								// Offset
                BlockSize,
				0);								// Direction

		// Set PCI address
		pDesc[0]	= PciAddr;
		// Set local address
		pDesc[1]	= (UInt32)(  pCh->m_pRegBase + DTA1XX_RX_REG_FIFO_FIRST
							   - pCh->m_pFdo->m_Dta1xxRegsBase.m_pKernel );
		// Set transfer count
		pDesc[2]	= BlockSize;

		// Set next desc pointer
		PhysAddrSgl	+= 4*sizeof(UInt32);
		// Check for last (make cyclic)
		if ( Temp <= 0 )
			pDesc[3]	= PhysAddrSglFirst | PCI905X_DMADPR_DESCLOC_PCI;
		else
			pDesc[3]	= PhysAddrSgl | PCI905X_DMADPR_DESCLOC_PCI;

#if LOG_LEVEL_CDMA > 2
                DTA1XX_LOG5( KERN_INFO, "SGL Descriptor [%d]: PCI=0x%08X, Local=0x%08X, "
                                        "Size=%d, Next=0x%08X",
                                         i,
                                         (UInt)(pDesc[0]), (UInt)(pDesc[1]),
                                         (UInt)(pDesc[2]), (UInt)(pDesc[3]) );
#endif
                // Move to next desc position
                pDesc+=4;

	}
#if LOG_LEVEL_CDMA > 1
    pDesc = (UInt32*)(pShBufDesc->m_pSglBuf + pShBufDesc->m_FirstDescOffset);
    DTA1XX_LOG4( KERN_INFO, "First SGL Descriptor: PCI=0x%08X, Local=0x%08X, "
                                        "Size=%d, Next=0x%08X",
                                         (UInt)(pDesc[0]), (UInt)(pDesc[1]),
                                         (UInt)(pDesc[2]), (UInt)(pDesc[3]) );
#endif

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Init CDMA segments -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	// Get first descriptor
	pDesc = (UInt32*)(pShBufDesc->m_pSglBuf + pShBufDesc->m_FirstDescOffset);

	// CDMA buffer is divided into 4 segments 0..3
	// Start with CDMA segment# 0
	Segment = 0;
	pCh->m_CdmaSegmDesc[0].m_CdmaRelAddr = 0;
	pCh->m_CdmaSegmDesc[0].m_SglAddr = PhysAddrSglFirst;
	PhysAddrSgl = PhysAddrSglFirst;
	AccuLength = 0;

	for ( i=0; i<pShBufDesc->m_NumPages; i++ )
	{
		// Update accumulated size
		AccuLength += pDesc[2];

		// Determine if we have passed a segment boundary
		// Segment boundaries are at 1/4, 2/4, 3/4 and end of the CDMA buffer
		// For each segment boundary:
		//  - Set "Interrupt after Terminal Count" bit, to generate interrupts at
		//	  these time instances
		//  - Save Scatter-Gather-List address
		if ( AccuLength >= (Segment+1) * pCh->m_CdmaBufSize / 4) {

			// Set interrupt flag
			pDesc[3] |= PCI905X_DMADPR_INTAFTERTC;
			if (Segment < 3) {
				pCh->m_CdmaSegmDesc[Segment+1].m_CdmaRelAddr = AccuLength;
				pCh->m_CdmaSegmDesc[Segment+1].m_SglAddr = PhysAddrSgl;

#if LOG_LEVEL_CDMA > 1
                        DTA1XX_LOG5( KERN_INFO, "Set Segment [%d] boundary:\n"
                                                " SGL Descriptor: PCI=0x%08X, Local=0x%08X, "
                                                "Size=%d, Next=0x%08X",
                                                Segment,
                                                (UInt)(pDesc[0]), (UInt)(pDesc[1]),
                                                (UInt)(pDesc[2]), (UInt)(pDesc[3]) );
#endif

			}
            Segment++;
		}

		// Move to next descriptor
		pDesc += 4;
		PhysAddrSgl += 4*sizeof(UInt32);
	}
	return 0;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxCdmaSglCleanUp -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
int  Dta1xxCdmaSglCleanUp(CHANNEL* pCh)
{
	Int i, BlockSize;
	UInt32 PciAddr;
	UInt32* pDesc=NULL;
	ShBufDesc* pShBufDesc = &(pCh->m_CdmaShBuf);

#if LOG_LEVEL_CDMA > 0
    DTA1XX_LOG1( KERN_INFO, "[%d] Dta1xxCdmaSglCleanUp: Clean-up CDMA scatter-gather-list",
                 pCh->m_nChanIdx );
#endif


	// Do we have a SGL list to release?
	if ( pCh->m_CdmaShBuf.m_SglAllocated == FALSE )
        {
#if LOG_LEVEL_CDMA > 0
              DTA1XX_LOG1( KERN_INFO, "[%d] Dta1xxCdmaSglCleanUp: No CDMA sgl allocated",
                            pCh->m_nChanIdx );
#endif

		return 0; // Nothing to free
        }

	// Get first descriptor
	pDesc = (UInt32*)(pShBufDesc->m_pSglBuf + pShBufDesc->m_FirstDescOffset);
	// Loop through pages and unmap them
	for ( i=0; i<pShBufDesc->m_NumPages; i++ )
	{
		PciAddr = pDesc[0];
		BlockSize = pDesc[2];

		pci_unmap_page(
				pCh->m_pFdo->m_pPdo,			// Physical device object
				PciAddr,						// PCI bus address
				BlockSize,						// Block size
				0);								// Direction

		// Move to next descriptor
		pDesc+=4;
	}

    // Free buffer allocated for scatter-gather-descriptors
    pci_free_consistent(
			pCh->m_pFdo->m_pPdo,
			pShBufDesc->m_SglBufSize,
			pShBufDesc->m_pSglBuf,
			pShBufDesc->m_PhysAddrSglBuf);
    return 0;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- bindiv -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
// 
// 64 bit division  returns  num/denom and rest 
//
Int64 bindiv (Int64 num, Int64 denom, Int64* pRest) 
{ 

    int denombits, numbits ;
    Int64 restbits, restmask, rest; 
    Int64 answer = 0; 
    int sign = 0;

    if (num < 0) { 
        num = -num; 
        sign = 1; 
    } 
    if (denom < 0) { 
        denom = -denom; 
        sign ^= 1; 
    } 

    if (num < denom) 
        return 0; 

    denombits = 0;
    while ((Int64)1 << denombits < denom) 
        denombits++; 

    numbits = denombits; 
    while ((Int64)1 << numbits < num) 
        numbits++; 

    restbits = numbits - denombits; 
    restmask = ((Int64)1 << restbits) - 1; 
    rest = num >> restbits; 
    num = num & restmask; 

    while (1) 
    { 
        answer <<= 1; 
        if (rest >= denom) 
        { 
            answer |= 1; 
            rest -= denom; 
        } 

        if (restbits == 0) 
            break; 
        restbits--; 
        restmask >>= 1; 
        rest = (rest << 1) | (num >> restbits); 
        num = num & restmask; 
    } 

    if (pRest) 
        *pRest = rest; 

    if (sign) 
        return -answer; 
    return answer; 
} 
