//*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* Dta1xx.c *#*#*#*#*#*#*#*#*# (C) 2000-2005 DEKTEC

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- License -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// This program is free software; You can freely use/modify this source code in any way
// you desire, including for commercial applications.
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	TD	2007.01.29		Added support for DTA-160 ASI-only
//	TD	23-01-2007	CDMA Buffer cleanup added i.b.o. DTA110in Dta1xx release
//	KvR	14-08-2006	DTA-105 registers changed, Vpd-data handling changed
//	JH	13-07-2005	Added support for DTA-110 and DTA-105/545 (non-PLX)
//	MG	26-10-2004	Added support for DTA-107 and initial support for DTA-124
//	JH	16-08-2004	Made code compatible with kernel 2.6
//	MG	06-05-2003	Added #include <linux/init.h>
//	MG	08-04-2003	Added MODULE_LICENSE("GPL") to prevent tainted message when loading
//					the module
//	MG	15-01-2003	Added search for devices with a PLX9056 (i.e. DTA-140)
//	MG	21-10-2002	Created

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#include <linux/kernel.h>	// printk() stuff
#include <linux/module.h>	// MODULE_PARM
#include <linux/init.h>
#include <linux/version.h>

#include "../Include/Dta1xx.h"
#include "../Include/Dta1xxRegs.h"
#include "../Include/DriverVersion.h"

#ifndef CONFIG_PCI
	#error "PCI support is required"
#endif

// Detect kernel version and check if supported
#ifndef KERNEL_VERSION
	#error "Only kernel 2.4 or higher is currently supported"
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
	#error "Only kernel 2.4 or higher is currently supported"
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0) && LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
	#define KERNELVERSION 24
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0) && LINUX_VERSION_CODE < KERNEL_VERSION(2,7,0)
	#define KERNELVERSION 26
#endif

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Type definitions -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Internal function declarations -.-.-.-.-.-.-.-.-.-.-.-.-.-.

//-.-.-.-.-.-.-.-.-.-.-.-.-.- Data declaration and definition -.-.-.-.-.-.-.-.-.-.-.-.-.-.

MODULE_AUTHOR("DEKTEC Digital Video B.V.");
MODULE_DESCRIPTION("Dta1xx series driver");

// ADDED TO PREVENT TAINTED MESSAGE. THIS MODULE CAN BE USED FREELY (SEE LICENSE
// STATEMENT ABOVE).
MODULE_LICENSE("GPL");

int g_nDta1xxMajor	= DTA1XX_MAJOR;		// Driver major number

// The file operations
struct file_operations g_Dta1xxFOps = {
open: 		Dta1xxOpen,
release:	Dta1xxRelease,
ioctl:		Dta1xxIoCtl
};

DTA1XX_FDO	g_Dta1xxCards[DTA1XX_MAX_NMB_CARDS];	// DTA-1xx card data
int			g_nNumOfCards = 0;						// Number of DTA-1xx cards found in the system

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Implementation +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxInitModule -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Initialise the module. Registers the module. Scans the system for the available DTA-1xx
// cards and registers the driver and allocates all needed resources.
//
int Dta1xxInitModule()
{
	int r=0;
	struct pci_dev* pPciCard = NULL;

#ifdef DEBUG
	DTA1XX_LOG(KERN_DEBUG, "Initialising module");
#endif

	#if KERNELVERSION < 26
	// NO SYMBOLS ARE EXPORTED (kernel 2.4 only)
	EXPORT_NO_SYMBOLS;

	//-.-.-.-.-.-.-.-. Check for availabilty of PCI bus (kernel 2.4 only) .-.-.-.-.-.-.-.-
	if ( !pci_present() ) {
		DTA1XX_LOG(KERN_DEBUG, "No PCI-bus available");
		return -EFAULT;
	}
	#endif

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA-1XX array -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	memset( g_Dta1xxCards, 0, ( sizeof(DTA1XX_FDO) * DTA1XX_MAX_NMB_CARDS ) );
	g_nNumOfCards = 0;

	//-.-.-.-.-.-.-.-.-.-.- Scan system for DTA-1XX series PCI cards -.-.-.-.-.-.-.-.-.-.-

	// Scan for devices with 9054
	pPciCard = pci_find_device(PLX_VENDORID, PLX9054_DEVICEID, NULL);

	while ( (pPciCard!=NULL) && (g_nNumOfCards<DTA1XX_MAX_NMB_CARDS) ) {

		// Check that this is a DEKTEC card
		if (    ( pPciCard->subsystem_vendor==DTA1XX_VENDORID )
			 && (    pPciCard->subsystem_device==DTA100_DEVICEID
			      || pPciCard->subsystem_device==DTA102_DEVICEID
				  || pPciCard->subsystem_device==DTA120_DEVICEID
				  || pPciCard->subsystem_device==DTA122_DEVICEID ) )
		{
			if ( 0==Dta1xxStartCard(pPciCard, &g_Dta1xxCards[g_nNumOfCards]) )
			{
				DTA1XX_LOG2(KERN_INFO,
					"The DTA-%d (Firmware Version %d) has started successfully.",
					g_Dta1xxCards[g_nNumOfCards].m_nTypeNumber,
					g_Dta1xxCards[g_nNumOfCards].m_nFirmwareVersion);
			}
			g_nNumOfCards++;
		}
		// Get next card
		pPciCard = pci_find_device(PLX_VENDORID, PLX9054_DEVICEID, pPciCard);
	}

	// Scan for devices with 9056
	pPciCard = NULL;
	pPciCard = pci_find_device(PLX_VENDORID, PLX9056_DEVICEID, NULL);

	while ( (pPciCard!=NULL) && (g_nNumOfCards<DTA1XX_MAX_NMB_CARDS) ) {

		// Check that this is a DEKTEC card
		if ( (    ( pPciCard->subsystem_vendor==DTA1XX_VENDORID )
			   && (    pPciCard->subsystem_device==DTA107_DEVICEID
			        || pPciCard->subsystem_device==DTA110_DEVICEID
			        || pPciCard->subsystem_device==DTA124_DEVICEID
			        || pPciCard->subsystem_device==DTA140_DEVICEID) )
			// Or DekTec card with vendor ID
			|| (  ( pPciCard->subsystem_vendor==DTA1XX_VENDORID_DEKTEC )
		 	   && (    pPciCard->subsystem_device==DTA160_DEVICEID)))
		{
			if ( 0==Dta1xxStartCard(pPciCard, &g_Dta1xxCards[g_nNumOfCards]) )
			{
				DTA1XX_LOG2(KERN_INFO,
					"The DTA-%d (Firmware Version %d) has started successfully.",
					g_Dta1xxCards[g_nNumOfCards].m_nTypeNumber,
					g_Dta1xxCards[g_nNumOfCards].m_nFirmwareVersion);
			}
			g_nNumOfCards++;
		}
		// Get next card
		pPciCard = pci_find_device(PLX_VENDORID, PLX9056_DEVICEID, pPciCard);
	}

	// Scan for DTA-105s
	pPciCard = NULL;
	pPciCard = pci_find_device(DTA1XX_VENDORID_DEKTEC, DTA105_DEVICEID, pPciCard);

	while ( (pPciCard!=NULL) && (g_nNumOfCards<DTA1XX_MAX_NMB_CARDS) ) {

		if ( 0==Dta1xxStartCard(pPciCard, &g_Dta1xxCards[g_nNumOfCards]) )
		{
			DTA1XX_LOG2(KERN_INFO,
				"The DTA-%d (Firmware Version %d) has started successfully.",
				g_Dta1xxCards[g_nNumOfCards].m_nTypeNumber,
				g_Dta1xxCards[g_nNumOfCards].m_nFirmwareVersion);
		}
		g_nNumOfCards++;

		// look for more DTA-105s
		pPciCard = pci_find_device(DTA1XX_VENDORID_DEKTEC, DTA105_DEVICEID, pPciCard);
	}

	// Scan for DTA-145s
	pPciCard = NULL;
	pPciCard = pci_find_device(DTA1XX_VENDORID_DEKTEC, DTA145_DEVICEID, pPciCard);

	while ( (pPciCard!=NULL) && (g_nNumOfCards<DTA1XX_MAX_NMB_CARDS) ) {

		if ( 0==Dta1xxStartCard(pPciCard, &g_Dta1xxCards[g_nNumOfCards]) )
		{
			DTA1XX_LOG2(KERN_INFO,
				"The DTA-%d (Firmware Version %d) has started successfully.",
				g_Dta1xxCards[g_nNumOfCards].m_nTypeNumber,
				g_Dta1xxCards[g_nNumOfCards].m_nFirmwareVersion);
		}
		g_nNumOfCards++;

		// look for more DTA-145s
		pPciCard = pci_find_device(DTA1XX_VENDORID_DEKTEC, DTA145_DEVICEID, pPciCard);
	}

	// Scan for DTA-2145s
	pPciCard = NULL;
	pPciCard = pci_find_device(DTA1XX_VENDORID_DEKTEC, DTA2145_DEVICEID, pPciCard);

	while ( (pPciCard!=NULL) && (g_nNumOfCards<DTA1XX_MAX_NMB_CARDS) ) {

		if ( 0==Dta1xxStartCard(pPciCard, &g_Dta1xxCards[g_nNumOfCards]) )
		{
			DTA1XX_LOG2(KERN_INFO,
				"The DTA-%d (Firmware Version %d) has started successfully.",
				g_Dta1xxCards[g_nNumOfCards].m_nTypeNumber,
				g_Dta1xxCards[g_nNumOfCards].m_nFirmwareVersion);
		}
		g_nNumOfCards++;

		// look for more DTA-2145s
		pPciCard = pci_find_device(DTA1XX_VENDORID_DEKTEC, DTA2145_DEVICEID, pPciCard);
	}


	// Scan for DTA-545s
	pPciCard = NULL;
	pPciCard = pci_find_device(DTA1XX_VENDORID, DTA545_DEVICEID, pPciCard);

	while ( (pPciCard!=NULL) && (g_nNumOfCards<DTA1XX_MAX_NMB_CARDS) ) {

		if ( 0==Dta1xxStartCard(pPciCard, &g_Dta1xxCards[g_nNumOfCards]) )
		{
			DTA1XX_LOG2(KERN_INFO,
				"The DTA-%d (Firmware Version %d) has started successfully.",
				g_Dta1xxCards[g_nNumOfCards].m_nTypeNumber,
				g_Dta1xxCards[g_nNumOfCards].m_nFirmwareVersion);
		}
		g_nNumOfCards++;

		// look for more DTA-545s
		pPciCard = pci_find_device(DTA1XX_VENDORID, DTA545_DEVICEID, pPciCard);
	}

	DTA1XX_LOG1(KERN_INFO, "Found %d DTA-1xx boards in the system", g_nNumOfCards);

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Register driver -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	r = register_chrdev(g_nDta1xxMajor, DRIVER_NAME, &g_Dta1xxFOps);
	if ( 0 >= r) {
		DTA1XX_LOG1(KERN_CRIT, "Cannot get major %d", g_nDta1xxMajor);
	}

	if ( g_nDta1xxMajor==0 ) {
		// Dynamic major used ==> store returned number
		g_nDta1xxMajor = r;
		DTA1XX_LOG1(KERN_DEBUG, "Major: %d", g_nDta1xxMajor);
	}

#if KERNELVERSION < 26
	// Set the module owner (kernel 2.4 only)
	SET_MODULE_OWNER(&g_Dta1xxFOps);
#endif

	DTA1XX_LOG4(KERN_INFO, "The driver (Rev %d.%d.%d.%d) has loaded successfully.",
				DTA1XX_VERSION_MAJOR, DTA1XX_VERSION_MINOR, DTA1XX_VERSION_MICRO,
				DTA1XX_VERSION_BUILD);

    return 0;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxCleanUpModule -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Module clean-up. Unregisters the module. Frees the resources used for the DTA-1xx
// cards.
//
void Dta1xxCleanUpModule()
{
	int i;

	DTA1XX_LOG(KERN_DEBUG, "Module clean-up");

	for (i=0; i<g_nNumOfCards; i++) {
   		Dta1xxStopCard(&g_Dta1xxCards[i]);
	}
 	g_nNumOfCards = 0;

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Unregister driver (major) -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	unregister_chrdev(g_nDta1xxMajor, DRIVER_NAME);
}
module_init(Dta1xxInitModule);
module_exit(Dta1xxCleanUpModule);

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxStartCard -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Initialisises the DTA-1xx card described by pPdo and allocates all resources needed to
// control the card.
//
int Dta1xxStartCard(
	struct pci_dev* pPdo,	// Physical device object, representing the DTA-1xx card
	DTA1XX_FDO* pFdo)		// Functional device object, representing the DTA-1xx card
{
	int i=0;

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Initialise FDO -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	// Store pointer to PCI device structure
	pFdo->m_pPdo = pPdo;

	// Determine whether this PCI card uses a PLX bridge chip (PCI9054/9056)
	pFdo->m_nUsesPlxChip = (pPdo->vendor == 0x10B5) ? 1 : 0;

        // Determine type of DTA-1xx card (from Subsystem ID or Device ID, depending on PLX)
	pFdo->m_nTypeNumber = Dta1xxDeviceId2TypeNumber(pFdo->m_nUsesPlxChip ?
		pPdo->subsystem_device : pPdo->device);

	// Enable the card
	pci_enable_device(pFdo->m_pPdo);
	pci_set_master(pFdo->m_pPdo);

	// Aquire resources
	if ( 0!=pci_request_regions(pFdo->m_pPdo, DRIVER_NAME) ) {
		DTA1XX_LOG(KERN_CRIT, "Failed to get resources");
		return -EFAULT;
	}

	// Memory mapping for devices with PLX chip (BAR0 and BAR2)
	if (pFdo->m_nUsesPlxChip) {
		// Get PLX address range (BAR0)
		pFdo->m_Pci905XConfRegs.m_uiPciAddr		= pci_resource_start(pFdo->m_pPdo, 0);
		pFdo->m_Pci905XConfRegs.m_uiLength		= pci_resource_len(pFdo->m_pPdo, 0);
		// Map into kernel space
		pFdo->m_Pci905XConfRegs.m_pKernel		= ioremap_nocache(pFdo->m_Pci905XConfRegs.m_uiPciAddr,
			pFdo->m_Pci905XConfRegs.m_uiLength);

		if ( (pFdo->m_Pci905XConfRegs.m_pKernel==NULL)) {
			Dta1xxStopCard(pFdo);
			return -EFAULT;
		}

		// Get DTA1xx operational address range (BAR2)
		pFdo->m_Dta1xxRegsBase.m_uiPciAddr		= pci_resource_start(pFdo->m_pPdo, 2);
		pFdo->m_Dta1xxRegsBase.m_uiLength		= pci_resource_len(pFdo->m_pPdo, 2);
		// Map into kernel space
		pFdo->m_Dta1xxRegsBase.m_pKernel		= ioremap_nocache(pFdo->m_Dta1xxRegsBase.m_uiPciAddr,
			pFdo->m_Dta1xxRegsBase.m_uiLength);

		if ( (pFdo->m_Dta1xxRegsBase.m_pKernel==NULL)) {
			Dta1xxStopCard(pFdo);
			return -EFAULT;
		}
	}
	// Memory mapping for devices without PLX chip (only BAR0)
	else {
		// Get DTA1xx memory address range (BAR0)
		pFdo->m_Dta1xxRegsBase.m_uiPciAddr		= pci_resource_start(pFdo->m_pPdo, 0);
		pFdo->m_Dta1xxRegsBase.m_uiLength		= pci_resource_len(pFdo->m_pPdo, 0);
		// Map into kernel space
		pFdo->m_Dta1xxRegsBase.m_pKernel		= ioremap_nocache(pFdo->m_Dta1xxRegsBase.m_uiPciAddr,
			pFdo->m_Dta1xxRegsBase.m_uiLength);
#if LOG_LEVEL >1			
		DTA1XX_LOG2(KERN_CRIT, "PhysAddr: %x Length: %x", 
			pFdo->m_Dta1xxRegsBase.m_uiPciAddr,
			pFdo->m_Dta1xxRegsBase.m_uiLength);
#endif
		if ( (pFdo->m_Dta1xxRegsBase.m_pKernel==NULL)) {
			Dta1xxStopCard(pFdo);
			return -EFAULT;
		}
	}

	//+=+=+=+=+=+=+=+=+=+ FROM HERE ON WE CAN ACCESS DTA-1XX REGISTERS +=+=+=+=+=+=+=+=+=+

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Initialise Hardware -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	// Initialise PCI-9054/9056 (BAR0 must be mapped) if PLX chip is present
        if (pFdo->m_nUsesPlxChip) {
			Dta1xxInitPlx(pFdo);			// Cannot fail...
		}

	// Init device
	if ( 0!=Dta1xxInitDevice(pFdo) ) {
		Dta1xxStopCard(pFdo);
		return DTA1XX_E_DEV_DRIVER;
	}

	// Initialise Target Adapter Detection
	if (pFdo->m_nTypeNumber==102) {
		pFdo->m_TsTx[0]->m_TrgIdDet.m_nTdState = DTA1XX_TD_INITIAL;
		pFdo->m_TsTx[0]->m_TrgIdDet.m_nTargetError = 0;
	}
	else if (pFdo->m_nTypeNumber==122) {
		pFdo->m_TsRx[0]->m_TrgIdDet.m_nTdState = DTA1XX_TD_INITIAL;
		pFdo->m_TsRx[0]->m_TrgIdDet.m_nTargetError = 0;
	}

	// Initialise Bit-Rate Measurement
	// Init all TX channels
	for ( i=0; i<pFdo->m_nNumTxChannels; i++ ) {
		pFdo->m_TsTx[i]->m_BrMeas.m_nNumValidSamps = 0;	// No valid samples yet
		pFdo->m_TsTx[i]->m_BrMeas.m_nValidCount256 = 0;	// Initial bit rate is 0
	}

	// Init all RX channels
	for ( i=0; i<pFdo->m_nNumRxChannels; i++ ) {
		pFdo->m_TsRx[i]->m_BrMeas.m_nNumValidSamps = 0;	// No valid samples yet
		pFdo->m_TsRx[i]->m_BrMeas.m_nValidCount256 = 0;	// Initial bit rate is 0
	}


	//.-.-.-.-.-.-.-.-.-.-.-.-.- Register the interrupt handler -.-.-.-.-.-.-.-.-.-.-.-.-.

    // Request the interrupt
	if ( 0 > request_irq(pFdo->m_pPdo->irq,
						 (void*)Dta1xxIsr,	// interrupt service routine
						 SA_SHIRQ,	// Shared interrupt
						 DRIVER_NAME,
						 pFdo) )	// is parameter
	{
		pFdo->m_nIntAttached = 0;
		Dta1xxStopCard(pFdo);
		DTA1XX_LOG1(KERN_CRIT, "Failed register interrupt handler (IRQ: %d",
					pFdo->m_pPdo->irq);
		return -EFAULT;
	}
	// Enable interrupts
	Dta1xxEnableInterrupts(pFdo);
	pFdo->m_nIntAttached = 1;

	return 0;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxStopCard -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Stops the card and deallocates all resources used by the card
//
void Dta1xxStopCard(
	DTA1XX_FDO* pFdo)	// Functional device object, representing the DTA-1xx card
{

	int i;
	CHANNEL* pCh=NULL;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Free interrupt resources -.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	if ( 0!=pFdo->m_nIntAttached ) {
        Dta1xxDisableInterrupts(pFdo);
		free_irq(pFdo->m_pPdo->irq, pFdo);
		pFdo->m_nIntAttached = 0;
	}

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Release memory resources -.-.-.-.-.-.-.-.-.-.-.-.-.-.-

        if (pFdo->m_nUsesPlxChip) { // PLX memory release only when PLX present
            if ( pFdo->m_Pci905XConfRegs.m_pKernel!=NULL ) {
		iounmap( (char*)(pFdo->m_Pci905XConfRegs.m_pKernel) );
		pFdo->m_Pci905XConfRegs.m_pKernel = NULL;
            }
        }

	if ( pFdo->m_Dta1xxRegsBase.m_pKernel!=NULL ) {
     	iounmap( (char*)(pFdo->m_Dta1xxRegsBase.m_pKernel) );
		pFdo->m_Dta1xxRegsBase.m_pKernel = NULL;
        }

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Free DMA buffer memory -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	if ( pFdo->m_pPdo!=NULL ) {
		for (i=0; i<pFdo->m_nNumChannels; i++ ) {

			pCh = &pFdo->m_Channel[i];

			if ( pCh->m_Dma.m_DmaBuffer.m_pVirtStartAddr!=NULL ) {
				// Free buffer
				pci_free_consistent(pFdo->m_pPdo, DTA1XX_DMA_BUF_SIZE,
							(void*)pCh->m_Dma.m_DmaBuffer.m_pVirtStartAddr,
							pCh->m_Dma.m_DmaBuffer.m_PhysStartAddr);
				pCh->m_Dma.m_DmaBuffer.m_pVirtStartAddr = NULL;
			}
			// Free scatter gather list descriptor
			if ( pCh->m_Dma.m_pSglDesc != NULL ) {
				// Free buffer
				pci_free_consistent( pFdo->m_pPdo, sizeof(SglDesc),
									(void*)pCh->m_Dma.m_pSglDesc,
									pCh->m_Dma.m_PhysAddrSglDesc);
				pCh->m_Dma.m_pSglDesc = NULL;
			}

			// Donot forget the CDMA buffers
			if ( pCh->m_CdmaShBuf.m_Allocated != FALSE )
			{
				Dta1xxSharedBufRelease( pCh, i, &pCh->m_CdmaShBuf,
						    DTA1XX_SHBUF_DIRIQ_CDMA);
			}
		}
	}

    if ( pFdo->m_pPdo!=NULL ) {
        pci_release_regions(pFdo->m_pPdo);
		pFdo->m_pPdo = NULL;
    }

	// Free previously allocated memory
	if ( pFdo->m_pEepromCache != NULL ) {
		// Memory was allocated
		kfree( (char *)pFdo->m_pEepromCache );
	}

	//clear reference count 
	pFdo->m_nDvcRefCount = 0;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxOpen -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Opens a connection to a specific DTA-1xx card.
//
int Dta1xxOpen(
	struct inode *inode,
	struct file *filp)
{
	int num;
	DTA1XX_FDO* pFdo;


#if LOG_LEVEL > 1
 	DTA1XX_LOG(KERN_DEBUG, "Dta1xxOpen is called");
#endif

 	//.-.-.-.-.-.-.-.-.-.-.- Get the cards relative sequence number -.-.-.-.-.-.-.-.-.-.-.
	num = MINOR(inode->i_rdev);

 	// Do we have that many cards??
	if ( num >= g_nNumOfCards) {
		return -ENODEV;
	}
	// Store pointer to card in private structure
	filp->private_data	= &g_Dta1xxCards[num];
	
	//increment number of times opened
	pFdo = &g_Dta1xxCards[num];

	pFdo->m_nDvcRefCount++;
#if LOG_LEVEL > 1
	DTA1XX_LOG2(KERN_DEBUG, "Dta1xxOpen BoardType: %d, RefCount: %d",
		pFdo->m_nTypeNumber,
		pFdo->m_nDvcRefCount);
#endif

	#if KERNELVERSION < 26
	// Increment reference count (kernel 2.4 only)
	MOD_INC_USE_COUNT;
	#endif

	return 0;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRelease -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Releases a connection to a specific DTA-1xx card
//
int Dta1xxRelease(
	struct inode *inode,
	struct file *filp)
{
	int result = 0;
	DTA1XX_FDO* pFdo = (DTA1XX_FDO*)filp->private_data;
	CHANNEL* pCh;

#if LOG_LEVEL > 1
 	DTA1XX_LOG(KERN_DEBUG, "Dta1xxRelease is called");
#endif

	// Get associated card info
	if (pFdo != NULL) {
		// decrement number of times opened
		pFdo->m_nDvcRefCount--;
		
#if LOG_LEVEL > 1
	 	DTA1XX_LOG2(KERN_DEBUG, "Dta1xxRelease BoardType: %d, RefCount: %d",
			pFdo->m_nTypeNumber,
			pFdo->m_nDvcRefCount);
#endif		
		// DTA110 is not referenced anymore?
		if ( (pFdo->m_nDvcRefCount == 0)
		  && (pFdo->m_nTypeNumber == 110)
		  && (pFdo->m_nNumTxChannels == 1)
		  && (pFdo->m_TsTx[0] != NULL))
		{
			// Stop CDMA
			pCh = pFdo->m_TsTx[0];
			Dta1xxCdmaStop( pCh);
			
			// Released shared buffer
			result = Dta1xxSharedBufRelease(
					pCh,
					0,
					&pCh->m_CdmaShBuf,
					DTA1XX_SHBUF_DIRIQ_CDMA);

#if LOG_LEVEL > 0
			DTA1XX_LOG1(KERN_DEBUG, "Dta1xxRelease CDMA Stop/Release result %d",
				result);
#endif
		}

	}


	// Set to NULL (i.e. no card associated)
	filp->private_data	= NULL;
	


	#if KERNELVERSION < 26
 	// Decrement reference count (kernel 2.4 only)
	MOD_DEC_USE_COUNT;
	#endif

	return 0;
}
