//*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* Flags.c *#*#*#*#*#*#*#*#*#* (C) 2000-2005 DEKTEC

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- License -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// This program is free software; You can freely use/modify this source code in any way
// you desire, including for commercial applications.
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	MG	18-11-2002	Created	

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#include "../Include/Dta1xx.h"
#include "../Include/Dta1xxRegs.h"

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Type definitions -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Internal functionnIntCsr declarations -.-.-.-.-.-.-.-.-.-.-.-.-.-.

//-.-.-.-.-.-.-.-.-.-.-.-.-.- Data declaration and definition -.-.-.-.-.-.-.-.-.-.-.-.-.-.

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Implementation +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxProcessStatusFlags -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Process flags in Receive-Status register:
//	- Copy status flags from Rx Status register to channel's Status variable
//	- Update channel's Latched flags
//
// This routine is called:
//	- Every periodic interrupt (77ms).
//	- From DeviceIoControl (GetFlags), at elevated IRQL!
//
// PRECONDITION
//	- pTsRx points to a valid transmit channel!
//
void Dta1xxRxProcessStatusFlags(
	CHANNEL* pTsRx)	// Transport-Stream Receive Channel
{
	int  nStatus;
	
	nStatus = 0;
	if ( 0!=Dta1xxRxStatusRegGetOvfInt( (char*)pTsRx->m_pRegBase ) ) {
		nStatus |= DTA1XX_RX_FIFO_OVF;
		Dta1xxRxStatusRegClrOvfInt( (char*)pTsRx->m_pRegBase );
	}
	if ( 0!=Dta1xxRxStatusRegGetSyncInt( (char*)pTsRx->m_pRegBase ) ) {
		nStatus |= DTA1XX_RX_SYNC_ERR;
		Dta1xxRxStatusRegClrSyncInt( (char*)pTsRx->m_pRegBase );
	}
	if ( 0!=Dta1xxRxStatusRegGetRateOvfInt( (char*)pTsRx->m_pRegBase ) ) {
		nStatus |= DTA1XX_RX_RATE_OVF;
		Dta1xxRxStatusRegClrRateOvfInt( (char*)pTsRx->m_pRegBase );
	}

	// Process target-error flag from target-detection state machine
	if (pTsRx->m_TrgIdDet.m_nTargetError)
		nStatus |= DTA1XX_TX_TARGET_ERR;

	// TODO: DMA PENDING?!

	// Latch status flags
	pTsRx->m_nLatched |= nStatus;
	pTsRx->m_nStatus = nStatus;

	// Clear target-error flag if target-detection state machine not in state ERROR
	if (pTsRx->m_TrgIdDet.m_nTdState!=DTA1XX_TD_TARGET_ERROR)
		pTsRx->m_TrgIdDet.m_nTargetError = 0;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxProcessStatusFlags -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Process flags in Transmit-Status register:
//	- Copy status flags from Tx Status register to channel's Status variable
//	- Update channel's Latched flags
//
// This routine is called:
//	- Every periodic interrupt (77ms).
//	- From DeviceIoControl (GetFlags), at elevated IRQL!
//
// PRECONDITION
//	- pTsTx points to a valid transmit channel!
//
void Dta1xxTxProcessStatusFlags(

	CHANNEL* pTsTx)		// Transport-Stream transmit Channel
{
	int  nStatus;

	nStatus = 0;
	if ( 0!=Dta1xxTxStatusRegGetUflInt( (char*)pTsTx->m_pRegBase ) ) {
		nStatus |= DTA1XX_TX_FIFO_UFL;
		Dta1xxTxStatusRegClrUflInt( (char*)pTsTx->m_pRegBase );
		
	}
	if ( 0!=Dta1xxTxStatusRegGetSyncInt( (char*)pTsTx->m_pRegBase ) ) {
		nStatus |= DTA1XX_TX_SYNC_ERR;
		Dta1xxTxStatusRegClrSyncInt( (char*)pTsTx->m_pRegBase );
	}
	if ( 0!=Dta1xxTxStatusRegGetShortInt( (char*)pTsTx->m_pRegBase ) ) {
		nStatus |= DTA1XX_TX_READBACK_ERR;
		Dta1xxTxStatusRegClrShortInt( (char*)pTsTx->m_pRegBase );
	}
	// Process target-error flag from target-detection state machine
	if (pTsTx->m_TrgIdDet.m_nTargetError)
		nStatus |= DTA1XX_TX_TARGET_ERR;

	// TODO: DMA PENDING?!

	// Latch status flags
	pTsTx->m_nLatched |= nStatus;
	pTsTx->m_nStatus = nStatus;

	// Clear target-error flag if target-detection state machine not in state ERROR
	if (pTsTx->m_TrgIdDet.m_nTdState!=DTA1XX_TD_TARGET_ERROR)
		pTsTx->m_TrgIdDet.m_nTargetError = 0;
}
