//#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* Init.c *#*#*#*#*#*#*#*#*#* (C) 2000-2005 DEKTEC

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- License -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// This program is free software; You can freely use/modify this source code in any way
// you desire, including for commercial applications.
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	TD	2007.01.29	Addes support for DTA-160 ASI-only
//	KvR	14-08-2006	DTA-105 registers changed, Vpd-data handling changed
//  JH  13-07-2005  Added support for DTA-110 and DTA-105/545 (non-PLX)
//	MG	26-10-2004	Added support for DTA-107 and initial support for DTA-124
//	JH	16-08-2004	Added some castings to avoid compiler warnings
//	MG	30-06-2003	Added support for FW4 of DTA-120/122
//	MG	11-03-2003	Added initialisation of DMA done wait_queue
//	MG	21-10-2002	Created

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#include "../Include/Dta1xx.h"
#include "../Include/Dta1xxRegs.h"
#include <asm/bitops.h> // set_bit

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Type definitions -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Constants -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
// The default IO configuration by specifying number of RX channels.  The remaining are
// channels becom TX channels
#define ENABLE_DTA105_DBLYBUF			0		// Doubly Buffering (0 = OFF; 1 = ON)
#define NUM_DTAX145_RX_CHANNELS			1		// Number of DTA-145/2145 Rx channels (0..1)
#define NUM_DTA160_RX_CHANNELS			1		// Number of DTA-160 Rx channels (0..3)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Internal function declarations -.-.-.-.-.-.-.-.-.-.-.-.-.-.

int Dta1xxInitDmaAdapter( CHANNEL* );
int Dta1xxInitVpd( DTA1XX_FDO* );
void Dta1xxInitAsiSdi( CHANNEL*  pCh );

//-.-.-.-.-.-.-.-.-.-.-.-.-.- Data declaration and definition -.-.-.-.-.-.-.-.-.-.-.-.-.-.

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Implementation +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=


//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxInitDevice -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Initialise operation of Dta1xx device.
//
// PRE-CONDITION:
//	+ Device memory has been mapped into kernel space
//	+ PCI-9054/PCI-9056 has been initialised
//	+ pFdo->m_TypeNumber has been set
//
int Dta1xxInitDevice(
					 DTA1XX_FDO* pFdo)	// Functional device object, representing the DTA-1xx card
{
	int i, CntrlReg, Status;
	int NumRx, NumTx;
	CHANNEL* pCh = NULL;
	volatile char*  pRegBase = pFdo->m_Dta1xxRegsBase.m_pKernel;
	volatile char*  pPci905XRegs = pFdo->m_Pci905XConfRegs.m_pKernel;

	// Generic register block starts at the same address for all DEKTEC boards
	// Pointer has to be initialised here, because Dta1xxHandleUnitialisedEeprom
	// needs it
	pFdo->m_pGenRegs = pRegBase;

	// Check firmware revision
	if ( 0!=Dta1xxCheckFirmwareVersion(pFdo) ) {
		return -EFAULT;
	}

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Initialise FDO Variables -.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	// General initialisation of Channel data structure
	for (i=0; i<MAX_NUM_CHANNELS; i++) {

		// Channel pointer
		pCh = &pFdo->m_Channel[i];

		// Identity: index and pointer back to Device Extension
		pCh->m_nChanIdx = i;
		pCh->m_pFdo = pFdo;

		// Channel type starts as UNUSED
		pCh->m_nChanType = DTA1XX_CHANNEL_UNUSED;

		// Initialise register pointer for LEGACY boards
		pCh->m_pRegBase = pRegBase;

		// Set scatter/gather pointers to NULL, in case initialisation fails prematurely
		pCh->m_Dma.m_pSglDesc = NULL;
	}

	// Set default value for some of the board-dependent variables
	// These values may be overriden in the "big switch" that follows
	pFdo->m_nNumChannels = 0;
	pFdo->m_nNumRxChannels = 0;
	pFdo->m_nNumTxChannels = 0;

	pFdo->m_nUsesDmaInFpga = 0;
	pFdo->m_nUsesGenReg = 1;
	pFdo->m_nUses9054 = 0;
	pFdo->m_nUses9056 = 0;
	pFdo->m_UseFpgaForVpd = 0;
	pFdo->m_VpdEepromSize = PLX_VPD_SIZE;

	switch (pFdo->m_nTypeNumber) {

	case 100:
	case 102:	pFdo->m_nUses9054 = 1;
		pFdo->m_nUsesGenReg = 0;
		pFdo->m_nNumTxChannels = 1;
		pFdo->m_Channel[0].m_nChanType = DTA1XX_TS_TX_CHANNEL;
		pFdo->m_TsTx[0] = &pFdo->m_Channel[0];

		// Init DMA lock
		init_MUTEX(&pFdo->m_TsTx[0]->m_Dma.m_Lock);
		// Init DMA wait queue
		init_waitqueue_head( &(pFdo->m_TsTx[0]->m_Dma.m_DmaDoneQueue) );
		// Set PLX DMA channel
		pFdo->m_TsTx[0]->m_Dma.m_PlxChannel = 0;
		break;

	case 105:	pFdo->m_nUsesDmaInFpga = 1;
#if ENABLE_DTA105_DBLYBUF == 1
		NumTx = 1;
#else
		NumTx = 2;
#endif

		pFdo->m_nNumTxChannels = NumTx;
		for (i=0; i < NumTx; i++) {
			pFdo->m_Channel[i].m_nChanType = DTA1XX_TS_TX_CHANNEL;

			pFdo->m_Channel[i].m_pRegBase =
				(char*)(pRegBase + 0x0180 + (i * 0x0200));

			pFdo->m_TsTx[i] = &pFdo->m_Channel[i];

			// Init DMA lock
			init_MUTEX( &pFdo->m_TsTx[i]->m_Dma.m_Lock );

			// Init DMA wait queue
			init_waitqueue_head( &pFdo->m_TsTx[i]->m_Dma.m_DmaDoneQueue );
		}
		//pFdo->m_EepromSize = 128*1024;
		//pFdo->m_EepromSectorSize = 32*1024;
		//pFdo->m_UseFpgaForVpd = 1;
		pFdo->m_VpdEepromPageSize = 16;
		break;

	case 107:
	case 110:	
		pFdo->m_nUses9056 = 1;
		pFdo->m_nNumTxChannels = 1;
		pFdo->m_Channel[0].m_nChanType = DTA1XX_TS_TX_CHANNEL;
		pFdo->m_TsTx[0] = &pFdo->m_Channel[0];

		// Init DMA lock
		init_MUTEX(&pFdo->m_TsTx[0]->m_Dma.m_Lock);
		// Init DMA wait queue
		init_waitqueue_head( &pFdo->m_TsTx[0]->m_Dma.m_DmaDoneQueue );
		// Set PLX DMA channel
		pFdo->m_TsTx[0]->m_Dma.m_PlxChannel = 0;

		pFdo->m_TsTx[0]->m_pRegBase = pRegBase + 0x80;
		break;

	case 120:
	case 122:	
		pFdo->m_nUses9054 = 1;
		pFdo->m_nUsesGenReg = (pFdo->m_nFirmwareVersion>=4) ? 1 : 0;
		pFdo->m_nNumRxChannels = 1;
		pFdo->m_Channel[0].m_nChanType = DTA1XX_TS_RX_CHANNEL;
		pFdo->m_TsRx[0] = &pFdo->m_Channel[0];

		// Firmware Revision >= 4: Rx registers located at base+0x20
		if ( pFdo->m_nFirmwareVersion >= 4 ) {
			pFdo->m_TsRx[0]->m_pRegBase = (char*)(pRegBase+0x20);
		}

		// Init DMA lock
		init_MUTEX(&pFdo->m_TsRx[0]->m_Dma.m_Lock);
		// Init DMA wait queue
		init_waitqueue_head( &pFdo->m_TsRx[0]->m_Dma.m_DmaDoneQueue );
		// Set PLX DMA channel
		pFdo->m_TsRx[0]->m_Dma.m_PlxChannel = 0;
		break;

	case 124:	
		pFdo->m_nUses9056 = 1;
		pFdo->m_nUsesDmaInFpga = 1;
		pFdo->m_nNumRxChannels = 4;

		for (i=0; i<4; i++) {
			pFdo->m_Channel[i].m_nChanType = DTA1XX_TS_RX_CHANNEL;
			pFdo->m_TsRx[i] = &pFdo->m_Channel[i];

			// Init DMA lock
			init_MUTEX(&pFdo->m_TsRx[i]->m_Dma.m_Lock);
			// Init DMA wait queue
			init_waitqueue_head( &pFdo->m_TsRx[i]->m_Dma.m_DmaDoneQueue );

			pFdo->m_TsRx[i]->m_pRegBase = (char*)(pRegBase + (0x80 * (i+1)));
		}
		// Set General Purpose Output of PCI9056 to '1'
		READ_LONG(CntrlReg, pPci905XRegs, PCI905X_EEPROM_CTRL_STAT);
		CntrlReg |= PCI905X_CNTRL_GENPURPOUTP;
		WRITE_LONG(CntrlReg, pPci905XRegs, PCI905X_EEPROM_CTRL_STAT);
		break;

	case 140:	
		pFdo->m_nUses9056 = 1;

	
		// Receive channel
		pFdo->m_nNumRxChannels = 1;
		pFdo->m_Channel[0].m_nChanType = DTA1XX_TS_RX_CHANNEL;
		pFdo->m_TsRx[0] = &pFdo->m_Channel[0];
		pFdo->m_TsRx[0]->m_pRegBase = pRegBase + 0x0100;

		init_MUTEX(&pFdo->m_TsRx[0]->m_Dma.m_Lock);
		// Init DMA wait queue
		init_waitqueue_head( &pFdo->m_TsRx[0]->m_Dma.m_DmaDoneQueue );
		// Set PLX DMA channel
		pFdo->m_TsRx[0]->m_Dma.m_PlxChannel = 1;
	
		// Transmit channel
		pFdo->m_nNumTxChannels = 1;
		pFdo->m_Channel[1].m_nChanType = DTA1XX_TS_TX_CHANNEL;
		pFdo->m_TsTx[0] = &pFdo->m_Channel[1];

		pFdo->m_TsTx[0]->m_pRegBase = pRegBase + 0x80;

		init_MUTEX(&pFdo->m_TsTx[0]->m_Dma.m_Lock);
		// Init DMA wait queue
		init_waitqueue_head( &pFdo->m_TsTx[0]->m_Dma.m_DmaDoneQueue );
		// Set PLX DMA channel
		pFdo->m_TsTx[0]->m_Dma.m_PlxChannel = 0;
	break;

	case 145:
	case 2145:
		// set the number of RxChannels DTA 145 and DTA 2145 (0..1)
		NumRx = NUM_DTAX145_RX_CHANNELS;

		pFdo->m_nUsesDmaInFpga = 1;		

		//First receive channels must be set then the transmit channels !!
		//Receive Channel
		pFdo->m_nNumRxChannels = NumRx;
		for (i=0; i<pFdo->m_nNumRxChannels; i++)
		{
			pFdo->m_Channel[i].m_nChanType = DTA1XX_TS_RX_CHANNEL;
			pFdo->m_TsRx[i] = &pFdo->m_Channel[i];

			// Init DMA lock
			init_MUTEX( &pFdo->m_TsRx[i]->m_Dma.m_Lock );
			// Init DMA wait queue
			init_waitqueue_head( &pFdo->m_TsRx[i]->m_Dma.m_DmaDoneQueue );
			pFdo->m_TsRx[i]->m_pRegBase = (char*)(pRegBase + 0x100 + (0x200*i));
		}

		//Transmit Channel
		NumTx = 2-NumRx;
		pFdo->m_nNumTxChannels = NumTx;
		for (i=NumRx; i<(NumRx+NumTx); i++) {
			int ti = i - NumRx;
			pFdo->m_Channel[i].m_nChanType = DTA1XX_TS_TX_CHANNEL;
			pFdo->m_TsTx[ti] = &pFdo->m_Channel[i];

			// Init DMA lock
			init_MUTEX(&pFdo->m_TsTx[ti]->m_Dma.m_Lock);
			// Init DMA wait queue
			init_waitqueue_head( &pFdo->m_TsTx[ti]->m_Dma.m_DmaDoneQueue );

			pFdo->m_TsTx[ti]->m_pRegBase = (char*)(pRegBase + 0x180 + (0x200*i));
		}
		//pFdo->m_EepromSize = 128*1024;
		//pFdo->m_EepromSectorSize = 32*1024;
		//pFdo->m_UseFpgaForVpd = 1;
		pFdo->m_VpdEepromPageSize = 16;
		break;

	case 160:
		// set the number of RxChannels for the DTA 160 (0..3)
		NumRx = NUM_DTA160_RX_CHANNELS;

		pFdo->m_nUses9056 = 1;
		pFdo->m_nUsesDmaInFpga = 1;
		//First receive channels must be set then the transmit channels !!
		//Receive Channels
		pFdo->m_nNumRxChannels = NumRx;
		for (i=0; i<NumRx; i++) {
			pFdo->m_Channel[i].m_nChanType = DTA1XX_TS_RX_CHANNEL;
			pFdo->m_TsRx[i] = &pFdo->m_Channel[i];

			// Init DMA lock
			init_MUTEX(&pFdo->m_TsRx[i]->m_Dma.m_Lock);
			// Init DMA wait queue
			init_waitqueue_head( &pFdo->m_TsRx[i]->m_Dma.m_DmaDoneQueue );

			pFdo->m_TsRx[i]->m_pRegBase = (char*)(pRegBase + 0x100 + (0x200*i));
		}

		//First receive channels must be set then the transmit channels !!
		//Transmit Channels
		NumTx = 3-NumRx;
		pFdo->m_nNumTxChannels = NumTx;
		for (i=NumRx; i<(NumRx+NumTx); i++) {
			int ti = i - NumRx;
			pFdo->m_Channel[i].m_nChanType = DTA1XX_TS_TX_CHANNEL;
			pFdo->m_TsTx[ti] = &pFdo->m_Channel[i];

			// Init DMA lock
			init_MUTEX(&pFdo->m_TsTx[ti]->m_Dma.m_Lock);
			// Init DMA wait queue
			init_waitqueue_head( &pFdo->m_TsTx[ti]->m_Dma.m_DmaDoneQueue );

			pFdo->m_TsTx[ti]->m_pRegBase = (char*)(pRegBase + 0x180 + (0x200*i));
		}

		// Set General Purpose Output of PCI9056 to '1'
		READ_LONG(CntrlReg, pPci905XRegs, PCI905X_EEPROM_CTRL_STAT);
		CntrlReg |= PCI905X_CNTRL_GENPURPOUTP;
		WRITE_LONG(CntrlReg, pPci905XRegs, PCI905X_EEPROM_CTRL_STAT);
		break;


	case 545:  
		pFdo->m_nUsesDmaInFpga = 1;

		//First receive channels must be set then the transmit channels !!
		//Receive Channel
		pFdo->m_nNumRxChannels = 1;
		pFdo->m_Channel[0].m_nChanType = DTA1XX_TS_RX_CHANNEL;
		pFdo->m_Channel[0].m_pRegBase =(char*)(pRegBase + 0x0100);

		pFdo->m_TsRx[0] = &pFdo->m_Channel[0];

		// Init DMA lock
		init_MUTEX( &pFdo->m_TsTx[0]->m_Dma.m_Lock );

		// Init DMA wait queue
		init_waitqueue_head( &pFdo->m_TsTx[0]->m_Dma.m_DmaDoneQueue );

		//Transmit Channel
		pFdo->m_nNumTxChannels = 1;
		pFdo->m_Channel[1].m_nChanType = DTA1XX_TS_TX_CHANNEL;

		pFdo->m_Channel[1].m_pRegBase =(char*)(pRegBase + 0x0380);

		pFdo->m_TsTx[0] = &pFdo->m_Channel[1];

		pFdo->m_EepromSize = 512*1024;
		pFdo->m_EepromSectorSize = 64*1024;
		pFdo->m_UseFpgaForVpd = 1;
		break;
	}
	// Store number of channels
	pFdo->m_nNumChannels = pFdo->m_nNumRxChannels + pFdo->m_nNumTxChannels;

#ifdef DEBUG
	// Some debug output to check the results of the big switch
	DTA1XX_LOG1(KERN_DEBUG, "Uses PLX chip=%d", pFdo->m_nUsesPlxChip);
	DTA1XX_LOG1(KERN_DEBUG, "Uses PCI9054=%d", pFdo->m_nUses9054);
	DTA1XX_LOG1(KERN_DEBUG, "Uses PCI9056=%d", pFdo->m_nUses9056);
	DTA1XX_LOG1(KERN_DEBUG, "Uses DMA Controller in FPGA=%d", pFdo->m_nUsesDmaInFpga);
	DTA1XX_LOG1(KERN_DEBUG, "General-Register block=%d", pFdo->m_nUsesGenReg);
	DTA1XX_LOG3(KERN_DEBUG, "Num Tx channels: %d, Num Rx Channels: %d => Total: %d",
		pFdo->m_nNumTxChannels, pFdo->m_nNumRxChannels, pFdo->m_nNumChannels);

	// Check for correct VPD/Eeprom settings (ASSERT in Windows)
	if(!( 	!(pFdo->m_UseFpgaForVpd && pFdo->m_EepromSize && pFdo->m_EepromSectorSize) ||
		!(!pFdo->m_UseFpgaForVpd && pFdo->m_VpdEepromPageSize) ||
		!pFdo->m_nUsesPlxChip  ))
	{
		DTA1XX_LOG( KERN_DEBUG, "ASSERT; incorrect VPD/Eeprom settings!" );
		return -EFAULT;
	}
#endif

	if (pFdo->m_UseFpgaForVpd) {
		// Compute address of last sector in EEPROM
		pFdo->m_EepromAddrLastSector = pFdo->m_EepromSize - pFdo->m_EepromSectorSize;

		// Relative address of VPD data in m_pEepromCache (EEPROM cache)
		pFdo->m_EepromRelAddrVpd = pFdo->m_EepromSectorSize - pFdo->m_VpdEepromSize;
	}

	//-.-.-.-.-.-.-.-.-.-.-.-.-.- Initialise the DMA buffer(s) -.-.-.-.-.-.-.-.-.-.-.-.-.-

	for (i=0; i<pFdo->m_nNumChannels; i++) {
		pCh = &pFdo->m_Channel[i];
		if ( Dta1xxInitDmaAdapter(pCh) != 0 ) {
			return -EFAULT;
		}
	}

	// Initialise CDMA resources
	for (i=0; i<pFdo->m_nNumChannels; i++ ) {
		pCh = &pFdo->m_Channel[i];
		init_waitqueue_head( &pCh->m_CdmaEvent );
		// Reset CDMA done flag
		clear_bit( 0, (void*)&pCh->m_CdmaDone );
	}


	// Initialise operational registers
	if ( 0!= Dta1xxInitDeviceHardware(pFdo) ) {
		return -EFAULT;
	}

	//-.-.-.-.-.-.-.-.-.-.-.-.-.- Final channel initialisation -.-.-.-.-.-.-.-.-.-.-.-.-.-

	for (i=0; i<pFdo->m_nNumChannels; i++) {

		// Copy type number and firmware version
		pFdo->m_Channel[i].m_nTypeNumber	= pFdo->m_nTypeNumber;
		pFdo->m_Channel[i].m_nFirmwareVersion	= pFdo->m_nFirmwareVersion;
	}

	//-.-.-.-.-.-.-.-.-.-.-.-.-.- Initialise VPD Caching -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	if ( (Status=Dta1xxInitVpd( pFdo )) != 1 ) {
		return Status;
	}

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- LED Flashing -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	// Common initialisation
	pFdo->m_nLedFlashFlashCounter = 0;
	pFdo->m_nLedFlashStateCounter = 0;
	pFdo->m_nLedFlashState = DTA1XX_LEDFLASH_INIT;

	// Start LED flashing for DTA-102 (Firmware Version>=5), 
	// DTA-107/110/120/122/124/140/145/160/2145
	// Parameters depend on clock frequency
	if (   (pFdo->m_nTypeNumber==102 && pFdo->m_nFirmwareVersion>=5)
		||  pFdo->m_nTypeNumber==107
		||  pFdo->m_nTypeNumber==110
		||  pFdo->m_nTypeNumber==122
		||  pFdo->m_nTypeNumber==124
		||  pFdo->m_nTypeNumber==145
		||  pFdo->m_nTypeNumber==2145
		||  pFdo->m_nTypeNumber==160) {			// 27-MHz boards

			pFdo->m_nLedFlashDurationInit	= 7;
			pFdo->m_nLedFlashDurationOn		= 1;
			pFdo->m_nLedFlashDurationOff	= 1;
			pFdo->m_nLedFlashDurationExit	= 7;
			pFdo->m_nLedFlashNumFlashes		= 7;
			pFdo->m_nLedFlashStartUp		= 1;
			Dta1xxFlashLed(pFdo, 0);
	}
	else if (pFdo->m_nTypeNumber==120 || pFdo->m_nTypeNumber==140) {

		pFdo->m_nLedFlashDurationInit	= 10;
		pFdo->m_nLedFlashDurationOn		= 1;
		pFdo->m_nLedFlashDurationOff	= 2;
		pFdo->m_nLedFlashDurationExit	= 10;
		pFdo->m_nLedFlashNumFlashes		= 7;
		pFdo->m_nLedFlashStartUp		= 1;
		Dta1xxFlashLed(pFdo, 0);
	} else {
		// Boards without generic LED: do not start LED flashing
		pFdo->m_nLedFlashStartUp		= 0;
	}
	return 0;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxInitDeviceHardware -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Initialise PCI card. Called from StartDevice.
//
int Dta1xxInitDeviceHardware(
							 DTA1XX_FDO* pFdo)	// Functional device object, representing the DTA-1xx card
{
	char*  pGenRegs	= (char*)pFdo->m_pGenRegs;
	char*  pRxRegs	= NULL;
	char*  pTxRegs	= NULL;
	CHANNEL* pCh = NULL;
	int i = 0;

	if ( pFdo->m_nNumTxChannels >= 1 )
		pTxRegs			= (char*)pFdo->m_TsTx[0]->m_pRegBase;
	if ( pFdo->m_nNumRxChannels >= 1 )
		pRxRegs			= (char*)pFdo->m_TsRx[0]->m_pRegBase;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xx Registers -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	// Initialse PCI-card-specific registers

	switch (pFdo->m_nTypeNumber) {

	case 100:

		// Init to default state
		Dta1xxTxSetTxCtrlReg(pTxRegs, 0x00000000);
		// Init bit-rate to zero
		Dta1xxTxSetTxClockReg(pTxRegs, 0x00000000);
		// Set maximum fifo size
		Dta1xxTxSetFifoSizeReg(pTxRegs, ((8*1024*1024)-16));
		// No loopback
		Dta1xxTxDiagRegSetLoopBack(pTxRegs, 0);
		break;

	case 102:

		// Init to default state
		Dta1xxTxSetTxCtrlReg(pTxRegs, 0x00000000);
		// Tri state the LVDS output
		Dta1xxTxCtrlRegSetTxDis(pTxRegs, 1);
		// Init bit-rate to zero
		Dta1xxTxSetTxClockReg(pTxRegs, 0x00000000);
		// Set maximum fifo size
		Dta1xxTxSetFifoSizeReg(pTxRegs, ((8*1024*1024)-16));
		// No loopback
		Dta1xxTxDiagRegSetLoopBack(pTxRegs, 0);
		break;


	case 107:
	case 110:
		// Init to default state
		Dta1xxGenSetGenCtrlReg(pGenRegs, 0x00000000);
		break;

	case 120:

		// Init to default state
		Dta1xxRxSetRxCtrlReg(pRxRegs, 0x00000000);
		// No loopback
		Dta1xxRxDiagRegSetLoopBack(pRxRegs, 0);
		break;

	case 122:

		// Init to default state
		Dta1xxRxSetRxCtrlReg(pRxRegs, 0x00000000);
		// No loopback
		Dta1xxRxDiagRegSetLoopBack(pRxRegs, 0);
		break;

	case 124:
		// Init to default state
		Dta1xxGenSetGenCtrlReg(pGenRegs, 0x00000000);

		// Initialise DMA controller

		// Channel 0: digital video data to PCI bus
		WRITE_LONG( PCI905X_DMAMODE_BUSWIDTH_32 | PCI905X_DMAMODE_READY_INPEN	|
			PCI905X_DMAMODE_BTERM_INPEN | PCI905X_DMAMODE_LOCALBURST	|
			PCI905X_DMAMODE_DONE_INTEN	| PCI905X_DMAMODE_LOC_CONST,
			pFdo->m_Pci905XConfRegs.m_pKernel, PCI905X_DMA0_MODE);

		// Channel 1: scatter-gather data from PCI bus
		WRITE_LONG( PCI905X_DMAMODE_BUSWIDTH_32 | PCI905X_DMAMODE_READY_INPEN	|
			PCI905X_DMAMODE_BTERM_INPEN | PCI905X_DMAMODE_LOCALBURST	|
			PCI905X_DMAMODE_DONE_INTEN,
			pFdo->m_Pci905XConfRegs.m_pKernel, PCI905X_DMA1_MODE);

		// DMA threshold register
		WRITE_LONG(0x00000010, pFdo->m_Pci905XConfRegs.m_pKernel, PCI905X_DMA_THRESHOLD);
		break;

	case 140:

		// Init to default state
		Dta1xxGenSetGenCtrlReg(pGenRegs, 0x00000000);

		// Init to default state
		Dta1xxTxSetTxCtrlReg(pTxRegs, 0x00000000);
		// Init bit-rate to zero
		Dta1xxTxSetTxClockReg(pTxRegs, 0x00000000);
		// Set maximum fifo size
		Dta1xxTxSetFifoSizeReg(pTxRegs, ((8*1024*1024)-16));
		// No loopback
		Dta1xxTxDiagRegSetLoopBack(pTxRegs, 0);

		// Init to default state
		Dta1xxRxSetRxCtrlReg(pRxRegs, 0x00000000);
		// No loopback
		Dta1xxRxDiagRegSetLoopBack(pRxRegs, 0);

		break;


	case 105:
	case 545:
		// Init to default state
		Dta1xxGenSetGenCtrlReg(pGenRegs, 0x00000000);

		//set periodic interrupt to 38.83 ms
		Dta1xxGenCtrlRegSetPerIntVal(pGenRegs, 4);

		// Initialise TS Tx/Rx
		for (i=0; i<pFdo->m_nNumChannels; i++) {
			pCh = &pFdo->m_Channel[i];

			Dta1xxInitAsiSdi(pCh);
			Dta1xxDmaTimeOutSet(pCh->m_Dma.m_pRegBase, 0);
		}

		if ((pFdo->m_nTypeNumber == 105) && (pFdo->m_nNumTxChannels==1)) {
			// If output 2 of DTA105 is not used, enable doubly buffered mode
			Dta1xxGenDblBufEnableSet((Int*)(pGenRegs), 1);
		}

		break;

	case 145:
	case 2145:
		// Init to default state
		Dta1xxGenSetGenCtrlReg(pGenRegs, 0x00000000);

		//set periodic interrupt to 38.83 ms
		Dta1xxGenCtrlRegSetPerIntVal(pGenRegs, 4);

		// Initialise TS Tx/Rx
		for (i=0; i<pFdo->m_nNumChannels; i++) {
			pCh = &pFdo->m_Channel[i];

			Dta1xxInitAsiSdi(pCh);
			// Set configurable IO
			if (pCh->m_nChanType == DTA1XX_TS_RX_CHANNEL) 
			{
				Dta1xxTxCtrlRegSetOutputEn((unsigned char*)((unsigned char*)pCh->m_pRegBase+0x80), 0);
			} else {
				Dta1xxTxCtrlRegSetOutputEn((unsigned char*)((unsigned char*)pCh->m_pRegBase), 1);
			}
			Dta1xxDmaTimeOutSet(pCh->m_Dma.m_pRegBase, 0);
		}
		break;

	case 160:
		// Init to default state
		Dta1xxGenSetGenCtrlReg(pGenRegs, 0x00000000);

		//set periodic interrupt to 3 = 19.42 ms
		Dta1xxGenCtrlRegSetPerIntVal(pGenRegs, 3);
		
		// Initialise TS Tx/Rx
		for (i=0; i<(pFdo->m_nNumRxChannels + pFdo->m_nNumTxChannels); i++) {
			pCh = &pFdo->m_Channel[i];

			Dta1xxInitAsiSdi(pCh);
			// Set configurable IO
			if (pCh->m_nChanType == DTA1XX_TS_RX_CHANNEL) 
			{
				Dta1xxTxCtrlRegSetOutputEn((unsigned char*)((unsigned char*)pCh->m_pRegBase+0x80), 0);
			} else {
				Dta1xxTxCtrlRegSetOutputEn((unsigned char*)((unsigned char*)pCh->m_pRegBase), 1);
			}
			
			Dta1xxDmaTimeOutSet(pCh->m_Dma.m_pRegBase, 0);
		}

		// Initialise DMA controller

		// Channel 0: digital video data to PCI bus
		WRITE_LONG( PCI905X_DMAMODE_BUSWIDTH_32 | PCI905X_DMAMODE_READY_INPEN	|
			PCI905X_DMAMODE_BTERM_INPEN 	| PCI905X_DMAMODE_LOCALBURST	|
			PCI905X_DMAMODE_DEMAND		| PCI905X_DMAMODE_LOC_CONST,
			pFdo->m_Pci905XConfRegs.m_pKernel, PCI905X_DMA0_MODE);

		// Channel 1: scatter-gather data from PCI bus
		WRITE_LONG(PCI905X_DMAMODE_BUSWIDTH_32 | PCI905X_DMAMODE_READY_INPEN	|
			PCI905X_DMAMODE_BTERM_INPEN 	| PCI905X_DMAMODE_LOCALBURST	|
			PCI905X_DMAMODE_DEMAND		| PCI905X_DMAMODE_LOC_CONST,
			pFdo->m_Pci905XConfRegs.m_pKernel, PCI905X_DMA1_MODE);

		// DMA threshold register
		WRITE_LONG(0x00110110, pFdo->m_Pci905XConfRegs.m_pKernel, PCI905X_DMA_THRESHOLD);
		break;
	}

	return 0;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxInitPlx -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// In principle, initialising the PCI-9054 or PCI-9056 is required only if the serial
// EEPROM has not been programmed yet.
// However, due to a bug in the PCI-9054 chip (see Errata #9 in 9054ABerrata-16.pdf),
// the serial EEPROM is not reloaded when the Power Management State is changed from
// D3hot to D0. This is a problem e.g. when disabling and enabling a device driver.
//
// So: we simply always initialise the PCI-9054, and also the PCI-9056.
//
// REGISTER VALUES MUST MATCH VALUES PROGRAMMED INTO THE SERIAL EEPROM!
//
int Dta1xxInitPlx(
				  DTA1XX_FDO* pFdo)	// Functional device object, representing the DTA-1xx card
{
	volatile char *p = pFdo->m_Pci905XConfRegs.m_pKernel;

	// LAS0RR is a problem, because value should be different for DTA-107/124/140
	// compared to DTA-100, DTA-102, DTA-120, DTA-122.
	//
	// Uninitialised DTA-1xx board => don't initialise LAS0RR register at all

	if ( pFdo->m_pPdo->vendor!=0x10B5 ) {
		if (     pFdo->m_nTypeNumber==107 || pFdo->m_nTypeNumber==140 ) {
			WRITE_LONG( 0xFFFFFE00, p, PCI905X_SPACE0_RANGE );	// LAS0RR
		} else if ( pFdo->m_nTypeNumber==124 ) { 
			WRITE_LONG( 0xFFFFFC00, p, PCI905X_SPACE0_RANGE );	// LAS0RR
		} else if ( pFdo->m_nTypeNumber==160 ) { 
			WRITE_LONG( 0xFFFFF000, p, PCI905X_SPACE0_RANGE );	// LAS0RR
		} else {
			WRITE_LONG( 0xFFFFFF80, p, PCI905X_SPACE0_RANGE );	// LAS0RR
		}
	}

	// Other registers
	WRITE_LONG( 1, p ,PCI905X_SPACE0_REMAP );			// LAS0BA
	if (   pFdo->m_nTypeNumber==112 ||pFdo->m_nTypeNumber==115
		|| pFdo->m_nTypeNumber==160 ) {
		// Dma priority rotating
		WRITE_LONG( 0x02000000, p, PCI905X_LOCAL_DMA_ARBIT );	// MARBR
	} else {
		// Dma priority channel 0 (DTA-124)
		WRITE_LONG( 0x02080000, p, PCI905X_LOCAL_DMA_ARBIT );	// MARBR
	}
	WRITE_LONG( 0x02080000, p, PCI905X_LOCAL_DMA_ARBIT );	// MARBR
	WRITE_BYTE( 0, p, PCI905X_ENDIAN_DESC );			// BIGEND
	WRITE_BYTE( 4, p, PCI905X_LOCAL_MISC );				// LMISC
	WRITE_WORD( 0x007F, p, PCI905X_PROT_AREA );			// PROT_AREA
	WRITE_LONG( 0x0000, p, PCI905X_EXP_ROM_RANGE );		// EROMRR
	WRITE_LONG( 0x0000, p, PCI905X_EXP_ROM_REMAP );		// EROMBA
	WRITE_LONG( 0x43430143, p, PCI905X_SPACE0_ROM_DESC );// LBRD0

	WRITE_LONG( 0, p, PCI905X_DM_RANGE );				// DMRR
	WRITE_LONG( 0, p, PCI905X_DM_MEM_BASE );			// DMLBAM
	WRITE_LONG( 0, p, PCI905X_DM_IO_BASE );				// DMLBAI
	WRITE_LONG( 0, p, PCI905X_DM_PCI_MEM_REMAP );		// DMPBAM

	WRITE_LONG( 0, p, PCI905X_DM_PCI_IO_CONFIG );		// DMCFGA

	WRITE_LONG( 0, p, PCI905X_SPACE1_RANGE );			// LAS1RR
	WRITE_LONG( 0, p, PCI905X_SPACE1_REMAP );			// LAS1BA
	WRITE_LONG( 0x03430000, p, PCI905X_SPACE1_DESC );	// LBRD1

	return 0;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxCheckFirmwareVersion -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Check whether the firmware version, as reported by the Altera itself, is compatible
// with this device driver.
//
int Dta1xxCheckFirmwareVersion(
							   DTA1XX_FDO* pFdo)	// Functional device object, representing the DTA-1xx card
{
	int nAlteraCompatible = 0;

	// Mirror Altera revision in device extension
	pFdo->m_nFirmwareVersion = Dta1xxGenCtrlRegGetFirmwareRev( (char*)pFdo->m_pGenRegs );

	// Supported Altera revisions:
	//	DTA-100	: >= V1
	//	DTA-102	: >= V1
	//	DTA-105 : >= V0
	//	DTA-107, DTA-110	: >= V0
	//	DTA-120 : >= V0
	//	DTA-122 : >= V0
	//	DTA-124	: >= V0
	//	DTA-140 : >= V1
	//	DTA-145 : >= V0
	//	DTA-2140: >= V0
	//	DTA-160 : >= V0
	//	DTA-545 : >= V0

	switch (pFdo->m_nTypeNumber) {

	case 100:
		nAlteraCompatible = (pFdo->m_nFirmwareVersion >= 1);
		break;

	case 102:
		nAlteraCompatible = (pFdo->m_nFirmwareVersion >= 1);
		break;

	case 105:
		nAlteraCompatible = (pFdo->m_nFirmwareVersion >= 0);
		break;

	case 107:
	case 110:
		nAlteraCompatible = (pFdo->m_nFirmwareVersion >= 0);
		break;

	case 120:
		nAlteraCompatible = (pFdo->m_nFirmwareVersion >= 0);
		break;

	case 122:
		nAlteraCompatible = (pFdo->m_nFirmwareVersion >= 0);
		break;

	case 124:
		nAlteraCompatible = (pFdo->m_nFirmwareVersion >= 0);
		break;

	case 140:
		nAlteraCompatible = (pFdo->m_nFirmwareVersion >= 1);
		break;

	case 145:
		nAlteraCompatible = (pFdo->m_nFirmwareVersion >= 0);
		break;

	case 2145:
		nAlteraCompatible = (pFdo->m_nFirmwareVersion >= 0);
		break;

	case 160:
		nAlteraCompatible = (pFdo->m_nFirmwareVersion >= 0);
		break;

	case 545:
		nAlteraCompatible = (pFdo->m_nFirmwareVersion >= 0);
		break;

	default:
		nAlteraCompatible = 0;
		break;
	}

	// Altera not compatible => log message in event log and return
	if ( nAlteraCompatible==0 ) {
		DTA1XX_LOG2(KERN_INFO,
			"Firmware Version=%d not compatible with DTA-%d. Don't start.",
			pFdo->m_nFirmwareVersion, pFdo->m_nTypeNumber);
		return -EFAULT;
	}
	return 0;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxInitDmaAdapter -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Initialise DMA adapter
//
int Dta1xxInitDmaAdapter(
						 CHANNEL* pCh)		// Channel pointer
{
	unsigned char* pBase;

	// Direction
	if ( pCh->m_nChanType==DTA1XX_TS_RX_CHANNEL ) {
		pCh->m_Dma.m_nDmaDir = DTA1XX_DMA_READ;
	} else {
		pCh->m_Dma.m_nDmaDir = DTA1XX_DMA_WRITE;
	}

	// Initialise register pointers
	
	if (    pCh->m_pFdo->m_nUsesDmaInFpga 
		&& (pCh->m_pFdo->m_nTypeNumber != 124 || (pCh->m_pFdo->m_nFirmwareVersion > 0)))
	{
		if ((pCh->m_nChanType == DTA1XX_TS_RX_CHANNEL) ||
			(pCh->m_nChanType == DTA1XX_TS_TX_CHANNEL)) 
		{
			pBase  = (unsigned char*)pCh->m_pFdo->m_Dta1xxRegsBase.m_pKernel;
			pBase += (pCh->m_nChanIdx * 0x200) + 0x200;
			pCh->m_Dma.m_pRegBase = pBase;
			pCh->m_Dma.m_pRegDmaDesc = pBase + 0x4;
			pCh->m_Dma.m_pRegCmdStat = pBase + 0x8;
		} else {	//For now only for DTA-160
			pBase = (unsigned char*) pCh->m_pRegBase;
			pCh->m_Dma.m_pRegBase = pBase + 0x80;
			pCh->m_Dma.m_pRegDmaDesc = pBase + 0x84;
			pCh->m_Dma.m_pRegCmdStat = pBase + 0x88;
		}
	} else {
		pBase = (unsigned char*) pCh->m_pRegBase;
		pCh->m_Dma.m_pRegBase = NULL;		// m_pRegBase is not used for now
		pCh->m_Dma.m_pRegDmaDesc = pBase + DTA1XX_RX_REG_DMA_DESC_PTR;
		pCh->m_Dma.m_pRegCmdStat = pBase + DTA1XX_RX_REG_DMA_COMMAND_STAT;
	}

	// Allocate buffer
	pCh->m_Dma.m_DmaBuffer.m_pVirtStartAddr =
		pci_alloc_consistent(pCh->m_pFdo->m_pPdo, DTA1XX_DMA_BUF_SIZE,
		&pCh->m_Dma.m_DmaBuffer.m_PhysStartAddr);

	// Allocation succeeded??
	if ( pCh->m_Dma.m_DmaBuffer.m_pVirtStartAddr==NULL ) {
		DTA1XX_LOG(KERN_CRIT, "Failed to allocate memory for DMA buffer");
		return -EFAULT;
	}

	// Allocate buffer for Scatter/Gather descriptor 
	// (only for Devices with DMA in FPGA)
	pCh->m_Dma.m_pSglDesc = NULL;
	if ( pCh->m_pFdo->m_nUsesDmaInFpga == 0) 
	{
		return 0;
	}

	pCh->m_Dma.m_pSglDesc = pci_alloc_consistent(pCh->m_pFdo->m_pPdo, sizeof(SglDesc),
		&pCh->m_Dma.m_PhysAddrSglDesc);

	// Allocation succeeded??
	if ( pCh->m_Dma.m_pSglDesc == NULL ) {
		DTA1XX_LOG(KERN_CRIT, "Failed to allocate memory for Scatter/Gather buffer");
		return -EFAULT;
	}

	return 0;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxInitVpd -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Initialise VPD caching, read and interpret VPD configuration resources
//
int Dta1xxInitVpd(
				  DTA1XX_FDO*  pFdo)
{
	size_t AllocSize;
	int Status;

	// Initialisation VPD read mutex
	init_MUTEX(&pFdo->m_VpdEepromMutex);

	// Allocating EEPROM cache buffer
	if (pFdo->m_nUsesPlxChip || !pFdo->m_UseFpgaForVpd) {
		AllocSize = pFdo->m_VpdEepromSize;
	} else {
		AllocSize = pFdo->m_EepromSectorSize;
	}

#ifdef DEBUG
	DTA1XX_LOG1( KERN_DEBUG, "Dta1xxInitVpd: Allocate EEPROM cache buffer (%d bytes)",
		AllocSize);
#endif

	pFdo->m_pEepromCache = kmalloc( AllocSize, GFP_KERNEL );
	if (pFdo->m_pEepromCache == NULL) {
		DTA1XX_LOG( KERN_DEBUG, "Dta1xxInitVpd: Allocating EEPROM cache buffer FAILS");
		return -ENOMEM;
	}

#ifdef DEBUG
	DTA1XX_LOG( KERN_DEBUG, "Dta1xx: Dta1xxInitVpd: EEPROM cache buffer allocated succesfully" );
#endif

	// Read EEPROM content
	if (pFdo->m_nUsesPlxChip) {
		// Read entire content of VPD EEPROM (PLX) into cache buffer
		Status = Dta1xxVpdRead(pFdo, (char *)pFdo->m_pEepromCache, 0, PLX_VPD_SIZE);
		if (Status != 0) {
			return Status;
		}
		pFdo->m_pVpdEeprom = pFdo->m_pEepromCache;
	} else if (pFdo->m_UseFpgaForVpd) {
		// Read last page of FPGA EEPROM into cache buffer
		Status = Dta1xxEepromReadLastPage(pFdo);
		if (Status != 0) {
			return Status;
		}
		pFdo->m_pVpdEeprom = pFdo->m_pEepromCache + pFdo->m_EepromRelAddrVpd;
	} else {
		// Read separate EEPROM connected to FPGA for VPD data
		Status = Dta1xxEepromSpiRead(pFdo);
		if (Status != 0) {
			return Status;
		}
		pFdo->m_pVpdEeprom = pFdo->m_pEepromCache;
	}

	//.-.-.-.-.-.-.-.-.-.-.-.-.- Initialisation from VPD Items -.-.-.-.-.-.-.-.-.-.-.-.-.-

	// Initialise MAC address on cards with an Ethernet network connection
	/*	for (i=0; i<pFdo->m_NumIpPorts; i++) {
	Dta1xxNwMacInitMacAddress(&(pFdo->m_NwStore[i]));
	}	*/

	/*	// DTA-110: check whether card is licensed for DTA-110T
	Dta1xxInitDta110T(pFdo);	*/

	/*	// Initialise TxDAC offsets on modulator cards
	Dta1xxInitModulatorOffset(pFdo);	*/

	/*	// Initialise frequency offset on DTA-106
	Dta1xxInitDta106FreqOffset(pFdo);	*/

	return 0;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxInitAsiSdi -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Initialise Asi/Sdi Tx/Rx ports
//
// Pre: pCh->m_BiDirTs must be set for Bi-Directional Ts ports
// For m_BiDirTs==TRUE: pBase_Tx(i) = pBase_Rx(i)+0x80;
//
void Dta1xxInitAsiSdi(CHANNEL*  pCh)
{
	if (pCh->m_nChanType == DTA1XX_TS_TX_CHANNEL) {
		// ASI/SDI Tx
		Dta1xxTxReset(pCh, DTA1XX_RST_CHANNEL);
		Dta1xxTxCtrlRegSetTxMode((char *)pCh->m_pRegBase, 0);		// Transmit Mode := 188
		Dta1xxTxCtrlRegSetPckStuff((char *)pCh->m_pRegBase, 0);		// No Null-Packet Stuffing
		Dta1xxTxCtrlRegSetTxCtrl((char *)pCh->m_pRegBase, 0);		// Transmit Control := Idle
		Dta1xxTxCtrlRegSetUflIntEn((char *)pCh->m_pRegBase, 0);		// Clear interrupt enables
		Dta1xxTxCtrlRegSetSyncIntEn((char *)pCh->m_pRegBase, 0);	//   ,,     ,,       ,,
		Dta1xxTxCtrlRegSetSdiMode((char *)pCh->m_pRegBase, 0);		// Disable SDI mode
		Dta1xxTxSetFifoSizeReg((char *)pCh->m_pRegBase, 8388608);	// Maximum FIFO size
		Dta1xxTxSetTxClockReg((char *)pCh->m_pRegBase, 0);			// Clock to 0 bps
		Dta1xxTxDiagRegSetLoopBack((char *)pCh->m_pRegBase, 0);		// Disable loopback
		Dta1xxTxCtrlSetTxAsiInv((char *)pCh->m_pRegBase, 0);		// No Inversion
		/*		if (pCh->m_Capability & DTA1XX_CHCAP_BIDIR_PORT) {
		// Enable output
		Dta1xxTxCtrlRegSetOutputEn((unsigned char*)((unsigned char*)pCh->m_pRegBase), 1);
		}	*/
	} else {
		// ASI/SDI Rx
		Dta1xxRxReset(pCh, DTA1XX_RST_CHANNEL);
		Dta1xxRxCtrlRegSetRxMode((char *)pCh->m_pRegBase, 0);		// Receive Mode := St188
		Dta1xxRxCtrlRegSetRxCtrl((char *)pCh->m_pRegBase, 0);		// Receive Control := Idle
		Dta1xxRxCtrlRegSetOvfIntEn((char *)pCh->m_pRegBase, 0);		// Disable overflow int en.
		Dta1xxRxCtrlRegSetSyncIntEn((char *)pCh->m_pRegBase, 0);	// Disable sync int. enable
		Dta1xxRxCtrlRegSetAsiInv((char *)pCh->m_pRegBase, 0);		// No ASI Invert
		Dta1xxRxCtrlRegSetTimeStamping((char *)pCh->m_pRegBase, 0);	// No timestamp inserting
		Dta1xxRxLedControl((char *)pCh->m_pRegBase, 0);				// Disable LED overrule
		Dta1xxRxCtrlRegSetSdiMode((char *)pCh->m_pRegBase, 0);		// Disable SDI mode

		/*		if (pCh->m_Capability & DTA1XX_CHCAP_BIDIR_PORT) {
		// Disable output enable for the Tx channel (Base+0x80)
		Dta1xxTxCtrlRegSetOutputEn((unsigned char*)((unsigned char*)pCh->m_pRegBase+0x80), 0);
		}	*/
	}
	return;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxGetIoConfig -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
int  Dta1xxGetIoConfig(DTA1XX_FDO* pFdo, int nChIdx, int* pIoConfig)
{
	CHANNEL* pCh;
	if (nChIdx<0 || nChIdx>=pFdo->m_nNumChannels)
		return -1;

	pCh = &pFdo->m_Channel[nChIdx];

	// Simple IO configuration retrieval
	if (pCh->m_nChanType == DTA1XX_TS_TX_CHANNEL)
	{
		*pIoConfig = DTA1XX_IOCONFIG_OUTPUT;
	} 
	else if (pCh->m_nChanType == DTA1XX_TS_RX_CHANNEL) {
		*pIoConfig = DTA1XX_IOCONFIG_INPUT;
	}
	return 0;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxSetIoConfig -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
int  Dta1xxSetIoConfig(DTA1XX_FDO* pFdo, int nChIdx, int nIoConfig)
{
	// SetIoConfig not implemented
	return -1;
}
