//*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* Interrupt.c *#*#*#*#*#*#*#*#* (C) 2000-2005 DEKTEC

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- License -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// This program is free software; You can freely use/modify this source code in any way
// you desire, including for commercial applications.
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	TD	2007.01.29	Addes support for DTA-160 ASI-only
//  MG  26-11-2004      Only return IRQ_NONE and IRQ_HANDLED for kernel version 2.6
//	MG	30-08-2004	Dta1xxIsr now returns IRQ_NONE or IRQ_HANDLED
//	JH	16-08-2004	Added some castings to avoid compiler warnings
//	MG	30-06-2003	Added support for FW4 of DTA-120/122
//	MG	21-10-2002	Created

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#include <linux/version.h>

#include "../Include/Dta1xx.h"
#include "../Include/Dta1xxRegs.h"
#include <asm/bitops.h> // set_bit

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Type definitions -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0) && LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
	#define KERNELVERSION 24
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0) && LINUX_VERSION_CODE < KERNEL_VERSION(2,7,0)
	#define KERNELVERSION 26
#endif


//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Internal functionnIntCsr declarations -.-.-.-.-.-.-.-.-.-.-.-.-.-.

//-.-.-.-.-.-.-.-.-.-.-.-.-.- Data declaration and definition -.-.-.-.-.-.-.-.-.-.-.-.-.-.

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Implementation +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxIsr -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Interrupt service routine. Determines which interrupt(s), if any, is asserted and calls
// the appropiate interrupt handler
//
unsigned long Dta1xxIsr(
	int nIrq,				// IRQ number (NOT USED)
	void* pContext,			// pointer to the FDO object
	struct pt_regs *pRegs)	// snapshot of processors contexts (NOT USED)
{
	int i;
	DTA1XX_FDO* pFdo			= (DTA1XX_FDO*)pContext;
	volatile char* pPlxRegs		= NULL;
	CHANNEL* pCh				= NULL; // Channel pointer
	int nIntCsr					= 0;
	int nDmaStatReg				= 0;
	int nUseGenRegs				= 0;
	int nOurInterrupt			= 0;
	UInt32 DescPtr=0;

	//.-.-.-.-.-.-.-.-.-.-.- Check for master PCI interrupt enable -.-.-.-.-.-.-.-.-.-.-.-

	if (pFdo->m_nUsesPlxChip)
	{
		// Get pointer to PCI-9054/9056
		pPlxRegs = pFdo->m_Pci905XConfRegs.m_pKernel;

		// Read Interrupt Control / Status register in PCI 9054/9056
		READ_LONG(nIntCsr, pPlxRegs, PCI905X_INT_CTRL_STAT);
		if ( 0==(nIntCsr & PCI905X_INTCSR_PCI_INTEN) )
		{
	#if KERNELVERSION < 26
			return 0;
	#else
			return IRQ_NONE; // Can't be ours
	#endif
		}
	}

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Check periodic interrupt -.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	// Use general register
	nUseGenRegs = pFdo->m_nUsesGenReg;

	if (    ( nUseGenRegs && Dta1xxGenStatusRegGetPerInt( (char*)pFdo->m_pGenRegs ) )
		 || ( !nUseGenRegs
			  && Dta1xxRxStatusRegGetPerInt( (char*)pFdo->m_Dta1xxRegsBase.m_pKernel ) ) )
	{
		if ( nUseGenRegs ) {
			Dta1xxGenStatusRegClrPerInt( (char*)pFdo->m_pGenRegs );
		} else {
			// Address map of older boards start where general registers are located
			// on newer board, therefor we use the genreags pointer here as well
			Dta1xxRxStatusRegClrPerInt( (char*)pFdo->m_pGenRegs );
		}

		// Process failsafe output
		// For now only DTA-145/2145
		if ((pFdo->m_nTypeNumber == 145) || (pFdo->m_nTypeNumber == 2145)) {
			// No failsafe supported, toggle always
			Dta1xxGenPulseWatchdog((Int*)pFdo->m_pGenRegs);
		}

		// Read valid-counter sample values
		if (pFdo->m_nTypeNumber==102) {
			pFdo->m_TsTx[0]->m_BrMeas.m_nValidCountSample =
				Dta1xxTxGetClkCountReg( (char*)pFdo->m_TsTx[0]->m_pRegBase );
		} else  {
			// Read valid counter for all Rx channels
			for (i=0; i<pFdo->m_nNumRxChannels; i++) {
				pFdo->m_TsRx[i]->m_BrMeas.m_nValidCountSample =
					Dta1xxRxGetValidCountReg( (char*)pFdo->m_TsRx[i]->m_pRegBase );
			}
		}

		// Call periodic interrupt handler routine
		Dta1xxPeriodicIntHandler(pFdo);

		nOurInterrupt = 1;
	}

	// For new cards that use DMA controller in Altera (DTA-105/124/145/2145/160/545):
	// Check for DMA channel interrupts in operational registers, not in PLX
	if (pFdo->m_nUsesDmaInFpga)
	{
		// Check all channels
		for (i=0; i<pFdo->m_nNumChannels; i++)
		{
            pCh = &pFdo->m_Channel[i];

			// DMA interrupt from this channel?
			READ_BYTE( nDmaStatReg, pCh->m_Dma.m_pRegCmdStat, 0 );
			if ( 0 != (nDmaStatReg & DTA1XX_DMACSR_INTACT) )
			{
                         	//-.-.-.-.-.-.-.-.-.- DMA INTERRUPT FROM THIS CHANNEL -.-.-.-.-.-.-.-.-.-.

				// Clear DMA Channel interrupt (disables the channel, too)
				WRITE_BYTE( PCI905X_DMACSR_CLEARINT, pCh->m_Dma.m_pRegCmdStat, 0 );

				// Set the DMA done flag
				set_bit(0, (void*)&pFdo->m_Channel[i].m_Dma.m_nDmaDone);
				// Awaken any waiting threads
	   			wake_up_interruptible( &pFdo->m_Channel[i].m_Dma.m_DmaDoneQueue );

				nOurInterrupt = 1;
			}
		}
	} else {
		// DMA controller in PLX: check for DMA channel interrupts in PCI905X

		//.-.-.-.-.-.-.-.-.-.-.- Check for DMA channel 0 interrupt -.-.-.-.-.-.-.-.-.-.-.-

		if ( 0!=(nIntCsr & PCI905X_INTCSR_DMA0_INTACT) ) {
			// Find channel related to PlxChannel 0
			int idx =0;
			for (idx=0; idx<pFdo->m_nNumChannels; idx++) {
				if (pFdo->m_Channel[idx].m_Dma.m_PlxChannel == 0)
					break;
			}
			if (idx < pFdo->m_nNumChannels)
			{
				// Check for CDMA
				if ( pFdo->m_Channel[idx].m_CdmaDmaRunning != FALSE )
				{
					// Get sample of the reference clock register
					UInt  RefClkSamp;
					RefClkSamp = Dta1xxGenGetRefClkCntReg( (char*)pFdo->m_pGenRegs );
					// Read DMA Channel 0 Descriptor Pointer to figure out where we are
					READ_LONG(DescPtr, pPlxRegs, PCI905X_DMA0_DESC_PTR);

					// Clear DMA Channel 0 interrupt, but leave DMA enabled
					WRITE_BYTE(PCI905X_DMACSR_ENABLE | PCI905X_DMACSR_CLEARINT,
						   pPlxRegs, PCI905X_DMA0_COMMAND_STAT);

					// Store reference pointer
					pFdo->m_Channel[idx].m_CdmaRefClkSamp = RefClkSamp;
				
					// Derive which segment is being processed now
					// If the Scatter-Gather-List pointer at the instance of the interrupt points to an
					// address in segment 0, then the current segment is also segment #0
					if		(DescPtr < pFdo->m_Channel[idx].m_CdmaSegmDesc[1].m_SglAddr)
						pFdo->m_Channel[idx].m_CdmaSegment = 0;
					else if (DescPtr < pFdo->m_Channel[idx].m_CdmaSegmDesc[2].m_SglAddr)
						pFdo->m_Channel[idx].m_CdmaSegment = 1;
					else if (DescPtr < pFdo->m_Channel[idx].m_CdmaSegmDesc[3].m_SglAddr)
						pFdo->m_Channel[idx].m_CdmaSegment = 2;
					else
						pFdo->m_Channel[idx].m_CdmaSegment = 3;
								
					// Set the CDMA done flag
					set_bit(0, (void*)&pFdo->m_Channel[idx].m_CdmaDone);
					// Awaken any waiting threads
					wake_up_interruptible( &pFdo->m_Channel[idx].m_CdmaEvent );
				}
				else
				{
					// Clear DMA Channel 0 interrupt (disables the channel, too)
					WRITE_BYTE(PCI905X_DMACSR_CLEARINT, pPlxRegs, PCI905X_DMA0_COMMAND_STAT);
	
  					// Set the DMA done flag
					set_bit(0, (void*)&pFdo->m_Channel[idx].m_Dma.m_nDmaDone);
					// Awaken any waiting threads
		   			wake_up_interruptible( &pFdo->m_Channel[idx].m_Dma.m_DmaDoneQueue );
				}
				nOurInterrupt = 1;
			}
		}

		//.-.-.-.-.-.-.-.-.-.-.- Check for DMA channel 1 interrupt -.-.-.-.-.-.-.-.-.-.-.-

		if ( 0!=(nIntCsr & PCI905X_INTCSR_DMA1_INTACT) )
		{
			// Find channel related to PlxChannel 1
			int idx =0;
			for (idx=0; idx<pFdo->m_nNumChannels; idx++) {
				if (pFdo->m_Channel[idx].m_Dma.m_PlxChannel == 1)
					break;
			}
			if (idx < pFdo->m_nNumChannels) {
			// Clear DMA Channel 1 interrupt (disables the channel, too)
				WRITE_BYTE(PCI905X_DMACSR_CLEARINT, pPlxRegs, PCI905X_DMA1_COMMAND_STAT);

				// Set the DMA done flag
				set_bit(0, (void*)&pFdo->m_Channel[idx].m_Dma.m_nDmaDone);
				// Awaken any waiting threads
	   			wake_up_interruptible( &pFdo->m_Channel[idx].m_Dma.m_DmaDoneQueue );

				nOurInterrupt = 1;
			}
		}
	}

	//-.-.-.-.-.-.-.-.-.-.-.- Clear all unhandled interrupt flags -.-.-.-.-.-.-.-.-.-.-.-.
	//
	// Make sure that all interrupt flags are removed even the ones that are not used by
	// the driver
	//
	for ( i=0; i<pFdo->m_nNumRxChannels; i++ ) {
		Dta1xxRxSetRxStatusReg(  (char*)pFdo->m_TsRx[i]->m_pRegBase,
							      DTA1XX_RXSTAT_PERINT
							    | DTA1XX_RXSTAT_OVFINT
							    | DTA1XX_RXSTAT_SYNCINT
							    | DTA1XX_RXSTAT_THRINT
							    | DTA1XX_RXSTAT_RATEOVFINT );
	}
	for ( i=0; i<pFdo->m_nNumTxChannels; i++ ) {
		Dta1xxTxSetTxStatusReg(  (char*)pFdo->m_TsTx[i]->m_pRegBase,
							      DTA1XX_TXSTAT_PERINT
							    | DTA1XX_TXSTAT_UFLINT
							    | DTA1XX_TXSTAT_SYNCINT
							    | DTA1XX_TXSTAT_THRINT
							    | DTA1XX_TXSTAT_SHORTINT );
	}

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Initial LED flashing -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	if ( pFdo->m_nLedFlashStartUp!=0 ) {

		// Compute next state
		switch (pFdo->m_nLedFlashState) {

		case DTA1XX_LEDFLASH_INIT:
			if (pFdo->m_nLedFlashStateCounter >= pFdo->m_nLedFlashDurationInit) {

				Dta1xxFlashLed(pFdo, 1);
				pFdo->m_nLedFlashState = DTA1XX_LEDFLASH_ON;
				pFdo->m_nLedFlashStateCounter = 0;
			}
			break;

		case DTA1XX_LEDFLASH_ON:
			if (pFdo->m_nLedFlashStateCounter >= pFdo->m_nLedFlashDurationOn) {
				Dta1xxFlashLed(pFdo, 0);
				pFdo->m_nLedFlashState = DTA1XX_LEDFLASH_OFF;
				pFdo->m_nLedFlashStateCounter = 0;
			}
			break;

		case DTA1XX_LEDFLASH_OFF:
			if (pFdo->m_nLedFlashStateCounter >= pFdo->m_nLedFlashDurationOff) {
				pFdo->m_nLedFlashFlashCounter++;
				if (pFdo->m_nLedFlashFlashCounter >= pFdo->m_nLedFlashNumFlashes) {
					pFdo->m_nLedFlashState = DTA1XX_LEDFLASH_EXIT;
				} else {
					Dta1xxFlashLed(pFdo, 1);
					pFdo->m_nLedFlashState = DTA1XX_LEDFLASH_ON;
				}
				pFdo->m_nLedFlashStateCounter = 0;
			}
			break;

		case DTA1XX_LEDFLASH_EXIT:
			if (pFdo->m_nLedFlashStateCounter >= pFdo->m_nLedFlashDurationExit) {
				Dta1xxFlashLed(pFdo, -1);
				pFdo->m_nLedFlashStartUp = 0;
			}
			break;
		}
		pFdo->m_nLedFlashStateCounter++;
	}

#if KERNELVERSION < 26
	return 0;
#else
	if ( nOurInterrupt ) {
		return IRQ_HANDLED;
	}
	else {
		return IRQ_NONE; // Can't be ours
	}
#endif
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxDisableInterrupts -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Disables the interrupts used by the driver
//
void Dta1xxDisableInterrupts(
	DTA1XX_FDO* pFdo)	// Functional device object, representing the DTA-1xx card
{
	int nIntCsr;
 	char* pPlxRegs		= (char*)pFdo->m_Pci905XConfRegs.m_pKernel;

	//.-.-.-.-.-.-.-.-.- Reset the global interrupt enable bits in PLX -.-.-.-.-.-.-.-.-.-
	//
	// Also disable the DMA channel 0 & 1 interrupts
	//
	if ( pFdo->m_nUsesPlxChip!=0 )
	{
		READ_LONG(nIntCsr, pPlxRegs, PCI905X_INT_CTRL_STAT);

		nIntCsr &= ~( PCI905X_INTCSR_PCI_INTEN |
					PCI905X_INTCSR_LOCAL_INTEN |
					PCI905X_INTCSR_DMA0_INTEN |
					PCI905X_INTCSR_DMA1_INTEN);

		WRITE_LONG(nIntCsr, pPlxRegs, PCI905X_INT_CTRL_STAT);
	}

	//.-.-.-.-.-.-.-.-.-.-.-.-.- Reset the cards interrupt bits -.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	// The driver only uses the periodic interrupt
	//
	if ( pFdo->m_nUsesGenReg!=0 )
		Dta1xxGenCtrlRegSetPerIntEn( (char*)pFdo->m_pGenRegs, 0);
	else // LEGACY
		Dta1xxTxCtrlRegSetPerIntEn( (char*)pFdo->m_Channel[0].m_pRegBase, 0 );
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxEnableInterrupts -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Enables the interrupts used by the driver
//
void Dta1xxEnableInterrupts(
	DTA1XX_FDO* pFdo)	// Functional device object, representing the DTA-1xx card
{
	int nIntCsr;
 	char* pPlxRegs		= (char*)pFdo->m_Pci905XConfRegs.m_pKernel;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.- Set the cards interrupt bits -.-.-.-.-.-.-.-.-.-.-.-.-.-
	//
	// The driver only uses the periodic interrupt
	//
	if ( pFdo->m_nUsesGenReg!=0 )
		Dta1xxGenCtrlRegSetPerIntEn( (char*)pFdo->m_pGenRegs, 1);
	else // LEGACY
		Dta1xxTxCtrlRegSetPerIntEn( (char*)pFdo->m_Channel[0].m_pRegBase, 1 );

	//-.-.-.-.-.-.-.-.-.- Set the global interrupt enable bits in PLX -.-.-.-.-.-.-.-.-.-.
	//
	// Also enable the DMA channel 0 and/or 1 interrupts
	//
	if (pFdo->m_nUsesPlxChip !=0 )
	{
		READ_LONG(nIntCsr, pPlxRegs, PCI905X_INT_CTRL_STAT);

		nIntCsr |=  (   PCI905X_INTCSR_PCI_INTEN
         		      | PCI905X_INTCSR_LOCAL_INTEN
             	      | PCI905X_INTCSR_DMA0_INTEN );

		// DTA-124/140/160: enable interrupt for DMA channel 1 too
		if (   pFdo->m_nTypeNumber==124
			|| pFdo->m_nTypeNumber==140
			|| pFdo->m_nTypeNumber==160)
			nIntCsr |= PCI905X_INTCSR_DMA1_INTEN;

		WRITE_LONG(nIntCsr, pPlxRegs, PCI905X_INT_CTRL_STAT);
	}
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxPeriodicIntHandler -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Periodic Interrupt handler routine.

//
void Dta1xxPeriodicIntHandler(
	DTA1XX_FDO* pFdo)	// Functional device object, representing the DTA-1xx card
{
 	unsigned int dDiff, ValidDiff, EldestIndex, NewIndex, ValidCnt, n, i, Idx;
 	uint64_t  n64ValidCountTemp;
	BIT_RATE_MEAS*  pBrm;		// Bit-rate measurement variables

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Estimate Input Bit Rate -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	for ( Idx=0; Idx<pFdo->m_nNumChannels; Idx++ )
	{
		// For DTA-102						: Estimate frequency of External-Clock input
		// For DTA-120/122/124/140/145/2145	: Estimate rate of Transport-Stream input
		if (   pFdo->m_nTypeNumber==102 || pFdo->m_nTypeNumber==120
			|| pFdo->m_nTypeNumber==122 || pFdo->m_nTypeNumber==124
			|| ( (pFdo->m_nTypeNumber==140
			   || pFdo->m_nTypeNumber==145 || pFdo->m_nTypeNumber==2145
	 		   || pFdo->m_nTypeNumber==160 || pFdo->m_nTypeNumber==545)
			&& (pFdo->m_Channel[Idx].m_nChanType == DTA1XX_TS_RX_CHANNEL)))
		{
			if (pFdo->m_nTypeNumber==102) {
				pBrm = &pFdo->m_TsTx[0]->m_BrMeas;
			} else if (pFdo->m_nTypeNumber==140) {
				pBrm = &pFdo->m_TsRx[0]->m_BrMeas;
			} else {
				pBrm = &pFdo->m_Channel[Idx].m_BrMeas;
			}

			// Get Valid Counter and compute difference with last difference
			ValidCnt = pBrm->m_nValidCountSample;
			ValidDiff = ValidCnt - pBrm->m_nLastValidCnt;
			pBrm->m_nLastValidCnt = ValidCnt;

			// Compute index of location where to store new sample, with wrap-around
			NewIndex = pBrm->m_nLastIndex + 1;
			if (NewIndex >= NUM_RX_MEAS_PERIODS) {
				NewIndex = 0;
			}

			// Compute index of eldest sample
			EldestIndex = NewIndex - pBrm->m_nNumValidSamps;
			if (EldestIndex < 0) {
				EldestIndex += NUM_RX_MEAS_PERIODS;
			}

			// Compute delta between new and last difference
			dDiff = ValidDiff - pBrm->m_ValidDiff[pBrm->m_nLastIndex];
			if (dDiff<-2 || dDiff>2) {
				//.-.-.- New difference differs significantly from previous difference -.-.-.-

				// Start all over again
				pBrm->m_nNumBytesTotal = 0;
				pBrm->m_nNumValidSamps = 1;
			} else {
				//.-.-.-.-.-.-.- New difference is close to previous difference -.-.-.-.-.-.-.

				// If buffer is full => subtract eldest element
				if (pBrm->m_nNumValidSamps==NUM_RX_MEAS_PERIODS) {
					pBrm->m_nNumBytesTotal -= pBrm->m_ValidDiff[EldestIndex];
				} else	{// Buffer not full => we have an extra sample
					pBrm->m_nNumValidSamps++;
    			}
			}

			// Process the Valid-Counter differential value
			pBrm->m_ValidDiff[NewIndex] = ValidDiff;
			pBrm->m_nNumBytesTotal += ValidDiff;

			// Next time, last index is current new index
			pBrm->m_nLastIndex = NewIndex;

			// Compute estimate for Valid Count
			n64ValidCountTemp  = (uint64_t)pBrm->m_nNumBytesTotal;
			n64ValidCountTemp <<= 8;

	// THE FOLLOWING LINE CAUSES A LINK ERROR "unresolved symbol __divdi3"
			//n64ValidCountTemp /= (uint64_t)pBrm->m_nNumValidSamps;
			n = pBrm->m_nNumValidSamps;
  			i = 0;
			while (n > 1 ) { n /= 2; i++; }
			n64ValidCountTemp = n64ValidCountTemp >> i;
			pBrm->m_nValidCount256 = (unsigned int)n64ValidCountTemp;
		}
	}

	//.-.-.-.-.-.-.-.-.-.-.-.- DTA-102, DTA-122: Target Detection -.-.-.-.-.-.-.-.-.-.-.-.

	if (pFdo->m_nTypeNumber==102 || pFdo->m_nTypeNumber==122) {
		//Dta1xxTargetDetectStateMachine(pFdo, 0);	// Target detection
	}

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Statistics -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	// DTA-120/140/145/160/2145/545: Sample code-violation counter
	if (   pFdo->m_nTypeNumber==120 || pFdo->m_nTypeNumber==140
		|| pFdo->m_nTypeNumber==145 || pFdo->m_nTypeNumber==2145
		|| pFdo->m_nTypeNumber==160 || pFdo->m_nTypeNumber==545) {
		for ( i=0; i<pFdo->m_nNumRxChannels; i++ ) {
            pFdo->m_TsRx[i]->m_nViolCountSamp =
					Dta1xxRxGetViolCountReg( (char*)pFdo->m_TsRx[i]->m_pRegBase );
		}
	}

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Process status flags -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	for (i=0; i<pFdo->m_nNumRxChannels; i++) {
		Dta1xxRxProcessStatusFlags(pFdo->m_TsRx[i]);
	}
	for (i=0; i<pFdo->m_nNumTxChannels; i++) {
		Dta1xxTxProcessStatusFlags(pFdo->m_TsTx[i]);
	}
}
