//*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* IoCtl.c *#*#*#*#*#*#*#*#*#* (C) 2000-2005 DEKTEC

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- License -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// This program is free software; You can freely use/modify this source code in any way
// you desire, including for commercial applications.
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	TD	2007.01.31	Added GetIoConfig support
//	TD	2007.01.29	Added support for DTA-160 ASI-only
//	KvR	14-08-2006	DTA-105 registers changed, Vpd-data handling changed
//	JH	13-07-2005	Added support for DTA-105, DTA-110 and DTA-545
//	JH	16-08-2004	Removed unused variable "i" in Dta1xxIoCtl()
//	MG	2003.06.16	Added missing calls to RxIoCtl commands (GetFlags and ClearFlags)
//	MG	2003.05.09	Added get firmware version command
//	MG	2003.04.23	Added VPD read and write commands
//	MG	21-10-2002	Created

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#include "../Include/Dta1xx.h"
#include "../Include/Dta1xxRegs.h"
#include <asm/uaccess.h>

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Type definitions -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Internal function declarations -.-.-.-.-.-.-.-.-.-.-.-.-.-.

//-.-.-.-.-.-.-.-.-.-.-.-.-.- Data declaration and definition -.-.-.-.-.-.-.-.-.-.-.-.-.-.

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Implementation +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxIoCtl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// IO control handler function. Dispatches an IO control command to the appropriate
// handler
//
int Dta1xxIoCtl(
	struct inode *inode,	//
	struct file *filp,		//
    unsigned int cmd,		// Command
	unsigned long arg)		// Pointer to the IoCtl paramater
{
	DTA1XX_RESULT ret=DTA1XX_OK;
	DTA1XX_IOCTL_DATA  io;
	DTA1XX_FDO* pFdo = (DTA1XX_FDO*)filp->private_data;

	// Check command that this is one of our commands
	if ( DTA1XX_IOCTL_MAGIC!=_IOC_TYPE(cmd) ) { return DTA1XX_E; }

	// Check memory
	if (  _IOC_DIR(cmd)==_IOC_READ) {
		if ( 0==access_ok(VERIFY_WRITE, (void*)arg, _IOC_SIZE(cmd)) ) {
            return DTA1XX_E_DEV_DRIVER;
		}
	}
	if ( _IOC_DIR(cmd)==_IOC_WRITE ) {
		if ( 0==access_ok(VERIFY_READ, (void*)arg, _IOC_SIZE(cmd)) ) {
            return DTA1XX_E_DEV_DRIVER;
		}
	}

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Copy IoCtl data from user -.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	if ( copy_from_user(&io, (DTA1XX_IOCTL_DATA*)arg, sizeof(DTA1XX_IOCTL_DATA)) ) {
#if LOG_LEVEL > 0
		DTA1XX_LOG( KERN_INFO, "Could not copy data from user!" );
#endif
		return DTA1XX_E_DEV_DRIVER;
	}

#if LOG_LEVEL > 2
	DTA1XX_LOG1( KERN_DEBUG, "IoCtl call received: 0x%08X", cmd );
#endif

	//.-.-.-.-.-.-.-.-.-.-.-.-.- Dispatch/Handle ioctl command -.-.-.-.-.-.-.-.-.-.-.-.-.-
	//
	switch (cmd)
    {

	//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ CDMA +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+

	case DTA1XX_IOCTL_CDMA_CTRL:
		ret = Dta1xxCdmaCtrl(pFdo, &(io.m_Data.m_CdmaCtrl));
		if ( copy_to_user((DTA1XX_IOCTL_DATA*)arg, &io, sizeof(DTA1XX_IOCTL_DATA)) ) {
#if LOG_LEVEL > 0
			DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
			return DTA1XX_E_DEV_DRIVER;
		}
		break;

	case DTA1XX_IOCTL_SHAREDBUF_ALLOC:
		ret = Dta1xxSharedBufAlloc(pFdo, &(io.m_Data.m_ShBufCtrl) );
		if ( copy_to_user((DTA1XX_IOCTL_DATA*)arg, &io, sizeof(DTA1XX_IOCTL_DATA)) ) {
#if LOG_LEVEL > 0
			DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
			return DTA1XX_E_DEV_DRIVER;
		}
		break;

	case DTA1XX_IOCTL_SHAREDBUF_FREE:
		ret = Dta1xxSharedBufFree(pFdo, &(io.m_Data.m_ShBufCtrl) );
		if ( copy_to_user((DTA1XX_IOCTL_DATA*)arg, &io, sizeof(DTA1XX_IOCTL_DATA)) ) {
#if LOG_LEVEL > 0
			DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
			return DTA1XX_E_DEV_DRIVER;
		}
		break;

        // Direct register access

        case DTA1XX_IOCTL_WRITE_REG:
                ret = Dta1xxWriteRegister(pFdo, io.m_Data.m_RegCtrl.m_Address,
                                          io.m_Data.m_RegCtrl.m_RegValue);
                break;

        case DTA1XX_IOCTL_READ_REG:
                ret = Dta1xxReadRegister(pFdo, io.m_Data.m_RegCtrl.m_Address,
                                          &(io.m_Data.m_RegCtrl.m_RegValue));
                if ( copy_to_user((DTA1XX_IOCTL_DATA*)arg, &io, sizeof(DTA1XX_IOCTL_DATA)) ) {
#if LOG_LEVEL > 0
                        DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
                        return DTA1XX_E_DEV_DRIVER;
                }
                break;


	//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ General IO control +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

	//-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_GET_REF_CLOCK_COUNT -.-.-.-.-.-.-.-.-.-.-.-.-
	//
	case DTA1XX_IOCTL_GET_REF_CLOCK_COUNT:
		ret = Dta1xxGetRefenceClockCount(pFdo, &(io.m_Data.m_uiValue));
        if ( copy_to_user((DTA1XX_IOCTL_DATA*)arg, &io, sizeof(DTA1XX_IOCTL_DATA)) ) {
#if LOG_LEVEL > 0
			DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
			return DTA1XX_E_DEV_DRIVER;
		}
		break;

	//-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_GET_DRIVER_VERSION -.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	case DTA1XX_IOCTL_GET_DRIVER_VERSION:
	    io.m_Data.m_DriverVersion.m_nVersionMajor	= DTA1XX_VERSION_MAJOR;
		io.m_Data.m_DriverVersion.m_nVersionMinor	= DTA1XX_VERSION_MINOR;
		io.m_Data.m_DriverVersion.m_nVersionMicro	= DTA1XX_VERSION_MICRO;
		io.m_Data.m_DriverVersion.m_nVersionBuild	= DTA1XX_VERSION_BUILD;
  		if ( copy_to_user((DTA1XX_IOCTL_DATA*)arg, &io, sizeof(DTA1XX_IOCTL_DATA)) ) {
#if LOG_LEVEL > 0
			DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
			return DTA1XX_E_DEV_DRIVER;
		}
      	        break;

	//.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_GET_LOCATION_INFO -.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	case DTA1XX_IOCTL_GET_LOCATION_INFO:
		io.m_Data.m_LocInfo.m_nVendorId		= pFdo->m_pPdo->vendor;
		io.m_Data.m_LocInfo.m_nDeviceId		= pFdo->m_pPdo->device;
		io.m_Data.m_LocInfo.m_nSubVendorId	= pFdo->m_pPdo->subsystem_vendor;
		io.m_Data.m_LocInfo.m_nSubSystemId	= pFdo->m_pPdo->subsystem_device;
		io.m_Data.m_LocInfo.m_nBusNumber	= pFdo->m_pPdo->bus->number;
		io.m_Data.m_LocInfo.m_nSlotNumber	= PCI_SLOT(pFdo->m_pPdo->devfn);
		if ( copy_to_user((DTA1XX_IOCTL_DATA*)arg, &io, sizeof(DTA1XX_IOCTL_DATA)) ) {
#if LOG_LEVEL > 0
			DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
			return DTA1XX_E_DEV_DRIVER;
		}
		break;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_GET_FW_VERSION -.-.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	case DTA1XX_IOCTL_GET_FW_VERSION:
		io.m_Data.m_uiValue					= pFdo->m_nFirmwareVersion;
		if ( copy_to_user((DTA1XX_IOCTL_DATA*)arg, &io, sizeof(DTA1XX_IOCTL_DATA)) ) {
#if LOG_LEVEL > 0
			DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
			return DTA1XX_E_DEV_DRIVER;
		}
		break;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_LED_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	//
	case DTA1XX_IOCTL_LED_CONTROL:
		ret = Dta1xxIoCtlLedControl(pFdo, io.m_Data.m_uiValue);
		break;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_RESET_SOFT -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	case DTA1XX_IOCTL_RESET_SOFT:
		ret = Dta1xxIoCtlResetSoft(pFdo);
		break;

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_READ_VPD -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	//
	case DTA1XX_IOCTL_READ_VPD:
		ret = Dta1xxIoCtlReadVpd(pFdo,
								 io.m_Data.m_Vpd.m_nStart,
								 io.m_Data.m_Vpd.m_nLength,
								 io.m_Data.m_Vpd.m_pVpdBuf);
		break;

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_WRITE_VPD -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	case DTA1XX_IOCTL_WRITE_VPD:
		ret = Dta1xxIoCtlWriteVpd(pFdo,
								  io.m_Data.m_Vpd.m_nStart,
								  io.m_Data.m_Vpd.m_nLength,
								  io.m_Data.m_Vpd.m_pVpdBuf);
		break;

	//.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_SET_IOCONFIG -.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	case DTA1XX_IOCTL_SET_IOCONFIG:
		ret = Dta1xxSetIoConfig(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
		break;

	//.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_GET_IOCONFIG -.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	case DTA1XX_IOCTL_GET_IOCONFIG:
		ret = Dta1xxGetIoConfig(pFdo, io.m_nChIdx, &(io.m_Data.m_uiValue));
		if ( copy_to_user((DTA1XX_IOCTL_DATA*)arg, &io, sizeof(DTA1XX_IOCTL_DATA)) ) {
#if LOG_LEVEL > 0
			DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
			return DTA1XX_E_DEV_DRIVER;
		}
		break;


	//.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_DMA_USERMEM_READ -.-.-.-.-.-.-.-.-.-.-.-.-.-
	//
	case DTA1XX_IOCTL_DMA_USERMEM_READ:
		ret = Dta1xxRxIoCtlDma(pFdo, io.m_nChIdx,
							   io.m_Data.m_DataTransDesc.m_pBuffer,
							   io.m_Data.m_DataTransDesc.m_nNumBytesToTr);
		break;

	//.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_IOCTL_DMA_USERMEM_WRITE -.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	case DTA1XX_IOCTL_DMA_USERMEM_WRITE:
		ret = Dta1xxTxIoCtlDma(pFdo, io.m_nChIdx,
							   io.m_Data.m_DataTransDesc.m_pBuffer,
							   io.m_Data.m_DataTransDesc.m_nNumBytesToTr);
		break;

	//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Rx channel IO control +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_EQUALISE -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	//
	case DTA1XX_RX_IOCTL_EQUALISE:
		ret = Dta1xxRxIoCtlEqualise(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
		break;

	//.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_GET_FIFO_LOAD -.-.-.-.-.-.-.-.-.-.-.-.-.-
	//
	case DTA1XX_RX_IOCTL_GET_FIFO_LOAD:
		ret = Dta1xxRxIoCtlGetFifoLoad(pFdo, io.m_nChIdx, &io.m_Data.m_uiValue);
		if ( copy_to_user((DTA1XX_IOCTL_DATA*)arg, &io, sizeof(DTA1XX_IOCTL_DATA)) ) {
#if LOG_LEVEL > 0
			DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
			return DTA1XX_E_DEV_DRIVER;
		}
		break;

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_GET_FLAGS -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	//
	case DTA1XX_RX_IOCTL_GET_FLAGS:
		ret = Dta1xxRxIoCtlGetFlags(pFdo, io.m_nChIdx,
									&io.m_Data.m_Flags.m_nStatus,
									&io.m_Data.m_Flags.m_nLatched);
		if ( copy_to_user((DTA1XX_IOCTL_DATA*)arg, &io, sizeof(DTA1XX_IOCTL_DATA)) ) {
#if LOG_LEVEL > 0
			DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
			return DTA1XX_E_DEV_DRIVER;
		}
		break;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_CLEAR_FLAGS -.-.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	case DTA1XX_RX_IOCTL_CLEAR_FLAGS:
		ret = Dta1xxRxIoCtlClearFlags(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
		break;

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_GET_STATUS -.-.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	case DTA1XX_RX_IOCTL_GET_STATUS:
		ret = Dta1xxRxIoCtlGetStatus(pFdo, io.m_nChIdx,
									 &io.m_Data.m_RxStatus.m_nPacketSize,
									 &io.m_Data.m_RxStatus.m_nNumInv,
									 &io.m_Data.m_RxStatus.m_nClkDet,
									 &io.m_Data.m_RxStatus.m_nAsiLock,
									 &io.m_Data.m_RxStatus.m_nRateOk,
									 &io.m_Data.m_RxStatus.m_nAsiInv);

		if ( copy_to_user((DTA1XX_IOCTL_DATA*)arg, &io, sizeof(DTA1XX_IOCTL_DATA)) ) {
#if LOG_LEVEL > 0
			DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
			return DTA1XX_E_DEV_DRIVER;
		}
		break;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_GET_TARGETID -.-.-.-.-.-.-.-.-.-.-.-.-.-
	//
	case DTA1XX_RX_IOCTL_GET_TARGETID:
		ret = Dta1xxRxIoCtlGetTargetId(pFdo, io.m_nChIdx,
									   &io.m_Data.m_TargetInfo.m_nPresent,
									   &io.m_Data.m_TargetInfo.m_nTargetId);

		if ( copy_to_user((DTA1XX_IOCTL_DATA*)arg, &io, sizeof(DTA1XX_IOCTL_DATA)) ) {
#if LOG_LEVEL > 0
			DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
			return DTA1XX_E_DEV_DRIVER;
		}
		break;

	//-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_GET_TS_RATE_BPS -.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	case DTA1XX_RX_IOCTL_GET_TS_RATE_BPS:
		ret = Dta1xxRxIoCtlGetTsRateBps(pFdo, io.m_nChIdx, &io.m_Data.m_uiValue);
		if ( copy_to_user((DTA1XX_IOCTL_DATA*)arg, &io, sizeof(DTA1XX_IOCTL_DATA)) ) {
#if LOG_LEVEL > 0
			DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
			return DTA1XX_E_DEV_DRIVER;
		}
		break;

	//-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_POLARITY_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-
	//
	case DTA1XX_RX_IOCTL_POLARITY_CONTROL:
		ret = Dta1xxRxIoCtlPolarityControl(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
		break;

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_RESET -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	//
	case DTA1XX_RX_IOCTL_RESET:
		ret = Dta1xxRxIoCtlReset(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
		break;

	//.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_SET_LOOPBACK_MODE -.-.-.-.-.-.-.-.-.-.-.-.-
	//
	case DTA1XX_RX_IOCTL_SET_LOOPBACK_MODE:
		ret = Dta1xxRxIoCtlSetLoopBackMode(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
		break;

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_SET_POWER -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	//
	case DTA1XX_RX_IOCTL_SET_POWER:
		ret = Dta1xxRxIoCtlSetPower(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
		break;

	//.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_SET_RX_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	case DTA1XX_RX_IOCTL_SET_RX_CONTROL:
		ret = Dta1xxRxIoCtlSetRxControl(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
		break;

	//.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_GET_RX_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	case DTA1XX_RX_IOCTL_GET_RX_CONTROL:
		ret = Dta1xxRxIoCtlGetRxControl(pFdo, io.m_nChIdx, &(io.m_Data.m_uiValue));
		if ( copy_to_user((DTA1XX_IOCTL_DATA*)arg, &io, sizeof(DTA1XX_IOCTL_DATA)) ) {
#if LOG_LEVEL > 0
			DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
			return DTA1XX_E_DEV_DRIVER;
		}
		break;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_SET_RX_MODE -.-.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	case DTA1XX_RX_IOCTL_SET_RX_MODE:
		ret = Dta1xxRxIoCtlSetRxMode(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
		break;

	//.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_RX_IOCTL_GET_STATISTICS -.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	case DTA1XX_RX_IOCTL_GET_STATISTICS:
		ret = Dta1xxRxIoCtlGetStatistics(pFdo, io.m_nChIdx, &(io.m_Data.m_uiValue));
		if ( copy_to_user((DTA1XX_IOCTL_DATA*)arg, &io, sizeof(DTA1XX_IOCTL_DATA)) ) {
#if LOG_LEVEL > 0
			DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
			return DTA1XX_E_DEV_DRIVER;
		}
		break;

	//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Tx channel IO control +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+

	//-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_CLEAR_FLAGS -.-.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	case DTA1XX_TX_IOCTL_CLEAR_FLAGS:
		ret = Dta1xxTxIoCtlClearFlags(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
		break;

	//-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_GET_EXT_CLK_FREQ -.-.-.-.-.-.-.-.-.-.-.-.-
	//
	case DTA1XX_TX_IOCTL_GET_EXT_CLK_FREQ:
		ret = Dta1xxTxIoCtlGetExtClkFreq(pFdo, io.m_nChIdx, &io.m_Data.m_uiValue);
		if ( copy_to_user((DTA1XX_IOCTL_DATA*)arg, &io, sizeof(DTA1XX_IOCTL_DATA)) ) {
#if LOG_LEVEL > 0
			DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
			return DTA1XX_E_DEV_DRIVER;
		}
		break;

	//.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_GET_FIFO_LOAD -.-.-.-.-.-.-.-.-.-.-.-.-.-
	//
	case DTA1XX_TX_IOCTL_GET_FIFO_LOAD:
		ret = Dta1xxTxIoCtlGetFifoLoad(pFdo, io.m_nChIdx, &io.m_Data.m_uiValue);
		if ( copy_to_user((DTA1XX_IOCTL_DATA*)arg, &io, sizeof(DTA1XX_IOCTL_DATA)) ) {
#if LOG_LEVEL > 0
			DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
			return DTA1XX_E_DEV_DRIVER;
		}
		break;

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_GET_FLAGS -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	//
	case DTA1XX_TX_IOCTL_GET_FLAGS:
		ret = Dta1xxTxIoCtlGetFlags(pFdo, io.m_nChIdx,
									&io.m_Data.m_Flags.m_nStatus,
									&io.m_Data.m_Flags.m_nLatched);
		if ( copy_to_user((DTA1XX_IOCTL_DATA*)arg, &io, sizeof(DTA1XX_IOCTL_DATA)) ) {
#if LOG_LEVEL > 0
			DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
			return DTA1XX_E_DEV_DRIVER;
		}
		break;

	//.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_GET_MAX_FIFO_SIZE -.-.-.-.-.-.-.-.-.-.-.-.-
	//
	case DTA1XX_TX_IOCTL_GET_MAX_FIFO_SIZE:
		ret = Dta1xxTxIoCtlGetMaxFifoSize(pFdo, io.m_nChIdx, &io.m_Data.m_uiValue);
		if ( copy_to_user((DTA1XX_IOCTL_DATA*)arg, &io, sizeof(DTA1XX_IOCTL_DATA)) ) {
#if LOG_LEVEL > 0
			DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
			return DTA1XX_E_DEV_DRIVER;
		}
		break;

	//.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_GET_TARGET_ID -.-.-.-.-.-.-.-.-.-.-.-.-.-
	//
	case DTA1XX_TX_IOCTL_GET_TARGET_ID:
		ret = Dta1xxTxIoCtlGetTargetId(pFdo, io.m_nChIdx,
									   &io.m_Data.m_TargetInfo.m_nPresent,
									   &io.m_Data.m_TargetInfo.m_nTargetId);
		if ( copy_to_user((DTA1XX_IOCTL_DATA*)arg, &io, sizeof(DTA1XX_IOCTL_DATA)) ) {
#if LOG_LEVEL > 0
			DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
			return DTA1XX_E_DEV_DRIVER;
		}
		break;

	//-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_GET_TS_RATE_BPS -.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	case DTA1XX_TX_IOCTL_GET_TS_RATE_BPS:
		// THIS FUNCTION IS NO LONGER SUPPORTED. USE DTA1XX_TX_IOCTL_GET_TX_CLOCK INSTEAD
		ret = DTA1XX_E_NOT_SUPPORTED;
		break;

	//.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_GET_TX_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	case DTA1XX_TX_IOCTL_GET_TX_CONTROL:
		ret = Dta1xxTxIoCtlGetTxControl(pFdo, io.m_nChIdx, &io.m_Data.m_uiValue);
		if ( copy_to_user((DTA1XX_IOCTL_DATA*)arg, &io, sizeof(DTA1XX_IOCTL_DATA)) ) {
#if LOG_LEVEL > 0
			DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
			return DTA1XX_E_DEV_DRIVER;
		}
		break;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_GET_TX_MODE -.-.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	case DTA1XX_TX_IOCTL_GET_TX_MODE:
		ret = Dta1xxTxIoCtlGetTxMode(pFdo, io.m_nChIdx,
									 &io.m_Data.m_TxMode.m_nTxPacketMode,
									 &io.m_Data.m_TxMode.m_nTxStuffMode);
		if ( copy_to_user((DTA1XX_IOCTL_DATA*)arg, &io, sizeof(DTA1XX_IOCTL_DATA)) ) {
#if LOG_LEVEL > 0
			DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
			return DTA1XX_E_DEV_DRIVER;
		}
		break;

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_RESET -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	//
	case DTA1XX_TX_IOCTL_RESET:
		ret = Dta1xxTxIoCtlReset(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
		break;

	//.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_SET_FIFO_SIZE -.-.-.-.-.-.-.-.-.-.-.-.-.-
	//
	case DTA1XX_TX_IOCTL_SET_FIFO_SIZE:
		ret = Dta1xxTxIoCtlSetFifoSize(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
		break;

	//.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_SET_LOOPBACK_MODE -.-.-.-.-.-.-.-.-.-.-.-.-
	//
	case DTA1XX_TX_IOCTL_SET_LOOPBACK_MODE:
		ret = Dta1xxTxIoCtlSetLoopBackMode(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
		break;

	//-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_RD_LOOPBACK_DATA -.-.-.-.-.-.-.-.-.-.-.-.-
	//
	case DTA1XX_TX_IOCTL_RD_LOOPBACK_DATA:
		ret = Dta1xxTxIoCtlReadLoopBackData(pFdo, io.m_nChIdx,
											io.m_Data.m_DataTransDesc.m_pBuffer,
											io.m_Data.m_DataTransDesc.m_nNumBytesToTr);
		break;

	//-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_SET_TX_POLARITY -.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	case DTA1XX_TX_IOCTL_SET_TX_POLARITY:
		ret = Dta1xxTxIoCtlSetTxPolarity(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
		break;

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_SET_POWER -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	//
	case DTA1XX_TX_IOCTL_SET_POWER:
		ret = Dta1xxTxIoCtlSetPower(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
		break;

	//-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_SET_TS_RATE_BPS -.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	case DTA1XX_TX_IOCTL_SET_TS_RATE_BPS:
		// THIS FUNCTION IS NO LONGER SUPPORTED. USE DTA1XX_TX_IOCTL_SET_TX_CLOCK INSTEAD
		ret = DTA1XX_E_NOT_SUPPORTED;
		break;

	//.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_SET_TX_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	case DTA1XX_TX_IOCTL_SET_TX_CONTROL:
		ret = Dta1xxTxIoCtlSetTxControl(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
		break;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_SET_TX_MODE -.-.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	case DTA1XX_TX_IOCTL_SET_TX_MODE:
		ret = Dta1xxTxIoCtlSetTxMode(pFdo, io.m_nChIdx,
									 io.m_Data.m_TxMode.m_nTxPacketMode,
									 io.m_Data.m_TxMode.m_nTxStuffMode);
		break;

	//-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_GET_MOD_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	case DTA1XX_TX_IOCTL_GET_MOD_CONTROL:
		ret = Dta1xxTxIoCtlGetModControl(pFdo, io.m_nChIdx,
										 &(io.m_Data.m_ModCtrl.m_nModType),
										 &(io.m_Data.m_ModCtrl.m_nCodeRate),
										 &(io.m_Data.m_ModCtrl.m_nParXtra1),
										 &(io.m_Data.m_ModCtrl.m_nParXtra2));
		if ( copy_to_user((DTA1XX_IOCTL_DATA*)arg, &io, sizeof(DTA1XX_IOCTL_DATA)) ) {
#if LOG_LEVEL > 0
			DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
			return DTA1XX_E_DEV_DRIVER;
		}
		break;

	//-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_SET_MOD_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	case DTA1XX_TX_IOCTL_SET_MOD_CONTROL:
		ret = Dta1xxTxIoCtlSetModControl(pFdo, io.m_nChIdx,
										 io.m_Data.m_ModCtrl.m_nModType,
										 io.m_Data.m_ModCtrl.m_nCodeRate,
										 io.m_Data.m_ModCtrl.m_nParXtra1,
										 io.m_Data.m_ModCtrl.m_nParXtra2);

		break;

	//.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_GET_RF_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	case DTA1XX_TX_IOCTL_GET_RF_CONTROL:
		ret = Dta1xxTxIoCtlGetRfControl(pFdo, io.m_nChIdx,
										&(io.m_Data.m_RfCtrl.m_nPllA),
										&(io.m_Data.m_RfCtrl.m_nPllB),
										&(io.m_Data.m_RfCtrl.m_nLockStatus));
		if ( copy_to_user((DTA1XX_IOCTL_DATA*)arg, &io, sizeof(DTA1XX_IOCTL_DATA)) ) {
#if LOG_LEVEL > 0
			DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
			return DTA1XX_E_DEV_DRIVER;
		}
		break;

	//.-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_SET_RF_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	case DTA1XX_TX_IOCTL_SET_RF_CONTROL:
		ret = Dta1xxTxIoCtlSetRfControl(pFdo, io.m_nChIdx, io.m_Data.m_RfCtrl.m_nPllA,
			                            io.m_Data.m_RfCtrl.m_nPllB);
		break;

	//-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_GET_TX_CLOCK -.-.-.-.-.-.-.-.-.-.-.-.-
	//
	case DTA1XX_TX_IOCTL_GET_TX_CLOCK:
		ret = Dta1xxTxIoCtlGetTxClock(pFdo, io.m_nChIdx,
									  &(io.m_Data.m_TxClockInfo.m_nClockGenMode),
									  &(io.m_Data.m_TxClockInfo.m_nPhaseIncr),
									  &(io.m_Data.m_TxClockInfo.m_nSymbRateMult));
		if ( copy_to_user((DTA1XX_IOCTL_DATA*)arg, &io, sizeof(DTA1XX_IOCTL_DATA)) ) {
#if LOG_LEVEL > 0
			DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
			return DTA1XX_E_DEV_DRIVER;
		}
		break;

	//-.-.-.-.-.-.-.-.-.-.-.-.- DTA1XX_TX_IOCTL_SET_TX_CLOCK -.-.-.-.-.-.-.-.-.-.-.-.-
	//
	case DTA1XX_TX_IOCTL_SET_TX_CLOCK:
		ret = Dta1xxTxIoCtlSetTxClock(pFdo, io.m_nChIdx,
									  io.m_Data.m_TxClockInfo.m_nClockGenMode,
									  io.m_Data.m_TxClockInfo.m_nPhaseIncr,
									  io.m_Data.m_TxClockInfo.m_nSymbRateMult);
		break;

	default:
		ret = DTA1XX_E_DEV_DRIVER;
		break;
	}
	return (int)ret;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxGetRefenceClockCount -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// IoCtl handler for getting teh refernce clock count value
//
DTA1XX_RESULT Dta1xxGetRefenceClockCount(
	DTA1XX_FDO* pFdo,		// Functional device object, representing the DTA-1xx card
	unsigned int* pRefClkCnt)	// Reference clock value
{

	// Supported by: DTA-105, DTA-107, DTA-110, DTA-120 (FW>=4), DTA-122 (FW>=4), DTA-124
	// DTA-140, DTA-145, DTA-160, DTA-2145
	if ( !(   (pFdo->m_nTypeNumber==105) || (pFdo->m_nTypeNumber==107) || (pFdo->m_nTypeNumber==110)
		   || (pFdo->m_nTypeNumber==120 && pFdo->m_nFirmwareVersion>=4)
		   || (pFdo->m_nTypeNumber==122 && pFdo->m_nFirmwareVersion>=4)
		   || (pFdo->m_nTypeNumber==124) || (pFdo->m_nTypeNumber==140) || (pFdo->m_nTypeNumber==145)
		   || (pFdo->m_nTypeNumber==160) || (pFdo->m_nTypeNumber==2145)) )
	{
		return DTA1XX_E_NOT_SUPPORTED;
	}
	// Get the sample
	*pRefClkCnt = Dta1xxGenGetRefClkCntReg((char*)(pFdo->m_pGenRegs));
	return DTA1XX_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxIoCtlLedControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// IoCtl handler for the led control command
//
DTA1XX_RESULT Dta1xxIoCtlLedControl(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nLedControl)	// LED control value
{
	int nUseGenRegs = 0;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Check parameter -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	if (nLedControl!=DTA1XX_LED_HARDWARE && nLedControl!=DTA1XX_LED_OFF &&
		nLedControl!=DTA1XX_LED_GREEN && nLedControl!=DTA1XX_LED_RED &&
		nLedControl!=DTA1XX_LED_YELLOW)
	{
		return DTA1XX_E_INVALID_MODE;
	}

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Set LED control -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	// Use general register?
	nUseGenRegs = pFdo->m_nUsesGenReg;

	if ( nUseGenRegs ) {
		Dta1xxGenLedControl( (char*)(pFdo->m_pGenRegs), nLedControl);
	} else {
		if ( pFdo->m_nTypeNumber==102 )
			Dta1xxTxLedControl( (char*)(pFdo->m_TsTx[0]->m_pRegBase), nLedControl);
		else if ( pFdo->m_nTypeNumber==120 || pFdo->m_nTypeNumber==122 )
			Dta1xxRxLedControl( (char*)(pFdo->m_TsRx[0]->m_pRegBase), nLedControl);
	}
	return DTA1XX_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxIoCtlResetSoft -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// IoCtl handler for the reset card command
//
DTA1XX_RESULT Dta1xxIoCtlResetSoft(
	DTA1XX_FDO* pFdo)	// Functional device object, representing the DTA-1xx card
{
	switch (pFdo->m_nTypeNumber)
	{
	case 100:
	case 102:
		Dta1xxTxGenCtrlRegReset( (char*)(pFdo->m_TsTx[0]->m_pRegBase) );
		break;
	case 120:
		if ( pFdo->m_nFirmwareVersion>=4 ) {
			// DTA-120s with Firmware Revision>=4, have address map with a general
			// section
			Dta1xxGenCtrlRegReset( (char*)(pFdo->m_pGenRegs) );
		} else {
			Dta1xxRxGenCtrlRegReset( (char*)(pFdo->m_TsRx[0]->m_pRegBase) );
		}
		break;
	case 122:
		Dta1xxRxGenCtrlRegReset( (char*)(pFdo->m_TsRx[0]->m_pRegBase) );
		break;

    case 105:
	case 107:
    case 110:
	case 124:
	case 140:
	case 145:
	case 160:
	case 2145:
    case 545:
		Dta1xxGenCtrlRegReset( (char*)(pFdo->m_pGenRegs) );
		break;
	}
 	return DTA1XX_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxIoCtlReadVpd -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
DTA1XX_RESULT Dta1xxIoCtlReadVpd(
	DTA1XX_FDO* pFdo,		// Functional device object, representing the DTA-1xx card
	int nStart,				// Start BYTE address
	int nCount,				// Number of VPD bytes to transfer
	unsigned char* pData)	// Buffer with VPD bytes
{
	DTA1XX_RESULT dr = DTA1XX_E_EEPROM_READ;
	unsigned char Buf[DTA1XX_VPD_MAX_TRANSFER];

#if LOG_LEVEL > 1
	DTA1XX_LOG2( KERN_INFO, "Read VPD data, start=0x%03x, size=%d", nStart, nCount );
#endif

	if ( nStart >= DTA1XX_VPD_MAX_TRANSFER) {
#if LOG_LEVEL > 0
		DTA1XX_LOG( KERN_INFO, "Start address not valid" );
#endif
	} else if (  (nStart+nCount) > DTA1XX_VPD_MAX_TRANSFER ) {
#if LOG_LEVEL > 0
		DTA1XX_LOG( KERN_INFO, "Length not valid" );
#endif
	} else {
		if ( 0==Dta1xxVpdRead(pFdo, Buf, nStart, nCount) ) {
			// Copy data to transfer to user buffer
			if ( copy_to_user(pData, Buf, nCount) ) {
#if LOG_LEVEL > 0
				DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
				return DTA1XX_E_EEPROM_READ;
			}
			dr = DTA1XX_OK;
		} else {
			dr = DTA1XX_E_EEPROM_READ;
		}
	}
	return dr;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxIoCtlWriteVpd -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
DTA1XX_RESULT Dta1xxIoCtlWriteVpd(
	DTA1XX_FDO* pFdo,		// Functional device object, representing the DTA-1xx card
	int nStart,				// Start BYTE address
	int nCount,				// Number of VPD bytes to transfer
	unsigned char* pData)	// Buffer with VPD bytes
{
	DTA1XX_RESULT dr = DTA1XX_E_EEPROM_WRITE;
	unsigned char Buf[DTA1XX_VPD_MAX_TRANSFER];

#if LOG_LEVEL > 1
	DTA1XX_LOG2( KERN_INFO, "Write VPD data, start=0x%03x, size=%d", nStart, nCount );
#endif

	if ( nStart >= DTA1XX_VPD_MAX_TRANSFER) {
#if LOG_LEVEL > 0
		DTA1XX_LOG( KERN_INFO, "Start address not valid" );
#endif
	} else if (  (nStart+nCount) > DTA1XX_VPD_MAX_TRANSFER ) {
#if LOG_LEVEL > 0
		DTA1XX_LOG( KERN_INFO, "Length not valid" );
#endif
	} else {
		// Copy data to transfer from user buffer
		if ( copy_from_user(Buf, pData, nCount) ) {
#if LOG_LEVEL > 0
			DTA1XX_LOG( KERN_INFO, "Could not copy data to user!" );
#endif
			return DTA1XX_E_EEPROM_WRITE;
		}

		if ( 0==Dta1xxVpdWrite(pFdo, Buf, nStart, nCount) ) {
			dr = DTA1XX_OK;
		}
	}
	return dr;
}
