//#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* LowLevel.c *#*#*#*#*#*#*#*#* (C) 2000-2005 DEKTEC

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- License -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// This program is free software; You can freely use/modify this source code in any way
// you desire, including for commercial applications.
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	TD	2007.01.29	Addes support for DTA-160 ASI-only
//	MG	21-10-2002	Created

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#include "../Include/Dta1xx.h"
#include "../Include/Dta1xxRegs.h"

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Type definitions -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Internal function declarations -.-.-.-.-.-.-.-.-.-.-.-.-.-.

//-.-.-.-.-.-.-.-.-.-.-.-.-.- Data declaration and definition -.-.-.-.-.-.-.-.-.-.-.-.-.-.

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Implementation +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxFlashLed -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Set state of generic front-panel LED.
//
int Dta1xxFlashLed(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int  LedState)		// -1 = LED controlled by hardware
							//  0 = LED off
							//  1 = LED orange
{
	int nUseGenRegs = 0;

	// Use general register?
	nUseGenRegs = pFdo->m_nUsesGenReg;

	switch (LedState) {

	case 0:		// LED off
		if ( nUseGenRegs ) {
			Dta1xxGenLedControl( (char*)pFdo->m_pGenRegs, 0x1 );
		} else {
			// LEGACY: DTA-120 (FW<4), DTA-122
			Dta1xxRxLedControl(  (char*)pFdo->m_pGenRegs, 0x1 );
		}
		break;

	case 1:		// LED orange
		if ( nUseGenRegs ) {
			Dta1xxGenLedControl( (char*)pFdo->m_pGenRegs, 0x7 );
		} else {
			// LEGACY: DTA-120 (FW<4), DTA-122 (FW<4)
			Dta1xxRxLedControl( (char*)pFdo->m_pGenRegs, 0x7 );
		}
		break;

	default:	// Transfer LED control back to hardware
		if ( nUseGenRegs ) {
			Dta1xxGenLedControl( (char*)pFdo->m_pGenRegs, 0 );
		} else {
			// LEGACY: DTA-120 (FW<4), DTA-122 (FW<4)
			Dta1xxRxLedControl( (char*)pFdo->m_pGenRegs, 0 );
		}
		break;
	}
	return 0;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxReset -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Issue soft reset of transmit channel.
//
int Dta1xxTxReset(
	CHANNEL*  pCh,			// Transport-Stream Transmit Channel
	int  ResetMode)			// DTAPI_FIFO_RESET / DTAPI_FULL_RESET
{
	int  TypeNumber = pCh->m_pFdo->m_nTypeNumber;
	int  FirmwareVersion = pCh->m_pFdo->m_nFirmwareVersion;

	// ResetMode determines which reset bit is used
	if (ResetMode == DTA1XX_RST_CLEARFIFO) {

		// For old revisions: Use mode DTA1XX_RST_CHANNEL with a recursive call
		if (   (TypeNumber==100 && FirmwareVersion<=2)
			|| (TypeNumber==102 && FirmwareVersion<=4)) {
			Dta1xxTxReset(pCh, DTA1XX_RST_CHANNEL);
		}
		else {
			// Card is sufficiently up-to-date to just reset the FIFO
			((Dta1xxTx*)(pCh->m_pRegBase))->m_TxControl.m_ClrFifo = 1;
		}
	}
	else if (ResetMode == DTA1XX_RST_CHANNEL) {

		// Full reset on DTA-100/102, Transmit-Channel reset on new boards (DTA-140...)
		if (TypeNumber==100 || TypeNumber==102)
			((Dta1xxGen*)(((Dta1xxTx*)(pCh->m_pRegBase))))->m_GenControl.m_Reset = 1;
		else
			((Dta1xxTx*)(pCh->m_pRegBase))->m_TxControl.m_TxRst = 1;
	}

	return 0;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxReset -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Issue soft reset of receive channel.
//
int  Dta1xxRxReset(
	CHANNEL*  pTsRx,		// Transport-Stream Receive Channel
	int  ResetMode)			// DTAPI_FIFO_RESET / DTAPI_FULL_RESET
{
	int  TypeNumber = pTsRx->m_pFdo->m_nTypeNumber;
	int  FirmwareVersion = pTsRx->m_pFdo->m_nFirmwareVersion;

	// ResetMode determines which reset bit is used
	if (ResetMode == DTA1XX_RST_CLEARFIFO) {

		// For old revisions: Use mode DTA1XX_RST_CHANNEL with a recursive call
		if (   (TypeNumber==120 && FirmwareVersion<=3)
			|| (TypeNumber==122 && FirmwareVersion<=3)
			|| (TypeNumber==140 && FirmwareVersion<1)) {
			Dta1xxRxReset(pTsRx, DTA1XX_RST_CHANNEL);
		}
		else {
			// Card is sufficiently up-to-date to just reset the FIFO
			((Dta1xxRx*)(pTsRx->m_pRegBase))->m_RxControl.m_RxClrFifo = 1;
		}
	}
	else if (ResetMode == DTA1XX_RST_CHANNEL) {

		// DTA-120/122, old revision  => full reset
		// DTA-120/122, new revision  => Receive-Channel reset
		// DTA-140                    => Receive-Channel reset
		if (   (TypeNumber==120 && FirmwareVersion<=3)
			|| (TypeNumber==122 && FirmwareVersion<=3)) {
			((Dta1xxGen*)(((Dta1xxRx*)(pTsRx->m_pRegBase))))->m_GenControl.m_Reset = 1;
		}
		else {
			((Dta1xxRx*)(pTsRx->m_pRegBase))->m_RxControl.m_RxRst = 1;
		}
	}
	return 0;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxReadRegister -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
int  Dta1xxReadRegister(DTA1XX_FDO* pFdo, Int Address, Int* pValue)
{
  int32_t Val;
  volatile char* pBase = pFdo->m_Dta1xxRegsBase.m_pKernel;
  READ_LONG(Val, pBase, Address);
  *pValue = (Int)Val;
  return 0;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxWriteRegister -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
int  Dta1xxWriteRegister(DTA1XX_FDO* pFdo, Int Address, Int Value)
{
  volatile char* pBase = pFdo->m_Dta1xxRegsBase.m_pKernel;
  WRITE_LONG((int32_t)(Value), pBase, Address);
  return 0;
}
