//*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* MapMemory.c *#*#*#*#*#*#*#*#*#*#*# (C) 2006 DekTec
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	2006.12.18	MG	Created

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#include "../Include/Dta1xx.h"
#include "../Include/Dta1xxRegs.h"

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxSharedBufAlloc -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
int Dta1xxSharedBufAlloc(
	DTA1XX_FDO* pFdo,
	DTA1XX_SHAREDBUF_CTRL* pShBufCtrl)
{
	Int rc, TotalNumPages=0;
	CHANNEL* pCh=NULL;
	ShBufDesc* pShBufDesc=NULL;

#if LOG_LEVEL > 1
		DTA1XX_LOG1( KERN_INFO, "[%d] Dta1xxSharedBufAlloc: Allocating shared buffer",
					 pShBufCtrl->m_PortIndex);
#endif

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Pre-Allocation Checks -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	// FOR NOW: we only support the DTA-110(T)
	if ( pFdo->m_nTypeNumber != 110 || pShBufCtrl->m_PortIndex != 0 ) {
		return -1;
	}
	// FOR NOW: we only support shared buffers for CDMA purposes
	if ( pShBufCtrl->m_Purpose != DTA1XX_SHBUF_DIRIQ_CDMA ) {
		return -1;
	}

	// Get channel & shared buffer descriptor
	pCh = &(pFdo->m_Channel[pShBufCtrl->m_PortIndex]);
	pShBufDesc = &(pCh->m_CdmaShBuf);

	// Sanity check: no CDMA may be running
	if ( pCh->m_CdmaDmaRunning != FALSE )
	{
#if LOG_LEVEL > 0
		DTA1XX_LOG1( KERN_INFO, "[%d] Dta1xxSharedBufAlloc: CDMA STILL RUNNING?!?",
					 pShBufCtrl->m_PortIndex);
#endif
		return -1;
	}

	// Shared buffer may not have been allocated already
	if ( pShBufDesc->m_Allocated != FALSE )
	{
#if LOG_LEVEL > 0
		DTA1XX_LOG1( KERN_INFO, "[%d] Dta1xxSharedBufAlloc: BUFFER ALREADY ALLOCATED",
				     pShBufCtrl->m_PortIndex);
#endif
		return -1;
	}

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Main Allocation Code -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	// Init shared-buffer descriptor
	pShBufDesc->m_PageList = NULL;
	pShBufDesc->m_NumPages = 0;
	pShBufDesc->m_SglAllocated = FALSE;
    pShBufDesc->m_pUserBuf = pShBufCtrl->m_pSharedBuf;
    pCh->m_CdmaBufSize = pShBufCtrl->m_SharedBufSize;

	// Compute number of pages
        TotalNumPages = pCh->m_CdmaBufSize/PAGE_SIZE;
        // Must be multiple of pages
        if ( (pCh->m_CdmaBufSize % PAGE_SIZE)!=0 ) {
#if LOG_LEVEL > 0
                DTA1XX_LOG3( KERN_INFO,
                             "[%d] Dta1xxSharedBufAlloc: Buffer (size: %d) is not a "
                             "multiple of pages (page size: %d)",
                             pShBufCtrl->m_PortIndex, pCh->m_CdmaBufSize,
                             (Int)(PAGE_SIZE));
#endif

          return -1;
        }

	// Lock user buffer, but first allocate memory for page list
	pShBufDesc->m_PageList = kmalloc( sizeof(struct page*) * TotalNumPages, GFP_KERNEL );
	if ( pShBufDesc->m_PageList == NULL )
	{
#if LOG_LEVEL > 0
		DTA1XX_LOG1( KERN_INFO,
					 "[%d] Dta1xxSharedBufAlloc: Failed to allocate page-list",
				     pShBufCtrl->m_PortIndex);
#endif
		return -1;
	}
	pShBufDesc->m_NumPages = TotalNumPages;


	// Get mmap reader/writer semaphore
	down_read(&current->mm->mmap_sem);

	// Lock pages
	rc = get_user_pages(
			current,        // Task performing I/O
			current->mm,    // The tasks memory-management structure
			(UInt32)(pShBufDesc->m_pUserBuf),   // Page-aligned starting address of user buffer
			TotalNumPages,  // Length of the buffer in pages
			1,              // Map for read access (i.e. user app performing a write)
			0,              // Do not force an override of page protections
			pShBufDesc->m_PageList, // Will contain list of page pointers describing buffer
			NULL);          // Will contain list of associated VMAs

	// Release mmap semaphore
	up_read(&current->mm->mmap_sem);

	if ( rc != TotalNumPages )
	{
#if LOG_LEVEL > 0
		DTA1XX_LOG1( KERN_INFO,
					 "[%d] Dta1xxSharedBufAlloc: Failed to lock user buffer",
				     pShBufCtrl->m_PortIndex);
#endif

		kfree( pShBufDesc->m_PageList );
		pShBufDesc->m_PageList = NULL;
		pShBufDesc->m_NumPages = 0;
		pShBufDesc->m_Allocated = FALSE;
		pShBufDesc->m_SglAllocated = FALSE;
		return -1;
	}
	pShBufDesc->m_Allocated = TRUE;

#if LOG_LEVEL > 1
	DTA1XX_LOG2( KERN_INFO,
				 "[%d] Dta1xxSharedBufAlloc: Locked user-buffer (num pages:%d)",
				 pShBufCtrl->m_PortIndex, TotalNumPages );
#endif

	// Allocate buffer for scatter-gather descriptors
	if ( pShBufCtrl->m_Purpose == DTA1XX_SHBUF_DIRIQ_CDMA ) {
		if ( 0 == Dta1xxCdmaSglCreate(pCh) )
			pShBufDesc->m_SglAllocated = TRUE;
	}
	return 0;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxSharedBufFree -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
int Dta1xxSharedBufFree(
	DTA1XX_FDO* pFdo,
	DTA1XX_SHAREDBUF_CTRL* pShBufCtrl)
{
	CHANNEL* pCh=NULL;
	ShBufDesc* pShBufDesc=NULL;

#if LOG_LEVEL > 1
		DTA1XX_LOG1( KERN_INFO, "[%d] Dta1xxSharedBufRelease: free shared buffer",
					 pShBufCtrl->m_PortIndex );
#endif

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Pre-Release Checks -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	// FOR NOW: we only support the DTA-110(T)
	if ( pFdo->m_nTypeNumber != 110 || pShBufCtrl->m_PortIndex != 0 ) {
		return -1;
	}
	// FOR NOW: we only support shared buffers for CDMA purposes
	if ( pShBufCtrl->m_Purpose != DTA1XX_SHBUF_DIRIQ_CDMA ) {
		return -1;
	}

	// Get channel & shared buffer descriptor
	pCh = &(pFdo->m_Channel[pShBufCtrl->m_PortIndex]);
	pShBufDesc = &(pCh->m_CdmaShBuf);

	// Sanity check: no CDMA may be running
	if ( pCh->m_CdmaDmaRunning != FALSE )
	{
#if LOG_LEVEL > 0
		DTA1XX_LOG1( KERN_INFO, "[%d] Dta1xxSharedBufFree: CDMA STILL RUNNING?!?",
					 pShBufCtrl->m_PortIndex);
#endif
		return -1;
	}

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Buffer Release -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	// In a separate routine, because clean-up also has to release buffers
	if ( pShBufCtrl->m_Purpose == DTA1XX_SHBUF_DIRIQ_CDMA ) {
		return Dta1xxSharedBufRelease(pCh, pShBufCtrl->m_PortIndex, pShBufDesc,
									  pShBufCtrl->m_Purpose);
	}
	return 0;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxSharedBufRelease -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
int  Dta1xxSharedBufRelease(
	CHANNEL* pCh,
	Int PortIndex,
	ShBufDesc* pShBufDesc,
	Int Purpose)
{
	Int i;

#if LOG_LEVEL > 2
		DTA1XX_LOG1( KERN_INFO, "[%d] Dta1xxSharedBufRelease: releasing shared buffer",
					 PortIndex );
#endif

	// Buffer shouldn't have been released already
	if ( !pShBufDesc->m_Allocated )
	{
#if LOG_LEVEL > 0
		DTA1XX_LOG1( KERN_INFO, "[%d] Dta1xxSharedBufRelease: ERROR, BUFFER HAS BEEN "
								 "RELEASED ALREADY", PortIndex );
#endif
		return -1;
	}

	// Special processing for CDMA
	if ( Purpose == DTA1XX_SHBUF_DIRIQ_CDMA ) {
		// If CDMA is still running => stop CDMA first
		if ( pCh->m_CdmaDmaRunning )
			Dta1xxCdmaStop(pCh);

		// Release Scatter-Gather List, if it was allocated successfully
		Dta1xxCdmaSglCleanUp(pCh);
		pShBufDesc->m_SglAllocated = FALSE;
                pShBufDesc->m_pSglBuf = NULL;
                pShBufDesc->m_SglBufSize = 0;
	}

	// Unlock pages
	for ( i=0; i<pShBufDesc->m_NumPages; i++ )
          page_cache_release(pShBufDesc->m_PageList[i]);

	kfree( pShBufDesc->m_PageList );
	pShBufDesc->m_PageList = NULL;
	pShBufDesc->m_NumPages = 0;
	pShBufDesc->m_Allocated = FALSE;

	return 0;
}
