//#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* Rate.c *#*#*#*#*#*#*#*#*#* (C) 2000-2005 DEKTEC

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- License -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// This program is free software; You can freely use/modify this source code in any way
// you desire, including for commercial applications.
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	TD	2007.01.29	Addes support for DTA-160 ASI-only
//	MG	21-10-2002	Created	

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#include "../Include/Dta1xx.h"
#include "../Include/Dta1xxRegs.h"

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Type definitions -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Internal function declarations -.-.-.-.-.-.-.-.-.-.-.-.-.-.

//-.-.-.-.-.-.-.-.-.-.-.-.-.- Data declaration and definition -.-.-.-.-.-.-.-.-.-.-.-.-.-.

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Implementation +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxGetFreqEst -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Get frequency estimate of external clock-input (DTA-102) or Transport-Stream input
// rate (DTA-120, DTA-122, DTA140, DTA-145, DTA-2145).
//
int Dta1xxGetFreqEst(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int	nChannelType,	// Channel type
	int nChIdx,			// Channel index
	int*  pFreqEst)		// Frequency estimate
{
	uint64_t  Count, ClockFreq;
	Int PerIntFact = 1;

	// Read Valid Counter from the right channel
	if ( nChannelType==DTA1XX_TS_TX_CHANNEL ) {

		// Do not crash on erroneous channel index
		if ( nChIdx<0 || nChIdx>=pFdo->m_nNumTxChannels ) {
			DTA1XX_LOG(KERN_DEBUG, "TX CHANNEL DOES NOT EXIST");
			return -EFAULT;
		}
		Count = pFdo->m_TsTx[nChIdx]->m_BrMeas.m_nValidCount256;
	
	} else if ( nChannelType==DTA1XX_TS_RX_CHANNEL ) {

		// Do not crash on erroneous channel index
		if ( nChIdx<0 || nChIdx>=pFdo->m_nNumRxChannels ) {
			DTA1XX_LOG(KERN_DEBUG, "RX CHANNEL DOES NOT EXIST");
			return -EFAULT;
		}
		Count = pFdo->m_TsRx[nChIdx]->m_BrMeas.m_nValidCount256;
	} else {
		DTA1XX_LOG1(KERN_DEBUG, "ILLEGAL channel type %d\n", nChannelType);
		return -EFAULT;
	}

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Determine Clock Frequency -.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	// Clock frequency depends on board type.
	// For DTA-120, the frequency depends the firmware revision:
	//	- If last bit is '1', it's a "Cypress" DTA-120 with 27-MHz crystal and *1.5 PLL 
	//	- If last bit is '0', it's a "Gennum" DTA-120 with 40-MHz crystal

	if (pFdo->m_nTypeNumber==120 && pFdo->m_nFirmwareVersion<4) {
		if ((pFdo->m_nFirmwareVersion & 1) != 0)
			ClockFreq = 40500000;
		else
			ClockFreq = 40000000;
	}
	else if (  pFdo->m_nTypeNumber==120 
		    || ((pFdo->m_nTypeNumber==124) && (pFdo->m_nFirmwareVersion == 0))
		    || pFdo->m_nTypeNumber==140)
		ClockFreq = 40500000;
	else if ( pFdo->m_nTypeNumber==160 ) {
		ClockFreq = 54000000;
		PerIntFact = 2;	// Per. int 2 times faster then nominal.
	}
	else if (   (pFdo->m_nTypeNumber==105) || (pFdo->m_nTypeNumber==145)
		     || (pFdo->m_nTypeNumber==112) || (pFdo->m_nTypeNumber==115)
             || (pFdo->m_nTypeNumber==545) || (pFdo->m_nTypeNumber==2145)
		     || (pFdo->m_nTypeNumber==124))
		ClockFreq = 54000000;
	else
		ClockFreq = 27000000;

	// Compute frequency estimate
	*pFreqEst = (Int) ((Count * ClockFreq * PerIntFact) >> 26);

	return 0;
}
