//#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* RxIoCtl.c *#*#*#*#*#*#*#*#*# (C) 2000-2005 DEKTEC

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- License -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// This program is free software; You can freely use/modify this source code in any way
// you desire, including for commercial applications.
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	MG	21-10-2002	Created

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#include "../Include/Dta1xx.h"
#include "../Include/Dta1xxRegs.h"
#include <asm/uaccess.h>

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Type definitions -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Internal function declarations -.-.-.-.-.-.-.-.-.-.-.-.-.-.

//-.-.-.-.-.-.-.-.-.-.-.-.-.- Data declaration and definition -.-.-.-.-.-.-.-.-.-.-.-.-.-.

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxIoCtlClearFlags -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Clear the status flags
//
DTA1XX_RESULT Dta1xxRxIoCtlClearFlags(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int nLatched)		// Latched status flags to be cleared
{
	char* pRxRegs;

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumRxChannels ) { return DTA1XX_E_NO_SUCH_INPUT; }

	pRxRegs = (char*)pFdo->m_TsRx[nChIdx]->m_pRegBase;

	// Clear latched flags
	pFdo->m_TsRx[nChIdx]->m_nLatched &= ~nLatched;

	// Also clear flags in Receive Status register, to avoid that flags in
	// m_nLatched are set again in next periodic interrupt.
	if ( 0!=(nLatched & DTA1XX_RX_FIFO_OVF) )
		Dta1xxRxStatusRegClrOvfInt(pRxRegs);
	if ( 0!=(nLatched & DTA1XX_RX_SYNC_ERR) )
		Dta1xxRxStatusRegClrSyncInt(pRxRegs);
	if ( 0!=(nLatched & DTA1XX_RX_RATE_OVF) )
        Dta1xxRxStatusRegClrRateOvfInt(pRxRegs);

	return DTA1XX_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxIoCtlEqualise -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Turn DVB/ASI cable equalisation on or off (DTA-120 only).
//
DTA1XX_RESULT Dta1xxRxIoCtlEqualise(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int nEqualiserSetting)
{
	char* pRxRegs;

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumRxChannels ) { return DTA1XX_E_NO_SUCH_INPUT; }

	pRxRegs = (char*)pFdo->m_TsRx[nChIdx]->m_pRegBase;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Check parameter -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	if (   (DTA1XX_EQUALISER_ON!=nEqualiserSetting)
		&& (DTA1XX_EQUALISER_OFF!=nEqualiserSetting) )
	{
		return DTA1XX_E_INVALID_MODE;
	}

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Enable/Disable equaliser -.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	Dta1xxRxCtrlRegSetEqualise(pRxRegs, nEqualiserSetting);

	return DTA1XX_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxIoCtlGetFifoLoad -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Get the current load of the channel's Output FIFO.
//
DTA1XX_RESULT Dta1xxRxIoCtlGetFifoLoad(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int* pFifoLoad)
{
	char* pRxRegs;

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumRxChannels ) { return DTA1XX_E_NO_SUCH_INPUT; }

	pRxRegs = (char*)pFdo->m_TsRx[nChIdx]->m_pRegBase;

	// Get FIFO load
	*pFifoLoad = Dta1xxRxGetFifoLoad(pRxRegs);

	return DTA1XX_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxIoCtlGetFlags -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Get status flags
//
DTA1XX_RESULT Dta1xxRxIoCtlGetFlags(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int*  pStatus,		// Status flags
	int*  pLatched)		// Latched status flags
{
	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumRxChannels ) { return DTA1XX_E_NO_SUCH_INPUT; }

	// Update staus and latched flags field
	Dta1xxRxProcessStatusFlags(pFdo->m_TsRx[nChIdx]);

	// Copy flags
	*pStatus	= pFdo->m_TsRx[nChIdx]->m_nStatus;
	*pLatched	= pFdo->m_TsRx[nChIdx]->m_nLatched;

	return DTA1XX_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxIoCtlGetStatus -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Get status information from the input channel. If a PCI card does not support a
// certain feature, the corresponding status variable is set to DTA1XX_E_NOT_SUPPORTED.
//
// The status flags are read directly from the hardware (not through device driver).
//
DTA1XX_RESULT Dta1xxRxIoCtlGetStatus(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int* pPacketSize,	// Size of incoming transport packets: DTA1XX_PCKSIZE_XXX
	int* pNumInv,		// Number of invalid bytes per packet: DTA1XX_NUMINV_XXX
	int* pClkDet,		// Receive-clock signal detected: DTA1XX_CLKDET_XXX
	int* pAsiLock,		// DVB/ASI PLL locked: DTA1XX_ASILOCK_XXX
	int* pRateOk,		// Input rate above minimum
	int* pAsiInv)		// DVB/ASI-Input status
{

	char* pRxRegs;

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumRxChannels ) { return DTA1XX_E_NO_SUCH_INPUT; }

	pRxRegs = (char*)pFdo->m_TsRx[nChIdx]->m_pRegBase;

	// Packet size
	*pPacketSize = Dta1xxRxStatusRegGetPckSize(pRxRegs);

	//.-.-.-.-.-.-.-.- Number of invalid bytes per packet (DTA-122 only) -.-.-.-.-.-.-.-.-

	if (pFdo->m_nTypeNumber==122)
		*pNumInv = Dta1xxRxStatusRegGetNumInv(pRxRegs);
	else
		*pNumInv = DTA1XX_NOT_SUPPORTED;

	//-.-.-.-.-.-.-.-.-.-.-.- Receive-clock (=CD) signal detected -.-.-.-.-.-.-.-.-.-.-.-.

	*pClkDet = Dta1xxRxStatusRegGetAsiCD(pRxRegs);

	//.-.-.-.-.-.- DVB/ASI PLL locked (DTA-120/124/140/145/2145/160/545 only) -.-.-.-.-.-.-
	if (   pFdo->m_nTypeNumber==120 || pFdo->m_nTypeNumber==124 || pFdo->m_nTypeNumber==140
		|| pFdo->m_nTypeNumber==145 || pFdo->m_nTypeNumber==2145 || pFdo->m_nTypeNumber==160
		|| pFdo->m_nTypeNumber==545)
		*pAsiLock = Dta1xxRxStatusRegGetAsiLock(pRxRegs);
	else
		*pAsiLock = DTA1XX_NOT_SUPPORTED;

	//-.-.-.-.-.- DVB/ASI iput rate OK (DTA-120/124/140/145/2145/160/545 only) -.-.-.-.-.-
	if (   pFdo->m_nTypeNumber==120 || pFdo->m_nTypeNumber==124 || pFdo->m_nTypeNumber==140
		|| pFdo->m_nTypeNumber==145 || pFdo->m_nTypeNumber==2145 || pFdo->m_nTypeNumber==160
		|| pFdo->m_nTypeNumber==545)
		*pRateOk = Dta1xxRxStatusRegGetRateOk(pRxRegs);
	else
		*pRateOk = DTA1XX_NOT_SUPPORTED;

	//-.-.-.-.- DVB/ASI signal invert status (DTA-120/140/145/2145/160/545 only) -.-.-.-.-
	if (   pFdo->m_nTypeNumber==120 ||  pFdo->m_nTypeNumber==140
		|| pFdo->m_nTypeNumber==145 || pFdo->m_nTypeNumber==2145 || pFdo->m_nTypeNumber==160
		|| pFdo->m_nTypeNumber==545)
		*pAsiInv = Dta1xxRxStatusRegGetAsiInv(pRxRegs);
	else
		*pAsiInv = DTA1XX_NOT_SUPPORTED;


 	return DTA1XX_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxIoCtlGetTargetId -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Returns the ID of the connected target adapter (if present)
//
DTA1XX_RESULT Dta1xxRxIoCtlGetTargetId(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int* pPresent,		// Target adapter present?
	int* pTargetId)		// Target-adapter identifier
{
	char* pRxRegs;

	// For DTA-122 only
	if ( pFdo->m_nTypeNumber!=122 ) { return DTA1XX_E_NOT_SUPPORTED; }

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumRxChannels ) { return DTA1XX_E_NO_SUCH_INPUT; }

	pRxRegs = (char*)pFdo->m_TsRx[nChIdx]->m_pRegBase;

	*pPresent	= pFdo->m_TsRx[nChIdx]->m_TrgIdDet.m_nPresent;
	*pTargetId	= pFdo->m_TsRx[nChIdx]->m_TrgIdDet.m_nTargetId;

	return DTA1XX_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxIoCtlGetTsRateBps -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Gets the transport stream bit-rate.
//
DTA1XX_RESULT Dta1xxRxIoCtlGetTsRateBps(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int* pTsRate)		// Transport-stream rate in bps
{
	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumRxChannels ) { return DTA1XX_E_NO_SUCH_INPUT; }

	return Dta1xxGetFreqEst(pFdo, DTA1XX_TS_RX_CHANNEL, nChIdx, pTsRate);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxIoCtlPolarityControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Control the automatic polarity-detection circuitry of a DVB/ASI Transport-Stream
// input channel (DTA-120/140/145/2145/160/545 only).
//
DTA1XX_RESULT Dta1xxRxIoCtlPolarityControl(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int nPolarity)		// Polarity control setting
{
	char* pRxRegs;

	// For DTA-120/140/145/2145/160/545 only)
	if ( !(pFdo->m_nTypeNumber==120 ||  pFdo->m_nTypeNumber==140
		|| pFdo->m_nTypeNumber==145 || pFdo->m_nTypeNumber==2145 || pFdo->m_nTypeNumber==160
		|| pFdo->m_nTypeNumber==545))
	{
		return DTA1XX_E_NOT_SUPPORTED;
	}

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumRxChannels ) { return DTA1XX_E_NO_SUCH_INPUT; }

	pRxRegs = (char*)pFdo->m_TsRx[nChIdx]->m_pRegBase;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Check parameter -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	if (    (DTA1XX_POLARITY_AUTO!=nPolarity)
		 && (DTA1XX_POLARITY_INVERTED!=nPolarity)
		 && (DTA1XX_POLARITY_NORMAL!=nPolarity) )
	{
		return DTA1XX_E_INVALID_MODE;
	}

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Set ASI-inversion control -.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	Dta1xxRxCtrlRegSetAsiInv(pRxRegs, nPolarity);

	return DTA1XX_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxIoCtlReset -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Reset transport-stream input channel.
//
DTA1XX_RESULT Dta1xxRxIoCtlReset(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int nResetMode)		// How to reset the channel
{
	char* pRxRegs;

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumRxChannels ) { return DTA1XX_E_NO_SUCH_INPUT; }

	pRxRegs = (char*)pFdo->m_TsRx[nChIdx]->m_pRegBase;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Check parameter -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	if ( (DTA1XX_RST_CHANNEL!=nResetMode) && (DTA1XX_RST_CLEARFIFO!=nResetMode) ) {
		return DTA1XX_E_INVALID_MODE;
	}

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Reset channel -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	if (   ( pFdo->m_nTypeNumber==120 && pFdo->m_nFirmwareVersion <= 3)
	    || ( pFdo->m_nTypeNumber==122 && pFdo->m_nFirmwareVersion <= 3))
	{
		// Old card; Reset whole card
		Dta1xxRxGenCtrlRegReset(pRxRegs);
	}else {
		// Use reset channel bit
		Dta1xxRxChannelReset(pRxRegs);
	}
	return DTA1XX_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxIoCtlSetLoopBackMode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Set the channel's loop-back mode.
//
DTA1XX_RESULT Dta1xxRxIoCtlSetLoopBackMode(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int nMode)			// New loop-back mode
{
	char* pRxRegs;

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumRxChannels ) { return DTA1XX_E_NO_SUCH_INPUT; }

	pRxRegs = (char*)pFdo->m_TsRx[nChIdx]->m_pRegBase;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Check parameter -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	if ( (DTA1XX_NO_LOOPBACK!=nMode) && (DTA1XX_LOOPBACK_MODE!=nMode) ) {
		return DTA1XX_E_INVALID_MODE;
	}

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Set loop-back mode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	Dta1xxRxDiagRegSetLoopBack(pRxRegs, nMode);

	return DTA1XX_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxIoCtlSetPower -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Set the channel's power mode.
//
DTA1XX_RESULT Dta1xxRxIoCtlSetPower(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int nPower)			// New power mode
{
	char* pRxRegs;

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumRxChannels ) { return DTA1XX_E_NO_SUCH_INPUT; }

	pRxRegs = (char*)pFdo->m_TsRx[nChIdx]->m_pRegBase;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Check parameter -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	if ( (DTA1XX_POWER_OFF!=nPower) && (DTA1XX_POWER_ON!=nPower) ) {
		return DTA1XX_E_INVALID_MODE;
	}

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Set power mode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	Dta1xxRxCtrlRegSetEnaPwr(pRxRegs, nPower);

	return DTA1XX_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxIoCtlSetRxControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Set the channel's receive-control state.
//
DTA1XX_RESULT Dta1xxRxIoCtlSetRxControl(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int nRxControl)		// Receive control
{
	char* pRxRegs;

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumRxChannels ) { return DTA1XX_E_NO_SUCH_INPUT; }

	pRxRegs = (char*)pFdo->m_TsRx[nChIdx]->m_pRegBase;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Check parameter -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	if ( (DTA1XX_RXCTRL_IDLE!=nRxControl) && (DTA1XX_RXCTRL_RCV!=nRxControl) ) {
		return DTA1XX_E_INVALID_MODE;
	}

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Set receive-control state -.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	Dta1xxRxCtrlRegSetRxCtrl(pRxRegs, nRxControl);

	return DTA1XX_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxIoCtlGetRxControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Get the channel's receive-control state.
//
DTA1XX_RESULT Dta1xxRxIoCtlGetRxControl(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int* pRxControl)	// Receive control
{
	char* pRxRegs;

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumRxChannels ) { return DTA1XX_E_NO_SUCH_INPUT; }

	pRxRegs = (char*)pFdo->m_TsRx[nChIdx]->m_pRegBase;

	// Get Rx-Control
	*pRxControl = Dta1xxRxCtrlRegGetRxCtrl(pRxRegs);

	return DTA1XX_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxIoCtlSetRxMode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Set the channel's receive mode.
//
DTA1XX_RESULT Dta1xxRxIoCtlSetRxMode(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int nRxMode)		// Receive mode
{
	char* pRxRegs;
	int nStMode				= 0;
	int nTimeStamps			= 0;
	int nSupportMp2Mode		= 0;
	int nSupportTimeStamp	= 0;

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumRxChannels ) { return DTA1XX_E_NO_SUCH_INPUT; }

	pRxRegs = (char*)pFdo->m_TsRx[nChIdx]->m_pRegBase;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Check parameter -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	// StoreMp2 is is not supported DTA-120 (FW<= 3) and DTA-122 (FW<= 3)
    nSupportMp2Mode =    !( pFdo->m_nTypeNumber==120 && pFdo->m_nFirmwareVersion <= 3 )
					  && !( pFdo->m_nTypeNumber==122 && pFdo->m_nFirmwareVersion <= 3 );
	// Timestamping is not supported DTA-120 (FW<= 3) and DTA-122 (FW<= 3)
	nSupportTimeStamp =   !( pFdo->m_nTypeNumber==120 && pFdo->m_nFirmwareVersion <= 3 )
					   && !( pFdo->m_nTypeNumber==122 && pFdo->m_nFirmwareVersion <= 3 );

	// Get "store" mode and timestamp flag
	nStMode		= nRxMode & ~DTA1XX_RX_TIMESTAMP;
	nTimeStamps = ((nRxMode & DTA1XX_RX_TIMESTAMP) != 0);

	if (    (DTA1XX_RXMODE_ST188!=nStMode)
		 && (DTA1XX_RXMODE_ST204!=nStMode)
		 && (DTA1XX_RXMODE_STRAW!=nStMode)
		 && (DTA1XX_RXMODE_STMP2!=nStMode))
	{
		// Undefined mode
		return DTA1XX_E_INVALID_MODE;
	}
	if ( (nStMode==DTA1XX_RXMODE_STRAW) && nTimeStamps ) {
		// Illegal combination
		return DTA1XX_E_INVALID_MODE;
	}
	if ( !nSupportMp2Mode && nStMode==DTA1XX_RXMODE_STMP2 ) {
		// Not supported by used hardware
		return DTA1XX_E_NOT_SUPPORTED;
	}
	if ( !nSupportTimeStamp && nTimeStamps ) {
		// Not supported by used hardware
		return DTA1XX_E_NOT_SUPPORTED;
	}

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Set receive mode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	Dta1xxRxCtrlRegSetRxMode(pRxRegs, nStMode);
        // Set timestamping
        Dta1xxRxCtrlRegSetTimeStamping(pRxRegs, nTimeStamps);

	return DTA1XX_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxIoCtlGetStatistics -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Get the curret value of the violation counter
//
DTA1XX_RESULT Dta1xxRxIoCtlGetStatistics(
	DTA1XX_FDO* pFdo,		// Functional device object, representing the DTA-1xx card
	int nChIdx,				// Channel index
	int* pViolCount)		// Violation count
{
	char* pRxRegs;

	// Valid channel ind2ex???
 	if ( nChIdx >= pFdo->m_nNumRxChannels ) { return DTA1XX_E_NO_SUCH_INPUT; }

	pRxRegs = (char*)pFdo->m_TsRx[nChIdx]->m_pRegBase;
	// Get violation count
	*pViolCount = Dta1xxRxGetViolCountReg(pRxRegs);

	return DTA1XX_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxRxIoCtlDma -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Transfer data from the input channel HW fifo to a user buffer, using a DMA transfer
//
DTA1XX_RESULT Dta1xxRxIoCtlDma(
	DTA1XX_FDO* pFdo,		// Functional device object, representing the DTA-1xx card
	int nChIdx,				// Channel index
	char* pUserBuf,			// Pointer to the user buffer
	int nBytesToTransfer)	// #Bytes to transfer
{
	char* pWrPtr;
	int nLeftToTransfer, nDmaSize, nLocalDmaAddr;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Check parameter -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	// Number of bytes must be a multiple of 4
	if (nBytesToTransfer<0 || nBytesToTransfer%4!=0) {
		return DTA1XX_E_INVALID_SIZE;
	}
	// User-buffer address must 32-bit aligned
	if ((unsigned int)(pUserBuf)%4!=0) {
		return DTA1XX_E_INVALID_BUF;
	}

	//.-.-.- Divide the transfer into bursts with a maximum size of the DMA buffer -.-.-.-

    pWrPtr				= pUserBuf;
	nLeftToTransfer		= nBytesToTransfer;
	nLocalDmaAddr		= (int)(  pFdo->m_TsRx[nChIdx]->m_pRegBase
							    + DTA1XX_RX_REG_FIFO_FIRST
								- pFdo->m_Dta1xxRegsBase.m_pKernel);

	while ( nLeftToTransfer > 0 ) {

		// Determine DMA burst size
		nDmaSize = (nLeftToTransfer <= DTA1XX_DMA_BUF_SIZE) ?
												nLeftToTransfer : DTA1XX_DMA_BUF_SIZE;
		nDmaSize &= ~0x3; // Must be 4-byte aligned

		// Start a DMA transfer
		if ( 0 != Dta1xxDmaTransfer(pFdo, nChIdx, DTA1XX_TS_RX_CHANNEL,
									nLocalDmaAddr, nDmaSize, DTA1XX_DMA_READ) )
		{
			// DMA transfer failed ==> return error
			return DTA1XX_E;
		}


		// Copy the received data to the user buffer
		if ( copy_to_user(pWrPtr, pFdo->m_TsRx[nChIdx]->m_Dma.m_DmaBuffer.m_pVirtStartAddr, nDmaSize) ) {
			DTA1XX_LOG( KERN_DEBUG, "Could not copy data to user!" );
			return DTA1XX_E;
		}

		// Move write pointer and update number of bytes left to transfer
		pWrPtr			+= nDmaSize;
		nLeftToTransfer	-= nDmaSize;
	}

	return DTA1XX_OK;
}
