//#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* TxIoCtl.c *#*#*#*#*#*#*#*#*# (C) 2000-2005 DEKTEC

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- License -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// This program is free software; You can freely use/modify this source code in any way
// you desire, including for commercial applications.
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	MG	2004.28.10	Fixed bug in TsRate computation algorithm
//	MG	2003.04.16	Made sure output of DTA-102 is enabled when TxCtrl!=DTA1XX_TXCTRL_IDLE
//	MG	21-10-2002	Created

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#include "../Include/Dta1xx.h"
#include "../Include/Dta1xxRegs.h"
#include <asm/uaccess.h>

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Type definitions -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Internal function declarations -.-.-.-.-.-.-.-.-.-.-.-.-.-.

//-.-.-.-.-.-.-.-.-.-.-.-.-.- Data declaration and definition -.-.-.-.-.-.-.-.-.-.-.-.-.-.

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxIoCtlClearFlags -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Clear the status flags
//
DTA1XX_RESULT Dta1xxTxIoCtlClearFlags(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int nLatched)		// Latched status flags to be cleared
{
	char* pTxRegs;

	// Valid channel index???
	if ( nChIdx >= pFdo->m_nNumTxChannels ) { return DTA1XX_E_NO_SUCH_OUTPUT; }

	pTxRegs = (char*)pFdo->m_TsTx[nChIdx]->m_pRegBase;

	// Clear latched flags
	pFdo->m_TsTx[nChIdx]->m_nLatched &= ~nLatched;

	// Also clear flags in Transmit Status register, to avoid that flags in
	// m_nLatched are set again in next periodic interrupt.
	if ( 0!=(nLatched & DTA1XX_TX_FIFO_UFL) ) {
		Dta1xxTxStatusRegClrUflInt(pTxRegs);
	}
	if ( 0!=(nLatched & DTA1XX_TX_SYNC_ERR) ) {
		Dta1xxTxStatusRegClrSyncInt(pTxRegs);
	}
	if ( 0!=(nLatched & DTA1XX_TX_READBACK_ERR) ) {
        Dta1xxTxStatusRegClrShortInt(pTxRegs);
	}

	return DTA1XX_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxIoCtlGetExtClkFreq -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Returns the Transport-stream rate of external clock in bps
//
DTA1XX_RESULT Dta1xxTxIoCtlGetExtClkFreq(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int* pExtClkFreq)	// Transport-stream rate of external clock in bps
{
	int nFreqEst=0;
	char* pTxRegs;

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumTxChannels ) { return DTA1XX_E_NO_SUCH_OUTPUT; }

	pTxRegs = (char*)pFdo->m_TsTx[nChIdx]->m_pRegBase;

	// Get the frequency estimation
	if ( 0==Dta1xxGetFreqEst(pFdo, DTA1XX_TS_TX_CHANNEL, nChIdx, &nFreqEst) ) {
		// Estimation is in bps ==> divide by 8 for Hz
		*pExtClkFreq	= (nFreqEst/8);
		return DTA1XX_OK;
	} else {
		*pExtClkFreq	= 0;
		return DTA1XX_E_DEV_DRIVER;
	}
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxIoCtlGetFifoLoad -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Get the current load of the channel's Output FIFO.
//
DTA1XX_RESULT Dta1xxTxIoCtlGetFifoLoad(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int* pFifoLoad)		// Current fifo load
{
	char* pTxRegs;

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumTxChannels ) { return DTA1XX_E_NO_SUCH_OUTPUT; }

	pTxRegs = (char*)pFdo->m_TsTx[nChIdx]->m_pRegBase;

	// Get the fifo load
	*pFifoLoad = Dta1xxTxGetFifoLoadReg(pTxRegs);

	return DTA1XX_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxIoCtlGetFlags -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Returns the current value of the status and latched flags
//
DTA1XX_RESULT Dta1xxTxIoCtlGetFlags(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int* pStatus,		// Status flags
	int* pLatched)		// Latched status flags
{
	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumTxChannels ) { return DTA1XX_E_NO_SUCH_OUTPUT; }

	// Update staus and latched flags field
	Dta1xxTxProcessStatusFlags(pFdo->m_TsTx[nChIdx]);

	// Copy flags
	*pStatus	= pFdo->m_TsTx[nChIdx]->m_nStatus;
	*pLatched	= pFdo->m_TsTx[nChIdx]->m_nLatched;

	return DTA1XX_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxIoCtlGetMaxFifoSize -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Returns the maximum possible fifo size
//
DTA1XX_RESULT Dta1xxTxIoCtlGetMaxFifoSize(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int* pMaxFifoSize)	// MAX fifo size
{
	// Get maximum FIFO size
	*pMaxFifoSize = 8*1024*1024;
	return DTA1XX_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxIoCtlGetTargetId -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Returns the ID of the connected target adapter (if present)
//
DTA1XX_RESULT Dta1xxTxIoCtlGetTargetId(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int* pPresent,		// Target adapter present?
	int* pTargetId)		// Target-adapter identifier
{
	char* pTxRegs;

	// DTA-102 only
	if ( pFdo->m_nTypeNumber!=102 ) { return DTA1XX_E_NOT_SUPPORTED; }

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumTxChannels ) { return DTA1XX_E_NO_SUCH_OUTPUT; }

	pTxRegs = (char*)pFdo->m_TsTx[nChIdx]->m_pRegBase;

	*pPresent	= pFdo->m_TsTx[nChIdx]->m_TrgIdDet.m_nPresent;
	*pTargetId	= pFdo->m_TsTx[nChIdx]->m_TrgIdDet.m_nTargetId;

	return DTA1XX_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxIoCtlGetTxClock -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Get the transmit clock
//
DTA1XX_RESULT Dta1xxTxIoCtlGetTxClock(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int* pClockGenMode,	// Internal or external clock
	int* pPhaseIncr,	// Transmit clock pahse increment
	int* pSymbRateMult)	// Symbol rate multiplier
{
	char* pTxRegs;

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumTxChannels ) { return DTA1XX_E_NO_SUCH_OUTPUT; }

	pTxRegs = (char*)pFdo->m_TsTx[nChIdx]->m_pRegBase;

	// Get phase increment
	*pPhaseIncr = Dta1xxTxGetTxClockReg(pTxRegs);

	// Modulator only: get symbol rate multiplier
	if ( pFdo->m_nTypeNumber == 107 )
		*pSymbRateMult = Dta1xxTxModcRegGetM(pTxRegs);
	else
		*pSymbRateMult = 0;

	// DTA-102 only: get clock-generator mode
	if (pFdo->m_nTypeNumber==102 ) {
		*pClockGenMode = Dta1xxTxCtrlRegGetUseExtClk(pTxRegs);
	} else {
		*pClockGenMode = 0;
	}
	return DTA1XX_OK;
}


//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxIoCtlGetTxControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Get the channel's transmit-control state.
//
DTA1XX_RESULT Dta1xxTxIoCtlGetTxControl(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int* pTxControl)	// Transmit-control state
{
	char* pTxRegs;

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumTxChannels ) { return DTA1XX_E_NO_SUCH_OUTPUT; }

	pTxRegs = (char*)pFdo->m_TsTx[nChIdx]->m_pRegBase;

	// Get TxCtrl field value
	*pTxControl = Dta1xxTxCtrlRegGetTxCtrl(pTxRegs);

	return DTA1XX_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxIoCtlGetTxMode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Get the channel's transmit mode.
//
DTA1XX_RESULT Dta1xxTxIoCtlGetTxMode(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int* pTxPacketMode,	// Packet transmit mode
	int* pTxStuffMode)  // Null-packet stuffing on/off
{
	char* pTxRegs;

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumTxChannels ) { return DTA1XX_E_NO_SUCH_OUTPUT; }

	pTxRegs = (char*)pFdo->m_TsTx[nChIdx]->m_pRegBase;

	// Get packet mode and stuff mode
	*pTxPacketMode	= Dta1xxTxCtrlRegGetTxMode(pTxRegs);
	*pTxStuffMode	= Dta1xxTxCtrlRegGetPckStuff(pTxRegs);

	// Get burst mode for DTA-100, DTA-105, DTA-140, DTA-145, DTA-2145, DTA-160, DTA-545
	if (   pFdo->m_nTypeNumber==100 || pFdo->m_nTypeNumber==105 || pFdo->m_nTypeNumber==140
		|| pFdo->m_nTypeNumber==145 || pFdo->m_nTypeNumber==2145 || pFdo->m_nTypeNumber==160
		|| pFdo->m_nTypeNumber==545)
	{
		*pTxPacketMode = Dta1xxTxCtrlRegGetBurstMode(pTxRegs);
	} else {
		*pTxPacketMode = 0;
	}

 	return DTA1XX_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxIoCtlReadLoopBackData -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Reads a specified number of bytes from the HW fifo via the loopback registers
//
DTA1XX_RESULT Dta1xxTxIoCtlReadLoopBackData(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	char* pBuffer,		// Pointer to data read back from Output FIFO
	int nBytesToRead)	// Number of bytes to be read
{
	char* pWrPtr;
	int nLeftToRead, data;
	char* pTxRegs;

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumTxChannels ) { return DTA1XX_E_NO_SUCH_OUTPUT; }

	pTxRegs = (char*)pFdo->m_TsTx[nChIdx]->m_pRegBase;

	//-.-.-.-.-.-.-.-.-.-.- Read the data from the loopback register -.-.-.-.-.-.-.-.-.-.-

	pWrPtr			= pBuffer;
	nLeftToRead		= nBytesToRead;

	while ( nLeftToRead > 0 ) {
		data = Dta1xxTxGetLoopBackDataReg(pTxRegs);
		if ( copy_to_user(pWrPtr, (char*)(&data), 1) ) {
			DTA1XX_LOG( KERN_DEBUG, "Could not copy data to user!" );
			return DTA1XX_E;
		}
        pWrPtr++;
		nLeftToRead--;
	}
	return DTA1XX_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxIoCtlReset -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Reset transport-stream output channel.
//
DTA1XX_RESULT Dta1xxTxIoCtlReset(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int nResetMode)		// How to reset the channel
{
	char* pTxRegs;

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumTxChannels ) { return DTA1XX_E_NO_SUCH_OUTPUT; }

	pTxRegs = (char*)pFdo->m_TsTx[nChIdx]->m_pRegBase;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Check parameter -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	if ( (DTA1XX_RST_CHANNEL!=nResetMode) && (DTA1XX_RST_CLEARFIFO!=nResetMode) ) {
		return DTA1XX_E_INVALID_MODE;
	}

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Reset channel -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

    // ResetMode determines which reset bit is used
	if (nResetMode==DTA1XX_RST_CLEARFIFO) {

		// For old revisions: Use mode DTA1XX_RST_CHANNEL with a recursive call
		if (   (pFdo->m_nTypeNumber==100 && pFdo->m_nFirmwareVersion<=2)
			|| (pFdo->m_nTypeNumber==102 && pFdo->m_nFirmwareVersion<=4)) {
			Dta1xxTxIoCtlReset(pFdo, nChIdx, DTA1XX_RST_CHANNEL);
		} else {
			// Card is sufficiently up-to-date to just reset the FIFO
			Dta1xxTxClrFifo(pTxRegs);
		}

	} else if (nResetMode==DTA1XX_RST_CHANNEL) {

		// Full reset on DTA-100/102, Transmit-Channel reset on new boards (DTA-140...)
		if (pFdo->m_nTypeNumber==100 || pFdo->m_nTypeNumber==102) {
			Dta1xxTxGenCtrlRegReset(pTxRegs);
		} else {
			Dta1xxTxChannelReset(pTxRegs);
		}
	}
	return DTA1XX_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxIoCtlSetFifoSize -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Sets the channels fifo size
//
DTA1XX_RESULT Dta1xxTxIoCtlSetFifoSize(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int nFifoSize)		// Size of FIFO in bytes
{
	char* pTxRegs;

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumTxChannels ) { return DTA1XX_E_NO_SUCH_OUTPUT; }

	pTxRegs = (char*)pFdo->m_TsTx[nChIdx]->m_pRegBase;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Check parameters -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	if (nFifoSize<16 || (nFifoSize%16)!=0 || nFifoSize>8*1024*1024) {
		return DTA1XX_E_INVALID_SIZE;
	}

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Set FIFO size -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	// Size of precisely 8Mbytes is a special case
	if (nFifoSize==8*1024*1024) { nFifoSize -= 16; }

	Dta1xxTxSetFifoSizeReg(pTxRegs, nFifoSize);

  	return DTA1XX_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxIoCtlSetFifoSizeMax -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Set FIFO size to the maximum valid value.
//
DTA1XX_RESULT Dta1xxTxIoCtlSetFifoSizeMax(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx)			// Channel ID
{
	char* pTxRegs;

	// Valid channel index???

 	if ( nChIdx >= pFdo->m_nNumTxChannels ) { return DTA1XX_E_NO_SUCH_OUTPUT; }

	pTxRegs = (char*)pFdo->m_TsTx[nChIdx]->m_pRegBase;


	// Set FIFO size
	Dta1xxTxSetFifoSizeReg(pTxRegs, (8*1024*1024 - 16));

  	return DTA1XX_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxIoCtlSetLoopBackMode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Set the channel's loop-back mode.
//
DTA1XX_RESULT Dta1xxTxIoCtlSetLoopBackMode(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int nMode)			// New loop-back mode
{
	char* pTxRegs;

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumTxChannels ) { return DTA1XX_E_NO_SUCH_OUTPUT; }

	pTxRegs = (char*)pFdo->m_TsTx[nChIdx]->m_pRegBase;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Check parameters -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	if ( (nMode!=DTA1XX_NO_LOOPBACK) && (nMode!=DTA1XX_LOOPBACK_MODE) ) {
		return DTA1XX_E_INVALID_MODE;
	}

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Set loop-back mode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	Dta1xxTxDiagRegSetLoopBack(pTxRegs, nMode);

	return DTA1XX_OK;
}


//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxIoCtlSetPower -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Set the channel's power mode.
//
DTA1XX_RESULT Dta1xxTxIoCtlSetPower(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int nPower)			// New power mode
{
	char* pTxRegs;

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumTxChannels ) { return DTA1XX_E_NO_SUCH_OUTPUT; }

	pTxRegs = (char*)pFdo->m_TsTx[nChIdx]->m_pRegBase;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Check parameters -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	if ( (nPower!=DTA1XX_POWER_OFF) && (nPower!=DTA1XX_POWER_ON) ) {
		return DTA1XX_E_INVALID_MODE;
	}

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Set power -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	Dta1xxTxCtrlRegSetEnaPwr(pTxRegs, nPower);

	return DTA1XX_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxIoCtlSetTxClock -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Set the transmit clock
//
DTA1XX_RESULT Dta1xxTxIoCtlSetTxClock(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int nClockGenMode,	// Internal or external clock
	int nPhaseIncr,		// Transmit clock pahse increment
	int nSymbRateMult)	// Symbol rate multiplier
{
	char* pTxRegs;

	if (nClockGenMode!=DTA1XX_TXCLOCK_EXTERNAL && nClockGenMode!=DTA1XX_TXCLOCK_INTERNAL) {
		return DTA1XX_E_INVALID_CGMODE;
	}

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumTxChannels ) { return DTA1XX_E_NO_SUCH_OUTPUT; }

	pTxRegs = (char*)pFdo->m_TsTx[nChIdx]->m_pRegBase;

	// Set phase increment
	Dta1xxTxSetTxClockReg(pTxRegs, nPhaseIncr);

	// Modulator only: Set symbol rate multiplier
	if ( pFdo->m_nTypeNumber == 107 )
		Dta1xxTxModcRegSetM(pTxRegs, nSymbRateMult);


	// DTA-102 only: set clock-generator mode
	if ( pFdo->m_nTypeNumber==102 ) {
		Dta1xxTxCtrlRegSetUseExtClk(pTxRegs, nClockGenMode);
	}
	return DTA1XX_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxIoCtlSetTxControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Set the channel's transmit-control state.
//
DTA1XX_RESULT Dta1xxTxIoCtlSetTxControl(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int nTxControl)		// Transmit-control state
{
	char* pTxRegs;

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumTxChannels ) { return DTA1XX_E_NO_SUCH_OUTPUT; }

	pTxRegs = (char*)pFdo->m_TsTx[nChIdx]->m_pRegBase;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Check parameters -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	if (    (nTxControl!=DTA1XX_TXCTRL_IDLE) && (DTA1XX_TXCTRL_HOLD!=nTxControl)
		 && (DTA1XX_TXCTRL_SEND!=nTxControl) )
	{
		return DTA1XX_E_INVALID_MODE;
	}


	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Set Tx control -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	Dta1xxTxCtrlRegSetTxCtrl(pTxRegs, nTxControl);

	// Enable output for DTA-102
	if ( (nTxControl!=DTA1XX_TXCTRL_IDLE) && (pFdo->m_nTypeNumber==102) ) {
		Dta1xxTxCtrlRegSetTxDis(pTxRegs, 0);
	}

  	return DTA1XX_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxIoCtlSetTxMode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Set the channel's transmit mode.
//
DTA1XX_RESULT Dta1xxTxIoCtlSetTxMode(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int nTxPacketMode,	// Packet transmit mode
	int nTxStuffMode)	// Null-packet stuffing on/off
{
	int  nTxMode;			// Transmit mode (188/192/204/Add16/Raw)
	char* pTxRegs;

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumTxChannels ) { return DTA1XX_E_NO_SUCH_OUTPUT; }

	// Get pointer to transmit registers
	pTxRegs = (char*)pFdo->m_TsTx[nChIdx]->m_pRegBase;

	// Get transmit mode from nTxPacketMode (AND-off burst-mode bit)
	nTxMode = nTxPacketMode & DTA1XX_TXMODE_MASK;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Check parameter -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	if (    (DTA1XX_TXMODE_188!=nTxMode) && (DTA1XX_TXMODE_192!=nTxMode)
		 && (DTA1XX_TXMODE_204!=nTxMode) && (DTA1XX_TXMODE_RAW!=nTxMode)
		 && (DTA1XX_TXMODE_130!=nTxMode) && (DTA1XX_TXMODE_ADD16!=nTxMode)
		 && (DTA1XX_TXMODE_MIN16!=nTxMode) )
	{
		return DTA1XX_E_INVALID_MODE;
	}
	if ( (DTA1XX_STUFFMODE_OFF!=nTxStuffMode) && (DTA1XX_STUFFMODE_ON!=nTxStuffMode) )
	{
		return DTA1XX_E_INVALID_MODE;
	}
	// Check for campatibilty with DTA1xx type and firmware version
	if ( nTxMode==DTA1XX_TXMODE_MIN16 ) {
		// Correct firmware for MIN16 mode??
		if (   (pFdo->m_nTypeNumber==100 && pFdo->m_nFirmwareVersion<5 )
			|| (pFdo->m_nTypeNumber==102 && pFdo->m_nFirmwareVersion<8 ) )
		{
			return DTA1XX_E_INVALID_MODE;
		}
	}
	
	if (pFdo->m_nTypeNumber!=102) {
		// Unsupported modes??
		if ( (nTxMode==DTA1XX_TXMODE_130) || (nTxMode==DTA1XX_TXMODE_192) ) {
			return DTA1XX_E_INVALID_MODE;
		}
	} else  {
		// Burst mode is not supported by DTA-102
		if ( 0!=(nTxMode & DTA1XX_TXMODE_BURST)) {
			return DTA1XX_E_INVALID_MODE;
		}
		// Correct firmware??
		if (    ( pFdo->m_nFirmwareVersion<2 && nTxMode==DTA1XX_TXMODE_192 )
			 || ( pFdo->m_nFirmwareVersion<7 && nTxMode==DTA1XX_TXMODE_130 ) )
		{
			return DTA1XX_E_INVALID_MODE;
		}
	}

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Set tx mode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	// Change packet transmit mode
	Dta1xxTxCtrlRegSetTxMode(pTxRegs,  (nTxMode&DTA1XX_TXMODE_MASK));
	// Set packet stuff mode
	Dta1xxTxCtrlRegSetPckStuff(pTxRegs, nTxStuffMode);

	// Set burst mode for DTA-100, DTA-105, DTA-140, DTA-145, DTA-2145, DTA-160, DTA-545
	if (   pFdo->m_nTypeNumber==100 || pFdo->m_nTypeNumber==105 || pFdo->m_nTypeNumber==140
		|| pFdo->m_nTypeNumber==145 || pFdo->m_nTypeNumber==2145 || pFdo->m_nTypeNumber==160
		|| pFdo->m_nTypeNumber==545)
	{
		if ( 0!=(nTxPacketMode & DTA1XX_TXMODE_BURST)) {
			Dta1xxTxCtrlRegSetBurstMode(pTxRegs, DTA1XX_BURSTMODE_ON);
		} else {
			Dta1xxTxCtrlRegSetBurstMode(pTxRegs, DTA1XX_BURSTMODE_OFF);
		}
	}
	return DTA1XX_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxIoCtlSetTxPolarity -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Set polarity of DVB/ASI output signal.
//
DTA1XX_RESULT Dta1xxTxIoCtlSetTxPolarity(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int nPolarity)		// DTA1XX_POLARITY_NORMAL / DTA1XX_POLARITY_INVERTED
{
	char* pTxRegs;

	// Set polarity for  DTA-105, DTA-140, DTA-145, DTA-2145, DTA-160, DTA-545
	if (! (   pFdo->m_nTypeNumber==105 || pFdo->m_nTypeNumber==140
	 	   || pFdo->m_nTypeNumber==145 || pFdo->m_nTypeNumber==2145 
		   || pFdo->m_nTypeNumber==160 || pFdo->m_nTypeNumber==545))
	{
		 return DTA1XX_NOT_SUPPORTED; 
	}

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumTxChannels ) { return DTA1XX_E_NO_SUCH_OUTPUT; }

	pTxRegs = (char*)pFdo->m_TsTx[nChIdx]->m_pRegBase;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Check parameter -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	if ( (DTA1XX_TXPOL_NORMAL!=nPolarity) && (DTA1XX_TXPOL_INVERTED!=nPolarity) )	{
		return DTA1XX_E_INVALID_MODE;
	}

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Set ASI-inversion control -.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	Dta1xxTxCtrlSetTxAsiInv(pTxRegs, nPolarity);

	return DTA1XX_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxIoCtlGetModControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Get modulation-control parameters
//
DTA1XX_RESULT Dta1xxTxIoCtlGetModControl(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int* pModType,		// Modulation type
	int* pCodeRate,		// Convolutional rate/roll-off for DTA-110
	int* pPar1,			// Extra parameters
	int* pPar2)			// Extra parameters
{
	char* pTxRegs;

	// Check type number: must be a modulator card
	if ( pFdo->m_nTypeNumber!=107 && pFdo->m_nTypeNumber!=110 )
        return DTA1XX_NOT_SUPPORTED;

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumTxChannels ) { return DTA1XX_E_NO_SUCH_OUTPUT; }

	pTxRegs = (char*)pFdo->m_TsTx[nChIdx]->m_pRegBase;

	// Retrieve modulation type and code rate
	*pModType = Dta1xxTxModcRegGetModType(pTxRegs);
    if ( pFdo->m_nTypeNumber==107 )
  	  *pCodeRate = Dta1xxTxModcRegGetCodeRate(pTxRegs);
    else // DTA-110
      *pCodeRate = Dta1xxTxModcRegGetRollOffFactor(pTxRegs);

	// Special case: modtype == IQDIRECT and we ahve a DTA-110 => set mod-type to OFDM
	if ( pFdo->m_nTypeNumber == 110 && *pModType==DTA1XX_MOD_IQDIRECT )
		*pModType = DTA1XX_MOD_OFDM;

	// EXTRA PARAMS NOT USED YET
	*pPar1 = *pPar2 = 0;

	return DTA1XX_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxIoCtlSetModControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Set modulation-control parameters
//
DTA1XX_RESULT Dta1xxTxIoCtlSetModControl(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int nModType,		// Modulation type
	int nCodeRate,		// Convolutional rate/ Roll-off for DTA-110
	int nPar1,			// Extra parameters
	int nPar2)			// Extra parameters
{
	char* pTxRegs;

	// Check type number: must be a modulator card
	if ( pFdo->m_nTypeNumber!=107 && pFdo->m_nTypeNumber!=110 )
    {
        return DTA1XX_NOT_SUPPORTED;
    }

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumTxChannels ) { return DTA1XX_E_NO_SUCH_OUTPUT; }

	// Check modulation type is supported
	if ( pFdo->m_nTypeNumber==107 )
	{
		// DTA-107 is DVB-S modulator
		if ( nModType!=DTA1XX_MOD_DVBS_QPSK && nModType!=DTA1XX_MOD_BPSK )
			return DTA1XX_E_INVALID_MODE;

		// check code rate
		if (    nCodeRate!=DTA1XX_MOD_1_2 && nCodeRate!=DTA1XX_MOD_2_3
             && nCodeRate!=DTA1XX_MOD_3_4 && nCodeRate!=DTA1XX_MOD_4_5
             && nCodeRate!=DTA1XX_MOD_5_6 && nCodeRate!=DTA1XX_MOD_6_7
             && nCodeRate!=DTA1XX_MOD_7_8 )
		{
			 return DTA1XX_E_INVALID_RATE;
		}
	}
	else if ( pFdo->m_nTypeNumber==110 )
	{

		// DTA-110(T) is DVB-C/T modulator
        if (   nModType!=DTA1XX_MOD_QAM4   && nModType!=DTA1XX_MOD_QAM16
			&& nModType!=DTA1XX_MOD_QAM32  && nModType!=DTA1XX_MOD_QAM64
            && nModType!=DTA1XX_MOD_QAM128 && nModType!=DTA1XX_MOD_QAM256
			&& nModType!=DTA1XX_MOD_OFDM )
		{
            return DTA1XX_E_INVALID_MODE;
        }

		// Check roll-off (only for QAM modulation types)
		if (   nModType!=DTA1XX_MOD_OFDM
			&& nCodeRate!=DTA1XX_MOD_ROLLOFF_13 && nCodeRate!=DTA1XX_MOD_ROLLOFF_15 )
		{
			return DTA1XX_E_INVALID_ROLLOFF;
		}
	}

	pTxRegs = (char*)pFdo->m_TsTx[nChIdx]->m_pRegBase;

	// Set modulation type and code rate (NOTE: for OFDM we set IQDIRECT)
	if ( nModType != DTA1XX_MOD_OFDM )
        Dta1xxTxModcRegSetModType(pTxRegs, nModType);
	else
		Dta1xxTxModcRegSetModType(pTxRegs, DTA1XX_MOD_IQDIRECT);

    if ( pFdo->m_nTypeNumber==107 )
        Dta1xxTxModcRegSetCodeRate(pTxRegs, nCodeRate);
    else // DTA-110
	{
		// Only set roll-off for QAM modulations
		if ( nModType != DTA1XX_MOD_OFDM )
			Dta1xxTxModcRegSetRollOffFactor(pTxRegs, nCodeRate);
	}
	return DTA1XX_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxIoCtlGetRfControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Get RF-control settings (e.g. RF frequency)
//
DTA1XX_RESULT Dta1xxTxIoCtlGetRfControl(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int* pPllA,			// PLL A setting
	int* pPllB,			// PLL B setting
	int* pLockStatus)	// Lock status of RF PLL
{
	char* pTxRegs;

	// Check type number: must be a modulator card
	if ( pFdo->m_nTypeNumber!=107 && pFdo->m_nTypeNumber!=110 ) {
           return DTA1XX_NOT_SUPPORTED;
        }

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumTxChannels ) { return DTA1XX_E_NO_SUCH_OUTPUT; }

	pTxRegs = (char*)pFdo->m_TsTx[nChIdx]->m_pRegBase;

	// Get A and B from PLL
	*pPllA = Dta1xxTxRfCtrlRegGetPllA(pTxRegs);
	*pPllB = Dta1xxTxRfCtrlRegGetPllB(pTxRegs);

	// Copy lock status
	*pLockStatus = Dta1xxTxRfCtrlRegGetPllLock(pTxRegs);

	return DTA1XX_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxIoCtlSetRfControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Set RF-control settings (e.g. RF frequency)
//
DTA1XX_RESULT Dta1xxTxIoCtlSetRfControl(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	int nPllA,			// PLL A setting
	int nPllB)			// PLL B setting
{
	char* pTxRegs;

	// Check type number: must be a modulator card
	if ( pFdo->m_nTypeNumber!=107 && pFdo->m_nTypeNumber!=110 ) {
             return DTA1XX_NOT_SUPPORTED;
        }

	// Valid channel index???
 	if ( nChIdx >= pFdo->m_nNumTxChannels ) { return DTA1XX_E_NO_SUCH_OUTPUT; }

	pTxRegs = (char*)pFdo->m_TsTx[nChIdx]->m_pRegBase;

	// Write A and B to PLL
	Dta1xxTxRfCtrlRegSetPllA(pTxRegs, nPllA);
	Dta1xxTxRfCtrlRegSetPllB(pTxRegs, nPllB);

	return DTA1XX_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dta1xxTxIoCtlDma -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Transfer a user buffer to the output channel using a DMA transfer
//
DTA1XX_RESULT Dta1xxTxIoCtlDma(
	DTA1XX_FDO* pFdo,	// Functional device object, representing the DTA-1xx card
	int nChIdx,			// Channel index
	char* pUserBuf,		// Pointer to the user buffer
	int nBytesToTransfer)	// Bytes to transfer
{
	char* pRdPtr;
	int nLeftToTransfer, nDmaSize, nLocalDmaAddr;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Check parameters -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	// Number of bytes must be a multiple of 4
	if (nBytesToTransfer<0 || nBytesToTransfer%4!=0) {
		return DTA1XX_E_INVALID_SIZE;
	}
	// User-buffer address must 32-bit aligned
	if ((unsigned int)(pUserBuf)%4!=0) {
		return DTA1XX_E_INVALID_BUF;
	}

	//.-.-.- Divide the transfer into bursts with a maximum size of the DMA buffer -.-.-.-

    pRdPtr				= pUserBuf;
	nLeftToTransfer		= nBytesToTransfer;

	nLocalDmaAddr		= (int)(  pFdo->m_TsTx[nChIdx]->m_pRegBase
								+ DTA1XX_TX_REG_FIFO_FIRST
								- pFdo->m_Dta1xxRegsBase.m_pKernel);

	while ( nLeftToTransfer > 0 ) {

		// Determine DMA burst size
		nDmaSize = (nLeftToTransfer <= DTA1XX_DMA_BUF_SIZE) ?
												nLeftToTransfer : DTA1XX_DMA_BUF_SIZE;
		nDmaSize &= ~0x3; // Must be 4-byte aligned

  		// Copy the data from the user buffer to the DMA buffer
		if ( copy_from_user(pFdo->m_TsTx[nChIdx]->m_Dma.m_DmaBuffer.m_pVirtStartAddr, pRdPtr,
					   nDmaSize) ) {
			DTA1XX_LOG( KERN_DEBUG, "Could not copy data from user!" );
			return DTA1XX_E;
		}

		// Start a DMA transfer
		if ( 0 != Dta1xxDmaTransfer(
						pFdo, nChIdx, DTA1XX_TS_TX_CHANNEL,
						nLocalDmaAddr, nDmaSize, DTA1XX_DMA_WRITE) )
		{
			// DMA transfer failed ==> return error
			return DTA1XX_E;
		}

		// Move read pointer and update number of bytes left to transfer
		pRdPtr			+= nDmaSize;
		nLeftToTransfer	-= nDmaSize;
	}
	return DTA1XX_OK;
}
