//*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* Dtu2xx.h *#*#*#*#*#*#*#*#*# (C) 2000-2006 DekTec
//
// This code is the Intellectual property of DekTec Digital Video B.V. and may not be
// distributed, in its original or in altered form, without the explicit permission of
// DekTec Digital Video B.V.

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//  2006.10.01	MG	Clean-up of code
//  2006.08.14	KvR	Changed to new interface, added support for DTU-245, several bugfixes
//	2003.04.23	MG	Added VPD support
//	2003.03.11	MG	Added DMA done event to DMA_CHANNEL
//	2002.10.21	MG	Created

#ifndef __DTU2XX_H
#define __DTU2XX_H

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#ifdef __KERNEL__

#include <linux/slab.h>		// kmalloc ()
#include <linux/fs.h>		// inode, file
#include <linux/errno.h>	// error codes
#include <linux/wait.h>		// wait queue
#include <linux/sched.h>	// wait queue
#include <linux/ioctl.h>
#include <asm/semaphore.h>	// semaphore
#include <linux/usb.h>
#include <linux/kref.h>

#endif // #ifdef __KERNEL__

#include <asm/types.h>
#include <asm/atomic.h>

#include "DriverTypes.h"
#include "Dtu2xxCodes.h"
#include "Dtu2xxCommon.h"
#include "Dtu2xxRegs.h"
#include "DriverVersion.h"

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- LOG LEVEL -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

// LOG LEVEL with KdPrint
//  0	Off
//	1	Minimal (setting-up / breaking-down of channels)
//	2	Average (up to 10 per second)
//	3	All

// USB Bulk-transfer related
#define LOG_LEVEL_DMA             0

// Other logs (0=off not supported)
#define LOG_LEVEL                 0


//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Type definitions and constants -.-.-.-.-.-.-.-.-.-.-.-.-.-.

// Module ID information
#define DTU2XX_MAJOR 		0				// Default major number
#define DRIVER_NAME			"Dtu2xx"		// Name of the driver
#define DTU2XX_MINOR        176             // Driver minor number. TODO: register usb
                                            // minor
// DTU-2XX product IDs
#define DTU2XX_VENDORID     		0x1297	// DekTec vendor ID
#define PID_DTU205_NOFIRMWARE		0x2050	// Uninitilaised DTU-205 PID
#define PID_DTU205					0x0205	// Initilaised DTU-205 PID
#define PID_DTU225_NOFIRMWARE		0x2190	// Uninitilaised DTU-225 PID
#define PID_DTU225					0x0219	// Initilaised DTU-225 PID
#define PID_DTU234_NOFIRMWARE		0x2220	// Uninitilaised DTU-234 PID
#define PID_DTU234					0x0222	// Initilaised DTU-234 PID
#define PID_DTU235_NOFIRMWARE		0x2230	// Uninitilaised DTU-235 PID
#define PID_DTU235					0x0223	// Initilaised DTU-235 PID
#define PID_DTU245					0x022D	// DTU-245 PID

// Driver log message helpers
#define DTU2XX_LOG(l,m)					printk(l DRIVER_NAME ": " m "\n");
#define DTU2XX_LOG1(l,m,p1)				printk(l DRIVER_NAME ": " m "\n", p1);
#define DTU2XX_LOG2(l,m,p1,p2)			printk(l DRIVER_NAME ": " m "\n", p1, p2);
#define DTU2XX_LOG3(l,m,p1,p2,p3)		printk(l DRIVER_NAME ": " m "\n", p1, p2, p3);
#define DTU2XX_LOG4(l,m,p1,p2,p3,p4)	printk(l DRIVER_NAME ": " m "\n", p1, p2, p3, p4);

// VPD helpers
#define DTU2XX_VPD_BUF_SIZE				512 // Size of VPD buffer

// Align bulk transfer to:
#define DTU2XX_BULK_TRANSFER_ALIGN		1024
// Size of Rx channel ring buffers (to store none alligned data)
#define DTU2XX_RX_RING_BUF_SIZE			(2 * DTU2XX_BULK_TRANSFER_ALIGN)
// Size of Tx channel ring buffers (to store none alligned data)
#define DTU2XX_TX_RING_BUF_SIZE			(2 * DTU2XX_BULK_TRANSFER_ALIGN)

// Firmware level
#define DTU2XX_FWLEVEL_NONE		0		// No firmware uploaded
#define DTU2XX_FWLEVEL_FX2_ONLY 1		// Only EzUsb firmware uploaded
#define DTU2XX_FWLEVEL_FULL		2		// EzUsb + Altera Cyclone firmware uploaded

// Channel types
#define DTU2XX_TX_CHANNEL		1
#define DTU2XX_RX_CHANNEL		2

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Helper Structs +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ DTU2XX_INTEL_HEX_RECORD +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=
//
// INTEL hex record structure used for EzUsb firmware
//
#define MAX_INTEL_HEX_RECORD_LENGTH 16
typedef struct _INTEL_HEX_RECORD
{
   UInt8  Length;
   UInt16 Address;
   UInt8  Type;
   UInt8  Data[MAX_INTEL_HEX_RECORD_LENGTH];
} DTU2XX_INTEL_HEX_RECORD, *PDTU2XX_INTEL_HEX_RECORD;

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ DTU2XX_RING_BUFFER +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=
//
// Ring buffer used for bulk transfers
//
typedef struct _DTU2XX_RING_BUFFER
{
	UInt8*		m_pStart;		// Buffer start

	UInt8*		m_pWrPtr;		// Buffer write pointer
	UInt8*		m_pRdPtr;		// Buffer read pointer

	Int			m_Load;			// Buffer load (# bytes)
	Int			m_TotalSize;	// Size of the buffer

	spinlock_t	m_SpinLock;		// Buffer lock to protect pointers and or load

} DTU2XX_RING_BUFFER, *PDTU2XX_RING_BUFFER;

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ TS_RX_CHANNEL +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=
//
// Describes a single transmit or receive channel
//
typedef struct _DTU2XX_CHANNEL
{
	// General
	Int m_ChanType;				// Channel Type
	atomic_t m_UsageCount;			// Current usage count
	Int  m_FirmwareVersion;			// Copy of Firmware Version
	Int  m_TypeNumber;			// Copy of type number, for card-dependent actions

	// Status Flags: Updated every periodic interrupt
	Int  m_Status;					// Actual value of status flags
	Int  m_Latched;					// Latched value of status flags

	// Channel Buffers
	DTU2XX_RING_BUFFER m_RingBuf;	// Channel ring buffer
	UInt8* m_pTempBuf;				// Temporary buffer used internally to align
									// data	during Bulk transfers
	Int m_TempBufSize;				// Size of temporary buffer

	// Register cache
	union
	{
		Dtu2xxRx m_RxRegs;			// Receive registers
		Dtu2xxTx m_TxRegs;			// Transmit channels
	} m_ChRegs;

} DTU2XX_CHANNEL, *PDTU2XX_CHANNEL;

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ struct DTU2XX_FDO +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
//
typedef struct _DTU2XX_FDO
{
	// Cross-reference pointers
    struct usb_device* m_pUDev;			// USB device
    struct usb_interface* m_pUsbItf;		// USB interface
    struct kref m_KRef;					// Keep track of reference counts
    struct usb_class_driver* m_pUsbClass;	// USB class

	// State info
	Int m_FirmwareLevel;			// Current firmware level
	atomic_t m_UsageCount;			// Current usage count
	Dtu2xxGen m_GenRegs;			// Cached general registers

    // Location info
	Int m_TypeNumber;				// DTU-2xx type number (e.g. 205 or 225)
	Int m_FirmwareVersion;			// Firmware revision number
        Int m_IsDtu2x5Hw;                       // True if the device uses DTU-2x5 hardware

	// VPD cache
    UInt8 m_Vpd[DTU2XX_VPD_BUF_SIZE];

	// Channel-specific data
	Int	m_NumRxChannels;			// #receive channels (for error checking)
	DTU2XX_CHANNEL  m_TsRx[1];		// Transport-Stream Receive channels
	Int	m_NumTxChannels;			// #transmit channels (for error checking)
	DTU2XX_CHANNEL  m_TsTx[1];		// Transport-Stream Transmit channels

} DTU2XX_FDO;

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xx.c -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
DTU2XX_RESULT Dtu2xxCacheOpRegs(DTU2XX_FDO*);
DTU2XX_RESULT Dtu2xxAllocRingBuffer(DTU2XX_RING_BUFFER*, Int);
DTU2XX_RESULT Dtu2xx8051Reset(DTU2XX_FDO*, UInt8);
DTU2XX_RESULT Dtu2xxProgramEzUsb(DTU2XX_FDO*, const DTU2XX_INTEL_HEX_RECORD*);
DTU2XX_RESULT Dtu2xxProgramAltera(DTU2XX_FDO*, Int, UInt8*);
Int           Dtu2xxCheckFirmwareLevel(DTU2XX_FDO*);
DTU2XX_RESULT Dtu2xxDeviceInit(DTU2XX_FDO*);
DTU2XX_RESULT Dtu2xxInitDeviceHardware(DTU2XX_FDO*);

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxMoc.c -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
void Dtu2xxDecrUsageCount(DTU2XX_FDO*);
void Dtu2xxIncrUsageCount(DTU2XX_FDO*);
void Dtu2xxDevDelete(struct kref*);
ssize_t Dtu2xxRead(struct file*, char __user*, size_t, loff_t*);
ssize_t Dtu2xxWrite(struct file*, const char __user*, size_t, loff_t*);
int Dtu2xxOpen(struct inode*, struct file*);
int Dtu2xxRelease(struct inode*, struct file*);
int Dtu2xxProbe(struct usb_interface*, const struct usb_device_id*);
void Dtu2xxDisconnect(struct usb_interface*);
int __init Dtu2xxInitModule(void);
void __exit Dtu2xxCleanupModule(void);

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- IoCtl.c -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
Int  Dtu2xxIoCtl(struct inode *inode, struct file *filp,UInt cmd, unsigned long arg);
DTU2XX_RESULT Dtu2xxIoCtlResetSoft(DTU2XX_FDO*);
DTU2XX_RESULT Dtu2xxIoCtlLedControl(DTU2XX_FDO*, Int);
DTU2XX_RESULT Dtu2xxIoCtlReadVpd(DTU2XX_FDO*, Int, Int, UInt8*);
DTU2XX_RESULT Dtu2xxIoCtlWriteVpd(DTU2XX_FDO*, Int, Int, UInt8*);
DTU2XX_RESULT Dtu2xxIoCtlGetDeviceInfo(DTU2XX_FDO*, UInt8* , UInt8*);
DTU2XX_RESULT Dtu2xxIoCtlReadReg(DTU2XX_FDO*, UInt16, UInt32*);
DTU2XX_RESULT Dtu2xxIoCtlWriteReg(DTU2XX_FDO*, UInt16, UInt32);

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- RxIoCtl.c -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
DTU2XX_RESULT Dtu2xxRxIoCtlReset(DTU2XX_FDO*, Int, Int);
DTU2XX_RESULT Dtu2xxRxIoCtlGetStatus(DTU2XX_FDO*, Int, Int*, Int*, Int*, Int*);
DTU2XX_RESULT Dtu2xxRxIoCtlGetFlags(DTU2XX_FDO*, Int, Int*, Int*);
DTU2XX_RESULT Dtu2xxRxIoCtlClearFlags(DTU2XX_FDO*, Int, Int);
DTU2XX_RESULT Dtu2xxRxIoCtlGetTsRateBps(DTU2XX_FDO*, Int, Int*);
DTU2XX_RESULT Dtu2xxRxIoCtlGetFifoLoad(DTU2XX_FDO*, Int, Int*);
DTU2XX_RESULT Dtu2xxRxIoCtlSetPower(DTU2XX_FDO*, Int, Int);
DTU2XX_RESULT Dtu2xxRxIoCtlGetTargetId(DTU2XX_FDO*, Int, Int*, Int*);
DTU2XX_RESULT Dtu2xxRxIoCtlSetRxControl(DTU2XX_FDO*, Int, Int);
DTU2XX_RESULT Dtu2xxRxIoCtlSetRxMode(DTU2XX_FDO*, Int, Int);
DTU2XX_RESULT Dtu2xxRxIoCtlEqualise(DTU2XX_FDO*, Int, Int);
DTU2XX_RESULT Dtu2xxRxIoCtlPolarityControl(DTU2XX_FDO*, Int, Int);
DTU2XX_RESULT Dtu2xxRxIoCtlSetLoopBackMode(DTU2XX_FDO*, Int, Int);
DTU2XX_RESULT Dtu2xxRxIoCtlDma(DTU2XX_FDO*, Int, Int8*, Int);
DTU2XX_RESULT Dtu2xxRxIoCtlGetRxControl(DTU2XX_FDO*, Int, Int*);
DTU2XX_RESULT Dtu2xxRxIoCtlGetRxMode(DTU2XX_FDO*, Int, Int*);
DTU2XX_RESULT Dtu2xxRxIoCtlLedControl(DTU2XX_FDO*, Int, Int);
DTU2XX_RESULT Dtu2xxRxIoCtlClearFifo(DTU2XX_FDO*, Int);
DTU2XX_RESULT Dtu2xxRxIoCtlGetViolCount(DTU2XX_FDO*, Int, Int*);
DTU2XX_RESULT Dtu2xxRxIoCtlBulkRead(DTU2XX_FDO*, Int, Int, Int*, UInt8*);

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TxIoCtl.c -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
DTU2XX_RESULT Dtu2xxTxIoCtlReset(DTU2XX_FDO*, Int, Int);
DTU2XX_RESULT Dtu2xxTxIoCtlGetExtClkFreq(DTU2XX_FDO*, Int, Int*);
DTU2XX_RESULT Dtu2xxTxIoCtlGetFlags(DTU2XX_FDO*, Int, Int*, Int*);
DTU2XX_RESULT Dtu2xxTxIoCtlClearFifo(DTU2XX_FDO*, Int);
DTU2XX_RESULT Dtu2xxTxIoCtlClearFlags(DTU2XX_FDO*, Int, Int);
DTU2XX_RESULT Dtu2xxTxIoCtlSetTxClock(DTU2XX_FDO*, Int, Int);
DTU2XX_RESULT Dtu2xxTxIoCtlSetTxControl(DTU2XX_FDO*, Int, Int);
DTU2XX_RESULT Dtu2xxTxIoCtlSetTxMode(DTU2XX_FDO*, Int, Int, Int);
DTU2XX_RESULT Dtu2xxTxIoCtlSetPower(DTU2XX_FDO*, Int, Int);
DTU2XX_RESULT Dtu2xxTxIoCtlGetTargetId(DTU2XX_FDO*, Int, Int*, Int*);
DTU2XX_RESULT Dtu2xxTxIoCtlSetFifoSize(DTU2XX_FDO*, Int, Int);
DTU2XX_RESULT Dtu2xxTxIoCtlSetFifoSizeMax(DTU2XX_FDO*, Int);
DTU2XX_RESULT Dtu2xxTxIoCtlGetFifoLoad(DTU2XX_FDO*, Int, Int*);
DTU2XX_RESULT Dtu2xxTxIoCtlGetMaxFifoSize(DTU2XX_FDO*, Int, Int*);
DTU2XX_RESULT Dtu2xxTxIoCtlGetTxClock(DTU2XX_FDO*, Int, Int*);
DTU2XX_RESULT Dtu2xxTxIoCtlGetTxControl(DTU2XX_FDO*, Int, Int*);
DTU2XX_RESULT Dtu2xxTxIoCtlGetTxMode(DTU2XX_FDO*, Int, Int*, Int*);
DTU2XX_RESULT Dtu2xxTxIoCtlReadLoopBackData(DTU2XX_FDO*, Int, Int8*, Int);
DTU2XX_RESULT Dtu2xxTxIoCtlSetLoopBackMode(DTU2XX_FDO*, Int, Int);
DTU2XX_RESULT Dtu2xxTxIoCtlSetTxPolarity(DTU2XX_FDO*, Int, Int);
DTU2XX_RESULT Dtu2xxTxIoCtlDma(DTU2XX_FDO*, Int, Int8*, Int);
DTU2XX_RESULT Dtu2xxTxIoCtlBulkWrite(DTU2XX_FDO*, Int, Int, UInt8*);

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Utility.c -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
Int  Dtu2xxTxMode2PacketSize(Int nTxPacketMode);
Int  Dtu2xxBuf2RingBuffer(UInt8*, Int, PDTU2XX_RING_BUFFER);
Int  Dtu2xxRingBuffer2Buf(PDTU2XX_RING_BUFFER, Int, UInt8*);

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Vpd.c -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
DTU2XX_RESULT Dtu2xxVpdRead(DTU2XX_FDO*, UInt8*, UInt, UInt);
DTU2XX_RESULT Dtu2xxVpdWrite(DTU2XX_FDO*, UInt8*, UInt, UInt);
DTU2XX_RESULT Dtu2xxReadEeProm(DTU2XX_FDO*, Int, Int, UInt8*);
DTU2XX_RESULT Dtu2xxWriteEeProm(DTU2XX_FDO*, Int, Int, UInt8*);

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- GenUsb.c -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
DTU2XX_RESULT Dtu2xxVendorRequest(DTU2XX_FDO*, UInt8, UInt16, UInt16, Int, Int, UInt8*, Int*);
DTU2XX_RESULT Dtu2xxSimpleBulkTransfer(DTU2XX_FDO*, Int, Int, Int, UInt8*);

//Int SendCtrlMsg(DTU2XX_FDO* pFdo, UInt8 request, UInt16 value);
//Int GetCtrl(DTU2XX_FDO* pFdo, UInt8 request, UInt16 value, void* data, UInt16 size);
//Int SetCtrl(DTU2XX_FDO* pFdo, UInt8 request, UInt16 value, void* data, UInt16 size);

#endif //__DTU2XX_H
