//#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* Dtu2xxCommon.h *#*#*#*#*#*#*#* (C) 2000-2006 DekTec
//
// This code is the intellectual property of DekTec Digital Video B.V. and may not be
// distributed, in its original or in altered form, without the explicit permission of
// DekTec Digital Video B.V.

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//  2004.11.29  MG  Created 

#ifndef _DTU2XX_COMMON_H
#define _DTU2XX_COMMON_H

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Constants -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define	DTU2XX_IOCTL_MAGIC		0xAB	// Magic number
#define	DTU2XX_IOCTL_BASE		0x40	// Offset for Function
#define	DTU2XX_VPD_MAX_TRANSFER	512		// Maximum supported VPD transfer length

//+=+=+=+=+=+=+=+=+=+=+=+=+ Ioctl Input / Output Data Structures +=+=+=+=+=+=+=+=+=+=+=+=+

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Device-Driver Versioning Info. -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
typedef struct _DTU2XX_DRIVER_VERSION {
	Int		m_nVersionMajor;		// Major version number
	Int		m_nVersionMinor;		// Minor version number
	Int		m_nVersionMicro;		// Micro version number (bug fix)
	Int		m_nVersionBuild;		// Build number
} DTU2XX_DRIVER_VERSION;

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Location Info -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
typedef struct _DTU2XX_GET_LOC_INFO {
	UInt16 	m_nDeviceId;	// Device ID
	UInt16	m_nVendorId;	// Vendor ID
	Int		m_nSubSystemId;			// Subsystem ID
	Int		m_nSubVendorId;			// Subsystem Vendor ID
    Int     m_nUsbAddress;          // USB address
} DTU2XX_LOC_INFO;

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Flags -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
typedef struct _DTU2XX_FLAGS {
	Int		m_nStatus;				// Flags
	Int		m_nLatched;				// Latched flags (to be cleared)
} DTU2XX_FLAGS;

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Target Info -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
typedef struct _DTU2XX_TARGET_INFO {
	Int		m_nPresent;				// 0=Nothing connected; 1=DVB-SPI sink;
									// 2=Target adapter connected
	Int		m_nTargetId;			// Target-adapter identifier
} DTU2XX_TARGET_INFO;

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Rx status information -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
typedef struct _DTU2XX_RX_STATUS {
	Int			m_nPacketSize;		// Packet size
	Int			m_nNumInv;			// Number invalid bytes in packet
	Int			m_nClkDet;			// Clock detected (YES/NO)
	Int			m_nAsiLock;			// Asi lock detected (YES/NO)
	Int			m_nRateOk;			// Rate is OK (higher than minimum)
	Int			m_nAsiInv;			// ASI signal inverted (YES/NO)
} DTU2XX_RX_STATUS;

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Transport-Stream Rate Info -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// NOTE: only used with Transport-Stream transmit channel
//
typedef struct _DTU2XX_RATE_INFO {
	Int		m_nClockGenMode;		// Internal / external mode
	Int		m_nTxClock;				// Transport-stream clock
} DTU2XX_RATE_INFO;

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Transmit Mode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// NOTE: only used with Transport-Stream transmit channel
//
typedef struct _DTU2XX_TX_MODE {
	Int		m_nTxPacketMode;		// Packet transmit mode
	Int		m_nTxStuffMode;			// Null-packet stuffing on/off
} DTU2XX_TX_MODE;

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Data transfer decsriptor -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
typedef struct _DTU2XX_DATA_TRANSFER_DESC
{
	Int8*	m_pBuffer;				// Pointer to user buffer (receiving/providing data to
									// transfer)
	Int		m_nNumBytesToTr;		// Number of bytes to transfer (buffer must be at
									// least this size)
} DTU2XX_DATA_TRANSFER_DESC;

//-.-.-.-.-.-.-.-.-.-.-.- Vital Product Data for VPD read or write -.-.-.-.-.-.-.-.-.-.-.-
//
typedef struct _DTU2XX_VPD_TRANSFER {
	Int				m_nStart;		// Start BYTE address
	Int				m_nLength;		// Number of BYTES to transfer (buffer must be at
									// least this size)
	UInt8*	m_pVpdBuf;		// Pointer to user buffer (receiving/providing VPD data
									// to transfer)
} DTU2XX_VPD_TRANSFER;


//-.-.-.-.-.-.-.-.-.-.-.- Device information -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
typedef struct _DTU2XX_DEVICE_INFO {
    UInt8    m_nAlteraFwVersion;
    UInt8    m_nDevType;
} DTU2XX_DEVICE_INFO;

//-.-.-.-.-.-.-.-.-.-.-.- Register info
//
typedef struct _DTU2XX_REGISTER_INFO {
    UInt16 address;
    UInt value;
} DTU2XX_REGISTER_INFO;

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Union of all data structures -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
typedef struct _DTU2XX_IOCTL_DATA {
	union {
		DTU2XX_DRIVER_VERSION	m_DriverVersion;	// Versioning info
		DTU2XX_LOC_INFO			m_LocInfo;			// Location info
		DTU2XX_FLAGS			m_Flags;			// Flags
		DTU2XX_RX_STATUS		m_RxStatus;			// Rx Status
		DTU2XX_TARGET_INFO		m_TargetInfo;		// Target-adapter info
		DTU2XX_RATE_INFO		m_RateInfo;			// Transport-stream rate info
		DTU2XX_TX_MODE			m_TxMode;			// Transmit mode
		DTU2XX_DATA_TRANSFER_DESC	
								m_DataTransDesc;	// Data transfer descriptor
		DTU2XX_VPD_TRANSFER		m_Vpd;				// VPD transfer data
        DTU2XX_DEVICE_INFO      m_DevInfo;          // Device info
        DTU2XX_REGISTER_INFO    m_Register;         // Register info
		UInt			m_uiValue;			// Simple value
	}			m_Data;
	Int			m_nChIdx;		// Channel index (0=first channel, ...)
} DTU2XX_IOCTL_DATA;

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Function Codes +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=
enum {
	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Common functions -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	DTU2XX_FUNCTION_GET_DRIVER_VERSION  = DTU2XX_IOCTL_BASE,
	DTU2XX_FUNCTION_GET_LOCATION_INFO,
	DTU2XX_FUNCTION_DMA_USERMEM,
	DTU2XX_FUNCTION_LED_CONTROL,
	DTU2XX_FUNCTION_RESET_SOFT,

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Rx channel functions -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	DTU2XX_RX_FUNCTION_EQUALISE,
    DTU2XX_RX_FUNCTION_GET_FIFO_LOAD,
	DTU2XX_RX_FUNCTION_GET_FLAGS,
	DTU2XX_RX_FUNCTION_CLEAR_FLAGS,
	DTU2XX_RX_FUNCTION_GET_STATUS,
	DTU2XX_RX_FUNCTION_GET_TARGETID,
	DTU2XX_RX_FUNCTION_GET_RX_CLOCK,
	DTU2XX_RX_FUNCTION_POLARITY_CONTROL,
	DTU2XX_RX_FUNCTION_RESET,
	DTU2XX_RX_FUNCTION_SET_LOOPBACK_MODE,
	DTU2XX_RX_FUNCTION_SET_POWER,
	DTU2XX_RX_FUNCTION_SET_RX_CONTROL,
	DTU2XX_RX_FUNCTION_SET_RX_MODE,

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Tx channel functions -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	DTU2XX_TX_FUNCTION_CLEAR_FIFO,
	DTU2XX_TX_FUNCTION_CLEAR_FLAGS,
	DTU2XX_TX_FUNCTION_GET_EXT_CLK_FREQ,
	DTU2XX_TX_FUNCTION_GET_FIFO_LOAD,
	DTU2XX_TX_FUNCTION_GET_FLAGS,
	DTU2XX_TX_FUNCTION_GET_MAX_FIFO_SIZE,
	DTU2XX_TX_FUNCTION_GET_TARGET_ID,
	DTU2XX_TX_FUNCTION_GET_TX_CLOCK,
	DTU2XX_TX_FUNCTION_GET_TX_CONTROL,
	DTU2XX_TX_FUNCTION_GET_TX_MODE,
    DTU2XX_TX_FUNCTION_RD_LOOPBACK_DATA,
	DTU2XX_TX_FUNCTION_RESET,
	DTU2XX_TX_FUNCTION_SET_TX_POLARITY,
	DTU2XX_TX_FUNCTION_SET_FIFO_SIZE,
	DTU2XX_TX_FUNCTION_SET_LOOPBACK_MODE,
	DTU2XX_TX_FUNCTION_SET_POWER,
	DTU2XX_TX_FUNCTION_SET_TX_CLOCK,
	DTU2XX_TX_FUNCTION_SET_TX_CONTROL,
	DTU2XX_TX_FUNCTION_SET_TX_MODE,

	//.-.-.-.-.-.- New functions add to end for backward compatibilty reasons -.-.-.-.-.-.
	DTU2XX_FUNCTION_READ_VPD,
	DTU2XX_FUNCTION_WRITE_VPD,
	DTU2XX_FUNCTION_GET_FW_VERSION,
    DTU2XX_FUNCTION_GET_DEVICE_INFO,
	DTU2XX_RX_FUNCTION_GET_RX_CONTROL,
	DTU2XX_RX_FUNCTION_GET_RX_MODE,
    DTU2XX_RX_FUNCTION_LED_CONTROL,
    DTU2XX_RX_FUNCTION_CLEAR_FIFO,
    DTU2XX_RX_FUNCTION_GET_VIOL_COUNT,
    DTU2XX_FUNCTION_READ_REG,
    DTU2XX_FUNCTION_WRITE_REG,
	DTU2XX_RX_FUNCTION_BULK_USER_READ,
	DTU2XX_TX_FUNCTION_BULK_USER_WRITE
};

//-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_IOCTL_GET_DRIVER_VERSION -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_DriverVersion
// Channel index	: not used
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Get device-driver version.
//
#define DTU2XX_IOCTL_GET_DRIVER_VERSION	_IOR(DTU2XX_IOCTL_MAGIC, DTU2XX_FUNCTION_GET_DRIVER_VERSION, DTU2XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_IOCTL_GET_LOCATION_INFO -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_LocInfo
// Channel index	: not used
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Returns the cards location information
//
#define DTU2XX_IOCTL_GET_LOCATION_INFO	_IOR(DTU2XX_IOCTL_MAGIC, DTU2XX_FUNCTION_GET_LOCATION_INFO, DTU2XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_IOCTL_GET_DEVICE_INFO -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_DevInfo
// Channel index	: not used
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Returns the device information
//
#define DTU2XX_IOCTL_GET_DEVICE_INFO	_IOR(DTU2XX_IOCTL_MAGIC, DTU2XX_FUNCTION_GET_DEVICE_INFO, DTU2XX_IOCTL_DATA)


//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_IOCTL_GET_FW_VERSION -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_uiValue (firmware version)
// Channel index	: not used
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Returns the cards firmware version
//
#define DTU2XX_IOCTL_GET_FW_VERSION	_IOR(DTU2XX_IOCTL_MAGIC, DTU2XX_FUNCTION_GET_FW_VERSION, DTU2XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_IOCTL_LED_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (e.g. DTU2XX_LED_OFF, DTU2XX_LED_GREEN, etc)
// Channel index	: not used
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Front panel LED control
//
#define DTU2XX_IOCTL_LED_CONTROL		_IOW(DTU2XX_IOCTL_MAGIC, DTU2XX_FUNCTION_LED_CONTROL, DTU2XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_IOCTL_DMA_USERMEM_READ -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_DataTransDesc 
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Read data (using DMA) in an application-supplied buffer in User-Space memory.
//
#define DTU2XX_IOCTL_DMA_USERMEM_READ	_IOR(DTU2XX_IOCTL_MAGIC, DTU2XX_FUNCTION_DMA_USERMEM, DTU2XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_IOCTL_DMA_USERMEM_WRITE -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_DataTransDesc 
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Write data (using DMA) from an application-supplied buffer in User-Space memory.
//
#define DTU2XX_IOCTL_DMA_USERMEM_WRITE	_IOW(DTU2XX_IOCTL_MAGIC, DTU2XX_FUNCTION_DMA_USERMEM, DTU2XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_IOCTL_RESET_SOFT -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: none
// Channel index	: not used
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Reset the DTU-2xx card.
//
#define DTU2XX_IOCTL_RESET_SOFT			_IOW(DTU2XX_IOCTL_MAGIC, DTU2XX_FUNCTION_RESET_SOFT, DTU2XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_IOCTL_RESET_SOFT -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_Vpd
// Channel index	: not used
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Read specified range of VPD data BYTES.
// DTU2XX_VPD_TRANSFER.m_pVpdBuf[0] will contain first the byte read from EEPROM address m_nStart.
//
// NOTE: m_nLength should not be larger than DTU2XX_VPD_MAX_TRANSFER
//
#define DTU2XX_IOCTL_READ_VPD		_IOR(DTU2XX_IOCTL_MAGIC, DTU2XX_FUNCTION_READ_VPD, DTU2XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_IOCTL_RESET_SOFT -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_Vpd
// Channel index	: not used
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Write specified range of VPD data BYTES
// DTU2XX_VPD_TRANSFER.m_pVpdBuf[0] shall contain the first byte to be written to EEPROM address
// m_nStart. 
//
// NOTE: m_nLength should not be larger than DTU2XX_VPD_MAX_TRANSFER
//
#define DTU2XX_IOCTL_WRITE_VPD		_IOW(DTU2XX_IOCTL_MAGIC, DTU2XX_FUNCTION_WRITE_VPD, DTU2XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_IOCTL_READ_REG
//
// Data member      : m_Register
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Debugging purposes only
//
#define DTU2XX_IOCTL_READ_REG       _IOR(DTU2XX_IOCTL_MAGIC, DTU2XX_FUNCTION_READ_REG, DTU2XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_IOCTL_WRITE_REG
//
// Data member      : m_Register
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Debugging purposes only
//
#define DTU2XX_IOCTL_WRITE_REG       _IOW(DTU2XX_IOCTL_MAGIC, DTU2XX_FUNCTION_WRITE_REG, DTU2XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_EQUALISE -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (DTU2XX_EQUALISER_ON or DTU2XX_EQUALISER_OFF)
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Enable/Disable the equaliser
//
#define DTU2XX_RX_IOCTL_EQUALISE		_IOW(DTU2XX_IOCTL_MAGIC, DTU2XX_RX_FUNCTION_EQUALISE, DTU2XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_GET_FIFO_LOAD -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (returns the fifo load)
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Get the channels hardware fifo load
//
#define DTU2XX_RX_IOCTL_GET_FIFO_LOAD	_IOR(DTU2XX_IOCTL_MAGIC, DTU2XX_RX_FUNCTION_GET_FIFO_LOAD, DTU2XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_GET_FLAGS -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_Flags
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Get current value of status flags and latched status flags.
//
#define DTU2XX_RX_IOCTL_GET_FLAGS		_IOR(DTU2XX_IOCTL_MAGIC, DTU2XX_RX_FUNCTION_GET_FLAGS, DTU2XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_CLEAR_FLAGS -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_uiValue (Latched flags to clear)
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Clear selected flags
//
#define DTU2XX_RX_IOCTL_CLEAR_FLAGS		_IOW(DTU2XX_IOCTL_MAGIC, DTU2XX_RX_FUNCTION_CLEAR_FLAGS, DTU2XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_GET_STATUS -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_RxStatus
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Get the receive status (Packet size, ASI lock, clock detect ect) 
//
#define DTU2XX_RX_IOCTL_GET_STATUS		_IOR(DTU2XX_IOCTL_MAGIC, DTU2XX_RX_FUNCTION_GET_STATUS, DTU2XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_GET_TARGETID -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_TargetInfo
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Get the target adapter ID
//
#define DTU2XX_RX_IOCTL_GET_TARGETID	_IOR(DTU2XX_IOCTL_MAGIC, DTU2XX_RX_FUNCTION_GET_TARGETID, DTU2XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_GET_RX_CLOCK -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_uiValue (measured bit-rate)
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Get the measured transport-stream bit-rate
//
#define DTU2XX_RX_IOCTL_GET_RX_CLOCK	_IOR(DTU2XX_IOCTL_MAGIC, DTU2XX_RX_FUNCTION_GET_RX_CLOCK, DTU2XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_POLARITY_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (DTU2XX_POLARITY_AUTO,  DTU2XX_POLARITY_NORMAL or
//								 DTU2XX_POLARITY_INVERTED)
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Set the ASI polarity control
//
#define DTU2XX_RX_IOCTL_POLARITY_CONTROL	_IOW(DTU2XX_IOCTL_MAGIC, DTU2XX_RX_FUNCTION_POLARITY_CONTROL, DTU2XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_RESET -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (DTU2XX_RST_CLEARFIFO or DTU2XX_RST_CHANNEL)
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Reset the Rx Channel
//
#define DTU2XX_RX_IOCTL_RESET			_IOW(DTU2XX_IOCTL_MAGIC, DTU2XX_RX_FUNCTION_RESET, DTU2XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_SET_LOOPBACK_MODE -.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (DTU2XX_NO_LOOPBACK or DTU2XX_LOOPBACK_MODE)
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Set the loopback mode
//
#define DTU2XX_RX_IOCTL_SET_LOOPBACK_MODE	_IOW(DTU2XX_IOCTL_MAGIC, DTU2XX_RX_FUNCTION_SET_LOOPBACK_MODE, DTU2XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_SET_POWER -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (DTU2XX_POWER_OFF or DTU2XX_POWER_ON)
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Power enable control
//
#define DTU2XX_RX_IOCTL_SET_POWER		_IOW(DTU2XX_IOCTL_MAGIC, DTU2XX_RX_FUNCTION_SET_POWER, DTU2XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_SET_RX_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_uiValue (DTU2XX_RXCTRL_IDLE or DTU2XX_RXCTRL_RCV)
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Set receive control
//
#define DTU2XX_RX_IOCTL_SET_RX_CONTROL	_IOW(DTU2XX_IOCTL_MAGIC, DTU2XX_RX_FUNCTION_SET_RX_CONTROL, DTU2XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_SET_RX_MODE -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_uiValue (e.g. DTU2XX_RXMODE_ST188, DTU2XX_RXMODE_ST204 etc)
// Channel index	: Index of the input channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Set the rx mode
//
#define DTU2XX_RX_IOCTL_SET_RX_MODE		_IOW(DTU2XX_IOCTL_MAGIC, DTU2XX_RX_FUNCTION_SET_RX_MODE, DTU2XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_GET_RX_CONTROL
//
// Data member      : m_uiValue (DTU2XX_RXCTRL_IDLE or DTU2XX_RXCTRL_RCV)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Get the rx control
//
#define DTU2XX_RX_IOCTL_GET_RX_CONTROL  _IOR(DTU2XX_IOCTL_MAGIC, DTU2XX_RX_FUNCTION_GET_RX_CONTROL, DTU2XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_GET_RX_MODE
//
// Data member      : m_uiValue (e.g. DTU2XX_RXMODE_ST188, DTU2XX_RXMODE_ST204 etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Get the rx mode
//
#define DTU2XX_RX_IOCTL_GET_RX_MODE  _IOR(DTU2XX_IOCTL_MAGIC, DTU2XX_RX_FUNCTION_GET_RX_MODE, DTU2XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_LED_CONTROL
//
// Data member      : m_uiValue (e.g. DTU2XX_LED_OFF, DTU2XX_LED_GREEN, etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Control the receive status LED
//
#define DTU2XX_RX_IOCTL_LED_CONTROL  _IOW(DTU2XX_IOCTL_MAGIC, DTU2XX_RX_FUNCTION_LED_CONTROL, DTU2XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_CLEAR_FIFO
//
// Data member      : none
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Issue a FIFO reset
//
#define DTU2XX_RX_IOCTL_CLEAR_FIFO  _IOW(DTU2XX_IOCTL_MAGIC, DTU2XX_RX_FUNCTION_CLEAR_FIFO, DTU2XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_GET_VIOL_COUNT
//
// Data member      : m_uiValue
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Get the number of code violations detected
//
#define DTU2XX_RX_IOCTL_GET_VIOL_COUNT  _IOR(DTU2XX_IOCTL_MAGIC, DTU2XX_RX_FUNCTION_GET_VIOL_COUNT, DTU2XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.- IOCTL_DTU2XX_RX_BULK_USERMEM_READ -.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// InBuffer : Data buffer with data to be DMA-ed.
// OutBuffer: -
// Result   : DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Read data (using DMA) in an application-supplied buffer in User-Space memory.
//
#define DTU2XX_RX_IOCTL_BULK_USERMEM_READ	_IOR(DTU2XX_IOCTL_MAGIC, DTU2XX_RX_FUNCTION_BULK_USER_READ, DTU2XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_CLEAR_FIFO -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: none
// Channel index	: not used
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Clear the tx fifo
//
#define DTU2XX_TX_IOCTL_CLEAR_FIFO		_IOW(DTU2XX_IOCTL_MAGIC, DTU2XX_TX_FUNCTION_CLEAR_FIFO, DTU2XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_CLEAR_FLAGS -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_uiValue (Latched flags to clear)
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Clear the specified latched status flags
//
#define DTU2XX_TX_IOCTL_CLEAR_FLAGS		_IOW(DTU2XX_IOCTL_MAGIC, DTU2XX_TX_FUNCTION_CLEAR_FLAGS, DTU2XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_GET_EXT_CLK_FREQ -.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (freq. of the external clock)
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Get the measured frequency of the external clock
//
#define DTU2XX_TX_IOCTL_GET_EXT_CLK_FREQ	_IOR(DTU2XX_IOCTL_MAGIC, DTU2XX_TX_FUNCTION_GET_EXT_CLK_FREQ, DTU2XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_GET_FIFO_LOAD -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (the fifo load)
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Get the HW fifo load
//
#define DTU2XX_TX_IOCTL_GET_FIFO_LOAD	_IOR(DTU2XX_IOCTL_MAGIC, DTU2XX_TX_FUNCTION_GET_FIFO_LOAD, DTU2XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_GET_FLAGS -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_Flags
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Get current value of status flags and latched status flags.
//
#define DTU2XX_TX_IOCTL_GET_FLAGS		_IOR(DTU2XX_IOCTL_MAGIC, DTU2XX_TX_FUNCTION_GET_FLAGS, DTU2XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_GET_MAX_FIFO_SIZE -.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (the maximum allowed fifo size)
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Returns the maximum allowed fifo size
//
#define DTU2XX_TX_IOCTL_GET_MAX_FIFO_SIZE	_IOR(DTU2XX_IOCTL_MAGIC, DTU2XX_TX_FUNCTION_GET_MAX_FIFO_SIZE, DTU2XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_GET_TARGET_ID -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_TargetInfo
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Get the target adapter ID
//
#define DTU2XX_TX_IOCTL_GET_TARGET_ID	_IOR(DTU2XX_IOCTL_MAGIC, DTU2XX_TX_FUNCTION_GET_TARGET_ID, DTU2XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_GET_TX_CLOCK -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_uiValue (the set output bit-rate)
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Gets the set transport-stream bit-rate
//
#define DTU2XX_TX_IOCTL_GET_TX_CLOCK	_IOR(DTU2XX_IOCTL_MAGIC, DTU2XX_TX_FUNCTION_GET_TX_CLOCK, DTU2XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_GET_TX_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_uiValue (DTU2XX_TXCTRL_IDLE, DTU2XX_TXCTRL_HOLD or
//								 DTU2XX_TXCTRL_SEND)
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Get the channel's transmit-control state.
//
#define DTU2XX_TX_IOCTL_GET_TX_CONTROL	_IOR(DTU2XX_IOCTL_MAGIC, DTU2XX_TX_FUNCTION_GET_TX_CONTROL, DTU2XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_GET_TX_MODE -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_TxMode 
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Get the channel's transmit-mode state.
//
#define DTU2XX_TX_IOCTL_GET_TX_MODE		_IOR(DTU2XX_IOCTL_MAGIC, DTU2XX_TX_FUNCTION_GET_TX_MODE, DTU2XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_RD_LOOPBACK_DATA -.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_DataTransDesc 
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Get the channel's transmit-mode state.
//
#define DTU2XX_TX_IOCTL_RD_LOOPBACK_DATA	_IOR(DTU2XX_IOCTL_MAGIC, DTU2XX_TX_FUNCTION_RD_LOOPBACK_DATA, DTU2XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_RESET -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (DTU2XX_RST_CLEARFIFO or DTU2XX_RST_CHANNEL)
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Resets the Tx channel
//
#define DTU2XX_TX_IOCTL_RESET			_IOW(DTU2XX_IOCTL_MAGIC, DTU2XX_TX_FUNCTION_RESET, DTU2XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_SET_FIFO_SIZE -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (the fifo size)
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Sets the fifo size
//
#define DTU2XX_TX_IOCTL_SET_FIFO_SIZE	_IOW(DTU2XX_IOCTL_MAGIC, DTU2XX_TX_FUNCTION_SET_FIFO_SIZE,  DTU2XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_SET_LOOPBACK_MODE -.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (DTU2XX_NO_LOOPBACK or DTU2XX_LOOPBACK_MODE)
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Set the loopback mode
//
#define DTU2XX_TX_IOCTL_SET_LOOPBACK_MODE	_IOW(DTU2XX_IOCTL_MAGIC, DTU2XX_TX_FUNCTION_SET_LOOPBACK_MODE, DTU2XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_SET_TX_POLARITY -.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (DTUPI_TXPOL_NORMAL or DTUPI_TXPOL_INVERTED)
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Set the ASI tx polarity
//
#define DTU2XX_TX_IOCTL_SET_TX_POLARITY _IOW(DTU2XX_IOCTL_MAGIC, DTU2XX_TX_FUNCTION_SET_TX_POLARITY, DTU2XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_SET_POWER -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Data member		: m_uiValue (DTU2XX_POWER_OFF or DTU2XX_POWER_ON)
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Set the channels power mode
//
#define DTU2XX_TX_IOCTL_SET_POWER		_IOW(DTU2XX_IOCTL_MAGIC, DTU2XX_TX_FUNCTION_SET_POWER, DTU2XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_SET_TX_CLOCK -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_RateInfo (DTU2XX_POWER_OFF or DTU2XX_POWER_ON)
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Sets the channels transport-stream bit-rate 
//
#define DTU2XX_TX_IOCTL_SET_TX_CLOCK	_IOW(DTU2XX_IOCTL_MAGIC, DTU2XX_TX_FUNCTION_SET_TX_CLOCK, DTU2XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_SET_TX_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_uiValue (DTU2XX_TXCTRL_IDLE, DTU2XX_TXCTRL_HOLD or
//								 DTU2XX_TXCTRL_SEND)
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Sets the channels transmit-control state
//
#define DTU2XX_TX_IOCTL_SET_TX_CONTROL	_IOW(DTU2XX_IOCTL_MAGIC, DTU2XX_TX_FUNCTION_SET_TX_CONTROL, DTU2XX_IOCTL_DATA)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_SET_TX_MODE -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Data member		: m_TxMode
// Channel index	: Index of the output channel (0=first, 1=second etc)
// Result			: DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Sets the channels transmit-mode state
//
#define DTU2XX_TX_IOCTL_SET_TX_MODE		_IOW(DTU2XX_IOCTL_MAGIC, DTU2XX_TX_FUNCTION_SET_TX_MODE, DTU2XX_IOCTL_DATA)

//.-.-.-.-.-.-.-.-.-.-.-.-.- IOCTL_DTU2XX_TX_BULK_USERMEM_WRITE -.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// InBuffer : -
// OutBuffer: Data buffer with data to be DMA-ed.
// Result   : DTU2XX_OK if OK, DTU2XX_E_??? error code if failed
//
// Write data (using DMA) from an application-supplied buffer in User-Space memory.
//
#define DTU2XX_TX_IOCTL_BULK_USERMEM_WRITE	_IOW(DTU2XX_IOCTL_MAGIC, DTU2XX_TX_FUNCTION_BULK_USER_WRITE, DTU2XX_IOCTL_DATA)

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Helper Definitions +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Channel Definitions -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTU2XX_GEN_REGS			-1		// Special value for GENERIC registers
#define  DTU2XX_TS_TX_CHANNEL		0		// Transport-Stream Transmit Channel
#define  DTU2XX_TS_RX_CHANNEL		1		// Transport-Stream Receive Channel

// Transfer direction
#define DTU2XX_DIRECTION_READ		0		// Read transfer (DTU-2XX=>driver)
#define DTU2XX_DIRECTION_WRITE		1		// Write transfer (DTU-2XX<=driver)

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Reset Modes -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define  DTU2XX_RST_CLEARFIFO		0		// Clear FIFO without disrupting channel
#define  DTU2XX_RST_CHANNEL			1		// Channel reset

#endif // _DTU2XX_COMMON_H

