//*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* Dtu2xxRegs.h *#*#*#*#*#*#*#*# (C) 2000-2006 DEKTEC
//
// Definition of USB commands for DTU-2XX USB devices.

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	RZ	2003.11.09	Created 

#ifndef __DTU2XXREGS_H
#define __DTU2XXREGS_H

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- General USB Vendor Commands -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define DTU2XX_USB_RESET_DEVICE			0x80
#define DTU2XX_USB_SET_PROG_MODE		0x81
#define	DTU2XX_USB_PROGRAM_DONE			0x82
#define	DTU2XX_USB_DVC_LED_CONTROL		0x83	// OBSOLETE. USE DTU2XX_USB_WRITE_REG

#define DTU2XX_USB_READ_EEPROM			0x84
#define DTU2XX_USB_WRITE_EEPROM			0x85
#define DTU2XX_USB_GET_DEVICE_INF0		0x86
#define DTU2XX_USB_I2C_WRITE			0x87
#define DTU2XX_USB_I2C_READ				0x88

#define DTU2XX_USB_GET_USB_SPEED		0x8A
#define DTU2XX_USB_EEPROM_CONTROL		0x8B
#define DTU2XX_USB_FW_LOADED			0x8C

#define DTU2XX_USB_WRITE_REG			0x90
#define DTU2XX_USB_READ_REG				0x91

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- RX channel commands -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define DTU2XX_USB_RX_SET_RX_CONTROL	0x30	// OBSOLETE. USE DTU2XX_USB_WRITE_REG
#define DTU2XX_USB_RX_GET_RX_CONTROL	0x31	// OBSOLETE. USE DTU2XX_USB_READ_REG
#define DTU2XX_USB_RX_SET_RX_MODE		0x32	// OBSOLETE. USE DTU2XX_USB_WRITE_REG
#define DTU2XX_USB_RX_GET_RX_MODE		0x33	// OBSOLETE. USE DTU2XX_USB_READ_REG
#define DTU2XX_USB_RX_EQUALISE			0x34	// OBSOLETE. USE DTU2XX_USB_WRITE_REG
#define DTU2XX_USB_RX_GET_FIFO_LOAD		0x35	// OBSOLETE. USE DTU2XX_USB_READ_REG
#define DTU2XX_USB_RX_GET_FLAGS			0x36	// OBSOLETE. USE DTU2XX_USB_READ_REG
#define DTU2XX_USB_RX_CLEAR_FLAGS		0x37	// OBSOLETE. USE DTU2XX_USB_WRITE_REG
#define DTU2XX_USB_RX_GET_STATUS		0x38	// OBSOLETE. USE DTU2XX_USB_READ_REG
#define DTU2XX_USB_RX_GET_TS_RATE_BPS	0x39	// OBSOLETE. USE DTU2XX_USB_READ_REG
#define DTU2XX_USB_RX_RESET				0x40	
#define DTU2XX_USB_RX_SET_LOOPBACK_MODE	0x41	// OBSOLETE. USE DTU2XX_USB_WRITE_REG
#define DTU2XX_USB_RX_LED_CONTROL		0x42	// OBSOLETE. USE DTU2XX_USB_WRITE_REG
#define	DTU2XX_USB_RX_CLEAR_FIFO		0x43	
#define	DTU2XX_USB_RX_GET_VIOL_COUNT	0x44	// OBSOLETE. USE DTU2XX_USB_READ_REG

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- TX channel commands -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
#define	DTU2XX_USB_TX_SET_TX_CONTROL	0x51	// OBSOLETE. USE DTU2XX_USB_WRITE_REG
#define	DTU2XX_USB_TX_GET_TX_CONTROL	0x52	// OBSOLETE. USE DTU2XX_USB_READ_REG
#define	DTU2XX_USB_TX_SET_TX_MODE		0x53	// OBSOLETE. USE DTU2XX_USB_WRITE_REG
#define	DTU2XX_USB_TX_GET_TX_MODE		0x54	// OBSOLETE. USE DTU2XX_USB_READ_REG
#define	DTU2XX_USB_TX_RESET				0x55	
#define	DTU2XX_USB_TX_CLEAR_FIFO		0x56	
#define	DTU2XX_USB_TX_GET_FLAGS			0x57	// OBSOLETE. USE DTU2XX_USB_READ_REG
#define	DTU2XX_USB_TX_CLEAR_FLAGS		0x58	// OBSOLETE. USE DTU2XX_USB_WRITE_REG
#define	DTU2XX_USB_TX_GET_FIFO_LOAD		0x59	// OBSOLETE. USE DTU2XX_USB_READ_REG
#define	DTU2XX_USB_TX_SET_FIFO_SIZE		0x60	// OBSOLETE. USE DTU2XX_USB_WRITE_REG
#define	DTU2XX_USB_TX_SET_TX_CLOCK		0x61	// OBSOLETE. USE DTU2XX_USB_WRITE_REG
#define	DTU2XX_USB_TX_GET_TX_CLOCK		0x62	// OBSOLETE. USE DTU2XX_USB_READ_REG
#define	DTU2XX_USB_TX_SET_LOOPBACK_MODE	0x63	// OBSOLETE. USE DTU2XX_USB_WRITE_REG
#define	DTU2XX_USB_TX_RD_LOOPBACK_DATA	0x64	// OBSOLETE. USE DTU2XX_USB_READ_REG

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Helper values -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#define DTU2XX_BULK_PACKETSIZE			0xFC00
#define DTU2XX_BULK_BUFCOUNT			0x2

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Operation register structures +=+=+=+=+=+=+=+=+=+=+=+=+=+=+
//
#ifdef DTU2XX_DRIVER

// The general registers base address
#define DTU2XX_GEN_BASE_ADDR			0x0000

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ General register offset +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=
#define DTU2XX_GEN_REG_GENCTRL			0x0000
#define DTU2XX_GEN_REG_GENSTATUS		0x0004

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Dtu2xxGen +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
//
// Register block for GENERIC fields.
//
typedef struct _Dtu2xxGen {

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- General-Control register -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	union {
	struct {								
	volatile UInt  m_PE				: 1;	// Serial EEPROM Program Enable
	volatile UInt  m_PRE			: 1;	// Serial EEPROM Protect Register Enable
	volatile UInt  m_Reset			: 1;	// Board-wide software reset
			 UInt  m_Reserved1		: 1;	//
	volatile UInt  m_PerIntEn		: 1;	// Periodic-Interrupt Enable
			 UInt  m_Reserved2		: 3;	//
			 UInt  m_FirmwRev		: 8;	// Firmware Revision
			 UInt  m_TypeNum		: 8;	// Board Type Number
	volatile UInt  m_LedControl		: 1;	// Take over LED Control
	volatile UInt  m_LedGreen		: 1;	// State of Green LED (if LedControl is set)
	volatile UInt  m_LedRed			: 1;	// State of Red LED (if LedControl is set)
	}			     Fields;				//  For access to individual fields
			 UInt32  All;					//  For access to complete register
	}  m_GenControl;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- General-Status register -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
	union {
	 struct {
			  UInt  m_Reserved1		: 4;	//
	 volatile UInt  m_PerInt		: 1;	// Periodic Interrupt
	 }		 Fields;						// For access to individual field
	 UInt32  All;							// For access to GenStatus in one go
	}  m_GenStatus;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Programming register -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	union {
	struct {								
	volatile UInt  m_TRST			: 1;	// Control of TRST# pin
	volatile UInt  m_TDO			: 1;	// Status of TDO pin
	volatile UInt  m_TMS			: 1;	// Control of TMS pin
	volatile UInt  m_TCK			: 1;	// Control of TCK pin
	volatile UInt  m_TDI			: 1;	// Control of TDI pin
	volatile UInt  m_PrgEpc			: 1;	// Enable access to programming i/f of EPC2
	}			     Fields;				//  For access to individual fields
			 UInt32  All;					//  For access to complete register
	}  m_Programming;

	UInt32  m_RefClkCnt;					// Reference clock counter
	UInt32  m_EpcsCtrl;						// EPCS Control register
	UInt32  m_EpcsData;						// EPCS Data register
	UInt32  m_Reserved[26];					// Unused memory addresses
} Dtu2xxGen;

// The transmit-channel base address
#define DTU2XX_TX_BASE_ADDR				0x0080

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ TX register offset +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=
#define DTU2XX_TX_REG_GENCTRL			0x0000
#define DTU2XX_TX_REG_TXCTRL			0x0004
#define DTU2XX_TX_REG_TXSTATUS			0x0008
#define DTU2XX_TX_REG_TXCLOCK			0x000C
#define DTU2XX_TX_REG_FIFOSIZE			0x0014
#define DTU2XX_TX_REG_FIFOLOAD			0x0018
#define DTU2XX_TX_REG_DIAG				0x001C
#define DTU2XX_TX_REG_LOOPBKDATA		0x0020
#define DTU2XX_TX_REG_FIFO_FIRST		0x0040
#define DTU2XX_TX_REG_FIFO_LAST			0x007C

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Dtu2xxTx +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
//
// Register block for a TRANSMISSION channel.
//
typedef struct _Dtu2xxTx {

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- General-Control register -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	//
	// On DTA-140, these fields are moved to the generic register block.
	//
	union {
	struct {								// General-Control register
	volatile UInt  m_PE				: 1;	//  Serial EEPROM Program Enable
	volatile UInt  m_PRE			: 1;	//  Serial EEPROM Protect Register Enable
	volatile UInt  m_Reset			: 1;	//  Soft reset of DTA 100 circuitry
			 UInt  m_Reserved1		: 5;	//
			 UInt  m_FirmwRev		: 8;	//  Firmware Revision
			 UInt  m_TypeNum		: 8;	//  Board Type Number
	}			     Fields;				//  For access to individual fields
			 UInt32  All;					//  For access to complete register
	}  m_GenControl;

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Transmit-Control register -.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	union {
	struct {								
	volatile UInt  m_TxMode			: 2;	//  Transmit Mode: 188/204/Add16/Raw
	volatile UInt  m_BurstMode		: 1;	//  Burst Mode: 1 = send packets in bursts
	volatile UInt  m_TxModeExt		: 1;	//	Transmit Mode extension: 192
	volatile UInt  m_PckStuff		: 1;	//  Null-Packet Stuffing On/Off
	volatile UInt  m_TxCtrl			: 2;	//  Transmit Control: Idle/Hold/Send
	volatile UInt  m_TxDis			: 1;	//  Disable LVDS outputs
	volatile UInt  m_PerIntEn		: 1;	//  Periodic-Interrupt Enable
	volatile UInt  m_UflIntEn		: 1;	//  Output-FIFO Underflow Interrupt Enable
	volatile UInt  m_SyncIntEn		: 1;	//  Synchronisation-Error Interrupt Enable
	volatile UInt  m_ShortIntEn		: 1;	//  Short-Circuit-Detected Interrupt Enable
	volatile UInt  m_ThrIntEn		: 1;	//  Threshold-Crossed Interrupt Enable
			 UInt  m_Reserved2		: 3;	//
	volatile UInt  m_UseExtClk		: 1;	//  Use External Clock input
	volatile UInt  m_EnaPwr			: 1;	//  Enable Power to target adapter
	volatile UInt  m_ShortErr		: 1;	//  Short-Circuit-Detected Error
	volatile UInt  m_LedControl		: 1;	//  Take over LED Control
	volatile UInt  m_LedGreen		: 1;	//  State of Green LED (if LedControl is set)
	volatile UInt  m_LedRed			: 1;	//  State of Red LED (if LedControl is set)
	volatile UInt  m_ClrFifo		: 1;	//  Clear FIFO (write only)
	volatile UInt  m_TxRst			: 1;	//  Reset Transmit Channel (write only)
	volatile UInt  m_TxInvAsi		: 1;	//  Invert DVB/ASI output signal
	volatile UInt  m_SdiMode		: 1;	//  SDI mode enable/disbale
	volatile UInt  m_Sdi10Bit		: 1;	//  SDI 10-bit mode
	}			     Fields;				//  For access to individual fields
			 UInt32  All;					//  For access to complete register
	}  m_TxControl;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Transmit-Status register -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	//
	// Status register is a union.
	// WATCH OUT: DO NOT clear interrupts by something like p->Fields.m_PerInt = 1;
	// THIS WOULD CLEAR ALL STATUS FLAGS!!
	union {
	struct {
	volatile UInt  m_FifoFilled		: 1;	//  Output-FIFO Filled flag
			 UInt  m_Reserved1		: 3;	//
	volatile UInt  m_SdramSize		: 4;	//  SDRAM Size
	volatile UInt  m_PerInt			: 1;	//  Periodic Interrupt
	volatile UInt  m_UflInt			: 1;	//  Output-FIFO Underflow Interrupt
	volatile UInt  m_SyncInt		: 1;	//  Synchronisation-Error Interrupt
	volatile UInt  m_ShortInt		: 1;	//  Short-Circuit-Detected Interrupt
	volatile UInt  m_ThrInt			: 1;	//  Threshold-Crossed Interrupt
	}				 Fields;				// For access to individual field
			 UInt32  All;					// For access to TxStatus in one go
	} m_TxStatus;

	volatile UInt32  m_TxClock;				// Transmit-Clock register
	volatile UInt32  m_ClkCnt;				// Clock-Count register
	volatile UInt32  m_FifoSize;			// FIFO-Size register
	volatile UInt32  m_FifoLoad;			// FIFO-Load register

	union {
	struct {								// Diagnostics register
	volatile UInt  m_SfData			: 8;	//  Data at output of Smoothing FIFO
	volatile UInt  m_LoopBack		: 1;	//	Loop-back output of Smoothing FIFO
	volatile UInt  m_SfDtValid		: 1;	//  Smoothing FIFO Data Vaild
	volatile UInt  m_BfFull			: 1;	//	Burst-FIFO-Full flag
	volatile UInt  m_DmaReq			: 1;	//  DMA Request from Burst FIFO flag
	volatile UInt  m_BfCanBurst		: 1;	//  Burst-FIFO Can-Burst flag
	volatile UInt  m_SfCanBurst		: 1;	//  Smoothing-FIFO Can-Burst flag
	volatile UInt  m_SfLoad			: 10;	//  Smoothing-FIFO Load in #bytes
	volatile UInt  m_BfLoad			: 8;	//  Burst-FIFO Load in #32-bit words
	}			     Fields;				//  For access to individual fields
			 UInt32  All;					//  For access to complete register
	}  m_Diag;

	volatile UInt32  m_LoopBackData;		// Data at output of Smoothing FIFO

	union {
	struct {								// Threshold-Control register
	volatile UInt  m_UpdBusy		: 1;	//  Threshold-Update process busy
	volatile UInt  m_CmpB			: 1;	//  Output of comparator B
	volatile UInt  m_CmpA			: 1;	//  Output of comparator A
	volatile UInt  m_AorB			: 1;	//  Update threshold A or threshold B
			 UInt  m_Reserved1		: 4;	//
	volatile UInt  m_ThreshA		: 8;	//  Threshold A
	volatile UInt  m_ThreshB		: 8;	//  Threshold B
	}			     Fields;				//  For access to individual fields
			 UInt32  All;					//  For access to complete register
	}  m_ThreshControl;

	volatile UInt32  m_Reserved[6];			// Not used
	volatile UInt32  m_FifoData[16];		// Transport-stream data: 4 bytes at a time
} Dtu2xxTx;

// The receive-channel base address
#define DTU2XX_RX_BASE_ADDR				0x0100

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ RX register offset +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

#define DTU2XX_RX_REG_GENCTRL			0x0000
#define DTU2XX_RX_REG_RXCTRL			0x0004
#define DTU2XX_RX_REG_RXSTATUS			0x0008
#define DTU2XX_RX_REG_FIFOLOAD			0x0018
#define DTU2XX_RX_REG_DIAG				0x001C
#define DTU2XX_RX_REG_LOOPBKDATA		0x0020
#define DTU2XX_RX_REG_THRESHCTRL		0x0024
#define DTU2XX_RX_REG_PCKCOUNT			0x0028
#define DTU2XX_RX_REG_CLKCOUNT			0x002C
#define DTU2XX_RX_REG_VALIDCOUNT		0x0030
#define DTU2XX_RX_REG_VIOLCOUNT			0x0034

#define DTU2XX_RX_REG_FIFO_FIRST		0x0040
#define DTU2XX_RX_REG_FIFO_LAST			0x0044

#define DTU2XX_RX_REG_BITRATE			0x004C

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Dtu2xxRx +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
//
// Register block for a RECEIVE channel.
//
typedef struct _Dtu2xxRx {

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- General-Control register -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	//
	// On DTA-140, these fields are moved to the generic register block.
	//
	union {
	struct {
	volatile UInt  m_PE				: 1;	// Serial EEPROM Program Enable
	volatile UInt  m_PRE			: 1;	// Serial EEPROM Protect Register Enable
	volatile UInt  m_Reset			: 1;	// Soft reset of DTA 100 circuitry
			 UInt  m_Reserved1		: 5;	//
			 UInt  m_FirmwRev		: 8;	// Firmware Revision
			 UInt  m_TypeNum		: 8;	// Board Type Number
	}			     Fields;				//  For access to individual fields
			 UInt32  All;					//  For access to complete register
	}  m_GenControl;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Receive-Control register -.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	union {
	struct {								
	volatile UInt  m_RxMode			: 2;	//  Receive Mode: St188/St204/StMp2/StRaw
	volatile UInt  m_AsiInv			: 2;	//  Invert ASI-input control (ASI cards)
	volatile UInt  m_Equalise		: 1;	//  Equaliser control (ASI cards)
	volatile UInt  m_RxCtrl			: 1;	//  Receive Control: Idle/Rcv
			 UInt  m_Reserved2		: 1;	//
	volatile UInt  m_RxTimeStamp	: 1;	//  Insert Time Stamps
	volatile UInt  m_PerIntEn		: 1;	//  Periodic-Interrupt Enable
	volatile UInt  m_OvfIntEn		: 1;	//  Input-FIFO Overflow Interrupt Enable
	volatile UInt  m_SyncIntEn		: 1;	//  Synchronisation-Error Interrupt Enable
			 UInt  m_Reserved3		: 1;	//
	volatile UInt  m_ThrIntEn		: 1;	//  Threshold-Crossed Interrupt Enable
	volatile UInt  m_RateOvfIntEn	: 1;	//  Input-Rate Overflow Interrupt Enable
			 UInt  m_Reserved4		: 3;	//
	volatile UInt  m_EnaPwr			: 1;	//  Enable Power to target adapter
			 UInt  m_Reserved5		: 1;	//
	volatile UInt  m_LedControl		: 1;	//  Take over LED Control
	volatile UInt  m_LedGreen		: 1;	//  State of Green LED (if LedControl is set)
	volatile UInt  m_LedRed			: 1;	//  State of Red LED (if LedControl is set)
	volatile UInt  m_RxClrFifo		: 1;	//  Clear Receive FIFO (write only)
	volatile UInt  m_RxRst			: 1;	//  Reset Receive Channel (write only)
	volatile UInt  m_SdiMode		: 1;	//  SDI mode enable/disbale
	volatile UInt  m_Sdi10Bit		: 1;	//  SDI 10-bit mode
	}			     Fields;				//  For access to individual fields
			 UInt32  All;					//  For access to complete register
	}  m_RxControl;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Receive-Status register -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
	//
	// Status register is a union.
	// WATCH OUT: DO NOT clear interrupts by something like p->Fields.m_PerInt = 1;
	// THIS WOULD CLEAR ALL STATUS FLAGS!!
	union {
	struct {
	volatile UInt  m_PckSize		: 2;	//  Packet size: Rx188/Rx204/RxInv
	volatile UInt  m_NumInv			: 2;	//  Number of invalid bytes per packet
	volatile UInt  m_SdramSize		: 4;	//  SDRAM Size
	volatile UInt  m_PerInt			: 1;	//  Periodic Interrupt
	volatile UInt  m_OvfInt			: 1;	//  Input-FIFO Overflow Interrupt
	volatile UInt  m_SyncInt		: 1;	//  Synchronisation-Error Interrupt
			 UInt  m_Reserved1		: 1;	//
	volatile UInt  m_ThrInt			: 1;	//  Threshold-Crossed Interrupt
	volatile UInt  m_RateOvfInt		: 1;	//  Input-Rate Overflow Interrupt
	volatile UInt  m_ClkDet			: 1;	//  Receive-Clock Detected (DTA-120: Asi CD)
	volatile UInt  m_AsiLock		: 1;	//  DVB/ASI PLL locked to input signal
	volatile UInt  m_RateOk			: 1;	//  Input rate above minimum input rate
	volatile UInt  m_AsiInv			: 1;	//  DVB/ASI-Input Invert - Status
	}			     Fields;				//  For access to individual fields
			 UInt32  All;					//  For access to complete register
	} m_RxStatus;

	volatile UInt32  m_Reserved[3];			// Not used

	volatile UInt32  m_FifoLoad;			// FIFO-Load register

	union {
	struct {								// Diagnostics register
			 UInt  m_Reserved1		: 8;	//
	volatile UInt  m_LoopBack		: 1;	//	Loop-back mode
	volatile UInt  m_CdcRateOvf		: 1;	//	Clock-Domain-Converter Rate Overflow
	volatile UInt  m_RfRateOvf		: 1;	//	Receive-FIFO Rate Overflow
	volatile UInt  m_SdFull			: 1;	//	SDRAM-Full Flag
			 UInt  m_Reserved2		: 7;	//

	// ONLY FOR: DTA-120, Firmware Revision < 4 (Later revs: Programming register is used)
	volatile UInt  m_TDO			: 1;	// Status of TDO pin
	volatile UInt  m_TMS			: 1;	// Control of TMS pin
	volatile UInt  m_TCK			: 1;	// Control of TCK pin
	volatile UInt  m_TDI			: 1;	// Control of TDI pin
	volatile UInt  m_PrgEpc			: 1;	// Enable access to programming i/f of EPC2
	}			     Fields;				//  For access to individual fields
			 UInt32  All;					//  For access to complete register
	}  m_Diag;

	volatile UInt32  m_LbData;				// Data written to Input FIFO (8 sign. bits)

	union {
	struct {								// Threshold-Control register
	volatile UInt  m_UpdBusy		: 1;	//  Threshold-Update process busy
	volatile UInt  m_CmpB			: 1;	//  Output of comparator B
	volatile UInt  m_CmpA			: 1;	//  Output of comparator A
	volatile UInt  m_AorB			: 1;	//  Update threshold A or threshold B
			 UInt  m_Reserved1		: 4;	//
	volatile UInt  m_ThreshA		: 8;	//  Threshold A
	volatile UInt  m_ThreshB		: 8;	//  Threshold B
	}			     Fields;				//  For access to individual fields
			 UInt32  All;					//  For access to complete register
	}  m_ThreshControl;

	volatile UInt32  m_PckCnt;				// Packet Count (DTA-122)
	volatile UInt32  m_ClkCnt;				// Clock Count (DTA-122)
	volatile UInt32  m_ValidCnt;			// Valid Count (DTA-120, DTA-122)
	volatile UInt32  m_CodeViolCnt;			// Code-Violation Count (DTA-120)

	volatile UInt32  m_Reserved2[1];		// Not used

	volatile UInt32  m_LockFilter;			// Control value for Lock Filter (DTA-120)

	volatile UInt32  m_FifoData[2];			// Transport-stream data: 4 bytes at a time
	volatile UInt32  m_Bitrate;				// Bit-rate
	volatile UInt32  m_Reserved3[13];		// Not used
} Dtu2xxRx;

#endif // #ifdef DTU2XX_DRIVER
#endif // #ifndef __DTU2XXREGS_H

