//*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* Dtu2xx.c *#*#*#*#*#*#*#*#*#*#*#* (C) 2006 DekTec
//

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//	2006.09.28	MG	Created

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#include <linux/device.h>
#include <linux/usb.h>
#include <asm/uaccess.h>
#include <linux/firmware.h>
#include <linux/version.h>
#include "Dtu2xx.h"
#include "Dtu2xxRegs.h"
#include "EzUsb.h"

// Altera Cyclone FPGA firmware
#include "Cyclone205.h"
#include "Cyclone225.h"
#include "Cyclone245.h"
#include "Cyclone2x5.h"

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxDeviceInit -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
DTU2XX_RESULT Dtu2xxDeviceInit(
	DTU2XX_FDO* pFdo)
{
	DTU2XX_RESULT dr;

	Int i, FwSize=0, Dummy;
	UInt8* pFirmware=NULL;
	DTU2XX_CHANNEL* pChan = NULL;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Get the EEPROM contents -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	dr = Dtu2xxReadEeProm(pFdo, 0, DTU2XX_VPD_BUF_SIZE, pFdo->m_Vpd);
	if ( dr != DTU2XX_OK )
	{
		DTU2XX_LOG( KERN_INFO, "Failed to init EEPROM cache" );
		return dr;
	}

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Program PLD firmware -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

#if LOG_LEVEL > 0
        DTU2XX_LOG1( KERN_INFO, "Start programming of DTU-%d PLD", pFdo->m_TypeNumber );
#endif

        // Check for 2x5 hardware
        if ( pFdo->m_IsDtu2x5Hw != 0 )
        {
          FwSize = sizeof(c_CycloneCode2x5);
          pFirmware = (UInt8*)c_CycloneCode2x5;
        }
        else
        {
          switch ( pFdo->m_TypeNumber )
          {

          case 205:
                  FwSize = sizeof(c_CycloneCode205);
                  pFirmware = (UInt8*)c_CycloneCode205;
                  break;

          case 225:
                  FwSize = sizeof(c_CycloneCode225);
                  pFirmware = (UInt8*)c_CycloneCode225;
                  break;

          case 245:
                  FwSize = sizeof(c_CycloneCode245);
                  pFirmware = (UInt8*)c_CycloneCode245;
                  break;

          default:
                  FwSize = 0;
                  pFirmware = NULL;
                  break;

          }
        }
	dr = Dtu2xxProgramAltera(pFdo, FwSize, pFirmware);
	if ( dr != DTU2XX_OK )
	{
		DTU2XX_LOG1( KERN_INFO, "Failed to program DTU-%d PLD", pFdo->m_TypeNumber );
		return dr;
	}

	/* Update firmware level */
	pFdo->m_FirmwareLevel = DTU2XX_FWLEVEL_FULL;

	// Get firmware version
	dr = Dtu2xxIoCtlGetDeviceInfo(pFdo, (UInt8*)(&(pFdo->m_FirmwareVersion)), (UInt8*)(&Dummy));
	if ( dr != DTU2XX_OK )
		return dr;

	DTU2XX_LOG2( KERN_INFO, "Programmed DTU-%d PLD firmware: v%d", pFdo->m_TypeNumber,
				 pFdo->m_FirmwareVersion );

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Init number of channels -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	// Initially assume no channels at all
	pFdo->m_NumTxChannels = 0;
	pFdo->m_NumRxChannels = 0;

	switch ( pFdo->m_TypeNumber )
	{

	case 205:
		pFdo->m_NumTxChannels = 1;
		break;

	case 225:
		pFdo->m_NumRxChannels = 1;
		break;

	case 245:
		pFdo->m_NumTxChannels = 1;
		pFdo->m_NumRxChannels = 1;
		break;

	default:
		break;

	}

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Init channels -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	for ( i=0; i<pFdo->m_NumTxChannels+pFdo->m_NumRxChannels; i++ )
	{
		if ( i<pFdo->m_NumTxChannels )
		{
			pChan = &pFdo->m_TsTx[i];
			pChan->m_ChanType = DTU2XX_TX_CHANNEL;
		}
		else
		{
			pChan = &pFdo->m_TsRx[i-pFdo->m_NumTxChannels];
			pChan->m_ChanType = DTU2XX_RX_CHANNEL;
		}

		// Copy of type number
		pChan->m_TypeNumber = pFdo->m_TypeNumber;
		pChan->m_FirmwareVersion = pFdo->m_FirmwareVersion;

		// init exclusive acces object
		atomic_set(&pChan->m_UsageCount, 1);

		// Create ring buffer
		dr = DTU2XX_OK;
		if ( pChan->m_ChanType==DTU2XX_TX_CHANNEL && DTU2XX_TX_RING_BUF_SIZE>0 )
		{
			dr = Dtu2xxAllocRingBuffer(&pChan->m_RingBuf, DTU2XX_TX_RING_BUF_SIZE);
		}
		else if ( pChan->m_ChanType==DTU2XX_RX_CHANNEL && DTU2XX_RX_RING_BUF_SIZE>0 )
		{
			dr = Dtu2xxAllocRingBuffer(&pChan->m_RingBuf, DTU2XX_RX_RING_BUF_SIZE);
		}

		if ( dr != DTU2XX_OK )
		{
			DTU2XX_LOG( KERN_INFO, "Failed to create ring-buffer" );
			return dr;
		}

		// Create temporary buffer
		pChan->m_pTempBuf = NULL;
		pChan->m_pTempBuf = (UInt8*)kmalloc( DTU2XX_BULK_PACKETSIZE, GFP_KERNEL );
		pChan->m_TempBufSize = DTU2XX_BULK_PACKETSIZE;
		if ( pChan->m_pTempBuf == NULL )
		{
			DTU2XX_LOG( KERN_INFO, "Failed to create temporary buffer" );
			return DTU2XX_E;
		}
	}

	// Finally perform device specific initialisation
	return Dtu2xxInitDeviceHardware(pFdo);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxInitDeviceHardware -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
DTU2XX_RESULT Dtu2xxInitDeviceHardware(
	DTU2XX_FDO* pFdo)
{
	DTU2XX_RESULT dr;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.- Cache operational registers -.-.-.-.-.-.-.-.-.-.-.-.-.-.

	dr = Dtu2xxCacheOpRegs(pFdo);
	if ( dr != DTU2XX_OK )
		return dr;

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Init device registers -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	switch ( pFdo->m_TypeNumber )
	{

	case 205:
		// Transmit Control := Idle
		dr = Dtu2xxTxIoCtlSetTxControl( pFdo, 0, DTU2XX_TXCTRL_IDLE );
		if ( dr != DTU2XX_OK )
			return dr;

		// Transmit Mode := 188
		// Burst Mode Off
		// Null-Packet Stuffing Off
		dr = Dtu2xxTxIoCtlSetTxMode( pFdo, 0, DTU2XX_TXMODE_188, DTU2XX_TXMODE_STUFF_OFF );
		if ( dr != DTU2XX_OK )
			return dr;

		// Set maximum FIFO size
		dr = Dtu2xxTxIoCtlSetFifoSizeMax(pFdo, 0);
		if ( dr != DTU2XX_OK )
			return dr;

		break;

	case 225:

		// Receive Control := Idle
		dr = Dtu2xxRxIoCtlSetRxControl(pFdo, 0, DTU2XX_RXCTRL_IDLE);
		if ( dr != DTU2XX_OK )
			return dr;
		// Receive mode := ST188
		dr = Dtu2xxRxIoCtlSetRxMode(pFdo, 0, DTU2XX_RXMODE_ST188);
		if ( dr != DTU2XX_OK )
			return dr;

		break;

        case 245:

                // Transmit Control := Idle
                dr = Dtu2xxTxIoCtlSetTxControl( pFdo, 0, DTU2XX_TXCTRL_IDLE );
                if ( dr != DTU2XX_OK )
                  return dr;

                // Transmit Mode := 188
                // Burst Mode Off
                // Null-Packet Stuffing Off
                dr = Dtu2xxTxIoCtlSetTxMode( pFdo, 0, DTU2XX_TXMODE_188, DTU2XX_TXMODE_STUFF_OFF );
                if ( dr != DTU2XX_OK )
                  return dr;

                // Set maximum FIFO size
                dr = Dtu2xxTxIoCtlSetFifoSizeMax(pFdo, 0);
                if ( dr != DTU2XX_OK )
                  return dr;


                // Receive Control := Idle
                dr = Dtu2xxRxIoCtlSetRxControl(pFdo, 0, DTU2XX_RXCTRL_IDLE);
                if ( dr != DTU2XX_OK )
                        return dr;
                // Receive mode := ST188
                dr = Dtu2xxRxIoCtlSetRxMode(pFdo, 0, DTU2XX_RXMODE_ST188);
                if ( dr != DTU2XX_OK )
                        return dr;

                break;

	default:
		break;
	}
	return DTU2XX_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxCacheOpRegs -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Cache the operational registers
//
DTU2XX_RESULT Dtu2xxCacheOpRegs(
	DTU2XX_FDO* pFdo)
{
	UInt32 *pReg;
	Int RegAddr, Status = 0;

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Cache general registers -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	pReg = (UInt32 *)(&(pFdo->m_GenRegs));

	for( RegAddr = DTU2XX_GEN_REG_GENCTRL; RegAddr <= DTU2XX_GEN_REG_GENSTATUS; RegAddr += 4 )
	{
		// Get register
		Status = Dtu2xxIoCtlReadReg( pFdo, RegAddr+DTU2XX_GEN_BASE_ADDR, pReg );
		if ( Status!=0 )
			return DTU2XX_E;

#if LOG_LEVEL > 0
		DTU2XX_LOG3( KERN_INFO,
			   		 "Caching general-register: 0x%08X, pReg=0x%p, Value: 0x%08X",
					 RegAddr+DTU2XX_GEN_BASE_ADDR, pReg, (unsigned int)*pReg );
#endif

		// Move to next register in map
		pReg++;
	}

	//-.-.-.-.-.-.-.-.-.-.-.-.- Cache transmit-channel registers -.-.-.-.-.-.-.-.-.-.-.-.-

	if( pFdo->m_NumTxChannels != 0 )
	{
		pReg = (UInt32 *)(&(pFdo->m_TsTx[0].m_ChRegs.m_TxRegs));
		for ( RegAddr = DTU2XX_TX_REG_GENCTRL; RegAddr <= (DTU2XX_TX_REG_FIFO_LAST); RegAddr += 4)
		{
			// Get register
			Status = Dtu2xxIoCtlReadReg( pFdo, RegAddr+DTU2XX_TX_BASE_ADDR, pReg );
			if ( Status!=0 )
				return DTU2XX_E;

#if LOG_LEVEL > 0
			DTU2XX_LOG3( KERN_INFO,
						 "Caching transmit-register: 0x%08X, pReg=0x%p, Value: 0x%08X",
						 RegAddr+DTU2XX_TX_BASE_ADDR, pReg, (unsigned int)*pReg );
#endif

			// Move to next register in map
			pReg++;
		}
	}

	//-.-.-.-.-.-.-.-.-.-.-.-.- Cache receive-channel registers -.-.-.-.-.-.-.-.-.-.-.-.-.

	if ( pFdo->m_NumRxChannels != 0 )
	{
		pReg = (UInt32 *)(&(pFdo->m_TsRx[0].m_ChRegs.m_RxRegs));
		for( RegAddr = DTU2XX_RX_REG_GENCTRL; RegAddr <= DTU2XX_RX_REG_BITRATE; RegAddr += 4 )
		{
			// Get register
			Status = Dtu2xxIoCtlReadReg( pFdo, RegAddr+DTU2XX_RX_BASE_ADDR, pReg );
			if ( Status!=0 )
				return DTU2XX_E;

#if LOG_LEVEL > 0
			DTU2XX_LOG3( KERN_INFO,
						"Caching receive-register: 0x%08X, pReg=0x%p, Value: 0x%08X",
						 RegAddr+DTU2XX_RX_BASE_ADDR, pReg, (unsigned int)*pReg );
#endif

			// Move to next register in map
			pReg++;
		}
	}
	return DTU2XX_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxAllocRingBuffer -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
DTU2XX_RESULT Dtu2xxAllocRingBuffer(
	DTU2XX_RING_BUFFER* pRingBuf,
	Int Size)
{
	// Init all members to 0
	pRingBuf->m_pStart		= NULL;
	pRingBuf->m_pRdPtr		= NULL;
	pRingBuf->m_pWrPtr		= NULL;
	pRingBuf->m_TotalSize	= 0;
	pRingBuf->m_Load		= 0;

	if ( Size == 0 )
		return DTU2XX_E;

	// Allocate the memory
	pRingBuf->m_pStart = kmalloc( Size, GFP_KERNEL );
	if ( pRingBuf->m_pStart == NULL )
	{
		DTU2XX_LOG( KERN_INFO, "Dtu2xxAllocRingBuffer: Failed to allocate memory "
                                       "for the ring buffer" );
		return DTU2XX_E;
	}

#if LOG_LEVEL > 0
	DTU2XX_LOG1( KERN_INFO, "Dtu2xxAllocRingBuffer: Allocated %d bytes for the ring "
                                "buffer", Size );
#endif

	// Set size
	pRingBuf->m_TotalSize = Size;
	// Init pointers
	pRingBuf->m_pRdPtr = pRingBuf->m_pWrPtr = pRingBuf->m_pStart;

	return DTU2XX_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xx8051Reset -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
DTU2XX_RESULT Dtu2xx8051Reset(
	DTU2XX_FDO* pFdo,
	UInt8 ResetBit)
{
	Int Status = 0, BytesTransf=0;

#if LOG_LEVEL > 0
	DTU2XX_LOG( KERN_INFO, "Dtu2xx8051Reset is called");
#endif

	Status = Dtu2xxVendorRequest(
					pFdo,
					ANCHOR_LOAD_INTERNAL, CPUCS_REG_FX2, 0,
					DTU2XX_DIRECTION_WRITE,
					sizeof(ResetBit), (UInt8*)(&ResetBit), &BytesTransf);
	if ( Status != 0 )
		return DTU2XX_E;

	return DTU2XX_OK;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxProgramEzUsb -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
DTU2XX_RESULT Dtu2xxProgramEzUsb(
	DTU2XX_FDO* pFdo,
	const DTU2XX_INTEL_HEX_RECORD* pFirmware)
{
	Int Status = 0, BytesTransf=0;
	DTU2XX_INTEL_HEX_RECORD* pPtr;
	UInt8* pBuf = NULL;

#if LOG_LEVEL > 0
	DTU2XX_LOG( KERN_INFO, "DEBUG: Dtu2xxProgramEzUsb is called" );
#endif

	// Helper buffer for usb_control_msg
	pBuf = (UInt8 *)kmalloc( MAX_INTEL_HEX_RECORD_LENGTH, GFP_KERNEL );
	if ( pBuf == NULL )
		return DTU2XX_E;

	//.-.-.-.-.-.-.-.-.-.-.-.- Download firmware to internal RAM -.-.-.-.-.-.-.-.-.-.-.-.-
	//
    // Before starting the download, stop the 8051

	DTU2XX_LOG1( KERN_INFO, "Uploading EzUsb firmware for DTU-%d...", pFdo->m_TypeNumber);

	Dtu2xx8051Reset(pFdo, 1);
	pPtr = (DTU2XX_INTEL_HEX_RECORD*)pFirmware;
	while( pPtr->Type == 0 )
	{
		// Load firmware into EzUsb device using a buffer, 'usb_control_msg' cannot handle
		// statically allocated arrays as request data.
		memcpy( pBuf, pPtr->Data, pPtr->Length );

		// Send load command
		Status = Dtu2xxVendorRequest(
					pFdo,
					ANCHOR_LOAD_INTERNAL, pPtr->Address, 0,
					DTU2XX_DIRECTION_WRITE,
					pPtr->Length, pBuf, &BytesTransf);
		// Stop on an error
		if ( Status != 0 )
			break;
		pPtr++;
	}
	// Free our helper buffer
	kfree( pBuf );

	// On success: restart the 8051
	if( Status == 0 ) {
		Dtu2xx8051Reset( pFdo, 0 );
		return DTU2XX_OK;
	}
	return DTU2XX_E;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxProgramAltera -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
DTU2XX_RESULT Dtu2xxProgramAltera(
	DTU2XX_FDO* pFdo,
	Int BufSize,
	UInt8* pFirmware)
{
	UInt8 BitTest;
	UInt Total, i, l, BitNr;
	UInt16 WordBits[1024];
	Int Status = 0, BytesTransf=0;

	Total=0;
	BitNr=0;

	//.-.-.-.-.-.-.-.-.-.- Reset the ALTERA and activate program mode -.-.-.-.-.-.-.-.-.-.
	//
	// Toggles nConfig on the ALTERA

	Status = Dtu2xxVendorRequest(
					pFdo,
					DTU2XX_USB_SET_PROG_MODE, DTU2XX_STOP_PROG_CYCLE, 0,
					DTU2XX_DIRECTION_WRITE,
					0, NULL, &BytesTransf);
    if ( Status != 0 )
        return DTU2XX_E;
	Status = Dtu2xxVendorRequest(
					pFdo,
					DTU2XX_USB_SET_PROG_MODE, DTU2XX_START_PROG_CYCLE, 0,
					DTU2XX_DIRECTION_WRITE,
					0, NULL, &BytesTransf);
    if ( Status != 0 )
        return DTU2XX_E;

	//-.-.-.-.-.-.-.-.-.- Read all bytes and send them to the DTU-2xx -.-.-.-.-.-.-.-.-.-.

	// Add 10 bytes extra to have 80 (=10*8) extra clock cycles to init Altera
	for ( i=0; i<BufSize+10; i++)
	{
		if ( i<BufSize )
			BitTest = pFirmware[i];
		else
			BitTest = 0; // Do not really care about data here; we only want some extra
						 // clock cycles to make sure the ALTERA intialises

		// Generate clock data waveform for byte

		for ( l=0; l<8; l++)
		{
			if (BitTest&0x01)
			{
				WordBits[BitNr]=1;
				WordBits[BitNr+1]=3;
			}
			else
			{
				WordBits[BitNr]=0;
				WordBits[BitNr+1]=2;
			}
			BitNr+=2;
			BitTest>>=1;
		}

		// Max number of bits (send data to DTU-2xx)
		if (BitNr==1024)
                {
                        Status = Dtu2xxSimpleBulkTransfer(pFdo, DTU2XX_DIRECTION_WRITE, EZUSB_ENDPOINT_2,
                                                          (BitNr*2), (UInt8*)WordBits);

			if ( Status != 0)
				return DTU2XX_E;
			BitNr=0;
		}
	}
        // Write last chunk
        if ( BitNr > 0 )
        {
          Status = Dtu2xxSimpleBulkTransfer(pFdo, DTU2XX_DIRECTION_WRITE, EZUSB_ENDPOINT_2,
                                            (BitNr*2), (UInt8*)WordBits);
          if ( Status != 0)
		return DTU2XX_E;
        }

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Signal end of program -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	Status = Dtu2xxVendorRequest(
					pFdo,
					DTU2XX_USB_PROGRAM_DONE, 1, 0,
					DTU2XX_DIRECTION_WRITE,
					0, NULL, &BytesTransf);
    if ( Status != 0)
        return DTU2XX_E;

    return DTU2XX_OK;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxCheckFirmwareLevel -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
Int Dtu2xxCheckFirmwareLevel(
	DTU2XX_FDO* pFdo)
{
	Int FwLevel = DTU2XX_FWLEVEL_NONE;
	Int ret = 0;
	UInt8* pBuf = NULL;

	// Allocate memory for helper buffer
	pBuf = (UInt8 *)kmalloc( 128, GFP_KERNEL );
	if ( pBuf == NULL )
		return DTU2XX_FWLEVEL_NONE;

	// Get the string descriptor
	ret = usb_get_descriptor( pFdo->m_pUDev, USB_DT_STRING, 1, pBuf, 128 );

	// Check if usb_get_descriptor succeeded; string descriptor must be of size >3 bytes
	if ( ret >= 3 ) {
		// Assume EzUsb firmware is loaded
		FwLevel = DTU2XX_FWLEVEL_FX2_ONLY;
	}
	else
		FwLevel = DTU2XX_FWLEVEL_NONE;

	// Free allocated memory
	kfree( pBuf );
	return FwLevel;
}
