//*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* Dtu2xxMod.c *#*#*#*#*#*#*#*#* (C) 2000-2006 DekTec
//
// This code is the intellectual property of DekTec Digital Video B.V. and may not be
// distributed, in its original or in altered form, without the explicit permission of
// DekTec Digital Video B.V.

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//  2007.01.12	KvR	Fix error on restart of firmware on kernels >2.6.18
//  2006.10.01	MG	Clean-up of code
//  2006.08.14	KvR	Changed to new interface, added support for DTU-245, several bugfixes
//	2005.03.17	MG	Removed max usage m_Count
//	2004.11.24	MG	Created


//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#include "Dtu2xx.h"
#include "Dtu2xxRegs.h"
#include <linux/device.h>
#include <linux/usb.h>
#include <asm/uaccess.h>
#include <linux/firmware.h>
#include <linux/version.h>
#include "EzUsb.h"

// Cypress EzUsb firmware
#include "Cypress205.h"
#include "Cypress225.h"
#include "Cypress245.h"


//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- USB device ID structure -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// This structure makes hotplug associate the driver with the right device ID's.
//
// NOTE: some deivces have two ID: 1) for no firmware loaded 2) for firmware loaded
//
static struct usb_device_id usb_dtu2xx_ids[] = {
    { USB_DEVICE(DTU2XX_VENDORID, PID_DTU205_NOFIRMWARE) },
    { USB_DEVICE(DTU2XX_VENDORID, PID_DTU225_NOFIRMWARE) },
    { USB_DEVICE(DTU2XX_VENDORID, PID_DTU205) },
    { USB_DEVICE(DTU2XX_VENDORID, PID_DTU225) },
    { USB_DEVICE(DTU2XX_VENDORID, PID_DTU245) },
    {}
};
MODULE_DEVICE_TABLE(usb, usb_dtu2xx_ids);

static struct usb_driver dtu2xx_driver =
{
	/* Only for pre-2.6.16 kernels. The '.owner' field has been removed. */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 16)
    .owner = THIS_MODULE,
#endif
	.name = "dtu2xx",
	.probe = Dtu2xxProbe,
	.disconnect = Dtu2xxDisconnect,
	.id_table = usb_dtu2xx_ids,
};

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ File operation structur +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

struct file_operations dtu2xx_fops =
{
    .owner = THIS_MODULE,
    .llseek = no_llseek,
    .read = Dtu2xxRead,
    .write = Dtu2xxWrite,
    .ioctl = Dtu2xxIoCtl,
    .open = Dtu2xxOpen,
    .release = Dtu2xxRelease,
};

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ USB classes +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

// DTU-205
static struct usb_class_driver dtu205_class =
{
    .name = "usb/dektec/dtu2xx-%d",
    .fops = &dtu2xx_fops,
	/* Only for pre-2.6.12 kernels. The '.mode' field has been removed. */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 12)
    .mode = S_IFCHR | S_IWUSR | S_IWGRP | S_IWOTH,
#endif
    .minor_base = DTU2XX_MINOR,
};

// DTU-225
static struct usb_class_driver dtu225_class =
{
    .name = "usb/dektec/dtu2xx-%d",
    .fops = &dtu2xx_fops,
	/* Only for pre-2.6.12 kernels. The '.mode' field has been removed. */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 12)
    .mode = S_IFCHR | S_IRUSR | S_IRGRP | S_IROTH,
#endif
    .minor_base = DTU2XX_MINOR,
};

// DTU-245
static struct usb_class_driver dtu245_class =
{
    .name = "usb/dektec/dtu2xx-%d",
    .fops = &dtu2xx_fops,
	/* Only for pre-2.6.12 kernels. The '.mode' field has been removed. */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 12)
    .mode = S_IFCHR | S_IRUSR | S_IRGRP | S_IROTH | S_IWOTH,
#endif
    .minor_base = DTU2XX_MINOR,
};

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ File operations +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxOpen -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Dtu2xxOpen - open a device; called upon a open systemcall
//
// registered as file operation
// a device can only be opened by 1 process
//
int Dtu2xxOpen(
    struct inode* inode,
    struct file *file)
{
    Int minor;
    struct usb_interface *pUsbItf;
    DTU2XX_FDO *pFdo;

#if LOG_LEVEL > 0
	DTU2XX_LOG( KERN_INFO, "Dtu2xxOpen is called" );
#endif

    minor = iminor(inode);

    pUsbItf = usb_find_interface(&dtu2xx_driver, minor);
    if (!pUsbItf) {
		DTU2XX_LOG( KERN_INFO, "usb_find_interface failed" );
		return -ENODEV;
	}

    pFdo = usb_get_intfdata(pUsbItf);
    if (!pFdo) {
		DTU2XX_LOG( KERN_INFO, "usb_get_intfdata failed" );
		return -ENODEV;
	}

//    if (!atomic_dec_and_test(&pFdo->m_Count)) {
//        // device busy
//        atomic_inc(&pFdo->m_Count);
//        return -EBUSY;
//    }

    Dtu2xxIncrUsageCount(pFdo);
    file->private_data = pFdo;

    return 0;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxRelease -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Dtu2xxRelease - close device; called upon close systemcall
//
int Dtu2xxRelease(
    struct inode* inode,
    struct file *file)
{
    DTU2XX_FDO *pFdo;

#if LOG_LEVEL > 0
	DTU2XX_LOG( KERN_INFO, "Dtu2xxRelease is called" );
#endif

    pFdo = (DTU2XX_FDO*)(file->private_data);
    if (!pFdo) { return -ENODEV; }
    Dtu2xxDecrUsageCount(pFdo);
    //atomic_inc(&pFdo->m_Count);
    return 0;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxRead -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Dtu2xxRead - bulk read from device; called upon a read systemcall
//
ssize_t Dtu2xxRead(
    struct file *file,
    char __user *pBuf,
    size_t TransferSize,
    loff_t *ppos)
{
	Int pBytesRead = 0;
    DTU2XX_FDO* pFdo = (DTU2XX_FDO*)(file->private_data);

#if LOG_LEVEL > 2
	DTU2XX_LOG( KERN_INFO, "Dtu2xxRead is called" );
#endif

	if ( Dtu2xxRxIoCtlBulkRead( pFdo, 0, TransferSize, &pBytesRead, (UInt8*)pBuf ) != 0 ) {
		// Read error!
		return -EIO;
	}

	// Read ok. Return the amount of bytes read.
	return pBytesRead;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxWrite -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Dtu2xxWrite - bulk write to device; called upon a write systemcall
//
ssize_t Dtu2xxWrite(
    struct file *file,
    const char __user *pBuf,
    size_t TransferSize,
    loff_t *ppos)
{
	DTU2XX_FDO* pFdo = (DTU2XX_FDO*)(file->private_data);

#if LOG_LEVEL > 2
	DTU2XX_LOG( KERN_INFO, "Dtu2xxWrite is called" );
#endif

	if ( Dtu2xxTxIoCtlBulkWrite( pFdo, 0, TransferSize, (UInt8*)pBuf ) != 0 ) {
		// Write error!
		return -EIO;
	}

	// Write ok. Return amount of transfered bytes.
	return TransferSize;
}

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Device operations +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxProbe -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Dtu2xxProbe - probe for a new device; called when a usb-device is inserted
//
int Dtu2xxProbe(
    struct usb_interface *pUsbItf,
    const struct usb_device_id *pUsbDevId)
{
	Int ret=0;
    DTU2XX_FDO* pFdo=NULL;
	struct usb_device_descriptor* pDvcDesc=NULL;


#if LOG_LEVEL > 0
	DTU2XX_LOG( KERN_INFO, "Dtu2xxProbe is called" );
#endif

	// We must have a DekTec device
	if (pUsbDevId->idVendor != DTU2XX_VENDORID)
		return -EBADR;

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Create FDO object -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

	// Allocate memory for FDO object
	pFdo = kmalloc(sizeof(DTU2XX_FDO), GFP_KERNEL);
    if ( pFdo == NULL )
		return -ENOMEM;

	// Init FDO object to initial 'safe' value
	memset( pFdo, 0, sizeof(DTU2XX_FDO) );

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,9)
    kref_init(&pFdo->m_KRef, Dtu2xxDevDelete);
#else
    kref_init(&pFdo->m_KRef);	// 2.6.9
#endif

	// Set device and interface members
	pFdo->m_pUDev		= usb_get_dev(interface_to_usbdev(pUsbItf));
    pFdo->m_pUsbItf		= pUsbItf;
	usb_set_intfdata(pUsbItf, pFdo);

	// Get pointer to the device descriptor
	pDvcDesc = &(pFdo->m_pUDev->descriptor);

	//.-.-.-.-.-.-.-.-.-.-.- Check which device we are dealing with -.-.-.-.-.-.-.-.-.-.-.

        // Check for new 2x5 hardware ( product version >=5 => DTU-2x5)
        if ( pDvcDesc->bcdDevice >= 5 )
          pFdo->m_IsDtu2x5Hw = 1;
        else
          pFdo->m_IsDtu2x5Hw = 0;


	switch (pDvcDesc->idProduct)
    {

	case PID_DTU205:
	case PID_DTU205_NOFIRMWARE:
		pFdo->m_TypeNumber = 205;
		pFdo->m_pUsbClass = &dtu205_class;

		// Check current firmware level
		pFdo->m_FirmwareLevel = Dtu2xxCheckFirmwareLevel(pFdo);
		break;

	case PID_DTU225:
	case PID_DTU225_NOFIRMWARE:
		pFdo->m_TypeNumber = 225;
		pFdo->m_pUsbClass = &dtu225_class;

		// Check current firmware level
		pFdo->m_FirmwareLevel = Dtu2xxCheckFirmwareLevel(pFdo);
		break;

	case PID_DTU245:
		pFdo->m_TypeNumber = 245;
		pFdo->m_pUsbClass = &dtu245_class;

		// Check current firmware level
		pFdo->m_FirmwareLevel = Dtu2xxCheckFirmwareLevel(pFdo);
		break;

	default:
		// Unknown device
		pFdo->m_TypeNumber = 0;
		break;
	}
	DTU2XX_LOG2( KERN_INFO, "Found a DTU-%d (Is 2x5: %d)",
                     pFdo->m_TypeNumber, pFdo->m_IsDtu2x5Hw );

	//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Init USB specific -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
	ret = usb_register_dev(pUsbItf, pFdo->m_pUsbClass);
	if ( ret != 0 )
		goto out;

	//-.-.-.-.-.-.-.-.-.-.- Download EzUsb (FX2) firmware is needed -.-.-.-.-.-.-.-.-.-.-.

	if ( pFdo->m_FirmwareLevel < DTU2XX_FWLEVEL_FX2_ONLY )
	{
		DTU2XX_LOG1( KERN_INFO, "Program FX2 firmware (DTU-%d)", pFdo->m_TypeNumber);

		switch ( pFdo->m_TypeNumber )
		{
		case 205: ret=Dtu2xxProgramEzUsb(pFdo, c_CypressCode205); break;
		case 225: ret=Dtu2xxProgramEzUsb(pFdo, c_CypressCode225); break;
		case 245: ret=Dtu2xxProgramEzUsb(pFdo, c_CypressCode245); break;
		default:  break;
		}

		if ( ret != 0 )
		{
			DTU2XX_LOG(KERN_INFO, "Failed to program EzUsb firmware!");
			goto out;
		}
		pFdo->m_FirmwareLevel = DTU2XX_FWLEVEL_FX2_ONLY;

		// We will now re-enumerate (i.e. device will be disconnected and reconnected
		// 	with a new PID)
		return 0;
	}

	//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Init the device -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.

	ret = Dtu2xxDeviceInit(pFdo);
	if ( ret != 0 )
		goto out;

    DTU2XX_LOG2( KERN_INFO, "Found DTU-%d device, minor nb: %d",
                 pFdo->m_TypeNumber, pFdo->m_pUsbClass->minor_base);
	DTU2XX_LOG4( KERN_INFO, "Dtu2xx driver (v%d.%d.%d.%d) started sucessfully",
				 DTU2XX_VERSION_MAJOR, DTU2XX_VERSION_MINOR, DTU2XX_VERSION_MICRO,
				 DTU2XX_VERSION_BUILD );

    return ret;
out:
    Dtu2xxDecrUsageCount(pFdo);
    return ret;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxDisconnect -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Dtu2xxDisconnect - disconnect device, called when the usb-device is disconnected.
//
void Dtu2xxDisconnect(
    struct usb_interface *pUsbItf)
{
    DTU2XX_FDO *pFdo;

#if LOG_LEVEL > 0
	DTU2XX_LOG( KERN_INFO, "Dtu2xxDisconnect is called" );
#endif

    /* Prevent Dtu2xxOpen from racing Dtu2xxDisconnect by a big kernel lock */
    lock_kernel();

    pFdo = usb_get_intfdata(pUsbItf);
    usb_set_intfdata(pUsbItf, NULL);
    usb_deregister_dev(pUsbItf, pFdo->m_pUsbClass);

    unlock_kernel();

    Dtu2xxDecrUsageCount(pFdo);
}

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Module opertations +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxInitModule -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// module init; called when the driver is loaded
//
int __init Dtu2xxInitModule(void)
{
#if LOG_LEVEL > 0
	DTU2XX_LOG( KERN_INFO, "Dtu2xxInitModule is called" );
#endif

    return usb_register(&dtu2xx_driver);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxCleanupModule -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// module cleanup; called when the driver is unloaded
//
void __exit Dtu2xxCleanupModule(void)
{
#if LOG_LEVEL > 0
	DTU2XX_LOG( KERN_INFO, "Dtu2xxCleanupModule is called" );
#endif

    usb_deregister(&dtu2xx_driver);
}

module_init(Dtu2xxInitModule);
module_exit(Dtu2xxCleanupModule);

MODULE_AUTHOR("DekTec Digital Video B.V.");
MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("Hardware driver for Dtu2xx usb-devices");

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Internal helpers +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxIncrUsageCount -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
//
// Dtu2xxIncrUsageCount -- increment use m_UsafeCount of this device
//
// Must be called when this devices is opened.
// Do not call this after creation.
//
void Dtu2xxIncrUsageCount(
    DTU2XX_FDO* pFdo)
{
    kref_get(&pFdo->m_KRef);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxDecrUsageCount -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Dtu2xxDecrUsageCount -- decrease use m_Count of this device
//
// Must be called when this device is closed or cleaned up (disconnect).
// When the reference m_Count = 0, all structures used will be freed.
//
void Dtu2xxDecrUsageCount(
    DTU2XX_FDO* pFdo)
{
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,9)
    kref_put(&pFdo->m_KRef);
#else
    kref_put(&pFdo->m_KRef, Dtu2xxDevDelete); // >2.6.9
#endif
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxDevDelete -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Dtu2xxDevDelete -- delete device structure
//
// Will be called when reference m_UsageCount = 0
//
void Dtu2xxDevDelete(
    struct kref* kref)
{
	Int i;
    DTU2XX_FDO* pFdo = NULL;
	DTU2XX_CHANNEL* pChan = NULL;

	// Get FDO pointer
    pFdo = container_of(kref, DTU2XX_FDO, m_KRef);
	if ( pFdo == NULL )
		return;

	DTU2XX_LOG1( KERN_INFO, "Performing final USB device clean-up (DTU-%d)", pFdo->m_TypeNumber );

	// Free channel resources
	for ( i=0; i<pFdo->m_NumTxChannels+pFdo->m_NumRxChannels; i++)
	{
		if ( i<pFdo->m_NumTxChannels )
		{
			pChan = &pFdo->m_TsTx[i];
			pChan->m_ChanType = DTU2XX_TX_CHANNEL;
		}
		else
		{
			pChan = &pFdo->m_TsRx[i-pFdo->m_NumTxChannels];
			pChan->m_ChanType = DTU2XX_RX_CHANNEL;
		}

		// Free temp buffer
		if ( pChan->m_pTempBuf != NULL )
		{
			kfree(pChan->m_pTempBuf);
			pChan->m_pTempBuf = NULL;
			pChan->m_TempBufSize = 0;
		}
		// Free ring buffer
		if ( pChan->m_RingBuf.m_pStart != NULL )
		{
			kfree(pChan->m_RingBuf.m_pStart);
			pChan->m_RingBuf.m_pStart = NULL;
			pChan->m_RingBuf.m_TotalSize = 0;
		}
	}

	// Release link to USB device
	usb_put_dev(pFdo->m_pUDev);

	// Finally free the FDO object
    kfree(pFdo);
}
