//*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* GenUsb.c *#*#*#*#*#*#*#*#*# (C) 2000-2006 DekTec
//
// This code is the intellectual property of DekTec Digital Video B.V. and may not be
// distributed, in its original or in altered form, without the explicit permission of
// DekTec Digital Video B.V.

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//  2006.10.01	MG	Clean-up of code
//  2006.08.14	KvR	Changed to new interface, added support for DTU-245, several bugfixes
//  2004.11.24  MG  Created

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#include "Dtu2xx.h"
#include "EzUsb.h"


//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxVendorRequest -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
DTU2XX_RESULT Dtu2xxVendorRequest(
	DTU2XX_FDO* pFdo,			// The device object
	UInt8 RequestCode,			// Request code
	UInt16 RequestValue,		// Request specific value
	UInt16 Index,				// Specifies the device-defined index
	Int Dir,					// Transfer direction
	Int BufSize,				// Size of buffer
	UInt8* pBuffer,				// Data buffer
	Int* pBytesTransf)			// Number of bytes transferred
{
	Int Ret, RequestType, Pipe;

	// Init to 'safe' value
	*pBytesTransf = 0;

	if ( Dir == DTU2XX_DIRECTION_READ )	{
		RequestType = USB_TYPE_VENDOR | USB_DIR_IN | USB_RECIP_DEVICE;
		Pipe = usb_rcvctrlpipe(pFdo->m_pUDev, 0);
	} else {
		RequestType = USB_TYPE_VENDOR | USB_DIR_OUT | USB_RECIP_DEVICE;
		Pipe = usb_sndctrlpipe(pFdo->m_pUDev, 0);
	}

	Ret = usb_control_msg(
				pFdo->m_pUDev, Pipe,
				RequestCode,
				RequestType,
				RequestValue,
				Index,
				(char*)pBuffer,
				BufSize,
				10000);

	if ( Ret >= 0 ) {
		*pBytesTransf = Ret;
		return DTU2XX_OK;
	}
	return DTU2XX_E;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxSimpleBulkTransfer -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
DTU2XX_RESULT Dtu2xxSimpleBulkTransfer(
	DTU2XX_FDO* pFdo,			// The functional device object
	Int Direction,				// Transfer direction
	Int EndPoint,				// The end point number
	Int BufSize,				// Size of the data buffer (i.e. number of bytes to transfer)
	UInt8* pData)				// Data buffer
{
	Int Ret, Pipe=-1, NumBytesTransf=0;

#if LOG_LEVEL_DMA > 2
	DTU2XX_LOG2( KERN_INFO, "Dtu2xxSimpleBulkTransfer: BufSize=%d, EndPoint=%d",
                     BufSize, EndPoint );
#endif

	// Nothing to transfer
	if ( BufSize == 0 )
		return DTU2XX_OK;

	//.-.-.-.-.-.-.-.-.-.-.-.- Check if we have a valid end-point -.-.-.-.-.-.-.-.-.-.-.-.

	switch (EndPoint)
	{
	case EZUSB_ENDPOINT_0:
		// Not allowed to send data to EP0
		return DTU2XX_E;

	case EZUSB_ENDPOINT_2:
		// Supported:  By all DTU-2xx devices
		Pipe = usb_sndbulkpipe(pFdo->m_pUDev, EndPoint);
		break;

	case EZUSB_ENDPOINT_4:
	case EZUSB_ENDPOINT_8:
		// Not support by DTU-2xx devices
		DTU2XX_LOG1( KERN_INFO, "Invalid end-point (%d)", EndPoint );
		return DTU2XX_E;

	case EZUSB_ENDPOINT_6:
		// Supported:  DTU-225, DTU-234, DTU-235 and DTU-245
		if (   pFdo->m_TypeNumber!=225 && pFdo->m_TypeNumber!=234
                    && pFdo->m_TypeNumber!=235 && pFdo->m_TypeNumber!=245 )
		{
			DTU2XX_LOG2( KERN_INFO, "Invalid end-point (%d) for DTU-%d", EndPoint,
						 pFdo->m_TypeNumber );
			return DTU2XX_E;
		}
		Pipe = usb_rcvbulkpipe(pFdo->m_pUDev, EndPoint);
		break;

	default:
		return DTU2XX_E;
	}

	// Transfer data to/from pipe
	Ret = usb_bulk_msg(pFdo->m_pUDev, Pipe, pData,  BufSize, &NumBytesTransf, 2000);
	if ( Ret != 0 )
		return DTU2XX_E;
	return DTU2XX_OK;
}
