//*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* IoCtl.c *#*#*#*#*#*#*#*#*#* (C) 2000-2006 DekTec
//
// This code is the intellectual property of DekTec Digital Video B.V. and may not be
// distributed, in its original or in altered form, without the explicit permission of
// DekTec Digital Video B.V.

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//  2006.10.01	MG	Clean-up of code
//  2006.08.14	KvR	Changed to new interface, added support for DTU-245, several bugfixes
//	2003.06.16	MG	Added missing calls to RxIoCtl commands (GetFlags and ClearFlags)
//	2003.05.09	MG	Added get firmware version command
//	2003.04.23	MG	Added VPD read and write commands
//	2002.10.21	MG	Created


//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#include "Dtu2xx.h"
#include "Dtu2xxRegs.h"
#include <asm/uaccess.h>

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Type definitions -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Internal function declarations -.-.-.-.-.-.-.-.-.-.-.-.-.-.

//-.-.-.-.-.-.-.-.-.-.-.-.-.- Data declaration and definition -.-.-.-.-.-.-.-.-.-.-.-.-.-.

//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Implementation +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxIoCtl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// IO control handler function. Dispatches an IO control command to the appropriate
// handler
//
Int Dtu2xxIoCtl(
    struct inode *inode,	// 
    struct file *filp,		// 
    UInt cmd,		// Command
    UInt32 arg)		// Pointer to the IoCtl parameter 
{
    DTU2XX_RESULT ret=DTU2XX_OK;
    DTU2XX_IOCTL_DATA  io;
    DTU2XX_FDO* pFdo = (DTU2XX_FDO*)filp->private_data;
	Int pBytesRead = 0;

    // Check command that this is one of our commands
    if ( DTU2XX_IOCTL_MAGIC!=_IOC_TYPE(cmd) ) { return DTU2XX_E; }

    // Check memory
    if (  _IOC_DIR(cmd)==_IOC_READ) {
        if ( 0==access_ok(VERIFY_WRITE, (void*)arg, _IOC_SIZE(cmd)) ) {
            return DTU2XX_E_DEV_DRIVER;
        }
    }
    if ( _IOC_DIR(cmd)==_IOC_WRITE ) {
        if ( 0==access_ok(VERIFY_READ, (void*)arg, _IOC_SIZE(cmd)) ) {
            return DTU2XX_E_DEV_DRIVER;
        }
    }

    //.-.-.-.-.-.-.-.-.-.-.-.-.-.- Copy IoCtl data from user -.-.-.-.-.-.-.-.-.-.-.-.-.-.-

    if ( copy_from_user(&io, (DTU2XX_IOCTL_DATA*)arg, sizeof(DTU2XX_IOCTL_DATA)) ) {
		DTU2XX_LOG( KERN_DEBUG, "Could not copy data from user!" );
		return DTU2XX_E_DEV_DRIVER;
	}

    //.-.-.-.-.-.-.-.-.-.-.-.-.- Dispatch/Handle ioctl command -.-.-.-.-.-.-.-.-.-.-.-.-.-
    //
    switch (cmd)
    {

        //=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ General IO control +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

        //-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_IOCTL_GET_DRIVER_VERSION -.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_IOCTL_GET_DRIVER_VERSION:
            io.m_Data.m_DriverVersion.m_nVersionMajor	= DTU2XX_VERSION_MAJOR;
            io.m_Data.m_DriverVersion.m_nVersionMinor	= DTU2XX_VERSION_MINOR;
            io.m_Data.m_DriverVersion.m_nVersionMicro	= DTU2XX_VERSION_MICRO;
            io.m_Data.m_DriverVersion.m_nVersionBuild	= DTU2XX_VERSION_BUILD;
            if ( copy_to_user((DTU2XX_IOCTL_DATA*)arg, &io, sizeof(DTU2XX_IOCTL_DATA)) ) {
				DTU2XX_LOG( KERN_DEBUG, "Could not copy data to user!" );
				return DTU2XX_E_DEV_DRIVER;
			}
            break;

        //.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_IOCTL_GET_LOCATION_INFO -.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_IOCTL_GET_LOCATION_INFO:
            io.m_Data.m_LocInfo.m_nVendorId		= pFdo->m_pUDev->descriptor.idVendor;
            io.m_Data.m_LocInfo.m_nDeviceId		= pFdo->m_pUDev->descriptor.idProduct;

            // Sub_vendorid and sub_systemid have no real meaning for USB devices
            // Encode sub_systemid to match fromat for DTA-1xx cards
            io.m_Data.m_LocInfo.m_nSubVendorId	= 0;
            io.m_Data.m_LocInfo.m_nSubSystemId	= 0xD100 + pFdo->m_TypeNumber - 100;
            io.m_Data.m_LocInfo.m_nUsbAddress   = pFdo->m_pUDev->devnum;
            if ( copy_to_user((DTU2XX_IOCTL_DATA*)arg, &io, sizeof(DTU2XX_IOCTL_DATA)) ) {
				DTU2XX_LOG( KERN_DEBUG, "Could not copy data to user!" );
				return DTU2XX_E_DEV_DRIVER;
			}
            break;

        //.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_IOCTL_GET_DEVICE_INFO -.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_IOCTL_GET_DEVICE_INFO:
            Dtu2xxIoCtlGetDeviceInfo(pFdo, &io.m_Data.m_DevInfo.m_nAlteraFwVersion,
                                     &io.m_Data.m_DevInfo.m_nDevType);
            if ( copy_to_user((DTU2XX_IOCTL_DATA*)arg, &io, sizeof(DTU2XX_IOCTL_DATA)) ) {
				DTU2XX_LOG( KERN_DEBUG, "Could not copy data to user!" );
				return DTU2XX_E_DEV_DRIVER;
			}
            break;

        //-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_IOCTL_GET_FW_VERSION -.-.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_IOCTL_GET_FW_VERSION:
            io.m_Data.m_uiValue	= pFdo->m_FirmwareVersion;
            if ( copy_to_user((DTU2XX_IOCTL_DATA*)arg, &io, sizeof(DTU2XX_IOCTL_DATA)) ) {
				DTU2XX_LOG( KERN_DEBUG, "Could not copy data to user!" );
				return DTU2XX_E_DEV_DRIVER;
			}
            break;

        //-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_IOCTL_LED_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
        //
        case DTU2XX_IOCTL_LED_CONTROL:
            ret = Dtu2xxIoCtlLedControl(pFdo, io.m_Data.m_uiValue);
            break;

        //-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_IOCTL_RESET_SOFT -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_IOCTL_RESET_SOFT:
            ret = Dtu2xxIoCtlResetSoft(pFdo);
            break;

        //.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_IOCTL_READ_VPD -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
        //
        case DTU2XX_IOCTL_READ_VPD:
            ret = Dtu2xxIoCtlReadVpd(pFdo,
                io.m_Data.m_Vpd.m_nStart,
                io.m_Data.m_Vpd.m_nLength,
                io.m_Data.m_Vpd.m_pVpdBuf);
            break;

        //.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_IOCTL_WRITE_VPD -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_IOCTL_WRITE_VPD:
            ret = Dtu2xxIoCtlWriteVpd(pFdo,
                io.m_Data.m_Vpd.m_nStart,
                io.m_Data.m_Vpd.m_nLength,
                io.m_Data.m_Vpd.m_pVpdBuf);
            break;

        //.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_IOCTL_DMA_USERMEM_READ -.-.-.-.-.-.-.-.-.-.-.-.-.-
        //
        case DTU2XX_IOCTL_DMA_USERMEM_READ:
            ret = Dtu2xxRxIoCtlDma(pFdo, io.m_nChIdx,
                io.m_Data.m_DataTransDesc.m_pBuffer,
                io.m_Data.m_DataTransDesc.m_nNumBytesToTr);
            break;

        //.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_IOCTL_DMA_USERMEM_WRITE -.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_IOCTL_DMA_USERMEM_WRITE:
            ret = Dtu2xxTxIoCtlDma(pFdo, io.m_nChIdx,
                io.m_Data.m_DataTransDesc.m_pBuffer,
                io.m_Data.m_DataTransDesc.m_nNumBytesToTr);
            break;

        //.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_IOCTL_READ_REG -.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_IOCTL_READ_REG:
            ret = Dtu2xxIoCtlReadReg(pFdo,
                io.m_Data.m_Register.address,
                (UInt32*)&(io.m_Data.m_Register.value));
            if ( copy_to_user((DTU2XX_IOCTL_DATA*)arg, &io, sizeof(DTU2XX_IOCTL_DATA)) ) {
				DTU2XX_LOG( KERN_DEBUG, "Could not copy data to user!" );
				return DTU2XX_E_DEV_DRIVER;
			}
            break;

       //.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_IOCTL_WRITE_REG -.-.-.-.-.-.-.-.-.-.-.-.-.
       //
        case DTU2XX_IOCTL_WRITE_REG:
            ret = Dtu2xxIoCtlWriteReg(pFdo,
                io.m_Data.m_Register.address,
                io.m_Data.m_Register.value);
            break;

        //=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Rx channel IO control +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+

        //-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_EQUALISE -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
        //
        case DTU2XX_RX_IOCTL_EQUALISE:
            ret = Dtu2xxRxIoCtlEqualise(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
            break;

        //.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_GET_FIFO_LOAD -.-.-.-.-.-.-.-.-.-.-.-.-.-
        //
        case DTU2XX_RX_IOCTL_GET_FIFO_LOAD:

            ret = Dtu2xxRxIoCtlGetFifoLoad(pFdo, io.m_nChIdx, &io.m_Data.m_uiValue);
            if ( copy_to_user((DTU2XX_IOCTL_DATA*)arg, &io, sizeof(DTU2XX_IOCTL_DATA)) ) {
				DTU2XX_LOG( KERN_DEBUG, "Could not copy data to user!" );
				return DTU2XX_E_DEV_DRIVER;
			}
            break;

        //.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_GET_FLAGS -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
        //
        case DTU2XX_RX_IOCTL_GET_FLAGS:
            ret = Dtu2xxRxIoCtlGetFlags(pFdo, io.m_nChIdx, 
                &io.m_Data.m_Flags.m_nStatus,
                &io.m_Data.m_Flags.m_nLatched);
            if ( copy_to_user((DTU2XX_IOCTL_DATA*)arg, &io, sizeof(DTU2XX_IOCTL_DATA)) ) {
				DTU2XX_LOG( KERN_DEBUG, "Could not copy data to user!" );
				return DTU2XX_E_DEV_DRIVER;
			}
            break;

        //-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_CLEAR_FLAGS -.-.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_RX_IOCTL_CLEAR_FLAGS:
            ret = Dtu2xxRxIoCtlClearFlags(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
            break;

        //.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_GET_STATUS -.-.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_RX_IOCTL_GET_STATUS:
            ret = Dtu2xxRxIoCtlGetStatus(pFdo, io.m_nChIdx,  
                &io.m_Data.m_RxStatus.m_nPacketSize,
                &io.m_Data.m_RxStatus.m_nClkDet,
                &io.m_Data.m_RxStatus.m_nAsiLock,
                &io.m_Data.m_RxStatus.m_nRateOk);

            if ( copy_to_user((DTU2XX_IOCTL_DATA*)arg, &io, sizeof(DTU2XX_IOCTL_DATA)) ) {
				DTU2XX_LOG( KERN_DEBUG, "Could not copy data to user!" );
				return DTU2XX_E_DEV_DRIVER;
			}
            break;

        //-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_GET_TARGETID -.-.-.-.-.-.-.-.-.-.-.-.-.-
        //
        case DTU2XX_RX_IOCTL_GET_TARGETID:
            ret = Dtu2xxRxIoCtlGetTargetId(pFdo, io.m_nChIdx, 
                &io.m_Data.m_TargetInfo.m_nPresent,
                &io.m_Data.m_TargetInfo.m_nTargetId);

            if ( copy_to_user((DTU2XX_IOCTL_DATA*)arg, &io, sizeof(DTU2XX_IOCTL_DATA)) ) {
				DTU2XX_LOG( KERN_DEBUG, "Could not copy data to user!" );
				return DTU2XX_E_DEV_DRIVER;
			}
            break;

        //-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_GET_TS_RATE_BPS -.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_RX_IOCTL_GET_RX_CLOCK:
            ret = Dtu2xxRxIoCtlGetTsRateBps(pFdo, io.m_nChIdx, &io.m_Data.m_uiValue);

            if ( copy_to_user((DTU2XX_IOCTL_DATA*)arg, &io, sizeof(DTU2XX_IOCTL_DATA)) ) {
				DTU2XX_LOG( KERN_DEBUG, "Could not copy data to user!" );
				return DTU2XX_E_DEV_DRIVER;
			}
            break;

        //-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_POLARITY_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-
        //
        case DTU2XX_RX_IOCTL_POLARITY_CONTROL:
            ret = Dtu2xxRxIoCtlPolarityControl(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
            break;

        //.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_RESET -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
        //
        case DTU2XX_RX_IOCTL_RESET:
            ret = Dtu2xxRxIoCtlReset(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
            break;

        //.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_SET_LOOPBACK_MODE -.-.-.-.-.-.-.-.-.-.-.-.-
        //
        case DTU2XX_RX_IOCTL_SET_LOOPBACK_MODE:
            ret = Dtu2xxRxIoCtlSetLoopBackMode(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
            break;

        //.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_SET_POWER -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
        //
        case DTU2XX_RX_IOCTL_SET_POWER:
            ret = Dtu2xxRxIoCtlSetPower(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
            break;

        //.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_SET_RX_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_RX_IOCTL_SET_RX_CONTROL:
            ret = Dtu2xxRxIoCtlSetRxControl(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
            break;

        //-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_SET_RX_MODE -.-.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_RX_IOCTL_SET_RX_MODE:
            ret = Dtu2xxRxIoCtlSetRxMode(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
            break;

        //-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_GET_RX_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_RX_IOCTL_GET_RX_CONTROL:
            ret = Dtu2xxRxIoCtlGetRxControl(pFdo, io.m_nChIdx, &io.m_Data.m_uiValue);
            if ( copy_to_user((DTU2XX_IOCTL_DATA*)arg, &io, sizeof(DTU2XX_IOCTL_DATA)) ) {
				DTU2XX_LOG( KERN_DEBUG, "Could not copy data to user!" );
				return DTU2XX_E_DEV_DRIVER;
			}
            break;

        //-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_GET_RX_MODE -.-.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_RX_IOCTL_GET_RX_MODE:
            ret = Dtu2xxRxIoCtlGetRxMode(pFdo, io.m_nChIdx, &io.m_Data.m_uiValue);
            if ( copy_to_user((DTU2XX_IOCTL_DATA*)arg, &io, sizeof(DTU2XX_IOCTL_DATA)) ) {
				DTU2XX_LOG( KERN_DEBUG, "Could not copy data to user!" );
				return DTU2XX_E_DEV_DRIVER;
			}
            break;

        //-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_LED_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_RX_IOCTL_LED_CONTROL:
            ret = Dtu2xxRxIoCtlLedControl(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
            break;

        //-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_CLEAR_FIFO -.-.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_RX_IOCTL_CLEAR_FIFO:
            ret = Dtu2xxRxIoCtlClearFifo(pFdo, io.m_nChIdx);
            break;

        //-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_GET_VIOL_COUNT -.-.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_RX_IOCTL_GET_VIOL_COUNT:
            ret = Dtu2xxRxIoCtlGetViolCount(pFdo, io.m_nChIdx, &io.m_Data.m_uiValue);
            if ( copy_to_user((DTU2XX_IOCTL_DATA*)arg, &io, sizeof(DTU2XX_IOCTL_DATA)) ) {
				DTU2XX_LOG( KERN_DEBUG, "Could not copy data to user!" );
				return DTU2XX_E_DEV_DRIVER;
			}
            break;

		//-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_RX_IOCTL_BULK_USERMEM_READ -.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_RX_IOCTL_BULK_USERMEM_READ:
            ret = Dtu2xxRxIoCtlBulkRead( pFdo, io.m_nChIdx,
					io.m_Data.m_DataTransDesc.m_nNumBytesToTr,
					&pBytesRead,
					io.m_Data.m_DataTransDesc.m_pBuffer );
            break;

        //=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Tx channel IO control +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+

        //-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_CLEAR_FIFO -.-.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_TX_IOCTL_CLEAR_FIFO:
            ret = Dtu2xxTxIoCtlClearFifo(pFdo, io.m_nChIdx);
            break;

        //-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_CLEAR_FLAGS -.-.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_TX_IOCTL_CLEAR_FLAGS:
            ret = Dtu2xxTxIoCtlClearFlags(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
            break;

        //-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_GET_EXT_CLK_FREQ -.-.-.-.-.-.-.-.-.-.-.-.-
        //
        case DTU2XX_TX_IOCTL_GET_EXT_CLK_FREQ:
            ret = Dtu2xxTxIoCtlGetExtClkFreq(pFdo, io.m_nChIdx, &io.m_Data.m_uiValue);
            if ( copy_to_user((DTU2XX_IOCTL_DATA*)arg, &io, sizeof(DTU2XX_IOCTL_DATA)) ) {
				DTU2XX_LOG( KERN_DEBUG, "Could not copy data to user!" );
				return DTU2XX_E_DEV_DRIVER;
			}
            break;

        //.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_GET_FIFO_LOAD -.-.-.-.-.-.-.-.-.-.-.-.-.-
        //
        case DTU2XX_TX_IOCTL_GET_FIFO_LOAD:
            ret = Dtu2xxTxIoCtlGetFifoLoad(pFdo, io.m_nChIdx, &io.m_Data.m_uiValue);
            if ( copy_to_user((DTU2XX_IOCTL_DATA*)arg, &io, sizeof(DTU2XX_IOCTL_DATA)) ) {
				DTU2XX_LOG( KERN_DEBUG, "Could not copy data to user!" );
				return DTU2XX_E_DEV_DRIVER;
			}
            break;

        //.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_GET_FLAGS -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
        //
        case DTU2XX_TX_IOCTL_GET_FLAGS:
            ret = Dtu2xxTxIoCtlGetFlags(pFdo, io.m_nChIdx,
                &io.m_Data.m_Flags.m_nStatus,
                &io.m_Data.m_Flags.m_nLatched);
            if ( copy_to_user((DTU2XX_IOCTL_DATA*)arg, &io, sizeof(DTU2XX_IOCTL_DATA)) ) {
				DTU2XX_LOG( KERN_DEBUG, "Could not copy data to user!" );
				return DTU2XX_E_DEV_DRIVER;
			}
            break;

        //.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_GET_MAX_FIFO_SIZE -.-.-.-.-.-.-.-.-.-.-.-.-
        //
        case DTU2XX_TX_IOCTL_GET_MAX_FIFO_SIZE:
            ret = Dtu2xxTxIoCtlGetMaxFifoSize(pFdo, io.m_nChIdx, &io.m_Data.m_uiValue);
            if ( copy_to_user((DTU2XX_IOCTL_DATA*)arg, &io, sizeof(DTU2XX_IOCTL_DATA)) ) {
				DTU2XX_LOG( KERN_DEBUG, "Could not copy data to user!" );
				return DTU2XX_E_DEV_DRIVER;
			}
            break;

        //.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_GET_TARGET_ID -.-.-.-.-.-.-.-.-.-.-.-.-.-
        //
        case DTU2XX_TX_IOCTL_GET_TARGET_ID:
            ret = Dtu2xxTxIoCtlGetTargetId(pFdo, io.m_nChIdx,
                &io.m_Data.m_TargetInfo.m_nPresent,
                &io.m_Data.m_TargetInfo.m_nTargetId);
            if ( copy_to_user((DTU2XX_IOCTL_DATA*)arg, &io, sizeof(DTU2XX_IOCTL_DATA)) ) {
				DTU2XX_LOG( KERN_DEBUG, "Could not copy data to user!" );
				return DTU2XX_E_DEV_DRIVER;
			}
            break;

        //-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_GET_TS_RATE_BPS -.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_TX_IOCTL_GET_TX_CLOCK:
            ret = Dtu2xxTxIoCtlGetTxClock(pFdo, io.m_nChIdx,
                &io.m_Data.m_RateInfo.m_nTxClock);
            if ( copy_to_user((DTU2XX_IOCTL_DATA*)arg, &io, sizeof(DTU2XX_IOCTL_DATA)) ) {
				DTU2XX_LOG( KERN_DEBUG, "Could not copy data to user!" );
				return DTU2XX_E_DEV_DRIVER;
			}
            break;

        //.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_GET_TX_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_TX_IOCTL_GET_TX_CONTROL:
            ret = Dtu2xxTxIoCtlGetTxControl(pFdo, io.m_nChIdx, &io.m_Data.m_uiValue);
            if ( copy_to_user((DTU2XX_IOCTL_DATA*)arg, &io, sizeof(DTU2XX_IOCTL_DATA)) ) {
				DTU2XX_LOG( KERN_DEBUG, "Could not copy data to user!" );
				return DTU2XX_E_DEV_DRIVER;
			}
            break;

        //-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_GET_TX_MODE -.-.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_TX_IOCTL_GET_TX_MODE:
            ret = Dtu2xxTxIoCtlGetTxMode(pFdo, io.m_nChIdx,
                &io.m_Data.m_TxMode.m_nTxPacketMode,
                &io.m_Data.m_TxMode.m_nTxStuffMode);
            if ( copy_to_user((DTU2XX_IOCTL_DATA*)arg, &io, sizeof(DTU2XX_IOCTL_DATA)) ) {
				DTU2XX_LOG( KERN_DEBUG, "Could not copy data to user!" );
				return DTU2XX_E_DEV_DRIVER;
			}
            break;

        //.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_RESET -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
        //
        case DTU2XX_TX_IOCTL_RESET:
            ret = Dtu2xxTxIoCtlReset(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
            break;

        //.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_SET_FIFO_SIZE -.-.-.-.-.-.-.-.-.-.-.-.-.-
        //
        case DTU2XX_TX_IOCTL_SET_FIFO_SIZE:
            ret = Dtu2xxTxIoCtlSetFifoSize(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
            break;

        //.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_SET_LOOPBACK_MODE -.-.-.-.-.-.-.-.-.-.-.-.-
        //
        case DTU2XX_TX_IOCTL_SET_LOOPBACK_MODE:
            ret = Dtu2xxTxIoCtlSetLoopBackMode(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
            break;

        //-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_RD_LOOPBACK_DATA -.-.-.-.-.-.-.-.-.-.-.-.-
        //
        case DTU2XX_TX_IOCTL_RD_LOOPBACK_DATA:
            ret = Dtu2xxTxIoCtlReadLoopBackData(pFdo, io.m_nChIdx,
                io.m_Data.m_DataTransDesc.m_pBuffer,
                io.m_Data.m_DataTransDesc.m_nNumBytesToTr);
            break;

        //-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_SET_TX_POLARITY -.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_TX_IOCTL_SET_TX_POLARITY:
            ret = Dtu2xxTxIoCtlSetTxPolarity(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
            break;

        //.-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_SET_POWER -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
        //
        case DTU2XX_TX_IOCTL_SET_POWER:
            ret = Dtu2xxTxIoCtlSetPower(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
            break;

        //-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_SET_TS_RATE_BPS -.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_TX_IOCTL_SET_TX_CLOCK:
            ret = Dtu2xxTxIoCtlSetTxClock(pFdo, io.m_nChIdx,
                io.m_Data.m_RateInfo.m_nTxClock);
            break;

        //.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_SET_TX_CONTROL -.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_TX_IOCTL_SET_TX_CONTROL:
            ret = Dtu2xxTxIoCtlSetTxControl(pFdo, io.m_nChIdx, io.m_Data.m_uiValue);
            break;

        //-.-.-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_SET_TX_MODE -.-.-.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_TX_IOCTL_SET_TX_MODE:
            ret = Dtu2xxTxIoCtlSetTxMode(pFdo, io.m_nChIdx,
                io.m_Data.m_TxMode.m_nTxPacketMode,
                io.m_Data.m_TxMode.m_nTxStuffMode);
            break;

		//-.-.-.-.-.-.-.-.-.-.-.- DTU2XX_TX_IOCTL_BULK_USERMEM_WRITE -.-.-.-.-.-.-.-.-.-.-.-.
        //
        case DTU2XX_TX_IOCTL_BULK_USERMEM_WRITE:
			ret = Dtu2xxTxIoCtlBulkWrite( pFdo, io.m_nChIdx,
				io.m_Data.m_DataTransDesc.m_nNumBytesToTr,
				io.m_Data.m_DataTransDesc.m_pBuffer );
            break;

        default:
            ret = DTU2XX_E_DEV_DRIVER;
            break;
    }
    return (Int)ret;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxIoCtlLedControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// IoCtl handler for the led control command 
//
DTU2XX_RESULT Dtu2xxIoCtlLedControl(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
    Int LedControl)		// LED control value
{
	// Get register cache
	Dtu2xxGen* pGenRegs = &(pFdo->m_GenRegs);

	// Check for valid value
	if (   LedControl != DTU2XX_LED_HARDWARE && LedControl != DTU2XX_LED_OFF
		&& LedControl != DTU2XX_LED_GREEN && LedControl != DTU2XX_LED_RED
		&& LedControl != DTU2XX_LED_YELLOW )
	{
		return DTU2XX_E_INVALID_MODE;
	}
			
	// Modify LED bits
	switch ( LedControl )
	{
	case DTU2XX_LED_HARDWARE:	pGenRegs->m_GenControl.Fields.m_LedControl	= 0;
								break;
	
	case DTU2XX_LED_OFF:		pGenRegs->m_GenControl.Fields.m_LedControl	= 1;
								pGenRegs->m_GenControl.Fields.m_LedGreen	= 0;
								pGenRegs->m_GenControl.Fields.m_LedRed		= 0;
                                break;
	
	case DTU2XX_LED_GREEN:		pGenRegs->m_GenControl.Fields.m_LedControl	= 1;
								pGenRegs->m_GenControl.Fields.m_LedGreen	= 1;
								pGenRegs->m_GenControl.Fields.m_LedRed		= 0;
                                break;
	
	case DTU2XX_LED_RED:		pGenRegs->m_GenControl.Fields.m_LedControl	= 1;
								pGenRegs->m_GenControl.Fields.m_LedGreen	= 0;
								pGenRegs->m_GenControl.Fields.m_LedRed		= 1;
                                break;
	
	case DTU2XX_LED_YELLOW:		pGenRegs->m_GenControl.Fields.m_LedControl	= 1;
								pGenRegs->m_GenControl.Fields.m_LedGreen	= 1;
								pGenRegs->m_GenControl.Fields.m_LedRed		= 1;
                                break;
	}

	// Write the new value
	return Dtu2xxIoCtlWriteReg( pFdo, DTU2XX_GEN_BASE_ADDR+DTU2XX_GEN_REG_GENCTRL,
									(Int)(pGenRegs->m_GenControl.All) );
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxIoCtlResetSoft -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// IoCtl handler for the reset card command
//
DTU2XX_RESULT Dtu2xxIoCtlResetSoft(
    DTU2XX_FDO* pFdo)	// Functional device object, representing the DTU-2xx card
{
	Int i=0, Status=0, BytesTransf=0;

    // Reset bulk buffers for all output channels
	for ( i = 0; i < pFdo->m_NumTxChannels; i++ )
	{
		// Clear internal buffer
		pFdo->m_TsTx[0].m_RingBuf.m_pRdPtr 
				= pFdo->m_TsTx[0].m_RingBuf.m_pStart;
		pFdo->m_TsTx[0].m_RingBuf.m_pWrPtr 
				= pFdo->m_TsTx[0].m_RingBuf.m_pStart;
		pFdo->m_TsTx[0].m_RingBuf.m_Load = 0;
	}

	// Reset bulk buffers for all input channels
	for ( i = 0; i < pFdo->m_NumRxChannels; i++ )
	{
		// Clear internal buffer
		pFdo->m_TsRx[0].m_RingBuf.m_pRdPtr 
				= pFdo->m_TsRx[0].m_RingBuf.m_pStart;
		pFdo->m_TsRx[0].m_RingBuf.m_pWrPtr 
				= pFdo->m_TsRx[0].m_RingBuf.m_pStart;
		pFdo->m_TsRx[0].m_RingBuf.m_Load = 0;
	}

	// Simply send the vendor command for resetting the device
	Status =  Dtu2xxVendorRequest(
					pFdo,
					DTU2XX_USB_RESET_DEVICE, 0, 0,
					DTU2XX_DIRECTION_WRITE,
					0, NULL, &BytesTransf);
	if ( Status != 0 ) {
		return Status;
	}
	// Update cache after a reset
	return Dtu2xxCacheOpRegs( pFdo );
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxIoCtlReadVpd -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
DTU2XX_RESULT Dtu2xxIoCtlReadVpd(
    DTU2XX_FDO* pFdo,		// Functional device object, representing the DTU-2xx card
    Int Start,				// Start BYTE address
    Int Count,				// Number of VPD bytes to transfer
    UInt8* pData)			// Buffer with VPD bytes
{
    DTU2XX_RESULT dr = DTU2XX_E_EEPROM_READ;

#ifdef DEBUG
    DTU2XX_LOG2( KERN_DEBUG, "Read VPD data, start=0x%03x, size=%d", nStart, Count );
#endif

    if ( Start >= DTU2XX_VPD_MAX_TRANSFER) {
        DTU2XX_LOG( KERN_DEBUG, "Start address not valid" );
    } else if (  (Start+Count) > DTU2XX_VPD_MAX_TRANSFER ) {
        DTU2XX_LOG( KERN_DEBUG, "Length not valid" );
    } else {
        if ( copy_to_user(pData, pFdo->m_Vpd + Start, Count) ) {
			DTU2XX_LOG( KERN_DEBUG, "Could not copy data to user!" );
			return DTU2XX_E_DEV_DRIVER;
		} else
			dr = DTU2XX_OK;
    }
    return dr;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxIoCtlWriteVpd -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
DTU2XX_RESULT Dtu2xxIoCtlWriteVpd(
    DTU2XX_FDO* pFdo,		// Functional device object, representing the DTU-2xx card
    Int Start,				// Start BYTE address
    Int Count,				// Number of VPD bytes to transfer
    UInt8* pData)	// Buffer with VPD bytes
{
    DTU2XX_RESULT dr = DTU2XX_E_EEPROM_WRITE;
    UInt8 Buf[DTU2XX_VPD_MAX_TRANSFER];

#ifdef DEBUG
    DTU2XX_LOG2( KERN_DEBUG, "Write VPD data, start=0x%03x, size=%d", nStart, Count );
#endif

    if ( Start>=DTU2XX_VPD_MAX_TRANSFER) {
        DTU2XX_LOG( KERN_DEBUG, "Start address not valid" );
    } else if (  (Start+Count)>DTU2XX_VPD_MAX_TRANSFER ) {
        DTU2XX_LOG( KERN_DEBUG, "Length not valid" );
    } else {
        // Copy data to transfer from user buffer
        if ( copy_from_user(Buf, pData, Count) ) {
			DTU2XX_LOG( KERN_DEBUG, "Could not copy data from user!" );
			return DTU2XX_E_DEV_DRIVER;
		}
        if ( DTU2XX_OK==(dr=Dtu2xxVpdWrite(pFdo, Buf, Start, Count)) )
            memcpy(pFdo->m_Vpd + Start, Buf, Count);
    }
    return dr;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxIoCtlGetDeviceInfo -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
DTU2XX_RESULT Dtu2xxIoCtlGetDeviceInfo(
    DTU2XX_FDO* pFdo,			// The device descriptor
    UInt8  *pFwVersion,			// Altera FPGA firmware version
    UInt8  *pTypeNumber)		// Device type number
{
	Int Status = 0, BytesTransf=0;
	UInt8* pTempBuf = NULL;

	// Create request buffer (3 bytes)
	pTempBuf = (UInt8 *)kmalloc( sizeof(UInt8)*3, GFP_KERNEL );
    if ( pTempBuf == NULL ) {
		return -ENOMEM;
	}

	// Send vendor request
	Status =  Dtu2xxVendorRequest(
					pFdo,
					DTU2XX_USB_GET_DEVICE_INF0, 0, 0,
					DTU2XX_DIRECTION_READ,
					3, pTempBuf, &BytesTransf);
	if ( BytesTransf >= 2 ) {
		*pFwVersion  = pTempBuf[0];
		*pTypeNumber = pTempBuf[1];
	} else {
		// Error!
		Status = 0;
	}

	// Free buffer
	kfree( pTempBuf );
	
	return Status;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxIoCtlReadReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
DTU2XX_RESULT Dtu2xxIoCtlReadReg(
    DTU2XX_FDO* pFdo,
    UInt16 Address,
    UInt32* pValue)
{
	Int ret=0, BytesTransf=0;

	ret = Dtu2xxVendorRequest(
					pFdo,
					DTU2XX_USB_READ_REG, Address, 0,
					DTU2XX_DIRECTION_READ,
					4, (UInt8*)(pValue), &BytesTransf);
	return ret;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxIoCtlWriteReg -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
DTU2XX_RESULT Dtu2xxIoCtlWriteReg(
    DTU2XX_FDO* pFdo,
    UInt16 Address,
    UInt32 Value)
{
	Int ret=0, BytesTransf=0;
	ret = Dtu2xxVendorRequest(
					pFdo,
					DTU2XX_USB_WRITE_REG, Address, 0,
					DTU2XX_DIRECTION_WRITE,
					4, (UInt8*)(&Value), &BytesTransf);
	return ret;
}
