//#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* RxIoCtl.c *#*#*#*#*#*#*#*#*# (C) 2000-2006 DekTec
//
// This code is the intellectual property of DekTec Digital Video B.V. and may not be
// distributed, in its original or in altered form, without the explicit permission of
// DekTec Digital Video B.V.

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//  2006.10.01	MG	Clean-up of code
//  2006.08.14	KvR	Changed to new interface, added support for DTU-245, several bugfixes
//	2002.10.21	MG	Created

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#include <asm/uaccess.h>
#include "Dtu2xx.h"
#include "Dtu2xxRegs.h"
#include "EzUsb.h"

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Type definitions -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Internal function declarations -.-.-.-.-.-.-.-.-.-.-.-.-.-.

//-.-.-.-.-.-.-.-.-.-.-.-.-.- Data declaration and definition -.-.-.-.-.-.-.-.-.-.-.-.-.-.

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxRxIoCtlClearFlags -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Clear the status flags
//
DTU2XX_RESULT Dtu2xxRxIoCtlClearFlags(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int Latched)		// Latched status flags to be cleared
{
	Int Status = 0;

    // Valid channel index???
    if ( pFdo->m_NumRxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_INPUT;

	// Clear the latched flags
	pFdo->m_TsRx[ChIdx].m_Latched &= ~Latched;

	return Status;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxRxIoCtlEqualise -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Turn DVB/ASI cable equalisation on or off (DTA-120 only).
//
DTU2XX_RESULT Dtu2xxRxIoCtlEqualise(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int nEqualiserSetting)
{
    // Valid channel index???
    if ( pFdo->m_NumRxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_INPUT;

    return DTU2XX_E_NOT_SUPPORTED;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxRxIoCtlGetFifoLoad -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Get the current load of the channel's Output FIFO.
//
DTU2XX_RESULT Dtu2xxRxIoCtlGetFifoLoad(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int* pFifoLoad)
{
	Int Status = 0;
	Dtu2xxRx* pRxRegs = NULL;

    // Valid channel index???
    if ( pFdo->m_NumRxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_INPUT;

	// Get register cache
	pRxRegs = &(pFdo->m_TsRx[ChIdx].m_ChRegs.m_RxRegs);

	// Init to zero
	*pFifoLoad = 0;

	// Do not read from register cache but directly from hardware
	Status = Dtu2xxIoCtlReadReg( pFdo,
						DTU2XX_RX_BASE_ADDR+DTU2XX_RX_REG_FIFOLOAD,
						(UInt32*)pFifoLoad );

	// Update cache
	pRxRegs->m_FifoLoad = *pFifoLoad;

	// Add load of already pre-fetched data
	*pFifoLoad += pFdo->m_TsRx[ChIdx].m_RingBuf.m_Load;

	return Status;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxRxIoCtlGetFlags -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Get status flags
//
DTU2XX_RESULT Dtu2xxRxIoCtlGetFlags(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int*  pStatus,		// Status flags
    Int*  pLatched)		// Latched status flags
{
	Int Status = 0;
	Dtu2xxRx* pRxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumRxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_INPUT;
	// Get register cache
	pRxRegs = &(pFdo->m_TsRx[ChIdx].m_ChRegs.m_RxRegs);

	// Init to zero
	*pStatus = *pLatched = 0;

	// Update cache
	Status = Dtu2xxIoCtlReadReg( pFdo, DTU2XX_RX_BASE_ADDR+DTU2XX_RX_REG_RXSTATUS,
									  (UInt32*)(&(pRxRegs->m_RxStatus.All)) );
	if ( Status != 0 ) {
		return Status;
	}

	// Check for status flags
	if ( pRxRegs->m_RxStatus.Fields.m_OvfInt == 1 ) {
		*pStatus |= DTU2XX_RX_FIFO_OVF;
	}
	if ( pRxRegs->m_RxStatus.Fields.m_SyncInt == 1 ) {
		*pStatus |= DTU2XX_RX_SYNC_ERR;
	}
	if ( pRxRegs->m_RxStatus.Fields.m_RateOvfInt == 1 ) {
		*pStatus |= DTU2XX_RX_RATE_OVF;
	}

	// Clear status flags (write back register value)
	Status = Dtu2xxIoCtlWriteReg( pFdo, DTU2XX_RX_BASE_ADDR+DTU2XX_RX_REG_RXSTATUS,
									   (Int)(pRxRegs->m_RxStatus.All) );
	if ( Status != 0 ) {
		return Status;
	}

	// Clear flags in cache
	pRxRegs->m_RxStatus.All &= ~0x3F00;

	// Update the status of the latched flags
	pFdo->m_TsRx[ChIdx].m_Latched |= *pStatus;
	// Return the latched flags
	*pLatched = pFdo->m_TsRx[ChIdx].m_Latched;

	return Status;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxRxIoCtlGetStatus -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Get status information from the input channel. If a PCI card does not support a
// certain feature, the corresponding status variable is set to DTU2XX_E_NOT_SUPPORTED.
//
// The status flags are read directly from the hardware (not through device driver).
//
DTU2XX_RESULT Dtu2xxRxIoCtlGetStatus(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int* pPacketSize,	// Size of incoming transport packets: DTU2XX_PCKSIZE_XXX
    Int* pClkDet,		// Receive-clock signal detected: DTU2XX_CLKDET_XXX
    Int* pAsiLock,		// DVB/ASI PLL locked: DTU2XX_ASILOCK_XXX
    Int* pRateOk )		// Input rate above minimum
{
	Int Status = 0;
	Dtu2xxRx* pRxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumRxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_INPUT;
	// Get register cache
	pRxRegs = &(pFdo->m_TsRx[ChIdx].m_ChRegs.m_RxRegs);

	// Init to zero
	*pPacketSize	= 0;
	*pClkDet		= 0;
	*pAsiLock		= 0;
	*pRateOk		= 0;

	// Update cache
	Status = Dtu2xxIoCtlReadReg( pFdo, DTU2XX_RX_BASE_ADDR+DTU2XX_RX_REG_RXSTATUS,
									  (UInt32*)(&(pRxRegs->m_RxStatus.All)) );
	if ( Status != 0 ) {
		return Status;
	}

	// Get status fields
	*pPacketSize	= pRxRegs->m_RxStatus.Fields.m_PckSize;
	*pClkDet		= pRxRegs->m_RxStatus.Fields.m_ClkDet;
	*pAsiLock		= pRxRegs->m_RxStatus.Fields.m_AsiLock;
	*pRateOk		= pRxRegs->m_RxStatus.Fields.m_RateOk;

	return Status;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxRxIoCtlGetTargetId -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Returns the ID of the connected target adapter (if present)
//
DTU2XX_RESULT Dtu2xxRxIoCtlGetTargetId(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int* pPresent,		// Target adapter present?
    Int* pTargetId)		// Target-adapter identifier
{
	Dtu2xxRx* pRxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumRxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_INPUT;
	// Get register cache
	pRxRegs = &(pFdo->m_TsRx[ChIdx].m_ChRegs.m_RxRegs);

    return DTU2XX_E_NOT_SUPPORTED;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxRxIoCtlGetTsRateBps -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Gets the transport stream bit-rate.
//
DTU2XX_RESULT Dtu2xxRxIoCtlGetTsRateBps(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int* pTsRate)		// Transport-stream rate in bps
{
	Int Status = 0;
	Dtu2xxRx* pRxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumRxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_INPUT;
	// Get register cache
	pRxRegs = &(pFdo->m_TsRx[ChIdx].m_ChRegs.m_RxRegs);

	// Init to zero
	*pTsRate = 0;

  	// Read the bit-rate register (not from cache)
	Status = Dtu2xxIoCtlReadReg( pFdo,
						DTU2XX_RX_BASE_ADDR+DTU2XX_RX_REG_BITRATE,
						(UInt32*)pTsRate );

	if ( Status != 0 ) {
		return Status;
	}

	// Update cache to match current register value
	pRxRegs->m_Bitrate = *pTsRate;

	return Status;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxRxIoCtlPolarityControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Control the automatic polarity-detection circuitry of a DVB/ASI Transport-Stream
// input channel.
//
DTU2XX_RESULT Dtu2xxRxIoCtlPolarityControl(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int Polarity)		// Polarity control setting
{
	Dtu2xxRx* pRxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumRxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_INPUT;
	// Get register cache
	pRxRegs = &(pFdo->m_TsRx[ChIdx].m_ChRegs.m_RxRegs);

	// Check for valid values
	if( Polarity != DTU2XX_POLARITY_AUTO && Polarity != DTU2XX_POLARITY_NORMAL
		&& Polarity != DTU2XX_POLARITY_INVERT ) {
		return -EINVAL;
	}

	// Set polarity
	pRxRegs->m_RxControl.Fields.m_AsiInv = Polarity;

	// Update register value to match cache
	return Dtu2xxIoCtlWriteReg( pFdo, DTU2XX_RX_BASE_ADDR+DTU2XX_RX_REG_RXCTRL,
									 (Int)(pRxRegs->m_RxControl.All) );
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxRxIoCtlReset -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Reset transport-stream input channel.
//
DTU2XX_RESULT Dtu2xxRxIoCtlReset(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int RstMode)		// How to reset the channel
{
	Int Status = 0, BytesTransf=0;
    UInt8 VendCmd = 0;
	Dtu2xxRx* pRxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumRxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_INPUT;
	// Get register cache
	pRxRegs = &(pFdo->m_TsRx[ChIdx].m_ChRegs.m_RxRegs);

	// Check for valid value
	if ( RstMode != DTU2XX_RST_CHANNEL && RstMode != DTU2XX_RST_CLEARFIFO ) {
		return -EINVAL;
	}

	if ( RstMode == DTU2XX_RST_CHANNEL ) {
		VendCmd = DTU2XX_USB_RX_RESET;
	} else {
		VendCmd = DTU2XX_USB_RX_CLEAR_FIFO;
	}

	// Clear all flags
	Dtu2xxRxIoCtlClearFlags( pFdo, ChIdx, 0xFFFFFFFF );

	// Clear internal buffer
	pFdo->m_TsRx[ChIdx].m_RingBuf.m_pRdPtr =
			pFdo->m_TsRx[ChIdx].m_RingBuf.m_pWrPtr;
	pFdo->m_TsRx[ChIdx].m_RingBuf.m_Load = 0;

	// Simply send the vendor command to the device
	Status = Dtu2xxVendorRequest(
					pFdo,
					VendCmd, 0, 0,
					DTU2XX_DIRECTION_WRITE,
					0, NULL, &BytesTransf);
	if ( Status < 0 )
		return DTU2XX_E;

	// We need to update all cached registers after a reset
	return Dtu2xxCacheOpRegs( pFdo );
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxRxIoCtlSetLoopBackMode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Set the channel's loop-back mode.
//
DTU2XX_RESULT Dtu2xxRxIoCtlSetLoopBackMode(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int nMode)			// New loop-back mode
{
    Dtu2xxRx* pRxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumRxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_INPUT;
	// Get register cache
	pRxRegs = &(pFdo->m_TsRx[ChIdx].m_ChRegs.m_RxRegs);

    return DTU2XX_E_NOT_SUPPORTED;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxRxIoCtlSetPower -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Set the channel's power mode.
//
DTU2XX_RESULT Dtu2xxRxIoCtlSetPower(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int nPower)			// New power mode
{
    Dtu2xxRx* pRxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumRxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_INPUT;
	// Get register cache
	pRxRegs = &(pFdo->m_TsRx[ChIdx].m_ChRegs.m_RxRegs);

    return DTU2XX_E_NOT_SUPPORTED;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxRxIoCtlSetRxControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Set the channel's receive-control state.
//
DTU2XX_RESULT Dtu2xxRxIoCtlSetRxControl(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int RxControl)		// Receive control
{
	Dtu2xxRx* pRxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumRxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_INPUT;
	// Get register cache
	pRxRegs = &(pFdo->m_TsRx[ChIdx].m_ChRegs.m_RxRegs);

	// Check for valid value
	if ( RxControl != DTU2XX_RXCTRL_IDLE && RxControl != DTU2XX_RXCTRL_RCV ) {
		return -EINVAL;
	}

	// Set control field in cache
	pRxRegs->m_RxControl.Fields.m_RxCtrl = RxControl;

	// Update register value to match cache
	return Dtu2xxIoCtlWriteReg( pFdo, DTU2XX_RX_BASE_ADDR+DTU2XX_RX_REG_RXCTRL,
									 (Int)(pRxRegs->m_RxControl.All) );
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxRxIoCtlSetRxMode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Set the channel's receive mode.
//
DTU2XX_RESULT Dtu2xxRxIoCtlSetRxMode(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int RxMode)			// Receive mode
{
   	Int StMode;
	Dtu2xxRx* pRxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumRxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_INPUT;
	// Get register cache
	pRxRegs = &(pFdo->m_TsRx[ChIdx].m_ChRegs.m_RxRegs);

	StMode = RxMode & DTU2XX_RXMODE_MASK;
	// Check for valid values
	if (    StMode != DTU2XX_RXMODE_ST188 && StMode != DTU2XX_RXMODE_ST204
		 && StMode != DTU2XX_RXMODE_STRAW && StMode != DTU2XX_RXMODE_STMP2
		 && StMode != DTU2XX_RXMODE_STTRP )
	{
		return -EINVAL;
	}
	if ( 0 != (RxMode & ~(DTU2XX_RXMODE_MASK | DTU2XX_RX_TIMESTAMP | DTU2XX_RXMODE_SDI
		                | DTU2XX_RXMODE_SDI_10BIT)) )
	{
		return -EINVAL;
	}

	// Set mode bits
	pRxRegs->m_RxControl.Fields.m_RxMode = StMode & 0x3;

	// Set timestamp
	if ( 0 != (RxMode & DTU2XX_RX_TIMESTAMP) ) {
		pRxRegs->m_RxControl.Fields.m_RxTimeStamp = 1;
	} else {
		pRxRegs->m_RxControl.Fields.m_RxTimeStamp = 0;
	}
	// Set SDI Mode
	if (  0 != (RxMode & DTU2XX_RXMODE_SDI )) {
		pRxRegs->m_RxControl.Fields.m_SdiMode = 1;
	} else {
		pRxRegs->m_RxControl.Fields.m_SdiMode = 0;
	}
	// Set SDI 10Bit Mode
	if ( 0 != (RxMode & DTU2XX_RXMODE_SDI_10BIT) ) {
		pRxRegs->m_RxControl.Fields.m_Sdi10Bit = 1;
	} else {
		pRxRegs->m_RxControl.Fields.m_Sdi10Bit = 0;
	}

	// Update register to match cache
	return Dtu2xxIoCtlWriteReg( pFdo, DTU2XX_RX_BASE_ADDR+DTU2XX_RX_REG_RXCTRL,
								 (Int)(pRxRegs->m_RxControl.All) );
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxRxIoCtlDma -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Transfer data from the input channel HW fifo to a user buffer, using a DMA transfer
//
DTU2XX_RESULT Dtu2xxRxIoCtlDma(
    DTU2XX_FDO* pFdo,		// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int8* pUserBuf,			// Pointer to the user buffer
    Int nBytesToTransfer)	// #Bytes to transfer
{
	Dtu2xxRx* pRxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumRxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_INPUT;
	// Get register cache
	pRxRegs = &(pFdo->m_TsRx[ChIdx].m_ChRegs.m_RxRegs);

    return DTU2XX_NOT_SUPPORTED;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxRxIoCtlGetRxControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
DTU2XX_RESULT Dtu2xxRxIoCtlGetRxControl(
	DTU2XX_FDO* pFdo,
	Int ChIdx,			// Channel index
	Int *pRxControl)
{
	Dtu2xxRx* pRxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumRxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_INPUT;
	// Get register cache
	pRxRegs = &(pFdo->m_TsRx[ChIdx].m_ChRegs.m_RxRegs);

	// Get from cache
	*pRxControl = pRxRegs->m_RxControl.Fields.m_RxCtrl;

	return 0;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxRxIoCtlGetRxMode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
DTU2XX_RESULT Dtu2xxRxIoCtlGetRxMode(
	DTU2XX_FDO* pFdo,
	Int ChIdx,			// Channel index
	Int *pRxMode)
{
	Dtu2xxRx* pRxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumRxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_INPUT;
	// Get register cache
	pRxRegs = &(pFdo->m_TsRx[ChIdx].m_ChRegs.m_RxRegs);

	// Get mode from cache
	*pRxMode = pRxRegs->m_RxControl.Fields.m_RxMode;

	// Check for Timestamp mode
	if ( pRxRegs->m_RxControl.Fields.m_RxTimeStamp == 1 ) {
		*pRxMode |= DTU2XX_RX_TIMESTAMP;
	}
	// Check for SDI Mode
	if ( pRxRegs->m_RxControl.Fields.m_SdiMode == 1 ) {
		*pRxMode |= DTU2XX_RXMODE_SDI;
	}
	// Check for SDI 10Bit Mode
	if ( pRxRegs->m_RxControl.Fields.m_Sdi10Bit == 1 ) {
		*pRxMode |= DTU2XX_RXMODE_SDI_10BIT;
	}

	return 0;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxRxIoCtlLedControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Io-Handler for controlling the receive status LED
//
DTU2XX_RESULT Dtu2xxRxIoCtlLedControl(
	DTU2XX_FDO* pFdo,	// Device object
	Int ChIdx,			// Channel index
	Int val)			// LED mode
{
	Dtu2xxRx* pRxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumRxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_INPUT;
	// Get register cache
	pRxRegs = &(pFdo->m_TsRx[ChIdx].m_ChRegs.m_RxRegs);

	// Check for valid value
	if (   val!=DTU2XX_LED_HARDWARE && val!=DTU2XX_LED_OFF
		&& val!=DTU2XX_LED_GREEN && val!=DTU2XX_LED_RED
		&& val!=DTU2XX_LED_YELLOW )
	{
		return -EINVAL;
	}

	// Modify LED bits
    switch(val)
    {
	case DTU2XX_LED_HARDWARE:	pRxRegs->m_RxControl.Fields.m_LedControl	= 0;
								break;

	case DTU2XX_LED_OFF:		pRxRegs->m_RxControl.Fields.m_LedControl	= 1;
								pRxRegs->m_RxControl.Fields.m_LedGreen		= 0;
								pRxRegs->m_RxControl.Fields.m_LedRed		= 0;
                                break;

	case DTU2XX_LED_GREEN:		pRxRegs->m_RxControl.Fields.m_LedControl	= 1;
								pRxRegs->m_RxControl.Fields.m_LedGreen		= 1;
								pRxRegs->m_RxControl.Fields.m_LedRed		= 0;
                                break;

	case DTU2XX_LED_RED:		pRxRegs->m_RxControl.Fields.m_LedControl	= 1;
								pRxRegs->m_RxControl.Fields.m_LedGreen		= 0;
								pRxRegs->m_RxControl.Fields.m_LedRed		= 1;
                                break;

	case DTU2XX_LED_YELLOW:		pRxRegs->m_RxControl.Fields.m_LedControl	= 1;
								pRxRegs->m_RxControl.Fields.m_LedGreen		= 1;
								pRxRegs->m_RxControl.Fields.m_LedRed		= 1;
                                break;

	default:					return DTU2XX_E_INVALID_MODE;
    }

	// Write cached value
    return Dtu2xxIoCtlWriteReg( pFdo, DTU2XX_RX_BASE_ADDR+DTU2XX_RX_REG_RXCTRL, pRxRegs->m_RxControl.All );
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxRxIoCtlClearFifo -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
DTU2XX_RESULT Dtu2xxRxIoCtlClearFifo(
	DTU2XX_FDO* pFdo,
	Int ChIdx)			// Channel index
{
    return Dtu2xxRxIoCtlReset(pFdo, ChIdx, DTU2XX_RST_CLEARFIFO);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxRxIoCtlGetViolm_Count -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
DTU2XX_RESULT Dtu2xxRxIoCtlGetViolCount(
	DTU2XX_FDO* pFdo,
	Int ChIdx,			// Channel index
	Int* pViolCount)
{
	Int Status = 0;
	Dtu2xxRx* pRxRegs = NULL;

	// Init to zero
	*pViolCount = 0;

	// Valid channel index???
    if ( pFdo->m_NumRxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_INPUT;
	// Get register cache
	pRxRegs = &(pFdo->m_TsRx[ChIdx].m_ChRegs.m_RxRegs);

	// Do not read from register cache but directly from hardware
	Status = Dtu2xxIoCtlReadReg( pFdo,
								 DTU2XX_RX_BASE_ADDR+DTU2XX_RX_REG_VIOLCOUNT,
								 (UInt32*)pViolCount );

	// Update cache
	pRxRegs->m_CodeViolCnt = *pViolCount;

	return Status;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxRxIoCtlBulkRead -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
DTU2XX_RESULT Dtu2xxRxIoCtlBulkRead(
	DTU2XX_FDO* pFdo,
	Int ChIdx,			// Channel index
	Int TransferSize,
	Int* pBytesRead,
	UInt8* pBuf)
{
	UInt8 *pTempBuf;
	Int Status=0, BytesLeft, BufIndex, ByteCount, NumCopy, Prefetch, TempBufSize;
	DTU2XX_RING_BUFFER* pRingBuf=NULL;

	// Init to zero
	*pBytesRead = 0;

	// Valid channel index???
    if ( pFdo->m_NumRxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_INPUT;

	// Handle nothing to transfer case
	if ( TransferSize == 0 )
		return DTU2XX_OK;
	// Check for valid buffer
	if ( pBuf == NULL )
		return DTU2XX_E_INVALID_BUF;

#if LOG_LEVEL_DMA > 1
	DTU2XX_LOG1( KERN_INFO, "Dtu2xxRxIoCtlBulkRead with TransferSize=%d", TransferSize );
#endif

	BytesLeft = TransferSize;
	BufIndex = 0;

	TempBufSize = pFdo->m_TsRx[ChIdx].m_TempBufSize;
	pTempBuf = pFdo->m_TsRx[ChIdx].m_pTempBuf;
	pRingBuf = &(pFdo->m_TsRx[ChIdx].m_RingBuf);

	//-.-.-.-.-.-.-.-.-.-.- First check for data already pre-fetched -.-.-.-.-.-.-.-.-.-.-

	if ( pRingBuf->m_Load != 0 )
	{
		NumCopy = (pRingBuf->m_Load > TransferSize) ? TransferSize : pRingBuf->m_Load;
		// Copy from ring buffer to user buffer (via temp buffer)
		Dtu2xxRingBuffer2Buf(pRingBuf, NumCopy, pTempBuf);
		Status = copy_to_user(&(pBuf[BufIndex]), pTempBuf, NumCopy);
		// Update # bytes left to copy + index
		BytesLeft -= NumCopy;
		BufIndex += NumCopy;
	}
	if ( BytesLeft == 0 ) {
		*pBytesRead = TransferSize;
		return DTU2XX_OK;
	}

	//.-.-.-.-.-.-.- Directly copy multiples of DTU2XX_BULK_TRANSFER_ALIGN -.-.-.-.-.-.-.-

	while ( BytesLeft>=DTU2XX_BULK_TRANSFER_ALIGN  )
	{
		ByteCount = (BytesLeft>DTU2XX_BULK_PACKETSIZE) ?
										DTU2XX_BULK_PACKETSIZE : BytesLeft;
		// Only multiples of DTU2XX_BULK_TRANSFER_ALIGN
		ByteCount = ByteCount - (ByteCount % DTU2XX_BULK_TRANSFER_ALIGN);

		// Execute bulk transfer
		Status = Dtu2xxSimpleBulkTransfer(
							pFdo,
							DTU2XX_DIRECTION_READ,
							EZUSB_ENDPOINT_6,
							ByteCount, pTempBuf );
		if ( Status != 0 )
			return DTU2XX_E;
		// Copy to user buffer
		Status = copy_to_user(&(pBuf[BufIndex]), pTempBuf, ByteCount);

		BytesLeft -= ByteCount;
		BufIndex += ByteCount;
 	}
	if ( BytesLeft==0 ) {
		*pBytesRead = TransferSize;
		return DTU2XX_OK;
	}

	//.-.-.- Get final bytes by pre-fetching multiple DTU2XX_BULK_TRANSFER_ALIGN -.-.-.

	// First prefetch as much as needed (must be multiple of DTU2XX_BULK_TRANSFER_ALIGN)
	Prefetch = (DTU2XX_BULK_TRANSFER_ALIGN-(BytesLeft%DTU2XX_BULK_TRANSFER_ALIGN))
						+ BytesLeft;


	// Transfer data to our temporary buffer
	Status = Dtu2xxSimpleBulkTransfer( pFdo, DTU2XX_DIRECTION_READ,
									   EZUSB_ENDPOINT_6, Prefetch,
									   pTempBuf );

	if ( Status == 0 )
	{
		// Copy remaining data to target buf
		Status = copy_to_user(&(pBuf[BufIndex]), pTempBuf, BytesLeft);

		// Store left over in ring buffer
		NumCopy = (Prefetch-BytesLeft);
		Dtu2xxBuf2RingBuffer(&pTempBuf[BytesLeft], NumCopy, pRingBuf);

		*pBytesRead = TransferSize;
		return DTU2XX_OK;
    } else
		return DTU2XX_E;
}
