//#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* TxIoCtl.c *#*#*#*#*#*#*#*#*# (C) 2000-2006 DekTec
//
// This code is the intellectual property of DekTec Digital Video B.V. and may not be
// distributed, in its original or in altered form, without the explicit permission of
// DekTec Digital Video B.V.

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Change History -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//  2006.10.01	MG	Clean-up of code
//  2006.08.14	KvR	Changed to new interface, added support for DTU-245, several bugfixes
//	2003.04.16	MG	Made sure output of DTU-202 is enabled when TxCtrl!=DTU2XX_TXCTRL_IDLE
//	2002.10.21	MG	Created


//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Include files -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
#include "Dtu2xx.h"
#include "Dtu2xxRegs.h"
#include "EzUsb.h"
#include <asm/uaccess.h>

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Type definitions -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Internal function declarations -.-.-.-.-.-.-.-.-.-.-.-.-.-.

//-.-.-.-.-.-.-.-.-.-.-.-.-.- Data declaration and definition -.-.-.-.-.-.-.-.-.-.-.-.-.-.

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxTxIoCtlClearFifo -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Clear the tx fifo
//
DTU2XX_RESULT Dtu2xxTxIoCtlClearFifo(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx)			// Channel index
{
	return Dtu2xxTxIoCtlReset(pFdo, ChIdx, DTU2XX_RST_CLEARFIFO);
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxTxIoCtlClearFlags -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Clear the status flags
//
DTU2XX_RESULT Dtu2xxTxIoCtlClearFlags(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int Latched)		// Latched status flags to be cleared
{
    Dtu2xxTx* pTxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumTxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_OUTPUT;
	// Get register cache
	pTxRegs = &(pFdo->m_TsTx[ChIdx].m_ChRegs.m_TxRegs);

	// Clear the latched flags
	pFdo->m_TsTx[ChIdx].m_Latched &= ~Latched;

    return 0;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxTxIoCtlGetExtClkFreq -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Returns the Transport-stream rate of external clock in bps
//
DTU2XX_RESULT Dtu2xxTxIoCtlGetExtClkFreq(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int* pExtClkFreq)	// Transport-stream rate of external clock in bps
{
	Dtu2xxTx* pTxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumTxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_OUTPUT;
	// Get register cache
	pTxRegs = &(pFdo->m_TsTx[ChIdx].m_ChRegs.m_TxRegs);

    return DTU2XX_E_NOT_SUPPORTED;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxTxIoCtlGetFifoLoad -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Get the current load of the channel's Output FIFO.
//
DTU2XX_RESULT Dtu2xxTxIoCtlGetFifoLoad(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int* pFifoLoad)		// Current fifo load
{
	Int Status = 0;
	Dtu2xxTx* pTxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumTxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_OUTPUT;
	// Get register cache
	pTxRegs = &(pFdo->m_TsTx[ChIdx].m_ChRegs.m_TxRegs);

	// Init to zero
	*pFifoLoad = 0;

	// Do not read from register cache but directly from hardware
	Status = Dtu2xxIoCtlReadReg( pFdo,
						DTU2XX_TX_BASE_ADDR+DTU2XX_TX_REG_FIFOLOAD,
						(UInt32*)pFifoLoad );

	// Update cache
	pTxRegs->m_FifoLoad = *pFifoLoad;

	// Add data in bulk transfer buffer to the load
	*pFifoLoad += pFdo->m_TsTx[ChIdx].m_RingBuf.m_Load;

	return Status;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxTxIoCtlGetFlags -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Returns the current value of the status and latched flags
//
DTU2XX_RESULT Dtu2xxTxIoCtlGetFlags(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int* pStatus,		// Status flags
    Int* pLatched)		// Latched status flags
{
	Int Status = 0;

	Dtu2xxTx* pTxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumTxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_OUTPUT;
	// Get register cache
	pTxRegs = &(pFdo->m_TsTx[ChIdx].m_ChRegs.m_TxRegs);

	// Init to zero
	*pStatus = *pLatched = 0;

	// Update cache
	Status = Dtu2xxIoCtlReadReg( pFdo, DTU2XX_TX_BASE_ADDR+DTU2XX_TX_REG_TXSTATUS,
									  (UInt32*)&(pTxRegs->m_TxStatus.All) );
	if ( Status != 0 ) {
		return Status;
	}

	// Check for status flags
	if ( pTxRegs->m_TxStatus.Fields.m_UflInt == 1 ) {
		*pStatus |= DTU2XX_TX_FIFO_UFL;
	}
	if ( pTxRegs->m_TxStatus.Fields.m_SyncInt==1 ) {
		*pStatus |= DTU2XX_TX_SYNC_ERR;
	}

	// Clear status flags (write back register value)
	Status = Dtu2xxIoCtlWriteReg( pFdo, DTU2XX_TX_BASE_ADDR+DTU2XX_TX_REG_TXSTATUS,
									   (Int)(pTxRegs->m_TxStatus.All));
	if ( Status != 0 ) {
		return Status;
	}

	// Clear flags in cache
	pTxRegs->m_TxStatus.All &= ~0x1F00;

	// Update the status of the latched flags
	pFdo->m_TsTx[ChIdx].m_Latched |= *pStatus;
	// Return the latched flags
	*pLatched = pFdo->m_TsTx[ChIdx].m_Latched;

	return Status;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxTxIoCtlGetMaxFifoSize -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Returns the maximum possible fifo size
//
DTU2XX_RESULT Dtu2xxTxIoCtlGetMaxFifoSize(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int* pSize)			// MAX fifo size
{
	Dtu2xxTx* pTxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumTxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_OUTPUT;
	// Get register cache
	pTxRegs = &(pFdo->m_TsTx[ChIdx].m_ChRegs.m_TxRegs);

	// Decode the SDRAM size field
	switch ( pTxRegs->m_TxStatus.Fields.m_SdramSize )
	{
	case DTU2XX_SDRAM_SIZE_8MB:
		*pSize = (8*1024*1024);
		break;
	case DTU2XX_SDRAM_SIZE_16MB:
		*pSize = (16*1024*1024);
		break;
	case DTU2XX_SDRAM_SIZE_32MB:
		*pSize = (32*1024*1024);
		break;
	default:
		*pSize = (8*1024*1024);
		break;
	}
	return 0;
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxTxIoCtlGetTargetId -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Returns the ID of the connected target adapter (if present)
//
DTU2XX_RESULT Dtu2xxTxIoCtlGetTargetId(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int* pPresent,		// Target adapter present?
    Int* pTargetId)		// Target-adapter identifier
{
    Dtu2xxTx* pTxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumTxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_OUTPUT;
	// Get register cache
	pTxRegs = &(pFdo->m_TsTx[ChIdx].m_ChRegs.m_TxRegs);

    return DTU2XX_E_NOT_SUPPORTED;

}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxTxIoCtlGetTsRateBps -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Gets the transport stream bit-rate.
//
DTU2XX_RESULT Dtu2xxTxIoCtlGetTxClock(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int* pTsRate)		// The bit-rate
{
	Int Status = 0;
	Int  TxPacketMode, TxStuffMode;
	UInt TxClock;

//	Int  PckSize;					// Packet size: 188 / 204
//	UInt64  PhaseIncr;				// The TxClock value
//	UInt64  tmpRate; 				// Temporary variable to pass to do_div()
	Dtu2xxTx* pTxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumTxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_OUTPUT;
	// Get register cache
	pTxRegs = &(pFdo->m_TsTx[ChIdx].m_ChRegs.m_TxRegs);

	// Init to zero
	*pTsRate = 0;

	// Get current value from cache
	TxClock = pTxRegs->m_TxClock;

	// Get current packet mode
	Status = Dtu2xxTxIoCtlGetTxMode(pFdo, ChIdx, &TxPacketMode, &TxStuffMode);
	if ( Status != 0 ) {
		return Status;
	}

/*
 * 	The calculation of the clockrate is already implemented in the DTAPI
 */

/*	// Determine current packet size
	PckSize = Dtu2xxTxMode2PacketSize( TxPacketMode );

	// Compute transport-stream rate, depends on phase increment and packet size
	PhaseIncr = TxClock;
	if (PckSize == 188) {
		// *pTsRate = (Int) ((PhaseIncr * 421875*2 / 8388608 + 1) >> 1);
		//
		// Divisions of 64 bit variables can be done after linking against gcclib;
		// 		but as this is a kernel module, this cannot be done. Therefore
		// 		macro 'do_div( UInt64 *, UInt32 )' from asm-generic/div64.h is used.

		tmpRate = (PhaseIncr * 421875 * 2);
		do_div( tmpRate, 8388608 );
		*pTsRate = (Int)((tmpRate + 1) >> 1);
	} else if (PckSize == 192) {
		// *pTsRate = (Int) ((PhaseIncr * 19828125*2 / 402653184 + 1) >> 1);

		tmpRate = (PhaseIncr * 19828125 * 2);
		do_div( tmpRate, 402653184 );
		*pTsRate = (Int)((tmpRate + 1) >> 1);
	} else {	// nPckSize == 204
		// *pTsRate = (Int) ((PhaseIncr * 6609375*2 / 142606336 + 1) >> 1);

		tmpRate = (PhaseIncr * 6609375 * 2);
		do_div( tmpRate, 142606336 );
		*pTsRate = (Int)((tmpRate + 1) >> 1);
	}	*/

	*pTsRate = TxClock;

	return Status;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxTxIoCtlGetTxControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Get the channel's transmit-control state.
//
DTU2XX_RESULT Dtu2xxTxIoCtlGetTxControl(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int* pTxControl)	// Transmit-control state
{
	Dtu2xxTx* pTxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumTxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_OUTPUT;
	// Get register cache
	pTxRegs = &(pFdo->m_TsTx[ChIdx].m_ChRegs.m_TxRegs);

	// Get control value from cache
	*pTxControl = pTxRegs->m_TxControl.Fields.m_TxCtrl;

	return 0;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxTxIoCtlGetTxMode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Get the channel's transmit mode.
//
DTU2XX_RESULT Dtu2xxTxIoCtlGetTxMode(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int* pTxPacketMode,	// Packet transmit mode
    Int* pTxStuffMode)  // Null-packet stuffing on/off
{
	Dtu2xxTx* pTxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumTxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_OUTPUT;
	// Get register cache
	pTxRegs = &(pFdo->m_TsTx[ChIdx].m_ChRegs.m_TxRegs);

	// Get current packet mode (from cache)
	*pTxPacketMode = pTxRegs->m_TxControl.Fields.m_TxMode;
	*pTxPacketMode |= pTxRegs->m_TxControl.Fields.m_TxModeExt << 2;
	// Get burst status
	if ( pTxRegs->m_TxControl.Fields.m_BurstMode == 1 ) {
		*pTxPacketMode |= DTU2XX_TXMODE_BURST;
	}
	// Get SDI mode bit
	if ( pTxRegs->m_TxControl.Fields.m_SdiMode ) {
		*pTxPacketMode |= DTU2XX_TXMODE_SDI;
	}
	// Get SDI 10-bit or 8-bit mode
	if ( pTxRegs->m_TxControl.Fields.m_Sdi10Bit ) {
		*pTxPacketMode |= DTU2XX_TXMODE_SDI_10BIT;
	}

	// Get stuff mode
	*pTxStuffMode = pTxRegs->m_TxControl.Fields.m_PckStuff;

	return 0;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxTxIoCtlReadLoopBackData -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Reads a specified number of bytes from the HW fifo via the loopback registers
//
DTU2XX_RESULT Dtu2xxTxIoCtlReadLoopBackData(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int8* pBuffer,		// Pointer to data read back from Output FIFO
    Int nBytesToRead)	// Number of bytes to be read
{
	Dtu2xxTx* pTxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumTxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_OUTPUT;
	// Get register cache
	pTxRegs = &(pFdo->m_TsTx[ChIdx].m_ChRegs.m_TxRegs);

    return DTU2XX_E_NOT_SUPPORTED;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxTxIoCtlReset -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Reset transport-stream output channel.
//
DTU2XX_RESULT Dtu2xxTxIoCtlReset(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int RstMode)		// How to reset the channel
{
	Int Status = 0, BytesTransf=0;
    UInt8 VendCmd = 0;
	Dtu2xxTx* pTxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumTxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_OUTPUT;
	// Get register cache
	pTxRegs = &(pFdo->m_TsTx[ChIdx].m_ChRegs.m_TxRegs);

	// Check for valid value
	if ( RstMode!=DTU2XX_RST_CHANNEL && RstMode!=DTU2XX_RST_CLEARFIFO ) {
		return -EINVAL;
	}

	if ( RstMode==DTU2XX_RST_CHANNEL ) {
		VendCmd = DTU2XX_USB_TX_RESET;
	} else {
		VendCmd = DTU2XX_USB_TX_CLEAR_FIFO;
	}

	// Clear all flags
	Dtu2xxTxIoCtlClearFlags( pFdo, ChIdx, 0xFFFFFFFF );

	// Clear internal buffer
	pFdo->m_TsTx[ChIdx].m_RingBuf.m_pRdPtr =
						pFdo->m_TsTx[ChIdx].m_RingBuf.m_pWrPtr;
	pFdo->m_TsTx[ChIdx].m_RingBuf.m_Load = 0;

	// Simply send the vendor command to the device
	Status = Dtu2xxVendorRequest(
					pFdo,
					VendCmd, 0, 0,
					DTU2XX_DIRECTION_WRITE,
					0, NULL, &BytesTransf);
	if ( Status < 0 )
		return DTU2XX_E;

	// We need to update all cached registers after a reset
	return Dtu2xxCacheOpRegs( pFdo );
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxTxIoCtlSetFifoSize -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Sets the channels fifo size
//
DTU2XX_RESULT Dtu2xxTxIoCtlSetFifoSize(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int FifoSize)		// Size of FIFO in bytes
{
	Dtu2xxTx* pTxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumTxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_OUTPUT;
	// Get register cache
	pTxRegs = &(pFdo->m_TsTx[ChIdx].m_ChRegs.m_TxRegs);

	// Check for valid values
	if ( FifoSize < 16 || 0 != (FifoSize % 16) || FifoSize > (16*1024*1024) ) {
		return -EINVAL;
	}

	// Update cache
	if ( FifoSize != -1 )
  	  pTxRegs->m_FifoSize = FifoSize;

	// Update register value to match cache
	return Dtu2xxIoCtlWriteReg( pFdo, DTU2XX_TX_BASE_ADDR+DTU2XX_TX_REG_FIFOSIZE,
									 (Int)(pTxRegs->m_FifoSize));
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxTxIoCtlSetFifoSizeMax -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Set FIFO size to the maximum valid value.
//
DTU2XX_RESULT Dtu2xxTxIoCtlSetFifoSizeMax(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx)			// Channel index
{
	Int MaxSize = 0;
    Dtu2xxTx* pTxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumTxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_OUTPUT;

	// Get register cache
	pTxRegs = &(pFdo->m_TsTx[ChIdx].m_ChRegs.m_TxRegs);

    // Get maximum size
    Dtu2xxTxIoCtlGetMaxFifoSize(pFdo, ChIdx, &MaxSize);

    // Set FIFO size
    return Dtu2xxTxIoCtlSetFifoSize(pFdo, ChIdx, (MaxSize-16));
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxTxIoCtlSetLoopBackMode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Set the channel's loop-back mode.
//
DTU2XX_RESULT Dtu2xxTxIoCtlSetLoopBackMode(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int nMode)			// New loop-back mode
{
	Dtu2xxTx* pTxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumTxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_OUTPUT;
	// Get register cache
	pTxRegs = &(pFdo->m_TsTx[ChIdx].m_ChRegs.m_TxRegs);

    return DTU2XX_E_NOT_SUPPORTED;
}


//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxTxIoCtlSetPower -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Set the channel's power mode.
//
DTU2XX_RESULT Dtu2xxTxIoCtlSetPower(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int nPower)			// New power mode
{
	Dtu2xxTx* pTxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumTxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_OUTPUT;
	// Get register cache
	pTxRegs = &(pFdo->m_TsTx[ChIdx].m_ChRegs.m_TxRegs);

    return DTU2XX_E_NOT_SUPPORTED;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxTxIoCtlSetTsRateBps -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Sets the transport-stream rate in bps
//
// FIXME: Overflow when rate is given close to 216 Mbps!
//
DTU2XX_RESULT Dtu2xxTxIoCtlSetTxClock(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int TsRateInt)		// Transport-stream rate in bps
{
//	Int Status = 0;
//	Int TxPacketMode, TxStuffMode;
//	Int PckSize;				// Packet size: 188 / 204
	UInt TxClock;				// The TxClock value
//	UInt64 TsRate = TsRateInt;
//	UInt64 tmpRate;		// Used for passing to the do_div() macro
	Dtu2xxTx* pTxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumTxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_OUTPUT;
	// Get register cache
	pTxRegs = &(pFdo->m_TsTx[ChIdx].m_ChRegs.m_TxRegs);

/*
 * 	The calculation of the clockrate is already implemented in the DTAPI
 */

/*	// Check for valid rate
	if ( (TsRate < 0) || (TsRate > 216000000) ) {
		return -EINVAL;
	}

	// Get current packet mode
	Status = Dtu2xxTxIoCtlGetTxMode( pFdo, &TxPacketMode, &TxStuffMode );
	if ( Status != 0 ) {
		return Status;
	}

	// Determine current packet size
	PckSize = Dtu2xxTxMode2PacketSize( TxPacketMode );

	// Set phase-increment register, depends on transport-stream rate and packet size
	if ( PckSize == 188 ) {
		// TxClock = (UInt32)(((TsRate * 8388608*2 / 421875) + 1) >> 1);
		//
		// Divisions of 64 bit variables can be done after linking against gcclib;
		// 		but as this is a kernel module, this cannot be done. Therefore
		// 		macro 'do_div( UInt64 *, UInt32 )' from asm-generic/div64.h is used.

		tmpRate = TsRate * 8388608 * 2;
		do_div( tmpRate, 421875 );
		tmpRate = (tmpRate + 1) >> 1;
		TxClock = (UInt32)tmpRate;
	} else if ( PckSize == 192 ) {
		// TxClock = (UInt32)(((TsRate * 402653184*2 / 19828125) + 1) >> 1);

		tmpRate = TsRate * 402653184 * 2;
		do_div( tmpRate, 19828125 );
		TxClock = (UInt32)((tmpRate + 1) >> 1);
	} else {	// PckSize == 204
		// TxClock = (UInt32)(((TsRate * 142606336*2 / 6609375) + 1) >> 1);

		tmpRate = TsRate * 142606336 * 2;
		do_div( tmpRate, 6609375 );
		TxClock = (UInt32)((tmpRate + 1) >> 1);
	}	*/

	TxClock = TsRateInt;

	// Update cache
	pTxRegs->m_TxClock = TxClock;

	// Update register to match cache
	return Dtu2xxIoCtlWriteReg( pFdo, DTU2XX_TX_BASE_ADDR+DTU2XX_TX_REG_TXCLOCK,
									 (Int)(pTxRegs->m_TxClock) );
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxTxIoCtlSetTxControl -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Set the channel's transmit-control state.
//
DTU2XX_RESULT Dtu2xxTxIoCtlSetTxControl(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int TxControl)		// Transmit-control state
{
	Dtu2xxTx* pTxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumTxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_OUTPUT;
	// Get register cache
	pTxRegs = &(pFdo->m_TsTx[ChIdx].m_ChRegs.m_TxRegs);

	// Check Transmit-Control value
	if (    TxControl != DTU2XX_TXCTRL_IDLE && TxControl != DTU2XX_TXCTRL_HOLD
		 && TxControl != DTU2XX_TXCTRL_SEND )
	{
		return -EINVAL;
	}

	// Set control field(s) in cache
	pTxRegs->m_TxControl.Fields.m_TxCtrl = TxControl;

	// Update register value to match cache
	return Dtu2xxIoCtlWriteReg( pFdo, DTU2XX_TX_BASE_ADDR+DTU2XX_TX_REG_TXCTRL,
									 (Int)(pTxRegs->m_TxControl.All));
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxTxIoCtlSetTxMode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Set the channel's transmit mode.
//
DTU2XX_RESULT Dtu2xxTxIoCtlSetTxMode(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int TxPacketMode,	// Packet transmit mode
    Int TxStuffMode)	// Null-packet stuffing on/off
{
	Int Status = 0;
	Int CurTsRateBps;
	UInt TxMode;
	Dtu2xxTx* pTxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumTxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_OUTPUT;
	// Get register cache
	pTxRegs = &(pFdo->m_TsTx[ChIdx].m_ChRegs.m_TxRegs);

	// Get current bit-rate (needed later to recopmpute the bit-rate)
	Status = Dtu2xxTxIoCtlGetTxClock( pFdo, ChIdx, &CurTsRateBps );
	if ( Status != 0 ) {
		return Status;
	}

	// Check for valid mode
	TxMode = TxPacketMode & DTU2XX_TXMODE_MASK;
	if (    TxMode != DTU2XX_TXMODE_188 && TxMode != DTU2XX_TXMODE_204
		 && TxMode != DTU2XX_TXMODE_ADD16 && TxMode != DTU2XX_TXMODE_MIN16
		 && TxMode != DTU2XX_TXMODE_RAW )
	{
		return -EINVAL;
	}
	// Check for valid stuff mode
	if ( TxStuffMode != DTU2XX_TXMODE_STUFF_ON && TxStuffMode != DTU2XX_TXMODE_STUFF_OFF )
	{
		return -EINVAL;
	}

	// Set mode field(s) in cache
	pTxRegs->m_TxControl.Fields.m_TxMode	= TxPacketMode & 0x3;
	pTxRegs->m_TxControl.Fields.m_TxModeExt = ((TxPacketMode & 0x4) >> 2);
	// Set burst mode
	if ( 0 != (TxPacketMode & DTU2XX_TXMODE_BURST) ) {
		pTxRegs->m_TxControl.Fields.m_BurstMode = 1;
	} else {
		pTxRegs->m_TxControl.Fields.m_BurstMode = 0;
	}
	// Set stuff mode
	pTxRegs->m_TxControl.Fields.m_PckStuff = TxStuffMode;
	// Set SDI mode bits
	if ( 0 != (TxPacketMode & DTU2XX_TXMODE_SDI) ) {
		pTxRegs->m_TxControl.Fields.m_SdiMode = 1;
	} else {
		pTxRegs->m_TxControl.Fields.m_SdiMode = 0;
	}
	// 10-bit or 8-bit modes
	if ( 0 != (TxPacketMode & DTU2XX_TXMODE_SDI_10BIT) ) {
		pTxRegs->m_TxControl.Fields.m_Sdi10Bit = 1;
	} else {
		pTxRegs->m_TxControl.Fields.m_Sdi10Bit = 0;
	}

	// Update register value to match cache
	Status = Dtu2xxIoCtlWriteReg( pFdo, DTU2XX_TX_BASE_ADDR+DTU2XX_TX_REG_TXCTRL,
									 (Int)(pTxRegs->m_TxControl.All));
	if ( Status != 0 ) {
		return Status;
	}

	// Reset bit-rate after changing the TxMode (i.e. to apply 204/188 factor)
	return Dtu2xxTxIoCtlSetTxClock( pFdo, ChIdx, CurTsRateBps );
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxTxIoCtlSetTxPolarity -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
// Set polarity of DVB/ASI output signal.
//
DTU2XX_RESULT Dtu2xxTxIoCtlSetTxPolarity(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int Polarity)		// DTU2XX_POLARITY_NORMAL / DTU2XX_POLARITY_INVERTED
{
	Dtu2xxTx* pTxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumTxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_OUTPUT;
	// Get register cache
	pTxRegs = &(pFdo->m_TsTx[ChIdx].m_ChRegs.m_TxRegs);

	// Check for valid values
	if ( Polarity != DTU2XX_TXPOL_NORMAL && Polarity != DTU2XX_TXPOL_INVERTED ) {
		return -EINVAL;
	}

	// Set polarity
	pTxRegs->m_TxControl.Fields.m_TxInvAsi = Polarity;

	// Update register value to match cache
	return Dtu2xxIoCtlWriteReg( pFdo, DTU2XX_TX_BASE_ADDR+DTU2XX_TX_REG_TXCTRL,
									 (Int)(pTxRegs->m_TxControl.All) );
}

//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxTxIoCtlDma -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Transfer a user buffer to the output channel using a DMA transfer
//
DTU2XX_RESULT Dtu2xxTxIoCtlDma(
    DTU2XX_FDO* pFdo,	// Functional device object, representing the DTU-2xx card
	Int ChIdx,			// Channel index
    Int8* pUserBuf,		// Pointer to the user buffer
    Int nBytesToTransfer)	// Bytes to transfer
{
	Dtu2xxTx* pTxRegs = NULL;

	// Valid channel index???
    if ( pFdo->m_NumTxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_OUTPUT;
	// Get register cache
	pTxRegs = &(pFdo->m_TsTx[ChIdx].m_ChRegs.m_TxRegs);

    return DTU2XX_E_NOT_SUPPORTED;
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Dtu2xxTxIoCtlBulkWrite -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
// Perform a bulk write action
//
DTU2XX_RESULT Dtu2xxTxIoCtlBulkWrite(
	DTU2XX_FDO* pFdo,
	Int ChIdx,			// Channel index
	Int TransferSize,
	UInt8* pBuf)
{
	Int Status=0, n, BytesLeft, BufIndex, NumCopy, CopiedFromIntBuf, TempBufSize;
    UInt8* pTempBuf=NULL;
	DTU2XX_RING_BUFFER* pRingBuf=NULL;

	// Valid channel index???
    if ( pFdo->m_NumTxChannels <= ChIdx )
		return DTU2XX_E_NO_SUCH_OUTPUT;

	if ( TransferSize == 0 )
		return DTU2XX_OK;
	if ( pBuf == NULL )
		return DTU2XX_E_INVALID_BUF;

	CopiedFromIntBuf = 0;
	BytesLeft = TransferSize;
	BufIndex = 0;

	TempBufSize = pFdo->m_TsTx[ChIdx].m_TempBufSize;
	pTempBuf = pFdo->m_TsTx[ChIdx].m_pTempBuf;
	pRingBuf = &(pFdo->m_TsTx[ChIdx].m_RingBuf);

#if LOG_LEVEL_DMA > 1
	DTU2XX_LOG3( KERN_INFO, "Dtu2xxTxIoCtlBulkWrite TransferSize: %d, pBuf: 0x%p, Int. Load: %d",
			     TransferSize, pBuf, pRingBuf->m_Load );
#endif

	//-.-.-.-.-.-.-.-.-.-.-.-.- Check for minimum transfer size -.-.-.-.-.-.-.-.-.-.-.-.-.

	// Do we have minimum required data?
	if ( (pRingBuf->m_Load + TransferSize) < DTU2XX_BULK_TRANSFER_ALIGN ) {

		// First copy from user buffer to our temporary buffer
		Status = copy_from_user(pTempBuf, pBuf, TransferSize);

		// Copy all data to the internal buffer
		if ( TransferSize!=Dtu2xxBuf2RingBuffer(pTempBuf, TransferSize, pRingBuf) )
			return DTU2XX_E;
		else
			return DTU2XX_OK;
	}

	// First transfer data from internal buffer
	if ( pRingBuf->m_Load>0 )
	{
		// Compute the ideal transfer size (i.e closest multiple of
		// DTU2XX_BULK_TRANSFER_ALIGN)
		NumCopy = (pRingBuf->m_Load / DTU2XX_BULK_TRANSFER_ALIGN) *
												DTU2XX_BULK_TRANSFER_ALIGN;
		if ( 0!=( pRingBuf->m_Load % DTU2XX_BULK_TRANSFER_ALIGN) )
			NumCopy += DTU2XX_BULK_TRANSFER_ALIGN;

		// If we donot have enough data for the ideal size copy as much as we can
		n = pRingBuf->m_Load + TransferSize;
		if ( NumCopy > n  )
			NumCopy = n - (n%DTU2XX_BULK_TRANSFER_ALIGN);

		// Donot copy more than what fits in our temporary buffer
		NumCopy = (NumCopy > TempBufSize) ? TempBufSize : NumCopy;
		NumCopy = NumCopy - (NumCopy % DTU2XX_BULK_TRANSFER_ALIGN);

		// Now copy data left in internal buffer to the temporary
		if ( NumCopy <= pRingBuf->m_Load )
			CopiedFromIntBuf = Dtu2xxRingBuffer2Buf(pRingBuf, NumCopy, pTempBuf);
		else
			CopiedFromIntBuf = Dtu2xxRingBuffer2Buf(pRingBuf, pRingBuf->m_Load,
													pTempBuf);

		// Do we need to add additional data from our buffer?
		if ( NumCopy > CopiedFromIntBuf )
		{
			Status = copy_from_user( &(pTempBuf[CopiedFromIntBuf]), &(pBuf[BufIndex]),
					         (NumCopy-CopiedFromIntBuf) );
			// Update index
			BufIndex += (NumCopy-CopiedFromIntBuf);
		}

		// Transfer data to device
		if ( NumCopy != 0 )
		{
			Status = Dtu2xxSimpleBulkTransfer(
							pFdo, DTU2XX_DIRECTION_WRITE, EZUSB_ENDPOINT_2,
							NumCopy, pTempBuf );

			if ( Status != 0 )
				return DTU2XX_E;
		}

		// Update data left count (do not count bytes that, where copied from the internal
		// buffer)
		BytesLeft -= (NumCopy-CopiedFromIntBuf);
	}

	//-.-.-.-.-.-.-.-.-.-.-.-.- Directly copy multiples of 1024 -.-.-.-.-.-.-.-.-.-.-.-.-.

	while ( BytesLeft>=DTU2XX_BULK_TRANSFER_ALIGN )
	{
		NumCopy = (BytesLeft>DTU2XX_BULK_PACKETSIZE) ? DTU2XX_BULK_PACKETSIZE : BytesLeft;
		// Only multiples of DTU2XX_BULK_TRANSFER_ALIGN
		NumCopy = NumCopy - (NumCopy%DTU2XX_BULK_TRANSFER_ALIGN);

		// Copy data from user buffer to our temporary buffer
		Status = copy_from_user(pTempBuf, &(pBuf[BufIndex]), NumCopy);
		if ( Status != 0 )
			break;

		Status = Dtu2xxSimpleBulkTransfer(
							pFdo,
							DTU2XX_DIRECTION_WRITE,
							EZUSB_ENDPOINT_2,
							NumCopy, pTempBuf );
		if ( Status != 0 )
			return DTU2XX_E;

		BytesLeft -= NumCopy;
		BufIndex += NumCopy;
 	}

	//.-.-.-.-.-.-.-.-.-.-.- Copy remaining data to internal buffer -.-.-.-.-.-.-.-.-.-.-.

	if ( BytesLeft>0 ) {

		// Sanity check: bytes left must fit in our temp and ring buffers
		if ( BytesLeft>TempBufSize || BytesLeft>pRingBuf->m_TotalSize )
			return DTU2XX_E;

		// First copy data from user buffer to our temporary buffer
		Status = copy_from_user(pTempBuf, &(pBuf[BufIndex]), BytesLeft);

		Dtu2xxBuf2RingBuffer(pTempBuf, BytesLeft, pRingBuf);
	}
	if ( Status != 0 )
	   return DTU2XX_E;

	return DTU2XX_OK;
}
